/*
 * translate.c - Global internationalization routines.
 *
 * Written by
 *  Marco van den Heuvel <blackystardust68@yahoo.com>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

#include "vice.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cmdline.h"

#ifdef HAS_TRANSLATION
#include "intl.h"
#endif

#include "lib.h"
#include "log.h"
#include "resources.h"
#include "translate_languages.h"
#include "translate.h"
#include "util.h"

#define countof(array) (sizeof(array) / sizeof((array)[0]))

typedef struct translate_s {
    int resource_id;
    char *text;
} translate_t;

#ifdef HAS_TRANSLATION
char *current_language = NULL;
int current_language_index = 0;
#endif

/* GLOBAL STRING ID TEXT TABLE */


/////// translate_text.c starts here
////



/*
 * translate_text.c - Translation texts to be included in translate.c
 *
 * Written by
 *  Marco van den Heuvel <blackystardust68@yahoo.com>
 *
 * This file is part of VICE, the Versatile Commodore Emulator.
 * See README for copyright notice.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 *
 */

translate_t string_table[] = {
	
	/* autostart.c */
	/* en */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE,    N_("Cannot load snapshot file.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_DA, "Kunne ikke indlÊse snapshot-fil."},
	/* de */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_DE, "Kann Snapshot Datei nicht laden."},
	/* es */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_ES, "No puedo cargar Instant·nea"},
	/* fr */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_FR, "Impossible de charger l'instantanÈ."},
	/* hu */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_HU, "Nem siker¸lt betˆlteni a pillanatkÈp f·jlt."},
	/* it */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_IT, "Non Ë possibile caricare il file di snapshot."},
	/* ko */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_NL, "Kan momentopnamebestand niet laden."},
	/* pl */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_PL, "Nie moøna wczytaÊ pliku zrzutu."},
	/* ru */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_RU, "Cannot load snapshot file."},
	/* sv */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_SV, "Kan inte l‰sa in ˆgonblicksbildfil."},
	/* tr */ {IDGS_CANNOT_LOAD_SNAPSHOT_FILE_TR, "Anl˝k gˆr¸nt¸ dosyas˝ y¸klenemedi."},
#endif
	
	/* debug.c */
	/* en */ {IDGS_PLAYBACK_ERROR_DIFFERENT,    N_("Playback error: %s different from line %d of file debug%06d")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_PLAYBACK_ERROR_DIFFERENT_DA, "Afspilningsfejl: %s forskellig fra linje %d i filen debug%06d"},
	/* de */ {IDGS_PLAYBACK_ERROR_DIFFERENT_DE, "Wiedergabe Fehler: %s ist unterschiedlich von Zeile %d in Datei debug%06d"},
	/* es */ {IDGS_PLAYBACK_ERROR_DIFFERENT_ES, "Error de reproducciÛn: %s diferente desde linea %d del fichero de depuraciÛn%06d"},
	/* fr */ {IDGS_PLAYBACK_ERROR_DIFFERENT_FR, "Erreur de lecture: %s diffÈrent de la ligne %d du fichier debug%06d"},
	/* hu */ {IDGS_PLAYBACK_ERROR_DIFFERENT_HU, "Visszaj·tsz·si hiba: %s k¸lˆnbˆzik a %d. sorban a debug%06d f·jl esetÈn"},
	/* it */ {IDGS_PLAYBACK_ERROR_DIFFERENT_IT, "Errore di riproduzione: %s Ë differente dalla linea %d del file di debug %06d"},
	/* ko */ {IDGS_PLAYBACK_ERROR_DIFFERENT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_PLAYBACK_ERROR_DIFFERENT_NL, "Afspeelfout: %s is anders dan regel %d van bestand debug%06d"},
	/* pl */ {IDGS_PLAYBACK_ERROR_DIFFERENT_PL, "B≥±d otwarzania: %s rÛøni siÍ od linii %d pliku debug%06d"},
	/* ru */ {IDGS_PLAYBACK_ERROR_DIFFERENT_RU, "Playback error: %s different from line %d of file debug%06d"},
	/* sv */ {IDGS_PLAYBACK_ERROR_DIFFERENT_SV, "≈terspelningsfel: %s inte lika som rad %d i filen debug%06d"},
	/* tr */ {IDGS_PLAYBACK_ERROR_DIFFERENT_TR, "Kay˝ttan y¸r¸tme hatas˝: %s dosyan˝n %d sat˝r˝ndan farkl˝ (hata ay˝klama%06d)"},
#endif
	
	/* event.c */
	/* en */ {IDGS_CANNOT_CREATE_IMAGE,    N_("Cannot create image file!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_CREATE_IMAGE_DA, "Kunne ikke oprette image-fil!"},
	/* de */ {IDGS_CANNOT_CREATE_IMAGE_DE, "Kann Imagedatei nicht erzeugen!"},
	/* es */ {IDGS_CANNOT_CREATE_IMAGE_ES, "°No puedo crear fichero imagen!"},
	/* fr */ {IDGS_CANNOT_CREATE_IMAGE_FR, "Impossible de crÈer le fichier image!"},
	/* hu */ {IDGS_CANNOT_CREATE_IMAGE_HU, "Nem siker¸lt a kÈpm·s f·jlt lÈtrehozni!"},
	/* it */ {IDGS_CANNOT_CREATE_IMAGE_IT, "Non Ë possibile creare il file immagine!"},
	/* ko */ {IDGS_CANNOT_CREATE_IMAGE_KO, "¿ÃπÃ¡ˆ ∆ƒ¿œ¿ª ª˝º∫«“ºˆ æ¯Ω¿¥œ¥Ÿ!"},
	/* nl */ {IDGS_CANNOT_CREATE_IMAGE_NL, "Kan bestand niet aanmaken!"},
	/* pl */ {IDGS_CANNOT_CREATE_IMAGE_PL, "Nie moøna utworzyÊ pliku obrazu!"},
	/* ru */ {IDGS_CANNOT_CREATE_IMAGE_RU, "Ω’ „‘–’‚·Ô ·ﬁ◊‘–‚Ï ‰–Ÿ€ ﬁ—‡–◊–!"},
	/* sv */ {IDGS_CANNOT_CREATE_IMAGE_SV, "Kan inte skapa avbildningsfil!"},
	/* tr */ {IDGS_CANNOT_CREATE_IMAGE_TR, "Imaj dosyas˝ yarat˝lamad˝!"},
#endif
	
	/* event.c */
	/* en */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S,    N_("Cannot write image file %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_DA, "Kunne ikks skrive image-fil %s"},
	/* de */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_DE, "Kann Datei %s nicht schreiben"},
	/* es */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_ES, "No puedo grabar fichero imagen %s"},
	/* fr */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_FR, "Impossible d'Ècrire le fichier image %s"},
	/* hu */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_HU, "Nem siker¸lt a %s kÈpm·s f·jlba Ìrni"},
	/* it */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_IT, "Non Ë possibile scrivere l'immagine %s"},
	/* ko */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_NL, "Kan niet schrijven naar bestand %s"},
	/* pl */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_PL, "Nie moøna zapisaÊ pliku obrazu %s"},
	/* ru */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_RU, "Cannot write image file %s"},
	/* sv */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_SV, "Kan inte skriva avbildningsfil %s"},
	/* tr */ {IDGS_CANNOT_WRITE_IMAGE_FILE_S_TR, "%s imaj dosyas˝na yaz˝lamad˝"},
#endif
	
	/* event.c */
	/* en */ {IDGS_CANNOT_FIND_MAPPED_NAME_S,    N_("Cannot find mapped name for %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_DA, "Kunne ikke finde mappet navn for %s"},
	/* de */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_DE, "Kann zugeordneten Namen f¸r `%s' nicht finden."},
	/* es */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_ES, "No puedo encontrar nombre mapeado para %s"},
	/* fr */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_FR, "Impossible de trouver le nom correspondant ‡ %s"},
	/* hu */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_HU, "A VICE %s lekÈpzett nevÈt nem tal·lja"},
	/* it */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_IT, "Non Ë possibile trovare il nome mappato per %s"},
	/* ko */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_NL, "Kan toegewezen naam voor %s niet vinden"},
	/* pl */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_PL, "Nie moøna znaleºÊ zmapowanej nazwy %s"},
	/* ru */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_RU, "Cannot find mapped name for %s"},
	/* sv */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_SV, "Kan inte kopplat namn fˆr %s"},
	/* tr */ {IDGS_CANNOT_FIND_MAPPED_NAME_S_TR, "%s iÁin e˛le˛me ismi bulunamad˝"},
#endif
	
	/* event.c */
	/* en */ {IDGS_CANT_CREATE_START_SNAP_S,    N_("Could not create start snapshot file %s.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANT_CREATE_START_SNAP_S_DA, "Kunne ikke oprette start-snapshot-filen %s."},
	/* de */ {IDGS_CANT_CREATE_START_SNAP_S_DE, "Kann Start Snapshot Datei nicht erzeugen: %s"},
	/* es */ {IDGS_CANT_CREATE_START_SNAP_S_ES, "No puedo crear inicio fichero Instant·nea %s."},
	/* fr */ {IDGS_CANT_CREATE_START_SNAP_S_FR, "Impossible de crÈer l'instantanÈ de dÈpart %s."},
	/* hu */ {IDGS_CANT_CREATE_START_SNAP_S_HU, "Nem siker¸lt a %s kezdeti pillanatkÈp f·jlt lÈtrehozni."},
	/* it */ {IDGS_CANT_CREATE_START_SNAP_S_IT, "Non Ë possibile creare il file di inizio snapshot %s."},
	/* ko */ {IDGS_CANT_CREATE_START_SNAP_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANT_CREATE_START_SNAP_S_NL, "Kon het start momentopnamebestand %s niet maken."},
	/* pl */ {IDGS_CANT_CREATE_START_SNAP_S_PL, "Nie moøna utworzyÊ pocz±tkowego pliku zrzutu %s."},
	/* ru */ {IDGS_CANT_CREATE_START_SNAP_S_RU, "Could not create start snapshot file %s."},
	/* sv */ {IDGS_CANT_CREATE_START_SNAP_S_SV, "Kunde inte skapa startˆgonblicksbildfilen %s."},
	/* tr */ {IDGS_CANT_CREATE_START_SNAP_S_TR, "Anl˝k gˆr¸nt¸ dosyas˝ %s yarat˝lamad˝."},
#endif
	
	/* event.c */
	/* en */ {IDGS_ERROR_READING_END_SNAP_S,    N_("Error reading end snapshot file %s.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_READING_END_SNAP_S_DA, "Fejl under lÊsning af slut-snapshot %s."},
	/* de */ {IDGS_ERROR_READING_END_SNAP_S_DE, "Kann Ende Snapshot Datei nicht lesen: %s"},
	/* es */ {IDGS_ERROR_READING_END_SNAP_S_ES, "Error leyendo final fichero Instant·nea %s."},
	/* fr */ {IDGS_ERROR_READING_END_SNAP_S_FR, "Erreur de lecture de l'instantanÈ de fin %s."},
	/* hu */ {IDGS_ERROR_READING_END_SNAP_S_HU, "Hiba az utolsÛ %s pillanatkÈp f·jl Ìr·sakor."},
	/* it */ {IDGS_ERROR_READING_END_SNAP_S_IT, "Errore durante la lettura del file di fine snapshot %s."},
	/* ko */ {IDGS_ERROR_READING_END_SNAP_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_READING_END_SNAP_S_NL, "Fout bij het lezen van het eind van het momentopnamebestand %s."},
	/* pl */ {IDGS_ERROR_READING_END_SNAP_S_PL, "B≥±d odczytu koÒcowego pliku zrzutu %s."},
	/* ru */ {IDGS_ERROR_READING_END_SNAP_S_RU, "Error reading end snapshot file %s."},
	/* sv */ {IDGS_ERROR_READING_END_SNAP_S_SV, "Fel vid l‰sning av slutˆgonblicksbildfilen %s."},
	/* tr */ {IDGS_ERROR_READING_END_SNAP_S_TR, "Anl˝k gˆr¸nt¸ dosyas˝ %s okuma hatas˝."},
#endif
	
	/* event.c */
	/* en */ {IDGS_CANT_CREATE_END_SNAP_S,    N_("Could not create end snapshot file %s.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANT_CREATE_END_SNAP_S_DA, "Kunne ikke oprette slut-snapshot-filen %s."},
	/* de */ {IDGS_CANT_CREATE_END_SNAP_S_DE, "Kann Ende Snapshot Datei nicht erzeugen: %s"},
	/* es */ {IDGS_CANT_CREATE_END_SNAP_S_ES, "No puedo crear fin fichero Instant·nea %s."},
	/* fr */ {IDGS_CANT_CREATE_END_SNAP_S_FR, "Impossible de crÈer l'instantanÈ de fin %s."},
	/* hu */ {IDGS_CANT_CREATE_END_SNAP_S_HU, "Nem siker¸lt a %s befejezı pillanatkÈp f·jlt lÈtrehozni."},
	/* it */ {IDGS_CANT_CREATE_END_SNAP_S_IT, "Non Ë possibile creare il file di fine snapshot %s."},
	/* ko */ {IDGS_CANT_CREATE_END_SNAP_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANT_CREATE_END_SNAP_S_NL, "Kon het eind momentopnamebestand %s niet maken."},
	/* pl */ {IDGS_CANT_CREATE_END_SNAP_S_PL, "Nie moøna utworzyÊ koÒcowego pliku zrzutu %s."},
	/* ru */ {IDGS_CANT_CREATE_END_SNAP_S_RU, "Could not create end snapshot file %s."},
	/* sv */ {IDGS_CANT_CREATE_END_SNAP_S_SV, "Kunde inte skapa slutˆgonblicksbildfilen %s."},
	/* tr */ {IDGS_CANT_CREATE_END_SNAP_S_TR, "Son anl˝k gˆr¸nt¸ dosyas˝ %s yarat˝lamad˝."},
#endif
	
	/* event.c */
	/* en */ {IDGS_CANT_OPEN_END_SNAP_S,    N_("Could not open end snapshot file %s.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANT_OPEN_END_SNAP_S_DA, "Kunne ikke Âbne slut-snapshot-filen %s."},
	/* de */ {IDGS_CANT_OPEN_END_SNAP_S_DE, "Kann Ende Snapshot Datei nicht ˆffnen: %s"},
	/* es */ {IDGS_CANT_OPEN_END_SNAP_S_ES, "No puedo abrir fin fichero Instant·nea %s."},
	/* fr */ {IDGS_CANT_OPEN_END_SNAP_S_FR, "Impossible d'ouvrir l'instantanÈ de fin %s."},
	/* hu */ {IDGS_CANT_OPEN_END_SNAP_S_HU, "Nem siker¸lt az utolsÛ %s pillanatkÈp f·jlt megnyitni."},
	/* it */ {IDGS_CANT_OPEN_END_SNAP_S_IT, "Non Ë possibile aprire il file di fine snapshot %s."},
	/* ko */ {IDGS_CANT_OPEN_END_SNAP_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANT_OPEN_END_SNAP_S_NL, "Kon het eind momentopnamebestand %s niet openen."},
	/* pl */ {IDGS_CANT_OPEN_END_SNAP_S_PL, "Nie moøna otworzyÊ koÒcowego pliku zrzutu %s."},
	/* ru */ {IDGS_CANT_OPEN_END_SNAP_S_RU, "Could not open end snapshot file %s."},
	/* sv */ {IDGS_CANT_OPEN_END_SNAP_S_SV, "Kunde inte ˆppna slutˆgonblicksbildfilen %s."},
	/* tr */ {IDGS_CANT_OPEN_END_SNAP_S_TR, "Son anl˝k gˆr¸nt¸ dosyas˝ %s aÁ˝lamad˝."},
#endif
	
	/* event.c */
	/* en */ {IDGS_CANT_FIND_SECTION_END_SNAP,    N_("Could not find event section in end snapshot file.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANT_FIND_SECTION_END_SNAP_DA, "Kunne ikke finde hÊndelses-sektionen i slut-snapshot-filen."},
	/* de */ {IDGS_CANT_FIND_SECTION_END_SNAP_DE, "Kann Sektion in Ende Snapshotdatei nicht finden."},
	/* es */ {IDGS_CANT_FIND_SECTION_END_SNAP_ES, "No puedo encontrar secciÛn eventos en fin fichero Instant·nea."},
	/* fr */ {IDGS_CANT_FIND_SECTION_END_SNAP_FR, "Impossible de trouver la section event dans l'instantanÈ de fin."},
	/* hu */ {IDGS_CANT_FIND_SECTION_END_SNAP_HU, "Nem tal·ltam esemÈny rÈszt az utolsÛ pillanatkÈp f·jlban."},
	/* it */ {IDGS_CANT_FIND_SECTION_END_SNAP_IT, "Non Ë possibile trovare la sezione eventi nel file di fine snapshot."},
	/* ko */ {IDGS_CANT_FIND_SECTION_END_SNAP_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANT_FIND_SECTION_END_SNAP_NL, "Kon de gebeurtenis sectie in eind momentopnamebestand niet vinden."},
	/* pl */ {IDGS_CANT_FIND_SECTION_END_SNAP_PL, "Nie znaleziono sekcji zdarzeÒ w pliku koÒcowym zrzutu"},
	/* ru */ {IDGS_CANT_FIND_SECTION_END_SNAP_RU, "Could not find event section in end snapshot file."},
	/* sv */ {IDGS_CANT_FIND_SECTION_END_SNAP_SV, "Kunde inte hinna h‰ndelsedelen i slutˆgonblicksbilden."},
	/* tr */ {IDGS_CANT_FIND_SECTION_END_SNAP_TR, "Son anl˝k gˆr¸nt¸ dosyas˝nda olay bˆl¸m¸ bulunamad˝."},
#endif
	
	/* event.c */
	/* en */ {IDGS_ERROR_READING_START_SNAP_TRIED,    N_("Error reading start snapshot file. Tried %s and %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_READING_START_SNAP_TRIED_DA, "Fejl under lÊsning af start-snapshot. Pr¯vede %s og %s"},
	/* de */ {IDGS_ERROR_READING_START_SNAP_TRIED_DE, "Fehler beim Lesen der Start Snapshot Datei. Versuch gescheitert bei %s und %s."},
	/* es */ {IDGS_ERROR_READING_START_SNAP_TRIED_ES, "Error leyendo inicio fichero Instant·nea. Intentado %s y %s"},
	/* fr */ {IDGS_ERROR_READING_START_SNAP_TRIED_FR, "Erreur de lecture de l'instantanÈ de dÈpart. EssayÈ %s et %s"},
	/* hu */ {IDGS_ERROR_READING_START_SNAP_TRIED_HU, "Nem siker¸lt az elsı pillanatkÈp f·jl olvas·sa. KiprÛb·lva: %s Ès %s"},
	/* it */ {IDGS_ERROR_READING_START_SNAP_TRIED_IT, "Errore durante la lettura del file di inizio snapshot. Ho provato %s e %s"},
	/* ko */ {IDGS_ERROR_READING_START_SNAP_TRIED_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_READING_START_SNAP_TRIED_NL, "Fout bij het lezen van het start momentopnamebestand. Heb %s en %s geprobeerd"},
	/* pl */ {IDGS_ERROR_READING_START_SNAP_TRIED_PL, "B≥±d odczytu pocz±tkowego pliku zrzutu. WyprÛbowano %s i %s"},
	/* ru */ {IDGS_ERROR_READING_START_SNAP_TRIED_RU, "Error reading start snapshot file. Tried %s and %s"},
	/* sv */ {IDGS_ERROR_READING_START_SNAP_TRIED_SV, "Fel vid l‰sning av startˆgonblicksbildfil. Fˆrsˆkte med %s och %s"},
	/* tr */ {IDGS_ERROR_READING_START_SNAP_TRIED_TR, "Ba˛lang˝Á anl˝k gˆr¸nt¸ dosyas˝ okuma hatas˝. %s ve %s denendi"},
#endif
	
	/* event.c */
	/* en */ {IDGS_ERROR_READING_START_SNAP,    N_("Error reading start snapshot file.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_READING_START_SNAP_DA, "Fejl under lÊsning af start-snapshot."},
	/* de */ {IDGS_ERROR_READING_START_SNAP_DE, "Fehler beim Lesen der Start Snapshot Datei."},
	/* es */ {IDGS_ERROR_READING_START_SNAP_ES, "Error leyendo inicio fichero Instant·nea."},
	/* fr */ {IDGS_ERROR_READING_START_SNAP_FR, "Erreur de lecture de l'instantanÈ de dÈpart."},
	/* hu */ {IDGS_ERROR_READING_START_SNAP_HU, "Nem siker¸lt az indÌtÛ pillanatkÈp f·jl olvas·sa."},
	/* it */ {IDGS_ERROR_READING_START_SNAP_IT, "Errore durante la lettura del file di inizio snapshot."},
	/* ko */ {IDGS_ERROR_READING_START_SNAP_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_READING_START_SNAP_NL, "Fout bij het lezen van het start momentopnamebestand."},
	/* pl */ {IDGS_ERROR_READING_START_SNAP_PL, "B≥±d odczytu pocz±tkowego pliku zrzutu."},
	/* ru */ {IDGS_ERROR_READING_START_SNAP_RU, "Error reading start snapshot file."},
	/* sv */ {IDGS_ERROR_READING_START_SNAP_SV, "Fel vid l‰sning av startˆgonblicksbildfil."},
	/* tr */ {IDGS_ERROR_READING_START_SNAP_TR, "Ba˛lang˝Á anl˝k gˆr¸nt¸ dosyas˝ okuma hatas˝."},
#endif
	
	/* screenshot.c */
	/* en */ {IDGS_SORRY_NO_MULTI_RECORDING,    N_("Sorry. Multiple recording is not supported.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SORRY_NO_MULTI_RECORDING_DA, "Kun en optagelse kan foretages ad gangen."},
	/* de */ {IDGS_SORRY_NO_MULTI_RECORDING_DE, "Eine Aufnahme ist zur Zeit aktiv. Mehrfache Aufnahme ist nicht mˆglich."},
	/* es */ {IDGS_SORRY_NO_MULTI_RECORDING_ES, "Lo siento, no est·n soportadas las grabaciones m˙ltiples."},
	/* fr */ {IDGS_SORRY_NO_MULTI_RECORDING_FR, "DÈsolÈ. De multiples enregistrements ne sont pas supportÈ."},
	/* hu */ {IDGS_SORRY_NO_MULTI_RECORDING_HU, "Sajn·lom: tˆbbszˆrˆs felvÈtel nem t·mogatott."},
	/* it */ {IDGS_SORRY_NO_MULTI_RECORDING_IT, "Le registrazioni multiple non sono supportate."},
	/* ko */ {IDGS_SORRY_NO_MULTI_RECORDING_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SORRY_NO_MULTI_RECORDING_NL, "Sorry. Meerdere gelijktijdige opnamen wordt niet ondersteund."},
	/* pl */ {IDGS_SORRY_NO_MULTI_RECORDING_PL, "Przepraszamy, wielo∂cieøkowy zapis nie jest obs≥ugiwany."},
	/* ru */ {IDGS_SORRY_NO_MULTI_RECORDING_RU, "Sorry. Multiple recording is not supported."},
	/* sv */ {IDGS_SORRY_NO_MULTI_RECORDING_SV, "Endast en inspelning kan gˆras Ât gÂngen."},
	/* tr */ {IDGS_SORRY_NO_MULTI_RECORDING_TR, "‹zg¸n¸z. «oklu kay˝t desteklenmiyor."},
#endif
	
	/* sound.c */
	/* en */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED,    N_("write to sound device failed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_DA, "Kunne ikke skrive til lydenhed."},
	/* de */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_DE, "Schreiben auf Sound Ger‰t ist fehlgeschlagen."},
	/* es */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_ES, "Fallo en grabaciÛn perifÈrico sonido."},
	/* fr */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_FR, "impossible d'Ècrire sur le pÈriphÈrique son."},
	/* hu */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_HU, "Nem siker¸lt a hangeszkˆzre Ìrni"},
	/* it */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_IT, "scrittura sulla scheda audio fallita."},
	/* ko */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_NL, "schrijfactie naar geluidsapparaat is mislukt."},
	/* pl */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_PL, "Zapis do urz±dzenia dºwiÍku nie powiÛd≥ siÍ."},
	/* ru */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_RU, "write to sound device failed."},
	/* sv */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_SV, "misslyckades att skriva till ljudenhet."},
	/* tr */ {IDGS_WRITE_TO_SOUND_DEVICE_FAILED_TR, "ses ayg˝t˝na yazma ba˛ar˝s˝z oldu."},
#endif
	
	/* sound.c */
	/* en */ {IDGS_CANNOT_OPEN_SID_ENGINE,    N_("Cannot open SID engine")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_OPEN_SID_ENGINE_DA, "Kunne ikke Âbne SID-motor"},
	/* de */ {IDGS_CANNOT_OPEN_SID_ENGINE_DE, "Kann SID Engine nicht ˆffnen"},
	/* es */ {IDGS_CANNOT_OPEN_SID_ENGINE_ES, "No puedo abrir motor SID"},
	/* fr */ {IDGS_CANNOT_OPEN_SID_ENGINE_FR, "Erreur d'ouverture de l'engin SID"},
	/* hu */ {IDGS_CANNOT_OPEN_SID_ENGINE_HU, "Nem siker¸lt a SID motort megnyitni"},
	/* it */ {IDGS_CANNOT_OPEN_SID_ENGINE_IT, "Non Ë possibile aprire il motore SID"},
	/* ko */ {IDGS_CANNOT_OPEN_SID_ENGINE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_OPEN_SID_ENGINE_NL, "Kan de SID kern niet openen"},
	/* pl */ {IDGS_CANNOT_OPEN_SID_ENGINE_PL, "Nie moøna otworzyÊ silnika SID"},
	/* ru */ {IDGS_CANNOT_OPEN_SID_ENGINE_RU, "Cannot open SID engine"},
	/* sv */ {IDGS_CANNOT_OPEN_SID_ENGINE_SV, "Kan inte ˆppna SID-motor"},
	/* tr */ {IDGS_CANNOT_OPEN_SID_ENGINE_TR, "SID motoru aÁ˝lam˝yor"},
#endif
	
	/* sound.c */
	/* en */ {IDGS_CANNOT_INIT_SID_ENGINE,    N_("Cannot initialize SID engine")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_INIT_SID_ENGINE_DA, "Kunne ikke initialisere SID-motor"},
	/* de */ {IDGS_CANNOT_INIT_SID_ENGINE_DE, "Kann SID Engine nicht initialisieren"},
	/* es */ {IDGS_CANNOT_INIT_SID_ENGINE_ES, "No puedo inicializar motor SID"},
	/* fr */ {IDGS_CANNOT_INIT_SID_ENGINE_FR, "Erreur d'initialisation de l'engin SID"},
	/* hu */ {IDGS_CANNOT_INIT_SID_ENGINE_HU, "Nem siker¸lt a SID motort elindÌtani"},
	/* it */ {IDGS_CANNOT_INIT_SID_ENGINE_IT, "Non Ë possibile inizializzare il motore SID"},
	/* ko */ {IDGS_CANNOT_INIT_SID_ENGINE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_INIT_SID_ENGINE_NL, "Kan de SID kern niet initialiseren"},
	/* pl */ {IDGS_CANNOT_INIT_SID_ENGINE_PL, "Nie moøna uruchomiÊ silnika SID"},
	/* ru */ {IDGS_CANNOT_INIT_SID_ENGINE_RU, "Cannot initialize SID engine"},
	/* sv */ {IDGS_CANNOT_INIT_SID_ENGINE_SV, "Kan inte initiera SID-motor"},
	/* tr */ {IDGS_CANNOT_INIT_SID_ENGINE_TR, "SID motorunun ba˛lang˝Á ayarlar˝ yap˝lam˝yor"},
#endif
	
	/* sound.c */
	/* en */ {IDGS_INIT_FAILED_FOR_DEVICE_S,    N_("initialization failed for device `%s'.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_INIT_FAILED_FOR_DEVICE_S_DA, "Initialisering fejlede for enhed \"%s\"."},
	/* de */ {IDGS_INIT_FAILED_FOR_DEVICE_S_DE, "Initialisierung von Ger‰t `%s' fehlgeschlagen."},
	/* es */ {IDGS_INIT_FAILED_FOR_DEVICE_S_ES, "Fallo inicializaciÛn para perifÈrico `%s'."},
	/* fr */ {IDGS_INIT_FAILED_FOR_DEVICE_S_FR, "erreur d'initialisation du pÈriphÈrique `%s'."},
	/* hu */ {IDGS_INIT_FAILED_FOR_DEVICE_S_HU, "A `%s' eszkˆzt nem siker¸lt elindÌtani."},
	/* it */ {IDGS_INIT_FAILED_FOR_DEVICE_S_IT, "inizializzazione fallita per il device `%s'."},
	/* ko */ {IDGS_INIT_FAILED_FOR_DEVICE_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_INIT_FAILED_FOR_DEVICE_S_NL, "initialisatie voor apparaat `%s' is mislukt."},
	/* pl */ {IDGS_INIT_FAILED_FOR_DEVICE_S_PL, "Inicjalizacja urz±dzenia `%s' nie powiod≥a siÍ."},
	/* ru */ {IDGS_INIT_FAILED_FOR_DEVICE_S_RU, "initialization failed for device `%s'."},
	/* sv */ {IDGS_INIT_FAILED_FOR_DEVICE_S_SV, "initiering misslyckades fˆr enhet \"%s\"."},
	/* tr */ {IDGS_INIT_FAILED_FOR_DEVICE_S_TR, "`%s' ayg˝t˝ iÁin ba˛lang˝Á ayarlar˝ ba˛ar˝s˝z oldu."},
#endif
	
	/* sound.c */
	/* en */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT,    N_("device '%s' not found or not supported.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_DA, "enhed \"%s\" ikke fundet eller underst¯ttet."},
	/* de */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_DE, "Ger‰t '%s' konnte nicht gefunden werden oder ist nicht unterst¸tzt."},
	/* es */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_ES, "PerifÈrico '%s' no encontrado o soportado."},
	/* fr */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_FR, "pÈriphÈrique '%s' non trouvÈ ou non supportÈ."},
	/* hu */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_HU, "A '%s' eszkˆz nem lÈtezik, vagy nem t·mogatott."},
	/* it */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_IT, "il device '%s' non Ë stato trovato oppure non Ë supportato."},
	/* ko */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_NL, "apparaat '%s' niet gevonden of niet ondersteund"},
	/* pl */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_PL, "Nie znaleziono urz±dzenia '%s' lub jest ono niewspierane"},
	/* ru */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_RU, "device '%s' not found or not supported."},
	/* sv */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_SV, "enheten \"%s\" hittades eller stˆds ej."},
	/* tr */ {IDGS_DEVICE_S_NOT_FOUND_SUPPORT_TR, "'%s' ayg˝t˝ yok ya da desteklenmiyor."},
#endif
	
	/* sound.c */
	/* en */ {IDGS_RECORD_DEVICE_S_NOT_EXIST,    N_("Recording device %s doesn't exist!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_DA, "Optageenheden %s findes ikke!"},
	/* de */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_DE, "Aufnahmeger‰t %s existiert nicht!"},
	/* es */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_ES, "No existe el perifÈrico %s de grabaciÛn "},
	/* fr */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_FR, "Le pÈriphÈrique d'enregistrement %s n'existe pas!"},
	/* hu */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_HU, "A %s felvevı eszkˆz nem lÈtezik!"},
	/* it */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_IT, "Il dispositivo di registrazione %s non esiste!"},
	/* ko */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_NL, "Opnameapparaat %s bestaat niet!"},
	/* pl */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_PL, "Urz±dzenie zapisu %s nie istnieje!"},
	/* ru */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_RU, "Recording device %s doesn't exist!"},
	/* sv */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_SV, "Inspelningsenhet %s finns inte!"},
	/* tr */ {IDGS_RECORD_DEVICE_S_NOT_EXIST_TR, "Kay˝t ayg˝t˝ %s mevcut deil!"},
#endif
	
	/* sound.c */
	/* en */ {IDGS_RECORD_DIFFERENT_PLAYBACK,    N_("Recording device must be different from playback device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_RECORD_DIFFERENT_PLAYBACK_DA, "Indspilningsenhed og optagelsesenhed skal vÊre forskellige"},
	/* de */ {IDGS_RECORD_DIFFERENT_PLAYBACK_DE, "Aufnahmeger‰t muss unterschiedlich vom Abspielger‰t sein"},
	/* es */ {IDGS_RECORD_DIFFERENT_PLAYBACK_ES, "PerifÈrico de grabaciÛn y reproducciÛn tienen que ser diferentes"},
	/* fr */ {IDGS_RECORD_DIFFERENT_PLAYBACK_FR, "Le pÈriphÈrique de son d'enregistrement doivent Ítre diffÈrents"},
	/* hu */ {IDGS_RECORD_DIFFERENT_PLAYBACK_HU, "A felvevı Ès lej·tszÛ eszkˆzˆknek k¸lˆnbˆznie kell"},
	/* it */ {IDGS_RECORD_DIFFERENT_PLAYBACK_IT, "Il dispositivo di registrazione deve essere differente da quello di riproduzione"},
	/* ko */ {IDGS_RECORD_DIFFERENT_PLAYBACK_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_RECORD_DIFFERENT_PLAYBACK_NL, "Opnameapparaat moet anders zijn dan afspeelapparaat"},
	/* pl */ {IDGS_RECORD_DIFFERENT_PLAYBACK_PL, "Urz±dzenie zapisu musi byÊ inne niø urz±dzenie odtwarzania"},
	/* ru */ {IDGS_RECORD_DIFFERENT_PLAYBACK_RU, "√·‚‡ﬁŸ·‚“ﬁ ◊–ﬂÿ·ÿ ‘ﬁ€÷›ﬁ ﬁ‚€ÿÁ–‚Ï·Ô ﬁ‚ „·‚‡ﬁŸ·‚“– “ﬁ·ﬂ‡ﬁÿ◊“’‘’›ÿÔ"},
	/* sv */ {IDGS_RECORD_DIFFERENT_PLAYBACK_SV, "Inspelningsenhet och Âterspelningsenhet kan inte vara samma"},
	/* tr */ {IDGS_RECORD_DIFFERENT_PLAYBACK_TR, "Kay˝t ayg˝t˝ kay˝ttan y¸r¸tme ayg˝t˝ ile farkl˝ olmak zorundad˝r"},
#endif
	
	/* sound.c */
	/* en */ {IDGS_WARNING_RECORDING_REALTIME,    N_("Warning! Recording device %s seems to be a realtime device!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_WARNING_RECORDING_REALTIME_DA, "Advarsel! Optageenheden %s ser ud til at vÊre en realtidsenhed!"},
	/* de */ {IDGS_WARNING_RECORDING_REALTIME_DE, "Warnung! Aufnahmeger‰t %s scheint ein Echtzeitger‰t zu sein!"},
	/* es */ {IDGS_WARNING_RECORDING_REALTIME_ES, "°PrecauciÛn! perifÈrico de grabaciÛn %s parece ser un perifÈrico en tiempo real"},
	/* fr */ {IDGS_WARNING_RECORDING_REALTIME_FR, "Attention! Le pÈriphÈrique d'enregistrement %s semble Ítre un pÈriphÈrique temps rÈel"},
	/* hu */ {IDGS_WARNING_RECORDING_REALTIME_HU, "Figyelem! A %s felvevı eszkˆz ˙gy t˚nik, valÛs idej˚ eszkˆz!"},
	/* it */ {IDGS_WARNING_RECORDING_REALTIME_IT, "Attenzione! Il dispositivo di registrazione %s sembra essere di tipo realtime!"},
	/* ko */ {IDGS_WARNING_RECORDING_REALTIME_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_WARNING_RECORDING_REALTIME_NL, "Waarschuwing! Opnameapparaat %s lijkt een realtime apparaat te zijn!"},
	/* pl */ {IDGS_WARNING_RECORDING_REALTIME_PL, "Ostrzeøenie! Urz±dzenie zapisu %s dzia≥a chyba w czasie rzeczywistym!"},
	/* ru */ {IDGS_WARNING_RECORDING_REALTIME_RU, "Warning! Recording device %s seems to be a realtime device!"},
	/* sv */ {IDGS_WARNING_RECORDING_REALTIME_SV, "Varning! Inspelningsenheten %s verkar vara en realtidsenhet!"},
	/* tr */ {IDGS_WARNING_RECORDING_REALTIME_TR, "Uyar˝! Kay˝t ayg˝t˝ %s gerÁek zamanl˝ bir ayg˝t gibi gˆr¸n¸yor!"},
#endif
	
	/* sound.c */
	/* en */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR,    N_("The recording device doesn't support current sound parameters")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_DA, "Optageenheden underst¯tter ikke de valgte lydparametre"},
	/* de */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_DE, "Aufnahmeger‰t unterst¸tzt derzeitige Soundeinstellungen nicht"},
	/* es */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_ES, "El perifÈrico de grabaciÛn no soporta los actuales par·metros de sonido"},
	/* fr */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_FR, "Le pÈriphÈrique d'enregistrement ne supporte pas les paramËtres actuels"},
	/* hu */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_HU, "A felvevı eszkˆz nem t·mogatja az aktu·lis hangbe·llÌt·sokat"},
	/* it */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_IT, "Il dispositivo di registrazione non supporta i parametri attuali"},
	/* ko */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_NL, "Het opnameapparaat ondersteunt de huidige geluidsopties niet"},
	/* pl */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_PL, "Urz±dzenie zapisu nie obs≥uguje obecnych parametrÛw dºwiÍku"},
	/* ru */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_RU, "The recording device doesn't support current sound parameters"},
	/* sv */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_SV, "Inspelningsenheten stˆder inte aktuella ljudinst‰llningar"},
	/* tr */ {IDGS_RECORD_NOT_SUPPORT_SOUND_PAR_TR, "Kay˝t ayg˝t˝ geÁerli ses parametrelerini desteklemiyor"},
#endif
	
	/* sound.c */
	/* en */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE,    N_("Sound buffer overflow (cycle based)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_DA, "Lydbuffer overl¯b (cykelbaseret)"},
	/* de */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_DE, "Sound Puffer ‹berlauf (Zyklus basiert)"},
	/* es */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_ES, "Desbordamiento Buffer sonido (Base ciclos)"},
	/* fr */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_FR, "Erreur de dÈpassement de tampon son (cycle based)"},
	/* hu */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_HU, "Hangpuffer t˙lcsordul·s (ciklus alap˙)"},
	/* it */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_IT, "Overflow del buffer audio (cycle based)"},
	/* ko */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_NL, "Geluidsbuffer overloop (cyclus gebaseerd)"},
	/* pl */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_PL, "Przepe≥niony bufor dºwiÍku (cykliczny)"},
	/* ru */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_RU, "Sound buffer overflow (cycle based)"},
	/* sv */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_SV, "Fˆr mycket data i ljudbufferten (cykelbaserad)"},
	/* tr */ {IDGS_SOUND_BUFFER_OVERFLOW_CYCLE_TR, "Ses arabellek ta˛mas˝ (cycle bazl˝)"},
#endif
	
	/* sound.c */
	/* en */ {IDGS_SOUND_BUFFER_OVERFLOW,    N_("Sound buffer overflow.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SOUND_BUFFER_OVERFLOW_DA, "Lydbuffer overl¯b."},
	/* de */ {IDGS_SOUND_BUFFER_OVERFLOW_DE, "Sound Puffer ‹berlauf."},
	/* es */ {IDGS_SOUND_BUFFER_OVERFLOW_ES, "Desbordamiento Buffer sonido"},
	/* fr */ {IDGS_SOUND_BUFFER_OVERFLOW_FR, "Erreur de dÈpassement de tampon son."},
	/* hu */ {IDGS_SOUND_BUFFER_OVERFLOW_HU, "Hangpuffer t˙lcsordul·s."},
	/* it */ {IDGS_SOUND_BUFFER_OVERFLOW_IT, "Overflow del buffer audio."},
	/* ko */ {IDGS_SOUND_BUFFER_OVERFLOW_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SOUND_BUFFER_OVERFLOW_NL, "Geluidsbuffer overloop."},
	/* pl */ {IDGS_SOUND_BUFFER_OVERFLOW_PL, "Przepe≥niony bufor dºwiÍku."},
	/* ru */ {IDGS_SOUND_BUFFER_OVERFLOW_RU, "Sound buffer overflow."},
	/* sv */ {IDGS_SOUND_BUFFER_OVERFLOW_SV, "Fˆr mycket data i ljudbufferten."},
	/* tr */ {IDGS_SOUND_BUFFER_OVERFLOW_TR, "Ses arabellek ta˛mas˝."},
#endif
	
	/* sound.c */
	/* en */ {IDGS_CANNOT_FLUSH,    N_("cannot flush.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_FLUSH_DA, "Kan ikke t¯mme."},
	/* de */ {IDGS_CANNOT_FLUSH_DE, "Entleerung nicht mˆglich."},
	/* es */ {IDGS_CANNOT_FLUSH_ES, "cannot flush."},
	/* fr */ {IDGS_CANNOT_FLUSH_FR, "impossible de vider."},
	/* hu */ {IDGS_CANNOT_FLUSH_HU, "nem siker¸lt ¸rÌteni a puffert."},
	/* it */ {IDGS_CANNOT_FLUSH_IT, "non Ë possibile svuotare."},
	/* ko */ {IDGS_CANNOT_FLUSH_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_FLUSH_NL, "kan niet leegmaken."},
	/* pl */ {IDGS_CANNOT_FLUSH_PL, "Nie moøna oprÛøniÊ."},
	/* ru */ {IDGS_CANNOT_FLUSH_RU, "cannot flush."},
	/* sv */ {IDGS_CANNOT_FLUSH_SV, "kan inte tˆmma."},
	/* tr */ {IDGS_CANNOT_FLUSH_TR, "temizlenemedi."},
#endif
	
	/* sound.c */
	/* en */ {IDGS_FRAGMENT_PROBLEMS,    N_("fragment problems.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_FRAGMENT_PROBLEMS_DA, "Fragmentproblem."},
	/* de */ {IDGS_FRAGMENT_PROBLEMS_DE, "Fragmentierungsproblem."},
	/* es */ {IDGS_FRAGMENT_PROBLEMS_ES, "Problema de fragmentaciÛn"},
	/* fr */ {IDGS_FRAGMENT_PROBLEMS_FR, "problËmes de fragments."},
	/* hu */ {IDGS_FRAGMENT_PROBLEMS_HU, "tˆredÈkrÈsz problÈm·k."},
	/* it */ {IDGS_FRAGMENT_PROBLEMS_IT, "problemi di frammentazione."},
	/* ko */ {IDGS_FRAGMENT_PROBLEMS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_FRAGMENT_PROBLEMS_NL, "fragment problemen."},
	/* pl */ {IDGS_FRAGMENT_PROBLEMS_PL, "Problemy z fragmentem"},
	/* ru */ {IDGS_FRAGMENT_PROBLEMS_RU, "fragment problems."},
	/* sv */ {IDGS_FRAGMENT_PROBLEMS_SV, "fragmentprogram."},
	/* tr */ {IDGS_FRAGMENT_PROBLEMS_TR, "parÁalara ay˝rma problemi."},
#endif
	
	/* sound.c */
	/* en */ {IDGS_RUNNING_TOO_SLOW,    N_("running too slow.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_RUNNING_TOO_SLOW_DA, "K¯rer for langsomt."},
	/* de */ {IDGS_RUNNING_TOO_SLOW_DE, "Ablauf zu langsam."},
	/* es */ {IDGS_RUNNING_TOO_SLOW_ES, "Funciona demasiado lento."},
	/* fr */ {IDGS_RUNNING_TOO_SLOW_FR, "exÈcution trop lente."},
	/* hu */ {IDGS_RUNNING_TOO_SLOW_HU, "t˙l lassan megy."},
	/* it */ {IDGS_RUNNING_TOO_SLOW_IT, "esecuzione troppo lenta."},
	/* ko */ {IDGS_RUNNING_TOO_SLOW_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_RUNNING_TOO_SLOW_NL, "draait te langzaam."},
	/* pl */ {IDGS_RUNNING_TOO_SLOW_PL, "dzia≥a za wolno."},
	/* ru */ {IDGS_RUNNING_TOO_SLOW_RU, "running too slow."},
	/* sv */ {IDGS_RUNNING_TOO_SLOW_SV, "gÂr fˆr lÂngsamt."},
	/* tr */ {IDGS_RUNNING_TOO_SLOW_TR, "Áok yava˛ Áal˝˛˝yor."},
#endif
	
	/* sound.c */
	/* en */ {IDGS_STORE_SOUNDDEVICE_FAILED,    N_("store to sounddevice failed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_STORE_SOUNDDEVICE_FAILED_DA, "skrivning til lydenhed fejlede."},
	/* de */ {IDGS_STORE_SOUNDDEVICE_FAILED_DE, "Speichern auf Sound Ger‰t ist fehlgeschlagen."},
	/* es */ {IDGS_STORE_SOUNDDEVICE_FAILED_ES, "Almacenamiento en perifÈrico de sonido fallido"},
	/* fr */ {IDGS_STORE_SOUNDDEVICE_FAILED_FR, "erreur d'enregistrement sur le pÈriphÈrique son."},
	/* hu */ {IDGS_STORE_SOUNDDEVICE_FAILED_HU, "a hangeszkˆzˆn tˆrtÈnı t·rol·s nem siker¸lt."},
	/* it */ {IDGS_STORE_SOUNDDEVICE_FAILED_IT, "memorizzazione sulla scheda audio fallita."},
	/* ko */ {IDGS_STORE_SOUNDDEVICE_FAILED_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_STORE_SOUNDDEVICE_FAILED_NL, "opslag naar geluidsapparaat is mislukt."},
	/* pl */ {IDGS_STORE_SOUNDDEVICE_FAILED_PL, "zapis do urz±dzenia dºwiÍku nie powiÛd≥ siÍ"},
	/* ru */ {IDGS_STORE_SOUNDDEVICE_FAILED_RU, "store to sounddevice failed."},
	/* sv */ {IDGS_STORE_SOUNDDEVICE_FAILED_SV, "misslyckades spara i ljudenheten."},
	/* tr */ {IDGS_STORE_SOUNDDEVICE_FAILED_TR, "ses ayg˝t˝na saklama ba˛ar˝s˝z oldu."},
#endif
	
#ifdef HAVE_NETWORK
	/* network.c */
	/* en */ {IDGS_TESTING_BEST_FRAME_DELAY,    N_("Testing best frame delay...")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_TESTING_BEST_FRAME_DELAY_DA, "Tester bedste billed-forsinkelse..."},
	/* de */ {IDGS_TESTING_BEST_FRAME_DELAY_DE, "Teste optimale Framerate..."},
	/* es */ {IDGS_TESTING_BEST_FRAME_DELAY_ES, "Probando mejor demora de cuadro..."},
	/* fr */ {IDGS_TESTING_BEST_FRAME_DELAY_FR, "Recherche du meilleur dÈlai d'image..."},
	/* hu */ {IDGS_TESTING_BEST_FRAME_DELAY_HU, ""},  /* fuzzy */
	/* it */ {IDGS_TESTING_BEST_FRAME_DELAY_IT, "Rileva il miglior ritardo tra frame..."},
	/* ko */ {IDGS_TESTING_BEST_FRAME_DELAY_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_TESTING_BEST_FRAME_DELAY_NL, "Beste framevertraging aan het testen..."},
	/* pl */ {IDGS_TESTING_BEST_FRAME_DELAY_PL, "Sprawdzam najlepsze opÛºnienie klatek..."},
	/* ru */ {IDGS_TESTING_BEST_FRAME_DELAY_RU, "Testing best frame delay..."},
	/* sv */ {IDGS_TESTING_BEST_FRAME_DELAY_SV, "Testar b‰sta ramfˆrdrˆjning..."},
	/* tr */ {IDGS_TESTING_BEST_FRAME_DELAY_TR, "En iyi frame bekleme s¸resi test ediliyor..."},
#endif
	
	/* network.c */
	/* en */ {IDGS_USING_D_FRAMES_DELAY,    N_("Using %d frames delay.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USING_D_FRAMES_DELAY_DA, "Bruger %d billeders forsinkelse."},
	/* de */ {IDGS_USING_D_FRAMES_DELAY_DE, "Benutze %d Frameverzˆgerung."},
	/* es */ {IDGS_USING_D_FRAMES_DELAY_ES, "Usando %d cuadros de demora."},
	/* fr */ {IDGS_USING_D_FRAMES_DELAY_FR, "Utilisation d'un dÈlai de %d images."},
	/* hu */ {IDGS_USING_D_FRAMES_DELAY_HU, "%d kÈpkocka kÈsleltetÈs haszn·lata."},
	/* it */ {IDGS_USING_D_FRAMES_DELAY_IT, "Utilizzo %d frame di ritardo."},
	/* ko */ {IDGS_USING_D_FRAMES_DELAY_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USING_D_FRAMES_DELAY_NL, "%d frames vertraging in gebruik."},
	/* pl */ {IDGS_USING_D_FRAMES_DELAY_PL, "Uøywam opÛºnienia klatek %d."},
	/* ru */ {IDGS_USING_D_FRAMES_DELAY_RU, "Using %d frames delay."},
	/* sv */ {IDGS_USING_D_FRAMES_DELAY_SV, "Anv‰nder %d ramars fˆrdrˆjning."},
	/* tr */ {IDGS_USING_D_FRAMES_DELAY_TR, "%d frame bekleme s¸resi kullan˝l˝yor."},
#endif
	
	/* network.c */
	/* en */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER,    N_("Cannot load snapshot file for transfer")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_DA, "Kunne ikke indlÊse snapshot til overf¯rsel."},
	/* de */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_DE, "Kann Snapshot Datei f¸r Transfer nicht laden"},
	/* es */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_ES, "No puedo cargar fichero Instant·nea  para trasferencia"},
	/* fr */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_FR, "Impossible de charger l'instantanÈ pour le transfer"},
	/* hu */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_HU, "Nem siker¸lt az ·tk¸ldendı pillanatkÈp f·jlt betˆlteni"},
	/* it */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_IT, "Non Ë possibile caricare il file di snapshot per il trasferimento"},
	/* ko */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_NL, "Kan het momentopnamebestand voor versturen niet laden"},
	/* pl */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_PL, "Nie moøna wczytaÊ pliku zrzutu do transferu"},
	/* ru */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_RU, "Cannot load snapshot file for transfer"},
	/* sv */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_SV, "Kan inte l‰sa in ˆgonblicksbildfil fˆr ˆverfˆring."},
	/* tr */ {IDGS_CANNOT_LOAD_SNAPSHOT_TRANSFER_TR, "Transfer iÁin anl˝k gˆr¸nt¸ dosyas˝ y¸klenemedi"},
#endif
	
	/* network.c */
	/* en */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT,    N_("Sending snapshot to client...")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_DA, "Sender snapshot til klient..."},
	/* de */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_DE, "Sende Snapshot zu Client..."},
	/* es */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_ES, "Remitiendo Instant·nea a cliente..."},
	/* fr */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_FR, "Envoi de l'instantanÈ au client..."},
	/* hu */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_IT, "Invio snapshot al client in corso..."},
	/* ko */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_NL, "Momentopname wordt naar de client verstuurd..."},
	/* pl */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_PL, "Wysy≥am zrzut do klienta..."},
	/* ru */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_RU, "Sending snapshot to client..."},
	/* sv */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_SV, "S‰nder ˆgonblicksbild till klient..."},
	/* tr */ {IDGS_SENDING_SNAPSHOT_TO_CLIENT_TR, "Anl˝k gˆr¸nt¸ dosyas˝ istemciye gˆnderiliyor..."},
#endif
	
	/* network.c */
	/* en */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT,    N_("Cannot send snapshot to client")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_DA, "Kunne ikke sende snapshot til klient"},
	/* de */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_DE, "Kann Snapshot Datei nicht senden"},
	/* es */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_ES, "No puedo remitir Instant·nea a cliente"},
	/* fr */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_FR, "Impossible d'envoyer l'instantanÈ au client"},
	/* hu */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_HU, "Nem siker¸lt pillanatkÈpet elk¸ldeni a klienshez"},
	/* it */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_IT, "Non Ë possibile inviare lo snapshot al client"},
	/* ko */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_NL, "Kan de momentopname niet naar de client versturen"},
	/* pl */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_PL, "Nie moøna wys≥aÊ zrzutu do klienta"},
	/* ru */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_RU, "Cannot send snapshot to client"},
	/* sv */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_SV, "Kan inte s‰nda ˆgonblicksbild till klient"},
	/* tr */ {IDGS_CANNOT_SEND_SNAPSHOT_TO_CLIENT_TR, "Anl˝k gˆr¸nt¸ dosyas˝ istemciye gˆnderilemedi"},
#endif
	
	/* network.c */
	/* en */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S,    N_("Cannot create snapshot file %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_DA, "Kunne ikke oprette snapshot-fil %s"},
	/* de */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_DE, "Kann Snapshot Datei `%s' nicht erzeugen"},
	/* es */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_ES, "No puedo crear fichero Instant·nea %s"},
	/* fr */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_FR, "Impossible de crÈer l'instantanÈ %s"},
	/* hu */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_HU, "Nem siker¸lt a pillanatkÈpf·jlt lÈtrehozni (%s)"},
	/* it */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_IT, "Non Ë possibile creare il file di snapshot %s"},
	/* ko */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_NL, "Kan het momentopnamebestand %s niet maken"},
	/* pl */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_PL, "Nie moøna utworzyÊ pliku zrzutu %s"},
	/* ru */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_RU, "Cannot create snapshot file %s"},
	/* sv */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_SV, "Kan inte skapa ˆgonblicksbildfil %s"},
	/* tr */ {IDGS_CANNOT_CREATE_SNAPSHOT_FILE_S_TR, "Anl˝k gˆr¸nt¸ dosyas˝ %s yarat˝lamad˝"},
#endif
	
	/* network.c */
	/* en */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S,    N_("Cannot open snapshot file %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_DA, "Kunne ikke Âbne snapshot-fil %s"},
	/* de */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_DE, "Kann Snapshot Datei `%s' nicht ˆffnen."},
	/* es */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_ES, "No puedo abrir fichero Instant·nea %s"},
	/* fr */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_FR, "Impossible d'ouvrir le fichier %s"},
	/* hu */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_HU, "Nem siker¸lt a pillanatkÈpf·jlt megnyitni (%s)"},
	/* it */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_IT, "Non Ë possibile aprire il file di snapshot %s"},
	/* ko */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_NL, "Kan het momentopnamebestand %s niet openen"},
	/* pl */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_PL, "Nie moøna otworzyÊ pliku zrzutu %s"},
	/* ru */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_RU, "Cannot open snapshot file %s"},
	/* sv */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_SV, "Kan inte ˆppna ˆgonblicksbildfil %s"},
	/* tr */ {IDGS_CANNOT_OPEN_SNAPSHOT_FILE_S_TR, "Anl˝k gˆr¸nt¸ dosyas˝ %s aÁ˝lamad˝"},
#endif
	
	/* network.c */
	/* en */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT,    N_("Server is waiting for a client...")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_DA, "Server venter pÂ en klient..."},
	/* de */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_DE, "Server wartet auf Client..."},
	/* es */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_ES, "El servidor est· esperando al cliente..."},
	/* fr */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_FR, "Le serveur attend un client..."},
	/* hu */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_IT, "Il server Ë in attesa di un client..."},
	/* ko */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_NL, "Server wacht op een client..."},
	/* pl */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_PL, "Serwer czeka na klienta..."},
	/* ru */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_RU, "Server is waiting for a client..."},
	/* sv */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_SV, "Servern v‰ntar pÂ en klient..."},
	/* tr */ {IDGS_SERVER_IS_WAITING_FOR_CLIENT_TR, "Sunucu bir istemcinin balanmas˝ iÁin bekliyor..."},
#endif
	
	/* network.c */
	/* en */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT,    N_("Cannot create snapshot file. Select different history directory!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_DA, "Kunne ikke oprette snapshot-fil. VÊlg et andet historik-katalog!"},
	/* de */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_DE, "Kann Snapshot Datei nicht erzeugen. Verwende ein anderes Verzeichnis!"},
	/* es */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_ES, "No puedo crear fichero Instant·nea. °Seleccionar diferente directorio histÛrico!"},
	/* fr */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_FR, "Impossible de crÈer le fichier d'instantanÈ. SÈlectionnez un rÈpertoire d'historique diffÈrent"},
	/* hu */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_HU, "Nem siker¸lt pillanatkÈp f·jlt lÈtrehozni. KÈrem v·lasszon m·s kˆnyvt·rat!"},
	/* it */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_IT, "Non Ë possibile creare il file di snapshot. Seleziona una directory diversa per la cronologia."},
	/* ko */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_NL, "Kan het momentopnamebestand niet maken. Selecteer een andere geschiedenismap!"},
	/* pl */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_PL, "Nie moøna utworzyÊ pliku zrzutu. Wybierz inny katalog historii!"},
	/* ru */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_RU, "Cannot create snapshot file. Select different history directory!"},
	/* sv */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_SV, "Kan inte skapa ˆgonblicksbildfil. V‰lj en annan historikkatalog!"},
	/* tr */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SELECT_TR, "Anl˝k gˆr¸nt¸ dosyas˝ yarat˝lamad˝. Farkl˝ bir tarihÁe dizini seÁin!"},
#endif
	
	/* network.c */
	/* en */ {IDGS_CANNOT_RESOLVE_S,    N_("Cannot resolve %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_RESOLVE_S_DA, "Kan ikke opslÂ %s"},
	/* de */ {IDGS_CANNOT_RESOLVE_S_DE, "Kann %s nicht auflˆsen"},
	/* es */ {IDGS_CANNOT_RESOLVE_S_ES, "No puedo resolver %s"},
	/* fr */ {IDGS_CANNOT_RESOLVE_S_FR, "Impossible de rÈsoudre %s"},
	/* hu */ {IDGS_CANNOT_RESOLVE_S_HU, "Nem siker¸lt %s-t feloldani"},
	/* it */ {IDGS_CANNOT_RESOLVE_S_IT, "Non Ë possibile risolvere %s"},
	/* ko */ {IDGS_CANNOT_RESOLVE_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_RESOLVE_S_NL, "Onbekende host %s"},
	/* pl */ {IDGS_CANNOT_RESOLVE_S_PL, "Nie moøna znaleºÊ %s"},
	/* ru */ {IDGS_CANNOT_RESOLVE_S_RU, "Cannot resolve %s"},
	/* sv */ {IDGS_CANNOT_RESOLVE_S_SV, "Kan inte slÂ upp %s"},
	/* tr */ {IDGS_CANNOT_RESOLVE_S_TR, "%s Áˆz¸lemedi"},
#endif
	
	/* network.c */
	/* en */ {IDGS_CANNOT_CONNECT_TO_S,    N_("Cannot connect to %s (no server running on port %d).")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_CONNECT_TO_S_DA, "Kan ikke forbinde til %s (ingen server pÂ port %d)."},
	/* de */ {IDGS_CANNOT_CONNECT_TO_S_DE, "Kann zu %s nicht verbinden (Kein Server aktiv auf Port %d)."},
	/* es */ {IDGS_CANNOT_CONNECT_TO_S_ES, "No puedo conectar a  %s (Ning˙n servidor en puerto %d)."},
	/* fr */ {IDGS_CANNOT_CONNECT_TO_S_FR, "Impossible de se connecter ‡ %s (aucun serveur sur le port %d)."},
	/* hu */ {IDGS_CANNOT_CONNECT_TO_S_HU, "Nem siker¸lt %s-hez kapcsolÛdni (a %d porton nem fut kiszolg·lÛ)"},
	/* it */ {IDGS_CANNOT_CONNECT_TO_S_IT, "Non Ë possibile connettersi a %s (nessun server Ë attivo sulla porta %d)."},
	/* ko */ {IDGS_CANNOT_CONNECT_TO_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_CONNECT_TO_S_NL, "Kan geen verbinding maken met %s (er is geen server aanwezig op poort %d)."},
	/* pl */ {IDGS_CANNOT_CONNECT_TO_S_PL, "Nie moøna pod≥±czyÊ do %s (na porcie %d nie dzia≥a øaden serwer)."},
	/* ru */ {IDGS_CANNOT_CONNECT_TO_S_RU, "Cannot connect to %s (no server running on port %d)."},
	/* sv */ {IDGS_CANNOT_CONNECT_TO_S_SV, "Kan inte ansluta till %s (ingen server igÂng pÂ port %d)."},
	/* tr */ {IDGS_CANNOT_CONNECT_TO_S_TR, "%s balant˝s˝ kurulamad˝ (port %d ¸zerinde Áal˝˛an sunucu yok)."},
#endif
	
	/* network.c */
	/* en */ {IDGS_RECEIVING_SNAPSHOT_SERVER,    N_("Receiving snapshot from server...")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_RECEIVING_SNAPSHOT_SERVER_DA, "Modtager snapshot fra server..."},
	/* de */ {IDGS_RECEIVING_SNAPSHOT_SERVER_DE, "Empfange Snapshot von Server..."},
	/* es */ {IDGS_RECEIVING_SNAPSHOT_SERVER_ES, "Recibiendo Instant·nea desde el servidor..."},
	/* fr */ {IDGS_RECEIVING_SNAPSHOT_SERVER_FR, "RÈception de l'instantanÈ du serveur..."},
	/* hu */ {IDGS_RECEIVING_SNAPSHOT_SERVER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_RECEIVING_SNAPSHOT_SERVER_IT, "Ricezione dello snapshot dal server in corso..."},
	/* ko */ {IDGS_RECEIVING_SNAPSHOT_SERVER_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_RECEIVING_SNAPSHOT_SERVER_NL, "Momentopname van de server wordt ontvangen..."},
	/* pl */ {IDGS_RECEIVING_SNAPSHOT_SERVER_PL, "Odbieranie zrzutu z serwera..."},
	/* ru */ {IDGS_RECEIVING_SNAPSHOT_SERVER_RU, "Receiving snapshot from server..."},
	/* sv */ {IDGS_RECEIVING_SNAPSHOT_SERVER_SV, "Tar emot ˆgoblicksbild frÂn server..."},
	/* tr */ {IDGS_RECEIVING_SNAPSHOT_SERVER_TR, "Sunucudan anl˝k gˆr¸nt¸ al˝n˝yor..."},
#endif
	
	/* network.c */
	/* en */ {IDGS_NETWORK_OUT_OF_SYNC,    N_("Network out of sync - disconnecting.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_NETWORK_OUT_OF_SYNC_DA, "NetvÊrk ude af synk - afbryder forbindelsen."},
	/* de */ {IDGS_NETWORK_OUT_OF_SYNC_DE, "Netzwerksynchronisationsproblem - Trenne Verbindung."},
	/* es */ {IDGS_NETWORK_OUT_OF_SYNC_ES, "Red fuera de sincronÌa - desconectando."},
	/* fr */ {IDGS_NETWORK_OUT_OF_SYNC_FR, "Le rÈseau n'est pas synchronisÈ (out of sync error). DÈconnexion."},
	/* hu */ {IDGS_NETWORK_OUT_OF_SYNC_HU, "A h·lÛzat kiesett a szinkronbÛl - szÈtkapcsolÛd·s"},
	/* it */ {IDGS_NETWORK_OUT_OF_SYNC_IT, "Rete non sincronizzata - disconnesione in corso."},
	/* ko */ {IDGS_NETWORK_OUT_OF_SYNC_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_NETWORK_OUT_OF_SYNC_NL, "Netwerk niet synchroon - verbinding wordt verbroken."},
	/* pl */ {IDGS_NETWORK_OUT_OF_SYNC_PL, "Desynchronizacja sieci - roz≥±czam."},
	/* ru */ {IDGS_NETWORK_OUT_OF_SYNC_RU, "Network out of sync - disconnecting."},
	/* sv */ {IDGS_NETWORK_OUT_OF_SYNC_SV, "N‰tverket ur synk - kopplar frÂn."},
	/* tr */ {IDGS_NETWORK_OUT_OF_SYNC_TR, "A senkronizasyonu bozuldu - balant˝ kesiliyor."},
#endif
	
	/* network.c */
	/* en */ {IDGS_REMOTE_HOST_DISCONNECTED,    N_("Remote host disconnected.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_REMOTE_HOST_DISCONNECTED_DA, "Fjernmaskine afbrudt."},
	/* de */ {IDGS_REMOTE_HOST_DISCONNECTED_DE, "Entfernter Rechner getrennt."},
	/* es */ {IDGS_REMOTE_HOST_DISCONNECTED_ES, "Host Remoto desconectado."},
	/* fr */ {IDGS_REMOTE_HOST_DISCONNECTED_FR, "L'hÙte distant s'est dÈconnectÈ."},
	/* hu */ {IDGS_REMOTE_HOST_DISCONNECTED_HU, "A t·voli sz·mÌtogÈp lekapcsolÛdott."},
	/* it */ {IDGS_REMOTE_HOST_DISCONNECTED_IT, "Host remoto disconnesso."},
	/* ko */ {IDGS_REMOTE_HOST_DISCONNECTED_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_REMOTE_HOST_DISCONNECTED_NL, "De andere computer heeft de verbinding verbroken."},
	/* pl */ {IDGS_REMOTE_HOST_DISCONNECTED_PL, "Zdalny host roz≥±czy≥ siÍ."},
	/* ru */ {IDGS_REMOTE_HOST_DISCONNECTED_RU, "Remote host disconnected."},
	/* sv */ {IDGS_REMOTE_HOST_DISCONNECTED_SV, "FrÂnkopplad frÂn fj‰rrv‰rd."},
	/* tr */ {IDGS_REMOTE_HOST_DISCONNECTED_TR, "Uzak sunucu balant˝s˝ kesildi."},
#endif
	
	/* network.c */
	/* en */ {IDGS_REMOTE_HOST_SUSPENDING,    N_("Remote host suspending...")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_REMOTE_HOST_SUSPENDING_DA, "Fjernmaskine pauser..."},
	/* de */ {IDGS_REMOTE_HOST_SUSPENDING_DE, "Entfernter Rechner angehaltedn..."},
	/* es */ {IDGS_REMOTE_HOST_SUSPENDING_ES, "Host remoto suspendido..."},
	/* fr */ {IDGS_REMOTE_HOST_SUSPENDING_FR, "L'hÙte distant ‡ suspendu..."},
	/* hu */ {IDGS_REMOTE_HOST_SUSPENDING_HU, ""},  /* fuzzy */
	/* it */ {IDGS_REMOTE_HOST_SUSPENDING_IT, "Sospesione host remoto in corso..."},
	/* ko */ {IDGS_REMOTE_HOST_SUSPENDING_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_REMOTE_HOST_SUSPENDING_NL, "De andere computer onderbreekt de netplay..."},
	/* pl */ {IDGS_REMOTE_HOST_SUSPENDING_PL, "Zawieszenie zdalnego hosta...."},
	/* ru */ {IDGS_REMOTE_HOST_SUSPENDING_RU, "Remote host suspending..."},
	/* sv */ {IDGS_REMOTE_HOST_SUSPENDING_SV, "FJ‰rrv‰rden pausar..."},
	/* tr */ {IDGS_REMOTE_HOST_SUSPENDING_TR, "Uzak sunucu ask˝da..."},
#endif
#endif
	
	/* gfxoutputdrv/ffmpegdrv.c */
	/* en */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM,    N_("ffmpegdrv: Cannot open video stream")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_DA, "ffmpegdrv: Kan ikke Âbne video-str¯m"},
	/* de */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_DE, "ffmpegdrv: Kann video stream nicht ˆffnen"},
	/* es */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_ES, "ffmpegdrv: no puedo abrir vÌdeo stream"},
	/* fr */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_FR, "ffmpegdrv: Impossible d'ouvrir le flux vidÈo"},
	/* hu */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_HU, "ffmpegdrv: Nem siker¸lt megnyitni a videÛ adatfolyamot"},
	/* it */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_IT, "ffmpegdrv: Non Ë possibile aprire il flusso video"},
	/* ko */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_NL, "ffmpegdrv: Kan videostroom niet openen"},
	/* pl */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_PL, "ffmpegdrv: Nie moøna otworzyÊ strumienia wideo"},
	/* ru */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_RU, "ffmpegdrv: Ω’ „‘–’‚·Ô ﬁ‚⁄‡Î‚Ï “ÿ‘’ﬁ ﬂﬁ‚ﬁ⁄"},
	/* sv */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_SV, "ffmpegdrv: Kan inte ˆppna videostrˆm"},
	/* tr */ {IDGS_FFMPEG_CANNOT_OPEN_VSTREAM_TR, "ffmpegdrv: Gˆr¸nt¸ ak˝˛˝ aÁ˝lamad˝"},
#endif
	
	/* gfxoutputdrv/ffmpegdrv.c */
	/* en */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM,    N_("ffmpegdrv: Cannot open audio stream")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_DA, "ffmpegdrv: Kan ikke Âbne lyd-str¯m"},
	/* de */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_DE, "ffmpegdrv: Kann audio stream nicht ˆffnen"},
	/* es */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_ES, "ffmpegdrv: No puedo abrir audio stream"},
	/* fr */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_FR, "ffmpegdrv: Impossible d'ouvrir le flux audio"},
	/* hu */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_HU, "ffmpegdrv: Nem siker¸lt megnyitni a audiÛ adatfolyamot"},
	/* it */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_IT, "ffmpegdrv: Non Ë possibile aprire il flusso audio"},
	/* ko */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_NL, "ffmpegdrv: Kan audiostroom niet openen"},
	/* pl */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_PL, "ffmpegdrv: Nie moøna otworzyÊ strumienia audio"},
	/* ru */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_RU, "ffmpegdrv: Ω’ „‘–’‚·Ô ﬁ‚⁄‡Î‚Ï –„‘ÿﬁ ﬂﬁ‚ﬁ⁄"},
	/* sv */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_SV, "ffmpegdrv: Kan inte ˆppna ljudstrˆm"},
	/* tr */ {IDGS_FFMPEG_CANNOT_OPEN_ASTREAM_TR, "ffmpegdrv: Ses ak˝˛˝ aÁ˝lamad˝"},
#endif
	
	/* gfxoutputdrv/ffmpegdrv.c */
	/* en */ {IDGS_FFMPEG_CANNOT_OPEN_S,    N_("ffmpegdrv: Cannot open %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_FFMPEG_CANNOT_OPEN_S_DA, "ffmpegdrv: Kan ikke Âbne %s"},
	/* de */ {IDGS_FFMPEG_CANNOT_OPEN_S_DE, "ffmpegdrv: ÷ffnen von %s fehlgeschlagen"},
	/* es */ {IDGS_FFMPEG_CANNOT_OPEN_S_ES, "ffmpegdrv: No puedo abrir %s"},
	/* fr */ {IDGS_FFMPEG_CANNOT_OPEN_S_FR, "ffmpegdrv: Impossible d'ouvrir %s"},
	/* hu */ {IDGS_FFMPEG_CANNOT_OPEN_S_HU, "ffmpegdrv: Nem siker¸lt megnyitni: %s"},
	/* it */ {IDGS_FFMPEG_CANNOT_OPEN_S_IT, "ffmpegdrv: Non Ë possibile aprire %s"},
	/* ko */ {IDGS_FFMPEG_CANNOT_OPEN_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_FFMPEG_CANNOT_OPEN_S_NL, "ffmpegdrv: Kan %s niet openen"},
	/* pl */ {IDGS_FFMPEG_CANNOT_OPEN_S_PL, "ffmpegdrv: Nie moøna otworzyÊ %s"},
	/* ru */ {IDGS_FFMPEG_CANNOT_OPEN_S_RU, "ffmpegdrv: Cannot open %s"},
	/* sv */ {IDGS_FFMPEG_CANNOT_OPEN_S_SV, "ffmpegdrv: Kan inte ˆppna %s"},
	/* tr */ {IDGS_FFMPEG_CANNOT_OPEN_S_TR, "ffmpegdrv: %s aÁ˝lamad˝"},
#endif
	
	/* c64/c64io.c, vic20/vic20io.c */
	/* en */ {IDGS_IO_READ_COLL_AT_X_FROM,    N_("I/O read collision at %X from ")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_IO_READ_COLL_AT_X_FROM_DA, "I/O-lÊsekollision ved %X fra "},
	/* de */ {IDGS_IO_READ_COLL_AT_X_FROM_DE, "I/O Lese Kollision bei %X von "},
	/* es */ {IDGS_IO_READ_COLL_AT_X_FROM_ES, "Entrada/Salida real colisiÛn en %X para"},
	/* fr */ {IDGS_IO_READ_COLL_AT_X_FROM_FR, "Collision lors de la lecture E/S at %X ‡ partir de "},
	/* hu */ {IDGS_IO_READ_COLL_AT_X_FROM_HU, ""},  /* fuzzy */
	/* it */ {IDGS_IO_READ_COLL_AT_X_FROM_IT, "Collisione di I/O in lettura a %X da "},
	/* ko */ {IDGS_IO_READ_COLL_AT_X_FROM_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_IO_READ_COLL_AT_X_FROM_NL, "I/O lees conflict op %X van "},
	/* pl */ {IDGS_IO_READ_COLL_AT_X_FROM_PL, "Kolizja odczytu we/wy w %X z"},
	/* ru */ {IDGS_IO_READ_COLL_AT_X_FROM_RU, "I/O read collision at %X from "},
	/* sv */ {IDGS_IO_READ_COLL_AT_X_FROM_SV, "I/O-l‰skollision vid %X frÂn "},
	/* tr */ {IDGS_IO_READ_COLL_AT_X_FROM_TR, "G/« okuma Áak˝˛mas˝ %X'de "},
#endif
	
	/* c64/c64io.c, vic20/vic20io.c */
	/* en */ {IDGS_AND,    N_(" and ")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_AND_DA, " og "},
	/* de */ {IDGS_AND_DE, " und "},
	/* es */ {IDGS_AND_ES, " y "},
	/* fr */ {IDGS_AND_FR, " et "},
	/* hu */ {IDGS_AND_HU, ""},  /* fuzzy */
	/* it */ {IDGS_AND_IT, " e "},
	/* ko */ {IDGS_AND_KO, " ±◊∏Æ∞Ì "},
	/* nl */ {IDGS_AND_NL, " en "},
	/* pl */ {IDGS_AND_PL, "i"},
	/* ru */ {IDGS_AND_RU, "ÿ"},
	/* sv */ {IDGS_AND_SV, " och "},
	/* tr */ {IDGS_AND_TR, " ve "},
#endif
	
	/* c64/c64io.c, vic20/vic20io.c */
	/* en */ {IDGS_ALL_DEVICES_DETACHED,    N_(".\nAll the named devices will be detached.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ALL_DEVICES_DETACHED_DA, ".\nAlle de navngivne enheder vil blive frakoblet."},
	/* de */ {IDGS_ALL_DEVICES_DETACHED_DE, ".\nAlle genannten Ger‰te werden entfernt."},
	/* es */ {IDGS_ALL_DEVICES_DETACHED_ES, ".\nTodos los perifÈricos nombrados se desconectar·n."},
	/* fr */ {IDGS_ALL_DEVICES_DETACHED_FR, ".\nTous les pÈriphÈriques nommÈs vont Ítre dÈtachÈs."},
	/* hu */ {IDGS_ALL_DEVICES_DETACHED_HU, ".\nMinden megnevezett eszkˆz le lesz v·lasztva."},
	/* it */ {IDGS_ALL_DEVICES_DETACHED_IT, ".\nTutti i device specificati sarranno rimossi."},
	/* ko */ {IDGS_ALL_DEVICES_DETACHED_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ALL_DEVICES_DETACHED_NL, ".\nAlle genoemde apparaten zullen worden ontkoppeld."},
	/* pl */ {IDGS_ALL_DEVICES_DETACHED_PL, ".\nWszystkie wymienione urz±dzenia zostan± odmontowane."},
	/* ru */ {IDGS_ALL_DEVICES_DETACHED_RU, ".\nAll the named devices will be detached."},
	/* sv */ {IDGS_ALL_DEVICES_DETACHED_SV, ".\nAlla namngivna enheter kommer kopplas frÂn."},
	/* tr */ {IDGS_ALL_DEVICES_DETACHED_TR, ".\nT¸m isimlendirilmi˛ ayg˝tlar Á˝kart˝lacakt˝r."},
#endif
	
	/* c64/c64io.c, vic20/vic20io.c */
	/* en */ {IDGS_ALL_DEVICES_EXCEPT,    N_(".\nAll devices except ")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ALL_DEVICES_EXCEPT_DA, ".\nAlle enheder undtagen "},
	/* de */ {IDGS_ALL_DEVICES_EXCEPT_DE, ".\nAlle Ger‰te auﬂer "},
	/* es */ {IDGS_ALL_DEVICES_EXCEPT_ES, ".\nTodos los perifÈricos excepto"},
	/* fr */ {IDGS_ALL_DEVICES_EXCEPT_FR, ".\nTous pÈriphÈriques exceptÈ  "},
	/* hu */ {IDGS_ALL_DEVICES_EXCEPT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ALL_DEVICES_EXCEPT_IT, ".\nTutti i device tranne "},
	/* ko */ {IDGS_ALL_DEVICES_EXCEPT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ALL_DEVICES_EXCEPT_NL, ".\nAlle apparaten behalve "},
	/* pl */ {IDGS_ALL_DEVICES_EXCEPT_PL, ".\nWszystkie urz±dzenia, prÛcz"},
	/* ru */ {IDGS_ALL_DEVICES_EXCEPT_RU, ".\nAll devices except "},
	/* sv */ {IDGS_ALL_DEVICES_EXCEPT_SV, ".\nAlla enheter fˆrutom "},
	/* tr */ {IDGS_ALL_DEVICES_EXCEPT_TR, ".\nBunlar hariÁ t¸m cihazlar "},
#endif
	
	/* c64/c64io.c, vic20/vic20io.c */
	/* en */ {IDGS_WILL_BE_DETACHED,    N_(" will be detached.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_WILL_BE_DETACHED_DA, " vil blive frakoblet."},
	/* de */ {IDGS_WILL_BE_DETACHED_DE, " werden entfernt."},
	/* es */ {IDGS_WILL_BE_DETACHED_ES, "se desconectar·n"},
	/* fr */ {IDGS_WILL_BE_DETACHED_FR, " vont Ítre dÈtachÈs."},
	/* hu */ {IDGS_WILL_BE_DETACHED_HU, ""},  /* fuzzy */
	/* it */ {IDGS_WILL_BE_DETACHED_IT, "sarranno rimossi."},
	/* ko */ {IDGS_WILL_BE_DETACHED_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_WILL_BE_DETACHED_NL, " zullen worden ontkoppeld."},
	/* pl */ {IDGS_WILL_BE_DETACHED_PL, " zostanie roz≥±czone."},
	/* ru */ {IDGS_WILL_BE_DETACHED_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_WILL_BE_DETACHED_SV, " kommer kopplas frÂn."},
	/* tr */ {IDGS_WILL_BE_DETACHED_TR, " haricindeki t¸m cihazlar Á˝kart˝lacakt˝r."},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDGS_NO_FILENAME,    N_("No filename!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_NO_FILENAME_DA, "Intet filnavn!"},
	/* de */ {IDGS_NO_FILENAME_DE, "Dateiname fehlt!"},
	/* es */ {IDGS_NO_FILENAME_ES, "°Sin nombre de fichero!"},
	/* fr */ {IDGS_NO_FILENAME_FR, "Aucun fichier!"},
	/* hu */ {IDGS_NO_FILENAME_HU, "Nincs f·jlnÈv!"},
	/* it */ {IDGS_NO_FILENAME_IT, "Nessun nome file!"},
	/* ko */ {IDGS_NO_FILENAME_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_NO_FILENAME_NL, "Geen bestandsnaam!"},
	/* pl */ {IDGS_NO_FILENAME_PL, "Brak nazwy pliku!"},
	/* ru */ {IDGS_NO_FILENAME_RU, "Ω’‚ ÿ‹’›ÿ ‰–Ÿ€–!"},
	/* sv */ {IDGS_NO_FILENAME_SV, "Filnamn saknas!"},
	/* tr */ {IDGS_NO_FILENAME_TR, "Dosya ismi yok!"},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDGS_ILLEGAL_FILENAME,    N_("Illegal filename!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ILLEGAL_FILENAME_DA, "Ugyldigt filnavn!"},
	/* de */ {IDGS_ILLEGAL_FILENAME_DE, "Ung¸ltiger Dateiname!"},
	/* es */ {IDGS_ILLEGAL_FILENAME_ES, "°Nombre fichero ilegal!"},
	/* fr */ {IDGS_ILLEGAL_FILENAME_FR, "Nom de fichier illÈgal!"},
	/* hu */ {IDGS_ILLEGAL_FILENAME_HU, "…rvÈnytelen f·jlnÈv!"},
	/* it */ {IDGS_ILLEGAL_FILENAME_IT, "Nome file non valido!"},
	/* ko */ {IDGS_ILLEGAL_FILENAME_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ILLEGAL_FILENAME_NL, "Bestandsnaam ongeldig!"},
	/* pl */ {IDGS_ILLEGAL_FILENAME_PL, "Z≥a nazwa pliku!"},
	/* ru */ {IDGS_ILLEGAL_FILENAME_RU, "Illegal filename!"},
	/* sv */ {IDGS_ILLEGAL_FILENAME_SV, "Ogiltigt filnamn!"},
	/* tr */ {IDGS_ILLEGAL_FILENAME_TR, "Kural d˝˛˝ dosya ismi!"},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDGS_ERROR_CREATING_FILE_S,    N_("Error creating file %s!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_CREATING_FILE_S_DA, "Fejl under oprettelse af filen %s!"},
	/* de */ {IDGS_ERROR_CREATING_FILE_S_DE, "Fehler beim Schreiben: %s."},
	/* es */ {IDGS_ERROR_CREATING_FILE_S_ES, "Error creando fichero %s!"},
	/* fr */ {IDGS_ERROR_CREATING_FILE_S_FR, "Erreur lors de la crÈation du fichier %s!"},
	/* hu */ {IDGS_ERROR_CREATING_FILE_S_HU, "Hiba %s f·jl lÈtrehoz·sa kˆzben!"},
	/* it */ {IDGS_ERROR_CREATING_FILE_S_IT, "Errore durante la creazione del file %s!"},
	/* ko */ {IDGS_ERROR_CREATING_FILE_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_CREATING_FILE_S_NL, "Fout bij het maken van bestand %s!"},
	/* pl */ {IDGS_ERROR_CREATING_FILE_S_PL, "B≥±d tworzenia pliku %s!"},
	/* ru */ {IDGS_ERROR_CREATING_FILE_S_RU, "Error creating file %s!"},
	/* sv */ {IDGS_ERROR_CREATING_FILE_S_SV, "Fel n‰r filen \"%s\" skulle skapas!"},
	/* tr */ {IDGS_ERROR_CREATING_FILE_S_TR, "%s dosyas˝ yarat˝l˝rken hata olu˛tu!"},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDGS_ERROR_WRITING_TO_FILE_S,    N_("Error writing to file %s!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_WRITING_TO_FILE_S_DA, "Fejl ved skrivning til filen %s!"},
	/* de */ {IDGS_ERROR_WRITING_TO_FILE_S_DE, "Fehler bei Schreibzugriff: %s!"},
	/* es */ {IDGS_ERROR_WRITING_TO_FILE_S_ES, "Error grabando fichero %s!"},
	/* fr */ {IDGS_ERROR_WRITING_TO_FILE_S_FR, "Erreur lors de l'Ècriture dans le fichier %s!"},
	/* hu */ {IDGS_ERROR_WRITING_TO_FILE_S_HU, "Hiba %s f·jlba Ìr·s kˆzben!"},
	/* it */ {IDGS_ERROR_WRITING_TO_FILE_S_IT, "Errore durante la scrittura sul file %s!"},
	/* ko */ {IDGS_ERROR_WRITING_TO_FILE_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_WRITING_TO_FILE_S_NL, "Fout bij het schrijven naar bestand %s!"},
	/* pl */ {IDGS_ERROR_WRITING_TO_FILE_S_PL, "B≥±d zapisu do pliku %s!"},
	/* ru */ {IDGS_ERROR_WRITING_TO_FILE_S_RU, "Error writing to file %s!"},
	/* sv */ {IDGS_ERROR_WRITING_TO_FILE_S_SV, "Fel vid skrivning till filen %s!"},
	/* tr */ {IDGS_ERROR_WRITING_TO_FILE_S_TR, "%s dosyas˝na yaz˝l˝rken hata olu˛tu!"},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDGS_DTV_ROM_CREATED,    N_("DTV ROM image created successfully")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DTV_ROM_CREATED_DA, "DTV-ROM-image oprettet med success"},
	/* de */ {IDGS_DTV_ROM_CREATED_DE, "DTV ROM Image erfolgreich erzeugt"},
	/* es */ {IDGS_DTV_ROM_CREATED_ES, "Imagen DTV ROM creada con Èxito"},
	/* fr */ {IDGS_DTV_ROM_CREATED_FR, "Image ROM DTV crÈÈe correctement"},
	/* hu */ {IDGS_DTV_ROM_CREATED_HU, "DTV ROM kÈpm·s sikeresen lÈtrehozva"},
	/* it */ {IDGS_DTV_ROM_CREATED_IT, "Immagine ROM DTV creata con successo"},
	/* ko */ {IDGS_DTV_ROM_CREATED_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DTV_ROM_CREATED_NL, "DTV ROM bestand gemaakt!"},
	/* pl */ {IDGS_DTV_ROM_CREATED_PL, "Z powodzeniem utworzono obraz ROM DTV"},
	/* ru */ {IDGS_DTV_ROM_CREATED_RU, "æ—‡–◊ DTV ROM ·ﬁ◊‘–› „·ﬂ’Ë›ﬁ"},
	/* sv */ {IDGS_DTV_ROM_CREATED_SV, "DTV-ROM-avbildning skapades framgÂngsrikt"},
	/* tr */ {IDGS_DTV_ROM_CREATED_TR, "DTV ROM imaj˝ ba˛ar˝yla olu˛turuldu"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_TYPE,    N_("Type")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_TYPE_DA, "Type"},
	/* de */ {IDGS_TYPE_DE, "Typ"},
	/* es */ {IDGS_TYPE_ES, "Tipo"},
	/* fr */ {IDGS_TYPE_FR, "type"},
	/* hu */ {IDGS_TYPE_HU, "TÌpus"},
	/* it */ {IDGS_TYPE_IT, "Tipo"},
	/* ko */ {IDGS_TYPE_KO, "¡æ∑˘"},
	/* nl */ {IDGS_TYPE_NL, "Soort"},
	/* pl */ {IDGS_TYPE_PL, "Typ"},
	/* ru */ {IDGS_TYPE_RU, "Type"},
	/* sv */ {IDGS_TYPE_SV, "Typ"},
	/* tr */ {IDGS_TYPE_TR, "Tip"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_COMMAND_S_UNKNOWN,    N_("Command `%s' unknown.\n")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_COMMAND_S_UNKNOWN_DA, "Ukendt kommando \"%s\".\n"},
	/* de */ {IDGS_COMMAND_S_UNKNOWN_DE, "Kommand `%s' ist unbekannt.\n"},
	/* es */ {IDGS_COMMAND_S_UNKNOWN_ES, "Comando `%s' desconocido.\n"},
	/* fr */ {IDGS_COMMAND_S_UNKNOWN_FR, "Commande `%s' inconnue.\n"},
	/* hu */ {IDGS_COMMAND_S_UNKNOWN_HU, "`%s' parancs ismeretlen.\n"},
	/* it */ {IDGS_COMMAND_S_UNKNOWN_IT, "Comando `%s' sconosciuto.\n"},
	/* ko */ {IDGS_COMMAND_S_UNKNOWN_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_COMMAND_S_UNKNOWN_NL, "Commando `%s' is onbekend.\n"},
	/* pl */ {IDGS_COMMAND_S_UNKNOWN_PL, "Polecenie `%s' nieznane.\n"},
	/* ru */ {IDGS_COMMAND_S_UNKNOWN_RU, "Command `%s' unknown.\n"},
	/* sv */ {IDGS_COMMAND_S_UNKNOWN_SV, "Kommandot \"%s\" ‰r ok‰nt.\n"},
	/* tr */ {IDGS_COMMAND_S_UNKNOWN_TR, "`%s' komutu bilinmiyor.\n"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_NO_HELP_AVAILABLE_FOR_S,    N_("No help available for `%s'\n")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_NO_HELP_AVAILABLE_FOR_S_DA, "Ingen hjÊlp tilgÊngelig for \"%s\"\n"},
	/* de */ {IDGS_NO_HELP_AVAILABLE_FOR_S_DE, "Es ist keine Hilfe f¸r `%s' verf¸gbar\n"},
	/* es */ {IDGS_NO_HELP_AVAILABLE_FOR_S_ES, "Sin ayuda disponible para `%s'\n"},
	/* fr */ {IDGS_NO_HELP_AVAILABLE_FOR_S_FR, "Aucune aide ‡ propos de`%s'\n"},
	/* hu */ {IDGS_NO_HELP_AVAILABLE_FOR_S_HU, "Nincs s˙gÛ a `%s' parancshoz\n"},
	/* it */ {IDGS_NO_HELP_AVAILABLE_FOR_S_IT, "Nessun help disponibile per `%s'\n"},
	/* ko */ {IDGS_NO_HELP_AVAILABLE_FOR_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_NO_HELP_AVAILABLE_FOR_S_NL, "Geen hulp beschikbaar voor `%s'\n"},
	/* pl */ {IDGS_NO_HELP_AVAILABLE_FOR_S_PL, "Pomoc dla '%s' niedostÍpna\n"},
	/* ru */ {IDGS_NO_HELP_AVAILABLE_FOR_S_RU, "No help available for `%s'\n"},
	/* sv */ {IDGS_NO_HELP_AVAILABLE_FOR_S_SV, "Hj‰lp saknas fˆr \"%s\"\n"},
	/* tr */ {IDGS_NO_HELP_AVAILABLE_FOR_S_TR, "`%s' iÁin mevcut bir yard˝m metni yok\n"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_SYNTAX_S_S,    N_("\nSyntax: %s %s\n")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SYNTAX_S_S_DA, "\nSyntax: %s %s\n"},
	/* de */ {IDGS_SYNTAX_S_S_DE, "\nSyntax: %s %s\n"},
	/* es */ {IDGS_SYNTAX_S_S_ES, "\nSintaxis: %s %s\n"},
	/* fr */ {IDGS_SYNTAX_S_S_FR, "\nSyntaxe: %s %s\n"},
	/* hu */ {IDGS_SYNTAX_S_S_HU, "\nSzintaxis: %s %s\n"},
	/* it */ {IDGS_SYNTAX_S_S_IT, "\nSintassi: %s %s\n"},
	/* ko */ {IDGS_SYNTAX_S_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SYNTAX_S_S_NL, "\nSyntaxis: %s %s\n"},
	/* pl */ {IDGS_SYNTAX_S_S_PL, "\nSk≥adnia: %s %s\n"},
	/* ru */ {IDGS_SYNTAX_S_S_RU, "\nSyntax: %s %s\n"},
	/* sv */ {IDGS_SYNTAX_S_S_SV, "\nSyntax: %s %s\n"},
	/* tr */ {IDGS_SYNTAX_S_S_TR, "\nSˆzdizimi: %s %s\n"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_ABBREVIATION_S,    N_("Abbreviation: %s\n")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ABBREVIATION_S_DA, "Forkortelse: %s\n"},
	/* de */ {IDGS_ABBREVIATION_S_DE, "Abk¸rzung: %s\n"},
	/* es */ {IDGS_ABBREVIATION_S_ES, "AbreviaciÛn: %s\n"},
	/* fr */ {IDGS_ABBREVIATION_S_FR, "AbbrÈviation: %s\n"},
	/* hu */ {IDGS_ABBREVIATION_S_HU, "RˆvidÌtÈs: %s\n"},
	/* it */ {IDGS_ABBREVIATION_S_IT, "Abbreviazione: %s\n"},
	/* ko */ {IDGS_ABBREVIATION_S_KO, "√‡æ‡: %s\n"},
	/* nl */ {IDGS_ABBREVIATION_S_NL, "Afkorting: %s\n"},
	/* pl */ {IDGS_ABBREVIATION_S_PL, "SkrÛt: %s\n"},
	/* ru */ {IDGS_ABBREVIATION_S_RU, "Abbreviation: %s\n"},
	/* sv */ {IDGS_ABBREVIATION_S_SV, "Fˆrkortning: %s\n"},
	/* tr */ {IDGS_ABBREVIATION_S_TR, "K˝saltma: %s\n"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_AVAILABLE_COMMANDS_ARE,    N_("Available commands are:")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_AVAILABLE_COMMANDS_ARE_DA, "TilgÊngelige kommandoer:"},
	/* de */ {IDGS_AVAILABLE_COMMANDS_ARE_DE, "Zul‰ssige Kommandos sind:"},
	/* es */ {IDGS_AVAILABLE_COMMANDS_ARE_ES, "Los comandos disponibles son:"},
	/* fr */ {IDGS_AVAILABLE_COMMANDS_ARE_FR, "Les commandes disponibles sont:"},
	/* hu */ {IDGS_AVAILABLE_COMMANDS_ARE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_AVAILABLE_COMMANDS_ARE_IT, "I comandi disponibili sono:"},
	/* ko */ {IDGS_AVAILABLE_COMMANDS_ARE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_AVAILABLE_COMMANDS_ARE_NL, "Beschikbare commando's zijn:"},
	/* pl */ {IDGS_AVAILABLE_COMMANDS_ARE_PL, "DostÍpne opcje to:"},
	/* ru */ {IDGS_AVAILABLE_COMMANDS_ARE_RU, "Available commands are:"},
	/* sv */ {IDGS_AVAILABLE_COMMANDS_ARE_SV, "Tillg‰ngliga kommandon:"},
	/* tr */ {IDGS_AVAILABLE_COMMANDS_ARE_TR, "Kullan˝labilir komutlar:"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MACHINE_STATE_COMMANDS,    N_("Machine state commands:")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MACHINE_STATE_COMMANDS_DA, "Kommandoer for maskinstatus:"},
	/* de */ {IDGS_MACHINE_STATE_COMMANDS_DE, "Maschinen Zustandskommandos:"},
	/* es */ {IDGS_MACHINE_STATE_COMMANDS_ES, "Comandos estado m·quina:"},
	/* fr */ {IDGS_MACHINE_STATE_COMMANDS_FR, "Commandes de statut de machine:"},
	/* hu */ {IDGS_MACHINE_STATE_COMMANDS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MACHINE_STATE_COMMANDS_IT, "Comandi di stato del computer:"},
	/* ko */ {IDGS_MACHINE_STATE_COMMANDS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MACHINE_STATE_COMMANDS_NL, "Machine status commando's:"},
	/* pl */ {IDGS_MACHINE_STATE_COMMANDS_PL, "Komendy stanu maszyny:"},
	/* ru */ {IDGS_MACHINE_STATE_COMMANDS_RU, "Machine state commands:"},
	/* sv */ {IDGS_MACHINE_STATE_COMMANDS_SV, "Kommandon fˆr maskinstatus:"},
	/* tr */ {IDGS_MACHINE_STATE_COMMANDS_TR, "Makine durum komutlar˝:"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MEMSPACE,    N_("memspace")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MEMSPACE_DA, "huk.omrÂde"},
	/* de */ {IDGS_MEMSPACE_DE, "Speicherbereich"},
	/* es */ {IDGS_MEMSPACE_ES, "espacio de memoria"},
	/* fr */ {IDGS_MEMSPACE_FR, "espace mÈmoire"},
	/* hu */ {IDGS_MEMSPACE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MEMSPACE_IT, "spazio di memoria"},
	/* ko */ {IDGS_MEMSPACE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MEMSPACE_NL, "geheugenruimte"},
	/* pl */ {IDGS_MEMSPACE_PL, "memspace"},
	/* ru */ {IDGS_MEMSPACE_RU, "memspace"},
	/* sv */ {IDGS_MEMSPACE_SV, "minnesomrÂde"},
	/* tr */ {IDGS_MEMSPACE_TR, "bellekalan˝"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_BANKNAME,    N_("bankname")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_BANKNAME_DA, "banknavn"},
	/* de */ {IDGS_BANKNAME_DE, "Speicherbank"},
	/* es */ {IDGS_BANKNAME_ES, "Nombre de banco"},
	/* fr */ {IDGS_BANKNAME_FR, "nom de la banque"},
	/* hu */ {IDGS_BANKNAME_HU, ""},  /* fuzzy */
	/* it */ {IDGS_BANKNAME_IT, "nome banco"},
	/* ko */ {IDGS_BANKNAME_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_BANKNAME_NL, "banknaam"},
	/* pl */ {IDGS_BANKNAME_PL, "bankname"},
	/* ru */ {IDGS_BANKNAME_RU, "bankname"},
	/* sv */ {IDGS_BANKNAME_SV, "banknamn"},
	/* tr */ {IDGS_BANKNAME_TR, "bankismi"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_BANK_DESCRIPTION,    N_("If bankname is not given, print the possible banks for the memspace.\nIf bankname is given set the current bank in the memspace to the given\nbank.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_BANK_DESCRIPTION_DA, "Hvis banknavn ikke angives vises mulige banker for hukommelsesomrÂdet.\nHvis banknavn angives sÊttes aktuel bank i hukommelsesomrÂdet til den\nangivne bank."},
	/* de */ {IDGS_MON_BANK_DESCRIPTION_DE, "Falls `bankname' nicht angegeben wird, Ausgabe der mˆglichen B‰nke in `memspace'.\nFalls `bankname' angegeben wird, setze aktuelle Bank in `memspace' mit angegebener Bank."},
	/* es */ {IDGS_MON_BANK_DESCRIPTION_ES, "Si no se pone el nombre del banco, imprimir los bancos posibles para el espacio de memoria.\nSi se da el nombre de banco se usa como espacio de memoria actual \nbanco."},
	/* fr */ {IDGS_MON_BANK_DESCRIPTION_FR, "Si le nom de la banque n'est pas donnÈ, afficher les banques possible pour\ncet espace mÈmoire. Si le nom de la banque est donnÈe, rÈgler la banques\ncourante de l'espace mÈmoire ‡ cette banque sÈlectionnÈe."},
	/* hu */ {IDGS_MON_BANK_DESCRIPTION_HU, "Ha a lap sz·ma nincs megadva, kiÌrja a lehetsÈges lapokat a memÛria tÈrbıl.\nHa meg van adva, az aktu·lis lap sz·mot a memÛria tÈrben a megadott lapra ·llÌtja."},
	/* it */ {IDGS_MON_BANK_DESCRIPTION_IT, "Se non viene specificato il nome del banco, stampa tutti i banchi\npossibili per lo spazio di memoria.\nSe viene specificato il nome del banco, imposta il banco attuale nello\nspazio di memoria specificato."},
	/* ko */ {IDGS_MON_BANK_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_BANK_DESCRIPTION_NL, "Als de banknaam niet is opgegeven, worden de mogelijke banken voor de\ngeheugenruimte weergegeven. Als de banknaam is opgegeven, wordt de\nhuidige bank in de geheugenruimte gezet volgens de opgegeven banknaam."},
	/* pl */ {IDGS_MON_BANK_DESCRIPTION_PL, "Je∂li nie podano nazwy banku, pokazuje moøliwe banki pamiÍci.\nJe∂li podano nazwÍ banku, ustawia obecny bank pamiÍci na\ndany bank."},
	/* ru */ {IDGS_MON_BANK_DESCRIPTION_RU, "If bankname is not given, print the possible banks for the memspace.\nIf bankname is given set the current bank in the memspace to the given\nbank."},
	/* sv */ {IDGS_MON_BANK_DESCRIPTION_SV, "Om banknamn inte anges, skriv ut mˆjliga bankar fˆr minnesomrÂdet.\nOm banknamn anges, s‰tt aktuell bank i minnesomrÂdet till den givna\nbanken."},
	/* tr */ {IDGS_MON_BANK_DESCRIPTION_TR, "Eer bank ismi verilmemi˛se, bellek alan˝ iÁin uygun banklar yazd˝r˝l˝r.\nEer bank ismi verilmi˛se, geÁerli bank bellek alan˝ iÁinde verilen\nbanka e˛itlenir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_BACKTRACE_DESCRIPTION,    N_("Print JSR call chain (most recent call first). Stack offset\nrelative to SP+1 is printed in parentheses. This is a best guess\nonly.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_BACKTRACE_DESCRIPTION_DA, "Vis JSR-kaldekÊde (seneste kald ¯verst). Stakafstand\nrelativ til SP+1 vises i parantes. Dette er kun et\ngÊt."},
	/* de */ {IDGS_MON_BACKTRACE_DESCRIPTION_DE, "Zeige JSR Aufrufkette (letzter Aufruf zuerst). Stack offset\nwird relative zu SP+1 in Klammern gezeigt. Dies ist jedoch nur\neine Sch‰tzung."},
	/* es */ {IDGS_MON_BACKTRACE_DESCRIPTION_ES, "Imprimir cadena de llamada JSR (la llamada m·s reciente primero). Stack offset\nrelativo a  SP+1 se imprime entre parÈntesis. Esta es una buena suposiciÛn\nsÛlo."},
	/* fr */ {IDGS_MON_BACKTRACE_DESCRIPTION_FR, "Afficher la chaine d'appels JSR (plus rÈcent appel en premier)\nL'emplacement de la pile relatif ‡ SP+1 est affichÈ entre\nparanthËses. Il ne s'agit que de la meilleure estimation possible."},
	/* hu */ {IDGS_MON_BACKTRACE_DESCRIPTION_HU, "JSR hÌv·s lista kiÌr·sa (legutÛbbi elıszˆr). A verem SP+1\nhelyen lÈvı tartalma z·rÛjelben. Ez csak\negy legjobb becslÈs."},
	/* it */ {IDGS_MON_BACKTRACE_DESCRIPTION_IT, "Stampa la catena di chiamate JSR (la chiamata pi˘ recente per prima).\nL'offset dello stack relativo a SP+1 Ë stampato tra parentesi. Questo Ë\nsolo l'ipotesi migliore."},
	/* ko */ {IDGS_MON_BACKTRACE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_BACKTRACE_DESCRIPTION_NL, "Toon JSR gebruik (meest recent gebruikt eerst). Het relatieve begin van de stack\nten opzichte van SP+1 zal worden getoond\ntussen de aanhalingstekens. Dit is op basis van de beste gok."},
	/* pl */ {IDGS_MON_BACKTRACE_DESCRIPTION_PL, "Pokazuje ≥aÒcuch wezwaÒ JSR. WyrÛwnanie stosu relatywne\ndo SP+1 jest pokazane w nawiasach. Warto∂Ê jest szacowana."},
	/* ru */ {IDGS_MON_BACKTRACE_DESCRIPTION_RU, "Print JSR call chain (most recent call first). Stack offset\nrelative to SP+1 is printed in parentheses. This is a best guess\nonly."},
	/* sv */ {IDGS_MON_BACKTRACE_DESCRIPTION_SV, "Skriv JSR-anropskedja (senaste anrop fˆrst). StackavstÂnd\nrelativt till SP+1 visas inom parantes. Detta ‰r bara en\ngissning."},
	/* tr */ {IDGS_MON_BACKTRACE_DESCRIPTION_TR, "JSR Áar˝ zincirini yazd˝r (en son Áar˝lar ˆncelikli). Y˝˝t ofseti\nSP+1'e bal˝ olarak parantez iÁersinde yazd˝r˝l˝r. Bu yaln˝zca en iyi\ntahmindir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_CPU_DESCRIPTION,    N_("Specify the type of CPU currently used (6502/z80).")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_CPU_DESCRIPTION_DA, "Angiv typen af CPU der anvendes for ¯jeblikket (6502/z80)."},
	/* de */ {IDGS_MON_CPU_DESCRIPTION_DE, "CPU Typ setzen (6502/z80)"},
	/* es */ {IDGS_MON_CPU_DESCRIPTION_ES, "Especificar el tipo de CPU usada (6502/z80)."},
	/* fr */ {IDGS_MON_CPU_DESCRIPTION_FR, "SpÈcifiez le type de processeur utilisÈ (6502/z80)."},
	/* hu */ {IDGS_MON_CPU_DESCRIPTION_HU, "A processzor tÌpus·nak megad·sa (6502/z80)."},
	/* it */ {IDGS_MON_CPU_DESCRIPTION_IT, "Specifica il tipo di CPU utilizzata al momento (6502/z80)."},
	/* ko */ {IDGS_MON_CPU_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_CPU_DESCRIPTION_NL, "Geef de soort CPU op die op dit moment in gebruik is (6502/z80)."},
	/* pl */ {IDGS_MON_CPU_DESCRIPTION_PL, "Okre∂l typ obecnie uøywanego CPU (6502/z80)."},
	/* ru */ {IDGS_MON_CPU_DESCRIPTION_RU, "Specify the type of CPU currently used (6502/z80)."},
	/* sv */ {IDGS_MON_CPU_DESCRIPTION_SV, "Ange typ av CPU som anv‰nds fˆr n‰rvarande (6502/z80)."},
	/* tr */ {IDGS_MON_CPU_DESCRIPTION_TR, "ﬁu anda kullan˝lmakta olan CPU tipini belirleyin (6502/z80)."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_COUNT,    N_("count")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_COUNT_DA, "antal"},
	/* de */ {IDGS_COUNT_DE, "Z‰hler"},
	/* es */ {IDGS_COUNT_ES, "contador"},
	/* fr */ {IDGS_COUNT_FR, "compte"},
	/* hu */ {IDGS_COUNT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_COUNT_IT, "contatore"},
	/* ko */ {IDGS_COUNT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_COUNT_NL, "hoeveelheid"},
	/* pl */ {IDGS_COUNT_PL, "count"},
	/* ru */ {IDGS_COUNT_RU, "count"},
	/* sv */ {IDGS_COUNT_SV, "antal"},
	/* tr */ {IDGS_COUNT_TR, "adet"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_CPUHISTORY_DESCRIPTION,    N_("Show <count> last executed commands.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_CPUHISTORY_DESCRIPTION_DA, "Vis <antal> senest eksekverede instruktioner."},
	/* de */ {IDGS_MON_CPUHISTORY_DESCRIPTION_DE, "Zeige [<Z‰hler>] zuletzt ausgef¸hrte Kommandos"},
	/* es */ {IDGS_MON_CPUHISTORY_DESCRIPTION_ES, "Show <contador> ˙ltimo comando ejecutado."},
	/* fr */ {IDGS_MON_CPUHISTORY_DESCRIPTION_FR, "Afficher les <count> derniËres commandes exÈcutÈes."},
	/* hu */ {IDGS_MON_CPUHISTORY_DESCRIPTION_HU, "<darab> utolj·ra vÈgrehajtott utasÌt·s mutat·sa."},
	/* it */ {IDGS_MON_CPUHISTORY_DESCRIPTION_IT, "Mostra gli ultimi <contatore> comandi eseguiti."},
	/* ko */ {IDGS_MON_CPUHISTORY_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_CPUHISTORY_DESCRIPTION_NL, "Toon <hoeveelheid> laatst uitgevoerde commando's."},
	/* pl */ {IDGS_MON_CPUHISTORY_DESCRIPTION_PL, "Pokazuje <ilo∂Ê> ostatnio wykonanych poleceÒ."},
	/* ru */ {IDGS_MON_CPUHISTORY_DESCRIPTION_RU, "Show <count> last executed commands."},
	/* sv */ {IDGS_MON_CPUHISTORY_DESCRIPTION_SV, "Visa <antal> senast exekverade kommandon."},
	/* tr */ {IDGS_MON_CPUHISTORY_DESCRIPTION_TR, "<adet> son Áal˝˛t˝r˝lan komutlar˝ gˆster."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_FILENAME,    N_("filename")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_FILENAME_DA, "filnavn"},
	/* de */ {IDGS_FILENAME_DE, "Dateiname"},
	/* es */ {IDGS_FILENAME_ES, "Nombre fichero"},
	/* fr */ {IDGS_FILENAME_FR, "nom de fichier"},
	/* hu */ {IDGS_FILENAME_HU, ""},  /* fuzzy */
	/* it */ {IDGS_FILENAME_IT, "nome file"},
	/* ko */ {IDGS_FILENAME_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_FILENAME_NL, "bestandsnaam"},
	/* pl */ {IDGS_FILENAME_PL, "filename"},
	/* ru */ {IDGS_FILENAME_RU, "ÿ‹Ô ‰–Ÿ€–"},
	/* sv */ {IDGS_FILENAME_SV, "filnamn"},
	/* tr */ {IDGS_FILENAME_TR, "dosyaismi"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DUMP_DESCRIPTION,    N_("Write a snapshot of the machine into the file specified.\nThis snapshot is compatible with a snapshot written out by the UI.\nNote: No ROM images are included into the dump.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DUMP_DESCRIPTION_DA, "Skriv et snapshot til den angivne fil.\nDette snapshot er kompatibelt med dem, der oprettes fra menuen.\nOBS: ROM-images inkluderes ikke i snapshot."},
	/* de */ {IDGS_MON_DUMP_DESCRIPTION_DE, "Schreibe eine Snapsho Datei der Maschine auf die angebene Datei.\nDieser Snapshot ist kompatible zu einem Snapshot initiiert aus dem Men¸.\nHinweis: Die ROM Images werden im Snapshot nicht inkludiert."},
	/* es */ {IDGS_MON_DUMP_DESCRIPTION_ES, "Grabar Instant·nea de la m·quina en el fichero especificado.\nEsta Instant·nea es compatible con la grabada por el UI.\nNota: Ninguna imagen ROM se incluye en la copia."},
	/* fr */ {IDGS_MON_DUMP_DESCRIPTION_FR, "…crire un instantanÈ de la machine dans le fichier spÈcifiÈ.\nCet instantanÈ est compatible avec les instantanÈs de l'IU.\nNote: Aucune image ROM n'est incluse dans l'instantanÈ."},
	/* hu */ {IDGS_MON_DUMP_DESCRIPTION_HU, "PillanatkÈp Ìr·sa a gÈprıl a megadott f·jlba. A pillanatkÈp\nkompatibilis azzal, amelyet a grafikus fel¸let hoz lÈtre.\nMegjegyzÈs: a ROM kÈpm·sok nem ker¸lnek bele."},
	/* it */ {IDGS_MON_DUMP_DESCRIPTION_IT, "Scrive uno snapshot del computer sul file specificato.\nQuesto snapshot Ë compatibile con uno snapshot scritto dall'IU.\nNota: Le immagini ROM non sono incluse nel dump."},
	/* ko */ {IDGS_MON_DUMP_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DUMP_DESCRIPTION_NL, "Schrijf een momentopname van de machine naar het opgegeven bestand.\nDeze momentopname is compatibel met een momentopname zoals geschreven \nvanuit het menu.\nNB: ROM bestanden zijn niet opgenomen in de momentopname."},
	/* pl */ {IDGS_MON_DUMP_DESCRIPTION_PL, "Zapis zrzutu maszyny do danego pliku.\nZrzut ten jest kompatybilny ze zrzutami zapisywanymi przez UI.\nUwaga: W zrzucie nie s± zawarte øadne obrazy ROM."},
	/* ru */ {IDGS_MON_DUMP_DESCRIPTION_RU, "Write a snapshot of the machine into the file specified.\nThis snapshot is compatible with a snapshot written out by the UI.\nNote: No ROM images are included into the dump."},
	/* sv */ {IDGS_MON_DUMP_DESCRIPTION_SV, "Skriv en ˆgonblicksbild av maskinen till den angivna filen.\n÷gonblicksbilden ‰r kompatibel med de som skapas frÂn menyn.\nObservera: ROM-avbildningar inkluderas inte i bilden."},
	/* tr */ {IDGS_MON_DUMP_DESCRIPTION_TR, "Belirtilen dosya iÁersine makinenin anl˝k gˆr¸nt¸s¸n¸ yaz.\nBu anl˝k gˆr¸nt¸ UI d˝˛˝ndan yazd˝r˝lan anl˝k gˆr¸nt¸ ile uyumludur.\nNot: HiÁbir ROM imaj˝ dˆk¸m iÁersinde yer almaz."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_ADDRESS,    N_("address")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ADDRESS_DA, "adresse"},
	/* de */ {IDGS_ADDRESS_DE, "Adresse"},
	/* es */ {IDGS_ADDRESS_ES, "direcciÛn"},
	/* fr */ {IDGS_ADDRESS_FR, "adresse"},
	/* hu */ {IDGS_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ADDRESS_IT, "indirizzo"},
	/* ko */ {IDGS_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ADDRESS_NL, "adres"},
	/* pl */ {IDGS_ADDRESS_PL, "address"},
	/* ru */ {IDGS_ADDRESS_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ADDRESS_SV, "adress"},
	/* tr */ {IDGS_ADDRESS_TR, "adres"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_GOTO_DESCRIPTION,    N_("Change the PC to ADDRESS and continue execution")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_GOTO_DESCRIPTION_DA, "∆ndr programtÊlleren (PC) til ADRESSE og fortsÊt eksekvering"},
	/* de */ {IDGS_MON_GOTO_DESCRIPTION_DE, "Setze PC auf ADRESSE und setze Ausf¸hrung fort"},
	/* es */ {IDGS_MON_GOTO_DESCRIPTION_ES, "Cambiar el  PC con direcciÛn y continuar la ejecuciÛn"},
	/* fr */ {IDGS_MON_GOTO_DESCRIPTION_FR, "Modifier le PC ‡ l'ADRESSE et continuer l'exÈcution"},
	/* hu */ {IDGS_MON_GOTO_DESCRIPTION_HU, "A PC ·llÌt·sa ADDRESS cÌmre Ès folytat·s"},
	/* it */ {IDGS_MON_GOTO_DESCRIPTION_IT, "Cambia il PC in INDIRIZZO e continua l'esecuzione"},
	/* ko */ {IDGS_MON_GOTO_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_GOTO_DESCRIPTION_NL, "Zet de PC naar ADRES en ga door met de uitvoering van instructies."},
	/* pl */ {IDGS_MON_GOTO_DESCRIPTION_PL, "ZmieÒ PC na ADRES i wykonuj dalej"},
	/* ru */ {IDGS_MON_GOTO_DESCRIPTION_RU, "∑–‹’›ÿ‚Ï ø∫ ›– ∞¥¿µ¡ ÿ ﬂ‡ﬁ‘ﬁ€÷ÿ‚Ï “Îﬂﬁ€›’›ÿ’"},
	/* sv */ {IDGS_MON_GOTO_DESCRIPTION_SV, "ƒndra PC till ADRESS och forts‰tt exekveringen"},
	/* tr */ {IDGS_MON_GOTO_DESCRIPTION_TR, "PC'yi ADDRESS'e Áevirip Áal˝˛t˝rmaya devam edin"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_IO_DESCRIPTION,    N_("Print out the I/O area of the emulated machine.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_IO_DESCRIPTION_DA, "Udskriv I/O-omrÂdet for den emulerede maskine."},
	/* de */ {IDGS_MON_IO_DESCRIPTION_DE, "Zeige den I/O Bereich der emulierten Maschine."},
	/* es */ {IDGS_MON_IO_DESCRIPTION_ES, "Imprimir el ·rea de entrada/salida de la m·quina emulada."},
	/* fr */ {IDGS_MON_IO_DESCRIPTION_FR, "Afficher la zone d'E/S de la machine ÈmulÈe."},
	/* hu */ {IDGS_MON_IO_DESCRIPTION_HU, "Az emul·lt gÈp I/O tartom·ny·nak mutat·sa."},
	/* it */ {IDGS_MON_IO_DESCRIPTION_IT, "Stampa l'area di I/O del computer emulato."},
	/* ko */ {IDGS_MON_IO_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_IO_DESCRIPTION_NL, "Toon het I/O gebied van de geÎmuleerde machine."},
	/* pl */ {IDGS_MON_IO_DESCRIPTION_PL, "Pokaø obszar we/wy emulowanej maszyny."},
	/* ru */ {IDGS_MON_IO_DESCRIPTION_RU, "Print out the I/O area of the emulated machine."},
	/* sv */ {IDGS_MON_IO_DESCRIPTION_SV, "Skriv I/O-omrÂdet fˆr den emulerade maskinen."},
	/* tr */ {IDGS_MON_IO_DESCRIPTION_TR, "Em¸le edilen makinenin G/« alan˝n˝ yazd˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_EXPORT_DESCRIPTION,    N_("Print out list of attached expansion port devices.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_EXPORT_DESCRIPTION_DA, "Udskriv liste over tilsluttede udviddelsesportenheder"},
	/* de */ {IDGS_MON_EXPORT_DESCRIPTION_DE, "Zeige eine Liste der Ger‰te am Expansion Port."},
	/* es */ {IDGS_MON_EXPORT_DESCRIPTION_ES, "Imprimir lista perifÈricos insertados en puertos de expansiÛn."},
	/* fr */ {IDGS_MON_EXPORT_DESCRIPTION_FR, "Afficher la liste des pÈriphÈriques attachÈs aux ports d'expansion."},
	/* hu */ {IDGS_MON_EXPORT_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_EXPORT_DESCRIPTION_IT, "Stampa lista dei dispositivi collegati sulla porta di espansione."},
	/* ko */ {IDGS_MON_EXPORT_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_EXPORT_DESCRIPTION_NL, "Druk een lijst van gekoppelde uitbreidingspoort apparaten af."},
	/* pl */ {IDGS_MON_EXPORT_DESCRIPTION_PL, "Wydrukuj listÍ pod≥±czonych urz±dzeÒ portÛw rozszerzeÒ."},
	/* ru */ {IDGS_MON_EXPORT_DESCRIPTION_RU, "Print out list of attached expansion port devices."},
	/* sv */ {IDGS_MON_EXPORT_DESCRIPTION_SV, "Skriv ut lista ˆver anslutna expansionsportsenheter."},
	/* tr */ {IDGS_MON_EXPORT_DESCRIPTION_TR, "Geni˛letme yuvas˝na tak˝l˝ ayg˝tlar˝n listesini yazd˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_NEXT_DESCRIPTION,    N_("Advance to the next instruction.  Subroutines are treated as\na single instruction.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_NEXT_DESCRIPTION_DA, "GÂ frem til nÊste instruktion. Subrutiner bliver behandlet som\nen enkelt instruktion."},
	/* de */ {IDGS_MON_NEXT_DESCRIPTION_DE, "Fahre bis zur n‰chsten Instruktion fort.\nUnterroutinen werden wie eine einzelne Instruktion behandelt."},
	/* es */ {IDGS_MON_NEXT_DESCRIPTION_ES, "Avanzar a la siguiente instrucciÛn.  Subrutinas ser·n tratadas como\ninstrucciones simples."},
	/* fr */ {IDGS_MON_NEXT_DESCRIPTION_FR, "Avancer ‡ la prochaine instruction. Les sous-routines sont\ntraitÈes comme Ètant une seule instruction."},
	/* hu */ {IDGS_MON_NEXT_DESCRIPTION_HU, "A kˆvetkezı utasÌt·sra lÈpÈs.  A szubrutinokat egy\nutasÌt·snak tekinti."},
	/* it */ {IDGS_MON_NEXT_DESCRIPTION_IT, "Passa alla prossima istruzione. Le subrutine sono trattate come\nuna singola istruzione."},
	/* ko */ {IDGS_MON_NEXT_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_NEXT_DESCRIPTION_NL, "Ga naar de volgende instructie. Subroutines worden behandeld als een\nenkele instructie."},
	/* pl */ {IDGS_MON_NEXT_DESCRIPTION_PL, "Przechodzi do nastÍpnej instrukcji. Procedury s± traktowane\njak pojedyncze instrukcje."},
	/* ru */ {IDGS_MON_NEXT_DESCRIPTION_RU, "Advance to the next instruction.  Subroutines are treated as\na single instruction."},
	/* sv */ {IDGS_MON_NEXT_DESCRIPTION_SV, "GÂ till n‰sta instruktion. Underrutiner hanteras som en\nenda instruktion."},
	/* tr */ {IDGS_MON_NEXT_DESCRIPTION_TR, "Sonraki komuta ilerle.  Altrutinler tek bir komutmu˛\ngibi i˛lendi."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_REG_NAME,    N_("reg_name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_REG_NAME_DA, "regnavn"},
	/* de */ {IDGS_REG_NAME_DE, "Register Name"},
	/* es */ {IDGS_REG_NAME_ES, "Nombre registro"},
	/* fr */ {IDGS_REG_NAME_FR, "reg_name"},
	/* hu */ {IDGS_REG_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDGS_REG_NAME_IT, "nome registro"},
	/* ko */ {IDGS_REG_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_REG_NAME_NL, "register naam"},
	/* pl */ {IDGS_REG_NAME_PL, "reg_name"},
	/* ru */ {IDGS_REG_NAME_RU, "reg_name"},
	/* sv */ {IDGS_REG_NAME_SV, "regnamn"},
	/* tr */ {IDGS_REG_NAME_TR, "reg_ismi"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_NUMBER,    N_("number")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_NUMBER_DA, "nummer"},
	/* de */ {IDGS_NUMBER_DE, "Nummer"},
	/* es */ {IDGS_NUMBER_ES, "n˙mero"},
	/* fr */ {IDGS_NUMBER_FR, "numÈro"},
	/* hu */ {IDGS_NUMBER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_NUMBER_IT, "numero"},
	/* ko */ {IDGS_NUMBER_KO, "º˝¿⁄"},
	/* nl */ {IDGS_NUMBER_NL, "nummer"},
	/* pl */ {IDGS_NUMBER_PL, "numer"},
	/* ru */ {IDGS_NUMBER_RU, "number"},
	/* sv */ {IDGS_NUMBER_SV, "nummer"},
	/* tr */ {IDGS_NUMBER_TR, "say˝"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_REGISTERS_DESCRIPTION,    N_("Assign respective registers.  With no parameters, display register\nvalues.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_REGISTERS_DESCRIPTION_DA, "Tildel vÊrdi til registre. Uden parametre vises nuvÊrende vÊrdier\ni registrene."},
	/* de */ {IDGS_MON_REGISTERS_DESCRIPTION_DE, "Zuweisung auf entsprechende Register. Ohne Parameter, zeige Register Werte."},
	/* es */ {IDGS_MON_REGISTERS_DESCRIPTION_ES, "Asignar registros respectivos. Sin par·metros, mostrar valores\nregistros."},
	/* fr */ {IDGS_MON_REGISTERS_DESCRIPTION_FR, "Assigner des registres respectifs. Sans paramËtres, affichera\nles valeurs des registres."},
	/* hu */ {IDGS_MON_REGISTERS_DESCRIPTION_HU, "Megadott regiszterek be·llÌt·sa. ParamÈterek nÈlk¸l, a regiszterek\ntartalm·nak kiÌr·sa."},
	/* it */ {IDGS_MON_REGISTERS_DESCRIPTION_IT, "Assegna i rispettivi registri. Senza parametri, mostra i valori dei\nregistri."},
	/* ko */ {IDGS_MON_REGISTERS_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_REGISTERS_DESCRIPTION_NL, "Vul opgegeven registers met de toebehorende waarde. Zonder parameter, toon\nde huidige waarden in de registers."},
	/* pl */ {IDGS_MON_REGISTERS_DESCRIPTION_PL, "Przyporz±dkowuje odpowiednie rejestry. Bez podanych parametrÛw\npokazuje warto∂ci rejestrÛw."},
	/* ru */ {IDGS_MON_REGISTERS_DESCRIPTION_RU, "Assign respective registers.  With no parameters, display register\nvalues."},
	/* sv */ {IDGS_MON_REGISTERS_DESCRIPTION_SV, "Tilldela respektive register. Utan parametrar visas innehÂllet\ni registren."},
	/* tr */ {IDGS_MON_REGISTERS_DESCRIPTION_TR, "S˝rayla registerlara ata.  parametreler olmadan, register deerlerini\ngˆr¸nt¸le."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_RESET_DESCRIPTION,    N_("Reset the machine or drive. Type: 0 = soft, 1 = hard, 8-11 = drive.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_RESET_DESCRIPTION_DA, "Reset maskine eller diskettedrev. Type: 0 = bl¯d, 1 = hÂrd,\n8-11 = diskettedrev."},
	/* de */ {IDGS_MON_RESET_DESCRIPTION_DE, "Reset der Maschine oder des Laufwerks. Typ: 0 = soft, 1 = hart, 8-11 = Laufwerk."},
	/* es */ {IDGS_MON_RESET_DESCRIPTION_ES, "Reiniciar m·quina o unidad disco. Tipo: 0 = soft, 1 = hard, 8-11 = unidad disco."},
	/* fr */ {IDGS_MON_RESET_DESCRIPTION_FR, "RÈinitialiser la machine ou le lecteur. (0=Soft, 1=Hard, 8-11=Lecteur)"},
	/* hu */ {IDGS_MON_RESET_DESCRIPTION_HU, "A sz·mÌtÛgÈp vagy meghajtÛ ˙jraindÌt·sa. TÌpus: 0 = meleg, 1 = hideg, 8-11 = meghajtÛ."},
	/* it */ {IDGS_MON_RESET_DESCRIPTION_IT, "Resetta il computer o il drive. Tipo: 0 = soft, 1 = hard, 8-11 = drive."},
	/* ko */ {IDGS_MON_RESET_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_RESET_DESCRIPTION_NL, "Reset de machine of drive. Soort: 0 = zacht, 1 = hard, 8-11 = drive."},
	/* pl */ {IDGS_MON_RESET_DESCRIPTION_PL, "Reset maszyny lub napÍdu. Typ: 0 = miÍkki, 1 = twardy, 8-11 = napÍdu."},
	/* ru */ {IDGS_MON_RESET_DESCRIPTION_RU, "Reset the machine or drive. Type: 0 = soft, 1 = hard, 8-11 = drive."},
	/* sv */ {IDGS_MON_RESET_DESCRIPTION_SV, "≈terst‰ll maskinen eller diskettstationen. Typ: 0 = mjuk, 1 = hÂrd,\n8-11 = diskettstation."},
	/* tr */ {IDGS_MON_RESET_DESCRIPTION_TR, "Makine ya da s¸r¸c¸y¸ resetle. Tip: 0 = soft, 1 = hard, 8-11 = drive."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_RETURN_DESCRIPTION,    N_("Continues execution and returns to the monitor just before the next\nRTS or RTI is executed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_RETURN_DESCRIPTION_DA, "FortsÊt eksekveringen og vend tilbage til monitoren lige inden nÊste\nRTS eller RTI udf¯res."},
	/* de */ {IDGS_MON_RETURN_DESCRIPTION_DE, "Setzt Ausf¸hrung fort und retourniert bevor der n‰chsten RTS oder RTI instruction."},
	/* es */ {IDGS_MON_RETURN_DESCRIPTION_ES, "Continuar ejecuciÛn y volver al monitor justo antes que la siguiente\nRTS o RTI sea ejecutada."},
	/* fr */ {IDGS_MON_RETURN_DESCRIPTION_FR, "Continuer l'exÈcution et retourner au moniteur juste avant la prochaine\nexÈcution de RTS ou RTI."},
	/* hu */ {IDGS_MON_RETURN_DESCRIPTION_HU, "Folytatja a vÈgrehajt·st Ès visszatÈr a monitorba a kˆvetkezı\nRTS vagy RTI utasÌt·s elıtt."},
	/* it */ {IDGS_MON_RETURN_DESCRIPTION_IT, "Continua l'esecuzione e ritorna al monitor subito prima della prossima\nesecuzione di RTS o RTI."},
	/* ko */ {IDGS_MON_RETURN_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_RETURN_DESCRIPTION_NL, "Ga door met uitvoeren van instructies en keer terug naar de monitor net voordat de\nvolgende RTS of RTI wordt uitgevoerd."},
	/* pl */ {IDGS_MON_RETURN_DESCRIPTION_PL, "Wykonuje dalej i wraca do monitora tuø przed wykonaniem\nnastÍpnego RTS lub RTI."},
	/* ru */ {IDGS_MON_RETURN_DESCRIPTION_RU, "Continues execution and returns to the monitor just before the next\nRTS or RTI is executed."},
	/* sv */ {IDGS_MON_RETURN_DESCRIPTION_SV, "Forts‰tt exekveringen och ÂtergÂ till monitorn precis innan n‰sta\nRTS eller RTI exekveras."},
	/* tr */ {IDGS_MON_RETURN_DESCRIPTION_TR, "«al˝˛maya devam eder ve sonraki RTS ya da RTI Áal˝˛t˝r˝lmadan hemen ˆnce\nmonitˆre geri dˆner."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_SCREEN_DESCRIPTION,    N_("Displays the contents of the screen.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_SCREEN_DESCRIPTION_DA, "Viser skÊrmens indhold."},
	/* de */ {IDGS_MON_SCREEN_DESCRIPTION_DE, "Zeigt Bildschirminhalt."},
	/* es */ {IDGS_MON_SCREEN_DESCRIPTION_ES, "Mostrar los contenidos de la pantalla."},
	/* fr */ {IDGS_MON_SCREEN_DESCRIPTION_FR, "Affiche le contenu de l'Ècran."},
	/* hu */ {IDGS_MON_SCREEN_DESCRIPTION_HU, "A kÈpernyı tartalm·nak mutat·sa."},
	/* it */ {IDGS_MON_SCREEN_DESCRIPTION_IT, "Mostra il contenuto dello schermo."},
	/* ko */ {IDGS_MON_SCREEN_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_SCREEN_DESCRIPTION_NL, "Toont de inhoud van het scherm."},
	/* pl */ {IDGS_MON_SCREEN_DESCRIPTION_PL, "Pokazuje zawarto∂Ê ekranu."},
	/* ru */ {IDGS_MON_SCREEN_DESCRIPTION_RU, "æ‚ﬁ—‡–÷–’‚ ·ﬁ‘’‡÷–›ÿ’ Ì⁄‡–›–."},
	/* sv */ {IDGS_MON_SCREEN_DESCRIPTION_SV, "Visar sk‰rmens innehÂll."},
	/* tr */ {IDGS_MON_SCREEN_DESCRIPTION_TR, "Ekran iÁeriini gˆr¸nt¸ler."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_STEP_DESCRIPTION,    N_("Single-step through instructions.  COUNT allows stepping\nmore than a single instruction at a time.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_STEP_DESCRIPTION_DA, "GÂ skridtvis gennem instruktioner. Hvis ANTAL angives kan\nflere instruktioner udf¯res i hvert skridt."},
	/* de */ {IDGS_MON_STEP_DESCRIPTION_DE, "Einzelschritt Ausf¸hrung durch Instruktionen. COUNT erlaubt die Ausf¸hrung\nmehrerer Instruktionen auf einmal."},
	/* es */ {IDGS_MON_STEP_DESCRIPTION_ES, "Salto simple a travÈs de la instrucciÛn.  el CONTADOR permite el salto de\nm·s de una instrucciÛn a la vez."},
	/* fr */ {IDGS_MON_STEP_DESCRIPTION_FR, "Instructions ‡ Ètape-unique. COUNT permet de faire pas-‡-pas plus qu'une seule instruction ‡ la fois."},
	/* hu */ {IDGS_MON_STEP_DESCRIPTION_HU, "LÈpkedÈs egyesÈvel az utasÌt·sokon.  A DARAB-bal be·llÌthatÛ,\nha tˆbb utasÌt·st kell ugrani egyszerre."},
	/* it */ {IDGS_MON_STEP_DESCRIPTION_IT, "Modalit‡ single step attraverso le istruzioni. Il CONTATORE permette\ndi attraversare pi˘ di una singola istruzione per volta."},
	/* ko */ {IDGS_MON_STEP_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_STEP_DESCRIPTION_NL, "Stap voor stap door de instructies. AANTAL zorgt ervoor dat er meerdere\ninstructies worden uitgevoerd in een keer."},
	/* pl */ {IDGS_MON_STEP_DESCRIPTION_PL, "Przechodzi przez instrukcje krok po kroku. COUNT pozwala\nprzechodziÊ przez wiÍcej niø jedn± instrukcjÍ na raz."},
	/* ru */ {IDGS_MON_STEP_DESCRIPTION_RU, "Single-step through instructions.  COUNT allows stepping\nmore than a single instruction at a time."},
	/* sv */ {IDGS_MON_STEP_DESCRIPTION_SV, "Stega genom instruktioner. ANTAL gˆr det mˆjligt att stega\ngenom mer ‰n en enkel instruktion i varje steg."},
	/* tr */ {IDGS_MON_STEP_DESCRIPTION_TR, "Komutlara doru tek-ad˝m.  COUNT bir defada birden fazla\nkomutu ad˝mlamaya izin verir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_STOPWATCH_DESCRIPTION,    N_("Print the CPU cycle counter of the current device. 'reset' sets the counter to 0.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_STOPWATCH_DESCRIPTION_DA, "Udskriv CPU-cyklus-tÊller for den nuvÊrende enhed. \"reset\" sÊtter tÊlleren til 0."},
	/* de */ {IDGS_MON_STOPWATCH_DESCRIPTION_DE, "Zeige CPU Zyklusz‰hler des aktuellen Ger‰ts. 'reset' setzt den Z‰hler\nauf 0."},
	/* es */ {IDGS_MON_STOPWATCH_DESCRIPTION_ES, "Imprimir contador de ciclos del perifÈrico actual. 'reset' pone el contador a 0."},
	/* fr */ {IDGS_MON_STOPWATCH_DESCRIPTION_FR, "Afficher le compteur du cycle CPU du pÈpiphÈrique actuel. 'reset' remet ce compteur ‡ 0."},
	/* hu */ {IDGS_MON_STOPWATCH_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_STOPWATCH_DESCRIPTION_IT, "Stampa il numero di cicli di CPU del device attuale. 'reset' imposta il contatore a 0."},
	/* ko */ {IDGS_MON_STOPWATCH_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_STOPWATCH_DESCRIPTION_NL, "Toon de CPU cycli teller voor het huidige apparaat. 'reset' zet de teller naar 0."},
	/* pl */ {IDGS_MON_STOPWATCH_DESCRIPTION_PL, "Pokaø licznik cykli CPU obecnego urz±dzenia. 'reset' zeruje licznik."},
	/* ru */ {IDGS_MON_STOPWATCH_DESCRIPTION_RU, "Print the CPU cycle counter of the current device. 'reset' sets the counter to 0."},
	/* sv */ {IDGS_MON_STOPWATCH_DESCRIPTION_SV, "Visa processorcykelr‰knaren fˆr aktuell enhet. \"reset\" s‰tter r‰knaren pÂ 0."},
	/* tr */ {IDGS_MON_STOPWATCH_DESCRIPTION_TR, "Mevcut cihaz˝n CPU cycle sayac˝n˝ yazd˝r. 'reset' sayac˝ 0'a getirir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_UNDUMP_DESCRIPTION,    N_("Read a snapshot of the machine from the file specified.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_UNDUMP_DESCRIPTION_DA, "IndlÊs snapshot af maskinen fra den angivne fil."},
	/* de */ {IDGS_MON_UNDUMP_DESCRIPTION_DE, "Lese einen Snapshot der Maschine aus der angegebenen Datei."},
	/* es */ {IDGS_MON_UNDUMP_DESCRIPTION_ES, "Leer Instant·nea de una m·quina desde el fichero especificado."},
	/* fr */ {IDGS_MON_UNDUMP_DESCRIPTION_FR, "Lire un instantanÈ de machine depuis le fichier spÈcifiÈ."},
	/* hu */ {IDGS_MON_UNDUMP_DESCRIPTION_HU, "A pillanatkÈp betˆltÈse a megadott f·jlbÛl."},
	/* it */ {IDGS_MON_UNDUMP_DESCRIPTION_IT, "Legge lo snapshot del computer dal file specificato."},
	/* ko */ {IDGS_MON_UNDUMP_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_UNDUMP_DESCRIPTION_NL, "Lees een momentopname van de machine vanuit het opgegeven bestand."},
	/* pl */ {IDGS_MON_UNDUMP_DESCRIPTION_PL, "Odczytaj zrzut maszyny z okre∂lonego pliku."},
	/* ru */ {IDGS_MON_UNDUMP_DESCRIPTION_RU, "Read a snapshot of the machine from the file specified."},
	/* sv */ {IDGS_MON_UNDUMP_DESCRIPTION_SV, "L‰s en ˆgonblicksbild fˆr maskinen frÂn den angivna filen."},
	/* tr */ {IDGS_MON_UNDUMP_DESCRIPTION_TR, "Belirtilen dosyadan makinenin anl˝k gˆr¸nt¸s¸n¸ oku."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_SYMBOL_TABLE_COMMANDS,    N_("Symbol table commands:")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SYMBOL_TABLE_COMMANDS_DA, "Symboltabel-kommandoer:"},
	/* de */ {IDGS_SYMBOL_TABLE_COMMANDS_DE, "Symbol Tabellen Kommandos:"},
	/* es */ {IDGS_SYMBOL_TABLE_COMMANDS_ES, "Tabla sÌmbolo comando:"},
	/* fr */ {IDGS_SYMBOL_TABLE_COMMANDS_FR, "Commandes de la table de symboles:"},
	/* hu */ {IDGS_SYMBOL_TABLE_COMMANDS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SYMBOL_TABLE_COMMANDS_IT, "Comandi tabella dei simboli:"},
	/* ko */ {IDGS_SYMBOL_TABLE_COMMANDS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SYMBOL_TABLE_COMMANDS_NL, "Symbool tabel commando's:"},
	/* pl */ {IDGS_SYMBOL_TABLE_COMMANDS_PL, "Polecenia tablicy symboli:"},
	/* ru */ {IDGS_SYMBOL_TABLE_COMMANDS_RU, "Symbol table commands:"},
	/* sv */ {IDGS_SYMBOL_TABLE_COMMANDS_SV, "Kommandon fˆr symboltabell:"},
	/* tr */ {IDGS_SYMBOL_TABLE_COMMANDS_TR, "Sembol tablosu komutlar˝:"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_LABEL,    N_("label")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_LABEL_DA, "etiket"},
	/* de */ {IDGS_LABEL_DE, "Bezeichner"},
	/* es */ {IDGS_LABEL_ES, "Etiqueta"},
	/* fr */ {IDGS_LABEL_FR, "label"},
	/* hu */ {IDGS_LABEL_HU, ""},  /* fuzzy */
	/* it */ {IDGS_LABEL_IT, "etichetta"},
	/* ko */ {IDGS_LABEL_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_LABEL_NL, "label"},
	/* pl */ {IDGS_LABEL_PL, "oznaczenie"},
	/* ru */ {IDGS_LABEL_RU, "label"},
	/* sv */ {IDGS_LABEL_SV, "etikett"},
	/* tr */ {IDGS_LABEL_TR, "etiket"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_ADD_LABEL_DESCRIPTION,    N_("<memspace> is one of: C: 8: 9: 10: 11:\n<address>  is the address which should get the label.\n<label>    is the name of the label; it must start with a dot (\".\").\n\nMap a given address to a label.  This label can be used when entering\nassembly code and is shown during disassembly.  Additionally, it can\nbe used whenever an address must be specified.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_ADD_LABEL_DESCRIPTION_DA, "<huk.omrÂde>   er en af C: 8: 9: 10: eller 11:\n<adresse>      Er adressen som skal have etiketten.\n<etiket>       er navnet pÂ etiketten; skal starte med punktum (\".\").\n\nSÊt en etiket for en adresse. Etiketten kan angives nÂr der indtastes\nassemblerkode, og vises ved disassemblering. Desuden kan den bruges\nhvor en adresse skal angives."},
	/* de */ {IDGS_MON_ADD_LABEL_DESCRIPTION_DE, "<Speicherbereich> ist aus: C: 8: 9: 10: 11:\n<Adresse>         bezeichnet die Adresse welche mit dem Bezeichner assoziert werden soll.\n<Bezeichner>      Name des Bezeichners; Muss mit einem Punkt (\".\") starten.\n\nAssoziiert einen Bezeichner mit einer Adresse. Dieser Bezeichner kann \nbei der Eingabe von Asssmblercode benutzt werden und wird auch beim\nDisassemblieren gezeigt. Dar¸ber hinaus kann der Bezeichner ¸berall\ndort genutzt werden, wo eine Adresseingabe nˆtig ist."},
	/* es */ {IDGS_MON_ADD_LABEL_DESCRIPTION_ES, "<espacio memoria> es uno de: C: 8: 9: 10: 11:\n<direcciÛn>  es la direcciÛn donde estar· la etiqueta.\n<etiqueta>   es el nombre de la etiqueta; debe empezar con un punto (\".\").\n\nMapea la direcciÛn dada a una etiqueta que se usar· cuando se inserte\ncÛdigo ensamblador y se mostrar· durante el desensamblado.  Adicionalmente\npuede ser usada siempre que se deba especificar una direcciÛn."},
	/* fr */ {IDGS_MON_ADD_LABEL_DESCRIPTION_FR, "<memspace> est un des choix suivants (C: 8: 9: 10: 11:)\n<address>  est l'adresse qui devrait prendre le label.\n<label>    est le nom de l'Ètiquette; il doit commencer par un point (\".\").\n\nAssocie une adresse a une Ètiquette. Cette Ètiquette peut Ítre utilisÈ en entrant\ndu code assembleur et est montrÈ durant le dÈassemblement. Aussi, vous pouvez\nl'utiliser ‡ tout moment ou une adresse doit Ítre entrÈe."},
	/* hu */ {IDGS_MON_ADD_LABEL_DESCRIPTION_HU, "A <memÛria tÈr> ezek kˆz¸l: C: 8: 9: 10: 11:\n<cÌm>  amelyik cÌmhez a cÌmkÈt rendelje.\n<cÌmke>    a cÌmke neve; ponttal kell kezdıdjˆn (\".\").\n\nAz adott cÌm rendelÈse a cÌmkÈhez.  Ez a cÌmke haszn·lhatÛ assembly\nprogram beÌr·sakor, Ès visszafejtÈskor megjelenik.  Mindig haszn·lhatÛ,\namikor valamilyen cÌmet kell megadni."},
	/* it */ {IDGS_MON_ADD_LABEL_DESCRIPTION_IT, "<spazio di memoria> Ë uno tra: C: 8: 9: 10: 11:\n<indirizzo> Ë l'indirizzo a cui deve essere associata l'etichettal.\n<etichetta> Ë il nome dell'etichetta; deve iniziare con un punto (\".\").\n\nAssocia l'indirizzo specificato all'etichetta. Questa etichetta puÚ\nessere usata quando si inserisce codice assembly ed Ë mostrata\nnella disassemblazione. In aggiunta, puÚ essere usata ogni volta che\ndeve essere specificato un indirizzo."},
	/* ko */ {IDGS_MON_ADD_LABEL_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_ADD_LABEL_DESCRIPTION_NL, "<geheugenruimte> is ÈÈn van: C: 8: 9: 10: 11:\n<adres>          is het adres waaraan het label gekoppeld moet worden.\n<label>          is de naam van het label; de naam moet beginnen met een punt\n                 (\".\").\n\nKoppel een opgegeven adres aan een label. Dit label kan worden gebruikt bij\nhet invoeren van assembleer code en wordt getoond bij een disassemblering.\nTevens kan het label gebruikt worden in plaats van het adres."},
	/* pl */ {IDGS_MON_ADD_LABEL_DESCRIPTION_PL, "<memspace> to jeden z: C: 8: 9: 10: 11:\n<address> to adres, pod ktÛry ma i∂Ê oznaczenie.\n<label> to nazwa oznaczenia, musi zaczynaÊ siÍ kropk± (\".\").\n\nMapuje dany adres pod oznaczenie. Moøna go uøyÊ\nprzy wprowadzaniu kodu asemblowania, a pokazywane jest\npodczas deasemblowania. Oznaczenia tego moøna go uøywaÊ\nzawsze gdy trzeba wskazaÊ dany adres."},
	/* ru */ {IDGS_MON_ADD_LABEL_DESCRIPTION_RU, "<memspace> is one of: C: 8: 9: 10: 11:\n<address>  is the address which should get the label.\n<label>    is the name of the label; it must start with a dot (\".\").\n\nMap a given address to a label.  This label can be used when entering\nassembly code and is shown during disassembly.  Additionally, it can\nbe used whenever an address must be specified."},
	/* sv */ {IDGS_MON_ADD_LABEL_DESCRIPTION_SV, "<minnesomrÂde> ‰r en av: C: 8: 9: 10: 11:\n<adress>       ‰r adressen som ska fÂ etiketten.\n<etikett>      ‰r namnet pÂ etiketten; mÂste starta med punkt (\".\").\n\nKoppla en adress till en etikett. Etiketten kan anges n‰r assemblerkod\nmatas in och visas vid disassemblering. I till‰gg kan den anv‰ndas\nn‰rhelst en adress mÂste anges."},
	/* tr */ {IDGS_MON_ADD_LABEL_DESCRIPTION_TR, "<bellekalan˝> bunlardan biridir: C: 8: 9: 10: 11:\n<adres>       etiketin duraca˝ adresi belirler.\n<etiket>      etiketin ismidir; nokta karakteriyle ba˛lamal˝d˝r (\".\").\n\nEtiketi verilen adrese e˛le˛tir.  Bu etiket assembly kod girilirken kullan˝l˝r\nve disassembly s˝ras˝nda gˆsterilir.  Ek olarak, ne zaman bir adres\nbelirtilmesi gerekse kullan˝labilir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DELETE_LABEL_DESCRIPTION,    N_("<memspace> is one of: C: 8: 9: 10: 11:\n<label>    is the name of the label; it must start with a dot (\".\").\n\nDelete a previously defined label.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_DA, "<huk.omrÂde>   er en af C: 8: 9: 10: eller 11:\n<etiket>       er navnet pÂ etiketten; skal starte med punktum (\".\").\n\nFjern en tidligere oprettet etiket."},
	/* de */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_DE, "<Speicherbereich> ist aus: C: 8: 9: 10: 11:\n<Bezeichner>      Name des Bezeichners; Muss mit einem Punkt (\".\") starten.\n\nLˆscht einen zuvor definierten Bezeichner."},
	/* es */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_ES, "<espacio memoria> es uno de: C: 8: 9: 10: 11:\n<etiqueta>    es el nombre de la etiqueta; y debe comenzar por un punto (\".\").\n\nBorrar una etiqueta previamente definida."},
	/* fr */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_FR, "<memspace> est un des choix suivants (C: 8: 9: 10: 11:)\n<label>    est le nom de l'Ètiquette et commence par un point (\".\").\n\nDÈtruit une Ètiquette prÈalablement crÈÈe."},
	/* hu */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_HU, "<memÛria tÈr> egy ezek kˆz¸l: C: 8: 9: 10: 11:\n<cÌmke>    a cÌmke neve; ponttal kell kezdıdjˆn (\".\").\n\nElızıleg megadott cÌmke tˆrlÈse."},
	/* it */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_IT, "<spazio di memoria> Ë uno tra: C: 8: 9: 10: 11:\n<etichetta> Ë il nome dell'etichetta; deve iniziare con un punto (\".\").\n\nCancella una etichetta definita precedentemente."},
	/* ko */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_NL, "<geheugenruimte> is ÈÈn van: C: 8: 9: 10: 11:\n<label>          is de naam van het label; de naam moet beginnen met een\npunt (\".\").\n\nVerwijder een gekoppeld label."},
	/* pl */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_PL, "<memspace> to jeden z: C: 8: 9: 10: 11:\n<label> to nazwa oznaczenia, musi zaczynaÊ siÍ kropk± (\".\").\n\nKasuje poprzednio zdefiniowane oznaczenie."},
	/* ru */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_RU, "<memspace> is one of: C: 8: 9: 10: 11:\n<label>    is the name of the label; it must start with a dot (\".\").\n\nDelete a previously defined label."},
	/* sv */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_SV, "<minnesomrÂde> ‰r en av: C: 8: 9: 10: 11:\n<etikett>      ‰r namnet pÂ etiketten; mÂste starta med punkt (\".\").\n\nTa bort en tidigare angiven etikett."},
	/* tr */ {IDGS_MON_DELETE_LABEL_DESCRIPTION_TR, "<bellekalan˝> bunlardan biridir: C: 8: 9: 10: 11:\n<etiket>      etiketin ismidir; nokta karakteriyle ba˛lamal˝d˝r (\".\").\n\n÷nceden tan˝mlanm˝˛ bir etiketi sil."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_LOAD_LABELS_DESCRIPTION,    N_("Load a file containing a mapping of labels to addresses.  If no memory\nspace is specified, the default readspace is used.\n\nThe format of the file is the one written out by the `save_labels' command;\nit consists of some `add_label' commands, written one after the other.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_DA, "IndlÊs en fil som indeholder mapning fra labels til adresser.\nHvis der ikke angives hukommelsesomrÂde, anvendes det normale lÊseomrÂde.\n\nFormatet pÂ filen er det, som skrives af \"save_labels\"-kommandoen;\ndet bestÂr af \"add_label\"-kommandoer efter hinanden."},
	/* de */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_DE, "L‰dt eine Datei mit Zuordnung von Adressen zu Bezeichnern. Wenn kein \nSpeicherbereich angegeben wurde, wird der Standard Speicherplatz\nverwendet. Das Format der Datei wird durch das Kommande `save_lables'\ndefiniert. Es besteht aus eine Reihe von `add_label' Kommandos,\nhintereinander angeordnet."},
	/* es */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_ES, "Carga fichero conteniendo mapeado de etiquetas de direcciones.  Si no\nhay espacio de memoria especificado, se usa el espacio por defecto.\n\nEl formato de fichero es el usado por el comando grabar etiquetas;\nConsiste en algunos comandos`aÒadir etiqueta' , grabado uno despuÈs del otro."},
	/* fr */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_FR, "Charge un fichier contenant des Ètiquettes d'adresses. Si aucune adresse\nmÈmoire n'est spÈcifiÈe, l'adressage par dÈfaut est utilisÈ.\n\nLe format du fichier est celui utilisÈ par la commande `save_labels';\nIl consiste de quelques commandes `add_label', Ècrites l'une aprËs l'autre."},
	/* hu */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_HU, "CÌmkÈkrıl cÌmekre lekÈpezÈseket tartalmazÛ f·jl betˆltÈse.  Ha nincs memÛria\ntÈr megadva, az alap tÈr lesz haszn·lva.\n\nA f·jl form·tuma megegyezik a `save_labels' parancs ·ltal kiÌrtÈval;\nnÈh·ny egym·s ut·n Ìrt `add_label' parancsbÛl ·ll."},
	/* it */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_IT, "Carica un file che contiente l'associazione tra le etichette e gli indirizzi.\nSe non Ë specificato alcuno spazio di memoria, viene usato lo spazio\ndi lettura predefinito.\n\nIl formato del file Ë quello dato in output dal comando `save_labels';\nË composto da alcuni comandi `add_label', scritti uno di seguito all'altro."},
	/* ko */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_NL, "Laad een bestand met de koppelingen tussen labels en adressen. Als er geen\ngeheugenruimte is opgegeven, zal de standaard geheugenruimte worden gebruikt.\nHet formaat van het bestand is die door het commando 'save_labels' wordt\ngeschreven; het bestaat uit een aantal 'add_label' commando's na elkaar."},
	/* pl */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_PL, "Wczytuje plik zawieraj±cy mapowanie oznaczeÒ pod adresy. Je∂li nie\nokre∂lono zakresu pamiÍci, uøywany jest domy∂lny zakres pamiÍci.\n\nFormat pliku jest zapisywany komend±`save_labels',\nktÛra sk≥ada siÍ z komend `add_label', zapisanych jedna po drugiej."},
	/* ru */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_RU, "Load a file containing a mapping of labels to addresses.  If no memory\nspace is specified, the default readspace is used.\n\nThe format of the file is the one written out by the `save_labels' command;\nit consists of some `add_label' commands, written one after the other."},
	/* sv */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_SV, "L‰s in en fil som innehÂller koppling frÂn etiketter till adresser.\nOm inget minnesomrÂde anges anv‰ndas standard-l‰sadressen.\n\nFormatet pÂ filen ‰r samma som skrivs av \"save_labels\"-kommandot;\ndet bestÂr av nÂgra \"add_label\"-kommandon efter varandra."},
	/* tr */ {IDGS_MON_LOAD_LABELS_DESCRIPTION_TR, "Etiketleri adreslerle e˛le˛tirilmi˛ bir dosya y¸kleyin.  Eer bellek\nalan˝ belirtilmemi˛se, varsay˝lan okuma alan˝ kullan˝l˝r.\n\nDosyan˝n biÁimi `save_labels' komutu ile yaz˝lm˝˛ oland˝r;\nbu pe˛ pe˛e yaz˝lm˝˛ `add_label' komutlar˝ndan ibarettir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_SAVE_LABELS_DESCRIPTION,    N_("Save labels to a file.  If no memory space is specified, all of the\nlabels are saved.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_DA, "Gem etiketter til fil. Hvis der ikke angives et hukommelsesomrÂde\ngemmes alle etiketter."},
	/* de */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_DE, "Speichere Bezeichner in eine Datei. Wenn kein Speicherbereich\nangegeben wurde, werden alle Bezeichner gespeichert."},
	/* es */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_ES, "Grabar etiquetas en fichero. Si no se define espacio de memoria,\nse graban todas las etiquetas."},
	/* fr */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_FR, "Enregistres les Ètiquettes dans un fichier. Si aucune adresse mÈmoire\nn'est spÈcifiÈe, toutes les Ètiquettes mÈmoires sont sauvegardÈes."},
	/* hu */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_HU, "CÌmkÈk mentÈse f·jlba.  Ha nincs memÛria tartom·ny megadva, az\nˆsszes cÌmkÈt menti."},
	/* it */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_IT, "Salva le etichette su file. Se non Ë specificato alcuno spazio di memoria,\nvengono salvate tutte le etichette."},
	/* ko */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_NL, "Sla de labels op in een bestand. Als er geen geheugenruimte is opgegeven,\nworden alle labels opgeslagen."},
	/* pl */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_PL, "Zapisuje oznaczenia do pliku. Je∂li nie okre∂lono obszaru pamiÍci,\nzapisywanie s± wszystkie oznaczenia."},
	/* ru */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_RU, "Save labels to a file.  If no memory space is specified, all of the\nlabels are saved."},
	/* sv */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_SV, "Spara etiketter till en fil. Om inget minnesomrÂde anges sparas alla\netiketterna."},
	/* tr */ {IDGS_MON_SAVE_LABELS_DESCRIPTION_TR, "Etiketleri bir dosyaya kaydet.  Eer bellek alan˝ belirtilmemi˛se, t¸m\netiketler kaydedilir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_SHOW_LABELS_DESCRIPTION,    N_("Display current label mappings.  If no memory space is specified, show\nall labels from default address space.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_DE, "Zeige aktuelle Bezeichner Zuordnungen. Wenn kein Speicherbereich angegeben\nwurde, werden alle Bezeichner im Standard Adressraum gezeigt."},
	/* es */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_NL, "Toon huidige label koppelingen. Als er geen geheugenruimte is opgegeven,\nworden alle labels van de standaard geheugenruimte getoond."},
	/* pl */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_PL, "Pokaø obecne mapowania oznaczeÒ. Je∂li nie okre∂lono obszaru pamiÍci,\nwy∂wietlane s± wszystkie oznaczenia z domy∂lnych obszarÛw pamiÍci. "},
	/* ru */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MON_SHOW_LABELS_DESCRIPTION_TR, ""},  /* fuzzy */
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION,    N_("Clear current label mappings.  If no memory space is specified, clear\nall labels from default address space.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_DE, "Lˆsche aktuelle Bezeichner Zuordnungen. Wenn kein Speicherbereich angegeben\nwurde, werden alle Bezeichner im Standard Adressraum gelˆscht."},
	/* es */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_NL, "Verwijder huidige label koppelingen. Als er geen geheugenruimte is opgegeven,\nworden alle labels van de standaard geheugenruimte verwijderd."},
	/* pl */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_PL, "Wyczy∂Ê obecne oznaczenia mapowania. Je∂li nie okre∂lono obszaru\npamiÍci, kasowane s± wszystkie oznaczenia z domy∂lnego obszaru pamiÍci."},
	/* ru */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MON_CLEAR_LABELS_DESCRIPTION_TR, ""},  /* fuzzy */
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS,    N_("Assembler and memory commands:")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_DA, "Assembler- og hukommelseskommandoer:"},
	/* de */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_DE, "Assembler und Speicher Kommandos:"},
	/* es */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_ES, "Comandos Ensamblador y de memoria:"},
	/* fr */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_FR, "Commandes assembleur et mÈmoire:"},
	/* hu */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_IT, "Comandi assembler e memoria:"},
	/* ko */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_NL, "Assembler en geheugen commando's:"},
	/* pl */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_PL, "Asembler i polecenia pamiÍci:"},
	/* ru */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_RU, "Assembler and memory commands:"},
	/* sv */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_SV, "Assembler- och minneskommandon:"},
	/* tr */ {IDGS_ASSEMBLER_AND_MEMORY_COMMANDS_TR, "Assembler ve bellek komutlar˝:"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_DATA_LIST,    N_("data_list")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DATA_LIST_DA, "dataliste"},
	/* de */ {IDGS_DATA_LIST_DE, "Datenliste"},
	/* es */ {IDGS_DATA_LIST_ES, "lista datos"},
	/* fr */ {IDGS_DATA_LIST_FR, "data_list"},
	/* hu */ {IDGS_DATA_LIST_HU, ""},  /* fuzzy */
	/* it */ {IDGS_DATA_LIST_IT, "lista di dati"},
	/* ko */ {IDGS_DATA_LIST_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DATA_LIST_NL, "data_lijst"},
	/* pl */ {IDGS_DATA_LIST_PL, "data_list"},
	/* ru */ {IDGS_DATA_LIST_RU, "data_list"},
	/* sv */ {IDGS_DATA_LIST_SV, "datalista"},
	/* tr */ {IDGS_DATA_LIST_TR, "veri_listesi"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_WRITE_DESCRIPTION,    N_("Write the specified data at `address'.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_WRITE_DESCRIPTION_DA, "Gem angivne data pÂ \"adresse\"."},
	/* de */ {IDGS_MON_WRITE_DESCRIPTION_DE, "Schreibe das angegebene Datum auf `Adresse'."},
	/* es */ {IDGS_MON_WRITE_DESCRIPTION_ES, "Grabar el dato especificado en `direcciÛn'."},
	/* fr */ {IDGS_MON_WRITE_DESCRIPTION_FR, "…crire toutes les donnÈes spÈcifiÈes ‡ `address'."},
	/* hu */ {IDGS_MON_WRITE_DESCRIPTION_HU, "A megadott adat `cÌm' helyre Ìr·sa."},
	/* it */ {IDGS_MON_WRITE_DESCRIPTION_IT, "Scrive il dato specificato all'indirizzo."},
	/* ko */ {IDGS_MON_WRITE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_WRITE_DESCRIPTION_NL, "Schrijf de opgegeven gegevens op `adres'."},
	/* pl */ {IDGS_MON_WRITE_DESCRIPTION_PL, "Zapisuje okre∂lone dane w 'adresie'."},
	/* ru */ {IDGS_MON_WRITE_DESCRIPTION_RU, "Write the specified data at `address'."},
	/* sv */ {IDGS_MON_WRITE_DESCRIPTION_SV, "Skriv angiven data pÂ \"adress\"."},
	/* tr */ {IDGS_MON_WRITE_DESCRIPTION_TR, "`adres' bˆl¸m¸nde belirlenmi˛ bir veri yaz."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_INSTRUCTION,    N_("instruction")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_INSTRUCTION_DA, "instruktion"},
	/* de */ {IDGS_INSTRUCTION_DE, "Instruktion"},
	/* es */ {IDGS_INSTRUCTION_ES, "InstrucciÛn"},
	/* fr */ {IDGS_INSTRUCTION_FR, "instruction"},
	/* hu */ {IDGS_INSTRUCTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_INSTRUCTION_IT, "istruzione"},
	/* ko */ {IDGS_INSTRUCTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_INSTRUCTION_NL, "instructie"},
	/* pl */ {IDGS_INSTRUCTION_PL, "instruction"},
	/* ru */ {IDGS_INSTRUCTION_RU, "instruction"},
	/* sv */ {IDGS_INSTRUCTION_SV, "instruktion"},
	/* tr */ {IDGS_INSTRUCTION_TR, "komut"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_ASSEMBLE_DESCRIPTION,    N_("Assemble instructions to the specified address.  If only one\ninstruction is specified, enter assembly mode (enter an empty line to\nexit assembly mode).")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_ASSEMBLE_DESCRIPTION_DA, "Skriv assemblerinstruktioner til adresse. Hvis kun en instruktion\nangives, skiftes til assembler-tilstand (indtast en tom linje for\nat afslutte assembler-tilstanden)."},
	/* de */ {IDGS_MON_ASSEMBLE_DESCRIPTION_DE, "Assmbler Instruktionen zur angebenen Adresse. Wenn nur eine Instruktion\nangegeben wurde, wird der assemblier Modus aktiviert (Eingabe einer leeren\nZeile beendet den assemblier Modus)."},
	/* es */ {IDGS_MON_ASSEMBLE_DESCRIPTION_ES, "Ensambla instrucciones en la direcciÛn especificada.  Si sÛlo se\nespecifica una instrucciÛn, entra en modo ensamblador, (entrar \nuna linea vacÌa para salir)."},
	/* fr */ {IDGS_MON_ASSEMBLE_DESCRIPTION_FR, "Assembler les instructions ‡ l'adresse spÈcifiÈe. Si seulement\nune instruction est spÈcifiÈe, entrez le mode assembleur (entrez\nune ligne vide pour quitter le mode assembleur)."},
	/* hu */ {IDGS_MON_ASSEMBLE_DESCRIPTION_HU, "UtasÌt·sok fordÌt·sa a megadott cÌmre. Ha csak egy utasÌt·s\nvan, fordÌtÛ mÛdba lÈp (¸res sort kell beÌrni a kilÈpÈshez)."},
	/* it */ {IDGS_MON_ASSEMBLE_DESCRIPTION_IT, "Assembla le istruzioni a partire dall'indirizzo specificato. Se solo una\nistruzione Ë specificata, entra in modalit‡ assembly (inserire una\nlinea vuota per uscire da questa modalit‡)."},
	/* ko */ {IDGS_MON_ASSEMBLE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_ASSEMBLE_DESCRIPTION_NL, "Assembleer instructies naar het opgegeven adres. Als er alleen een instructie\nis opgegeven, wordt de assembleer modus ingeschakeld (geef een lege regel\nin om de assembleer modus te verlaten)."},
	/* pl */ {IDGS_MON_ASSEMBLE_DESCRIPTION_PL, "Assembluje instrukcje pod podanym adresem. Je∂li podano tylko\njedn± instrukcjÍ, wchodzi w tryb asemblowania (pusta linia\nwychodzi z trybu asemblowania)."},
	/* ru */ {IDGS_MON_ASSEMBLE_DESCRIPTION_RU, "Assemble instructions to the specified address.  If only one\ninstruction is specified, enter assembly mode (enter an empty line to\nexit assembly mode)."},
	/* sv */ {IDGS_MON_ASSEMBLE_DESCRIPTION_SV, "Assemblera instruktioner till angiven adress. Om endast en\ninstruktion angavs, aktivera assemblerl‰ge (mata in en tom rad\nfˆr att avsluta assemblerl‰get)."},
	/* tr */ {IDGS_MON_ASSEMBLE_DESCRIPTION_TR, "Komutlar˝ belirlenmi˛ adrese Áevir.  Eer yaln˝zca bir\nkomut belirlenmi˛se, assembly moduna gir (assembly modundan Á˝kmak iÁin\nbo˛ bir sat˝r girin)."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_ADDRESS_RANGE,    N_("address_range")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ADDRESS_RANGE_DA, "adresseomrÂde"},
	/* de */ {IDGS_ADDRESS_RANGE_DE, "Adressbereich"},
	/* es */ {IDGS_ADDRESS_RANGE_ES, "Rango de direcciÛn"},
	/* fr */ {IDGS_ADDRESS_RANGE_FR, "plage_dadresses"},
	/* hu */ {IDGS_ADDRESS_RANGE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ADDRESS_RANGE_IT, "intervallo indirizzi"},
	/* ko */ {IDGS_ADDRESS_RANGE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ADDRESS_RANGE_NL, "adres_bereik"},
	/* pl */ {IDGS_ADDRESS_RANGE_PL, "address_range"},
	/* ru */ {IDGS_ADDRESS_RANGE_RU, "address_range"},
	/* sv */ {IDGS_ADDRESS_RANGE_SV, "adressomrÂde"},
	/* tr */ {IDGS_ADDRESS_RANGE_TR, "adres_aral˝˝"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_COMPARE_DESCRIPTION,    N_("Compare memory from the source specified by the address range to the\ndestination specified by the address.  The regions may overlap.  Any\nvalues that miscompare are displayed using the default displaytype.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_COMPARE_DESCRIPTION_DA, "Sammenlign hukommelse fra kilden, angivet ved adresseomrÂdet med\nmÂlet, angivet ved adresse. OmrÂderne kan overlappe. Alle vÊrdier\nsom er forskellige vil blive vist med standard visning."},
	/* de */ {IDGS_MON_COMPARE_DESCRIPTION_DE, "Vergleiche angegebenen Adressbereich mit der Zieladresse.\nDie Bereiche kˆnnen ¸berlappen. Ungleiche Werte werden mit dem\nStandardanzeigetyp gezeigt."},
	/* es */ {IDGS_MON_COMPARE_DESCRIPTION_ES, "Compara memoria desde la fuente especificada por el rango de direcciones\ndestino. La regiÛn puede estar superpuesta. Cualquier valor diferente se\nmostrar· usando el tipo de pantalla por defecto."},
	/* fr */ {IDGS_MON_COMPARE_DESCRIPTION_FR, "Comparer la mÈmoire de la source spÈcifiÈe par la plage d'adresse ‡ la\ndestination spÈcifiÈe par l'adresse. Les rÈgions peuvent se chevaucher.\nToute valeur discordante est affichÈe avec le type d'affichage par dÈfaut."},
	/* hu */ {IDGS_MON_COMPARE_DESCRIPTION_HU, "A forr·s memÛriatartom·ny ˆsszehasonlÌt·sa a cÈllal megadott\nmemÛriatartom·nnyal. A tartom·nyok ·tfedÈse megengedett.  Minden\nmeg nem egyezı ÈrtÈket kiÌr az alap adat tÌpussal."},
	/* it */ {IDGS_MON_COMPARE_DESCRIPTION_IT, "Confronta la memoria della sorgente specificata dall'intervallo di\nindirizzi con la destinazione specificata dall'indirizzo. Le regioni possono\nsovrapporsi. Qualsiasi valore che non corrisponde Ë visualizzato\nutilizzando il tipo di dato predefinito."},
	/* ko */ {IDGS_MON_COMPARE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_COMPARE_DESCRIPTION_NL, "Vergelijk geheugen van de bron opgegeven door de adres reeks met het doel\nopgegeven door het adres. De reeksen mogen overlappen. De data die niet\ngelijk is zal worden getoond volgens de huidige weergavesoort."},
	/* pl */ {IDGS_MON_COMPARE_DESCRIPTION_PL, "PorÛwnuje pamiÍÊ z podanego zakresu adresÛw z podanym adresem.\nRejony mog± siÍ pokrywaÊ. Wszystkie niepasuj±ce warto∂ci s± pokazane\ndomy∂lnym typem danych."},
	/* ru */ {IDGS_MON_COMPARE_DESCRIPTION_RU, "Compare memory from the source specified by the address range to the\ndestination specified by the address.  The regions may overlap.  Any\nvalues that miscompare are displayed using the default displaytype."},
	/* sv */ {IDGS_MON_COMPARE_DESCRIPTION_SV, "J‰mfˆr minne frÂn k‰llan som anges av adressomrÂdet mot destinationen\nsom anges av adressen. OmrÂdena kan vara ˆverlappande. Alla v‰rden\nsom skiljer sig vid j‰mfˆrelsen visas med standardvisningss‰ttet."},
	/* tr */ {IDGS_MON_COMPARE_DESCRIPTION_TR, "Bellei adresle belirtilen kaynaktan adresle belirtilen hedefin bulunduu\naral˝kta kar˛˝la˛t˝r.  Bˆlgeler belki ¸st ¸ste binebilir.  Uyu˛mayan\nherhangi bir deer varsay˝lan gˆr¸nt¸leme tipine bal˝ olarak gˆr¸nt¸lenir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_CHECKNUM,    N_("checknum")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CHECKNUM_DA, "checknum"},
	/* de */ {IDGS_CHECKNUM_DE, "Checknummer"},
	/* es */ {IDGS_CHECKNUM_ES, "n˙mero comprobaciÛn"},
	/* fr */ {IDGS_CHECKNUM_FR, "checknum"},
	/* hu */ {IDGS_CHECKNUM_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CHECKNUM_IT, "checknum"},
	/* ko */ {IDGS_CHECKNUM_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CHECKNUM_NL, "checknum"},
	/* pl */ {IDGS_CHECKNUM_PL, "checknum"},
	/* ru */ {IDGS_CHECKNUM_RU, "checknum"},
	/* sv */ {IDGS_CHECKNUM_SV, "kontrollnr"},
	/* tr */ {IDGS_CHECKNUM_TR, "kontrolnumaras˝"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DELETE_DESCRIPTION,    N_("Delete checkpoint `checknum'. If no checkpoint is specified delete all checkpoints.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DELETE_DESCRIPTION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MON_DELETE_DESCRIPTION_DE, "Lˆsche Checkpoint `Checknummer'. Falls kein Checkpoint angegeben wird,\nlˆsche alle Checkpoints."},
	/* es */ {IDGS_MON_DELETE_DESCRIPTION_ES, "Borrar Punto de control `checknum'. Si no se especifica ninguno, se borran todos."},
	/* fr */ {IDGS_MON_DELETE_DESCRIPTION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MON_DELETE_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_DELETE_DESCRIPTION_IT, "Cancella il punto di controllo `checknum'. Se non viene specificato alcun punto di controllo saranno cancellati tutti i punti di controllo."},
	/* ko */ {IDGS_MON_DELETE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DELETE_DESCRIPTION_NL, "Wis checkpunt 'checknum'. Als er geen checkpunt is gegeven, worden alle checkpunten gewist."},
	/* pl */ {IDGS_MON_DELETE_DESCRIPTION_PL, "Skasuj punkt kontrolny 'checknum'. Je∂li nie okre∂lono punktu kontrolnego, skasuj wszystkie."},
	/* ru */ {IDGS_MON_DELETE_DESCRIPTION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MON_DELETE_DESCRIPTION_SV, "Ta bort kontrollpunkten \"kontrollnr\". Om ingen kontrollpunkt har angivits, ta bort samtliga kontrollpunkter."},
	/* tr */ {IDGS_MON_DELETE_DESCRIPTION_TR, ""},  /* fuzzy */
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DISASS_DESCRIPTION,    N_("Disassemble instructions.  If two addresses are specified, they are\nused as a start and end address.  If only one is specified, it is\ntreated as the start address and a default number of instructions are\ndisassembled.  If no addresses are specified, a default number of\ninstructions are disassembled from the dot address.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DISASS_DESCRIPTION_DA, "DisassemblÈr instruktioner. Hvis der angives to adresser anvendes disse\nsom start- og slutadresse. Angives kun en adresse bruges denne som startadresse\nog et standard antal instruktioner disassembleres. Hvis ingen adresse\nangives disassembleres et standard antal instruktioner fra den aktuelle\nadresse."},
	/* de */ {IDGS_MON_DISASS_DESCRIPTION_DE, "Disassembliere Instruktionen. Wenn zwei Adressen angegeben werden,\nwerden diese als Start- und Zieladresse interpretiert. Wenn nur eine\nAdress angegeben wird, wird diese asl Startadresse interpretiert und\ndie Standardanzahl an Instruktionen werden disassembliert. Wenn keine\nAdresse angegeben wird, wird von der aktuellen Adresse disassembliert."},
	/* es */ {IDGS_MON_DISASS_DESCRIPTION_ES, "Desensamblar instrucciones. Si se especifican dos direcciones, se\nusan como direcciones de inicio y final. Si solo se suministra una, ser·\ntratada como direcciÛn de inicio y un n˙mero por defecto de  instrucciones\nser·n desensambladas. Si no se especifica ninguna direcciÛn, un n˙mero\npor defecto de instrucciones se desensamblaran desde la direcciÛn pre\ndefinida"},
	/* fr */ {IDGS_MON_DISASS_DESCRIPTION_FR, "DÈsassembler les instructions. Si deux adresse sont spÈcifiÈes, elles\nseront utilisÈes comme adresse de dÈpart et de fin. Si une seule adresse\nest spÈcifiÈe, elle sera traitÈe comme adresse de dÈpart et un nombre\npar dÈfaut d'instructions seront dÈsassemblÈes. Si aucune adresse n'est\nspÈcifiÈe, un nombre par dÈfaut d'instructions seront dÈsassemblÈes\ndepuis l'adresse marquÈe d'un point."},
	/* hu */ {IDGS_MON_DISASS_DESCRIPTION_HU, "UtasÌt·sok visszafejtÈse. Ha kÈt cÌm van megadva, elsı Ès utolsÛ\ncÌmkÈnt Èrtelmezi ıket. Ha csak egy, akkor elsı cÌmkÈnt, Ès az alap\nsz·m˙ utasÌt·s ker¸l visszafejtÈsre. Ha nincs cÌm megadva, az aktu·lis\ncÌmtıl kezdi a visszafejtÈst, Ès a szok·sos sz·m˙ utasÌt·st Ìr ki."},
	/* it */ {IDGS_MON_DISASS_DESCRIPTION_IT, "Disassembla le istruzioni. Se sono specificati due indirizzi, questi\nsono utilizzati come indirizzo di inizio e fine. Se ne viene specificato\nsolo uno, Ë usato come indirizzo di partenza e viene disassemblato\nil numero di istruzioni predefinito. Se non viene specificato nessun\nindirizzo, viene disassemblato il numero di istruzioni predefinito\ndall'indirizzo dot."},
	/* ko */ {IDGS_MON_DISASS_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DISASS_DESCRIPTION_NL, "Disassembleer instructies. Als er twee adressen zijn opgegeven, worden\ndeze gebruikt als een start en eind adres. Als er maar ÈÈn adres is\nopgegeven, wordt deze gebruikt als het start adres en een standaard\naantal instructies zal worden gedisassembleerd. Als er geen adressen zijn\nopgegeven, wordt een standaard aantal instructies gedisassembleerd vanaf\nhet huidige adres."},
	/* pl */ {IDGS_MON_DISASS_DESCRIPTION_PL, "Deasembluje instrukcje. Je∂li podano dwa adresy, uøywane s± jako\nadres pocz±tkowy i koÒcowy. Je∂li podano tylko jeden, jest uøywany\njako adres pocz±tkowy i deasemblowana jest domy∂lna liczba instrukcji\nz adresu dot."},
	/* ru */ {IDGS_MON_DISASS_DESCRIPTION_RU, "Disassemble instructions.  If two addresses are specified, they are\nused as a start and end address.  If only one is specified, it is\ntreated as the start address and a default number of instructions are\ndisassembled.  If no addresses are specified, a default number of\ninstructions are disassembled from the dot address."},
	/* sv */ {IDGS_MON_DISASS_DESCRIPTION_SV, "Disassemblera instruktioner. Om tvÂ adresser anges anv‰nds de som\nstart- och slutadress. Om endast en anges anv‰nds den som startadress\noch ett standardantal instruktioner disassembleras. Om ingen adress\nanges disassembleras ett standardantal instruktioner frÂn punktadressen."},
	/* tr */ {IDGS_MON_DISASS_DESCRIPTION_TR, "Komutlar˝ disassemble et.  Eer iki adres girildiyse, bunlar ba˛lang˝Á\nve biti˛ adresleri olarak kullan˝l˝r.  Eer tek adres girildiyse, bu\nadres ba˛lang˝Á adresi olarak kabul edilir ve varsay˝lan say˝da komutlar\ndisassemble edilir.  Eer hiÁ adres girilmediyse, varsay˝lan say˝da\nkomutlar nokta ba˛lang˝Á adresinden itibaren disassemble edilir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_FILL_DESCRIPTION,    N_("Fill memory in the specified address range with the data in\n<data_list>.  If the size of the address range is greater than the size\nof the data_list, the data_list is repeated.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_FILL_DESCRIPTION_DA, "Fyld hukommelsen i det angivne adresseomrÂde med data fra <dataliste>.\nHvis st¯rrelsen pÂ adresseomrÂdet er st¯rre end st¯rrelsen af dataliste\ngentages indholdet i dataliste."},
	/* de */ {IDGS_MON_FILL_DESCRIPTION_DE, "F¸lle den angegebenen Adressbereich mit den Daten aus <Datenliste>.\nWenn die Grˆﬂe des Adressbereichs grˆﬂer als <Datenliste> ist, werden\ndie Daten wiederholt."},
	/* es */ {IDGS_MON_FILL_DESCRIPTION_ES, "Rellenar memoria en el rango especificado con el dato contenido en\n <lista datos>. Si el tamaÒo del rango es mayor que el <lista datos>\nse repite lista datos."},
	/* fr */ {IDGS_MON_FILL_DESCRIPTION_FR, "Remplir la plage d'adresses mÈmoire avec les donnÈes de\n<data_list>. Si la longueur de la plage d'adresses est\nplus grande que cette de la data_list, alors la data_list\nsera rÈpÈtÈe."},
	/* hu */ {IDGS_MON_FILL_DESCRIPTION_HU, "A memÛria kitˆltÈse az <adat_lista> paramÈterben megadott adatokkal.\nHa a cÌmtartom·ny nagyobb, mint a lista, ismÈtelni fogja azt."},
	/* it */ {IDGS_MON_FILL_DESCRIPTION_IT, "Riempe la memoria nell'intervallo di indirizzi specificato con i dati\npresenti in <lista di dati>. Se la dimensione dell'intervallo Ë maggiore\ndella dimensione della lista di dati, la lista di dati Ë ripetuta."},
	/* ko */ {IDGS_MON_FILL_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_FILL_DESCRIPTION_NL, "Vul het geheugen van de opgegeven adresreeks op met de data in <data_lijst>.\nAls de grootte van de adresreeks groter is dan de grootte van de data_lijst,\nzal de data_lijst worden herhaald."},
	/* pl */ {IDGS_MON_FILL_DESCRIPTION_PL, "Wype≥nia pamiÍÊ w danym zakresie adresÛw danymi z <lista_danych>.\nJe∂li wielko∂Ê zakresu adresÛw jest wiÍksza niø wielko∂Ê listy_danych,\nlista_danych jest powtarzana."},
	/* ru */ {IDGS_MON_FILL_DESCRIPTION_RU, "Fill memory in the specified address range with the data in\n<data_list>.  If the size of the address range is greater than the size\nof the data_list, the data_list is repeated."},
	/* sv */ {IDGS_MON_FILL_DESCRIPTION_SV, "Fyll minnet i det angivna omrÂdet med data i <datalista>.\nOm storleken pÂ adressomrÂdet ‰r stˆrre ‰n storleken pÂ datalistan\nrepeteras innehÂllet i datalistan."},
	/* tr */ {IDGS_MON_FILL_DESCRIPTION_TR, "<veri_listesi>'nde bulunan verilerle belirtilen bellek aral˝˝n˝ doldur.\nEer verilen adres aral˝˝ veri_listesi'nden b¸y¸kse, veri_listesi'nde\nyer alan veriler tekrar ettirilir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_HUNT_DESCRIPTION,    N_("Hunt memory in the specified address range for the data in\n<data_list>.  If the data is found, the starting address of the match\nis displayed.  The entire range is searched for all possible matches.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_HUNT_DESCRIPTION_DA, "S¯g i hukommelsen i det angivne dataomrÂde efter <dataliste>.\nHvis data findes vises startadressen for matchet. Hele\nomrÂdet gennems¯ges for at finde alle mulige match."},
	/* de */ {IDGS_MON_HUNT_DESCRIPTION_DE, "Suche in angebenen Adressbereich nach <Datenliste>. Wenn die Daten gefunden\nwurden, wird die Startadresse gezeigt. Der komplette Bereich wird alle\nVorkommnisse von <Datenliste> untersucht."},
	/* es */ {IDGS_MON_HUNT_DESCRIPTION_ES, "Buscar en memoria en el rango de direcciones especificado por\n<lista datos>. Si se encuentra, se muestra la direcciÛn del dato.\nSe busca en todo el rango buscando posibles concordancias."},
	/* fr */ {IDGS_MON_HUNT_DESCRIPTION_FR, "Rechercher ‡ l'intÈrieur de la plage d'adresses pour les\ndonnÈes de <data_list>. Si les donnÈes sont trouvÈes,\nl'adresse de dÈpart de la concordance est affichÈe. La\nplage complËte est recherchÈe pour toutes les concordances\npossibles."},
	/* hu */ {IDGS_MON_HUNT_DESCRIPTION_HU, "Az <adat_lista> adat keresÈse a memÛri·ban. Ha megtal·lja a sorozatot,\na kezdıcÌmÈt kiÌrja. Az egÈsz tartom·nyt vÈgignÈzi az ˆsszes lehetsÈges\ntal·lat felfedezÈsÈhez."},
	/* it */ {IDGS_MON_HUNT_DESCRIPTION_IT, "Cerca nell'intervallo di indirizzi specificato i dati nella <lista di dati>.\nSe il dato Ë trovato, viene visualizzato l'indirizzo iniziale della\ncorrispondenza. Tutto l'intervallo viene ricercato per tutte le possibili\ncorrispondenze."},
	/* ko */ {IDGS_MON_HUNT_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_HUNT_DESCRIPTION_NL, "Zoek het adresbereik voor de data in <data_lijst>. Als de data is gevonden,\n wordt het startadres getoond. Er zal door de hele reeks worden gezocht."},
	/* pl */ {IDGS_MON_HUNT_DESCRIPTION_PL, "Szukaj danych z <lista_danych> w pamiÍci w podanym zakresie\nadresÛw. Je∂li znaleziono dane, pokazany jest pocz±tkowy adres\ntrafienia. Ca≥y zakres jest przeszukiwany dla wszystkich moøliwych	rafieÒ."},
	/* ru */ {IDGS_MON_HUNT_DESCRIPTION_RU, "Hunt memory in the specified address range for the data in\n<data_list>.  If the data is found, the starting address of the match\nis displayed.  The entire range is searched for all possible matches."},
	/* sv */ {IDGS_MON_HUNT_DESCRIPTION_SV, "Sˆk i minnet i det angivna omrÂdet med data efter <datalista>.\nOm data pÂtr‰ffas visas startadressen fˆr tr‰ffen. Hela\nomrÂdet genomsˆks fˆr att hitta alla mˆjliga tr‰ffar."},
	/* tr */ {IDGS_MON_HUNT_DESCRIPTION_TR, "<veri_listesi>'nde bulunan veriler iÁin belirtilen bellek aral˝˝nda\narama yap.  Eer veri bulunursa, e˛le˛en adres aral˝˝n˝n ba˛lang˝Á\nadresi gˆr¸nt¸lenir.  T¸m alan m¸mk¸n olan e˛le˛meler iÁin aranacakt˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_ADDRESS_OPT_RANGE,    N_("address_opt_range")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ADDRESS_OPT_RANGE_DA, "adresse_evt_omrÂde"},
	/* de */ {IDGS_ADDRESS_OPT_RANGE_DE, "Optionaler Adressbereich"},
	/* es */ {IDGS_ADDRESS_OPT_RANGE_ES, "Rango direcciÛn"},
	/* fr */ {IDGS_ADDRESS_OPT_RANGE_FR, "plage_dadresses_opt"},
	/* hu */ {IDGS_ADDRESS_OPT_RANGE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ADDRESS_OPT_RANGE_IT, "intervallo di indirizzi opzionale"},
	/* ko */ {IDGS_ADDRESS_OPT_RANGE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ADDRESS_OPT_RANGE_NL, "optionele_adres_bereik"},
	/* pl */ {IDGS_ADDRESS_OPT_RANGE_PL, "address_opt_range"},
	/* ru */ {IDGS_ADDRESS_OPT_RANGE_RU, "address_opt_range"},
	/* sv */ {IDGS_ADDRESS_OPT_RANGE_SV, "adress_ev_omrÂde"},
	/* tr */ {IDGS_ADDRESS_OPT_RANGE_TR, "opsiyonel_adres_aral˝˝"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_I_DESCRIPTION,    N_("Display memory contents as PETSCII text.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_I_DESCRIPTION_DA, "Vis hukommelsesindhold som PETSCII-text."},
	/* de */ {IDGS_MON_I_DESCRIPTION_DE, "Zeige Speicherinhalt als PETSCII Text."},
	/* es */ {IDGS_MON_I_DESCRIPTION_ES, "Mostrar contenido memoria como texto PETSCII."},
	/* fr */ {IDGS_MON_I_DESCRIPTION_FR, "Afficher le contenu de la mÈmoire en tant que texte PETSCII."},
	/* hu */ {IDGS_MON_I_DESCRIPTION_HU, "MemÛria tartalom mutat·sa PETSCII szˆvegkÈnt."},
	/* it */ {IDGS_MON_I_DESCRIPTION_IT, "Mostra il contenuto della memoria come testo PETSCII."},
	/* ko */ {IDGS_MON_I_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_I_DESCRIPTION_NL, "Toon geheugeninhoud als PETSCII tekst."},
	/* pl */ {IDGS_MON_I_DESCRIPTION_PL, "Pokaø zawarto∂Ê pamiÍci jako tekst PETSCII."},
	/* ru */ {IDGS_MON_I_DESCRIPTION_RU, "Display memory contents as PETSCII text."},
	/* sv */ {IDGS_MON_I_DESCRIPTION_SV, "Visa minnesinnehÂll som PETSCII-text."},
	/* tr */ {IDGS_MON_I_DESCRIPTION_TR, "Bellek iÁeriini PETSCII yaz˝ biÁiminde gˆr¸nt¸le."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_II_DESCRIPTION,    N_("Display memory contents as screen code text.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_II_DESCRIPTION_DA, "Vis hukommelses-indhold som skÊrm-koder."},
	/* de */ {IDGS_MON_II_DESCRIPTION_DE, "Zeige Speicherinhalt als Bildschirmcode Text."},
	/* es */ {IDGS_MON_II_DESCRIPTION_ES, "Mostrar contenidos de memoria con cÛdigo texto pantalla."},
	/* fr */ {IDGS_MON_II_DESCRIPTION_FR, "Afficher le contenu de la mÈmoire en tant que texte avec codes-Ècran."},
	/* hu */ {IDGS_MON_II_DESCRIPTION_HU, "MemÛria tartalom megjelenÌtÈse kÈpernyı kÛd szˆvegkÈnt."},
	/* it */ {IDGS_MON_II_DESCRIPTION_IT, "Mostra il contenuto della memoria come testo screen code."},
	/* ko */ {IDGS_MON_II_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_II_DESCRIPTION_NL, "Toon geheugen inhoud als schermcode tekst."},
	/* pl */ {IDGS_MON_II_DESCRIPTION_PL, "Pokaø zawarto∂Ê pamiÍci jako tekst screen code."},
	/* ru */ {IDGS_MON_II_DESCRIPTION_RU, "Display memory contents as screen code text."},
	/* sv */ {IDGS_MON_II_DESCRIPTION_SV, "Visa minnesinnehÂll som sk‰rmkodstext."},
	/* tr */ {IDGS_MON_II_DESCRIPTION_TR, "Bellek iÁeriini ekran kodu yaz˝ biÁiminde gˆr¸nt¸le."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_DATA_TYPE,    N_("data_type")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DATA_TYPE_DA, "datatype"},
	/* de */ {IDGS_DATA_TYPE_DE, "Datentyp"},
	/* es */ {IDGS_DATA_TYPE_ES, "tipo datos"},
	/* fr */ {IDGS_DATA_TYPE_FR, "type_de_donnÈes"},
	/* hu */ {IDGS_DATA_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_DATA_TYPE_IT, "dati"},
	/* ko */ {IDGS_DATA_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DATA_TYPE_NL, "data_soort"},
	/* pl */ {IDGS_DATA_TYPE_PL, "data_type"},
	/* ru */ {IDGS_DATA_TYPE_RU, "data_type"},
	/* sv */ {IDGS_DATA_TYPE_SV, "datatyp"},
	/* tr */ {IDGS_DATA_TYPE_TR, "veri_tipi"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_MEM_DESCRIPTION,    N_("Display the contents of memory.  If no datatype is given, the default\nis used.  If only one address is specified, the length of data\ndisplayed is based on the datatype.  If no addresses are given, the\n'dot' address is used.\nPlease note: due to the ambiguous meaning of 'b' and 'd' these data-\ntypes must be given in uppercase!")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_MEM_DESCRIPTION_DA, "Vis hukommelsens indhold. Hvis datatype ikke angives, bruges standard.\nHvis kun en adresse angives, justeres lÊngden pÂ visningen baseret\npÂ datatypen. Hvis der ikke angives nogen adresse anvendes \"punktum\"-adressen.\nNote: PÂ grund af tvetydighed for 'b' og 'd' skal disse datatyper\nangives med store bogstaver!"},
	/* de */ {IDGS_MON_MEM_DESCRIPTION_DE, "Zeige Speicherinhalt. Wenn kein Datentyp angegeben wurde, wird der\nStandarddatentyp verwendet. Wenn nur eine Adresse angegeben wird, h‰ngt\ndie L‰nge der Daten vom Datentyp ab. Wenn keine Adresse angeben wurde, \nwird die aktuelle Adresse verwendet.\nAchtung: Wegen der Mehrdeutigkeit von `b' und `d', m¸ssen die\nDatentypen in Groﬂbuchstaben angegeben werden!"},
	/* es */ {IDGS_MON_MEM_DESCRIPTION_ES, "Mostrar el contenido de memoria. Si no se da tipo de datos, se usa el\npor defecto. Si sÛlo se da una direcciÛn, la longitud de dato\nmostrado se basa en el tipo dato. Si no se dan direcciones, se \nusa la direcciÛn 'dot'Por favor, note que debido al ambiguo significado de 'b' y 'd' estos\ntipos de dato se mostrararn en may˙sculas."},
	/* fr */ {IDGS_MON_MEM_DESCRIPTION_FR, "Affiche le contenu de la mÈmoire. Si aucun type de donnÈes est donnÈ,\nle type par dÈfaut est utilisÈ. Si une seule adresse est spÈcifiÈe,\nla longueur des donnÈes affichÈes est basÈe sur le type de donnÈes.\nSi aucune adresse n'est donnÈe, l'adresse marquÈe d'un point est utilisÈe.\nSVP noter que vu l'ambiguÔtÈ de 'b' et 'd' ces types de donnÈes\ndoivent Ítres donnÈs en lettres majuscules!"},
	/* hu */ {IDGS_MON_MEM_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_MEM_DESCRIPTION_IT, "Mostra il contenuto della memoria. Se non Ë specificato alcun tipo di\ndato, verr‡ utilizzato quello predefinito. Se Ë specificato solo un indirizzo,\nla lunghezza del dato visualizzato Ë basata sul tipo. Se non Ë specificato\nnessun indirizzo, varr‡ utilizzato l'indirizzo 'dot'.Nota bene: a causa del significato ambiguo di 'b' e 'd' questi tipi di dati\ndevono essere inseriti in maiuscolo!"},
	/* ko */ {IDGS_MON_MEM_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_MEM_DESCRIPTION_NL, "Toon de geheugeninhoud. Als er geen datasoort is opgegeven, zal de standaard\nworden gebruikt. Als er maar ÈÈn adres is opgegeven, zal de lengte van de\ngetoonde data gebaseerd worden op de datasoort. Als er geen adres is opgegeven,\nzal het huidige adres worden gebruikt. Door de onduidelijke betekenis \nvan 'b' en 'd' moeten deze datasoorten opgegeven worden in hoofdletters!"},
	/* pl */ {IDGS_MON_MEM_DESCRIPTION_PL, "Pokazuje zawarto∂Ê pamiÍci. Je∂li nie ustalono typu danych, jest\nuøywany domy∂lny. Je∂li okre∂lono tylko jeden adres, d≥ugo∂Ê\npokazanych danych zaleøy od ich typu. Je∂li nie podano adresu,\nuøywany jest adres 'kropka'. Uwaga: w zwi±zku z wieloznaczno∂ci± 'b' i 'd', te typy danych\nnaleøy podaÊ wielk± liter±!"},
	/* ru */ {IDGS_MON_MEM_DESCRIPTION_RU, "Display the contents of memory.  If no datatype is given, the default\nis used.  If only one address is specified, the length of data\ndisplayed is based on the datatype.  If no addresses are given, the\n'dot' address is used.\nPlease note: due to the ambiguous meaning of 'b' and 'd' these data-\ntypes must be given in uppercase!"},
	/* sv */ {IDGS_MON_MEM_DESCRIPTION_SV, "Visa minnets innehÂll. Om ingen datatyp anges, anv‰nds standard.\nOm endast en adress anges, justeras l‰ngden pÂ visad data baserat\npÂ datatypen. Om ingen adress anges anv‰nds \"punkt\"-adressen.\nObservera: pÂ grund av tvetydig betydelse av \"b\" och \"d\" mÂste dessa\ndatatyper anges med stora bokst‰ver!"},
	/* tr */ {IDGS_MON_MEM_DESCRIPTION_TR, "Bellek iÁeriini gˆster.  Eer veri tipi girilmemi˛se, varsay˝lan\nkullan˝l˝r.  Yaln˝zca bir adres belirtilmi˛se, gˆsterilen verinin\nuzunluu veri tipine bal˝d˝r.  Eer hiÁ adres girilmemi˛se, 'nokta'\nadresi kullan˝l˝r.\nL¸tfen dikkat: 'b' ve 'd'nin karma˛˝k anlamlar˝ndan ˆt¸r¸ bu veri\ntipleri b¸y¸kharf olarak verilmelidir!"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_MEMCHAR_DESCRIPTION,    N_("Display the contents of memory as character data.  If only one address\nis specified, only one character is displayed.  If no addresses are\ngiven, the ``dot'' address is used.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_MEMCHAR_DESCRIPTION_DA, "Vis hukommelsens indhold som tegndata. Hvis kun en adresse\nangives, vises kun et tegn. Hvis ingen adresse angives benyttes\n\"punktum\"-adressen."},
	/* de */ {IDGS_MON_MEMCHAR_DESCRIPTION_DE, "Zeige den Inhalt von Speicher als Character Daten. Wenn nur eine Adresse\nangegeben wurde, wird nur ein Zeichen gezeigt. Wenn keine Adresse angegeben\nwurde, wird die aktuelle Adresse verwendet."},
	/* es */ {IDGS_MON_MEMCHAR_DESCRIPTION_ES, "Mostrar el contenido de memoria como datos de car·cter. Si sÛlo se una\ndirecciÛn es especificada, sÛlo se muestra un car·cter. Si no se da\nninguna direcciÛn, se usa la direcciÛn 'dot'."},
	/* fr */ {IDGS_MON_MEMCHAR_DESCRIPTION_FR, "Affiche le contenu de la mÈmoire en tant que caractËres. Si une seule\nadresse est spÈcifiÈe, un seul caractËre est donnÈ. si aucune adresse\nn'est spÈcifiÈe, alors l'adresse marquÈe d'un point est utilisÈe."},
	/* hu */ {IDGS_MON_MEMCHAR_DESCRIPTION_HU, "A memÛria tartalm·nak megjelenÌtÈse karakter grafikakÈnt.  Ha csak egy cÌm\nadott, egyetlen karakter rajzolÛdik ki.  Ha nincs cÌm megadva,\na ``pont'' cÌmet haszn·lja."},
	/* it */ {IDGS_MON_MEMCHAR_DESCRIPTION_IT, "Mosta il contenuto della memoria come dati caratteri. Se Ë specificato\nsolo un indirizzo, verr‡ visualizzato solo un carattere. Se non Ë\nspecificato alcun indirizzo, varr‡ utilizzato l'indirizzo ``dot'' ."},
	/* ko */ {IDGS_MON_MEMCHAR_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_MEMCHAR_DESCRIPTION_NL, "Toon de geheugeninhoud als karakterdata. Als er maar ÈÈn adres is\nopgegeven, zal er maar ÈÈn karakter worden weergegeven. Als er geen\nadres is opgegeven, zal het huidige adres gebruikt worden."},
	/* pl */ {IDGS_MON_MEMCHAR_DESCRIPTION_PL, "Pokaø zawarto∂Ê pamiÍci jako dane znakowe. Je∂li podano tylko jeden\nadres, pokazany jest tylko jeden znak. Je∂li nie podano adresÛw, uøyty\njest adres 'kropka'."},
	/* ru */ {IDGS_MON_MEMCHAR_DESCRIPTION_RU, "Display the contents of memory as character data.  If only one address\nis specified, only one character is displayed.  If no addresses are\ngiven, the ``dot'' address is used."},
	/* sv */ {IDGS_MON_MEMCHAR_DESCRIPTION_SV, "Visa minnets innehÂll som teckendata. Om endast en adress\nanges, visas endast ett tecken. Om ingen adress anges anv‰nds\n\"punkt\"-adressen."},
	/* tr */ {IDGS_MON_MEMCHAR_DESCRIPTION_TR, "Bellek iÁeriini karakter verisi olarak gˆster.  Yaln˝zca bir adres\nbelirtilmi˛se, yaln˝zca tek karakter gˆr¸nt¸lenir.  Eer hiÁ adres\ngirilmemi˛se, 'nokta' adresi kullan˝l˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_FORMAT,    N_("Format")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_FORMAT_DA, "Format"},
	/* de */ {IDGS_FORMAT_DE, "Format"},
	/* es */ {IDGS_FORMAT_ES, "Formato"},
	/* fr */ {IDGS_FORMAT_FR, "Format"},
	/* hu */ {IDGS_FORMAT_HU, "Form·tum"},
	/* it */ {IDGS_FORMAT_IT, "Formato"},
	/* ko */ {IDGS_FORMAT_KO, "±∏º∫ πÊΩƒ"},
	/* nl */ {IDGS_FORMAT_NL, "Formaat"},
	/* pl */ {IDGS_FORMAT_PL, "Format"},
	/* ru */ {IDGS_FORMAT_RU, "ƒﬁ‡‹–‚"},
	/* sv */ {IDGS_FORMAT_SV, "Format"},
	/* tr */ {IDGS_FORMAT_TR, "BiÁim"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION,    N_("Save the memmap as a picture. Format is:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_DA, "Gem memmap som billede. Format:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* de */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_DE, "Speicher die memmap als Bilddaten. Formate:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* es */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_ES, "Graba mapa memoria como gr·fico. el formato es:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* fr */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_FR, "Sauvegarder le plan de la mÈmoire comme une image.\nLes formats sont:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* hu */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_HU, "MemÛria tÈrkÈp mentÈse kÈpkÈnt. A form·tum:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* it */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_IT, "Salva la mappa di memoria come immagine. I Formati sono:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* ko */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_NL, "Sla de memmap op als een afbeelding. Formaat is:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* pl */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_PL, "Zapisz mapÍ pamiÍci jako obrazek. Format:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* ru */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_RU, "Save the memmap as a picture. Format is:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* sv */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_SV, "Sparar minneskartan som en bild. Format:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* tr */ {IDGS_MON_MEMMAPSAVE_DESCRIPTION_TR, "Bellek haritas˝n˝ resim olarak kaydet. BiÁim:\n0 = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MASK,    N_("mask")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MASK_DA, "maske"},
	/* de */ {IDGS_MASK_DE, "Maske"},
	/* es */ {IDGS_MASK_ES, "m·scara"},
	/* fr */ {IDGS_MASK_FR, "masque"},
	/* hu */ {IDGS_MASK_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MASK_IT, "maschera"},
	/* ko */ {IDGS_MASK_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MASK_NL, "masker"},
	/* pl */ {IDGS_MASK_PL, "mask"},
	/* ru */ {IDGS_MASK_RU, "mask"},
	/* sv */ {IDGS_MASK_SV, "mask"},
	/* tr */ {IDGS_MASK_TR, "maske"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION,    N_("Show the memmap. The mask can be specified to show only those\nlocations with accesses of certain type(s). The mask is a number\nwith the bits \"ioRWXrwx\", where RWX are for ROM and rwx for RAM.\nOptionally, an address range can be specified.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_DA, "Vis memmap. Masken kan angives for kun at vise de celler, som\ner tilgÂet med valgte typer tilgang. Masken er et tal med\nbitsne \"ioRWXrwx\", hvor RWX er ROM og rwx er RAM.\nEt adresseomrÂde kan eventuelt angives."},
	/* de */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_DE, "Zeige die memmap. Die Maske kann so definiert werden, dass nur jene Orte\nmit Zugriff auf spezifische Typ(en) gezeigt werden. Die Maske ist eine Nummer\nmit Bits \"ioRWXrwx\", wobei RWX f¸r ROM und rwx f¸r RAM steht.\nOptional kann ein Adressbereich definiert werden."},
	/* es */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_ES, "Mostrar el mapa de memoria. La mascara puede definirse para mostrar\nsÛlo aquellas localizaciones con un determinado tipo(s). La m·scara es\nun n˙mero con bits \"ioRVXrvx\", donde RWX son para ROM y rwx son\npara RAM. Opcionalmente, se puede especificar un rango de direcciones."},
	/* fr */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_FR, "Affiche le plan de la mÈmoire. Un masque peut Ítre spÈcifiÈ pour\nne montrer que les emplacements avec accËs ‡ certains type(s).\nLe masque est un nombre avec les bits \"ioRWXrwx\", ou RWX est\npour le ROM et rwx pour le RAM.\nOptionnellement, une plage d'adresses peut Ítre spÈcifiÈe."},
	/* hu */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_HU, "A memÛria tÈrkÈp mutat·sa. A maszk ˙gy ·llÌthatÛ, hogy csak azokat a\nhelyeket mutassa, amelyek a megadott elÈrÈssel rendelkeznek. A maszk\negy sz·m az \"ioRWXrwx\" bitekkel, ahol az RWX a ROM-ra, az rwx a RAM-ra\nvonatkozik. TetszÈs szerint egy cÌmtartom·ny is megadhatÛ."},
	/* it */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_IT, "Mostra la mappa di memoria. La maschera puÚ essere specificata per\nmostrare solo quelle locazioni con accessi di un certo tipo. La maschera\nË un numero con i bit \"ioRWXrwx\", dove RWX sono per la ROM e\nrwx per la RAM.\nOpzionalmente, puÚ essere specificato anche un intervallo di indirizzi."},
	/* ko */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_NL, "Toon de memmap. Het masker kan worden opgegeven zodat er alleen de\nadressen met toegangen van een bepaalde soort(en) zal worden getoond.\nHet masker is een nummer met de bits \"ioRWXrwx\", waar RWX staat voor ROM,\nen rwx voor RAM. Er kan optioneel een adresbereik worden opgegeven."},
	/* pl */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_PL, "Pokazuje mapÍ pamiÍci. By pokazaÊ tylko obszar dostÍpÛw danego typu,\nmoøna okre∂liÊ maskÍ. Maska to numer zawieraj±cy bity \"ioRWXrwx\", gdzie\nRWX to ROM, a rwx to RAM. Opcjonalnie moøna okre∂liÊ zakres adresÛw."},
	/* ru */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_RU, "Show the memmap. The mask can be specified to show only those\nlocations with accesses of certain type(s). The mask is a number\nwith the bits \"ioRWXrwx\", where RWX are for ROM and rwx for RAM.\nOptionally, an address range can be specified."},
	/* sv */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_SV, "Visa minneskartan. Masken kan anges fˆr att endast visa de\nceller som g‰ller specifik(a) typ(er). Masken ‰r ett tal\nmed bitarna \"ioRWXrwx\", d‰r RWX ‰r fˆr ROM och rwx ‰r fˆr\nRAM. Ett minnesomrÂde kan eventuellt anges."},
	/* tr */ {IDGS_MON_MEMMAPSHOW_DESCRIPTION_TR, "Bellek haritas˝n˝ gˆster. Maske, yaln˝zca belirli tiplerce eri˛ilebilen\nkonumlarda belirtilebilir. Maske \"ioRWXrwx\" bitlerinden olu˛an bir\nsay˝d˝r. RWX ROM'u, rwx RAM'i ifade eder. Opsiyonel olarak bir adres\naral˝˝ belirtilebilir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_MEMMAPZAP_DESCRIPTION,    N_("Clear the memmap.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_DA, "Nulstil memmap."},
	/* de */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_DE, "Lˆsche die memmap."},
	/* es */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_ES, "Limpiar mapa de memoria."},
	/* fr */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_FR, "Effacer le plan de la mÈmoire."},
	/* hu */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_HU, "A memÛria tÈrkÈp tˆrlÈse."},
	/* it */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_IT, "Cancella la mappa di meoria."},
	/* ko */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_NL, "Veeg de memmap schoon."},
	/* pl */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_PL, "Wyczy∂Ê mapÍ pamiÍci."},
	/* ru */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_RU, "Clear the memmap."},
	/* sv */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_SV, "Tˆm minneskartan."},
	/* tr */ {IDGS_MON_MEMMAPZAP_DESCRIPTION_TR, "Bellek haritas˝n˝ temizle."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_MEMSPRITE_DESCRIPTION,    N_("Display the contents of memory as sprite data.  If only one address is\nspecified, only one sprite is displayed.  If no addresses are given,\nthe ``dot'' address is used.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_MEMSPRITE_DESCRIPTION_DA, "Vis hukommelsens indhold som spritedata. Hvis kun en adresse\nangives, vises kun en sprite. Hvis ingen adresse angives bruges\n\"punktum\"-adressen."},
	/* de */ {IDGS_MON_MEMSPRITE_DESCRIPTION_DE, "Zeige den Speicher als Sprite Daten. Wenn nur eine Adresse angegeben wurde,\nwird nur ein Sprite gezeitg. Wenn keine Adresse angegeben wurde, wird\ndie aktuelle Adresse verwendet."},
	/* es */ {IDGS_MON_MEMSPRITE_DESCRIPTION_ES, "Mostrar el contenido de memoria como datos de sprites, Si sÛlo se \nespecifica una direcciÛn, solo se muestra un sprite. Si no se dan\ndirecciones, se usa la direcciÛn 'dot'."},
	/* fr */ {IDGS_MON_MEMSPRITE_DESCRIPTION_FR, "Afficher le contenu de la mÈmoire en tant que donnÈes sprite. Si une\nseule adresse est spÈcifiÈe, un seul sprite est affichÈ. Si aucune\nadresse n'est spÈcifiÈe, le sprite ‡ l'adresse marquÈe d'un point est\naffichÈ."},
	/* hu */ {IDGS_MON_MEMSPRITE_DESCRIPTION_HU, "A memÛria tartalm·nak megjelenÌtÈse sprite adatkÈne. Ha csak egy cÌm\nvan megadva, egyetlen sprite-ot mutat.  Ha egy·ltal·n nincs cÌm megadva,\naz aktu·lis cÌmet haszn·lja."},
	/* it */ {IDGS_MON_MEMSPRITE_DESCRIPTION_IT, "Mostra il contenuto della memoria come dati sprite. Se Ë specificato\nsolo un indirizzo, verr‡ mostrato solo uno sprite. Se non Ë specificato\nalcun indirizzo, varr‡ utilizzato l'indirizzo ``dot''."},
	/* ko */ {IDGS_MON_MEMSPRITE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_MEMSPRITE_DESCRIPTION_NL, "Toon de geheugeninhoud als sprite data. Als er maar ÈÈn adres is opgegeven,\nzal er maar ÈÈn sprite worden getoond. Als er geen adres is opgegeven,\nzal het huidige adres worden gebruikt."},
	/* pl */ {IDGS_MON_MEMSPRITE_DESCRIPTION_PL, "Pokazuje zawarto∂Ê pamiÍci jako dane duszkÛw. Je∂li podano tylko jeden\nadres, pokazywany jest tylko jeden duszek. Je∂li nie podano adresu,\nuøywany jest adres ``dot'."},
	/* ru */ {IDGS_MON_MEMSPRITE_DESCRIPTION_RU, "Display the contents of memory as sprite data.  If only one address is\nspecified, only one sprite is displayed.  If no addresses are given,\nthe ``dot'' address is used."},
	/* sv */ {IDGS_MON_MEMSPRITE_DESCRIPTION_SV, "Visa minnets innehÂll som spritedata. Om endast en adress\nanges, visas endast en sprite. Om ingen adress anges anv‰nds\n\"punkt\"-adressen."},
	/* tr */ {IDGS_MON_MEMSPRITE_DESCRIPTION_TR, "Bellek iÁeriini yarat˝k verisi olarak gˆster.  Yaln˝zca bir adres\nbelirtilmi˛se, yaln˝zca tek yarat˝k gˆr¸nt¸lenir.  Eer hiÁ adres\ngirilmemi˛se, 'nokta' adresi kullan˝l˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_MOVE_DESCRIPTION,    N_("Move memory from the source specified by the address range to\nthe destination specified by the address.  The regions may overlap.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_MOVE_DESCRIPTION_DA, "KopiÈr data fra kilden angivet af hukommelsesomrÂdet til destinationen\nsom angives af adressen. Regionerne kan overlappe."},
	/* de */ {IDGS_MON_MOVE_DESCRIPTION_DE, "Verschiebe Speicher von Startadresse definiert durch den Adressbereich zum\nZielbereich definiert durch die Adresse. Die Bereiche kˆnnen ¸berlappend sein."},
	/* es */ {IDGS_MON_MOVE_DESCRIPTION_ES, "Mueve memoria desde la fuente especificada por rango de direcciones\nal destino especificado en direcciÛn. La regiÛn puede estas superpuesta."},
	/* fr */ {IDGS_MON_MOVE_DESCRIPTION_FR, "DÈplacer la mÈmoire de la source spÈcifiÈe par la plage d'adresses\n‡ la destination spÈcifiÈe par l'adresse. Les rÈgions peuvent se\nchevaucher."},
	/* hu */ {IDGS_MON_MOVE_DESCRIPTION_HU, "MemÛria m·sol·sa a forr·s ter¸letrıl a cÈl cÌm ·ltal meghat·rozott\nter¸letre. A tartom·nyok ·tfedÈse megengedett."},
	/* it */ {IDGS_MON_MOVE_DESCRIPTION_IT, "Sposta la memoria dalla sorgente specificata dall'intervallo di indirizzi\nalla destinazione specificata dall'indirizzo. Le regioni possono\nsovrapporsi."},
	/* ko */ {IDGS_MON_MOVE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_MOVE_DESCRIPTION_NL, "Verplaats data van de opgegeven bron adres reeks naar het doel adres. De\nreeksen kunnen overlappen."},
	/* pl */ {IDGS_MON_MOVE_DESCRIPTION_PL, "Przesuwa pamiÍÊ z okre∂lonego zakresu adresÛw pod podany adres.\nZakresy mog± siÍ pokrywaÊ."},
	/* ru */ {IDGS_MON_MOVE_DESCRIPTION_RU, "Move memory from the source specified by the address range to\nthe destination specified by the address.  The regions may overlap."},
	/* sv */ {IDGS_MON_MOVE_DESCRIPTION_SV, "Flytta minne frÂn k‰lladressen som anges av minnesomrÂdet\ntill destinationen som anges av adressen. Regionerna kan vara\nˆverlappande."},
	/* tr */ {IDGS_MON_MOVE_DESCRIPTION_TR, "Bellei belirtilmi˛ kaynak adres aral˝˝ndan belirtilmi˛ hedef\nadresine ta˛˝.  Alanlar ¸st ¸ste binebilir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_CHECKPOINT_COMMANDS,    N_("Checkpoint commands:")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CHECKPOINT_COMMANDS_DA, "Checkpoint-kommandoer:"},
	/* de */ {IDGS_CHECKPOINT_COMMANDS_DE, "Checkpoint Kommandos:"},
	/* es */ {IDGS_CHECKPOINT_COMMANDS_ES, "Comandos de control:"},
	/* fr */ {IDGS_CHECKPOINT_COMMANDS_FR, "Commandes de points d'arrÍt:"},
	/* hu */ {IDGS_CHECKPOINT_COMMANDS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CHECKPOINT_COMMANDS_IT, "Comandi punti di controllo:"},
	/* ko */ {IDGS_CHECKPOINT_COMMANDS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CHECKPOINT_COMMANDS_NL, "Checkpunt commando's:"},
	/* pl */ {IDGS_CHECKPOINT_COMMANDS_PL, "Komendy punktÛw kontrolnych:"},
	/* ru */ {IDGS_CHECKPOINT_COMMANDS_RU, "Checkpoint commands:"},
	/* sv */ {IDGS_CHECKPOINT_COMMANDS_SV, "Kommandon fˆr brytpunkter:"},
	/* tr */ {IDGS_CHECKPOINT_COMMANDS_TR, "Kontrol Noktas˝ komutlar˝:"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_COND_EXPR,    N_("cond_expr")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_COND_EXPR_DA, "betingelse"},
	/* de */ {IDGS_COND_EXPR_DE, "bedingter Ausdruck"},
	/* es */ {IDGS_COND_EXPR_ES, "Expr_cond"},
	/* fr */ {IDGS_COND_EXPR_FR, "expr_cond"},
	/* hu */ {IDGS_COND_EXPR_HU, ""},  /* fuzzy */
	/* it */ {IDGS_COND_EXPR_IT, "espressione condizione"},
	/* ko */ {IDGS_COND_EXPR_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_COND_EXPR_NL, "cond_expr"},
	/* pl */ {IDGS_COND_EXPR_PL, "cond_expr"},
	/* ru */ {IDGS_COND_EXPR_RU, "cond_expr"},
	/* sv */ {IDGS_COND_EXPR_SV, "villkorsuttryck"},
	/* tr */ {IDGS_COND_EXPR_TR, "ko˛ul_idafesi"},
#endif
	
	/*
	 Translators: this command was extended:
	 
	 - "load|store|exec" option added
	 
	 TODO: update these languages: DA, DE, HU, IT, NL, SV, TR
	 */
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_BREAK_DESCRIPTION,    N_("Set a breakpoint, If no address is given, the currently valid break-\npoints are printed.\nIf an address is given, a breakpoint is set for that address and the\nbreakpoint number is printed.\n`load|store|exec' is either `load', `store' or `exec' (or any combina-\ntion of these) to specify on which operation the monitor breaks. If\nnot specified, the monitor breaks on `exec'.\nA conditional expression can also be specified for the breakpoint.\nFor more information on conditions, see the CONDITION command.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_BREAK_DESCRIPTION_DA, "SÊt et breakpoint. Hvis der ikke angives en adresse vises en liste\nover nuvÊrende gyldige breakpoints.\nHvis der angives en adresse vil der blive sat et breakpoint pÂ\ndenne, og breakpointets nummer udskrives.\n`load|store|exec' er enten `load', `store' eller `exec' (eller en\nvilkÂrlig kombination), og angiver pÂ hvilken operation monitoren\nafbryder. Hvis intet angives vil monitoren afbryde pÂ `exec'.\nDer kan ogsÂ angives en betingelse for det angivne breakpoint. For\nyderligere information om betingede udtryk, se kommandoen CONDITION."},
	/* de */ {IDGS_MON_BREAK_DESCRIPTION_DE, "Setze einen Breakpoint. Wenn keine Adresse angegeben wurde, werden die\naktuellen g¸ltigen Breakpoints gezeigt. \nWenn eine Adresse angegebene wurde, wird ein Breakpoint auf diese Adresse\ngesetzt und die Breakpoint Nummer wird ausgegeben.\n`load|store|exec' ist entweder `load', `store' oder `exec' (oder eine\nKombination) und dient zur Angabe, bei welcher Operation der Monitor\nanhalten soll. Wenn keine Angabe gemacht wird, stoppt der Monitor bei\n`exec'.\nEin bedingter Ausdruck kann zu Breakpoints zugeordnet werden. F¸r\nmehr Informations ¸ber bedingte Ausdr¸cke, bitte Kommando CONDITION\nansehen."},
	/* es */ {IDGS_MON_BREAK_DESCRIPTION_ES, "Poner punto de ruptura. Si no se da direcciÛn, se imprime el actual punto\nde control. Si se da una direcciÛn, se activa un punto de ruptura en la misma\ny se imprime el n˙mero de punto de ruptura.\n`load|store|exec' es cualquiera de `load', `store' or `exec' (o combinaciÛn\nde ellos) a especificar en que operaciÛn el monitor se detiene. Si no se\nespecifica, el monitor se detiene en `exec'.\nSe puede especificar tambiÈn una condiciÛn para el punto de ruptura.\nPara m·s informaciÛn sobre condiciones, ver el comando CONDITION."},
	/* fr */ {IDGS_MON_BREAK_DESCRIPTION_FR, "CrÈer un point d'arrÍt. Si aucune adresse n'est spÈcifiÈe, les points\nd'arrÍt valides sont affichÈs.\nsi une adresse est spÈcifiÈe, un point d'arrÍt est crÈÈ pour cette\nadresse et le numÈro de point d'arrÍt est affichÈ.\n`load|store|exec' est soit `load', `store' ou `exec' (ou toute\ncombinaison) pour spÈcifier ‡ quelle opÈration le moniteur arrÍte.\nSi aucun n'est spÈcifiÈ, le moniteur arrÍte ‡ `exec'.\nUne expression conditionnelle peut aussi Ítre spÈcifiÈe pour le\npoint d'arrÍt. Pour plus d'informations sur les conditions,\nvoir la commande CONDITION."},
	/* hu */ {IDGS_MON_BREAK_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_BREAK_DESCRIPTION_IT, "Imposta un breakpoint. Se non viene specificato alcun indirizzo,\nvengono stampati i breakpoint attuali.\nSe viene specificato un indirizzo, viene impostato un breakpoint\nper tale indirizzo e viene stampato il suo numero.\n`load|store|exec' Ë uno tra `load', `store' o `exec' (or una qualunque\ncombinazione di questi) per specificare su quale operazione il monitor\ninterromper‡. Se non specificato, il monitor interrompe su `exec'.\nPuÚ anche essere specificata una condizione per il breakpoint.\nPer ulteriori informazioni, vedere il comando CONDITION."},
	/* ko */ {IDGS_MON_BREAK_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_BREAK_DESCRIPTION_NL, "Zet een breekpunt, als er geen adres is opgegeven, zullen de huidige\ngeldige breekpunten worden geprint. Als er een adres is opgegeven,\nzal er een breekpunt gezet worden voor dat adres en het nummer van het\nbreekpunt zal worden getoond.\n`load|store|exec' is `load', `store' of `exec' (of een combinatie) die\ngebruikt zal worden om aan te geven bij welke operatie de monitor zou\nmoeten starten. Als er niets is opgegeven, zal de monitor op 'exec'\nstarten.\nEen conditie kan worden opgegeven voor het breekpunt. Voor meer\ninformatie over condities zie de CONDITION commando."},
	/* pl */ {IDGS_MON_BREAK_DESCRIPTION_PL, "Ustawia punkt wstrzymania. Je∂li nie podano adresu, pokazywane s±\nobecnie aktywne punkty wstrzymania.\nJe∂li podano adres, ustawiany jest punkt wstrzymania dla tego adresu\ni pokazywany jest numer punktu wstrzymania.\n`load|store|exec' to `load', `store' lub `exec' (albo ich po≥±czenia),\nokre∂laj±ce  podczas ktÛrych operacji nastÍpuje b≥±d. Je∂li nic\nnie okre∂lono, monitor przerywa dzia≥anie na `exec'.\nMoøna rÛwnieø okre∂liÊ wyraøenie warunkowe dla punktu wstrzymania.\nBy znaleºÊ wiÍcej informacji o warunkach, zobacz polecenie CONDITION."},
	/* ru */ {IDGS_MON_BREAK_DESCRIPTION_RU, "Set a breakpoint, If no address is given, the currently valid break-\npoints are printed.\nIf an address is given, a breakpoint is set for that address and the\nbreakpoint number is printed.\n`load|store|exec' is either `load', `store' or `exec' (or any combina-\ntion of these) to specify on which operation the monitor breaks. If\nnot specified, the monitor breaks on `exec'.\nA conditional expression can also be specified for the breakpoint.\nFor more information on conditions, see the CONDITION command."},
	/* sv */ {IDGS_MON_BREAK_DESCRIPTION_SV, "S‰tt en ˆvervakningspunkt, Om ingen adress anges visas de \nˆvervakningspunkter som ‰r giltiga.\nOm en adress anges s‰tts en brytpunkt pÂ den adressen och dess\nnummer visas.\n\"load|store|exec\" ‰r antingen \"load\", \"store\" eller \"exec\"\n(eller en kombination d‰rutav) fˆr att ange pÂ vilken operation\nmonitorn stoppar. Om det inte anges bryter monitorn pÂ \"exec\".\nEtt villkorsuttryck kan ocksÂ anges fˆr brytpunkten.\nFˆr mer information om villkor, se kommandot CONDITION."},
	/* tr */ {IDGS_MON_BREAK_DESCRIPTION_TR, "Kesme noktas˝ koy, eer adres girilmemi˛se, geÁerli mevcut izleme\nnoktalar˝ yazd˝r˝l˝r.\nEer bir adres girilmi˛se, o adres iÁin bir kesme noktas˝ i˛aretlenir\nve kesme noktas˝ say˝s˝ yazd˝r˝l˝r.\n`y¸kle|depola|y¸r¸t' iÁin `y¸kle', `depola' ya da `y¸r¸t' (ya da herhangi\nbir kombinasyonu) takip edilmek istenen operasyona gˆre belirtilmelidir.\nEer belirtilmemi˛se, takip i˛lemi `y¸r¸t'de sona erer.\nAyn˝ zamanda kesme noktas˝ iÁin bir ko˛ul ifadesi de belirtilebilir.\nKo˛ullar ile ilgili daha Áok bilgi almak iÁin, CONDITION komutuna\nbakabilirsiniz."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_COMMAND,    N_("Command")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_COMMAND_DA, "Kommando"},
	/* de */ {IDGS_COMMAND_DE, "Kommando"},
	/* es */ {IDGS_COMMAND_ES, "Comando"},
	/* fr */ {IDGS_COMMAND_FR, "Commande"},
	/* hu */ {IDGS_COMMAND_HU, ""},  /* fuzzy */
	/* it */ {IDGS_COMMAND_IT, "Comando"},
	/* ko */ {IDGS_COMMAND_KO, "∏Ì∑…"},
	/* nl */ {IDGS_COMMAND_NL, "Commando"},
	/* pl */ {IDGS_COMMAND_PL, "Polecenie"},
	/* ru */ {IDGS_COMMAND_RU, "Command"},
	/* sv */ {IDGS_COMMAND_SV, "Kommando"},
	/* tr */ {IDGS_COMMAND_TR, "Komut"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_COMMAND_DESCRIPTION,    N_("Specify `command' as the command to execute when checkpoint `checknum'\nis hit.  Note that the `x' command is not yet supported as a\ncommand argument.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_COMMAND_DESCRIPTION_DA, "Angiv \"kommando\" som kommando der skal eksekveres nÂr ckeckpoint\n\"checknum\" nÂs. VÊr opmÊrksom pÂ at kommandoen \"x\" endnu ikke\ner underst¯ttet som parameter."},
	/* de */ {IDGS_MON_COMMAND_DESCRIPTION_DE, "Definiere `Kommando', welches bei Checkpoint `Checknum' ausgef¸hrt wird.\nAchtung Kommando `x' wird zurzeit noch nicht als Kommando Argument\nunterst¸tzt."},
	/* es */ {IDGS_MON_COMMAND_DESCRIPTION_ES, "Especificar 'comando' como el comando a ejecutar cuando el punto de control\n'checknum' est· activo. notar que el comando 'x' no est· soportado como\nargumento de comando."},
	/* fr */ {IDGS_MON_COMMAND_DESCRIPTION_FR, "SpÈcifier `command' comme commande ‡ exÈcuter quand le point d'arrÍt\n`checknum' est atteint. Noter que la commande `x' n'est pas encore\nsupportÈe comme argument de commande."},
	/* hu */ {IDGS_MON_COMMAND_DESCRIPTION_HU, "`parancs' megad·sa egy `ell_pont_sz·m' sorsz·m·val adott ellenırzÈsi pont\nelÈrÈsekor tˆrtÈnı vÈgrehajt·sra.  Az `x' parancs egyelıre nem haszn·lhatÛ\nitt paramÈterkÈnt."},
	/* it */ {IDGS_MON_COMMAND_DESCRIPTION_IT, "Specifica `comando' come il comando da eseguire quando\nviene incontrato il punto di controllo `checknum'. Nota che \nil comando `x' non Ë ancora supportato come parametro."},
	/* ko */ {IDGS_MON_COMMAND_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_COMMAND_DESCRIPTION_NL, "Geef `commando' als het commando om uit te voeren wanneer checkpunt `checknum'\nwordt uitgevoerd. NB: het `x' commando is op dit moment geen geldige\nparameter."},
	/* pl */ {IDGS_MON_COMMAND_DESCRIPTION_PL, "Okre∂l 'command' jako polecenie do wykonania przy osi±gniÍciu punktu 'checknum.'\nPolecenie 'x' nie jest jeszcze obs≥ugiwane jako argument polecenia."},
	/* ru */ {IDGS_MON_COMMAND_DESCRIPTION_RU, "Specify `command' as the command to execute when checkpoint `checknum'\nis hit.  Note that the `x' command is not yet supported as a\ncommand argument."},
	/* sv */ {IDGS_MON_COMMAND_DESCRIPTION_SV, "Ange \"kommando\" som kommando att exekvera n‰r vid tr‰ff pÂ\nkontrollpunkten∑\"kontrollnr\". Observera att kommandot \"x\" ‰nnu\ninte stˆds som kommandoparameter."},
	/* tr */ {IDGS_MON_COMMAND_DESCRIPTION_TR, "Kontrol noktas˝ `kontrolnumaras˝'na ula˛˝ld˝˝nda `komut' olarak\nÁal˝˛t˝r˝lacak komutu belirt.  Hen¸z `x' komutunun bir komut arg¸man˝\nolarak desteklenmediini not edin."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_CONDITION_DESCRIPTION,    N_("Each time the specified checkpoint is examined, the condition is\nevaluated. If it evalutes to true, the checkpoint is activated.\nOtherwise, it is ignored. If registers are specified in the expression,\nthe values used are those at the time the checkpoint is examined, not\nwhen the condition is set.\nThe condition can use registers (A, X, Y, PC, SP, FL and other cpu\nspecific registers (see manual)) and compare them (==, !=, <, >, <=, >=)\nagainst other registers or constants.\nRegisters can be the registers of other devices; this is denoted by\na memspace prefix (i.e., c:, 8:, 9:, 10:, 11:\nExamples: A == 0, X == Y, 8:X == X)\n")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_CONDITION_DESCRIPTION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MON_CONDITION_DESCRIPTION_DE, "Bei jeder ‹berpr¸fung des Checkpoints, wird die Bedingung evaluiert.\nIst das Ergebnis `Wahr', wird der Checkpoint aktiviert, sonst ingnoriert.\nWenn Register benutzt werden, werden deren Werte zum Zeitpunkt der \n‹berpr¸fung herangezogen, nicht zum Zeitpunkt des Setzens der Bedingung.\nDie Bedingung kann Register benutzen (A, X, Y, PC, SP, FL und andere\nCPU spezifische Register (siehe Handbuch)) und gegen andere Register oder\nKonstanten vergleichen (==, !=, <, >, <=, >=).\nRegister kˆnnen auch Ger‰teregister sein; dazu wird die memspace Prefix-syntax \nverwendet: c:, 8:, 9:, 10:, 11:\nBeispiel: A == 0, X == Y, 8:X == X\n"},
	/* es */ {IDGS_MON_CONDITION_DESCRIPTION_ES, "Cada vez que un punto de control se examine, la condiciÛn se\nevaluar·. Si la condiciÛn es cierta, se activa el punto de control.\nSi no, se ignora. Si los registros est·n especificado en la condiciÛn,\nse usan los valores en el momento en el que se examina el punto\nde ruptura, no en el momento de poner la condiciÛn.\nLa condiciÛn puede usar los registros (.A, .X, .Y, .PC, .SP)  y compararlos\nm (==, !=, <, >, <=, >=) con otros registros y constantes.\nLos registros pueden ser de otros perifÈricos, esto se denota por el\nprefijo de espacio memoria (ej., c:, 8:, 9:, 10:, 11:\nEjemplos: .A == 0, .X == .Y, 8:.X == .X\n"},
	/* fr */ {IDGS_MON_CONDITION_DESCRIPTION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MON_CONDITION_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_CONDITION_DESCRIPTION_IT, "Ogni volta che si esamina il punto di controllo specificato, viene valutata\nla condizione. Se Ë vera, il punto di controllo Ë attivato.\nAltrimenti viene ignorato. Se nell'espressione sono specificati i registri,\nquesti valori sono quelli usati quando viene esaminato il punto di controllo,\nnon quando viene impostata la condizione.\nLa condizione puÚ fare uso dei registri (A, X, Y, PC, SP, FL e altri registri\nspecifici della cpu (vedi manuale)) e confrontarli (==, !=, <, >, <=, >=)\ncon altri registri o costanti.\nI registri possono essere registri di altre periferiche; questo Ë denotato da\nun prefisso che specifica lo spazio di memoria (cioÈ, c:, 8:, 9:, 10:, 11:)\nAlcuni esempi: A == 0, X == Y, 8:X == X)\n"},
	/* ko */ {IDGS_MON_CONDITION_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_CONDITION_DESCRIPTION_NL, "Elke keer dat het opgegeven checkpunt wordt onderzocht zal de conditie worden\ngeÎvalueerd. Als de evaluatie positief is dan zal de checkpunt geactiveerd\nworden, bij negatief zal er niets worden gedaan. Als er registers zijn\nopgegeven in de expressie dan zullen de waarden in die registers gebruikt\nworden op het moment dat de checkpunt wordt onderzocht, niet op het moment\ndat de conditie is opgegeven.\nDe conditie kan gebruik maken van registers (A, X, Y, PC, SP, FL en andere\nspecifieke registers (zie handleiding) en kan ze vergelijken\n (==, !=, <, >, <=, >=) met andere registers of constanten\nRegisters kunnen de registers van andere apparaten zijn; de manier van opgeven\nis door een geheugenruimte prefix (bijvoorbeeld: c:, 8:, 9:, 10:, 11:\nVoorbeelden: A == 0, X == Y, 8:X == X\n"},
	/* pl */ {IDGS_MON_CONDITION_DESCRIPTION_PL, "Podczas analizy punktu kontrolnego okre∂lany jest stan.\nJe∂li jest okre∂lony jako 'prawda', punkt kontrolny jest\nuaktywniany. W innym razie jest ignorowany. Je∂li w wyraøeniu\npodano rejestry, uøywane s± warto∂ci z czasu szacowania punktu\nkontrolnego, nie z czasu ustawienia stanu.\nStan moøe korzystaÊ z rejestrÛw A, X, Y, PC, SP, FL, jak rÛwnieø\ninnych, konkretnych dla danego procesora (patrz w instrukcji) i porÛwnywaÊ je\n(==, !=, <, >, <=, >=) z innymi rejestrami lub sta≥ymi. Rejestry mog± byÊ\nrejestrami innych urz±dzeÒ, co jest okre∂lone przedrostkiem przestrzeni\npamiÍci (np. c:, 8:, 9:, 10:, 11:\nPrzyk≥ady: A == 0, X == Y, 8:X == X)\n"},
	/* ru */ {IDGS_MON_CONDITION_DESCRIPTION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MON_CONDITION_DESCRIPTION_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MON_CONDITION_DESCRIPTION_TR, ""},  /* fuzzy */
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DISABLE_DESCRIPTION,    N_("Disable checkpoint `checknum'. If no checkpoint is given all checkpoints will be disabled.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DISABLE_DESCRIPTION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MON_DISABLE_DESCRIPTION_DE, "Deaktiviere Checkpoint `Checknummer'. Falls kein Checkpoint angegeben wird,\nwerden alle Checkpoints deaktiviert."},
	/* es */ {IDGS_MON_DISABLE_DESCRIPTION_ES, "Deshabilitar Punto de control `checknum'. Si no se especifica ninguno, se deshabilitan todos."},
	/* fr */ {IDGS_MON_DISABLE_DESCRIPTION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MON_DISABLE_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_DISABLE_DESCRIPTION_IT, "Disattiva il punto di controllo `checknum'. Se non viene specificato alcun punto di controllo saranno disattivati tutti i punti di controllo."},
	/* ko */ {IDGS_MON_DISABLE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DISABLE_DESCRIPTION_NL, "Checkpunt 'checknum' uitschakelen. Als er geen checkpunt is gegeven, worden alle checkpunten uitgeschakeld."},
	/* pl */ {IDGS_MON_DISABLE_DESCRIPTION_PL, "Wy≥±cz punkt kontrolny 'checknum'. Je∂li nie okre∂lono punktu kontrolnego, skasuj wszystkie."},
	/* ru */ {IDGS_MON_DISABLE_DESCRIPTION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MON_DISABLE_DESCRIPTION_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MON_DISABLE_DESCRIPTION_TR, ""},  /* fuzzy */
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_ENABLE_DESCRIPTION,    N_("Enable checkpoint `checknum'. If no checkpoint is given all checkpoints will be enabled.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_ENABLE_DESCRIPTION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MON_ENABLE_DESCRIPTION_DE, "Aktiviere Checkpoint `Checknummer'. Falls kein Checkpoint angegeben wird,\nwerden alle Checkpoints aktiviert."},
	/* es */ {IDGS_MON_ENABLE_DESCRIPTION_ES, "Permitir Punto de control `checknum'. Si no se especifica ninguno, se habilitan todos."},
	/* fr */ {IDGS_MON_ENABLE_DESCRIPTION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MON_ENABLE_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_ENABLE_DESCRIPTION_IT, "Attiva il punto di controllo `checknum'. Se non viene specificato alcun punto di controllo saranno attivati tutti i punti di controllo."},
	/* ko */ {IDGS_MON_ENABLE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_ENABLE_DESCRIPTION_NL, "Checkpunt 'checknum' inschakelen. Als er geen checkpunt is gegeven, worden alle checkpunten ingeschakeld."},
	/* pl */ {IDGS_MON_ENABLE_DESCRIPTION_PL, "W≥±cz punkt kontrolny 'checknum'. Je∂li nie okre∂lono punktu kontrolnego, skasuj wszystkie."},
	/* ru */ {IDGS_MON_ENABLE_DESCRIPTION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MON_ENABLE_DESCRIPTION_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MON_ENABLE_DESCRIPTION_TR, ""},  /* fuzzy */
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_IGNORE_DESCRIPTION,    N_("Ignore a checkpoint a given number of crossings.  If no count is given,\nthe default value is 1.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_IGNORE_DESCRIPTION_DA, "IgnorÈr et checkpoint det angivne antal gange. Hvis antal ikke angives\nbruges standardvÊrdien 1."},
	/* de */ {IDGS_MON_IGNORE_DESCRIPTION_DE, "Ignoriere den Checkpoint f¸r die angegebene Anzahl an Aktivierungen.\nWenn kein `Z‰hler' angegeben wurde, wird der Standardwert 1 verwendet."},
	/* es */ {IDGS_MON_IGNORE_DESCRIPTION_ES, "Ignorar un punto control un numero determinado de veces. Si no se da contador,\nel valor por defecto es 1."},
	/* fr */ {IDGS_MON_IGNORE_DESCRIPTION_FR, "Ignorer un point d'arrÍt un certain nombre de passages. Si aucun compte\nn'est donnÈe, la valeur par dÈfaut de 1 sera utilisÈe."},
	/* hu */ {IDGS_MON_IGNORE_DESCRIPTION_HU, "Az ellenırzÈsi pont figyelmen kÌv¸l hagy·sa megadott sz·m˙ ·thalad·skor.  Ha nincs\nsz·m megadva, az alapÈrtelmezett ÈrtÈk 1."},
	/* it */ {IDGS_MON_IGNORE_DESCRIPTION_IT, "Ignora un punto di controllo un numero specificato di occorrenze. Se\nnon Ë specificato alcun valore, quello predefinito Ë 1."},
	/* ko */ {IDGS_MON_IGNORE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_IGNORE_DESCRIPTION_NL, "Negeer een checkpunt een opgegeven aantal keer. Als er geen aantal is\nopgegeven, wordt de standaardwaarde van 1 gebruikt."},
	/* pl */ {IDGS_MON_IGNORE_DESCRIPTION_PL, "Ignoruje punkt kontrolny dan± ilo∂Ê razy. Je∂li nie podano tej ilo∂ci,\ndomy∂lna warto∂Ê wynosi 1."},
	/* ru */ {IDGS_MON_IGNORE_DESCRIPTION_RU, "Ignore a checkpoint a given number of crossings.  If no count is given,\nthe default value is 1."},
	/* sv */ {IDGS_MON_IGNORE_DESCRIPTION_SV, "Ignorera en kontrollpunkt det givna antalet gÂnger. Om inget antal ges\n‰r standardv‰rdet 1."},
	/* tr */ {IDGS_MON_IGNORE_DESCRIPTION_TR, "Bir kontrol noktas˝n˝ verilen adette geÁi˛ iÁin yoksay.  Eer adet verilmemi˛se,\nvarsay˝lan deer 1'dir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_UNTIL_DESCRIPTION,    N_("If no address is given, the currently valid breakpoints are printed.\nIf an address is given, a temporary breakpoint is set for that address\nand the breakpoint number is printed.  Control is returned to the\nemulator by this command.  The breakpoint is deleted once it is hit.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_UNTIL_DESCRIPTION_DA, "Hvis ingen adresse angives vises de aktuelle breakpoints. Hvis en\nadresse angives sÊttes et midlertidigt breakpoint, og nummeret pÂ\nbreakpointet vises. Kontrollen returneres til emulatoren af denne\nkommando. Breakpointet fjernes nÂr det nÂs."},
	/* de */ {IDGS_MON_UNTIL_DESCRIPTION_DE, "Wenn keine Adresse angegeben wurde, werden die aktuell g¸ltigen \nBreakpoints angezeigt. Wenn eine Adresse angegeben wurde, wird ein \ntempor‰rer Breakpoint auf dieser Adresse gesetzt und die Breakpoint Nummer wird\nangezeigt. Die Kontrolle wirdd bei diesem Kommando an die Emulation gegeben.\nDer Breakpoint wird gelˆscht, wenn er einmal getroffen wurde."},
	/* es */ {IDGS_MON_UNTIL_DESCRIPTION_ES, "Si no se da direcciÛn, se imprime el ˙ltimo punto de ruptura v·lido.\nSi se da direcciÛn, se activa un punto de ruptura para la direcciÛn\ny se imprime el n˙mero del punto de ruptura. El control se devuelve al\nemulador por el comando. El punto de ruptura se borra una vez se encuentra."},
	/* fr */ {IDGS_MON_UNTIL_DESCRIPTION_FR, "Si aucune adresse n'est donnÈe, les points d'arrÍts prÈsentement valides\nseront affichÈs. Si aucune adresse n'est donnÈe, un point d'arrÍt\ntemporaire est crÈÈ pour cette adresse et le numÈro de point d'arrÍt\nest affichÈ. Le contrÙle retourne ‡ l'Èmulateur suite ‡ cette commande.\nLe point d'arrÍt est supprimÈ une fois atteint."},
	/* hu */ {IDGS_MON_UNTIL_DESCRIPTION_HU, "Ha nincs cÌm megadva, a jelenleg ÈrvÈnyes tˆrÈspontokat Ìrja ki.\nHa van adott cÌm, egy ideiglenes tˆrÈspontot helyez el az adott cÌmre,\nÈs a tˆrÈspont sz·ma kiÌrÛdik.  A vezÈrlÈs visszaker¸l az emul·torhoz\ne parancs ·ltal. A tˆrÈspont elÈrÈsekor tˆrlıdik."},
	/* it */ {IDGS_MON_UNTIL_DESCRIPTION_IT, "Se non viene specificato alcun indirizzo, vengono stampati i\nbreakpoint attualmente validi. Se viene specificato un indirizzo,\nviene impostato un breakpoint temporaneo per tale indirizzo e\nviene stampato il suo numero. Il controllo ritorna all'emulatore\nper mezzo di questo comando. Il breakpoint Ë cancellato quando\nviene incontrato."},
	/* ko */ {IDGS_MON_UNTIL_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_UNTIL_DESCRIPTION_NL, "Als er geen adres is opgegeven, zullen de huidig geldige breekpunten\nworden geprint. Als er een adres is opgegeven, wordt er een tijdelijk\nbreekpunt gezet voor dat adres en het breekpunt nummer zal worden\ngetoond. Na het geven van dit commando zal er teruggekeerd worden naar de\nemulator. Het breekpunt zal worden verwijderd na het uitvoeren."},
	/* pl */ {IDGS_MON_UNTIL_DESCRIPTION_PL, "Je∂li nie podano adresu, pokazywane s± obecne punkty wstrzymania.\nJe∂li podano adres, ustawiany jest tymczasowy punkt wstrzymania\ni pokazywany jest numer punktu wstrzymania. To polecenie przywraca\nkontrolÍ do emulatora. Punkt wstrzymania jest kasowany po jego osi±gniÍciu."},
	/* ru */ {IDGS_MON_UNTIL_DESCRIPTION_RU, "If no address is given, the currently valid breakpoints are printed.\nIf an address is given, a temporary breakpoint is set for that address\nand the breakpoint number is printed.  Control is returned to the\nemulator by this command.  The breakpoint is deleted once it is hit."},
	/* sv */ {IDGS_MON_UNTIL_DESCRIPTION_SV, "Om du inte anger nÂgon adress visas de fˆr n‰rvarande giltiga\nbrytpunkterna. Om en adress anges s‰tts en tempor‰r brytpunkt fˆr\nden adressen och numret pÂ den nya brytpunkten visas. Styrningen\nÂterl‰mnas till emulatorn fˆr det h‰r kommandot. Brytpunkten\ntas bort n‰r den har nÂtts."},
	/* tr */ {IDGS_MON_UNTIL_DESCRIPTION_TR, "Eer adres girilmemi˛se, geÁerli mevcut kesme noktalar˝ yazd˝r˝l˝r.\nEer bir adres girilmi˛se, o adres iÁin geÁici bir kesme noktas˝\ni˛aretlenir ve kesme noktas˝ say˝s˝ yazd˝r˝l˝r.  Kontrol bu komutla\nem¸latˆre dˆnd¸r¸l¸r.  Bir kere eri˛ildii zaman kesme noktas˝ silinir."},
#endif
	
	/*
	 Translators: this command was extended:
	 
	 - "loadstore" option is now "load|store|exec"
	 - conditional expression was added
	 
	 TODO: update these languages: DA, DE, HU, IT, NL, SV, TR
	 */
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_WATCH_DESCRIPTION,    N_("Set a watchpoint. If no address is given, the currently valid watch-\npoints are printed. If a single address is specified, set a watchpoint\nfor that address.  If two addresses are specified, set a watchpoint\nfor the memory locations between the two addresses.\n`load|store|exec' is either `load', `store' or `exec' (or any combina-\ntion of these) to specify on which operation the monitor breaks. If\nnot specified, the monitor breaks on `load' and `store'.\n")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_WATCH_DESCRIPTION_DA, "SÊt et watchpoint. Hvis der ikke angives en adresse vises en liste\nover nuvÊrende gyldige watchpoints. Hvis der angives en enkelt\nadresse sÊttes et watchpoint pÂ adressen. Hvis der angives to adresser\nsÊttes et watchpoint for hukommelsesomrÂdet mellem adresserne.\n`load|store|exec' er enten `load', `store' eller `exec' (eller en\nvilkÂrlig kombination), og angiver pÂ hvilken operation monitoren\nafbryder. Hvis intet angives vil monitoren afbryde pÂ `load' og\n`store'.\n"},
	/* de */ {IDGS_MON_WATCH_DESCRIPTION_DE, "Setze einen Watchpoint. Wenn eine einzelne Adresse angegeben wurde, setze \nden Watchpoint f¸r diese Adresse. Wenn zwei Adressen angegeben wurden, \nsetze den Watchpoint auf die Adressen zwischen den beiden Adressen.\n`load|store|exec' ist entweder `load', `store' oder `exec' (oder eine\nKombination) und dient zur Angabe, bei welcher Operation der Monitor\nanhalten soll. Wenn keine Angabe gemacht wird, stoppt der Monitor bei\n`exec'.\nWenn keine Angabe gemacht wurde, h‰lt der Monitor bei bei `load' und\n`store' Operationen an.\n"},
	/* es */ {IDGS_MON_WATCH_DESCRIPTION_ES, "Poner punto de visualizaciÛn. Si no se da direcciÛn se mostrar· el\nactualmente v·lido. Si se da una direcciÛn simple, se pondr· un punto\nde visualizaciÛn en la misma\nSi se dan dos direcciones se activar· un punto de visualizaciÛn para las\nlocalizaciones de memoria entre ambas direcciones.\n`load|store|exec' es cualquiera de `load', `store' or `exec' (o alguna com-\nbinaciÛn de ellos) para especificar en que operaciÛn el monitor se detiene\nSi no se especifica, Se detiene en  `load' y `store'.\n"},
	/* fr */ {IDGS_MON_WATCH_DESCRIPTION_FR, "CrÈe un point de surveillance. Si aucune adresse n'est spÈcifiÈe, les\npoints de surveillance valides sont affichÈs. Si une adresse est\nspÈcifiÈe, crÈe un point de surveillance ‡ cette adresse. Si deux\nadresses sont spÈcifiÈs, crÈe un point de surveillance pour les\nadresses mÈmoire situÈes entre les deux adresses.\n`load|store|exec' est soit `load', `store' ou `exec' (or toute\ncombinaison) pour spÈcifier ‡ quelle opÈration le moniteur arrÍte. Si\naucune n'est spÈcifiÈ, le moniteur arrÍte ‡ `load' et `store'.\n"},
	/* hu */ {IDGS_MON_WATCH_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_WATCH_DESCRIPTION_IT, "Imposta un watchpoint. Se non viene specificato alcun indirizzo,\nvengono stampati i watchpoint attuali. Se viene specificato solo un\nsolo indirizzo, imposta un watchpoint per tale indirizzo. Se vengono\nspecificati due indirizzi, imposta un watchpoint per le locazioni di \nmemoria comprese tra i due indirizzi.\n`load|store|exec' Ë uno tra `load', `store' o `exec' (or una qualunque\ncombinazione di questi) per specificare su quale operazione il monitor\ninterromper‡. Se non specificato, il monitor interrompe su `load' e\n`store'.\n"},
	/* ko */ {IDGS_MON_WATCH_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_WATCH_DESCRIPTION_NL, "Zet een kijkpunt. Als er geen adres is opgegeven, zullen de huidige\ngeldige kijkpunten worden getoond. Als er een enkel adres is opgegeven,\nzal er een kijkpunt worden gezet voor dat adres. Als er twee\nadressen zijn opgegeven, zal er een kijkpunt worden gezet voor de\nadressen tussen de twee adressen.\n`load|store|exec' is `load',  `store' of `exec' om aan te geven bij\nwelke operatie de monitor zou moeten starten. Als deze niet opgegeven\nis, zal de monitor starten bij `load' en `store' operaties.\n"},
	/* pl */ {IDGS_MON_WATCH_DESCRIPTION_PL, "Ustawia punkt ∂ledzenia. Je∂li nie podano adresu, pokazywane s±\nobecnie aktualne punkty ∂ledzenia. Je∂li okre∂lono pojedynczy adres,\nustawia punkt ∂ledzenia na ten adres. JÍsli okre∂lono dwa adresy,\nustawia punkt ∂ledzenia dla obszaru pamiÍci pomiÍdzy tymi dwoma\nadresami.\n`load|store|exec' to `load', `store' lub `exec' (albo ich po≥±czenia),\nokre∂laj±ce  podczas ktÛrych operacji nastÍpuje b≥±d. Je∂li nic\nnie okre∂lono, monitor przerywa dzia≥anie na `load' i `store'.\n"},
	/* ru */ {IDGS_MON_WATCH_DESCRIPTION_RU, "Set a watchpoint. If no address is given, the currently valid watch-\npoints are printed. If a single address is specified, set a watchpoint\nfor that address.  If two addresses are specified, set a watchpoint\nfor the memory locations between the two addresses.\n`load|store|exec' is either `load', `store' or `exec' (or any combina-\ntion of these) to specify on which operation the monitor breaks. If\nnot specified, the monitor breaks on `load' and `store'.\n"},
	/* sv */ {IDGS_MON_WATCH_DESCRIPTION_SV, "S‰tt en ˆvervakningspunkt. Om du inte anger nÂgon adress, visas\nnu giltiga ˆvervakningspunkter. Om du anger en ensam adress s‰tts en\nˆvervakningspunkt fˆr adressen. Om tvÂ adresser anges s‰tts en\nˆvervakningspunkt fˆr minnesomrÂdet mellan adresserna.\n\"load|store|exec\" ‰r antingen \"load\". \"store\" eller \"exec\" (eller en\nkombination d‰rutav) och anger vilken operation monitorn skall\nstoppa fˆr. Om den inte anges stoppar emulatorn pÂ bÂde \"load\" och\n\"store\".\n"},
	/* tr */ {IDGS_MON_WATCH_DESCRIPTION_TR, "Bir gˆr¸nt¸leme noktas˝ koy. Eer hiÁ adres verilmemi˛se, mevcut geÁerli\ngˆr¸nt¸leme noktalar˝ yazd˝r˝l˝r. Eer tek bir adres belirtilmi˛se,\ngˆr¸nt¸leme noktas˝n˝ o adrese koy.  Eer iki adres belirtilmi˛se, gˆr¸nt¸leme\nnoktas˝n˝ o iki adres aras˝nda kalan adreslere koy.\n`y¸kle|depola|y¸r¸t' monitˆr¸n kesildii operatˆre bal˝ olarak `y¸kle'\n`depola' ya da `y¸r¸t' (ya da herhangi bir kombinasyonunda) i˛lemlerinde\nÁal˝˛abilir. Eer belirtilmemi˛se, monitˆr `y¸kle' ve `depola'da kesilecektir.\n"},
#endif
	
	/*
	 Translators: this command was extended:
	 
	 - "load|store|exec" option added
	 - conditional expression was added
	 
	 TODO: update these languages: DA, DE, HU, IT, NL, SV, TR
	 */
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_TRACE_DESCRIPTION,    N_("Set a tracepoint. If no address is given, the currently valid trace-\npoints are printed. If a single address is specified, set a tracepoint\nfor that address.  If two addresses are specified, set a tracepoint\nfor the memory locations between the two addresses.\n`load|store|exec' is either `load', `store' or `exec' (or any combina-\ntion of these) to specify on which operation the monitor breaks. If\nnot specified, the monitor traces all three operations.\n")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_TRACE_DESCRIPTION_DA, "SÊt et tracepoint. Hvis der ikke angives en adresse vises en liste\nover nuvÊrende gyldige tracepoints. Hvis der angives en enkelt\nadresse sÊttes et tracepoint pÂ adressen. Hvis der angives to adresser\nsÊttes et tracepoint for hukommelsesomrÂdet mellem adresserne.\n`load|store|exec' er enten `load', `store' eller `exec' (eller en\nvilkÂrlig kombination), og angiver pÂ hvilken operation monitoren\ntracer. Hvis intet angives vil monitoren trace alle 3 operationer.\n"},
	/* de */ {IDGS_MON_TRACE_DESCRIPTION_DE, "Setze einen Tracepoint. Wenn keine Adresse angegeben wird, werden die\naktuellen g¸ltigen Tracepoints ausgegeben. Wenn eine einzelne Adresse\nangegeben wurde, setze den Watchpoint f¸r diese Adresse. Wenn zwei\nAdressen angegeben wurden, setze den Watchpoint auf die Adressen\nzwischen den beiden Adressen.\n`load|store|exec' ist entweder `load', `store' oder `exec' (oder eine\nKombination) und dient zur Angabe, bei welcher Operation der Monitor\nanhalten soll. Wenn keine Angabe gemacht wird, beobachtet der Monitor\nalle Operationen.\n"},
	/* es */ {IDGS_MON_TRACE_DESCRIPTION_ES, "Poner punto rastreo. Si no se da direcciÛn, Se mostrar· el actualmente\nv·lido. Si se da una sola direcciÛn, se activa un punto para esa direcciÛn.\nSi se dan dos, se activa el punto para las localizaciones de memoria entre\nlas dos direcciones.\n`load|store|exec' es cualquiera `load', `store' or `exec' (o una combina-\nciÛn de ellas) para indicar en cual de ellas se detiene el monitor. Si no se\nespecifican el monitor rastrea todas las tres operaciones.\n"},
	/* fr */ {IDGS_MON_TRACE_DESCRIPTION_FR, "CrÈe un point de trace. Si aucune adresse n'est spÈcifiÈe, les\npoints de trace valides sont affichÈs. Si une adresse est spÈcifiÈe,\ncrÈe un point de trace ‡ cette adresse. Si deux adresses sont\nspÈcifiÈs, crÈe un point de trace pour les adresses mÈmoire situÈes\nentre les deux adresses.\n`load|store|exec' est soit `load', `store' ou `exec' (or toute\ncombinaison) pour spÈcifier ‡ quelle opÈration le moniteur arrÍte. Si\naucune n'est spÈcifiÈ, le moniteur trace toutes les opÈrations.\n"},
	/* hu */ {IDGS_MON_TRACE_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_TRACE_DESCRIPTION_IT, "Imposta un tracepoint. Se non viene specificato alcun indirizzo,\nvengono stampati i tracepoint attuali. Se viene specificato solo un\nsolo indirizzo, imposta un tracepoint per tale indirizzo. Se vengono\nspecificati due indirizzi, imposta un tracepoint per le locazioni di \nmemoria comprese tra i due indirizzi.\n`load|store|exec' Ë uno tra `load', `store' o `exec' (or una qualunque\ncombinazione di questi) per specificare su quale operazione il monitor\ninterromper‡. Se non specificato, il monitor traccer‡ tutte e tre le\noperazioni.\n"},
	/* ko */ {IDGS_MON_TRACE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_TRACE_DESCRIPTION_NL, "Zet een traceerpunt. Als er geen adres is opgegeven, zullen de huidige\ngeldige traceerpunten worden getoond. Als er een enkel adres is opgegeven,\nzal er een traceerpunt worden gezet voor dat adres. Als er twee\nadressen zijn opgegeven, zal er een traceerpunt worden gezet voor de\nadressen tussen de twee adressen.\n`load|store|exec' is `load', `store' of 'exec' om aan te geven bij welke\noperatie de monitor zou moeten starten. Als deze niet opgegeven is, zal\nde monitor starten bij alle operaties.\n"},
	/* pl */ {IDGS_MON_TRACE_DESCRIPTION_PL, "Ustawia punkt ∂ledzenia. Je∂li nie podano adresu, pokazywane s±\nw≥a∂ciwe obecnie punkty ∂ledzenia. Je∂li podano jeden adres,\nustawia punkt ∂ledzenia tego adresu. Je∂li podano dwa adresy,\nustawia punkt ∂ledzenia obszaru pamiÍci pomiÍdzy nimi.\n`load|store|exec' to `load', `store' lub `exec' (albo ich po≥±czenia),\nokre∂laj±ce  podczas ktÛrych operacji nastÍpuje b≥±d. Je∂li nic\nnie okre∂lono, monitor ∂ledzi wszystkie trzy rodzaje operacji.\n"},
	/* ru */ {IDGS_MON_TRACE_DESCRIPTION_RU, "Set a tracepoint. If no address is given, the currently valid trace-\npoints are printed. If a single address is specified, set a tracepoint\nfor that address.  If two addresses are specified, set a tracepoint\nfor the memory locations between the two addresses.\n`load|store|exec' is either `load', `store' or `exec' (or any combina-\ntion of these) to specify on which operation the monitor breaks. If\nnot specified, the monitor traces all three operations.\n"},
	/* sv */ {IDGS_MON_TRACE_DESCRIPTION_SV, "S‰tt en spÂrningspunkt. Om ingen adress anges visas befintliga\nspÂrningspunkter. Om du anger en ensam adress s‰tts en spÂrningspunkt\nfˆr adressen. Om tvÂ adresser anges s‰tts en spÂrningspunkt fˆr\nminnesomrÂdet mellan adresserna.\n\"load|store|exec\" ‰r antingen \"load\", \"store\" eller \"exec\" (eller en\nkombination d‰rutav) och anger vilken operation monitorn skall stoppa\nfˆr. Om den inte anges spÂrar monitorn alla tre operationerna.\n"},
	/* tr */ {IDGS_MON_TRACE_DESCRIPTION_TR, "Bir izleme noktas˝ koy. Eer bir adres belirtilmemi˛se, mevcut geÁerli\nizleme noktalar˝ yazd˝r˝l˝r. Eer tek bir adres belirtilmi˛se, izleme\nnoktas˝n˝ o adrese koy.  Eer iki adres belirtilmi˛se, izleme noktas˝n˝\no iki adres aras˝nda kalan adreslere koy.\n`y¸kle|depola|y¸r¸t' monitˆr¸n kesildii operatˆre bal˝ olarak `y¸kle'\n`depola' ya da `y¸r¸t' (ya da herhangi bir kombinasyonunda) i˛lemlerinde\nÁal˝˛abilir. Eer belirtilmemi˛se, monitˆr her ¸Á i˛lemi de izleyecektir.\n"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MONITOR_STATE_COMMANDS,    N_("Monitor state commands:")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MONITOR_STATE_COMMANDS_DA, "Monitorstatuskommandoer:"},
	/* de */ {IDGS_MONITOR_STATE_COMMANDS_DE, "Monitor Zustand Kommandos:"},
	/* es */ {IDGS_MONITOR_STATE_COMMANDS_ES, "Comandos de estado del Monitor:"},
	/* fr */ {IDGS_MONITOR_STATE_COMMANDS_FR, "Commandes du statut du moniteur:"},
	/* hu */ {IDGS_MONITOR_STATE_COMMANDS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MONITOR_STATE_COMMANDS_IT, "Comandi stato monitor:"},
	/* ko */ {IDGS_MONITOR_STATE_COMMANDS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MONITOR_STATE_COMMANDS_NL, "Monitor status commando's:"},
	/* pl */ {IDGS_MONITOR_STATE_COMMANDS_PL, "Monitoruj polecenia stanu:"},
	/* ru */ {IDGS_MONITOR_STATE_COMMANDS_RU, "Monitor state commands:"},
	/* sv */ {IDGS_MONITOR_STATE_COMMANDS_SV, "Kommandon fˆr monitorstatus:"},
	/* tr */ {IDGS_MONITOR_STATE_COMMANDS_TR, "Monitˆr durum komutlar˝:"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DEVICE_DESCRIPTION,    N_("Set the default memory device to either the computer `c:' or the\nspecified disk drive (`8:', `9:').")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DEVICE_DESCRIPTION_DA, "SÊt det valgte hukommelses omrÂde til enten computeren \"c:\" eller\ndet angivne diskettedrev (\"8:\", \"9:\")."},
	/* de */ {IDGS_MON_DEVICE_DESCRIPTION_DE, "Setze das aktuelle Speicherger‰t entweder auf `c:' \noder das angegebene Laufwerk (`8:', `9:')."},
	/* es */ {IDGS_MON_DEVICE_DESCRIPTION_ES, "Seleccionar el perifÈrico memoria por defecto entre  `c:' o la\nunidad de disco especificada (`8:', `9:')."},
	/* fr */ {IDGS_MON_DEVICE_DESCRIPTION_FR, "RËgle le pÈriphÈrique mÈmoire par dÈfaut ‡ soit l'ordinateur `c:'\nou au lecteur de disque spÈcifiÈ (`8:', `9:')."},
	/* hu */ {IDGS_MON_DEVICE_DESCRIPTION_HU, "Az alapÈrtelmezett memÛria eszkˆz be·llÌt·sa a sz·mÌtÛgÈpre `c:' vagy\na megadott lemezmeghajtÛra (`8:', `9:')."},
	/* it */ {IDGS_MON_DEVICE_DESCRIPTION_IT, "Imposta la periferica di memoria predefinita o sul computer `c:'\noppure sul disk drive specificato (`8:', `9:')."},
	/* ko */ {IDGS_MON_DEVICE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DEVICE_DESCRIPTION_NL, "Zet het standaard geheugen apparaat naar Úf de computer `c:' Úf de\nopgegeven disk drive (`8:', `9:')."},
	/* pl */ {IDGS_MON_DEVICE_DESCRIPTION_PL, "Ustawia komputer `c:', lub okre∂lon± stacjÍ dyskÛw (`8:', `9:')\njako domy∂lne urz±dzenie pamiÍci."},
	/* ru */ {IDGS_MON_DEVICE_DESCRIPTION_RU, "Set the default memory device to either the computer `c:' or the\nspecified disk drive (`8:', `9:')."},
	/* sv */ {IDGS_MON_DEVICE_DESCRIPTION_SV, "S‰tt fˆrvald minnesenhet antingen till datorn \"c:\" eller\nden angivna diskettenheten (\"8:\", \"9:\")."},
	/* tr */ {IDGS_MON_DEVICE_DESCRIPTION_TR, "Varsay˝lan bellek ayg˝t˝n˝ bilgisayar `c:' ya da (`8:', `9:') ile\nbelirtilmi˛ disket d¸r¸c¸ olarak belirle."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_EXIT_DESCRIPTION,    N_("Leave the monitor and return to execution.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_EXIT_DESCRIPTION_DA, "Forlad monitoren og gÂ tilbage til eksekvering."},
	/* de */ {IDGS_MON_EXIT_DESCRIPTION_DE, "Verlasse Monitor und setze Ausf¸hrung fort."},
	/* es */ {IDGS_MON_EXIT_DESCRIPTION_ES, "Dejar el monitor y volver a la ejecuciÛn."},
	/* fr */ {IDGS_MON_EXIT_DESCRIPTION_FR, "Quitte le moniteur et retourne ‡ l'exÈcution."},
	/* hu */ {IDGS_MON_EXIT_DESCRIPTION_HU, "A monitor elhagy·sa Ès a vÈgrehajt·s folytat·sa."},
	/* it */ {IDGS_MON_EXIT_DESCRIPTION_IT, "Esce dal monitor e ritorna all'esecuzione."},
	/* ko */ {IDGS_MON_EXIT_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_EXIT_DESCRIPTION_NL, "Verlaat de monitor en ga terug naar de uitvoering."},
	/* pl */ {IDGS_MON_EXIT_DESCRIPTION_PL, "Opu∂Ê monitor i wrÛÊ do wykonywania"},
	/* ru */ {IDGS_MON_EXIT_DESCRIPTION_RU, "Leave the monitor and return to execution."},
	/* sv */ {IDGS_MON_EXIT_DESCRIPTION_SV, "L‰mna monitorn och ÂtergÂ till programkˆrningen."},
	/* tr */ {IDGS_MON_EXIT_DESCRIPTION_TR, "Monitˆr¸ terk et ve Áal˝˛t˝rmaya geri dˆn."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_QUIT_DESCRIPTION,    N_("Exit the emulator immediately.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_QUIT_DESCRIPTION_DA, "Afslut emulatoren ¯jblikkeligt."},
	/* de */ {IDGS_MON_QUIT_DESCRIPTION_DE, "Emulator sofort beenden."},
	/* es */ {IDGS_MON_QUIT_DESCRIPTION_ES, "Salir inmediatamente del emulador."},
	/* fr */ {IDGS_MON_QUIT_DESCRIPTION_FR, "Quitter l'Èmulateur immÈdiatement."},
	/* hu */ {IDGS_MON_QUIT_DESCRIPTION_HU, "Azonnali kilÈpÈs az emul·torbÛl."},
	/* it */ {IDGS_MON_QUIT_DESCRIPTION_IT, "Esci immediatamente dall'emulatore."},
	/* ko */ {IDGS_MON_QUIT_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_QUIT_DESCRIPTION_NL, "Onmiddelijk de emulator afsluiten."},
	/* pl */ {IDGS_MON_QUIT_DESCRIPTION_PL, "Natychmiast wyjdº z emulatora."},
	/* ru */ {IDGS_MON_QUIT_DESCRIPTION_RU, "≤ÎŸ‚ÿ ÿ◊ Ì‹„€Ô‚ﬁ‡– ›’‹’‘€’››ﬁ."},
	/* sv */ {IDGS_MON_QUIT_DESCRIPTION_SV, "Avsluta emulatorn omedelbart."},
	/* tr */ {IDGS_MON_QUIT_DESCRIPTION_TR, "Em¸lasyondan derhal Á˝k."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_RADIX_DESCRIPTION,    N_("Set the default radix to hex, decimal, octal, or binary.  With no\nargument, the current radix is printed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_RADIX_DESCRIPTION_DA, "SÊt standardradix til hex, decimal, oktal eller binÊr. Hvis der\nikke angives et argument vises det aktuelle radix."},
	/* de */ {IDGS_MON_RADIX_DESCRIPTION_DE, "Setze die Standardzahlenbasis auf Hexadezimal, Dezimal, Oktal oder Bin‰r.\nBei keinem Argument wird die akteulle Zahlenbasis ausgegeben."},
	/* es */ {IDGS_MON_RADIX_DESCRIPTION_ES, "Seleccionar sistema numÈrico por defecto como hex, decimal, octal, o binario.\nSi no se da argumento, se imprime el sistema numÈrico en uso."},
	/* fr */ {IDGS_MON_RADIX_DESCRIPTION_FR, "RËgle la base par dÈfaut ‡ hex, decimal, octal ou binary. Avec\naucun argument, la base actuelle est affichÈ."},
	/* hu */ {IDGS_MON_RADIX_DESCRIPTION_HU, "Az alap sz·mrendszer hexadecim·lis, decim·lis, okt·lis vagy bin·ris.\nParamÈter nÈlk¸l az aktu·lisat mutatja."},
	/* it */ {IDGS_MON_RADIX_DESCRIPTION_IT, "Imposta la radice predefinita a esadecimale, decimale, ottale o\nbinaria. Senza parametro viene stampata la radice attuale."},
	/* ko */ {IDGS_MON_RADIX_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_RADIX_DESCRIPTION_NL, "Zet de standaard radix naar hex, decimaal, octaal of binair. Als er\ngeen parameter is opgegeven, zal de huidige radix worden getoond."},
	/* pl */ {IDGS_MON_RADIX_DESCRIPTION_PL, "Ustawia domy∂lny korzeÒ na heksadecymalny, decymalny, Û∂emkowy\nlub binarny. Bez podanego argumentu, pokazywany jest obecny korzeÒ."},
	/* ru */ {IDGS_MON_RADIX_DESCRIPTION_RU, "Set the default radix to hex, decimal, octal, or binary.  With no\nargument, the current radix is printed."},
	/* sv */ {IDGS_MON_RADIX_DESCRIPTION_SV, "S‰tt standardradix till hex, decimal, oktal eller bin‰r. Om du\ninte ger nÂgot argument visas nuvarande radix."},
	/* tr */ {IDGS_MON_RADIX_DESCRIPTION_TR, "Varsay˝lan say˝ taban˝n˝ 16'l˝, 10'lu, 8'li ya da 2'li yap.\nArg¸mans˝z olarak geÁerli say˝ taban˝ yazd˝r˝l˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_SIDEFX_DESCRIPTION,    N_("Control how monitor generated reads affect memory locations that have\nread side-effects.  If the argument is 'on' then reads may cause\nside-effects.  If the argument is 'off' then reads don't cause\nside-effects.  If the argument is 'toggle' then the current mode is\nswitched.  No argument displays the current state.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_SIDEFX_DESCRIPTION_DA, "Bestemmer hvordan lÊsninger i monitoren pÂvirker hukommelsesomrÂder,\nder har sedeeffekter ved lÊsning. Hvis argumentet er \"on\" kan lÊsninger\nforesage sideeffekter. Hvis argumentet er \"off\" vil lÊsninger ikke\nforesage sideeffekter. Hvis argumentet er \"toggle\" skiftes\nden nuvÊrende tilstand. Hvis der ikke angives noget argument vises\nden nuvÊrende tilstand."},
	/* de */ {IDGS_MON_SIDEFX_DESCRIPTION_DE, "Steuerung, wie die vom Monitor erzeugten Lesezugriffe die Speicherzellen\nmit Lese-Seiteneffekten betreffen.\nBei Argument \"on\" werden Lese-Seiteneffekte erzeugt.\nBei Argument \"off\" werden keine Lese-Seiteneffekte erzeugt.\nBei Argument \"toggle\" wird das Verhalten vertauscht.\nBei keinem Argument wird das aktuelle Verhalten ausgegeben."},
	/* es */ {IDGS_MON_SIDEFX_DESCRIPTION_ES, "Controla como manejar las lecturas de localizaciones de memoria generadas por el\nmonitor que tengan efectos colaterales, Si el argumento es 'on' las lecturas causar·n\nefectos colaterales. Si el argumento es 'off' no se causar·n efectos colaterales. Si el\nargumento es 'intercambiar' se intercambiara el modo en uso. Sin argumentos, se \nmostrar· el estado en uso."},
	/* fr */ {IDGS_MON_SIDEFX_DESCRIPTION_FR, "ContrÙle comment les lectures du moniteur affecte les emplacements\nmÈmoire ayant des effets secondaires de lecture. Si l'argument est\n'on' alors les lectures peuvent causer des effets secondaires. Si\nl'argument est 'off' les lectures n'en causeront pas. Si l'argument\nest 'toggle' alors le mode actuel est commutÈ. Aucun argument ne peut\nafficher le mode en cours."},
	/* hu */ {IDGS_MON_SIDEFX_DESCRIPTION_HU, "Be·llÌtja, hogyan hatnak a monitorprogram memÛria olvas·sai azokra a\nmemÛriahelyekre, amelyeknek olvas·s-mellÈkhat·sai vannak. Ha a paramÈter\n'on', az olvas·sok okozhatnak mellÈkhat·sokat. Ha a paramÈter 'off', akkor pedig\nnem okoznak. Ha a paramÈter 'toggle', az aktu·lis mÛdot ·tkapcsolja.\nParamÈter nÈlk¸li hÌv·s kiÌrja az aktu·lis mÛdot."},
	/* it */ {IDGS_MON_SIDEFX_DESCRIPTION_IT, "Controlla come le letture generate dal monitor affliggono le locazioni\ndi memoria che hanno degli effetti collaterali sulla lettura.\nSe il paremetro Ë 'on', le letture possono causare effetti collaterali.\nSe il parametro Ë 'off', le letture non causano effetti collaterali.\nSe il parametro Ë 'scambia', la modalit‡ attuale viene invertita.\nNessun parametro mostra lo stato attuale."},
	/* ko */ {IDGS_MON_SIDEFX_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_SIDEFX_DESCRIPTION_NL, "Geef aan hoe de leesacties van de monitor de geheugenlocaties met lees\nbij-effecten beÔnvloedt. Als de parameter 'off' is, zullen lees acties\ngeen bij-effecten vertonen. Als de parameter 'on' is, zullen lees\nacties bij-effecten vertonen. Als de parameter 'toggle' is, zal de\nhuidige modus worden gewisseld. Als er geen parameter is, zal de\nhuidige modus worden getoond."},
	/* pl */ {IDGS_MON_SIDEFX_DESCRIPTION_PL, "Kontroluje w jaki sposÛb wygenerowane w monitorze odczyty po≥oøenia\nwp≥ywaj± na obszary pamiÍci, ktÛre maj± efekty uboczne odczytu.\nJe∂li argument to 'on', wtedy odczyty mog± powodowaÊ efekty uboczne.\nJe∂li argument to 'off', wtedy odczyty nie powoduj± efektÛw ubocznych.\nJe∂li argument to 'toggle', obecny tryb jest prze≥±czany.\nBrak argumentu pokazuje obecny stan."},
	/* ru */ {IDGS_MON_SIDEFX_DESCRIPTION_RU, "Control how monitor generated reads affect memory locations that have\nread side-effects.  If the argument is 'on' then reads may cause\nside-effects.  If the argument is 'off' then reads don't cause\nside-effects.  If the argument is 'toggle' then the current mode is\nswitched.  No argument displays the current state."},
	/* sv */ {IDGS_MON_SIDEFX_DESCRIPTION_SV, "Styr hur l‰sningar i monitorn pÂverkar minnesomrÂden som har\nsidoeffekter vid l‰sning. Om argumentet ‰r \"on\" tillÂts l‰sningar\natt orsaka sidoeffekter. Om argumentet ‰r \"off\" tillÂts inte\nl‰sningar att orsaka sidoeffekter. Om argumentet ‰r \"toggle\" byts\ndet nuvarande l‰get. Om du inte anger nÂgra argument visas\nnuvarande l‰ge."},
	/* tr */ {IDGS_MON_SIDEFX_DESCRIPTION_TR, "Okuma yan etkileri bulunan bellek konumlar˝n˝ monitˆr taraf˝ndan olu˛turulan\nokumalar˝n nas˝l etkilediini kontrol et.  Arg¸man 'on' ise, okumalar yan\netkiye neden olabilir.  Arg¸man 'off' ise okumalar˝n bir yan etkisi\nolmayacakt˝r.  Eer arg¸man 'toggle' ise geÁerli mod dei˛tirilecektir.\nArg¸man verilmemesi o anki durumu gˆr¸nt¸ler."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_DISK_COMMANDS,    N_("Disk commands:")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DISK_COMMANDS_DA, "Diskettekommandoer:"},
	/* de */ {IDGS_DISK_COMMANDS_DE, "Disk Kommandos:"},
	/* es */ {IDGS_DISK_COMMANDS_ES, "Comandos de disco:"},
	/* fr */ {IDGS_DISK_COMMANDS_FR, "Commandes de disque:"},
	/* hu */ {IDGS_DISK_COMMANDS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_DISK_COMMANDS_IT, "Comandi disco:"},
	/* ko */ {IDGS_DISK_COMMANDS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DISK_COMMANDS_NL, "Disk commando's:"},
	/* pl */ {IDGS_DISK_COMMANDS_PL, "Polecenia dyskowe:"},
	/* ru */ {IDGS_DISK_COMMANDS_RU, "Disk commands:"},
	/* sv */ {IDGS_DISK_COMMANDS_SV, "Diskettkommandon:"},
	/* tr */ {IDGS_DISK_COMMANDS_TR, "Disk komutlar˝:"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_DISK_COMMAND,    N_("disk command")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DISK_COMMAND_DA, "diskkommando"},
	/* de */ {IDGS_DISK_COMMAND_DE, "Disk Kommando"},
	/* es */ {IDGS_DISK_COMMAND_ES, "comando disco"},
	/* fr */ {IDGS_DISK_COMMAND_FR, "commande de disque"},
	/* hu */ {IDGS_DISK_COMMAND_HU, ""},  /* fuzzy */
	/* it */ {IDGS_DISK_COMMAND_IT, "comando disco"},
	/* ko */ {IDGS_DISK_COMMAND_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DISK_COMMAND_NL, "disk commando"},
	/* pl */ {IDGS_DISK_COMMAND_PL, "disk command"},
	/* ru */ {IDGS_DISK_COMMAND_RU, "disk command"},
	/* sv */ {IDGS_DISK_COMMAND_SV, "diskkommando"},
	/* tr */ {IDGS_DISK_COMMAND_TR, "disk komutu"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_AT_DESCRIPTION,    N_("Perform a disk command on the currently attached disk image on drive 8.\nThe specified disk command is sent to the drive's channel #15.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_AT_DESCRIPTION_DA, "Udf¯rer en diskkommando pÂ det diskette-image, der er tilsluttet\ndrev 8. Diskkommandoen sendes til drevets kanal #15."},
	/* de */ {IDGS_MON_AT_DESCRIPTION_DE, "F¸hre ein Diskkommando auf dem aktuell eingelegtem Diskimage auf Laufwerk 8\naus.\nDas angegebene Kommando wird auf dem Laufwerks Kommandokanal #15 geschickt."},
	/* es */ {IDGS_MON_AT_DESCRIPTION_ES, "Realiza un comando de disco en el actualmente insertado en unidad disco 8.\nEl comando de disco especificado se envÌa a travÈs del canal de control #15."},
	/* fr */ {IDGS_MON_AT_DESCRIPTION_FR, "Performe une commande disque ‡ l'image disque actuellement attachÈe au\nlecteur #8. La commande spÈcifiÈe est envoyÈe au canal #15 du lecteur."},
	/* hu */ {IDGS_MON_AT_DESCRIPTION_HU, "Lemez parancs vÈgrehajt·sa az aktu·lisan csatolt lemezen, a 8-as egysÈgen.\nA megadott lemez parancsot a meghajtÛ #15-ˆs csatorn·j·ra k¸ldi."},
	/* it */ {IDGS_MON_AT_DESCRIPTION_IT, "Esegue un comando disco sull'immagine disco attualmente presente nel\ndrive 8. Il comando specificato Ë inviato al canale #15 del drive."},
	/* ko */ {IDGS_MON_AT_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_AT_DESCRIPTION_NL, "Voer een disk commando uit op het huidig gekoppeld disk bestand in drive\n8, Het opgegeven disk commando zal worden gestuurd naar kanaal #15 van de\ndrive."},
	/* pl */ {IDGS_MON_AT_DESCRIPTION_PL, "Wykonaj polecenie dyskowe na obecnie zamontowanym obrazie dyskietki\nw stacji 8. Dane polecenie jest wysy≥ane na kana≥ #15 stacji."},
	/* ru */ {IDGS_MON_AT_DESCRIPTION_RU, "Perform a disk command on the currently attached disk image on drive 8.\nThe specified disk command is sent to the drive's channel #15."},
	/* sv */ {IDGS_MON_AT_DESCRIPTION_SV, "Utfˆr ett diskkommando pÂ den diskettavbildning som ‰r ansluten till\nenhet 8. Diskkommandot s‰nds pÂ diskettenhetens kanal nummer 15."},
	/* tr */ {IDGS_MON_AT_DESCRIPTION_TR, "8 nolu s¸r¸c¸de tak˝l˝ disk imaj˝ ¸zerinde bir disk komutu Áal˝˛t˝r.\nBelirtilen disk komutu s¸r¸c¸n¸n 15 numaral˝ kanal˝na gˆnderilir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_DEVICE,    N_("device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DEVICE_DA, "enhed"},
	/* de */ {IDGS_DEVICE_DE, "Ger‰t"},
	/* es */ {IDGS_DEVICE_ES, "perifÈrico"},
	/* fr */ {IDGS_DEVICE_FR, "pÈriphÈrique"},
	/* hu */ {IDGS_DEVICE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_DEVICE_IT, "dispositivo"},
	/* ko */ {IDGS_DEVICE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DEVICE_NL, "apparaat"},
	/* pl */ {IDGS_DEVICE_PL, "device"},
	/* ru */ {IDGS_DEVICE_RU, "device"},
	/* sv */ {IDGS_DEVICE_SV, "enhet"},
	/* tr */ {IDGS_DEVICE_TR, "ayg˝t"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_FILE_INDEX,    N_("file_index")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_FILE_INDEX_DA, "fil_indeks"},
	/* de */ {IDGS_FILE_INDEX_DE, "Datei_Index"},
	/* es */ {IDGS_FILE_INDEX_ES, "Õndice de fichero"},
	/* fr */ {IDGS_FILE_INDEX_FR, "index_fichier"},
	/* hu */ {IDGS_FILE_INDEX_HU, ""},  /* fuzzy */
	/* it */ {IDGS_FILE_INDEX_IT, "file_index"},
	/* ko */ {IDGS_FILE_INDEX_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_FILE_INDEX_NL, "bestandsindex"},
	/* pl */ {IDGS_FILE_INDEX_PL, "file_index"},
	/* ru */ {IDGS_FILE_INDEX_RU, "file_index"},
	/* sv */ {IDGS_FILE_INDEX_SV, "filindex"},
	/* tr */ {IDGS_FILE_INDEX_TR, "dosya_indeksi"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_AUTOSTART_DESCRIPTION,    N_("autostart a given disk/tape image or program")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_AUTOSTART_DESCRIPTION_DA, "Autostart et givent disk-/bÂnd-image eller program"},
	/* de */ {IDGS_MON_AUTOSTART_DESCRIPTION_DE, "Autostart ein Disk/Band Image oder Programm"},
	/* es */ {IDGS_MON_AUTOSTART_DESCRIPTION_ES, "Autoarranque imagen de disco/cinta Û programa"},
	/* fr */ {IDGS_MON_AUTOSTART_DESCRIPTION_FR, "AutodÈmarrage d'une image disque/datassette ou programme"},
	/* hu */ {IDGS_MON_AUTOSTART_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_AUTOSTART_DESCRIPTION_IT, "Avvia automaticamente immagine un'immagine disco/cassetta o programma"},
	/* ko */ {IDGS_MON_AUTOSTART_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_AUTOSTART_DESCRIPTION_NL, "autostart een opgegeven disk-/tapebestand of programma"},
	/* pl */ {IDGS_MON_AUTOSTART_DESCRIPTION_PL, "automatycznie uruchom dany program albo obraz dysku lub ta∂my"},
	/* ru */ {IDGS_MON_AUTOSTART_DESCRIPTION_RU, "autostart a given disk/tape image or program"},
	/* sv */ {IDGS_MON_AUTOSTART_DESCRIPTION_SV, "starta given disk-/bandavbildningsfil eller program automatiskt"},
	/* tr */ {IDGS_MON_AUTOSTART_DESCRIPTION_TR, "verilen disk/teyp imaj˝n˝ ya da program˝ otomatik ba˛lat"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_AUTOLOAD_DESCRIPTION,    N_("autoload given disk/tape image or program")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_AUTOLOAD_DESCRIPTION_DA, "Autoload et givent disk-/bÂnd-image eller program"},
	/* de */ {IDGS_MON_AUTOLOAD_DESCRIPTION_DE, "Autoload ein Disk/Band Image oder Programm"},
	/* es */ {IDGS_MON_AUTOLOAD_DESCRIPTION_ES, "Autocarga imagen de disco/cinta Û programa"},
	/* fr */ {IDGS_MON_AUTOLOAD_DESCRIPTION_FR, "Charger automatiquement une image de disque/datassette"},
	/* hu */ {IDGS_MON_AUTOLOAD_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_AUTOLOAD_DESCRIPTION_IT, "Carica automaticamente immagine un'immagine disco/cassetta o programma"},
	/* ko */ {IDGS_MON_AUTOLOAD_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_AUTOLOAD_DESCRIPTION_NL, "autolaad een opgegeven disk-/tapebestand of programma"},
	/* pl */ {IDGS_MON_AUTOLOAD_DESCRIPTION_PL, "automatycznie wczytaj dany program albo obraz dysku lub ta∂my"},
	/* ru */ {IDGS_MON_AUTOLOAD_DESCRIPTION_RU, "autoload given disk/tape image or program"},
	/* sv */ {IDGS_MON_AUTOLOAD_DESCRIPTION_SV, "l‰s in given disk-/bandavbildningsfil eller program automatiskt"},
	/* tr */ {IDGS_MON_AUTOLOAD_DESCRIPTION_TR, "verilen disk/teyp imaj˝n˝ ya da program˝ otomatik y¸klet"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_ATTACH_DESCRIPTION,    N_("Attach file to device. (device 32 = cart)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_ATTACH_DESCRIPTION_DA, "Tilslut en fil til enhed. (enhed 32 = cartridge)"},
	/* de */ {IDGS_MON_ATTACH_DESCRIPTION_DE, "Benutze Datei f¸r das Ger‰t. (Ger‰t 32 = Modul)"},
	/* es */ {IDGS_MON_ATTACH_DESCRIPTION_ES, "Inserta fichero en perifÈrico (perifÈrico 32=cartucho)"},
	/* fr */ {IDGS_MON_ATTACH_DESCRIPTION_FR, "Attacher un fichier ‡ un pÈriphÈrique. (PÈriph. 32 = cartouche)"},
	/* hu */ {IDGS_MON_ATTACH_DESCRIPTION_HU, "F·jl csatol·sa az eszkˆzhˆz. (32-es eszkˆz = cart)"},
	/* it */ {IDGS_MON_ATTACH_DESCRIPTION_IT, "Seleziona il file per la periferica. (periferica 32 = cartuccia)"},
	/* ko */ {IDGS_MON_ATTACH_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_ATTACH_DESCRIPTION_NL, "Koppel bestand aan apparaat. (apparaat 32 = cart)"},
	/* pl */ {IDGS_MON_ATTACH_DESCRIPTION_PL, "Pod≥±cz plik pod urz±dzenie (urz±dzenie 32 = cart)"},
	/* ru */ {IDGS_MON_ATTACH_DESCRIPTION_RU, "Attach file to device. (device 32 = cart)"},
	/* sv */ {IDGS_MON_ATTACH_DESCRIPTION_SV, "Anslut en fil till en enhet. (enhet 32 = insticksmodul)"},
	/* tr */ {IDGS_MON_ATTACH_DESCRIPTION_TR, "Ayg˝ta dosya yerle˛tir. (ayg˝t 32 = kartu˛)"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_BLOAD_DESCRIPTION,    N_("Load the specified file into memory at the specified address.\nIf device is 0, the file is read from the file system.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_BLOAD_DESCRIPTION_DA, "IndlÊs den angivne fil til hukommelsen pÂ den angivne adresse.\nHvis enhed er 0 lÊses fra filsystemet."},
	/* de */ {IDGS_MON_BLOAD_DESCRIPTION_DE, "Lade angegebene Datei in den Speicher an die angegebene Adresse.\nBei Ger‰t 0, wird die Datei vom Dateisystem gelesen."},
	/* es */ {IDGS_MON_BLOAD_DESCRIPTION_ES, "Cargar el fichero especificado en memoria en la direcciÛn indicada en\ndirecciÛn. Si el perifÈrico es 0 el fichero se lee desde sistema."},
	/* fr */ {IDGS_MON_BLOAD_DESCRIPTION_FR, "Charge le fichier spÈcifiÈ en mÈmoire ‡ l'adresse spÈcifiÈe.\nSi le pÈriphÈrique est 0, le fichier est lu du systËme de fichier."},
	/* hu */ {IDGS_MON_BLOAD_DESCRIPTION_HU, "A megadott f·jl betˆltÈse a memÛri·ba a megadott cÌmre.\nHa az eszkˆz 0, a f·jlt a f·jlrendszerbıl olvassa."},
	/* it */ {IDGS_MON_BLOAD_DESCRIPTION_IT, "Carica il file specificato in memoria all'indirizzo specificato.\nSe la periferica Ë 0, il file Ë letto dal file system."},
	/* ko */ {IDGS_MON_BLOAD_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_BLOAD_DESCRIPTION_NL, "Laad het opgegeven bestand naar het geheugen met het opgegeven adres.\nAls het apparaat 0 is, zal het bestand van het bestandssysteem\nworden geladen."},
	/* pl */ {IDGS_MON_BLOAD_DESCRIPTION_PL, "Wczytaj dany plik do pamiÍci w okre∂lonym adresie.\nJe∂li urz±dzenie to 0, plik jest czytany z systemu plikÛw."},
	/* ru */ {IDGS_MON_BLOAD_DESCRIPTION_RU, "Load the specified file into memory at the specified address.\nIf device is 0, the file is read from the file system."},
	/* sv */ {IDGS_MON_BLOAD_DESCRIPTION_SV, "L‰s den angivna filen till minnet pÂ den angivna adressen.\nOm enheten ‰r 0 l‰ses filen frÂn filsystemet."},
	/* tr */ {IDGS_MON_BLOAD_DESCRIPTION_TR, "Belirtilmi˛ dosyay˝ belirtilen adresten itibaren bellee y¸kle.\nAyg˝t 0 ise, dosya dosya sistemi ¸zerinden okunur."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_TRACK,    N_("track")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_TRACK_DA, "spor"},
	/* de */ {IDGS_TRACK_DE, "Spur"},
	/* es */ {IDGS_TRACK_ES, "Pista"},
	/* fr */ {IDGS_TRACK_FR, "piste"},
	/* hu */ {IDGS_TRACK_HU, ""},  /* fuzzy */
	/* it */ {IDGS_TRACK_IT, "traccia"},
	/* ko */ {IDGS_TRACK_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_TRACK_NL, "spoor"},
	/* pl */ {IDGS_TRACK_PL, "track"},
	/* ru */ {IDGS_TRACK_RU, "track"},
	/* sv */ {IDGS_TRACK_SV, "spÂr"},
	/* tr */ {IDGS_TRACK_TR, "track"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_SECTOR,    N_("sector")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SECTOR_DA, "sektor"},
	/* de */ {IDGS_SECTOR_DE, "Sektor"},
	/* es */ {IDGS_SECTOR_ES, "sector"},
	/* fr */ {IDGS_SECTOR_FR, "secteur"},
	/* hu */ {IDGS_SECTOR_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SECTOR_IT, "settore"},
	/* ko */ {IDGS_SECTOR_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SECTOR_NL, "sector"},
	/* pl */ {IDGS_SECTOR_PL, "sector"},
	/* ru */ {IDGS_SECTOR_RU, "·’⁄‚ﬁ‡"},
	/* sv */ {IDGS_SECTOR_SV, "sektor"},
	/* tr */ {IDGS_SECTOR_TR, "sektˆr"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_BLOCK_READ_DESCRIPTION,    N_("Read the block at the specified track and sector.  If an address is\nspecified, the data is loaded into memory.  If no address is given, the\ndata is displayed using the default datatype.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_BLOCK_READ_DESCRIPTION_DA, "LÊser en blok fra det angivne spor og sektor. Hvis en adresse angives\nindlÊses data til hukommelsen. Hvis der ikke angives nogen adresse vises\nindholdet med standarddatatypen."},
	/* de */ {IDGS_MON_BLOCK_READ_DESCRIPTION_DE, "Lese den Block von angegebener Spur und Sektor. Wenn eine Adresse\nangegeben wurde, lade in auf die entsprechende Adresse. Wenn keine Adresse\nangegeben ist, zeige die Daten im aktuellen Anzeigemodus f¸r Daten."},
	/* es */ {IDGS_MON_BLOCK_READ_DESCRIPTION_ES, "Leer bloque en la pista y sector especificados. Si la direcciÛn se\nespecifica, el dato se carga en memoria, si no, el dato se muestra\nusando el tipo dato por defecto."},
	/* fr */ {IDGS_MON_BLOCK_READ_DESCRIPTION_FR, "Lit le bloc ‡ la piste et au secteur spÈcifiÈs. Si une adresse est\nspÈcifiÈe, les donnÈes sont chargÈes en mÈmoire. Si aucune adresse\nn'est spÈcifiÈe, les donnÈes sont affichÈs en utilisant le type de\ndonnÈes par dÈfaut."},
	/* hu */ {IDGS_MON_BLOCK_READ_DESCRIPTION_HU, "Blokk beolvas·sa a megadott s·vbÛl Ès szektorbÛl.,  Ha egy cÌm is meg\nvan adva, az adatot a memÛri·ba tˆlti.  Ha az nincs, az\nadatot az alapbe·llÌt·s szerinti form·ban mutatja."},
	/* it */ {IDGS_MON_BLOCK_READ_DESCRIPTION_IT, "Legge il blocco dalla traccia e settore specificati. Se Ë specificato\nun indirizzo, il dato Ë caricato in memoria. Se non Ë specificato alcun\nindirizzo, il dato Ë mostrato usando il tipo di dato predefinito."},
	/* ko */ {IDGS_MON_BLOCK_READ_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_BLOCK_READ_DESCRIPTION_NL, "Lees het blok van het opgegeven spoor en sector. Als er een adres is\nopgegeven, zal de data naar dat adres in het geheugen worden geladen.\nAls er geen adres is opgegeven, zal de data worden getoond met de\nhuidige datasoort."},
	/* pl */ {IDGS_MON_BLOCK_READ_DESCRIPTION_PL, "Czytaj blok z danej ∂cieøki i sektora. Je∂li okre∂lono adres, dane s±\nwczytywane do pamiÍci. Je∂li nie podano adresu, dane s± pokazane\nw domy∂lnym typie danych."},
	/* ru */ {IDGS_MON_BLOCK_READ_DESCRIPTION_RU, "Read the block at the specified track and sector.  If an address is\nspecified, the data is loaded into memory.  If no address is given, the\ndata is displayed using the default datatype."},
	/* sv */ {IDGS_MON_BLOCK_READ_DESCRIPTION_SV, "L‰ser blocket pÂ det angivna spÂret och sektorn. Om du anger en\nadress l‰ses data till minnet. Om du inte anger nÂgon adress visas\ninnehÂllet enligt standarddatatypen."},
	/* tr */ {IDGS_MON_BLOCK_READ_DESCRIPTION_TR, "Belirtilmi˛ track ve sektˆrdeki blou oku.  Eer bir adres belirtilmi˛se,\nveri bellee y¸klenir.  Eer adres belirtilmemi˛se, veri varsay˝lan veri\ntipi kullan˝larak gˆr¸nt¸lenir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_BSAVE_DESCRIPTION,    N_("Save the memory from address1 to address2 to the specified file.\nIf device is 0, the file is written to the file system.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_BSAVE_DESCRIPTION_DA, "Gem hukommelsen fra adresse1 til adresse2 til den angivne fil.\nHvis enhed er 0 skrives til filsystemet."},
	/* de */ {IDGS_MON_BSAVE_DESCRIPTION_DE, "Speichere den Speicherinhalt von Adresse1 zu Adresse2 in die angegebene Datei.\nWenn das Ger‰t 0 angegeben wurde, speichere im Dateisystem."},
	/* es */ {IDGS_MON_BSAVE_DESCRIPTION_ES, "Grabar la memoria entre direcciÛn1 y direcciÛn2 al fichero indicado.\nSi el perifÈrico es 0, el fichero se graba en el fichero del sistema."},
	/* fr */ {IDGS_MON_BSAVE_DESCRIPTION_FR, "Sauvegarde la mÈmoire depuis adress1 jusqu'‡ adress2 dans le\nfichier spÈcifiÈ. Si le pÈriphÈrique est 0, alors le fichier\nest Ècrit dans le systËme de fichier."},
	/* hu */ {IDGS_MON_BSAVE_DESCRIPTION_HU, "A memÛria cÌm1 Ès cÌm2 kˆzˆtti rÈszÈnek f·jlba mentÈse.\nHa az eszkˆz 0, a f·jl a f·jlrendszerbe ker¸l."},
	/* it */ {IDGS_MON_BSAVE_DESCRIPTION_IT, "Salva la memoria compresa tra indirizzo1 e indirizzo2 sul file specificato.\nSe la periferica Ë 0, il file Ë scritto sul file system."},
	/* ko */ {IDGS_MON_BSAVE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_BSAVE_DESCRIPTION_NL, "Sla het geheugen tussen adres1 en adres2 op naar het opgegeven bestand.\nAls het apparaat 0 is, zal het bestand naar het bestandssysteem worden\ngeschreven."},
	/* pl */ {IDGS_MON_BSAVE_DESCRIPTION_PL, "Zapisz pamiÍÊ z adres1 do adres2 do okre∂lonego pliku.\nJe∂li urz±dzenie to 0, plik jest zapisywany do systemu plikÛw."},
	/* ru */ {IDGS_MON_BSAVE_DESCRIPTION_RU, "Save the memory from address1 to address2 to the specified file.\nIf device is 0, the file is written to the file system."},
	/* sv */ {IDGS_MON_BSAVE_DESCRIPTION_SV, "Spara minnet frÂn adress1 till adress2 till den angivna filen.\nOm enheten ‰r 0 skrivs filen till filsystemet."},
	/* tr */ {IDGS_MON_BSAVE_DESCRIPTION_TR, "Adres1'den adres2'ye kadar olan bellei belirtilen dosyaya kaydet.\nEer ayg˝t 0 ise, dosya dosya sistemine yaz˝l˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION,    N_("Write a block of data at `address' on the specified track and sector\nof disk in drive 8.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_DA, "Skriv datablok pÂ \"adresse\" til det angivne spor og sektor\npÂ disken i drev 8."},
	/* de */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_DE, "Schreibe einen Datenblock von `Adresse' auf die angegebene Spure und Sektor\nin die Imagedatei von Laufwerk 8."},
	/* es */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_ES, "Grabar un bloque de datos en 'direcciÛn' en la pista y sector indicados\ndel disco en la unidad de discos 8."},
	/* fr */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_FR, "…crit un bloc de donnÈes ‡ `address' sur la piste et le secteur\nspÈcifiÈs du disque dans le lecteur #8."},
	/* hu */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_HU, "Egy adat blokk Ìr·sa `cÌm' helyrıl a megadott s·vba Ès szektorba\na 8-as meghajtÛn."},
	/* it */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_IT, "Scrive il blocco di dati all'indirizzo sulla traccia e settore specificati\ndel disco nel drive 8."},
	/* ko */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_NL, "Schrijf een data blok van `adres' naar het opgegeven spoor en sector van\nde disk in drive 8."},
	/* pl */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_PL, "Zapisz blok danych w `address' w podanej ∂cieøce i sektorze\ndysku w stacji 8."},
	/* ru */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_RU, "Write a block of data at `address' on the specified track and sector\nof disk in drive 8."},
	/* sv */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_SV, "Skriv ett datablock pÂ \"adress\" till det angivna spÂret och sektorn\npÂ disken i enhet 8."},
	/* tr */ {IDGS_MON_BLOCK_WRITE_DESCRIPTION_TR, "Belirtilmi˛ track ve sektˆrlerde `adres'te yer alan veri blounu 8\nnumaral˝ s¸r¸c¸deki diske yaz."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_DIRECTORY,    N_("Directory")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DIRECTORY_DA, "Katalog"},
	/* de */ {IDGS_DIRECTORY_DE, "Verzeichnis"},
	/* es */ {IDGS_DIRECTORY_ES, "Directorio"},
	/* fr */ {IDGS_DIRECTORY_FR, "RÈpertoire"},
	/* hu */ {IDGS_DIRECTORY_HU, ""},  /* fuzzy */
	/* it */ {IDGS_DIRECTORY_IT, "Directory"},
	/* ko */ {IDGS_DIRECTORY_KO, "µ∑∫≈‰∏Æ"},
	/* nl */ {IDGS_DIRECTORY_NL, "Directory"},
	/* pl */ {IDGS_DIRECTORY_PL, "Katalog"},
	/* ru */ {IDGS_DIRECTORY_RU, "ø–ﬂ⁄–"},
	/* sv */ {IDGS_DIRECTORY_SV, "Katalog"},
	/* tr */ {IDGS_DIRECTORY_TR, "Dizin"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_P_DIRECTORY,    N_("<Directory>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_P_DIRECTORY_DA, "<Katalog>"},
	/* de */ {IDGS_P_DIRECTORY_DE, "<Verzeichnis>"},
	/* es */ {IDGS_P_DIRECTORY_ES, "<Directorio>"},
	/* fr */ {IDGS_P_DIRECTORY_FR, "<RÈpertoire>"},
	/* hu */ {IDGS_P_DIRECTORY_HU, ""},  /* fuzzy */
	/* it */ {IDGS_P_DIRECTORY_IT, "<Directory>"},
	/* ko */ {IDGS_P_DIRECTORY_KO, "<µ∑∫≈‰∏Æ>"},
	/* nl */ {IDGS_P_DIRECTORY_NL, "<Directory>"},
	/* pl */ {IDGS_P_DIRECTORY_PL, "<Katalog>"},
	/* ru */ {IDGS_P_DIRECTORY_RU, "<ø–ﬂ⁄–>"},
	/* sv */ {IDGS_P_DIRECTORY_SV, "<Katalog>"},
	/* tr */ {IDGS_P_DIRECTORY_TR, "<Dizin>"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_CD_DESCRIPTION,    N_("Change current working directory.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_CD_DESCRIPTION_DA, "Skift aktuelt arbejdskatalog"},
	/* de */ {IDGS_MON_CD_DESCRIPTION_DE, "Arbeitsverzeichnis ‰ndern."},
	/* es */ {IDGS_MON_CD_DESCRIPTION_ES, "Cambiar directorio de trabajo actual."},
	/* fr */ {IDGS_MON_CD_DESCRIPTION_FR, "Modifier le rÈpertoire en cours."},
	/* hu */ {IDGS_MON_CD_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_CD_DESCRIPTION_IT, "Cambia la directory di lavoro."},
	/* ko */ {IDGS_MON_CD_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_CD_DESCRIPTION_NL, "Verander de huidige werkmap."},
	/* pl */ {IDGS_MON_CD_DESCRIPTION_PL, "ZmieÒ obecny katalog roboczy"},
	/* ru */ {IDGS_MON_CD_DESCRIPTION_RU, "¡‹’›ÿ‚Ï ‚’⁄„È„Ó ‡–—ﬁÁ„Ó ‘ÿ‡’⁄‚ﬁ‡ÿÓ."},
	/* sv */ {IDGS_MON_CD_DESCRIPTION_SV, "ƒndra aktuell arbetskatalog"},
	/* tr */ {IDGS_MON_CD_DESCRIPTION_TR, "GeÁerli Áal˝˛ma dizinini dei˛tir."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DETACH_DESCRIPTION,    N_("Detach file from device. (device 32 = cart)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DETACH_DESCRIPTION_DA, "Frakobl filen fra enheden. (enhed 32 = cartridge)"},
	/* de */ {IDGS_MON_DETACH_DESCRIPTION_DE, "Entferne Datei von Ger‰t. (Ger‰t 32 = Modul)"},
	/* es */ {IDGS_MON_DETACH_DESCRIPTION_ES, "Extraer fichero desde el perifÈrico (perifÈrico 32 = cartucho)"},
	/* fr */ {IDGS_MON_DETACH_DESCRIPTION_FR, "DÈtacher un fichier d'un pÈriphÈrique. (PÈriph. 32 = cartouche)"},
	/* hu */ {IDGS_MON_DETACH_DESCRIPTION_HU, "F·jl lev·laszt·sa az eszkˆzrıl. (32-es eszkˆz = cart)"},
	/* it */ {IDGS_MON_DETACH_DESCRIPTION_IT, "Rimuovi il file dalla periferica. (periferica 32 = cart)"},
	/* ko */ {IDGS_MON_DETACH_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DETACH_DESCRIPTION_NL, "Ontkoppel bestand van apparaat. (apparaat 32 = cart)"},
	/* pl */ {IDGS_MON_DETACH_DESCRIPTION_PL, "Od≥±cz plik od urz±dzenia (urz±dzenie 32 = cart)"},
	/* ru */ {IDGS_MON_DETACH_DESCRIPTION_RU, "Detach file from device. (device 32 = cart)"},
	/* sv */ {IDGS_MON_DETACH_DESCRIPTION_SV, "Koppla frÂn filen frÂn enheten. (enhet 32 = insticksmodul)"},
	/* tr */ {IDGS_MON_DETACH_DESCRIPTION_TR, "Ayg˝ttan dosya Á˝kart. (ayg˝t 32 = kartu˛)"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DIR_DESCRIPTION,    N_("Display the directory contents.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DIR_DESCRIPTION_DA, "Vis katalogets indhold."},
	/* de */ {IDGS_MON_DIR_DESCRIPTION_DE, "Zeige den Verzeichnisinhalt."},
	/* es */ {IDGS_MON_DIR_DESCRIPTION_ES, "Mostrar los contenidos del directorio"},
	/* fr */ {IDGS_MON_DIR_DESCRIPTION_FR, "Affiche le contenu du rÈpertoire."},
	/* hu */ {IDGS_MON_DIR_DESCRIPTION_HU, "A kˆnyvt·r tartalm·nak mutat·sa."},
	/* it */ {IDGS_MON_DIR_DESCRIPTION_IT, "Mostra il contenuto della directory."},
	/* ko */ {IDGS_MON_DIR_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DIR_DESCRIPTION_NL, "Toon de directory-inhoud."},
	/* pl */ {IDGS_MON_DIR_DESCRIPTION_PL, "Pokaø zawarto∂Ê katalogu."},
	/* ru */ {IDGS_MON_DIR_DESCRIPTION_RU, "æ‚ﬁ—‡–◊ÿ‚Ï ·ﬁ‘’‡÷–›ÿ’ ‘ÿ‡’⁄‚ﬁ‡ÿÿ."},
	/* sv */ {IDGS_MON_DIR_DESCRIPTION_SV, "Visa katalogens innehÂll."},
	/* tr */ {IDGS_MON_DIR_DESCRIPTION_TR, "Dizin iÁeriini gˆr¸nt¸le."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_LIST_DESCRIPTION,    N_("List disk contents.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_LIST_DESCRIPTION_DA, "Vis diskens indhold."},
	/* de */ {IDGS_MON_LIST_DESCRIPTION_DE, "Zeige den Verzeichnisinhalt."},
	/* es */ {IDGS_MON_LIST_DESCRIPTION_ES, "Listado contenidos disco"},
	/* fr */ {IDGS_MON_LIST_DESCRIPTION_FR, "Affiche le contenu du disque."},
	/* hu */ {IDGS_MON_LIST_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_LIST_DESCRIPTION_IT, "Elenca i contenuti del disco."},
	/* ko */ {IDGS_MON_LIST_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_LIST_DESCRIPTION_NL, "Toon disk inhoud."},
	/* pl */ {IDGS_MON_LIST_DESCRIPTION_PL, "Pokaø zawarto∂Ê dysku"},
	/* ru */ {IDGS_MON_LIST_DESCRIPTION_RU, "List disk contents."},
	/* sv */ {IDGS_MON_LIST_DESCRIPTION_SV, "Visa diskens innehÂll"},
	/* tr */ {IDGS_MON_LIST_DESCRIPTION_TR, "Disk iÁeriini listele."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_LOAD_DESCRIPTION,    N_("Load the specified file into memory at the specified address. Set BASIC\npointers appropriately if loaded into computer memory (not all emulators).\nUse (otherwise ignored) two-byte load address from file if no address\nspecified.\nIf device is 0, the file is read from the file system.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_LOAD_DESCRIPTION_DA, "IndlÊs filen i hukommelsen pÂ den angivne adresse. Opdaterer\nBASIC-pointers ved lÊsning til computerens hukommelse (ikke alle\nemulatorer). Hvis ingen adresse angives bruges load-adressen fra(som ellers ignoreres).\nHvis enhed er 0 lÊses fra filsystemet."},
	/* de */ {IDGS_MON_LOAD_DESCRIPTION_DE, "Lade die angegebene Datei in den Speicher auf die angegebene Adresse.\nSetze den BASIC Zeiger entsprechend, wenn in den Rechnerspeicher geladen \nwurde (nicht bei allen Emulatoren!).\nVerwende (sonst ignoriert) die zwei-Byte Lade-Adresse aus der Datei, wenn\nkeine Adresse angegeben wurde.\nBei Ger‰t 0, wird die Datei vom Dateisystem gelade."},
	/* es */ {IDGS_MON_LOAD_DESCRIPTION_ES, "Cargar el fichero especificado en la direcciÛn de memoria indicada. Actualizar\npunteros BASIC apropiadamente si se carga en la memoria del ordenador\n(no en todos los emuladores).\nUsar direcciÛn de dos bytes del fichero si no se indica otra direcciÛn.\nSi el perifÈrico es 0, el fichero se lee del sistema."},
	/* fr */ {IDGS_MON_LOAD_DESCRIPTION_FR, "Charge le fichier spÈcifiÈ en mÈmoire ‡ l'adresse spÈcifiÈe. RËgle les\npointeurs BASIC correctement si chargÈe dans la mÈmoire de l'ordinateur\n(sur certains Èmulateurs seulement). Utilise l'adresse ‡ deux bits du\nfichier si aucune adresse de chargement n'est spÈcifiÈe.\nSi le pÈriphÈrique est 0, le fichier est lu du systËme de fichiers."},
	/* hu */ {IDGS_MON_LOAD_DESCRIPTION_HU, "A megadott f·jl betˆltÈse a megadott cÌmre. A BASIC mutatÛkat\na megfelelı mÛdon be·llÌtja, ha a sz·mÌtÛgÈp memÛri·ba tˆlti az adatot\n(nem minden emul·torn·l). Ha nincs cÌm megadva, az am˙gy figyelmen hagyott\nkÈt b·jtos betˆltÈsi cÌmet haszn·lja a f·jlbÛl.\nHa az eszkˆz 0, a f·jlt a f·jlrendszerrıl olvassa."},
	/* it */ {IDGS_MON_LOAD_DESCRIPTION_IT, "Carica il file specificato in memoria all'indirizzo specificato. Imposta\ni puntatori del BASIC in modo appropriato se caricato nella memoria\ndel computer (non su tutti gli emulatori).\nUsa (altrimenti ignora) l'indirizzo di caricamento su due byte del file\nse non viene specificato alcun indirizzo.\nSe la periferica Ë 0, il file viene letto dal file system."},
	/* ko */ {IDGS_MON_LOAD_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_LOAD_DESCRIPTION_NL, "Laad het opgegeven bestand naar het geheugen op het opgegeven adres. Zet de BASIC\nstart en eind indicators correct als het bestand naar het computer geheugen\ngeladen wordt (niet alle emulators). De eerste twee bytes van het bestand\nzullen als laad adres worden gebruikt als er geen adres is opgegeven. Als\nhet apparaat 0 is, zal het bestand van het bestandssysteem worden geladen."},
	/* pl */ {IDGS_MON_LOAD_DESCRIPTION_PL, "Wczytaj podany plik do pamiÍci w konkretnym adresie. Ustaw wskaºniki\nBASIC odpowiednio, jesli zostan± wczytane do pamiÍci komputera\n(nie wszystkie emulatory). Uøyj (w innym razie ignoruje) dwubajtowego\nadresu wczytywania z pliku, je∂li nie zosta≥ podany adres.\nJe∂li urz±dzenie jest 0, plik zostaje oczytany z systemu plikÛw."},
	/* ru */ {IDGS_MON_LOAD_DESCRIPTION_RU, "Load the specified file into memory at the specified address. Set BASIC\npointers appropriately if loaded into computer memory (not all emulators).\nUse (otherwise ignored) two-byte load address from file if no address\nspecified.\nIf device is 0, the file is read from the file system."},
	/* sv */ {IDGS_MON_LOAD_DESCRIPTION_SV, "L‰s in den angivna filen i minnet pÂ den angivna adressen. S‰tter\nmotsvarande BASIC-pekare vid l‰sning till datorminnet (inte alla\nemulatorer). Om ingen adress anges anv‰nds tvÂbytesadressen i filen(ignoreras annars).\nOm enheten ‰r 0 l‰ses filen frÂn filsystemet."},
	/* tr */ {IDGS_MON_LOAD_DESCRIPTION_TR, "Belirtilmi˛ dosyay˝ belirtilen adresten itibaren bellee y¸kle. Eer\nbilgisayar belleine y¸klenmi˛se BASIC imleÁlerini uygun bir biÁimde\nayarla (t¸m em¸latˆrlerde deil). Eer adres belirtilmemi˛se iki bytel˝k\ndosya y¸kleme adresini kullan (dier ihtimalde yoksay).\nEer ayg˝t 0 ise, dosya dosya sisteminden okunur."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_PWD_DESCRIPTION,    N_("Show current working directory.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_PWD_DESCRIPTION_DA, "Vis aktuelt arbejdskatalog."},
	/* de */ {IDGS_MON_PWD_DESCRIPTION_DE, "Zeige aktuelles Arbeitsverzeichnis."},
	/* es */ {IDGS_MON_PWD_DESCRIPTION_ES, "Muestra el directorio de trabajo actual"},
	/* fr */ {IDGS_MON_PWD_DESCRIPTION_FR, "Voir le rÈpertoire en cours."},
	/* hu */ {IDGS_MON_PWD_DESCRIPTION_HU, "Munkakˆnyvt·r mutat·sa."},
	/* it */ {IDGS_MON_PWD_DESCRIPTION_IT, "Mostra la directory di lavoro attuale."},
	/* ko */ {IDGS_MON_PWD_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_PWD_DESCRIPTION_NL, "Toon de huidige werkdirectory."},
	/* pl */ {IDGS_MON_PWD_DESCRIPTION_PL, "Pokaø obecny katalog roboczy."},
	/* ru */ {IDGS_MON_PWD_DESCRIPTION_RU, "æ‚ﬁ—‡–◊ÿ‚Ï ‚’⁄„È„Ó ‡–—ﬁÁ„Ó ‘ÿ‡’⁄‚ﬁ‡ÿÓ."},
	/* sv */ {IDGS_MON_PWD_DESCRIPTION_SV, "Visa aktuell arbetskatalog."},
	/* tr */ {IDGS_MON_PWD_DESCRIPTION_TR, "GeÁerli Áal˝˛ma dizinini gˆster."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_SAVE_DESCRIPTION,    N_("Save the memory from address1 to address2 to the specified file.\nWrite two-byte load address.\nIf device is 0, the file is written to the file system.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_SAVE_DESCRIPTION_DA, "Gem hukommelsen fra  adresse1 til adresse2 til den angivne fil.\nSkriv 2-byte loadadresse i starten af filen.\nHvis enhed er 0 skrives til filsystemet."},
	/* de */ {IDGS_MON_SAVE_DESCRIPTION_DE, "Speichere den Speicher von Adresse1 bis Adresse2 in die angegebene Datei.\nSchreibe eine zwei-Byte Lade Adresse.\nBei Ger‰t 0, wird dei Datei in das Dateisystem geschrieben."},
	/* es */ {IDGS_MON_SAVE_DESCRIPTION_ES, "Graba memoria entre direcciÛn1 y direcciÛn2 al fichero indicado.\nGraba direcciÛn de carga de dos bytes.\nSi el perifÈrico es 0, se graba en el sistema."},
	/* fr */ {IDGS_MON_SAVE_DESCRIPTION_FR, "Sauvegarde la mÈmoire depuis adress1 jusqu'‡ adress2 dans le\nfichier spÈcifiÈ. …crit l'adresse ‡ deux bits de chargement dans\nle fichier. Si le pÈriphÈrique est 0, le fichier est Ècrit dans\nle systËme de fichier."},
	/* hu */ {IDGS_MON_SAVE_DESCRIPTION_HU, "A memÛria cÌm1 Ès cÌm2 kˆzˆtti tartom·ny·nak f·jlba mentÈse.\nA kÈtb·jtos betˆltÈsi cÌmet is kiÌrja.\nHa az eszkˆz 0, a f·jlt a f·jlrendszeren hozza lÈtre."},
	/* it */ {IDGS_MON_SAVE_DESCRIPTION_IT, "Salva la memoria compresa tra indirizzo1 e indirizzo2 sul file\nspecificato.\nScrive l'indirizzo di caricamento su due byte.\nSe la periferica Ë 0, il file Ë scritto sul file system."},
	/* ko */ {IDGS_MON_SAVE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_SAVE_DESCRIPTION_NL, "Sla het geheugen tussen adres1 en adres2 op naar het opgegeven bestand.\nSchrijf een laad adres van twee bytes.\nAls het apparaat 0 is, zal het bestand geschreven worden naar het\nbestandssysteem."},
	/* pl */ {IDGS_MON_SAVE_DESCRIPTION_PL, "Zapisz pamiÍÊ od address1 do address2 do podanego pliku.\nZapisz dwubajtowy adres wczytania.\nJe∂li urz±dzenie jest 0, plik zostaje zapisany do systemu plikÛw."},
	/* ru */ {IDGS_MON_SAVE_DESCRIPTION_RU, "Save the memory from address1 to address2 to the specified file.\nWrite two-byte load address.\nIf device is 0, the file is written to the file system."},
	/* sv */ {IDGS_MON_SAVE_DESCRIPTION_SV, "Spara minnet frÂn adress1 till adress2 till den angivna filen.\nSkriv tvÂbytes inl‰sningsadress.\nOm enheten ‰r 0 skrivs filen till filsystemet."},
	/* tr */ {IDGS_MON_SAVE_DESCRIPTION_TR, "Adres1'den adres2'ye kadar olan bellei belirtilen dosyaya kaydet.\n›ki bytel˝k y¸keme adresini yaz.\nEer ayg˝t 0 ise, dosya dosya sistemine yaz˝l˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_OTHER_COMMANDS,    N_("Other commands:")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_OTHER_COMMANDS_DA, "Andre kommandoer:"},
	/* de */ {IDGS_OTHER_COMMANDS_DE, "Andere Kommandos:"},
	/* es */ {IDGS_OTHER_COMMANDS_ES, "Otros comandos:"},
	/* fr */ {IDGS_OTHER_COMMANDS_FR, "Autres commandes:"},
	/* hu */ {IDGS_OTHER_COMMANDS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_OTHER_COMMANDS_IT, "Altri comandi:"},
	/* ko */ {IDGS_OTHER_COMMANDS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_OTHER_COMMANDS_NL, "Andere commando's:"},
	/* pl */ {IDGS_OTHER_COMMANDS_PL, "Pozosta≥e polecenia:"},
	/* ru */ {IDGS_OTHER_COMMANDS_RU, "¥‡„”ÿ’ ⁄ﬁ‹–›‘Î:"},
	/* sv */ {IDGS_OTHER_COMMANDS_SV, "Andra kommandon:"},
	/* tr */ {IDGS_OTHER_COMMANDS_TR, "Dier komutlar:"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION,    N_("Display the specified number in decimal, hex, octal and binary.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_DA, "Vis det angivne tal som decimal, hexadecimal, oktal og binÊrt."},
	/* de */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_DE, "Zeige die angegebene Zahl in dezimal, hexadezimal, oktal oder bin‰r."},
	/* es */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_ES, "Mostrar el numero indicado en decimal, hex, octal y binario."},
	/* fr */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_FR, "Affiche le nombre spÈcifiÈ en dÈcimal, hexa, octal et binaire."},
	/* hu */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_HU, "A megadott sz·m kiÌr·sa decim·lisan, hexadecim·lisan, okt·lisan Ès bin·risan."},
	/* it */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_IT, "Mostra il numero specificato in decimale, esadecimale, ottale e binario."},
	/* ko */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_NL, "Toon het opgegeven nummer in decimaal, hex, octaal en binair."},
	/* pl */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_PL, "Pokaø dan± liczbÍ w systemie dziesiÍtnym, szesnastkowym, Ûsemkowym i binarnym."},
	/* ru */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_RU, "Display the specified number in decimal, hex, octal and binary."},
	/* sv */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_SV, "Visa det angivna talet decimalt, hexadecimalt, oktalt och bin‰rt."},
	/* tr */ {IDGS_MON_DISPLAY_NUMBER_DESCRIPTION_TR, "Belirtilen say˝y˝ 10'lu, 16'l˝, 8'li ya da 2'li olarak gˆr¸nt¸le."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_CARTFREEZE_DESCRIPTION,    N_("Use cartridge freeze.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_CARTFREEZE_DESCRIPTION_DA, "Brug cartridge frys-funktion."},
	/* de */ {IDGS_MON_CARTFREEZE_DESCRIPTION_DE, "Erweiterungsmodul Freeze benutzen."},
	/* es */ {IDGS_MON_CARTFREEZE_DESCRIPTION_ES, "Usar congelaciÛn cartucho"},
	/* fr */ {IDGS_MON_CARTFREEZE_DESCRIPTION_FR, "Geler la cartouche."},
	/* hu */ {IDGS_MON_CARTFREEZE_DESCRIPTION_HU, "Cartridge fagyaszt·s haszn·lata."},
	/* it */ {IDGS_MON_CARTFREEZE_DESCRIPTION_IT, "Usa freeze della cartuccia."},
	/* ko */ {IDGS_MON_CARTFREEZE_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_CARTFREEZE_DESCRIPTION_NL, "Gebruik cartridge freeze."},
	/* pl */ {IDGS_MON_CARTFREEZE_DESCRIPTION_PL, "Uøyj zamroøenia kartridøa"},
	/* ru */ {IDGS_MON_CARTFREEZE_DESCRIPTION_RU, "Use cartridge freeze."},
	/* sv */ {IDGS_MON_CARTFREEZE_DESCRIPTION_SV, "Anv‰nd insticksmodulens frysfunktion."},
	/* tr */ {IDGS_MON_CARTFREEZE_DESCRIPTION_TR, "Kartu˛ freeze kullan"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_HELP_DESCRIPTION,    N_("If no argument is given, prints out a list of all available commands\nIf an argument is given, prints out specific help for that command.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_HELP_DESCRIPTION_DA, "Hvis der ikke angives et argument vises en liste over mulige kommandoer.\nAngives et argument vises hjÊlp for denne kommando."},
	/* de */ {IDGS_MON_HELP_DESCRIPTION_DE, "Wenn kein Argement angegeben wurde, werden alle Kommandos ausgegeben.\nMit Argument, zeige die Hilfe f¸r das angegebene Kommand."},
	/* es */ {IDGS_MON_HELP_DESCRIPTION_ES, "Si no se muestran argumentos, imprime salida de todos los comandos disponibles\nSi se da un argumento, imprime la ayuda especÌfica para el comando."},
	/* fr */ {IDGS_MON_HELP_DESCRIPTION_FR, "Si aucun argument n'est donnÈ, affiche une liste des commandes disponibles\nSi un argument est donnÈe, affiche l'aide spÈcifique ‡ la commande donnÈe."},
	/* hu */ {IDGS_MON_HELP_DESCRIPTION_HU, "Ha nincs paramÈter, az ˆsszes lÈtezı parancs list·j·t mutatja.\nHa van, az adott parancshoz mutatja a s˙gÛt."},
	/* it */ {IDGS_MON_HELP_DESCRIPTION_IT, "Se non viene specificato alcun argomeno, stampa l'elenco di tutti i\ncomandi disponibili.\nSe viene specificato un argomento, stampa l'aiuto relativo a quel\ncomando."},
	/* ko */ {IDGS_MON_HELP_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_HELP_DESCRIPTION_NL, "Als er geen parameter is opgegeven, zal een lijst van alle commando's\nworden weergegeven. Als er een parameter is opgegeven, zal de help voor\nhet opgegeven commando worden weergegeven."},
	/* pl */ {IDGS_MON_HELP_DESCRIPTION_PL, "Je∂li nie jest podany argument, pokazuje listÍ dostÍpnych poleceÒ.\nJe∂li zosta≥ podany argument, pokazuje pomoc do tego polecenia."},
	/* ru */ {IDGS_MON_HELP_DESCRIPTION_RU, "If no argument is given, prints out a list of all available commands\nIf an argument is given, prints out specific help for that command."},
	/* sv */ {IDGS_MON_HELP_DESCRIPTION_SV, "Om du inte anger nÂgra argument visas alla tillg‰ngliga kommandon.\nOm du anger ett argument visas hj‰lp fˆr det specifika kommandot."},
	/* tr */ {IDGS_MON_HELP_DESCRIPTION_TR, "Eer hiÁ arg¸man verilmemi˛se, t¸m mevcut komutlar˝n listesi yazd˝r˝l˝r\nEer bir arg¸man verilmi˛se, o komuta ˆzel yard˝m bilgisi yazd˝r˝l˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_STRING,    N_("string")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_STRING_DA, "streng"},
	/* de */ {IDGS_STRING_DE, "String"},
	/* es */ {IDGS_STRING_ES, "cadena"},
	/* fr */ {IDGS_STRING_FR, "chaine"},
	/* hu */ {IDGS_STRING_HU, ""},  /* fuzzy */
	/* it */ {IDGS_STRING_IT, "stringa"},
	/* ko */ {IDGS_STRING_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_STRING_NL, "tekenreeks"},
	/* pl */ {IDGS_STRING_PL, "string"},
	/* ru */ {IDGS_STRING_RU, "string"},
	/* sv */ {IDGS_STRING_SV, "str‰ng"},
	/* tr */ {IDGS_STRING_TR, "yaz˝"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_KEYBUF_DESCRIPTION,    N_("Put the specified string into the keyboard buffer.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_KEYBUF_DESCRIPTION_DA, "IndsÊt den angivne streng i tastatur-bufferen."},
	/* de */ {IDGS_MON_KEYBUF_DESCRIPTION_DE, "Definierte Eingabe in Tastaturpuffer bereitstellen."},
	/* es */ {IDGS_MON_KEYBUF_DESCRIPTION_ES, "Introduce la cadena indicada en el b˙fer de teclado."},
	/* fr */ {IDGS_MON_KEYBUF_DESCRIPTION_FR, "Placer la chaÓne spÈcifiÈe dans le tampon clavier."},
	/* hu */ {IDGS_MON_KEYBUF_DESCRIPTION_HU, "A megadott sztring elhelyezÈse a billenty˚zet pufferben."},
	/* it */ {IDGS_MON_KEYBUF_DESCRIPTION_IT, "Metti la stringa specificata nel buffer di tastiera."},
	/* ko */ {IDGS_MON_KEYBUF_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_KEYBUF_DESCRIPTION_NL, "Plaats de opgegeven tekenreeks in de toetsenbordbuffer."},
	/* pl */ {IDGS_MON_KEYBUF_DESCRIPTION_PL, "Umie∂Ê dany ci±g znakÛw w buforze klawiatury"},
	/* ru */ {IDGS_MON_KEYBUF_DESCRIPTION_RU, "Put the specified string into the keyboard buffer."},
	/* sv */ {IDGS_MON_KEYBUF_DESCRIPTION_SV, "L‰gg den angivna str‰ngen i tangentbordsbufferten."},
	/* tr */ {IDGS_MON_KEYBUF_DESCRIPTION_TR, "Klavye arabelleine belirtilen yaz˝y˝ koy."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_PLAYBACK_DESCRIPTION,    N_("Monitor commands from the specified file are read and executed.  This\ncommand stops at the end of file or when a STOP command is read.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_PLAYBACK_DESCRIPTION_DA, "Monitorkommandoer indlÊses fra den angivne fil og udf¯res. Kommandoen\nstopper nÂr slutningen af filen nÂs, eller nÂr der opdages en STOP-kommando."},
	/* de */ {IDGS_MON_PLAYBACK_DESCRIPTION_DE, "Monitor Kommandos aus der angegebenen Datei werden gelesen und ausgef¸hrt.\nDieses Kommando stoppt, wenn das Ende der Datei erreicht wurde, oder das\nKommando STOP gelesen wurde."},
	/* es */ {IDGS_MON_PLAYBACK_DESCRIPTION_ES, "Los comandos de monitor incluidos en el fichero indicado, han sido leÌdos y ejecutados\nEstos comandos se detendr·n al finalizar el fichero o cuando se lea un comando STOP."},
	/* fr */ {IDGS_MON_PLAYBACK_DESCRIPTION_FR, "Les commandes moniteur du fichier spÈcifiÈ sont lues et ÈxÈcutÈes.\nCette commande s'arrÍte ‡ la fin du fichier ou lorsqu'une commande\nSTOP est lue."},
	/* hu */ {IDGS_MON_PLAYBACK_DESCRIPTION_HU, "A monitor parancsokat beolvassa Ès vÈgrehajtja a megadott f·jlbÛl.\nA f·jl vÈgÈig halad, vagy egy beolvasott STOP parancsig."},
	/* it */ {IDGS_MON_PLAYBACK_DESCRIPTION_IT, "I comandi del monitor sono letti ed eseguiti dal file specificato. Questo\ncomando si arresta alla fine del file o quando viene letto il comando\nSTOP."},
	/* ko */ {IDGS_MON_PLAYBACK_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_PLAYBACK_DESCRIPTION_NL, "Monitor commando's van het opgegeven bestand zullen worden gelezen en\nuitgevoerd. Dit commando stopt aan het einde van het bestand of wanneer\neen STOP commando wordt gelezen."},
	/* pl */ {IDGS_MON_PLAYBACK_DESCRIPTION_PL, "Polecenia monitora z podanego pliku s± odczytywane i wykonywane.\nTa komenda zatrzymuje siÍ na koÒcu pliku, b±dº przy odczycie komendy STOP."},
	/* ru */ {IDGS_MON_PLAYBACK_DESCRIPTION_RU, "Monitor commands from the specified file are read and executed.  This\ncommand stops at the end of file or when a STOP command is read."},
	/* sv */ {IDGS_MON_PLAYBACK_DESCRIPTION_SV, "Monitorkommandon l‰ses in och kˆrs frÂn den angivna filen. Kommandot\nstoppar vid filslut eller n‰r ett STOP-kommando l‰ses."},
	/* tr */ {IDGS_MON_PLAYBACK_DESCRIPTION_TR, "Belirtilen dosyadaki monitˆr komutlar˝ okundu ve Áal˝˛t˝r˝ld˝.  Bu\nkomut dosya sonunda ya da STOP komutu okunduunda sonlan˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_EXPRESSION,    N_("expression")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_EXPRESSION_DA, "udtryk"},
	/* de */ {IDGS_EXPRESSION_DE, "Ausdruck"},
	/* es */ {IDGS_EXPRESSION_ES, "expresiÛn"},
	/* fr */ {IDGS_EXPRESSION_FR, "expression"},
	/* hu */ {IDGS_EXPRESSION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_EXPRESSION_IT, "espressione"},
	/* ko */ {IDGS_EXPRESSION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_EXPRESSION_NL, "expressie"},
	/* pl */ {IDGS_EXPRESSION_PL, "expression"},
	/* ru */ {IDGS_EXPRESSION_RU, "expression"},
	/* sv */ {IDGS_EXPRESSION_SV, "uttryck"},
	/* tr */ {IDGS_EXPRESSION_TR, "ifade"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_PRINT_DESCRIPTION,    N_("Evaluate the specified expression and output the result.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_PRINT_DESCRIPTION_DA, "EvaluÈr det angivne udtryk og vis resultatet."},
	/* de */ {IDGS_MON_PRINT_DESCRIPTION_DE, "Evaluiere den angegebenen Ausdruck und zeige das Ergebnis."},
	/* es */ {IDGS_MON_PRINT_DESCRIPTION_ES, "Evaluar la expresiÛn y mostrar los resultados."},
	/* fr */ {IDGS_MON_PRINT_DESCRIPTION_FR, "…value l'expression spÈcifiÈe et affiche le rÈsultat."},
	/* hu */ {IDGS_MON_PRINT_DESCRIPTION_HU, "KiÈrtÈkeli a megadott kifejezÈst Ès kiÌrja az eredmÈnyt."},
	/* it */ {IDGS_MON_PRINT_DESCRIPTION_IT, "Calcola l'espressione specificata e mostra il risultato."},
	/* ko */ {IDGS_MON_PRINT_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_PRINT_DESCRIPTION_NL, "Evalueer de opgegeven expressie en toon het resultaat."},
	/* pl */ {IDGS_MON_PRINT_DESCRIPTION_PL, "Oszacuj okre∂lone wyraøenie i podaj rezultat."},
	/* ru */ {IDGS_MON_PRINT_DESCRIPTION_RU, "Evaluate the specified expression and output the result."},
	/* sv */ {IDGS_MON_PRINT_DESCRIPTION_SV, "Utv‰rdera det angivna uttrycket och visa resultatet."},
	/* tr */ {IDGS_MON_PRINT_DESCRIPTION_TR, "Belirtilen ifadeyi deerlendir ve sonucun Á˝kt˝s˝n˝ ver."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_RECORD_DESCRIPTION,    N_("After this command, all commands entered are written to the specified\nfile until the STOP command is entered.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_RECORD_DESCRIPTION_DA, "Efter denne kommando skrives alle kommandoer til den angivne fil,\nindtil der angives en STOP-kommando."},
	/* de */ {IDGS_MON_RECORD_DESCRIPTION_DE, "Nach diesem Kommando werden alle eingebenen Kommands in die angegebene\nDatei gespeicher solang bis das Kommando STOP eingegeben wurde."},
	/* es */ {IDGS_MON_RECORD_DESCRIPTION_ES, "DespuÈs de este comando, todos los posteriormente introducidos se grabaran\nen el fichero indicado hasta que se entre el comando STOP."},
	/* fr */ {IDGS_MON_RECORD_DESCRIPTION_FR, "AprËs cette commande, toutes les commandes entrÈes sont Ècrites au\nfichier spÈcifiÈ jusqu'‡ ce qu'une commande STOP soit entrÈe."},
	/* hu */ {IDGS_MON_RECORD_DESCRIPTION_HU, "Ezut·n a parancs ut·n az ˆsszes parancsot a megadott f·jlba\nÌrja, amÌg STOP parancs nem Èrkezik."},
	/* it */ {IDGS_MON_RECORD_DESCRIPTION_IT, "Dopo questo comando, tutti i comandi inseriti saranno scritti sul file\nspecificato fino a quando verr‡ inserito il comando STOP."},
	/* ko */ {IDGS_MON_RECORD_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_RECORD_DESCRIPTION_NL, "Na dit commando zullen alle opgegeven commando's worden geschreven naar\nhet opgegeven bestand totdat het STOP commando wordt opgegeven."},
	/* pl */ {IDGS_MON_RECORD_DESCRIPTION_PL, "Po tym poleceniu wszystkie wprowadzone polecenia s± zapisywane\ndo podanego pliku, pÛki nie zostanie wprowadzone polecenie STOP."},
	/* ru */ {IDGS_MON_RECORD_DESCRIPTION_RU, "After this command, all commands entered are written to the specified\nfile until the STOP command is entered."},
	/* sv */ {IDGS_MON_RECORD_DESCRIPTION_SV, "Efter detta kommando kommer alla kommandon som anges att skrivas till\nden angivna filen tills ett STOP-kommando anges."},
	/* tr */ {IDGS_MON_RECORD_DESCRIPTION_TR, "Bu komuttan sonra, t¸m girilen komutlar ve STOP komutunu gˆrene dek\nbelirtilen dosyaya yaz˝lacakt˝r."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_RESOURCE,    N_("resource")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_RESOURCE_DA, "resource"},
	/* de */ {IDGS_RESOURCE_DE, "Ressource"},
	/* es */ {IDGS_RESOURCE_ES, "fuente"},
	/* fr */ {IDGS_RESOURCE_FR, "ressource"},
	/* hu */ {IDGS_RESOURCE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_RESOURCE_IT, "risorsa"},
	/* ko */ {IDGS_RESOURCE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_RESOURCE_NL, "bron"},
	/* pl */ {IDGS_RESOURCE_PL, "resource"},
	/* ru */ {IDGS_RESOURCE_RU, "resource"},
	/* sv */ {IDGS_RESOURCE_SV, "resurs"},
	/* tr */ {IDGS_RESOURCE_TR, "kaynak"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_RESOURCEGET_DESCRIPTION,    N_("Displays the value of the resource.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_RESOURCEGET_DESCRIPTION_DA, "Viser vÊrdien for ressourcen."},
	/* de */ {IDGS_MON_RESOURCEGET_DESCRIPTION_DE, "Zeige den Werde der Ressource."},
	/* es */ {IDGS_MON_RESOURCEGET_DESCRIPTION_ES, "Mostrar el valor del recurso"},
	/* fr */ {IDGS_MON_RESOURCEGET_DESCRIPTION_FR, "Affiche la valeur de la ressource."},
	/* hu */ {IDGS_MON_RESOURCEGET_DESCRIPTION_HU, "Az erıforr·s ÈrtÈkÈnek kiÌr·sa."},
	/* it */ {IDGS_MON_RESOURCEGET_DESCRIPTION_IT, "Mostra il valore della risorsa."},
	/* ko */ {IDGS_MON_RESOURCEGET_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_RESOURCEGET_DESCRIPTION_NL, "Toont de waarde van de bron."},
	/* pl */ {IDGS_MON_RESOURCEGET_DESCRIPTION_PL, "Pokazuje warto∂Ê zasobu"},
	/* ru */ {IDGS_MON_RESOURCEGET_DESCRIPTION_RU, "Displays the value of the resource."},
	/* sv */ {IDGS_MON_RESOURCEGET_DESCRIPTION_SV, "Visar v‰rdet fˆr resursen."},
	/* tr */ {IDGS_MON_RESOURCEGET_DESCRIPTION_TR, "Kayna˝n deerini gˆr¸nt¸ler."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_VALUE,    N_("value")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_VALUE_DA, "vÊrdi"},
	/* de */ {IDGS_VALUE_DE, "Wert"},
	/* es */ {IDGS_VALUE_ES, "valor"},
	/* fr */ {IDGS_VALUE_FR, "valeur"},
	/* hu */ {IDGS_VALUE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_VALUE_IT, "valore"},
	/* ko */ {IDGS_VALUE_KO, "∞°ƒ°"},
	/* nl */ {IDGS_VALUE_NL, "waarde"},
	/* pl */ {IDGS_VALUE_PL, "warto∂Ê"},
	/* ru */ {IDGS_VALUE_RU, "value"},
	/* sv */ {IDGS_VALUE_SV, "v‰rde"},
	/* tr */ {IDGS_VALUE_TR, "deer"},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_RESOURCESET_DESCRIPTION,    N_("Sets the value of the resource.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_RESOURCESET_DESCRIPTION_DA, "SÊtter vÊrdien for ressourcen."},
	/* de */ {IDGS_MON_RESOURCESET_DESCRIPTION_DE, "Setze den Werde der Ressource."},
	/* es */ {IDGS_MON_RESOURCESET_DESCRIPTION_ES, "Seleccionar el valor de los recursos"},
	/* fr */ {IDGS_MON_RESOURCESET_DESCRIPTION_FR, "DÈfinir la valeur de la ressource."},
	/* hu */ {IDGS_MON_RESOURCESET_DESCRIPTION_HU, "Az erıforr·s ÈrtÈkÈnek be·llÌt·sa."},
	/* it */ {IDGS_MON_RESOURCESET_DESCRIPTION_IT, "Imposta il valore della risorsa."},
	/* ko */ {IDGS_MON_RESOURCESET_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_RESOURCESET_DESCRIPTION_NL, "Stel de waarde in van de bron."},
	/* pl */ {IDGS_MON_RESOURCESET_DESCRIPTION_PL, "Ustawia warto∂Ê zasobu."},
	/* ru */ {IDGS_MON_RESOURCESET_DESCRIPTION_RU, "Sets the value of the resource."},
	/* sv */ {IDGS_MON_RESOURCESET_DESCRIPTION_SV, "St‰ller in ett v‰rde fˆr resursen."},
	/* tr */ {IDGS_MON_RESOURCESET_DESCRIPTION_TR, "Kayna˝n deerini belirler."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION,    N_("Loads resources from file.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_DA, "IndlÊs indstillinger fra fil."},
	/* de */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_DE, "Einstellungen aus Datei laden"},
	/* es */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_ES, "Cargar recursos desde fichero."},
	/* fr */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_FR, "Charger ressources depuis fichier."},
	/* hu */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_IT, "Carica le risorse da file."},
	/* ko */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_NL, "Laad instellingen van bestand"},
	/* pl */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_PL, "Wczytaj zasoby z pliku."},
	/* ru */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_SV, "L‰ser in resurser frÂn fil."},
	/* tr */ {IDGS_MON_LOAD_RESOURCES_DESCRIPTION_TR, "Kaynaklar˝ dosyadan y¸kler."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION,    N_("Saves resources to file.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_DA, "Gem indstillinger til fil."},
	/* de */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_DE, "Einstellungen speichern unter"},
	/* es */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_ES, "Grabar recursos a fichero."},
	/* fr */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_FR, "Enregistrer les ressources dans un fichier."},
	/* hu */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_IT, "Salva le risorse su file."},
	/* ko */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_NL, "Sla instellingen op naar bestand"},
	/* pl */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_PL, "Zapisz zasoby do pliku."},
	/* ru */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_SV, "Sparar resurser till fil."},
	/* tr */ {IDGS_MON_SAVE_RESOURCES_DESCRIPTION_TR, "Kaynaklar˝ dosyaya kaydeder."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_STOP_DESCRIPTION,    N_("Stop recording commands.  See `record'.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_STOP_DESCRIPTION_DA, "Afslut optagelse. Se \"record\"."},
	/* de */ {IDGS_MON_STOP_DESCRIPTION_DE, "Stoppe die Aufnahme von Kommandos. Siehe auch `Record'."},
	/* es */ {IDGS_MON_STOP_DESCRIPTION_ES, "Detener comando de grabaciÛn. Ver 'grabaciÛn'"},
	/* fr */ {IDGS_MON_STOP_DESCRIPTION_FR, "ArrÍte l'enregistrement de commandes. Voir `record'."},
	/* hu */ {IDGS_MON_STOP_DESCRIPTION_HU, "A parancsok rˆgzÌtÈsÈnek befejezÈse.  L·sd `record'"},
	/* it */ {IDGS_MON_STOP_DESCRIPTION_IT, "Ferma il comando di registrazione. Guarda `registra'."},
	/* ko */ {IDGS_MON_STOP_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_STOP_DESCRIPTION_NL, "Stop de opname van commando's. Zie 'record'."},
	/* pl */ {IDGS_MON_STOP_DESCRIPTION_PL, "PrzestaÒ zapisywaÊ polecenia. Zobacz `record'."},
	/* ru */ {IDGS_MON_STOP_DESCRIPTION_RU, "Stop recording commands.  See `record'."},
	/* sv */ {IDGS_MON_STOP_DESCRIPTION_SV, "Sluta spela in kommandon. Se \"record\"."},
	/* tr */ {IDGS_MON_STOP_DESCRIPTION_TR, "Kay˝t komutlar˝n˝ durdur.  `record'a bak."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_SCREENSHOT_DESCRIPTION,    N_("Take a screenshot. Format is:\ndefault = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_SCREENSHOT_DESCRIPTION_DA, "Gem et screenshot. Format er:\nstandard = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* de */ {IDGS_MON_SCREENSHOT_DESCRIPTION_DE, "Erzeuge Screenshot. Format ist:\nStandard = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* es */ {IDGS_MON_SCREENSHOT_DESCRIPTION_ES, "Tomar Captura de pantalla. El formato es:\npor defecto = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* fr */ {IDGS_MON_SCREENSHOT_DESCRIPTION_FR, "Prend un imprime-Ècran. Les formats sont:\nDÈfaut = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* hu */ {IDGS_MON_SCREENSHOT_DESCRIPTION_HU, "KÈpernyıkÈp kÈszÌtÈse. A form·tum lehet:\nalap = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* it */ {IDGS_MON_SCREENSHOT_DESCRIPTION_IT, "Cattura schermo. Il formato Ë:\npredefinito = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* ko */ {IDGS_MON_SCREENSHOT_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_SCREENSHOT_DESCRIPTION_NL, "Sla een schermafdruk op. Formaat is:\nstandaard = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* pl */ {IDGS_MON_SCREENSHOT_DESCRIPTION_PL, "ZrÛb zrzut ekranu. Format:\ndomy∂lny = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* ru */ {IDGS_MON_SCREENSHOT_DESCRIPTION_RU, "Take a screenshot. Format is:\ndefault = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* sv */ {IDGS_MON_SCREENSHOT_DESCRIPTION_SV, "Ta en sk‰rmbild. Format ‰r:\nstandard = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
	/* tr */ {IDGS_MON_SCREENSHOT_DESCRIPTION_TR, "Ekran gˆr¸nt¸s¸ al. BiÁim:\nvarsay˝lan = BMP, 1 = PCX, 2 = PNG, 3 = GIF, 4 = IFF."},
#endif
	
	/* monitor/mon_command.c */
	/* en */ {IDGS_MON_TAPECTRL_DESCRIPTION,    N_("Control the datasette. Valid commands:\n0 = stop, 1 = start, 2 = forward, 3 = rewind, 4 = record,\n5 = reset, 6 = reset counter.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MON_TAPECTRL_DESCRIPTION_DA, "Styr datasette. Lovlige kommandoer:\n0 = stop, 1 = start, 2 = spol frem, 3 = spol tilbage,\n4 = optag, 5 = reset, 6 = nulstil tÊller."},
	/* de */ {IDGS_MON_TAPECTRL_DESCRIPTION_DE, "Steuere die Datasette. G¸ltige Kommandos sind:\n0 = Stop, 1 = Start, 2 = Forw‰rts, 3 = R¸ckw‰rts, 4 = Aufnahme,\n5 = Reset, 6 = Reset Z‰hler."},
	/* es */ {IDGS_MON_TAPECTRL_DESCRIPTION_ES, "Control unidad cinta, comandos validos:\n0 = alto, 1 = inicio, 2 = avance, 3 = rebobinar, 4 = grabar,\n5 = reinicio, 6 = reiniciar contador."},
	/* fr */ {IDGS_MON_TAPECTRL_DESCRIPTION_FR, "ContrÙle du datassette. Les commandes valides sont :\n0 = STOP, 1 = PLAY, 2 = Avancer, 3 = Reculer, 4 = REC,\n5 = RÈinitialiser le ruban, 6 = RÈinitialiser le compteur."},
	/* hu */ {IDGS_MON_TAPECTRL_DESCRIPTION_HU, "Szalagos egysÈg vezÈrlÈse. …rvÈnyes parancsok:\n0 = start, 1 = indÌt, 2 = elıre, 3 = vissza, 4 = felvÈtel,\n5 = ˙jraindÌt, 6 = sz·ml·lÛt null·z"},
	/* it */ {IDGS_MON_TAPECTRL_DESCRIPTION_IT, "Controlla il registratore. Comandi validi:\n0 = ferma, 1 = avvia, 2 = avanti, 3 = indietro, 4 = registra,\n5 = reset, 6 = reset contatore."},
	/* ko */ {IDGS_MON_TAPECTRL_DESCRIPTION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MON_TAPECTRL_DESCRIPTION_NL, "Geef datasette commando's. Geldige commando's:\n0 = stop, 1 = start, 2 = vooruit, 3 = terug, 4 = opnemen,\n5 = reset, 6 = reset teller."},
	/* pl */ {IDGS_MON_TAPECTRL_DESCRIPTION_PL, "Kontrola magnetofonu. W≥a∂ciwe polecenia:\n0 = stop, 1 = start, 2 = naprzÛd, 3 = przewiÒ, 4 = nagrywaj,\n5 = reset, 6 = reset licznika."},
	/* ru */ {IDGS_MON_TAPECTRL_DESCRIPTION_RU, "Control the datasette. Valid commands:\n0 = stop, 1 = start, 2 = forward, 3 = rewind, 4 = record,\n5 = reset, 6 = reset counter."},
	/* sv */ {IDGS_MON_TAPECTRL_DESCRIPTION_SV, "Styr bandspelaren. Giltiga kommandon:\n0 = stoppa, 1 = starta, 2 = spola framÂt, 3 = spola bakÂt,\n4 = spela in, 5 = Âterst‰ll, 6 = nollst‰ll r‰knare."},
	/* tr */ {IDGS_MON_TAPECTRL_DESCRIPTION_TR, "Teybi kontrol et. GeÁerli komutlar:\n0 = durdur, 1 = ba˛lat, 2 = ileri, 3 = geri, 4 = kay˝t,\n5 = reset, 6 = reset sayac˝."},
#endif
	
	/* joyport.c */
	/* en */ {IDGS_CONTROL_PORT,    N_("Control port")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CONTROL_PORT_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CONTROL_PORT_DE, "Control Port"},
	/* es */ {IDGS_CONTROL_PORT_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CONTROL_PORT_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CONTROL_PORT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CONTROL_PORT_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CONTROL_PORT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CONTROL_PORT_NL, "Spel connector"},
	/* pl */ {IDGS_CONTROL_PORT_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CONTROL_PORT_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CONTROL_PORT_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CONTROL_PORT_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_CONTROL_PORT_1,    N_("Control port 1")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CONTROL_PORT_1_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CONTROL_PORT_1_DE, "Control Port 1"},
	/* es */ {IDGS_CONTROL_PORT_1_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CONTROL_PORT_1_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CONTROL_PORT_1_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CONTROL_PORT_1_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CONTROL_PORT_1_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CONTROL_PORT_1_NL, "Spel connector 1"},
	/* pl */ {IDGS_CONTROL_PORT_1_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CONTROL_PORT_1_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CONTROL_PORT_1_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CONTROL_PORT_1_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_CONTROL_PORT_2,    N_("Control port 2")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CONTROL_PORT_2_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CONTROL_PORT_2_DE, "Control Port 2"},
	/* es */ {IDGS_CONTROL_PORT_2_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CONTROL_PORT_2_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CONTROL_PORT_2_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CONTROL_PORT_2_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CONTROL_PORT_2_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CONTROL_PORT_2_NL, "Spel connector 2"},
	/* pl */ {IDGS_CONTROL_PORT_2_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CONTROL_PORT_2_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CONTROL_PORT_2_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CONTROL_PORT_2_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_SIDCARD_CONTROL_PORT,    N_("SIDCard control port")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SIDCARD_CONTROL_PORT_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SIDCARD_CONTROL_PORT_DE, "SIDCard Control Port"},
	/* es */ {IDGS_SIDCARD_CONTROL_PORT_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SIDCARD_CONTROL_PORT_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SIDCARD_CONTROL_PORT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SIDCARD_CONTROL_PORT_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SIDCARD_CONTROL_PORT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SIDCARD_CONTROL_PORT_NL, "SIDCard spel connector"},
	/* pl */ {IDGS_SIDCARD_CONTROL_PORT_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SIDCARD_CONTROL_PORT_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SIDCARD_CONTROL_PORT_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SIDCARD_CONTROL_PORT_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1,    N_("Userport joystick adapter port 1")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_DE, "Userport Joystick Adapter Port 1"},
	/* es */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_NL, "Userport joystick adapter poort 1"},
	/* pl */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_JOY_ADAPTER_PORT_1_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2,    N_("Userport joystick adapter port 2")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_DE, "Userport Joystick Adapter Port 2"},
	/* es */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_NL, "Userport joystick adapter poort 2"},
	/* pl */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_JOY_ADAPTER_PORT_2_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_SELECTED_PORT_NOT_PRESENT,    N_("Selected port (%d) is not present on this emulator")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SELECTED_PORT_NOT_PRESENT_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SELECTED_PORT_NOT_PRESENT_DE, "Ausgew‰hltes Port (%d) ist in diesem Emulator nicht verf¸gbar"},
	/* es */ {IDGS_SELECTED_PORT_NOT_PRESENT_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SELECTED_PORT_NOT_PRESENT_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SELECTED_PORT_NOT_PRESENT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SELECTED_PORT_NOT_PRESENT_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SELECTED_PORT_NOT_PRESENT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SELECTED_PORT_NOT_PRESENT_NL, "Geselecteerde poort (%d) is niet aanwezig in deze emulator"},
	/* pl */ {IDGS_SELECTED_PORT_NOT_PRESENT_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SELECTED_PORT_NOT_PRESENT_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SELECTED_PORT_NOT_PRESENT_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SELECTED_PORT_NOT_PRESENT_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG,    N_("Selected control port device %d is not registered")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_DE, "Ausgew‰hltes Port Ger‰t %d ist nicht registriert"},
	/* es */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_NL, "Geselecteerde spel connector apparaat %d is niet geregistreerd"},
	/* pl */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SELECTED_JOYPORT_DEV_NOT_REG_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED,    N_("Selected control port device %s on %s is already attached to %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_DE, "Ausgew‰hltes Control Port Ger‰t %s auf Port %s ist schon in Port %s in Verwendung"},
	/* es */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_NL, "Geselecteerde spel connector apparaat %s op %s is al gekoppeld aan %s"},
	/* pl */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SELECTED_JOYPORT_DEV_ALREADY_ATTACHED_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES,    N_("Selected control port device %s on %s uses same host input resource (%s) as the device attached to %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_DE, "Ausgew‰hltes Control Port Ger‰t %s auf Port %s benutzt die gleiche Host\nInput Resource (%s) wie das auf Port %s benutzte Ger‰t"},
	/* es */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_NL, "Geselecteerd spel connector apparaat %s op %s gebruikt dezelfde fysieke invoerinstelling (%s) als het apparaat dat gekoppeld is aan %s"},
	/* pl */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SELECTED_JOYPORT_SAME_INPUT_RES_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT,    N_("Selected control port device %s cannot be attached to %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_DE, "Ausgew‰hltes Control Port Ger‰t %s kann nicht auf Port %s verwendet werden"},
	/* es */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_NL, "Geselecteerd spel connector apparaat %s kan niet worden gekoppeld aan %s"},
	/* pl */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SELECTED_DEVICE_NOT_THIS_PORT_TR, ""},  /* fuzzy */
#endif
	
	
	/* joyport.c */
	/* en */ {IDGS_HOST_MOUSE,    N_("host mouse")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_HOST_MOUSE_DA, ""},  /* fuzzy */
	/* de */ {IDGS_HOST_MOUSE_DE, "Host Maus"},
	/* es */ {IDGS_HOST_MOUSE_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_HOST_MOUSE_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_HOST_MOUSE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_HOST_MOUSE_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_HOST_MOUSE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_HOST_MOUSE_NL, "fysieke muis"},
	/* pl */ {IDGS_HOST_MOUSE_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_HOST_MOUSE_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_HOST_MOUSE_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_HOST_MOUSE_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_HOST_SAMPLER,    N_("host sampler (audio input device)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_HOST_SAMPLER_DA, ""},  /* fuzzy */
	/* de */ {IDGS_HOST_SAMPLER_DE, "Host Sampler (Audio Input Ger‰t)"},
	/* es */ {IDGS_HOST_SAMPLER_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_HOST_SAMPLER_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_HOST_SAMPLER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_HOST_SAMPLER_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_HOST_SAMPLER_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_HOST_SAMPLER_NL, "fysieke sampler (geluidsinvoer apparaat)"},
	/* pl */ {IDGS_HOST_SAMPLER_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_HOST_SAMPLER_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_HOST_SAMPLER_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_HOST_SAMPLER_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_NONE,    N_("None")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_NONE_DA, "Ingen"},
	/* de */ {IDGS_NONE_DE, "Kein"},
	/* es */ {IDGS_NONE_ES, "No"},
	/* fr */ {IDGS_NONE_FR, "Aucun"},
	/* hu */ {IDGS_NONE_HU, "Nincs"},
	/* it */ {IDGS_NONE_IT, "Nessuno"},
	/* ko */ {IDGS_NONE_KO, "æ¯¿Ω"},
	/* nl */ {IDGS_NONE_NL, "Geen"},
	/* pl */ {IDGS_NONE_PL, "Brak"},
	/* ru */ {IDGS_NONE_RU, "None"},
	/* sv */ {IDGS_NONE_SV, "Ingen"},
	/* tr */ {IDGS_NONE_TR, "Yok"},
#endif
	
	/* joystick.c */
	/* en */ {IDGS_JOYSTICK,    N_("Joystick")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_JOYSTICK_DA, "Joystick"},
	/* de */ {IDGS_JOYSTICK_DE, "Joystick"},
	/* es */ {IDGS_JOYSTICK_ES, "Joystick"},
	/* fr */ {IDGS_JOYSTICK_FR, "Joystick"},
	/* hu */ {IDGS_JOYSTICK_HU, "Botkorm·ny"},
	/* it */ {IDGS_JOYSTICK_IT, "Joystick"},
	/* ko */ {IDGS_JOYSTICK_KO, "¡∂¿ÃΩ∫∆Ω"},
	/* nl */ {IDGS_JOYSTICK_NL, "Joystick"},
	/* pl */ {IDGS_JOYSTICK_PL, "Joystick"},
	/* ru */ {IDGS_JOYSTICK_RU, "¥÷ﬁŸ·‚ÿ⁄"},
	/* sv */ {IDGS_JOYSTICK_SV, "Styrspak"},
	/* tr */ {IDGS_JOYSTICK_TR, "Joystick"},
#endif
	
	/* mouse.c */
	/* en */ {IDGS_PADDLES,    N_("Paddles")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_PADDLES_DA, "Paddles"},
	/* de */ {IDGS_PADDLES_DE, "Paddles"},
	/* es */ {IDGS_PADDLES_ES, "Paddles"},
	/* fr */ {IDGS_PADDLES_FR, "Paddles"},
	/* hu */ {IDGS_PADDLES_HU, ""},  /* fuzzy */
	/* it */ {IDGS_PADDLES_IT, "Paddle"},
	/* ko */ {IDGS_PADDLES_KO, "∆–µÈ"},
	/* nl */ {IDGS_PADDLES_NL, "Paddles"},
	/* pl */ {IDGS_PADDLES_PL, "PokrÍt≥a steruj±ce"},
	/* ru */ {IDGS_PADDLES_RU, "Paddles"},
	/* sv */ {IDGS_PADDLES_SV, "Paddlar"},
	/* tr */ {IDGS_PADDLES_TR, "Paddlelar"},
#endif
	
	/* mouse.c */
	/* en */ {IDGS_MOUSE_1351,    N_("Mouse (1351)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MOUSE_1351_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MOUSE_1351_DE, "Maus (1351)"},
	/* es */ {IDGS_MOUSE_1351_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MOUSE_1351_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MOUSE_1351_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MOUSE_1351_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MOUSE_1351_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MOUSE_1351_NL, "Muis (1351)"},
	/* pl */ {IDGS_MOUSE_1351_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_MOUSE_1351_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MOUSE_1351_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MOUSE_1351_TR, ""},  /* fuzzy */
#endif
	
	/* mouse.c */
	/* en */ {IDGS_MOUSE_NEOS,    N_("Mouse (NEOS)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MOUSE_NEOS_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MOUSE_NEOS_DE, "Maus (NEOS)"},
	/* es */ {IDGS_MOUSE_NEOS_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MOUSE_NEOS_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MOUSE_NEOS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MOUSE_NEOS_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MOUSE_NEOS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MOUSE_NEOS_NL, "Muis (NEOS)"},
	/* pl */ {IDGS_MOUSE_NEOS_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_MOUSE_NEOS_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MOUSE_NEOS_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MOUSE_NEOS_TR, ""},  /* fuzzy */
#endif
	
	/* mouse.c */
	/* en */ {IDGS_MOUSE_AMIGA,    N_("Mouse (Amiga)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MOUSE_AMIGA_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MOUSE_AMIGA_DE, "Maus (Amiga)"},
	/* es */ {IDGS_MOUSE_AMIGA_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MOUSE_AMIGA_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MOUSE_AMIGA_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MOUSE_AMIGA_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MOUSE_AMIGA_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MOUSE_AMIGA_NL, "Muis (Amiga)"},
	/* pl */ {IDGS_MOUSE_AMIGA_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_MOUSE_AMIGA_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MOUSE_AMIGA_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MOUSE_AMIGA_TR, ""},  /* fuzzy */
#endif
	
	/* mouse.c */
	/* en */ {IDGS_MOUSE_CX22,    N_("Mouse (CX-22)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MOUSE_CX22_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MOUSE_CX22_DE, "Maus (CX-22)"},
	/* es */ {IDGS_MOUSE_CX22_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MOUSE_CX22_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MOUSE_CX22_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MOUSE_CX22_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MOUSE_CX22_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MOUSE_CX22_NL, "Muis (CX-22)"},
	/* pl */ {IDGS_MOUSE_CX22_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_MOUSE_CX22_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MOUSE_CX22_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MOUSE_CX22_TR, ""},  /* fuzzy */
#endif
	
	/* mouse.c */
	/* en */ {IDGS_MOUSE_ATARI_ST,    N_("Mouse (Atari ST)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MOUSE_ATARI_ST_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MOUSE_ATARI_ST_DE, "Maus (Atari ST)"},
	/* es */ {IDGS_MOUSE_ATARI_ST_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MOUSE_ATARI_ST_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MOUSE_ATARI_ST_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MOUSE_ATARI_ST_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MOUSE_ATARI_ST_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MOUSE_ATARI_ST_NL, "Muis (Atari ST)"},
	/* pl */ {IDGS_MOUSE_ATARI_ST_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_MOUSE_ATARI_ST_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MOUSE_ATARI_ST_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MOUSE_ATARI_ST_TR, ""},  /* fuzzy */
#endif
	
	/* mouse.c */
	/* en */ {IDGS_MOUSE_SMART,    N_("Mouse (SmartMouse)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MOUSE_SMART_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MOUSE_SMART_DE, "Maus (SmartMouse)"},
	/* es */ {IDGS_MOUSE_SMART_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MOUSE_SMART_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MOUSE_SMART_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MOUSE_SMART_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MOUSE_SMART_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MOUSE_SMART_NL, "Muis (SmartMouse)"},
	/* pl */ {IDGS_MOUSE_SMART_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_MOUSE_SMART_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MOUSE_SMART_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MOUSE_SMART_TR, ""},  /* fuzzy */
#endif
	
	/* mouse.c */
	/* en */ {IDGS_MOUSE_MICROMYS,    N_("Mouse (Micromys)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MOUSE_MICROMYS_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MOUSE_MICROMYS_DE, "Maus (Micromys)"},
	/* es */ {IDGS_MOUSE_MICROMYS_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MOUSE_MICROMYS_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MOUSE_MICROMYS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MOUSE_MICROMYS_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MOUSE_MICROMYS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MOUSE_MICROMYS_NL, "Muis (Micromys)"},
	/* pl */ {IDGS_MOUSE_MICROMYS_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_MOUSE_MICROMYS_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MOUSE_MICROMYS_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MOUSE_MICROMYS_TR, ""},  /* fuzzy */
#endif
	
	/* mouse.c */
	/* en */ {IDGS_KOALAPAD,    N_("KoalaPad")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_KOALAPAD_DA, ""},  /* fuzzy */
	/* de */ {IDGS_KOALAPAD_DE, "KoalaPad"},
	/* es */ {IDGS_KOALAPAD_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_KOALAPAD_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_KOALAPAD_HU, ""},  /* fuzzy */
	/* it */ {IDGS_KOALAPAD_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_KOALAPAD_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_KOALAPAD_NL, "KoalaPad"},
	/* pl */ {IDGS_KOALAPAD_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_KOALAPAD_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_KOALAPAD_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_KOALAPAD_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/sampler2bit.c */
	/* en */ {IDGS_SAMPLER_2BIT,    N_("Sampler (2bit)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SAMPLER_2BIT_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SAMPLER_2BIT_DE, "Sampler (2bit)"},
	/* es */ {IDGS_SAMPLER_2BIT_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SAMPLER_2BIT_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SAMPLER_2BIT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SAMPLER_2BIT_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SAMPLER_2BIT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SAMPLER_2BIT_NL, "Sampler (2bit)"},
	/* pl */ {IDGS_SAMPLER_2BIT_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SAMPLER_2BIT_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SAMPLER_2BIT_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SAMPLER_2BIT_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/sampler4bit.c */
	/* en */ {IDGS_SAMPLER_4BIT,    N_("Sampler (4bit)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SAMPLER_4BIT_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SAMPLER_4BIT_DE, "Sampler (4bit)"},
	/* es */ {IDGS_SAMPLER_4BIT_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SAMPLER_4BIT_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SAMPLER_4BIT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SAMPLER_4BIT_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SAMPLER_4BIT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SAMPLER_4BIT_NL, "Sampler (4bit)"},
	/* pl */ {IDGS_SAMPLER_4BIT_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SAMPLER_4BIT_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SAMPLER_4BIT_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SAMPLER_4BIT_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/bbrtc.c */
	/* en */ {IDGS_BBRTC,    N_("BBRTC")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_BBRTC_DA, ""},  /* fuzzy */
	/* de */ {IDGS_BBRTC_DE, "BBRTC"},
	/* es */ {IDGS_BBRTC_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_BBRTC_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_BBRTC_HU, ""},  /* fuzzy */
	/* it */ {IDGS_BBRTC_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_BBRTC_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_BBRTC_NL, "BBRTC"},
	/* pl */ {IDGS_BBRTC_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_BBRTC_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_BBRTC_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_BBRTC_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/paperclip64.c */
	/* en */ {IDGS_PAPERCLIP64_DONGLE,    N_("Paperclip64 dongle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_PAPERCLIP64_DONGLE_DA, ""},  /* fuzzy */
	/* de */ {IDGS_PAPERCLIP64_DONGLE_DE, "Paperclip64 Dongle"},
	/* es */ {IDGS_PAPERCLIP64_DONGLE_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_PAPERCLIP64_DONGLE_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_PAPERCLIP64_DONGLE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_PAPERCLIP64_DONGLE_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_PAPERCLIP64_DONGLE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_PAPERCLIP64_DONGLE_NL, "Paperclip64 dongle"},
	/* pl */ {IDGS_PAPERCLIP64_DONGLE_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_PAPERCLIP64_DONGLE_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_PAPERCLIP64_DONGLE_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_PAPERCLIP64_DONGLE_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/coplin_keypad.c */
	/* en */ {IDGS_COPLIN_KEYPAD,    N_("Coplin Keypad")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_COPLIN_KEYPAD_DA, ""},  /* fuzzy */
	/* de */ {IDGS_COPLIN_KEYPAD_DE, "Coplin Keypad"},
	/* es */ {IDGS_COPLIN_KEYPAD_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_COPLIN_KEYPAD_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_COPLIN_KEYPAD_HU, ""},  /* fuzzy */
	/* it */ {IDGS_COPLIN_KEYPAD_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_COPLIN_KEYPAD_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_COPLIN_KEYPAD_NL, "Coplin toetsenblok"},
	/* pl */ {IDGS_COPLIN_KEYPAD_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_COPLIN_KEYPAD_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_COPLIN_KEYPAD_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_COPLIN_KEYPAD_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/script64_dongle.c */
	/* en */ {IDGS_SCRIPT64_DONGLE,    N_("Script 64 dongle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SCRIPT64_DONGLE_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SCRIPT64_DONGLE_DE, ""},  /* fuzzy */
	/* es */ {IDGS_SCRIPT64_DONGLE_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SCRIPT64_DONGLE_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SCRIPT64_DONGLE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SCRIPT64_DONGLE_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SCRIPT64_DONGLE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SCRIPT64_DONGLE_NL, "Script 64 dongle"},
	/* pl */ {IDGS_SCRIPT64_DONGLE_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SCRIPT64_DONGLE_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SCRIPT64_DONGLE_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SCRIPT64_DONGLE_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/vizawrite64_dongle.c */
	/* en */ {IDGS_VIZAWRITE64_DONGLE,    N_("VizaWrite 64 dongle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_VIZAWRITE64_DONGLE_DA, ""},  /* fuzzy */
	/* de */ {IDGS_VIZAWRITE64_DONGLE_DE, ""},  /* fuzzy */
	/* es */ {IDGS_VIZAWRITE64_DONGLE_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_VIZAWRITE64_DONGLE_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_VIZAWRITE64_DONGLE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_VIZAWRITE64_DONGLE_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_VIZAWRITE64_DONGLE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_VIZAWRITE64_DONGLE_NL, "VizaWrite 64 dongle"},
	/* pl */ {IDGS_VIZAWRITE64_DONGLE_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_VIZAWRITE64_DONGLE_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_VIZAWRITE64_DONGLE_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_VIZAWRITE64_DONGLE_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/cx21.c */
	/* en */ {IDGS_CX21,    N_("Atari CX21 keypad")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CX21_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CX21_DE, "Atari CX-21 Keypad"},
	/* es */ {IDGS_CX21_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CX21_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CX21_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CX21_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CX21_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CX21_NL, "Atari CX21 toetsenblok"},
	/* pl */ {IDGS_CX21_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CX21_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CX21_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CX21_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/cx85.c */
	/* en */ {IDGS_CX85,    N_("Atari CX85 keypad")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CX85_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CX85_DE, "Atari CX85 keypad"},
	/* es */ {IDGS_CX85_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CX85_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CX85_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CX85_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CX85_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CX85_NL, "Atari CX85 toetsenblok"},
	/* pl */ {IDGS_CX85_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CX85_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CX85_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CX85_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/rushware_keypad.c */
	/* en */ {IDGS_RUSHWARE_KEYPAD,    N_("RushWare Keypad")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_RUSHWARE_KEYPAD_DA, ""},  /* fuzzy */
	/* de */ {IDGS_RUSHWARE_KEYPAD_DE, "RushWare Keypad"},
	/* es */ {IDGS_RUSHWARE_KEYPAD_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_RUSHWARE_KEYPAD_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_RUSHWARE_KEYPAD_HU, ""},  /* fuzzy */
	/* it */ {IDGS_RUSHWARE_KEYPAD_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_RUSHWARE_KEYPAD_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_RUSHWARE_KEYPAD_NL, "RushWare toetsenblok"},
	/* pl */ {IDGS_RUSHWARE_KEYPAD_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_RUSHWARE_KEYPAD_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_RUSHWARE_KEYPAD_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_RUSHWARE_KEYPAD_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/cardkey.c */
	/* en */ {IDGS_CARDKEY,    N_("Cardco Cardkey 1 keypad")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CARDKEY_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CARDKEY_DE, "Cardco Cardkey 1 Keypad"},
	/* es */ {IDGS_CARDKEY_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CARDKEY_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CARDKEY_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CARDKEY_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CARDKEY_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CARDKEY_NL, "Cardco Cardkey 1 toetsenblok"},
	/* pl */ {IDGS_CARDKEY_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CARDKEY_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CARDKEY_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CARDKEY_TR, ""},  /* fuzzy */
#endif
	
	/* lightpen.c */
	/* en */ {IDGS_LIGHTPEN_UP,    N_("Light Pen (up trigger)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_LIGHTPEN_UP_DA, ""},  /* fuzzy */
	/* de */ {IDGS_LIGHTPEN_UP_DE, "Lichgriffel (Up Trigger)"},
	/* es */ {IDGS_LIGHTPEN_UP_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_LIGHTPEN_UP_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_LIGHTPEN_UP_HU, ""},  /* fuzzy */
	/* it */ {IDGS_LIGHTPEN_UP_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_LIGHTPEN_UP_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_LIGHTPEN_UP_NL, "Light Pen (up trigger)"},
	/* pl */ {IDGS_LIGHTPEN_UP_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_LIGHTPEN_UP_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_LIGHTPEN_UP_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_LIGHTPEN_UP_TR, ""},  /* fuzzy */
#endif
	
	/* lightpen.c */
	/* en */ {IDGS_LIGHTPEN_LEFT,    N_("Light Pen (left trigger)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_LIGHTPEN_LEFT_DA, ""},  /* fuzzy */
	/* de */ {IDGS_LIGHTPEN_LEFT_DE, "Lichtgriffel (Left Trigger)"},
	/* es */ {IDGS_LIGHTPEN_LEFT_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_LIGHTPEN_LEFT_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_LIGHTPEN_LEFT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_LIGHTPEN_LEFT_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_LIGHTPEN_LEFT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_LIGHTPEN_LEFT_NL, "Light Pen (left trigger)"},
	/* pl */ {IDGS_LIGHTPEN_LEFT_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_LIGHTPEN_LEFT_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_LIGHTPEN_LEFT_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_LIGHTPEN_LEFT_TR, ""},  /* fuzzy */
#endif
	
	/* lightpen.c */
	/* en */ {IDGS_DATEL_LIGHTPEN,    N_("Datel Light Pen")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DATEL_LIGHTPEN_DA, ""},  /* fuzzy */
	/* de */ {IDGS_DATEL_LIGHTPEN_DE, "Datel Lichtgriffel"},
	/* es */ {IDGS_DATEL_LIGHTPEN_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_DATEL_LIGHTPEN_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_DATEL_LIGHTPEN_HU, ""},  /* fuzzy */
	/* it */ {IDGS_DATEL_LIGHTPEN_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_DATEL_LIGHTPEN_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DATEL_LIGHTPEN_NL, "Datel Light Pen"},
	/* pl */ {IDGS_DATEL_LIGHTPEN_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_DATEL_LIGHTPEN_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_DATEL_LIGHTPEN_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_DATEL_LIGHTPEN_TR, ""},  /* fuzzy */
#endif
	
	/* lightpen.c */
	/* en */ {IDGS_MAGNUM_LIGHT_PHASER,    N_("Magnum Light Phaser")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MAGNUM_LIGHT_PHASER_DA, "Magnum Light Phaser"},
	/* de */ {IDGS_MAGNUM_LIGHT_PHASER_DE, "Magnum Light Phaser"},
	/* es */ {IDGS_MAGNUM_LIGHT_PHASER_ES, "Magnum Light Phaser"},
	/* fr */ {IDGS_MAGNUM_LIGHT_PHASER_FR, "Magnum Light Phaser"},
	/* hu */ {IDGS_MAGNUM_LIGHT_PHASER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MAGNUM_LIGHT_PHASER_IT, "Magnum Light Phaser"},
	/* ko */ {IDGS_MAGNUM_LIGHT_PHASER_KO, "Magnum ∂Û¿Ã∆Æ ∆‰¿Ã¡Æ"},
	/* nl */ {IDGS_MAGNUM_LIGHT_PHASER_NL, "Magnum Light Phaser"},
	/* pl */ {IDGS_MAGNUM_LIGHT_PHASER_PL, "Magnum Light Phaser"},
	/* ru */ {IDGS_MAGNUM_LIGHT_PHASER_RU, "Magnum Light Phaser"},
	/* sv */ {IDGS_MAGNUM_LIGHT_PHASER_SV, "Magnum Light Phaser"},
	/* tr */ {IDGS_MAGNUM_LIGHT_PHASER_TR, "Magnum Light Phaser"},
#endif
	
	/* lightpen.c */
	/* en */ {IDGS_STACK_LIGHT_RIFLE,    N_("Stack Light Rifle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_STACK_LIGHT_RIFLE_DA, "Stack Light Rifle"},
	/* de */ {IDGS_STACK_LIGHT_RIFLE_DE, "Stack Light Rifle"},
	/* es */ {IDGS_STACK_LIGHT_RIFLE_ES, "Stack Light Rifle"},
	/* fr */ {IDGS_STACK_LIGHT_RIFLE_FR, "Stack Light Rifle"},
	/* hu */ {IDGS_STACK_LIGHT_RIFLE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_STACK_LIGHT_RIFLE_IT, "Stack Light Rifle"},
	/* ko */ {IDGS_STACK_LIGHT_RIFLE_KO, "Stack ∂Û¿Ã∆Æ ∂Û¿Ã«√"},
	/* nl */ {IDGS_STACK_LIGHT_RIFLE_NL, "Stack Light Rifle"},
	/* pl */ {IDGS_STACK_LIGHT_RIFLE_PL, "Stack Light Rifle"},
	/* ru */ {IDGS_STACK_LIGHT_RIFLE_RU, "Stack Light Rifle"},
	/* sv */ {IDGS_STACK_LIGHT_RIFLE_SV, "Stack Light Rifle"},
	/* tr */ {IDGS_STACK_LIGHT_RIFLE_TR, "Stack I˛˝n T¸fei"},
#endif
	
	/* lightpen.c */
	/* en */ {IDGS_INKWELL_LIGHTPEN,    N_("Inkwell Light Pen")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_INKWELL_LIGHTPEN_DA, ""},  /* fuzzy */
	/* de */ {IDGS_INKWELL_LIGHTPEN_DE, "Inkwell Lichgriffel"},
	/* es */ {IDGS_INKWELL_LIGHTPEN_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_INKWELL_LIGHTPEN_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_INKWELL_LIGHTPEN_HU, ""},  /* fuzzy */
	/* it */ {IDGS_INKWELL_LIGHTPEN_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_INKWELL_LIGHTPEN_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_INKWELL_LIGHTPEN_NL, "Inkwell Light Pen"},
	/* pl */ {IDGS_INKWELL_LIGHTPEN_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_INKWELL_LIGHTPEN_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_INKWELL_LIGHTPEN_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_INKWELL_LIGHTPEN_TR, ""},  /* fuzzy */
#endif
	
	/* joyport.c */
	/* en */ {IDGS_SET_JOYPORT_S_DEVICE,    N_("Set %s device (0: None")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SET_JOYPORT_S_DEVICE_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SET_JOYPORT_S_DEVICE_DE, "Ger‰t %s w‰hlen (0: Kein"},
	/* es */ {IDGS_SET_JOYPORT_S_DEVICE_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SET_JOYPORT_S_DEVICE_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SET_JOYPORT_S_DEVICE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SET_JOYPORT_S_DEVICE_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SET_JOYPORT_S_DEVICE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SET_JOYPORT_S_DEVICE_NL, "Stel %s apparaat in (0: Geen"},
	/* pl */ {IDGS_SET_JOYPORT_S_DEVICE_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SET_JOYPORT_S_DEVICE_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SET_JOYPORT_S_DEVICE_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SET_JOYPORT_S_DEVICE_TR, ""},  /* fuzzy */
#endif
	
	/* samplerdrv/sampler.c */
	/* en */ {IDGS_SAMPLER_USED_BY,    N_("Sampler system already in use by %s.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SAMPLER_USED_BY_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SAMPLER_USED_BY_DE, "Sample System bereits von %s benutzt."},
	/* es */ {IDGS_SAMPLER_USED_BY_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SAMPLER_USED_BY_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SAMPLER_USED_BY_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SAMPLER_USED_BY_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SAMPLER_USED_BY_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SAMPLER_USED_BY_NL, "Sampler systeem wordt al gebruikt door %s."},
	/* pl */ {IDGS_SAMPLER_USED_BY_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SAMPLER_USED_BY_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SAMPLER_USED_BY_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SAMPLER_USED_BY_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDGS_CGA_JOY_ADAPTER,    N_("CGA userport joy adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CGA_JOY_ADAPTER_DA, "CGA brugerportjoystickadapter"},
	/* de */ {IDGS_CGA_JOY_ADAPTER_DE, "CGA Userport Joystick Adapter"},
	/* es */ {IDGS_CGA_JOY_ADAPTER_ES, "Adaptador de puerto de juegos CGA"},
	/* fr */ {IDGS_CGA_JOY_ADAPTER_FR, "Adaptateur joystick port utilisateur CGA"},
	/* hu */ {IDGS_CGA_JOY_ADAPTER_HU, "CGA userport botkorm·ny adapter"},
	/* it */ {IDGS_CGA_JOY_ADAPTER_IT, "Adattatore joystick CGA su userport"},
	/* ko */ {IDGS_CGA_JOY_ADAPTER_KO, "CGA ¿Ø¡Æ∆˜∆Æ ¡∂¿Ã æ∆¥‰≈Õ"},
	/* nl */ {IDGS_CGA_JOY_ADAPTER_NL, "CGA userport joy adapter"},
	/* pl */ {IDGS_CGA_JOY_ADAPTER_PL, "Z≥±cze CGA userportu joysticka"},
	/* ru */ {IDGS_CGA_JOY_ADAPTER_RU, "CGA userport joy adapter"},
	/* sv */ {IDGS_CGA_JOY_ADAPTER_SV, "CGA-anv‰ndarportsspakadapter"},
	/* tr */ {IDGS_CGA_JOY_ADAPTER_TR, "CGA userport joy dˆn¸˛t¸r¸c¸"},
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDGS_PET_JOY_ADAPTER,    N_("PET userport joy adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_PET_JOY_ADAPTER_DA, "PET brugerportjoystickadapter"},
	/* de */ {IDGS_PET_JOY_ADAPTER_DE, "PET Userport Joystick Adapter"},
	/* es */ {IDGS_PET_JOY_ADAPTER_ES, "Adaptador de puerto de juegos PET"},
	/* fr */ {IDGS_PET_JOY_ADAPTER_FR, "Adaptateur joystick port utilisateur PET"},
	/* hu */ {IDGS_PET_JOY_ADAPTER_HU, "PET userport botkorm·ny adapter"},
	/* it */ {IDGS_PET_JOY_ADAPTER_IT, "Adattatore joystick PET su userport"},
	/* ko */ {IDGS_PET_JOY_ADAPTER_KO, "PET ¿Ø¡Æ∆˜∆Æ ¡∂¿Ã æ∆¥‰≈Õ"},
	/* nl */ {IDGS_PET_JOY_ADAPTER_NL, "PET userport joy adapter"},
	/* pl */ {IDGS_PET_JOY_ADAPTER_PL, "Z≥±cze PET userportu joysticka"},
	/* ru */ {IDGS_PET_JOY_ADAPTER_RU, "PET userport joy adapter"},
	/* sv */ {IDGS_PET_JOY_ADAPTER_SV, "PET-anv‰ndarportsspakadapter"},
	/* tr */ {IDGS_PET_JOY_ADAPTER_TR, "PET userport joy dˆn¸˛t¸r¸c¸"},
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDGS_HUMMER_JOY_ADAPTER,    N_("Hummer userport joy adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_HUMMER_JOY_ADAPTER_DA, "Hummer brugerport joystick adapter"},
	/* de */ {IDGS_HUMMER_JOY_ADAPTER_DE, "Hummer Userport Joystick Adapter"},
	/* es */ {IDGS_HUMMER_JOY_ADAPTER_ES, "Adaptador de puerto de juegos Hummer"},
	/* fr */ {IDGS_HUMMER_JOY_ADAPTER_FR, "Adaptateur joystick port utilisateur Hummer"},
	/* hu */ {IDGS_HUMMER_JOY_ADAPTER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_HUMMER_JOY_ADAPTER_IT, "Adattatore joystick HUMMER su userport"},
	/* ko */ {IDGS_HUMMER_JOY_ADAPTER_KO, "Hummer ¿Ø¡Æ∆˜∆Æ ¡∂¿Ã æ∆¥‰≈Õ"},
	/* nl */ {IDGS_HUMMER_JOY_ADAPTER_NL, "Hummer userport joystick adapter"},
	/* pl */ {IDGS_HUMMER_JOY_ADAPTER_PL, "Z≥±cze Hummer userportu joysticka"},
	/* ru */ {IDGS_HUMMER_JOY_ADAPTER_RU, "Hummer userport joy adapter"},
	/* sv */ {IDGS_HUMMER_JOY_ADAPTER_SV, "Hummer-anv‰ndarportsspakadapter"},
	/* tr */ {IDGS_HUMMER_JOY_ADAPTER_TR, "Hummer userport joy dˆn¸˛t¸r¸c¸"},
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDGS_OEM_JOY_ADAPTER,    N_("OEM userport joy adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_OEM_JOY_ADAPTER_DA, "OEM brugerportjoystickadapter"},
	/* de */ {IDGS_OEM_JOY_ADAPTER_DE, "OEM Userport Joystick Adapter"},
	/* es */ {IDGS_OEM_JOY_ADAPTER_ES, "Adaptador de puerto de juegos OEM"},
	/* fr */ {IDGS_OEM_JOY_ADAPTER_FR, "Adaptateur joystick port utilisateur OEM"},
	/* hu */ {IDGS_OEM_JOY_ADAPTER_HU, "OEM userport botkorm·ny adapter"},
	/* it */ {IDGS_OEM_JOY_ADAPTER_IT, "Adattatore joystick OEM su userport"},
	/* ko */ {IDGS_OEM_JOY_ADAPTER_KO, "OEM ¿Ø¡Æ∆˜∆Æ ¡∂¿Ã æ∆¥‰≈Õ"},
	/* nl */ {IDGS_OEM_JOY_ADAPTER_NL, "OEM userport joy adapter"},
	/* pl */ {IDGS_OEM_JOY_ADAPTER_PL, "Z≥±cze OEM userportu joysticka"},
	/* ru */ {IDGS_OEM_JOY_ADAPTER_RU, "OEM userport joy adapter"},
	/* sv */ {IDGS_OEM_JOY_ADAPTER_SV, "OEM-anv‰ndarportsspakadapter"},
	/* tr */ {IDGS_OEM_JOY_ADAPTER_TR, "OEM userport joy dˆn¸˛t¸r¸c¸"},
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDGS_HIT_JOY_ADAPTER,    N_("HIT userport joy adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_HIT_JOY_ADAPTER_DA, "HIT brugerportjoystickadapter"},
	/* de */ {IDGS_HIT_JOY_ADAPTER_DE, "HIT Userport Joystick Adapter"},
	/* es */ {IDGS_HIT_JOY_ADAPTER_ES, "Adaptador de puerto de juegos HIT"},
	/* fr */ {IDGS_HIT_JOY_ADAPTER_FR, "Adaptateur joystick port utilisateur HIT"},
	/* hu */ {IDGS_HIT_JOY_ADAPTER_HU, "HIT userport botkorm·ny adapter"},
	/* it */ {IDGS_HIT_JOY_ADAPTER_IT, "Adattatore joystick HIT su userport"},
	/* ko */ {IDGS_HIT_JOY_ADAPTER_KO, "HIT ¿Ø¡Æ∆˜∆Æ ¡∂¿Ã æ∆¥‰≈Õ"},
	/* nl */ {IDGS_HIT_JOY_ADAPTER_NL, "HIT userport joy adapter"},
	/* pl */ {IDGS_HIT_JOY_ADAPTER_PL, "Z≥±cze HIT userportu joysticka"},
	/* ru */ {IDGS_HIT_JOY_ADAPTER_RU, "HIT userport joy adapter"},
	/* sv */ {IDGS_HIT_JOY_ADAPTER_SV, "HIT-anv‰ndarportsspakadapter"},
	/* tr */ {IDGS_HIT_JOY_ADAPTER_TR, "HIT userport joy dˆn¸˛t¸r¸c¸"},
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDGS_KINGSOFT_JOY_ADAPTER,    N_("KingSoft userport joy adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_KINGSOFT_JOY_ADAPTER_DA, ""},  /* fuzzy */
	/* de */ {IDGS_KINGSOFT_JOY_ADAPTER_DE, "Kingsoft Userport Joy Adapter"},
	/* es */ {IDGS_KINGSOFT_JOY_ADAPTER_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_KINGSOFT_JOY_ADAPTER_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_KINGSOFT_JOY_ADAPTER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_KINGSOFT_JOY_ADAPTER_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_KINGSOFT_JOY_ADAPTER_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_KINGSOFT_JOY_ADAPTER_NL, "Kingsoft userport joy adapter"},
	/* pl */ {IDGS_KINGSOFT_JOY_ADAPTER_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_KINGSOFT_JOY_ADAPTER_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_KINGSOFT_JOY_ADAPTER_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_KINGSOFT_JOY_ADAPTER_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDGS_STARBYTE_JOY_ADAPTER,    N_("StarByte userport joy adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_STARBYTE_JOY_ADAPTER_DA, ""},  /* fuzzy */
	/* de */ {IDGS_STARBYTE_JOY_ADAPTER_DE, "Starbyte Userport Joy Adapter"},
	/* es */ {IDGS_STARBYTE_JOY_ADAPTER_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_STARBYTE_JOY_ADAPTER_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_STARBYTE_JOY_ADAPTER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_STARBYTE_JOY_ADAPTER_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_STARBYTE_JOY_ADAPTER_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_STARBYTE_JOY_ADAPTER_NL, "Starbyte userport joy adapter"},
	/* pl */ {IDGS_STARBYTE_JOY_ADAPTER_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_STARBYTE_JOY_ADAPTER_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_STARBYTE_JOY_ADAPTER_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_STARBYTE_JOY_ADAPTER_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_58321a.c */
	/* en */ {IDGS_USERPORT_RTC58321A,    N_("Userport RTC (RTC58321A)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_RTC58321A_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_RTC58321A_DE, "Userport RTC (RTC58321A)"},
	/* es */ {IDGS_USERPORT_RTC58321A_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_RTC58321A_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_RTC58321A_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_RTC58321A_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_RTC58321A_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_RTC58321A_NL, "Userport RTC (RTC58321A)"},
	/* pl */ {IDGS_USERPORT_RTC58321A_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_RTC58321A_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_RTC58321A_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_RTC58321A_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_ds1307.c */
	/* en */ {IDGS_USERPORT_DS1307,    N_("Userport RTC (DS1307)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_DS1307_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_DS1307_DE, "Userport RTC (DS1307)"},
	/* es */ {IDGS_USERPORT_DS1307_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_DS1307_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_DS1307_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_DS1307_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_DS1307_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_DS1307_NL, "Userport RTC (DS1307)"},
	/* pl */ {IDGS_USERPORT_DS1307_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_DS1307_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_DS1307_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_DS1307_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_diag_586220_harness.c */
	/* en */ {IDGS_USERPORT_DIAG_586220_HARNESS,    N_("Userport diag 586220 harness")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_DIAG_586220_HARNESS_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_DIAG_586220_HARNESS_DE, "Userport Diag 586220 Harness"},
	/* es */ {IDGS_USERPORT_DIAG_586220_HARNESS_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_DIAG_586220_HARNESS_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_DIAG_586220_HARNESS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_DIAG_586220_HARNESS_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_DIAG_586220_HARNESS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_DIAG_586220_HARNESS_NL, "Userport diag 586220 gareel"},
	/* pl */ {IDGS_USERPORT_DIAG_586220_HARNESS_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_DIAG_586220_HARNESS_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_DIAG_586220_HARNESS_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_DIAG_586220_HARNESS_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapertc.c */
	/* en */ {IDGS_TAPE_RTC,    N_("Tape RTC (PCF8583)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_TAPE_RTC_DA, ""},  /* fuzzy */
	/* de */ {IDGS_TAPE_RTC_DE, "Band RTC (PCF8583)"},
	/* es */ {IDGS_TAPE_RTC_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_TAPE_RTC_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_TAPE_RTC_HU, ""},  /* fuzzy */
	/* it */ {IDGS_TAPE_RTC_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_TAPE_RTC_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_TAPE_RTC_NL, "Tape RTC (PCF8583)"},
	/* pl */ {IDGS_TAPE_RTC_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_TAPE_RTC_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_TAPE_RTC_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_TAPE_RTC_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/sense-dongle.c */
	/* en */ {IDGS_SENSE_DONGLE,    N_("Sense dongle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SENSE_DONGLE_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SENSE_DONGLE_DE, "Sense Dongle"},
	/* es */ {IDGS_SENSE_DONGLE_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SENSE_DONGLE_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SENSE_DONGLE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SENSE_DONGLE_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SENSE_DONGLE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SENSE_DONGLE_NL, "Sense dongle"},
	/* pl */ {IDGS_SENSE_DONGLE_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SENSE_DONGLE_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SENSE_DONGLE_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SENSE_DONGLE_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_dac.c */
	/* en */ {IDGS_USERPORT_DAC,    N_("Userport DAC")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_DAC_DA, "Brugerport DAC"},
	/* de */ {IDGS_USERPORT_DAC_DE, "Userport DAC"},
	/* es */ {IDGS_USERPORT_DAC_ES, "Puerto de usuario DAC"},
	/* fr */ {IDGS_USERPORT_DAC_FR, "DAC port utilisateur"},
	/* hu */ {IDGS_USERPORT_DAC_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_DAC_IT, "DAC su userport "},
	/* ko */ {IDGS_USERPORT_DAC_KO, "¿Ø¡Æ∆˜∆Æ DAC"},
	/* nl */ {IDGS_USERPORT_DAC_NL, "Userport DAC"},
	/* pl */ {IDGS_USERPORT_DAC_PL, "Userport DAC"},
	/* ru */ {IDGS_USERPORT_DAC_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_DAC_SV, "Anv‰ndarport-D/A-omvandlare"},
	/* tr */ {IDGS_USERPORT_DAC_TR, "Userport DAC"},
#endif
	
	/* userport/userport_digimax.c */
	/* en */ {IDGS_USERPORT_DIGIMAX,    N_("Userport DigiMAX")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_DIGIMAX_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_DIGIMAX_DE, "Userport DigiMAX"},
	/* es */ {IDGS_USERPORT_DIGIMAX_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_DIGIMAX_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_DIGIMAX_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_DIGIMAX_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_DIGIMAX_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_DIGIMAX_NL, "Userport DigiMAX"},
	/* pl */ {IDGS_USERPORT_DIGIMAX_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_DIGIMAX_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_DIGIMAX_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_DIGIMAX_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_4bit_sampler.c */
	/* en */ {IDGS_USERPORT_4BIT_SAMPLER,    N_("Userport 4bit sampler")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_4BIT_SAMPLER_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_4BIT_SAMPLER_DE, "Userport 4bit Sampler"},
	/* es */ {IDGS_USERPORT_4BIT_SAMPLER_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_4BIT_SAMPLER_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_4BIT_SAMPLER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_4BIT_SAMPLER_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_4BIT_SAMPLER_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_4BIT_SAMPLER_NL, "Userport 4bit sampler"},
	/* pl */ {IDGS_USERPORT_4BIT_SAMPLER_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_4BIT_SAMPLER_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_4BIT_SAMPLER_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_4BIT_SAMPLER_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_8bss.c */
	/* en */ {IDGS_USERPORT_8BSS,    N_("Userport 8bit stereo sampler")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_8BSS_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_8BSS_DE, "Userport 8bit Stereo Sampler"},
	/* es */ {IDGS_USERPORT_8BSS_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_8BSS_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_8BSS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_8BSS_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_8BSS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_8BSS_NL, "Userport 8bit stereo sampler"},
	/* pl */ {IDGS_USERPORT_8BSS_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_8BSS_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_8BSS_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_8BSS_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/interface_userport.c */
	/* en */ {IDGS_USERPORT_PRINTER,    N_("Userport printer")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_USERPORT_PRINTER_DA, ""},  /* fuzzy */
	/* de */ {IDGS_USERPORT_PRINTER_DE, "Userport Drucker"},
	/* es */ {IDGS_USERPORT_PRINTER_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_USERPORT_PRINTER_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_USERPORT_PRINTER_HU, ""},  /* fuzzy */
	/* it */ {IDGS_USERPORT_PRINTER_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_USERPORT_PRINTER_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_USERPORT_PRINTER_NL, "Userport printer"},
	/* pl */ {IDGS_USERPORT_PRINTER_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_USERPORT_PRINTER_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_USERPORT_PRINTER_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_USERPORT_PRINTER_TR, ""},  /* fuzzy */
#endif
	
	/* datasette.c */
	/* en */ {IDGS_DATASETTE,    N_("Datasette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_DATASETTE_DA, ""},  /* fuzzy */
	/* de */ {IDGS_DATASETTE_DE, "Datasette"},
	/* es */ {IDGS_DATASETTE_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_DATASETTE_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_DATASETTE_HU, ""},  /* fuzzy */
	/* it */ {IDGS_DATASETTE_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_DATASETTE_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_DATASETTE_NL, "Datasette"},
	/* pl */ {IDGS_DATASETTE_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_DATASETTE_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_DATASETTE_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_DATASETTE_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapelog.c */
	/* en */ {IDGS_TAPE_LOG,    N_("Tape Log")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_TAPE_LOG_DA, ""},  /* fuzzy */
	/* de */ {IDGS_TAPE_LOG_DE, "Bandlog"},
	/* es */ {IDGS_TAPE_LOG_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_TAPE_LOG_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_TAPE_LOG_HU, ""},  /* fuzzy */
	/* it */ {IDGS_TAPE_LOG_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_TAPE_LOG_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_TAPE_LOG_NL, "Tape Log"},
	/* pl */ {IDGS_TAPE_LOG_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_TAPE_LOG_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_TAPE_LOG_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_TAPE_LOG_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tape_diag_586220_harness.c */
	/* en */ {IDGS_TAPE_DIAG_586220_HARNESS,    N_("Tape 586220 diagnostics harness module")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_TAPE_DIAG_586220_HARNESS_DA, ""},  /* fuzzy */
	/* de */ {IDGS_TAPE_DIAG_586220_HARNESS_DE, "Tape 586220 Diagnose Harness Modul"},
	/* es */ {IDGS_TAPE_DIAG_586220_HARNESS_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_TAPE_DIAG_586220_HARNESS_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_TAPE_DIAG_586220_HARNESS_HU, ""},  /* fuzzy */
	/* it */ {IDGS_TAPE_DIAG_586220_HARNESS_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_TAPE_DIAG_586220_HARNESS_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_TAPE_DIAG_586220_HARNESS_NL, "Tape 586220 diagnostics gareel module"},
	/* pl */ {IDGS_TAPE_DIAG_586220_HARNESS_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_TAPE_DIAG_586220_HARNESS_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_TAPE_DIAG_586220_HARNESS_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_TAPE_DIAG_586220_HARNESS_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S,    N_("EOF while writing to module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_DE, "EOF beim Schreben von Modul %s in Snapshot %s"},
	/* es */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_NL, "EOF tijdens het schrijven naar module %s in momentopnamebestand %s"},
	/* pl */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_EOF_WRITING_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_EOF_WRITING_SNAPSHOT_S,    N_("EOF while writing to snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_EOF_WRITING_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_EOF_WRITING_SNAPSHOT_S_DE, "EOF beim Schreiben auf Snapshot %s"},
	/* es */ {IDGS_EOF_WRITING_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_EOF_WRITING_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_EOF_WRITING_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_EOF_WRITING_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_EOF_WRITING_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_EOF_WRITING_SNAPSHOT_S_NL, "EOF tijdens het schrijven naar momentopnamebestand %s"},
	/* pl */ {IDGS_EOF_WRITING_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_EOF_WRITING_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_EOF_WRITING_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_EOF_WRITING_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S,    N_("Error writing array to module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_DE, "Fehler beim Schreiben (Array) auf Modul %s in Snapshot %s"},
	/* es */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_NL, "Fout bij het schrijven van een blok naar module %s in momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_WRITING_ARRAY_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S,    N_("Error writing array to snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_DE, "Fehler bei Schreiben (Array) auf Snapshot %s"},
	/* es */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_NL, "Fout bij het schrijven van een block naar momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_WRITING_ARRAY_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S,    N_("EOF while reading from module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_DE, "EOF beim Lesen von Modul %s in Snapshot %s"},
	/* es */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_NL, "EOF tijdens het lezen van module %s in momentopnamebestand %s"},
	/* pl */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_EOF_READING_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_EOF_READING_SNAPSHOT_S,    N_("EOF while reading from snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_EOF_READING_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_EOF_READING_SNAPSHOT_S_DE, "EOF beim Lesen von Snapshot %s"},
	/* es */ {IDGS_EOF_READING_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_EOF_READING_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_EOF_READING_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_EOF_READING_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_EOF_READING_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_EOF_READING_SNAPSHOT_S_NL, "EOF tijdens het lezen van momentopnamebestand %s"},
	/* pl */ {IDGS_EOF_READING_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_EOF_READING_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_EOF_READING_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_EOF_READING_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S,    N_("Error reading array from module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_DE, "Fehler beim Lesen von Modul %s in Snapshot %s"},
	/* es */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_NL, "Fout bij het lezen van een blok van module %s in momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_READING_ARRAY_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S,    N_("Error reading array from snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_DE, "Fehler beim Lesen von Snapshot %s"},
	/* es */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_NL, "Fout bij het lezen van een blok van momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_READING_ARRAY_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S,    N_("Error writing string to module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_DE, "Fehler beim Schreiben (String) auf Modul %s in Snapshot %s"},
	/* es */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_NL, "Fout bij het schrijven van tekst naar module %s in momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_WRITING_STRING_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S,    N_("Error writing string to snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_DE, "Fehler beim Schreiben (String)auf Snapshot %s"},
	/* es */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_NL, "Fout bij het schrijven van tekst naar momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_WRITING_STRING_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S,    N_("Out of bounds reading error in module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_DE, "Lesefehler (Out-of-bounds) in Modul %s in Snapshot %s"},
	/* es */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_NL, "Buiten de grenzen leesfout in module %s in momentopnamebestand %s"},
	/* pl */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_OUT_OF_BOUNDS_READING_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S,    N_("Out of bounds reading error in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_DE, "Lesefehler (Out-of-bounds) in Snapshot %s"},
	/* es */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_NL, "Buiten de grenzen fout bij het lezen van momentopnamebestand %s"},
	/* pl */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_OUT_OF_BOUNDS_READING_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S,    N_("Illegal offset while attempting to create module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_DE, "Fehler ber Erstellung von Modul %s in Snapshot %s (Illegal offset)"},
	/* es */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_NL, "Fout beginafstand tijdens het maken van module %s in momentopnamebestand %s"},
	/* pl */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ILLEGAL_OFFSET_CREATE_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S,    N_("Cannot find first module in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_DE, "Kann erstes Modul in Snapshot %s nicht finden"},
	/* es */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_NL, "Kan de eerste module in momentopnamebestand %s niet vinden"},
	/* pl */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_FIND_1ST_MODULE_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S,    N_("Error while reading module header in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_DE, "Fehler beim Lesen des Modulheaders in Snapshot %s"},
	/* es */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_NL, "Fout bij het lezen van het begin van een module in momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_MODULE_HEADER_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S,    N_("Cannot find module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_DE, "Kann Modul %s in Snapshot %s nicht finden"},
	/* es */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_NL, "Kan de module %s niet vinden in momentopnamebestand %s"},
	/* pl */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_FIND_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S,    N_("Error closing module %s in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_DE, "Fehler beim Schliessen von Modul %s in Snapshot %s"},
	/* es */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_NL, "Fout bij het sluiten van module %s in momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_CLOSING_MODULE_S_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S,    N_("Error skipping module in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_DE, "Fehler beim ‹berspringen eines Moduls in Snapshot %s"},
	/* es */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_NL, "Fout bij het overslaan van module in momentopnamebestand %s"},
	/* pl */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_ERROR_SKIPPING_MODULE_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_CREATE_SNAPSHOT_S,    N_("Cannot create snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_DE, "Kann Snapshot %s nicht erzeugen"},
	/* es */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_NL, "Kan momentopnamebestand %s niet maken"},
	/* pl */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_CREATE_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S,    N_("Cannot write magic string to snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_DE, "Kann Magic String in Snapshot %s nicht schreiben"},
	/* es */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_NL, "Kan de magische tekenreeks niet schrijven naar momentopnamebestand %s"},
	/* pl */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_WRITE_MAGIC_STRING_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S,    N_("Cannot write version to snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_DE, "Kann Version in Snapshot %s nicht schreiben"},
	/* es */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_NL, "Kan de versie niet schrijven naar momentopnamebestand %s"},
	/* pl */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_WRITE_VERSION_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S,    N_("Cannot write machine name to snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_DE, "Kann Maschinennamen in Snapshot %s nicht schreiben"},
	/* es */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_NL, "Kan de machine naam niet schrijven naar momentopnamebestand %s"},
	/* pl */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_WRITE_MACHINE_NAME_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING,    N_("Cannot open snapshot %s for reading")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_DE, "Kann Snapshot %s nicht ˆffnen"},
	/* es */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_NL, "Kan het momentopnamebestand %s niet openen voor lezen"},
	/* pl */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_OPEN_SNAPSHOT_S_READING_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S,    N_("Magic string mismatch in snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_DE, "Magic String Fehler in Snapshot %s"},
	/* es */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_NL, "Magische tekenreeks is niet wat is verwacht in momentopnamebestand %s"},
	/* pl */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_MAGIC_STRING_MISMATCH_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S,    N_("Cannot read version from snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_DE, "Kann Versionsnummer von Snapshot %s nicht lesen"},
	/* es */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_NL, "Kan de versie niet lezen van momentopnamebestand %s"},
	/* pl */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_READ_VERSION_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S,    N_("Cannot read machine name from snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_DE, "Kann Maschinenname von Snapshot %s nicht lesen"},
	/* es */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_NL, "Kan de machine naam niet lezen van momentopnamebestand %s"},
	/* pl */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CANNOT_READ_MACHINE_NAME_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S,    N_("Wrong machine type in snapshot %s, snapshot type: %s, current machine: %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_DE, "Falscher Maschinentyp in Snapshot %s, Typ: %s, aktuelle Maschine: %s"},
	/* es */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_NL, "Verkeerd machine soort in momentopnamebestand %s, soort in bestand: %s, huidige soort: %s"},
	/* pl */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_WRONG_MACHINE_TYPE_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_EOF_CLOSING_SNAPSHOT_S,    N_("EOF while closing snapshot %s")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_EOF_CLOSING_SNAPSHOT_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_EOF_CLOSING_SNAPSHOT_S_DE, "EOF beim Schliessen von Snapshot %s"},
	/* es */ {IDGS_EOF_CLOSING_SNAPSHOT_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_EOF_CLOSING_SNAPSHOT_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_EOF_CLOSING_SNAPSHOT_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_EOF_CLOSING_SNAPSHOT_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_EOF_CLOSING_SNAPSHOT_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_EOF_CLOSING_SNAPSHOT_S_NL, "EOF tijdens het afsluiten van momentopnamebestand %s"},
	/* pl */ {IDGS_EOF_CLOSING_SNAPSHOT_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_EOF_CLOSING_SNAPSHOT_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_EOF_CLOSING_SNAPSHOT_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_EOF_CLOSING_SNAPSHOT_S_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_SNAPSHOT_HIGHER_VERSION,    N_("Snapshot %s has a higher version than what your current emulator supports, please upgrade VICE")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SNAPSHOT_HIGHER_VERSION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SNAPSHOT_HIGHER_VERSION_DE, "Snapshot %s benˆtigt einen neuere Version von Vice. Bitte Upgraden!"},
	/* es */ {IDGS_SNAPSHOT_HIGHER_VERSION_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SNAPSHOT_HIGHER_VERSION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SNAPSHOT_HIGHER_VERSION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SNAPSHOT_HIGHER_VERSION_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SNAPSHOT_HIGHER_VERSION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SNAPSHOT_HIGHER_VERSION_NL, "Momentopnamebestand %s heeft een hogere versie dan wat de huidige emulator ondersteunt, download een recente versie van VICE"},
	/* pl */ {IDGS_SNAPSHOT_HIGHER_VERSION_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SNAPSHOT_HIGHER_VERSION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SNAPSHOT_HIGHER_VERSION_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SNAPSHOT_HIGHER_VERSION_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_INCOMPATIBLE_SNAPSHOT,    N_("Snapshot %s is incompatible (too old)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_INCOMPATIBLE_SNAPSHOT_DA, ""},  /* fuzzy */
	/* de */ {IDGS_INCOMPATIBLE_SNAPSHOT_DE, "Snapshot %s ist nicht kompatibel (zu alt)"},
	/* es */ {IDGS_INCOMPATIBLE_SNAPSHOT_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_INCOMPATIBLE_SNAPSHOT_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_INCOMPATIBLE_SNAPSHOT_HU, ""},  /* fuzzy */
	/* it */ {IDGS_INCOMPATIBLE_SNAPSHOT_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_INCOMPATIBLE_SNAPSHOT_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_INCOMPATIBLE_SNAPSHOT_NL, "Momentopname %s is niet compatibel (te oud)"},
	/* pl */ {IDGS_INCOMPATIBLE_SNAPSHOT_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_INCOMPATIBLE_SNAPSHOT_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_INCOMPATIBLE_SNAPSHOT_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_INCOMPATIBLE_SNAPSHOT_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_SNAPSHOT_VICE_VERSION,    N_("Snapshot was created by VICE Version %d.%d.%d")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SNAPSHOT_VICE_VERSION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SNAPSHOT_VICE_VERSION_DE, "Snapshot wurde von VICE Version %d.%d.%d erzeugt"},
	/* es */ {IDGS_SNAPSHOT_VICE_VERSION_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SNAPSHOT_VICE_VERSION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SNAPSHOT_VICE_VERSION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SNAPSHOT_VICE_VERSION_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SNAPSHOT_VICE_VERSION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SNAPSHOT_VICE_VERSION_NL, "Momentopname was gemaakt met VICE Versie %d.%d.%d"},
	/* pl */ {IDGS_SNAPSHOT_VICE_VERSION_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SNAPSHOT_VICE_VERSION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SNAPSHOT_VICE_VERSION_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SNAPSHOT_VICE_VERSION_TR, ""},  /* fuzzy */
#endif
	
	/* snapshot.c */
	/* en */ {IDGS_SNAPSHOT_OLD_VICE_VERSION,    N_("Snapshot was created by VICE Version 2.4.30 or older")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_DA, ""},  /* fuzzy */
	/* de */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_DE, "Snapshot wurde von VICE Version 2.4.30 oder ‰lter erzeugt"},
	/* es */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_HU, ""},  /* fuzzy */
	/* it */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_NL, "Momentopname was gemaakt met VICE Versie 2.4.30 of ouder"},
	/* pl */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_SNAPSHOT_OLD_VICE_VERSION_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/clockport-rrnet.c */
	/* en */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S,    N_("ClockPort RRNET already in use by %s.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_DE, "Clockport RRNET bereits von %s benutzt."},
	/* es */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_NL, "ClockPort RRNET wordt al gebruikt door %s."},
	/* pl */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CLOCKPORT_RRNET_IN_USE_BY_S_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/clockport-mp3at64.c */
	/* en */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S,    N_("ClockPort MP3@64 already in use by %s.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_DE, "Clockport MP3@64 bereits von %s benutzt."},
	/* es */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_NL, "ClockPort MP3@64 wordt al gebruikt door %s."},
	/* pl */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CLOCKPORT_MP3AT64_IN_USE_BY_S_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/cs8900io.c */
	/* en */ {IDGS_CS8900_IN_USE_BY_S,    N_("CS8900 already in use by %s.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDGS_CS8900_IN_USE_BY_S_DA, ""},  /* fuzzy */
	/* de */ {IDGS_CS8900_IN_USE_BY_S_DE, "CS8900 bereits von %s benutzt."},
	/* es */ {IDGS_CS8900_IN_USE_BY_S_ES, ""},  /* fuzzy */
	/* fr */ {IDGS_CS8900_IN_USE_BY_S_FR, ""},  /* fuzzy */
	/* hu */ {IDGS_CS8900_IN_USE_BY_S_HU, ""},  /* fuzzy */
	/* it */ {IDGS_CS8900_IN_USE_BY_S_IT, ""},  /* fuzzy */
	/* ko */ {IDGS_CS8900_IN_USE_BY_S_KO, ""},  /* fuzzy */
	/* nl */ {IDGS_CS8900_IN_USE_BY_S_NL, "CS8900 wordt al gebruikt door %s."},
	/* pl */ {IDGS_CS8900_IN_USE_BY_S_PL, ""},  /* fuzzy */
	/* ru */ {IDGS_CS8900_IN_USE_BY_S_RU, ""},  /* fuzzy */
	/* sv */ {IDGS_CS8900_IN_USE_BY_S_SV, ""},  /* fuzzy */
	/* tr */ {IDGS_CS8900_IN_USE_BY_S_TR, ""},  /* fuzzy */
#endif
	
	/* ------------------------ COMMAND LINE OPTION STRINGS -------------------- */
	
	/* joystick.c */
	/* en */ {IDCLS_ENABLE_JOY_OPPOSITE,    N_("Enable opposite joystick directions")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_JOY_OPPOSITE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_JOY_OPPOSITE_DE, "Gegen¸berliegende Joystick Richtungen zulassen"},
	/* es */ {IDCLS_ENABLE_JOY_OPPOSITE_ES, "Permitir direcciones opuestas del joystick"},
	/* fr */ {IDCLS_ENABLE_JOY_OPPOSITE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_JOY_OPPOSITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_JOY_OPPOSITE_IT, "Attiva direzioni opposte joystick"},
	/* ko */ {IDCLS_ENABLE_JOY_OPPOSITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_JOY_OPPOSITE_NL, "Tegenovergestelde richtingen toestaan"},
	/* pl */ {IDCLS_ENABLE_JOY_OPPOSITE_PL, "PozwÛl na przeciwstawne kierunki joysticka"},
	/* ru */ {IDCLS_ENABLE_JOY_OPPOSITE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_JOY_OPPOSITE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_JOY_OPPOSITE_TR, ""},  /* fuzzy */
#endif
	
	/* joystick.c */
	/* en */ {IDCLS_DISABLE_JOY_OPPOSITE,    N_("Disable opposite joystick directions")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_JOY_OPPOSITE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_JOY_OPPOSITE_DE, "Gegen¸berliegende Joystick Richtungen verhindern"},
	/* es */ {IDCLS_DISABLE_JOY_OPPOSITE_ES, "Inhabilitar direcciones opuestas del joystick"},
	/* fr */ {IDCLS_DISABLE_JOY_OPPOSITE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_JOY_OPPOSITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_JOY_OPPOSITE_IT, "Disattiva direzioni opposte joystick"},
	/* ko */ {IDCLS_DISABLE_JOY_OPPOSITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_JOY_OPPOSITE_NL, "Tegenovergestelde richtingen niet toestaan"},
	/* pl */ {IDCLS_DISABLE_JOY_OPPOSITE_PL, "Nie pozwÛl na przeciwstawne kierunki joysticka"},
	/* ru */ {IDCLS_DISABLE_JOY_OPPOSITE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_JOY_OPPOSITE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_JOY_OPPOSITE_TR, ""},  /* fuzzy */
#endif
	
	/* joystick.c */
	/* en */ {IDCLS_ENABLE_KEYSET,    N_("Enable keyset")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_KEYSET_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_KEYSET_DE, "Tastaturbelegung aktivieren"},
	/* es */ {IDCLS_ENABLE_KEYSET_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_KEYSET_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_KEYSET_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_KEYSET_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_KEYSET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_KEYSET_NL, "Keyset inschakelen"},
	/* pl */ {IDCLS_ENABLE_KEYSET_PL, "W≥±cz zestaw klawiszy"},
	/* ru */ {IDCLS_ENABLE_KEYSET_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_KEYSET_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_KEYSET_TR, ""},  /* fuzzy */
#endif
	
	/* joystick.c */
	/* en */ {IDCLS_DISABLE_KEYSET,    N_("Disable keyset")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_KEYSET_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_KEYSET_DE, "Tastaturbelegung deaktivieren"},
	/* es */ {IDCLS_DISABLE_KEYSET_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_KEYSET_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_KEYSET_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_KEYSET_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_KEYSET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_KEYSET_NL, "Keyset uitschakelen"},
	/* pl */ {IDCLS_DISABLE_KEYSET_PL, "Wy≥±cz zestaw klawiszy"},
	/* ru */ {IDCLS_DISABLE_KEYSET_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_KEYSET_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_KEYSET_TR, ""},  /* fuzzy */
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START,    N_("On autostart, load to BASIC start (without ',1')")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_DA, "IndlÊs til BASIC start ved autostart (uden ',1')"},
	/* de */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_DE, "Bei Autostart, lade an BASIC Start (ohne ',1')"},
	/* es */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_ES, "En autoarranque, carga en inicio del BASIC (sin ',1')"},
	/* fr */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_FR, "Lors de l'autodÈmarrage, charger au BASIC (sans ',1')"},
	/* hu */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_IT, "All'avvio automatico, carica all'inizio del BASIC (senza ',1')"},
	/* ko */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_NL, "Bij autostarten, laad naar BASIC start (zonder ',1')"},
	/* pl */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_PL, "Przy autostarcie wczytuj do BASIC-a (bez ',1')"},
	/* ru */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_RU, "On autostart, load to BASIC start (without ',1')"},
	/* sv */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_SV, "Vid autostart, h‰mta till BASIC-start (utan \",1\")"},
	/* tr */ {IDCLS_AUTOSTART_LOAD_TO_BASIC_START_TR, "Otomatik ba˛lat˝ld˝˝nda, BASIC ba˛lang˝c˝na y¸kle (',1' olmadan)"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_AUTOSTART_LOAD_WITH_1,    N_("On autostart, load with ',1'")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_AUTOSTART_LOAD_WITH_1_DA, "IndlÊs med ',1' ved autostart"},
	/* de */ {IDCLS_AUTOSTART_LOAD_WITH_1_DE, "Bei Autostart, lade mit ',1'"},
	/* es */ {IDCLS_AUTOSTART_LOAD_WITH_1_ES, "En autoarranque, carga con ',1'"},
	/* fr */ {IDCLS_AUTOSTART_LOAD_WITH_1_FR, "¿ l'autodÈmarrage, charger avec ',1'"},
	/* hu */ {IDCLS_AUTOSTART_LOAD_WITH_1_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_AUTOSTART_LOAD_WITH_1_IT, "All'avvio automatico, carica con ',1'"},
	/* ko */ {IDCLS_AUTOSTART_LOAD_WITH_1_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_AUTOSTART_LOAD_WITH_1_NL, "Bij autostarten, laad met ',1'"},
	/* pl */ {IDCLS_AUTOSTART_LOAD_WITH_1_PL, "Przy autostarcie wczytuj z ',1'"},
	/* ru */ {IDCLS_AUTOSTART_LOAD_WITH_1_RU, "On autostart, load with ',1'"},
	/* sv */ {IDCLS_AUTOSTART_LOAD_WITH_1_SV, "Vid autostart, h‰mta med \",1\""},
	/* tr */ {IDCLS_AUTOSTART_LOAD_WITH_1_TR, "Otomatik ba˛lat˝ld˝˝nda, ',1' ile y¸kle"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON,    N_("On autostart, use the 'RUN' command with a colon, i.e., 'RUN:'")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_DA, "Brug \"RUN\"-kommando med kolon ved autostart, dvs. \"RUN:\""},
	/* de */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_DE, "Bei Autostart soll 'RUN' mit Doppelpunkt ausgef¸hrt werden ('RUN:')"},
	/* es */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_ES, "En autoarranque, usa el comando 'RUN' con dos puntos, ej. 'RUN:'"},
	/* fr */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_FR, "¿ l'autodÈmarrage, utiliser la commande 'RUN' avec le deux-points (soit 'RUN:')"},
	/* hu */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_HU, "Automatikus indÌt·sn·l a 'RUN' parancs kettısponttal, vagyis 'RUN:'"},
	/* it */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_IT, "All'avvio automatico, usa il comando 'RUN' con i due punti; cioË 'RUN:'"},
	/* ko */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_NL, "Bij het autostarten, gebruik het 'RUN' commando met een dubbele punt, dus 'RUN:'"},
	/* pl */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_PL, "Przy autostarcie uøywaj polecenia 'RUN' z dwukropkiem, tj. 'RUN:'"},
	/* ru */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_RU, "On autostart, use the 'RUN' command with a colon, i.e., 'RUN:'"},
	/* sv */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_SV, "Vid autostart, anv‰nd \"RUN\"-kommando med kolon, dvs. \"RUN:\""},
	/* tr */ {IDCLS_ENABLE_AUTOSTARTWITHCOLON_TR, "Otomatik ba˛lat˝ld˝˝nda, 'RUN' komutunu ':' ile kullan, yani, 'RUN:'"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON,    N_("On autostart, do not use the 'RUN' command with a colon; i.e., 'RUN'")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_DA, "Brug ikke \"RUN\"-kommando med kolon ved autostart, dvs. \"RUN\""},
	/* de */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_DE, "Bei Autostart soll 'RUN' ohne Doppelpunkt ausgef¸hrt werden ('RUN')"},
	/* es */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_ES, "En autoarranque, no usar el comando 'RUN' con dos puntos, ej.'RUN'"},
	/* fr */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_FR, "¿ l'autodÈmarrage, NE PAS utiliser le deux-points avec la commande 'RUN'"},
	/* hu */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_HU, "Automatikus indÌt·snak a 'RUN' parancs kettıspont nÈlk¸l, vagyis 'RUN'"},
	/* it */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_IT, "All'avvio automatico, non usare il comando 'RUN' con i due punti; cioË 'RUN'"},
	/* ko */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_NL, "Bij het autostarten, gebruik het 'RUN' commando niet met een dubbele punt, dus 'RUN'"},
	/* pl */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_PL, "Przy autostarcie uøywaj polecenia 'RUN' bez dwukropka, tj. 'RUN'"},
	/* ru */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_RU, "On autostart, do not use the 'RUN' command with a colon; i.e., 'RUN'"},
	/* sv */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_SV, "Vid autostart, anv‰nd inte \"RUN\"-kommando med kolon, dvs. \"RUN\""},
	/* tr */ {IDCLS_DISABLE_AUTOSTARTWITHCOLON_TR, "Otomatik ba˛lat˝ld˝˝nda, 'RUN' komutunu ':' ile kullanma, yani, 'RUN'"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE,    N_("Handle True Drive Emulation on autostart")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_DA, "Anvend Êgte drev-emulering ved autostart"},
	/* de */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_DE, "Pr‰zise Floppy Emulation bei Autostart beachten"},
	/* es */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_ES, "Manejar en autoarranque emulaciÛn de disco verdadera"},
	/* fr */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_FR, "Activer l'…mulation RÈelle de Lecteur ‡ l'autodÈmarrage"},
	/* hu */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_HU, "ValÛs lemezegysÈg emul·ciÛ kezelÈse autostartn·l"},
	/* it */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_IT, "Emulazione hardware dei drive all'avvio automatico"},
	/* ko */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_NL, "Hanteer Hardwarematige Drive Emulatie bij autostarten"},
	/* pl */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_PL, "W≥±cz rzeczywist± emulacjÍ napÍdu przy autostarcie"},
	/* ru */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_RU, "Handle True Drive Emulation on autostart"},
	/* sv */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_SV, "Hantera ‰kta diskettenhetsemulering vid autostart"},
	/* tr */ {IDCLS_ENABLE_AUTOSTART_HANDLE_TDE_TR, "Otomatik ba˛latmada GerÁek S¸r¸c¸ Em¸lasyonu'nu denetle"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE,    N_("Do not handle True Drive Emulation on autostart")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_DA, "Anvend ikke Êgte drev-emulering ved autostart"},
	/* de */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_DE, "Pr‰zise Floppy Emulation bei Autostart nicht beachten"},
	/* es */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_ES, "No manejar en autoarranque emulaciÛn de disco verdadera"},
	/* fr */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_FR, "Ne pas utiliser l'Èmulation rÈelle de lecteur ‡ l'autodÈmarrage"},
	/* hu */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_HU, "ValÛs lemezegysÈg emul·ciÛt nem kezeli autostartn·l"},
	/* it */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_IT, "Non gestire l'emulazione hardware dei drive all'avvio automatico"},
	/* ko */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_NL, "Hanteer geen hardwarematige drive emulatie bij autostarten"},
	/* pl */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_PL, "Nie w≥±czaj rzeczywistej emulacji napÍdu przy autostarcie"},
	/* ru */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_RU, "Do not handle True Drive Emulation on autostart"},
	/* sv */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_SV, "Hantera inte ‰kta diskettenhetsemulering vid autostart"},
	/* tr */ {IDCLS_DISABLE_AUTOSTART_HANDLE_TDE_TR, "Ba˛lang˝Áta GerÁek S¸r¸c¸ Em¸lasyonu'nu Denetleme"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART,    N_("Enable warp mode during autostart")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_DA, "AktivÈr warp-tilstand under autostart"},
	/* de */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_DE, "Warp Mode w‰hrend Autostart aktivieren"},
	/* es */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_ES, "Permitir modo ultrarr·pido durante autoarranque"},
	/* fr */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_FR, "Activer Mode Turbo durant l'autodÈmarrage"},
	/* hu */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_HU, "Hipergyors mÛd engedÈlyezÈse autostartn·l"},
	/* it */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_IT, "Attiva la modalit‡ turbo all'avvio automatico"},
	/* ko */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_NL, "Warp modus tijdens autostarten inschakelen"},
	/* pl */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_PL, "W≥±cz tryb przy∂pieszony podczas automatycznego startu"},
	/* ru */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_RU, "Enable warp mode during autostart"},
	/* sv */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_SV, "Aktivera warpl‰ge fˆr autostart"},
	/* tr */ {IDCLS_ENABLE_WARP_MODE_AUTOSTART_TR, "Otomatik ba˛latma esnas˝nda warp modu aktif et"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART,    N_("Disable warp mode during autostart")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_DA, "DeaktivÈr warp-tilstand under autostart"},
	/* de */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_DE, "Warp Mode w‰hren Autostart deaktivieren"},
	/* es */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_ES, "Deshabilitar modo ultrarr·pido durante autoarranque"},
	/* fr */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_FR, "DÈsactiver le mode Turbo"},
	/* hu */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_HU, "Hipergyors mÛd tilt·sa autostartn·l"},
	/* it */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_IT, "Disattiva la modalit‡ turbo all'avvio automatico"},
	/* ko */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_NL, "Warp modus tijdens autostarten uitschakelen"},
	/* pl */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_PL, "Wy≥±cz tryb przy∂pieszony podczas automatycznego startu"},
	/* ru */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_RU, "Disable warp mode during autostart"},
	/* sv */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_SV, "Inaktivera warpl‰ge fˆr autostart"},
	/* tr */ {IDCLS_DISABLE_WARP_MODE_AUTOSTART_TR, "Otomatik ba˛latma esnas˝nda warp modu pasifle˛tir"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG,    N_("Set autostart mode for PRG files (0: VirtualFS, 1: Inject, 2: Disk image)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_DE, "Autstart Modus f¸r PRG Dateien definieren (0: VirtualFS, 1:Inject, 2:\nDiskimage)"},
	/* es */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_ES, "Seleccionar modo autoarranque para ficheros PRG (0: VirtualFS, 1: Inject, 2: Imagen disco)"},
	/* fr */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_IT, "Imposta avvio automatico per i file PRG (0: VirtualFS, 1: Inject, 2: Disk image)"},
	/* ko */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_NL, "Stel autostart modus in voor PRG bestanden (0: VirtualFS, 1: Injecteer, 2: Disk bestand)"},
	/* pl */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_PL, "Okre∂l tryb autostartu z plikÛw PRG (0: wirtualny system plikÛw, 1: wstrzykniÍcie, 2: obraz dysku)"},
	/* ru */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_AUTOSTART_MODE_FOR_PRG_TR, ""},  /* fuzzy */
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG,    N_("Set disk image for autostart of PRG files")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_DA, "VÊlg diskimage for autostart af PRG-filer"},
	/* de */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_DE, "Disk Imagedatei f¸r Autstart w‰hlen"},
	/* es */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_ES, "Seleccionar imagen disco para autoarranque ficheros PRG"},
	/* fr */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_FR, "RÈgler l'image disque pour autodÈmarrage de fichiers PRG"},
	/* hu */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_HU, "Lmeez kÈpm·s megad·sa PRG automatikus indÌt·shoz"},
	/* it */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_IT, "Imposta immagine disco per l'avvio automatico dei file PRG"},
	/* ko */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_NL, "Stel disk bestand in voor het autostarten van PRG bestanden"},
	/* pl */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_PL, "Okre∂l obraz dysku do autostartu z plikÛw PRG"},
	/* ru */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_RU, "Set disk image for autostart of PRG files"},
	/* sv */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_SV, "Ange diskavbildning fˆr autostart av PRG-filer"},
	/* tr */ {IDCLS_SET_DISK_IMAGE_FOR_AUTOSTART_PRG_TR, "PRG dosyalar˝n˝n otomatik ba˛lat˝lmas˝ iÁin disk imaj˝ ayarla"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_SET_AUTOSTART_DELAY,    N_("Set initial autostart delay (0: use default)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_AUTOSTART_DELAY_DA, "Angiv autostart-forsinkelse (0: brug standard)"},
	/* de */ {IDCLS_SET_AUTOSTART_DELAY_DE, "Setze initiale Autostart Verzˆgerung (0: benutze Standard)"},
	/* es */ {IDCLS_SET_AUTOSTART_DELAY_ES, "Seleccionar retardo inicial autoarranque (Por defecto 0)"},
	/* fr */ {IDCLS_SET_AUTOSTART_DELAY_FR, "DÈfinir le dÈlai initial d'autodÈmarrage (0: utiliser dÈfaut)"},
	/* hu */ {IDCLS_SET_AUTOSTART_DELAY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_AUTOSTART_DELAY_IT, "Imposta ritardo iniziale all'avvio automatico (0: usa predefinito)"},
	/* ko */ {IDCLS_SET_AUTOSTART_DELAY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_AUTOSTART_DELAY_NL, "Stel vertraging in bij begin van het autostarten (0: gebruik de standaard waarde)"},
	/* pl */ {IDCLS_SET_AUTOSTART_DELAY_PL, "Okre∂l pocz±tkowe opÛºnienie autostartu (0: domy∂lne)"},
	/* ru */ {IDCLS_SET_AUTOSTART_DELAY_RU, "Set initial autostart delay (0: use default)"},
	/* sv */ {IDCLS_SET_AUTOSTART_DELAY_SV, "Ange inledande fˆrdrˆjning fˆr autostart (0: anv‰nd standard)"},
	/* tr */ {IDCLS_SET_AUTOSTART_DELAY_TR, "Otomatik ba˛latmada rastgele gecikmeyi ayarla (0: varsay˝lan˝ kullan)"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY,    N_("Enable random initial autostart delay.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_DA, "AktivÈr tilfÊldig autostart-forsinkelse"},
	/* de */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_DE, "Zuf‰lle Verzˆgerung w‰hrend Autostart aktivieren,"},
	/* es */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_ES, "Permitir retardo arranque inicial aleatorio."},
	/* fr */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_FR, "Activer le Mode Turbo"},
	/* hu */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_IT, "Attiva ritardo casuale all'avvio automatico."},
	/* ko */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_NL, "Willekeurige vertraging bij begin van het autostarten inschakelen."},
	/* pl */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_PL, "W≥±cz losowe opÛºnienie na pocz±tku autostartu."},
	/* ru */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_RU, "Enable random initial autostart delay."},
	/* sv */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_SV, "Aktivera slumpm‰ssig inledande autostartfˆrdrˆjning"},
	/* tr */ {IDCLS_ENABLE_AUTOSTART_RANDOM_DELAY_TR, "Otomatik ba˛latmada rastgele gecikmeyi aktif et"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY,    N_("Disable random initial autostart delay.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_DA, "DeaktivÈr tilfÊldig autostart-forsinkelse"},
	/* de */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_DE, "Zuf‰lle Verzˆgerung w‰hrend Autostart deaktivieren."},
	/* es */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_ES, "Deshabilitar retardo arranque inicial aleatorio."},
	/* fr */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_FR, "DÈsactiver le dÈlai alÈatoire initial d'autodÈmarrage."},
	/* hu */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_IT, "Disattiva ritardo casuale all'avvio automatico."},
	/* ko */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_NL, "Willekeurige vertraging bij begin van het autostarten uitschakelen."},
	/* pl */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_PL, "Wy≥±cz losowe opÛºnienie na pocz±tku autostartu."},
	/* ru */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_RU, "Disable random initial autostart delay."},
	/* sv */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_SV, "Inaktivera slumpm‰ssig inledande autostartfˆrdrˆjning"},
	/* tr */ {IDCLS_DISABLE_AUTOSTART_RANDOM_DELAY_TR, "Otomatik ba˛latmada rastgele gecikmeyi pasifle˛tir"},
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_VFLI_MOD,    N_("Enable VFLI modification.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_VFLI_MOD_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_VFLI_MOD_DE, "VFLI Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_VFLI_MOD_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_VFLI_MOD_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_VFLI_MOD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_VFLI_MOD_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_VFLI_MOD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_VFLI_MOD_NL, "VFLI modificatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_VFLI_MOD_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_VFLI_MOD_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_VFLI_MOD_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_VFLI_MOD_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_VFLI_MOD,    N_("Disable VFLI modification.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_VFLI_MOD_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_VFLI_MOD_DE, "VFLI Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_VFLI_MOD_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_VFLI_MOD_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_VFLI_MOD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_VFLI_MOD_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_VFLI_MOD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_VFLI_MOD_NL, "VFLI modificatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_VFLI_MOD_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_VFLI_MOD_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_VFLI_MOD_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_VFLI_MOD_TR, ""},  /* fuzzy */
#endif
	
	/* machine.c */
	/* en */ {IDCLS_SET_MACHINE_JAM_ACTION,    N_("Set action on CPU JAM: (0: Ask, 1: continue, 2: Monitor, 3: Reset, 4: Hard Reset, 5: Quit Emulator)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_MACHINE_JAM_ACTION_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_MACHINE_JAM_ACTION_DE, "Aktion bei CPU JAM: (0: Nachfrage, 1: Weitermachen, 2: Monitor, 3:\nReset, 4: Hard Reset, 5: Emulator beenden)"},
	/* es */ {IDCLS_SET_MACHINE_JAM_ACTION_ES, "Seleccionar action on CPU JAM: (0: Preguntar, 1: continuar, 2: Monitor, 3: Reinicio, 4: Reinicio frio, 5: Salir emulador)"},
	/* fr */ {IDCLS_SET_MACHINE_JAM_ACTION_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_MACHINE_JAM_ACTION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_MACHINE_JAM_ACTION_IT, "Imposta azione al JAM della CPU: (0: Chiedi, 1: continua, 2: Monitor, 3: Reset, 4: Hard Reset, 5: Esci)"},
	/* ko */ {IDCLS_SET_MACHINE_JAM_ACTION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_MACHINE_JAM_ACTION_NL, "Geef de actie op bij een CPU VASTLOPER: (0: Vraag, 1: Doorgaan, 2: Monitor, 3: Reset, 4: Harde Reset, 5: Emulator uitschakelen)"},
	/* pl */ {IDCLS_SET_MACHINE_JAM_ACTION_PL, "Ustal dzia≥anie przy zatkaniu siÍ procesora: (0: zapytanie, 1: kontynuacja, 2: monitorowanie, 3: reset, 4: twardy reset, 5: wyj∂cie z emulatora)"},
	/* ru */ {IDCLS_SET_MACHINE_JAM_ACTION_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_MACHINE_JAM_ACTION_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_MACHINE_JAM_ACTION_TR, ""},  /* fuzzy */
#endif
	
	/* machine.c */
	/* en */ {IDCLS_SET_EXIT_SCREENSHOT,    N_("Set name of screenshot to save when emulator exits.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_EXIT_SCREENSHOT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_EXIT_SCREENSHOT_DE, "Name f¸r Screenshot Datei beim Beenden."},
	/* es */ {IDCLS_SET_EXIT_SCREENSHOT_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_EXIT_SCREENSHOT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_EXIT_SCREENSHOT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_EXIT_SCREENSHOT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_EXIT_SCREENSHOT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_EXIT_SCREENSHOT_NL, "Geef de naam op van de schermafdruk die wordt opgeslagen wanneer de emulator wordt afgesloten."},
	/* pl */ {IDCLS_SET_EXIT_SCREENSHOT_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SET_EXIT_SCREENSHOT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_EXIT_SCREENSHOT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_EXIT_SCREENSHOT_TR, ""},  /* fuzzy */
#endif
	
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c,
	 c64/psid.c, plus4/plus4-cmdline-options.c,
	 vic20/vic20-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_USE_PAL_SYNC_FACTOR,    N_("Use PAL sync factor")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_PAL_SYNC_FACTOR_DA, "Anvend PAL-synkfaktor"},
	/* de */ {IDCLS_USE_PAL_SYNC_FACTOR_DE, "PAL Sync Faktor benutzen"},
	/* es */ {IDCLS_USE_PAL_SYNC_FACTOR_ES, "Usar factor sincronizaciÛn PAL"},
	/* fr */ {IDCLS_USE_PAL_SYNC_FACTOR_FR, "Utiliser PAL"},
	/* hu */ {IDCLS_USE_PAL_SYNC_FACTOR_HU, "PAL szinkron faktor haszn·lata"},
	/* it */ {IDCLS_USE_PAL_SYNC_FACTOR_IT, "Usa fattore di sincronizzazione PAL"},
	/* ko */ {IDCLS_USE_PAL_SYNC_FACTOR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_PAL_SYNC_FACTOR_NL, "Gebruik PAL synchronisatiefactor"},
	/* pl */ {IDCLS_USE_PAL_SYNC_FACTOR_PL, "Uøywaj synchronizacji PAL"},
	/* ru */ {IDCLS_USE_PAL_SYNC_FACTOR_RU, "Use PAL sync factor"},
	/* sv */ {IDCLS_USE_PAL_SYNC_FACTOR_SV, "Anv‰nd PAL-synkfaktor"},
	/* tr */ {IDCLS_USE_PAL_SYNC_FACTOR_TR, "PAL senkron faktˆr¸ kullan"},
#endif
	
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c,
	 c64/psid.c, plus4/plus4-cmdline-options.c,
	 vic20/vic20-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_USE_NTSC_SYNC_FACTOR,    N_("Use NTSC sync factor")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_NTSC_SYNC_FACTOR_DA, "Anvend NTSC-synkfaktor"},
	/* de */ {IDCLS_USE_NTSC_SYNC_FACTOR_DE, "NTSC Sync Faktor benutzen"},
	/* es */ {IDCLS_USE_NTSC_SYNC_FACTOR_ES, "Usar factor sincronizaciÛn NTSC"},
	/* fr */ {IDCLS_USE_NTSC_SYNC_FACTOR_FR, "Utiliser NTSC"},
	/* hu */ {IDCLS_USE_NTSC_SYNC_FACTOR_HU, "NTSC szinkron faktor haszn·lata"},
	/* it */ {IDCLS_USE_NTSC_SYNC_FACTOR_IT, "Usa fattore di sincronizzazione NTSC"},
	/* ko */ {IDCLS_USE_NTSC_SYNC_FACTOR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_NTSC_SYNC_FACTOR_NL, "Gebruik NTSC synchronisatiefactor"},
	/* pl */ {IDCLS_USE_NTSC_SYNC_FACTOR_PL, "Uøywaj synchronizacji NTSC"},
	/* ru */ {IDCLS_USE_NTSC_SYNC_FACTOR_RU, "Use NTSC sync factor"},
	/* sv */ {IDCLS_USE_NTSC_SYNC_FACTOR_SV, "Anv‰nd NTSC-synkfaktor"},
	/* tr */ {IDCLS_USE_NTSC_SYNC_FACTOR_TR, "NTSC senkron faktˆr¸ kullan"},
#endif
	
	/* c64/c64-cmdline-options.c, c64/psid.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_USE_PALN_SYNC_FACTOR,    N_("Use PAL-N sync factor")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_PALN_SYNC_FACTOR_DA, "Anvend PAL-N synkfaktor"},
	/* de */ {IDCLS_USE_PALN_SYNC_FACTOR_DE, "PAL Sync Faktor benutzen"},
	/* es */ {IDCLS_USE_PALN_SYNC_FACTOR_ES, "Usar factor sincronizaciÛn PAL-N"},
	/* fr */ {IDCLS_USE_PALN_SYNC_FACTOR_FR, "Utiliser le facteur de synchronisation PAL-N"},
	/* hu */ {IDCLS_USE_PALN_SYNC_FACTOR_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_USE_PALN_SYNC_FACTOR_IT, "Usa fattore di sincronizzazione PAL-N"},
	/* ko */ {IDCLS_USE_PALN_SYNC_FACTOR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_PALN_SYNC_FACTOR_NL, "Gebruik PAL-N synchronisatiefactor"},
	/* pl */ {IDCLS_USE_PALN_SYNC_FACTOR_PL, "Uøywaj synchronizacji PAL-N"},
	/* ru */ {IDCLS_USE_PALN_SYNC_FACTOR_RU, "Use PAL-N sync factor"},
	/* sv */ {IDCLS_USE_PALN_SYNC_FACTOR_SV, "Anv‰nd PAL N-synkfaktor"},
	/* tr */ {IDCLS_USE_PALN_SYNC_FACTOR_TR, "PAL-N senkron faktˆr¸ kullan"},
#endif
	
	/* c64/c64-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SET_BURST_MOD,    N_("Burst modification (0 = None, 1 = CIA1, 2 = CIA2)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_BURST_MOD_DA, "Burst modification (0 = ingen, 1 = CIA1, 2 = CIA2)"},
	/* de */ {IDCLS_SET_BURST_MOD_DE, "Burst Modifikation (0 = Kein, 1 = CIA1, 2 = CIA2)"},
	/* es */ {IDCLS_SET_BURST_MOD_ES, "ModificaciÛn arranque (0=No, 1=CIA1, 2=CIA2)"},
	/* fr */ {IDCLS_SET_BURST_MOD_FR, "Modification Burst (0 = Aucune, 1 = CIA1, 2 = CIA2)"},
	/* hu */ {IDCLS_SET_BURST_MOD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_BURST_MOD_IT, "Burst modification (0 = Nessuna, 1 = CIA1, 2 = CIA2)"},
	/* ko */ {IDCLS_SET_BURST_MOD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_BURST_MOD_NL, "Burst modificatie (0 = Geen, 1 = CIA1, 2 = CIA2)"},
	/* pl */ {IDCLS_SET_BURST_MOD_PL, "Modyfikacja trybu synchronicznego (0 = Brak, 1 = CIA1, 2 = CIA2)"},
	/* ru */ {IDCLS_SET_BURST_MOD_RU, "Burst modification (0 = None, 1 = CIA1, 2 = CIA2)"},
	/* sv */ {IDCLS_SET_BURST_MOD_SV, "Burstmodifiering (0 = ingen, 1 = CIA1, 2 = CIA2)"},
	/* tr */ {IDCLS_SET_BURST_MOD_TR, "Burst dei˛iklii (0 = Yok, 1 = CIA1, 2 = CIA2)"},
#endif
	
	/* c64/c64-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SET_IEC_RESET,    N_("Computer reset goes to IEC bus (0 = No, 1 = Yes)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_IEC_RESET_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_IEC_RESET_DE, "Computer Reset auf IEC Bus (0 = Nein, 1 = Ja)"},
	/* es */ {IDCLS_SET_IEC_RESET_ES, "Reinicio del ordenador va a IEC bus (0 = No, 1 = SÌ)"},
	/* fr */ {IDCLS_SET_IEC_RESET_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_IEC_RESET_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_IEC_RESET_IT, "Reset del computer va sul bus IEC (0 = No, 1 = SÏ)"},
	/* ko */ {IDCLS_SET_IEC_RESET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_IEC_RESET_NL, "Computer reset gaat naar de IEC bus (0 = Nee, 1 = Ja)"},
	/* pl */ {IDCLS_SET_IEC_RESET_PL, "Reset komputera uøywa szyny IEC (0 = nie, 1 = tak)"},
	/* ru */ {IDCLS_SET_IEC_RESET_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_IEC_RESET_SV, "Datornollst‰llning gÂr till IEC-bussen (0 = nej, 1 = ja)"},
	/* tr */ {IDCLS_SET_IEC_RESET_TR, ""},  /* fuzzy */
#endif
	
	/* c128/c128-cmdline-options.c, c128/functionrom.c,
	 c64/c64-cmdline-options.c, c64/cart/georam.c,
	 c64/cart/ramcart.c, c64/cart/reu.c, c64/cart/c64cart.c,
	 c64/cart/ide64.c, cbm2/cbm2-cmdline-options.c,
	 drive/iec/iec-cmdline-options.c, drive/iec128dcr/iec128dcr-cmdline-options.c,
	 drive/ieee/ieee-cmdline-options.c, drive/tcbm/tcbm-cmdline-options.c,
	 fsdevice/fsdevice-cmdline-options.c, pet/pet-cmdline-options.c,
	 plus4/plus4-cmdline-options.c, printerdrv/driver-select.c,
	 printerdrv/output-select.c, printerdrv/output-text.c,
	 rs232drv/rs232drv.c, vic20/vic20-cmdline-options.c,
	 vic20/vic20cartridge.c, video/video-cmdline-options.c,
	 fliplist.c, initcmdline.c, log.c, sound.c, c64/plus60k.c,
	 c64/c64_256k.c, pet/petreu.c, c64/plus256k.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_P_NAME,    N_("<Name>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_NAME_DE, "<Name>"},
	/* es */ {IDCLS_P_NAME_ES, "<nombre>"},
	/* fr */ {IDCLS_P_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_NAME_IT, "<Nome>"},
	/* ko */ {IDCLS_P_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_NAME_NL, "<Naam>"},
	/* pl */ {IDCLS_P_NAME_PL, "<Nazwa>"},
	/* ru */ {IDCLS_P_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_INT_KERNEL_NAME,    N_("Specify name of international Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_INT_KERNEL_NAME_DA, "Angiv navn pÂ internationalt kerne-ROM-image"},
	/* de */ {IDCLS_SPECIFY_INT_KERNEL_NAME_DE, "Namen f¸r internationale Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_INT_KERNEL_NAME_ES, "Especificar nombre imagen ROM Kernal internacional"},
	/* fr */ {IDCLS_SPECIFY_INT_KERNEL_NAME_FR, "SpÈcifier le nom de l'image Kernal internationale"},
	/* hu */ {IDCLS_SPECIFY_INT_KERNEL_NAME_HU, "Adja meg a nemzetkˆzi Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_INT_KERNEL_NAME_IT, "Specifica il nome immagine della ROM del Kernal internazionale"},
	/* ko */ {IDCLS_SPECIFY_INT_KERNEL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_INT_KERNEL_NAME_NL, "Geef de naam van het internationaal Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_INT_KERNEL_NAME_PL, "Okre∂l nazwÍ obrazu miÍdzynarodowego ROM-u Kernala"},
	/* ru */ {IDCLS_SPECIFY_INT_KERNEL_NAME_RU, "Specify name of international Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_INT_KERNEL_NAME_SV, "Ange namn pÂ internationell kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_INT_KERNEL_NAME_TR, "Uluslararas˝ Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_DE_KERNEL_NAME,    N_("Specify name of German Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_DE_KERNEL_NAME_DA, "Angiv navn pÂ tysk kerne-ROM-image"},
	/* de */ {IDCLS_SPECIFY_DE_KERNEL_NAME_DE, "Name f¸r deutsche Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_DE_KERNEL_NAME_ES, "Especificar nombre imagen ROM Kernal alem·n"},
	/* fr */ {IDCLS_SPECIFY_DE_KERNEL_NAME_FR, "SpÈcifier le nom de l'image Kernal allemande"},
	/* hu */ {IDCLS_SPECIFY_DE_KERNEL_NAME_HU, "Adja meg a nÈmet Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_DE_KERNEL_NAME_IT, "Specifica il nome immagine della ROM del Kernal tedesca"},
	/* ko */ {IDCLS_SPECIFY_DE_KERNEL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_DE_KERNEL_NAME_NL, "Geef de naam van het Duitse Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_DE_KERNEL_NAME_PL, "Podaj nazwÍ obrazu niemieckiego ROM-u Kernala"},
	/* ru */ {IDCLS_SPECIFY_DE_KERNEL_NAME_RU, "Specify name of German Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_DE_KERNEL_NAME_SV, "Ange namn pÂ tysk kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_DE_KERNEL_NAME_TR, "Alman Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_FI_KERNEL_NAME,    N_("Specify name of Finnish Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_FI_KERNEL_NAME_DA, "Angiv navn pÂ finsk kerne-ROM-image"},
	/* de */ {IDCLS_SPECIFY_FI_KERNEL_NAME_DE, "Name f¸r finnische Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_FI_KERNEL_NAME_ES, "Especificar nombre imagen ROM Kernal finlandÈs"},
	/* fr */ {IDCLS_SPECIFY_FI_KERNEL_NAME_FR, "SpÈcifier le nom de l'image Kernal finlandaise"},
	/* hu */ {IDCLS_SPECIFY_FI_KERNEL_NAME_HU, "Adja meg a finn Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_FI_KERNEL_NAME_IT, "Specifica il nome immagine della ROM del Kernal finlandese"},
	/* ko */ {IDCLS_SPECIFY_FI_KERNEL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_FI_KERNEL_NAME_NL, "Geef de naam van het Finse Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_FI_KERNEL_NAME_PL, "Okre∂l nazwÍ obrazu fiÒskiego ROM-u Kernala"},
	/* ru */ {IDCLS_SPECIFY_FI_KERNEL_NAME_RU, "Specify name of Finnish Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_FI_KERNEL_NAME_SV, "Ange namn pÂ finsk kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_FI_KERNEL_NAME_TR, "Finlandiya Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_FR_KERNEL_NAME,    N_("Specify name of French Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_FR_KERNEL_NAME_DA, "Angiv navn pÂ fransk kerne-ROM-image"},
	/* de */ {IDCLS_SPECIFY_FR_KERNEL_NAME_DE, "Name f¸r franzˆsische Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_FR_KERNEL_NAME_ES, "Especificar nombre imagen ROM Kernal francÈs"},
	/* fr */ {IDCLS_SPECIFY_FR_KERNEL_NAME_FR, "SpÈcifier le nom de l'image Kernal franÁaise"},
	/* hu */ {IDCLS_SPECIFY_FR_KERNEL_NAME_HU, "Adja meg a francia Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_FR_KERNEL_NAME_IT, "Specifica il nome immagine della ROM del Kernal francese"},
	/* ko */ {IDCLS_SPECIFY_FR_KERNEL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_FR_KERNEL_NAME_NL, "Geef de naam van het Franse Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_FR_KERNEL_NAME_PL, "Okre∂l nazwÍ obrazu francuskiego ROM-u Kernala"},
	/* ru */ {IDCLS_SPECIFY_FR_KERNEL_NAME_RU, "Specify name of French Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_FR_KERNEL_NAME_SV, "Ange namn pÂ fransk kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_FR_KERNEL_NAME_TR, "Frans˝z Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_IT_KERNEL_NAME,    N_("Specify name of Italian Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_IT_KERNEL_NAME_DA, "Angiv navn pÂ italiensk kerne-ROM-image"},
	/* de */ {IDCLS_SPECIFY_IT_KERNEL_NAME_DE, "Name f¸r italienische Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_IT_KERNEL_NAME_ES, "Especificar nombre imagen ROM Kernal italiano"},
	/* fr */ {IDCLS_SPECIFY_IT_KERNEL_NAME_FR, "SpÈcifier le nom de l'image Kernal italienne"},
	/* hu */ {IDCLS_SPECIFY_IT_KERNEL_NAME_HU, "Adja meg az olasz Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_IT_KERNEL_NAME_IT, "Specifica il nome immagine della ROM del Kernal italiana"},
	/* ko */ {IDCLS_SPECIFY_IT_KERNEL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_IT_KERNEL_NAME_NL, "Geef de naam van het Italiaanse Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_IT_KERNEL_NAME_PL, "Okre∂l nazwÍ obrazu w≥oskiego ROM-u Kernala"},
	/* ru */ {IDCLS_SPECIFY_IT_KERNEL_NAME_RU, "Specify name of Italian Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_IT_KERNEL_NAME_SV, "Ange namn pÂ italiensk kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_IT_KERNEL_NAME_TR, "›talyan Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_NO_KERNEL_NAME,    N_("Specify name of Norwegian Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_NO_KERNEL_NAME_DA, "Angiv navn pÂ norsk kerne-ROM-image"},
	/* de */ {IDCLS_SPECIFY_NO_KERNEL_NAME_DE, "Name f¸r norwegische Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_NO_KERNEL_NAME_ES, "Especificar nombre imagen ROM noruego"},
	/* fr */ {IDCLS_SPECIFY_NO_KERNEL_NAME_FR, "SpÈcifier le nom de l'image Kernal norvÈgienne"},
	/* hu */ {IDCLS_SPECIFY_NO_KERNEL_NAME_HU, "Adja meg a norvÈg Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_NO_KERNEL_NAME_IT, "Specifica il nome immagine della ROM del Kernal norvegese"},
	/* ko */ {IDCLS_SPECIFY_NO_KERNEL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_NO_KERNEL_NAME_NL, "Geef de naam van het Noorse Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_NO_KERNEL_NAME_PL, "Okre∂l nazwÍ obrazu norweskiego ROM-u Kernala"},
	/* ru */ {IDCLS_SPECIFY_NO_KERNEL_NAME_RU, "Specify name of Norwegian Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_NO_KERNEL_NAME_SV, "Ange namn pÂ norsk kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_NO_KERNEL_NAME_TR, "NorveÁ Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SV_KERNEL_NAME,    N_("Specify name of Swedish Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SV_KERNEL_NAME_DA, "Angiv navn pÂ svensk kerne-ROM-image"},
	/* de */ {IDCLS_SPECIFY_SV_KERNEL_NAME_DE, "Name f¸r schwedische Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_SV_KERNEL_NAME_ES, "Especificar nombre imagen ROM Kernal sueco"},
	/* fr */ {IDCLS_SPECIFY_SV_KERNEL_NAME_FR, "SpÈcifier le nom de l'image Kernal suÈdoise"},
	/* hu */ {IDCLS_SPECIFY_SV_KERNEL_NAME_HU, "Adja meg s svÈd Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_SV_KERNEL_NAME_IT, "Specifica il nome immagine della ROM del Kernal svedese"},
	/* ko */ {IDCLS_SPECIFY_SV_KERNEL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SV_KERNEL_NAME_NL, "Geef de naam van het Zweedse Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_SV_KERNEL_NAME_PL, "Okre∂l nazwÍ obrazu szwedzkiego ROM-u Kernala"},
	/* ru */ {IDCLS_SPECIFY_SV_KERNEL_NAME_RU, "Specify name of Swedish Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_SV_KERNEL_NAME_SV, "Ange namn pÂ svensk kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_SV_KERNEL_NAME_TR, "›sveÁ Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CH_KERNEL_NAME,    N_("Specify name of Swiss Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CH_KERNEL_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_CH_KERNEL_NAME_DE, "Name f¸r schweizer Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_CH_KERNEL_NAME_ES, "Especificar nombre imagen Kernal ROM suizo"},
	/* fr */ {IDCLS_SPECIFY_CH_KERNEL_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_CH_KERNEL_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_CH_KERNEL_NAME_IT, "Specifica il nome immagine della ROM del Kernal svizzera"},
	/* ko */ {IDCLS_SPECIFY_CH_KERNEL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CH_KERNEL_NAME_NL, "Geef de naam van het Zwitserse Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_CH_KERNEL_NAME_PL, "Okre∂l nazwÍ obrazu szwajcarskiego ROM-u Kernala"},
	/* ru */ {IDCLS_SPECIFY_CH_KERNEL_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_CH_KERNEL_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_CH_KERNEL_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW,    N_("Specify name of BASIC ROM image (lower part)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_DA, "Angiv navn pÂ BASIC ROM-image (lav del)"},
	/* de */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_DE, "Namen f¸r Basic ROM Datei spezifizieren (unterer Teil)"},
	/* es */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_ES, "Especificar nombre imagen BASIC ROM (parte baja)"},
	/* fr */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_FR, "SpÈcifier le nom de l'image BASIC ROM (partie basse)"},
	/* hu */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_HU, "Adja meg a BASIC ROM kÈpm·s nevÈt (alsÛ rÈsz)"},
	/* it */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_IT, "Specifica il nome immagine della ROM del BASIC (parte inferiore)"},
	/* ko */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_NL, "Geef de naam van het BASIC ROM bestand (laag gedeelte)"},
	/* pl */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_PL, "Okre∂l nazwÍ obrazu ROM BASIC (niøsza partia)"},
	/* ru */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_RU, "Specify name of BASIC ROM image (lower part)"},
	/* sv */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_SV, "Ange namn pÂ BASIC ROM-avbildning (nedre delen)"},
	/* tr */ {IDCLS_SPECIFY_BASIC_ROM_NAME_LOW_TR, "BASIC ROM imaj˝n˝n (alt k˝s˝m) ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH,    N_("Specify name of BASIC ROM image (higher part)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_DA, "Angiv navn pÂ BASIC ROM-image (h¯j del)"},
	/* de */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_DE, "Namen f¸r Basic ROM Datei spezifizieren (oberer Teil)"},
	/* es */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_ES, "Especificar nombre imagen BASIC ROM (parte alta)"},
	/* fr */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_FR, "SpÈcifier le nom de l'image BASIC ROM (partie haute)"},
	/* hu */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_HU, "Adja meg a BASIC ROM kÈpm·s nevÈt (felsı rÈsz)"},
	/* it */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_IT, "Specifica il nome immagine della ROM del BASIC (parte superiore)"},
	/* ko */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_NL, "Geef de naam van het BASIC ROM bestand (hoog gedeelte)"},
	/* pl */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_PL, "Okre∂l nazwÍ obrazu ROM BASIC (wyøsza partia)"},
	/* ru */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_RU, "Specify name of BASIC ROM image (higher part)"},
	/* sv */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_SV, "Ange namn pÂ BASIC ROM-avbildning (ˆvre delen)"},
	/* tr */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HIGH_TR, "BASIC ROM imaj˝n˝n (¸st k˝s˝m) ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME,    N_("Specify name of international character generator ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_DA, "Angiv navn pÂ internationalt tegngenerator-ROM-image"},
	/* de */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_DE, "Name f¸r internationale Charakter ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_ES, "Especificar nombre imagen ROM generador caracteres internacional"},
	/* fr */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_FR, "SpÈcifier le nom de l'image du gÈnÈrateur de caractËres internationaux"},
	/* hu */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_HU, "Adja meg a nemzetkˆzi karaktergener·tor ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_IT, "Specifica il nome immagine della ROM del generatore di caratteri internazionale"},
	/* ko */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_NL, "Geef de naam van het internationaal CHARGEN ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM generatora znakÛw miÍdzynarodowych"},
	/* ru */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_RU, "Specify name of international character generator ROM image"},
	/* sv */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_SV, "Ange namn pÂ internationell teckengenerator-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_INT_CHARGEN_ROM_NAME_TR, "Uluslararas˝ Karakter Olu˛turucu ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME,    N_("Specify name of German character generator ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_DA, "Angiv navn pÂ tysk tegngenerator-ROM-image"},
	/* de */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_DE, "Name f¸r deutsche Charakter ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_ES, "Especificar nombre imagen ROM generador caracteres alem·n"},
	/* fr */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_FR, "SpÈcifier le nom de l'image du gÈnÈrateur de caractËres allemands"},
	/* hu */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_HU, "Adja meg a nÈmet karaktergener·tor ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_IT, "Specifica il nome immagine della ROM del generatore di caratteri tedesco"},
	/* ko */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_NL, "Geef de naam van het Duitse CHARGEN ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM generatora niemieckich znakÛw"},
	/* ru */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_RU, "√⁄–◊–‚Ï ÿ‹Ô ROM ﬁ—‡–◊– ”’›’‡–‚ﬁ‡– ›’‹’Ê⁄ÿÂ ·ÿ‹“ﬁ€ﬁ“"},
	/* sv */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_SV, "Ange namn pÂ tysk teckengenerator-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_DE_CHARGEN_ROM_NAME_TR, "Alman Karakter Olu˛turucu ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME,    N_("Specify name of French character generator ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_DA, "Angiv navn pÂ fransk tegngenerator-ROM-image"},
	/* de */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_DE, "Name f¸r franzˆsische Charakter ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_ES, "Especificar nombre imagen ROM generador caracteres francÈs"},
	/* fr */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_FR, "SpÈcifier le nom de l'image du gÈnÈrateur de caractËres franÁais"},
	/* hu */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_HU, "Adja meg a francia karaktergener·tor ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_IT, "Specifica il nome immagine della ROM del generatore di caratteri francese"},
	/* ko */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_NL, "Geef de naam van het Franse CHARGEN ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM generatora francuskich znakÛw"},
	/* ru */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_RU, "√⁄–◊–‚Ï ÿ‹Ô ROM ﬁ—‡–◊– ”’›’‡–‚ﬁ‡– ‰‡–›Ê„◊·⁄ÿÂ ·ÿ‹“ﬁ€ﬁ“"},
	/* sv */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_SV, "Ange namn pÂ fransk teckengenerator-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_FR_CHARGEN_ROM_NAME_TR, "Finlandiya Karakter Olu˛turucu ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME,    N_("Specify name of Swedish character generator ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_DA, "Angiv navn pÂ svensk tegngenerator-ROM-image"},
	/* de */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_DE, "Name f¸r schwedische Charakter ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_ES, "Especificar nombre imagen ROM generador caracteres sueco"},
	/* fr */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_FR, "SpÈcifier le nom de l'image du gÈnÈrateur de caractËres suÈdois"},
	/* hu */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_HU, "Adja meg a svÈd karaktergener·tor ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_IT, "Specifica il nome immagine della ROM del generatore di caratteri svedese"},
	/* ko */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_NL, "Geef de naam van het Zweedse CHARGEN ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM generatora szwedzkich znakÛw"},
	/* ru */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_RU, "√⁄–◊–‚Ï ÿ‹Ô ROM ﬁ—‡–◊– ”’›’‡–‚ﬁ‡– Ë“’‘·⁄ÿÂ ·ÿ‹“ﬁ€ﬁ“"},
	/* sv */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_SV, "Ange namn pÂ svensk teckengenerator-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_SV_CHARGEN_ROM_NAME_TR, "›sveÁ Karakter Olu˛turucu ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME,    N_("Specify name of Swedish character generator ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_DA, "Angiv navn pÂ svensk tegngenerator-ROM-image"},
	/* de */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_DE, "Name f¸r schwedische Charakter ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_ES, "Especificar nombre imagen ROM generador caracteres sueco"},
	/* fr */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_FR, "SpÈcifier le nom de l'image du gÈnÈrateur de caractËres suÈdois"},
	/* hu */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_HU, "Adja meg a svÈd karaktergener·tor ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_IT, "Specifica il nome immagine della ROM del generatore di caratteri svedese"},
	/* ko */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_NL, "Geef de naam van het Zweedse CHARGEN ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM generatora szwedzkich znakÛw"},
	/* ru */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_RU, "√⁄–◊–‚Ï ÿ‹Ô ROM ﬁ—‡–◊– ”’›’‡–‚ﬁ‡– Ë“’‘·⁄ÿÂ ·ÿ‹“ﬁ€ﬁ“"},
	/* sv */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_SV, "Ange namn pÂ svensk teckengenerator-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_CH_CHARGEN_ROM_NAME_TR, "›sveÁ Karakter Olu˛turucu ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME,    N_("Specify name of C64 mode Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_DA, "Angiv navn pÂ C64-kerne-ROM-image"},
	/* de */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_DE, "Name f¸r C64 Modus Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_ES, "Especificar nombre imagen ROM Kernal modo C64"},
	/* fr */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_FR, "SpÈcifier le nom de l'image Kernal du mode C64"},
	/* hu */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_HU, "Adja meg a C64 mÛd˙ Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_IT, "Specifica il nome immagine della ROM del Kernal in modalit‡ C64"},
	/* ko */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_NL, "Geef de naam van het Kernal ROM bestand voor de C64 modus"},
	/* pl */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_PL, "Okre∂l nazwÍ obrazu ROM Kernala trybu C64"},
	/* ru */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_RU, "Specify name of C64 mode Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_SV, "Ange namn pÂ C64-kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_C64_MODE_KERNAL_NAME_TR, "C64 modu Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME,    N_("Specify name of C64 mode BASIC ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_DA, "Angiv navn pÂ C64-BASIC-ROM-image"},
	/* de */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_DE, "Name f¸r C64 Modus Basic ROM Datei spezifizieren"},
	/* es */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_ES, "Especificar nombre imagen BASIC ROM modo C64"},
	/* fr */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_FR, "SpÈcifier le nom de l'image BASIC ROM du mode C64"},
	/* hu */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_HU, "Adja meg a C64 mÛd˙ BASIC ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_IT, "Specifica il nome immagine della ROM in modalit‡ C64"},
	/* ko */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_NL, "Geef de naam van het BASIC ROM bestand voor de C64 modus"},
	/* pl */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_PL, "Okre∂l nazwÍ obrazu ROM BASIC trybu C64"},
	/* ru */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_RU, "√⁄–◊–‚Ï ÿ‹Ô ROM ﬁ—‡–◊– ‡’÷ÿ‹– C64 BASIC"},
	/* sv */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_SV, "Ange namn pÂ C64-BASIC-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_C64_MODE_BASIC_NAME_TR, "C64 modu BASIC ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_OEM_JOY,    N_("Enable the OEM userport joystick adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_OEM_JOY_DA, "AktivÈr OEM brugerportjoystickadapter"},
	/* de */ {IDCLS_ENABLE_OEM_JOY_DE, "OEM Userport Joystick Adapter aktivieren"},
	/* es */ {IDCLS_ENABLE_OEM_JOY_ES, "Permitir adaptador puerto joystick OEM"},
	/* fr */ {IDCLS_ENABLE_OEM_JOY_FR, "Activer l'adaptateur joystick port utilisateur OEM"},
	/* hu */ {IDCLS_ENABLE_OEM_JOY_HU, "OEM userport botkorm·ny adapter engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_OEM_JOY_IT, "Attiva adattatore joystick OEM su userport"},
	/* ko */ {IDCLS_ENABLE_OEM_JOY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_OEM_JOY_NL, "OEM userport joystick adapter inschakelen"},
	/* pl */ {IDCLS_ENABLE_OEM_JOY_PL, "W≥±cz z≥±cze OEM userportu joysticka"},
	/* ru */ {IDCLS_ENABLE_OEM_JOY_RU, "Enable the OEM userport joystick adapter"},
	/* sv */ {IDCLS_ENABLE_OEM_JOY_SV, "Aktivera OEM-anv‰ndarportspelportsadapter"},
	/* tr */ {IDCLS_ENABLE_OEM_JOY_TR, "OEM userport joystick dˆn¸˛t¸r¸c¸y¸ aktif et"},
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_OEM_JOY,    N_("Disable the OEM userport joystick adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_OEM_JOY_DA, "DeaktivÈr OEM brugerportjoystickadapter"},
	/* de */ {IDCLS_DISABLE_OEM_JOY_DE, "OEM Userport Joystick Adapter deaktivieren"},
	/* es */ {IDCLS_DISABLE_OEM_JOY_ES, "Deshabilitar adaptador puerto joystick OEM"},
	/* fr */ {IDCLS_DISABLE_OEM_JOY_FR, "DÈsactiver l'adaptateur OEM Joystick port utilisateur"},
	/* hu */ {IDCLS_DISABLE_OEM_JOY_HU, "OEM userport botkorm·ny adapter tilt·sa"},
	/* it */ {IDCLS_DISABLE_OEM_JOY_IT, "Disattiva adattatore joystick OEM su userport"},
	/* ko */ {IDCLS_DISABLE_OEM_JOY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_OEM_JOY_NL, "OEM userport joystick adapter uitschakelen"},
	/* pl */ {IDCLS_DISABLE_OEM_JOY_PL, "Wy≥±cz z≥±cze OEM userportu joysticka"},
	/* ru */ {IDCLS_DISABLE_OEM_JOY_RU, "Disable the OEM userport joystick adapter"},
	/* sv */ {IDCLS_DISABLE_OEM_JOY_SV, "Inaktivera OEM-anv‰ndarportspelportsadapter"},
	/* tr */ {IDCLS_DISABLE_OEM_JOY_TR, "OEM userport joystick dˆn¸˛t¸r¸c¸y¸ pasifle˛tir"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_IEE488,    N_("Enable the IEEE488 interface emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEE488_DA, "AktivÈr emulering af IEEE488-interface"},
	/* de */ {IDCLS_ENABLE_IEE488_DE, "IEEE488 Schnittstellen-Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_IEE488_ES, "Permitir emulaciÛn interface IEEE488"},
	/* fr */ {IDCLS_ENABLE_IEE488_FR, "Activer l'interface d'Èmulation IEEE488"},
	/* hu */ {IDCLS_ENABLE_IEE488_HU, "IEEE488 interfÈsz emul·ciÛ engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_IEE488_IT, "Attiva emulazione interfaccia IEEE488"},
	/* ko */ {IDCLS_ENABLE_IEE488_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEE488_NL, "IEEE488 interface emulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEE488_PL, "W≥±cz emulacjÍ interfejsu IEEE488"},
	/* ru */ {IDCLS_ENABLE_IEE488_RU, "Enable the IEEE488 interface emulation"},
	/* sv */ {IDCLS_ENABLE_IEE488_SV, "Aktivera emulering av IEEE488-gr‰nssnitt"},
	/* tr */ {IDCLS_ENABLE_IEE488_TR, "IEEE488 arabirim em¸lasyonunu aktif et"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_IEE488,    N_("Disable the IEEE488 interface emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEE488_DA, "DeaktivÈr emulering af IEEE488-interface"},
	/* de */ {IDCLS_DISABLE_IEE488_DE, "IEEE488 Schnittstellen-Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEE488_ES, "Deshabilitar emulaciÛn interface IEEE488"},
	/* fr */ {IDCLS_DISABLE_IEE488_FR, "DÈsactiver l'interface d'Èmulation IEEE488"},
	/* hu */ {IDCLS_DISABLE_IEE488_HU, "IEEE488 interfÈsz emul·ciÛ tilt·sa"},
	/* it */ {IDCLS_DISABLE_IEE488_IT, "Disattiva emulazione interfaccia IEEE488"},
	/* ko */ {IDCLS_DISABLE_IEE488_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEE488_NL, "IEEE488 interface emulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEE488_PL, "Wy≥±cz emulacjÍ interfejsu IEEE488"},
	/* ru */ {IDCLS_DISABLE_IEE488_RU, "Disable the IEEE488 interface emulation"},
	/* sv */ {IDCLS_DISABLE_IEE488_SV, "Inaktivera emulering av IEEE488-gr‰nssnitt"},
	/* tr */ {IDCLS_DISABLE_IEE488_TR, "IEEE488 arabirim em¸lasyonunu pasifle˛tir"},
#endif
	
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c */
	/* en */ {IDCLS_P_REVISION,    N_("<Revision>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_REVISION_DA, "<Udgave>"},
	/* de */ {IDCLS_P_REVISION_DE, "<Revision>"},
	/* es */ {IDCLS_P_REVISION_ES, "<RevisiÛn>"},
	/* fr */ {IDCLS_P_REVISION_FR, "<RÈvision>"},
	/* hu */ {IDCLS_P_REVISION_HU, "<V·ltozatsz·m>"},
	/* it */ {IDCLS_P_REVISION_IT, "<Revisione>"},
	/* ko */ {IDCLS_P_REVISION_KO, "<ºˆ¡§>"},
	/* nl */ {IDCLS_P_REVISION_NL, "<Revisie>"},
	/* pl */ {IDCLS_P_REVISION_PL, "<Wersja>"},
	/* ru */ {IDCLS_P_REVISION_RU, "<Revision>"},
	/* sv */ {IDCLS_P_REVISION_SV, "<UtgÂva>"},
	/* tr */ {IDCLS_P_REVISION_TR, "<Revizyon>"},
#endif
	
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c */
	/* en */ {IDCLS_PATCH_KERNAL_TO_REVISION,    N_("Patch the Kernal ROM to the specified <revision> (1: rev. 1, 2: rev. 2, 3: rev. 3, 67/sx: sx64, 100/4064: 4064)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PATCH_KERNAL_TO_REVISION_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PATCH_KERNAL_TO_REVISION_DE, "Kernal ROM auf <revision> modifzieren (1: rev. 1, 2: rev. 2, 3: rev. 3, 67/sx: sx64, 100/4064: 4064)"},
	/* es */ {IDCLS_PATCH_KERNAL_TO_REVISION_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_PATCH_KERNAL_TO_REVISION_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PATCH_KERNAL_TO_REVISION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PATCH_KERNAL_TO_REVISION_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_PATCH_KERNAL_TO_REVISION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PATCH_KERNAL_TO_REVISION_NL, "Verbeter de Kernal ROM naar de opgegeven <revisie> (1: rev. 1, 2: rev. 2, 3: rev. 3, 67/sx: sx64, 100/4064: 4064)"},
	/* pl */ {IDCLS_PATCH_KERNAL_TO_REVISION_PL, "Za≥ataj ROM Kernala do konkretnej <wersji> (1: rev. 1, 2: rev. 2, 3: rev. 3, 67/sx: sx64, 100/4064: 4064)"},
	/* ru */ {IDCLS_PATCH_KERNAL_TO_REVISION_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PATCH_KERNAL_TO_REVISION_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PATCH_KERNAL_TO_REVISION_TR, ""},  /* fuzzy */
#endif
	
#if defined(HAVE_RS232DEV) || defined(HAVE_RS232NET)
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c
	 scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU,    N_("Enable the ACIA RS232 interface emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_DA, "AktivÈr ACIA RS232-interface emulering"},
	/* de */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_DE, "ACIA RS232 Schnittstellen-Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_ES, "Permitir emulaciÛn interface ACIA RS232"},
	/* fr */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_FR, "Activer l'Èmulation de l'interface ACIA RS232"},
	/* hu */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_IT, "Attiva emulazione interfaccia ACIA RS232"},
	/* ko */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_NL, "ACIA RS232 interface emulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_PL, "W≥±cz emulacjÍ interfejsu ACIA RS232"},
	/* ru */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_RU, "Enable the ACIA RS232 interface emulation"},
	/* sv */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_SV, "Aktivera ACIA RS232-gr‰nssnittsemulering"},
	/* tr */ {IDCLS_ENABLE_DEXX_ACIA_RS232_EMU_TR, "ACIA RS232 arabirim em¸lasyonunu aktif et"},
#endif
	
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c,
	 scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU,    N_("Disable the ACIA RS232 interface emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_DA, "DeaktivÈr ACIA-RS232-interface emulering"},
	/* de */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_DE, "ACIA RS232 Schnittstellen-Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_ES, "Deshabilitar emulaciÛn interface ACIA RS232"},
	/* fr */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_FR, "DÈsactiver l'Èmulation de l'interface ACIA RS232"},
	/* hu */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_IT, "Disattiva emulazione interfaccia ACIA RS232"},
	/* ko */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_NL, "ACIA RS232 interface emulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_PL, "Wy≥±cz emulacjÍ interfejsu ACIA RS232"},
	/* ru */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_RU, "Disable the ACIA RS232 interface emulation"},
	/* sv */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_SV, "Inaktivera ACIA-RS232-gr‰nssnittsemulering"},
	/* tr */ {IDCLS_DISABLE_DEXX_ACIA_RS232_EMU_TR, "ACIA RS232 arabirim em¸lasyonunu pasifle˛tir"},
#endif
	
	/* c64/cart/c64acia1.c */
	/* en */ {IDCLS_P_IRQ,    N_("<interrupt>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_IRQ_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_IRQ_DE, "<Interrupt>"},
	/* es */ {IDCLS_P_IRQ_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_P_IRQ_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_IRQ_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_IRQ_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_P_IRQ_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_IRQ_NL, "<interrupt>"},
	/* pl */ {IDCLS_P_IRQ_PL, "<przerwanie>"},
	/* ru */ {IDCLS_P_IRQ_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_IRQ_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_IRQ_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/c64acia1.c */
	/* en */ {IDCLS_SET_ACIA_IRQ,    N_("Set the ACIA interrupt (0: None, 1: NMI, 2: IRQ)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_ACIA_IRQ_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_ACIA_IRQ_DE, "ACIA Interrupt setzen (0: None, 1: NMI, 2: IRQ)"},
	/* es */ {IDCLS_SET_ACIA_IRQ_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_ACIA_IRQ_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_ACIA_IRQ_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_ACIA_IRQ_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_ACIA_IRQ_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_ACIA_IRQ_NL, "Stel de ACIA interrupt in (0: Geen, 1: NMI, 2: IRQ)"},
	/* pl */ {IDCLS_SET_ACIA_IRQ_PL, "Ustaw przerwanie ACIA (0:Brak, 1: NMI, 2: IRQ)"},
	/* ru */ {IDCLS_SET_ACIA_IRQ_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_ACIA_IRQ_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_ACIA_IRQ_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/c64acia1.c */
	/* en */ {IDCLS_SET_ACIA_MODE,    N_("Set the ACIA mode (0: Normal, 1: Swiftlink, 2: Turbo232)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_ACIA_MODE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_ACIA_MODE_DE, "ACIA Modus setzen (0: Normal, 1: Swiftlink, 2: Turbo232)"},
	/* es */ {IDCLS_SET_ACIA_MODE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_ACIA_MODE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_ACIA_MODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_ACIA_MODE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_ACIA_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_ACIA_MODE_NL, "Stel de ACIA modus in (0: normaal, 1: Swiftlink, 2: Turbo232)"},
	/* pl */ {IDCLS_SET_ACIA_MODE_PL, "Okre∂l tryb ACIA (0:Zwyk≥y, 1: Swiftlink, 2: Turbo232)"},
	/* ru */ {IDCLS_SET_ACIA_MODE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_ACIA_MODE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_ACIA_MODE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/c64acia1.c */
	/* en */ {IDCLS_SET_ACIA_BASE,    N_("Set the base address of the ACIA cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_ACIA_BASE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_ACIA_BASE_DE, "Basis Adresse f¸r ACIA Erweiterung"},
	/* es */ {IDCLS_SET_ACIA_BASE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_ACIA_BASE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_ACIA_BASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_ACIA_BASE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_ACIA_BASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_ACIA_BASE_NL, "Stel het basisadres in van de ACIA cartridge"},
	/* pl */ {IDCLS_SET_ACIA_BASE_PL, "Ustaw adres bazowy kartridøa ACIA"},
	/* ru */ {IDCLS_SET_ACIA_BASE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_ACIA_BASE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_ACIA_BASE_TR, ""},  /* fuzzy */
#endif
#endif
	
	
	/* plus4/plus4acia.c */
	/* en */ {IDCLS_ENABLE_ACIA_EMU,    N_("Enable the ACIA emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_ACIA_EMU_DA, "AktivÈr ACIA-emulering"},
	/* de */ {IDCLS_ENABLE_ACIA_EMU_DE, "ACIA Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_ACIA_EMU_ES, "Habilitar emulaciÛn ACIA"},
	/* fr */ {IDCLS_ENABLE_ACIA_EMU_FR, "Activer l'Èmulation ACIA"},
	/* hu */ {IDCLS_ENABLE_ACIA_EMU_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_ACIA_EMU_IT, "Attiva emulazione ACIA"},
	/* ko */ {IDCLS_ENABLE_ACIA_EMU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_ACIA_EMU_NL, "ACIA emulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_ACIA_EMU_PL, "W≥±cz emulacjÍ ACIA"},
	/* ru */ {IDCLS_ENABLE_ACIA_EMU_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_ACIA_EMU_SV, "Aktivera ACIA-emuleringen"},
	/* tr */ {IDCLS_ENABLE_ACIA_EMU_TR, "ACIA em¸lasyonunu aktif et"},
#endif
	
	/* plus4/plus4acia.c */
	/* en */ {IDCLS_DISABLE_ACIA_EMU,    N_("Disable the ACIA emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_ACIA_EMU_DA, "DeaktivÈr ACIA-emulering"},
	/* de */ {IDCLS_DISABLE_ACIA_EMU_DE, "ACIA Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_ACIA_EMU_ES, "Deshabilitar emulaciÛn ACIA"},
	/* fr */ {IDCLS_DISABLE_ACIA_EMU_FR, "DÈsactiver l'Èmulation ACIA"},
	/* hu */ {IDCLS_DISABLE_ACIA_EMU_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_ACIA_EMU_IT, "Disattiva emulazione ACIA"},
	/* ko */ {IDCLS_DISABLE_ACIA_EMU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_ACIA_EMU_NL, "ACIA emulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_ACIA_EMU_PL, "Wy≥±cz emulacjÍ ACIA"},
	/* ru */ {IDCLS_DISABLE_ACIA_EMU_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_ACIA_EMU_SV, "Inaktivera ACIA-emuleringen"},
	/* tr */ {IDCLS_DISABLE_ACIA_EMU_TR, "ACIA em¸lasyonunu pasifle˛tir"},
#endif
	
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c,
	 c64/psid.c, cbm2/cbm2-cmdline-options.c,
	 pet/pet-cmdline-options.c, plus4/plus4-cmdline-options.c,
	 vdc/vdc-cmdline-options.c, vic20/vic20-cmdline-options.c,
	 scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_P_NUMBER,    N_("<number>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_NUMBER_DA, "<nummer>"},
	/* de */ {IDCLS_P_NUMBER_DE, "<Nummer>"},
	/* es */ {IDCLS_P_NUMBER_ES, "<n˙mero>"},
	/* fr */ {IDCLS_P_NUMBER_FR, "<numÈro>"},
	/* hu */ {IDCLS_P_NUMBER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_NUMBER_IT, "<numero>"},
	/* ko */ {IDCLS_P_NUMBER_KO, "<º˝¿⁄>"},
	/* nl */ {IDCLS_P_NUMBER_NL, "<nummer>"},
	/* pl */ {IDCLS_P_NUMBER_PL, "<numer>"},
	/* ru */ {IDCLS_P_NUMBER_RU, "<number>"},
	/* sv */ {IDCLS_P_NUMBER_SV, "<nummer>"},
	/* tr */ {IDCLS_P_NUMBER_TR, "<say˝>"},
#endif
	
#ifdef COMMON_KBD
	/* keyboard.c */
	/* en */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX,    N_("Specify index of keymap file (0=symbolic, 1=positional, 2=symbolic (user), 3=positional (user))")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_DE, "Index f¸r Kymap Datei festlegen (0=symbol, 1=positional), 2=symbolisch\n(Benutzer), 3=positional (Benutzer))"},
	/* es */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_NL, "Geef index van het toetstoewijzingsbestand (0=symbolisch, 1=positioneel, 2=symbolisch (gebruiker), 3=positioneel (gebruiker))"},
	/* pl */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_PL, "Podaj indeks uk≥adu klawiatury (0=symbol, 1=pozycja, 2=symbol (uøytkownika), 3=pozycja (uøytkownika))"},
	/* ru */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_KEYMAP_FILE_INDEX_TR, ""},  /* fuzzy */
#endif
	
	/* keyboard.c */
	/* FIXME: generate dynamically */
	/* en */ {IDCLS_SPECIFY_KEYBOARD_TYPE,    N_("Specify emulated keyboard type")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_KEYBOARD_TYPE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_KEYBOARD_TYPE_DE, "Emulation f¸r Tastaturtyp ausw‰hlen"},
	/* es */ {IDCLS_SPECIFY_KEYBOARD_TYPE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_KEYBOARD_TYPE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_KEYBOARD_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_KEYBOARD_TYPE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_KEYBOARD_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_KEYBOARD_TYPE_NL, "Geef geÎmuleerd toetsenbord soort"},
	/* pl */ {IDCLS_SPECIFY_KEYBOARD_TYPE_PL, "Wybierz rodzaj emulowanej klawiatury"},
	/* ru */ {IDCLS_SPECIFY_KEYBOARD_TYPE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_KEYBOARD_TYPE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_KEYBOARD_TYPE_TR, ""},  /* fuzzy */
#endif
	
	/* keyboard.c */
	/* FIXME: generate dynamically */
	/* en */ {IDCLS_SPECIFY_KEYBOARD_MAPPING,    N_("Specify host keyboard layout")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_DE, "Host Tastatur Layout ausw‰hlen"},
	/* es */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_NL, "Geef host toetsenbordindeling"},
	/* pl */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_PL, "Okre∂l uk≥ad klawiatury hosta"},
	/* ru */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_KEYBOARD_MAPPING_TR, ""},  /* fuzzy */
#endif
	
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c,
	 plus4/plus4-cmdline-options.c, vic20/vic20-cmdline-options.c,
	 scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME,    N_("Specify name of symbolic keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_DA, "Angiv filnavn for symbolsk tastatur"},
	/* de */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_DE, "Bitte Namen f¸r symbolische Keymap Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_ES, "Especificar nombre fichero del mapa de teclado simbÛlico"},
	/* fr */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_FR, "SpÈcifier le nom du fichier de mappage clavier symbolique"},
	/* hu */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_HU, "Adja meg a szimbolikus lekÈpzÈsf·jl nevÈt"},
	/* it */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_IT, "Specifica il nome del file della mappa simbolica della tastiera"},
	/* ko */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_NL, "Geef naam van het symbolisch toetstoewijzingsbestand"},
	/* pl */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_PL, "Okre∂l nazwÍ pliku symbolicznego uk≥adu klawiatury"},
	/* ru */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_RU, "Specify name of symbolic keymap file"},
	/* sv */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_SV, "Ange fil fˆr symbolisk tangentbordsemulering"},
	/* tr */ {IDCLS_SPECIFY_SYM_KEYMAP_FILE_NAME_TR, "Sembolik tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
	
	/* c128/c128-cmdline-options.c, c64/c64-cmdline-options.c,
	 plus4/plus4-cmdline-options.c, vic20/vic20-cmdline-options.c,
	 scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME,    N_("Specify name of positional keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_DA, "Angiv fil for positionsbestemt tastatur"},
	/* de */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_DE, "Bitte Namen f¸r positions Keymap Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_ES, "Especificar nombre fichero del mapa de teclado posicional"},
	/* fr */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_FR, "SpÈcifier le nom du fichier de mappage clavier positionnel"},
	/* hu */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_HU, "Adja meg a pozÌciÛ szerinti lekÈpzÈsf·jl nevÈt"},
	/* it */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_IT, "Specifica il nome del file della mappa posizionale della tastiera"},
	/* ko */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_NL, "Geef naam van het positioneel toetstoewijzingsbestand"},
	/* pl */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_PL, "Okre∂l nazwÍ pliku pozycyjnego uk≥adu klawiatury"},
	/* ru */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_RU, "Specify name of positional keymap file"},
	/* sv */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_SV, "Ange fil fˆr positionsriktig tangentbordsemulering"},
	/* tr */ {IDCLS_SPECIFY_POS_KEYMAP_FILE_NAME_TR, "Konumsal tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
#endif
	
	/* c128/c128mmu.c */
	/* en */ {IDCLS_ACTIVATE_40_COL_MODE,    N_("Activate 40 column mode")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ACTIVATE_40_COL_MODE_DA, "AktivÈr 40-kolonners tilstand"},
	/* de */ {IDCLS_ACTIVATE_40_COL_MODE_DE, "40 Spalten Modus aktivieren"},
	/* es */ {IDCLS_ACTIVATE_40_COL_MODE_ES, "Activar modo 40 columnas"},
	/* fr */ {IDCLS_ACTIVATE_40_COL_MODE_FR, "Activer le mode 40 colonnes"},
	/* hu */ {IDCLS_ACTIVATE_40_COL_MODE_HU, "40 oszlopos mÛd aktiv·l·sa"},
	/* it */ {IDCLS_ACTIVATE_40_COL_MODE_IT, "Attiva modalit‡ a 40 colonne"},
	/* ko */ {IDCLS_ACTIVATE_40_COL_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ACTIVATE_40_COL_MODE_NL, "Activeer 40 kolomsmodus"},
	/* pl */ {IDCLS_ACTIVATE_40_COL_MODE_PL, "W≥±cz tryb 40 kolumn"},
	/* ru */ {IDCLS_ACTIVATE_40_COL_MODE_RU, "∞⁄‚ÿ“ÿ‡ﬁ“–‚Ï ‡’÷ÿ‹ 40 ⁄ﬁ€ﬁ›ﬁ⁄"},
	/* sv */ {IDCLS_ACTIVATE_40_COL_MODE_SV, "Aktivera 40-kolumnersl‰ge"},
	/* tr */ {IDCLS_ACTIVATE_40_COL_MODE_TR, "40 s¸tun modunu aktif et"},
#endif
	
	/* c128/c128mmu.c */
	/* en */ {IDCLS_ACTIVATE_80_COL_MODE,    N_("Activate 80 column mode")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ACTIVATE_80_COL_MODE_DA, "AktivÈr 80-kolonners tilstand"},
	/* de */ {IDCLS_ACTIVATE_80_COL_MODE_DE, "80 Spalten Modus aktivieren"},
	/* es */ {IDCLS_ACTIVATE_80_COL_MODE_ES, "Activar modo 80 columnas"},
	/* fr */ {IDCLS_ACTIVATE_80_COL_MODE_FR, "Activer le mode 80 colonnes"},
	/* hu */ {IDCLS_ACTIVATE_80_COL_MODE_HU, "80 oszlopos mÛd aktiv·l·sa"},
	/* it */ {IDCLS_ACTIVATE_80_COL_MODE_IT, "Attiva modalit‡ a 80 colonne"},
	/* ko */ {IDCLS_ACTIVATE_80_COL_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ACTIVATE_80_COL_MODE_NL, "Activeer 80 kolomsmodus"},
	/* pl */ {IDCLS_ACTIVATE_80_COL_MODE_PL, "W≥±cz tryb 80 kolumn"},
	/* ru */ {IDCLS_ACTIVATE_80_COL_MODE_RU, "∞⁄‚ÿ“ÿ‡ﬁ“–‚Ï ‡’÷ÿ‹ 80 ⁄ﬁ€ﬁ›ﬁ⁄"},
	/* sv */ {IDCLS_ACTIVATE_80_COL_MODE_SV, "Aktivera 80-kolumnersl‰ge"},
	/* tr */ {IDCLS_ACTIVATE_80_COL_MODE_TR, "80 s¸tun modunu aktif et"},
#endif
	
	/* c128/c128mmu.c */
	/* en */ {IDCLS_GO64_MODE,    N_("Always switch to C64 mode on reset")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_GO64_MODE_DA, "Skift altid til C64-tilstand ved reset"},
	/* de */ {IDCLS_GO64_MODE_DE, "Immer bei Reset in C64 Modus schalten"},
	/* es */ {IDCLS_GO64_MODE_ES, "Al reiniciar, pasar siempre al modo C64"},
	/* fr */ {IDCLS_GO64_MODE_FR, "Toujours revenir au mode C64 au redÈmarrage"},
	/* hu */ {IDCLS_GO64_MODE_HU, "Mindig C64 mÛdban ˙jraindÌt·s ut·n"},
	/* it */ {IDCLS_GO64_MODE_IT, "Vai sempre in modalit‡ C64 al reset"},
	/* ko */ {IDCLS_GO64_MODE_KO, "¥ŸΩ√ Ω√¿€«œ∏È «◊ªÛ C64 ∏µÂ ∑Œ πŸ≤Ú¥œ¥Ÿ"},
	/* nl */ {IDCLS_GO64_MODE_NL, "Start altijd op in C64 modus na reset"},
	/* pl */ {IDCLS_GO64_MODE_PL, "Przy restarcie zawsze prze≥±czaj w tryb C64"},
	/* ru */ {IDCLS_GO64_MODE_RU, "Always switch to C64 mode on reset"},
	/* sv */ {IDCLS_GO64_MODE_SV, "V‰xla alltid till C64-l‰ge vid Âterst‰llning"},
	/* tr */ {IDCLS_GO64_MODE_TR, "Resetlendiinde her zaman C64 moduna geÁ"},
#endif
	
	/* c128/c128mmu.c */
	/* en */ {IDCLS_GO128_MODE,    N_("Always switch to C128 mode on reset")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_GO128_MODE_DA, "Skift altid til C128-tilstand ved reset"},
	/* de */ {IDCLS_GO128_MODE_DE, "Immer be Reset in C128 Modus schalten"},
	/* es */ {IDCLS_GO128_MODE_ES, "Al reiniciar, pasar siempre al modo C128"},
	/* fr */ {IDCLS_GO128_MODE_FR, "Toujours revenir au mode C128 au redÈmarrage"},
	/* hu */ {IDCLS_GO128_MODE_HU, "Mindig C128 mÛd ˙jraindÌt·s ut·n"},
	/* it */ {IDCLS_GO128_MODE_IT, "Vai sempre in modalit‡ C128 al reset"},
	/* ko */ {IDCLS_GO128_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_GO128_MODE_NL, "Start altijd op in C128 modus na reset"},
	/* pl */ {IDCLS_GO128_MODE_PL, "Przy restarcie zawsze prze≥±czaj w tryb C128"},
	/* ru */ {IDCLS_GO128_MODE_RU, "≤·’”‘– ﬂ’‡’⁄€ÓÁ–‚Ï “ ‡’÷ÿ‹ C128 ﬂﬁ·€’ ﬂ’‡’◊–ﬂ„·⁄–"},
	/* sv */ {IDCLS_GO128_MODE_SV, "V‰xla alltid till C128-l‰ge vid Âterst‰llning"},
	/* tr */ {IDCLS_GO128_MODE_TR, "Resetlendiinde her zaman C128 moduna geÁ"},
#endif
	
	/* c128/functionrom.c */
	/* en */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME,    N_("Specify name of internal Function ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_DA, "Angiv navn pÂ internt funktions-ROM-image"},
	/* de */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_DE, "Name der internen Funktions ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_ES, "Especificar nombre imagen ROM funciÛn interna"},
	/* fr */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_FR, "SpÈcifier le nom de l'image ROM des fonctions internes"},
	/* hu */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_HU, "Adja meg a belsı Function ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_IT, "Specifica il nome immagine della Function ROM interna"},
	/* ko */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_NL, "Geef de naam van het intern functie ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_PL, "Okre∂l nazwÍ obrazu wewnÍtrznego Function ROM-u"},
	/* ru */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_RU, "Specify name of internal Function ROM image"},
	/* sv */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_SV, "Ange namn pÂ intern funktions-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_INT_FUNC_ROM_NAME_TR, "Dahili Function ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/functionrom.c */
	/* en */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME,    N_("Specify name of external Function ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_DA, "Angiv navn pÂ externt funktions-ROM-image"},
	/* de */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_DE, "Name der externen Funktions ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_ES, "Especificar nombre imagen ROM funciÛn externa"},
	/* fr */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_FR, "SpÈcifier le nom de l'image ROM des fonctions externes"},
	/* hu */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_HU, "Adja meg a k¸lsı Function ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_IT, "Specifica il nome immagine della Function ROM esterna"},
	/* ko */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_NL, "Geef de naam van het extern functie ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_PL, "Okre∂l nazwÍ obrazu zewnÍtrznego Function ROM-u"},
	/* ru */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_RU, "Specify name of external Function ROM image"},
	/* sv */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_SV, "Ange namn pÂ extern funktions-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_EXT_FUNC_ROM_NAME_TR, "Harici Function ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c128/functionrom.c */
	/* en */ {IDCLS_ENABLE_INT_FUNC_ROM,    N_("Type of internal Function ROM: (0: None, 1: ROM, 2: RAM, 3: RTC)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_INT_FUNC_ROM_DA, "Typen af intern funktions-ROM: (0: Ingen, 1: ROM, 2: RAM, 3:RTC)"},
	/* de */ {IDCLS_ENABLE_INT_FUNC_ROM_DE, "Typ des internen Funktions ROM: (0: Kein, 1: ROM, 2: RAM, 3: RTC)"},
	/* es */ {IDCLS_ENABLE_INT_FUNC_ROM_ES, "Tipo de Function ROM interna: (0: None, 1: ROM, 2: RAM, 3: RTC)"},
	/* fr */ {IDCLS_ENABLE_INT_FUNC_ROM_FR, "Type de ROM Fonction interne: (0: Aucun, 1: ROM, 2: RAM, 3: RTC)"},
	/* hu */ {IDCLS_ENABLE_INT_FUNC_ROM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_INT_FUNC_ROM_IT, "Tipo di Function ROM interna: (0: None, 1: ROM, 2: RAM, 3: RTC)"},
	/* ko */ {IDCLS_ENABLE_INT_FUNC_ROM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_INT_FUNC_ROM_NL, "Intern functie ROM soort: (0: Geen, 1: ROM, 2: RAM, 3: RTC)"},
	/* pl */ {IDCLS_ENABLE_INT_FUNC_ROM_PL, "Typ wewnÍtrznego Function ROM-u: (0: Øaden, 1: ROM, 2: RAM, 3: RTC)"},
	/* ru */ {IDCLS_ENABLE_INT_FUNC_ROM_RU, "Type of internal Function ROM: (0: None, 1: ROM, 2: RAM, 3: RTC)"},
	/* sv */ {IDCLS_ENABLE_INT_FUNC_ROM_SV, "Typ fˆr intern funktions-ROM (0: ingen, 1: ROM, 2: RAM, 3: RTC)"},
	/* tr */ {IDCLS_ENABLE_INT_FUNC_ROM_TR, "Dahili Function ROM Tipi: (0: Yok, 1: ROM, 2: RAM, 3: RTC)"},
#endif
	
	/* c128/functionrom.c */
	/* en */ {IDCLS_ENABLE_EXT_FUNC_ROM,    N_("Type of external Function ROM: (0: None, 1: ROM, 2: RAM, 3: RTC)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_EXT_FUNC_ROM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_EXT_FUNC_ROM_DE, "Typ des esternen Funktions ROM: (0: Kein, 1: ROM, 2: RAM, 3: RTC)"},
	/* es */ {IDCLS_ENABLE_EXT_FUNC_ROM_ES, "Tipo de Function ROM externa: (0: None, 1: ROM, 2: RAM, 3: RTC)"},
	/* fr */ {IDCLS_ENABLE_EXT_FUNC_ROM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_EXT_FUNC_ROM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_EXT_FUNC_ROM_IT, "Tipo di Function ROM esterna: (0: None, 1: ROM, 2: RAM, 3: RTC)"},
	/* ko */ {IDCLS_ENABLE_EXT_FUNC_ROM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_EXT_FUNC_ROM_NL, "Extern functie ROM soort: (0: Geen, 1: ROM, 2: RAM, 3: RTC)"},
	/* pl */ {IDCLS_ENABLE_EXT_FUNC_ROM_PL, "Typ zewnÍtrznego Function ROM-u: (0: Øaden, 1: ROM, 2: RAM, 3: RTC)"},
	/* ru */ {IDCLS_ENABLE_EXT_FUNC_ROM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_EXT_FUNC_ROM_SV, "Typ fˆr extern funktions-ROM (0: ingen, 1: ROM, 2: RAM, 3: RTC)"},
	/* tr */ {IDCLS_ENABLE_EXT_FUNC_ROM_TR, ""},  /* fuzzy */
#endif
	
	/* c128/functionrom.c */
	/* en */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE,    N_("Enable saving of the internal function RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_DE, "Interne Funktions RTC Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_ES, "Permitir grabar datos de Internal Function RTC cuando cambien"},
	/* fr */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_IT, "Attiva il salvataggio dei dati RTC della internal function alla modifica."},
	/* ko */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_NL, "Opslaan van de interne functie RTC gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_PL, "Zapisuj dane zegara czasu rzeczywistego wewnÍtrznego Function ROM-u przy zmianie"},
	/* ru */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_INT_FUNC_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* c128/functionrom.c */
	/* en */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE,    N_("Disable saving of the internal function RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_DE, "Interne Funktions RTC Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_ES, "Deshabilitar grabar datos de Internal Function RTC cuando cambien"},
	/* fr */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_IT, "Disattiva il salvataggio dei dati RTC della internal function alla modifica."},
	/* ko */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_NL, "Opslaan van de interne functie RTC gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_PL, "Nie zapisuj danych zegara czasu rzeczywistego wewnÍtrznego Function ROM-u przy zmianie"},
	/* ru */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_INT_FUNC_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* c128/functionrom.c */
	/* en */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE,    N_("Enable saving of the external function RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_DE, "Externe Funktions RTC Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_ES, "Permitir grabar datos de External Function RTC cuando cambien"},
	/* fr */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_IT, "Attiva il salvataggio dei dati RTC della external function alla modifica."},
	/* ko */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_NL, "Opslaan van de externe functie RTC gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_PL, "Zapisuj dane zegara czasu rzeczywistego zewnÍtrznego Function ROM-u przy zmianie"},
	/* ru */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_EXT_FUNC_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* c128/functionrom.c */
	/* en */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE,    N_("Disable saving of the external function RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_DE, "Externe Funktions RTC Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_ES, "Deshabilitar grabar datos de External Function RTC cuando cambien"},
	/* fr */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_IT, "Disattiva il salvataggio dei dati RTC della external function alla modifica."},
	/* ko */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_NL, "Opslaan van de externe functie RTC gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_PL, "Nie zapisuj danych zegara czasu rzeczywistego zewnÍtrznego Function ROM-u przy zmianie"},
	/* ru */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_EXT_FUNC_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/c64-cmdline-options.c, c64/psid.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR,    N_("Use old NTSC sync factor")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_DA, "Anvend gammel NTSC-synkfaktor"},
	/* de */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_DE, "Alten NTSC Sync Faktor verwenden"},
	/* es */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_ES, "Usar antiguo factor de sincronizaciÛn NTSC"},
	/* fr */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_FR, "Utiliser l'ancien mode NTSC"},
	/* hu */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_HU, "RÈgi NTSC szinkron faktor haszn·lata"},
	/* it */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_IT, "Usa il vecchio fattore di sincronizzazione NTSC"},
	/* ko */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_NL, "Gebruik oude NTSC synchronisatiefactor"},
	/* pl */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_PL, "Uøywaj starej synchronizacji NTSC"},
	/* ru */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_RU, "Use old NTSC sync factor"},
	/* sv */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_SV, "Anv‰nd gammal NTSC-synkfaktor"},
	/* tr */ {IDCLS_USE_OLD_NTSC_SYNC_FACTOR_TR, "Eski NTSC senkron faktˆr¸ kullan"},
#endif
	
	/* c64/c64-cmdline-options.c, viciisc/vicii-cmdline-options.c,
	 scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_P_MODEL,    N_("<Model>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_MODEL_DA, "<Model>"},
	/* de */ {IDCLS_P_MODEL_DE, "<Modell>"},
	/* es */ {IDCLS_P_MODEL_ES, "<Modelo>"},
	/* fr */ {IDCLS_P_MODEL_FR, "<ModËle>"},
	/* hu */ {IDCLS_P_MODEL_HU, "<Modell>"},
	/* it */ {IDCLS_P_MODEL_IT, "<Modello>"},
	/* ko */ {IDCLS_P_MODEL_KO, "<∏µ®>"},
	/* nl */ {IDCLS_P_MODEL_NL, "<Model>"},
	/* pl */ {IDCLS_P_MODEL_PL, "<Model>"},
	/* ru */ {IDCLS_P_MODEL_RU, "<ºﬁ‘’€Ï>"},
	/* sv */ {IDCLS_P_MODEL_SV, "<Modell>"},
	/* tr */ {IDCLS_P_MODEL_TR, "<Model>"},
#endif
	
	/* c64/c64-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SET_BOTH_CIA_MODELS,    N_("Set both CIA models (0 = old 6526, 1 = new 6526A)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_BOTH_CIA_MODELS_DA, "VÊlg type for begge CIA-kredse (0=gammel 6526, 1=ny 6526A)"},
	/* de */ {IDCLS_SET_BOTH_CIA_MODELS_DE, "W‰hle beide CIA Modelle (0 = alt 6526, 1 = neu 6526A)"},
	/* es */ {IDCLS_SET_BOTH_CIA_MODELS_ES, "Seleccionar ambos modelos de CIA (0 = antiguo 6526, 1 = nuevo 6526A)"},
	/* fr */ {IDCLS_SET_BOTH_CIA_MODELS_FR, "DÈfinir les deux modËles CIA (0 = ancien 5626, 1 = rÈcent 6526A)"},
	/* hu */ {IDCLS_SET_BOTH_CIA_MODELS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_BOTH_CIA_MODELS_IT, "Imposta entrambi i modelli CIA (0 = 6526 vecchio, 1 = 6526A nuovo)"},
	/* ko */ {IDCLS_SET_BOTH_CIA_MODELS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_BOTH_CIA_MODELS_NL, "Stel model in voor beide CIA's (0 = oud 6526, 1 = nieuw 6526A)"},
	/* pl */ {IDCLS_SET_BOTH_CIA_MODELS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SET_BOTH_CIA_MODELS_RU, "Set both CIA models (0 = old 6526, 1 = new 6526A)"},
	/* sv */ {IDCLS_SET_BOTH_CIA_MODELS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_BOTH_CIA_MODELS_TR, "Her iki CIA modelini belirle (0 = eski 6526, 1 = yeni 6526A)"},
#endif
	
	/* c64/c64-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SET_CIA1_MODEL,    N_("Set CIA 1 model (0 = old 6526, 1 = new 6526A)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_CIA1_MODEL_DA, "VÊlg CIA 1 type (0=gammel 6526, 1=ny 6526A)"},
	/* de */ {IDCLS_SET_CIA1_MODEL_DE, "W‰hle CIA 1 Modell (0 = alt 6526, 1 = neu 6526A)"},
	/* es */ {IDCLS_SET_CIA1_MODEL_ES, "Seleccionar CIA 1 Poner CIA 1 model (0 = antiguo 6526, 1 = nuevo 6526A)"},
	/* fr */ {IDCLS_SET_CIA1_MODEL_FR, "DÈfinir le modËle CIA 2 (0 = ancien 5626, 1 = rÈcent 6526A)"},
	/* hu */ {IDCLS_SET_CIA1_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_CIA1_MODEL_IT, "Imposta modello CIA 1 (0 = 6526 vecchio, 1 = 6526A nuovo)"},
	/* ko */ {IDCLS_SET_CIA1_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_CIA1_MODEL_NL, "Stel CIA 1 model in (0 = oud 6526, 1 = nieuw 6526A)"},
	/* pl */ {IDCLS_SET_CIA1_MODEL_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SET_CIA1_MODEL_RU, "Set CIA 1 model (0 = old 6526, 1 = new 6526A)"},
	/* sv */ {IDCLS_SET_CIA1_MODEL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_CIA1_MODEL_TR, "CIA 1 modelini belirle (0 = eski 6526, 1 = yeni 6526A)"},
#endif
	
	/* c64/c64-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SET_CIA2_MODEL,    N_("Set CIA 2 model (0 = old 6526, 1 = new 6526A)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_CIA2_MODEL_DA, "VÊlg CIA 2 type (0=gammel 6526, 1=ny 6526A)"},
	/* de */ {IDCLS_SET_CIA2_MODEL_DE, "W‰hle CIA 2 Modell (0 = alt 6526, 1 = neu 6526A)\""},
	/* es */ {IDCLS_SET_CIA2_MODEL_ES, "Seleccionar CIA 2 model (0 = antiguo 6526, 1 = nuevo 6526A)"},
	/* fr */ {IDCLS_SET_CIA2_MODEL_FR, "DÈfinir le modËle CIA 1 (0 = ancien 5626, 1 = rÈcent 6526A)"},
	/* hu */ {IDCLS_SET_CIA2_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_CIA2_MODEL_IT, "Imposta modello CIA 2 (0 = 6526 vecchio, 1 = 6526A nuovo)"},
	/* ko */ {IDCLS_SET_CIA2_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_CIA2_MODEL_NL, "Stel CIA 2 model in (0 = oud 6526, 1 = nieuw 6526A)"},
	/* pl */ {IDCLS_SET_CIA2_MODEL_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SET_CIA2_MODEL_RU, "Set CIA 2 model (0 = old 6526, 1 = new 6526A)"},
	/* sv */ {IDCLS_SET_CIA2_MODEL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_CIA2_MODEL_TR, "CIA 2 modelini belirle (0 = eski 6526, 1 = yeni 6526A)"},
#endif
	
	/* c64/c64-cmdline-options.c */
	/* en */ {IDCLS_SET_C64_MODEL,    N_("Set C64 model (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs, pet64, ultimax)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_C64_MODEL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_C64_MODEL_DE, "W‰hle C64 Modell (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs, pet64, ultimax)"},
	/* es */ {IDCLS_SET_C64_MODEL_ES, "Seleccionar modelo C64 (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs, pet64, ultimax)"},
	/* fr */ {IDCLS_SET_C64_MODEL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_C64_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_C64_MODEL_IT, "Imposta modello C64 (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs, pet64, ultimax)"},
	/* ko */ {IDCLS_SET_C64_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_C64_MODEL_NL, "Stel C64 model in (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs, pet64, ultimax)"},
	/* pl */ {IDCLS_SET_C64_MODEL_PL, "Wybierz model C64 (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs, pet64, ultimax)"},
	/* ru */ {IDCLS_SET_C64_MODEL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_C64_MODEL_SV, "V‰lj C64-modell (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs, pet64, ultimax)"},
	/* tr */ {IDCLS_SET_C64_MODEL_TR, ""},  /* fuzzy */
#endif
	
	/* scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SET_SCPU64_MODEL,    N_("Set C64 model (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_SCPU64_MODEL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_SCPU64_MODEL_DE, "W‰hle C64 Modell (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs)"},
	/* es */ {IDCLS_SET_SCPU64_MODEL_ES, "Seleccionar modelo C64 (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs)"},
	/* fr */ {IDCLS_SET_SCPU64_MODEL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_SCPU64_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_SCPU64_MODEL_IT, "Imposta modello C64 (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs)"},
	/* ko */ {IDCLS_SET_SCPU64_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_SCPU64_MODEL_NL, "Stel C64 model in (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs)"},
	/* pl */ {IDCLS_SET_SCPU64_MODEL_PL, "Wybierz model C64 (c64/c64c/c64old, ntsc/newntsc/oldntsc, drean, jap, c64gs)"},
	/* ru */ {IDCLS_SET_SCPU64_MODEL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_SCPU64_MODEL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_SCPU64_MODEL_TR, ""},  /* fuzzy */
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SET_C128_MODEL,    N_("Set C128 model (c128/c128dcr, pal/ntsc)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_C128_MODEL_DA, "Angiv C128-model (c128/c128dcr, pal/ntsc)"},
	/* de */ {IDCLS_SET_C128_MODEL_DE, "W‰hle C128 Modell (c128/c128dcr, pal/ntsc)"},
	/* es */ {IDCLS_SET_C128_MODEL_ES, "Seleccionar modelo C128 (c128/c128dcr, pal/ntsc)"},
	/* fr */ {IDCLS_SET_C128_MODEL_FR, "DÈfinir le modËle C128 (c128/c128dcr, pal/ntsc)"},
	/* hu */ {IDCLS_SET_C128_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_C128_MODEL_IT, "Imposta modello C128 (c128/c128dcr, pal/ntsc)"},
	/* ko */ {IDCLS_SET_C128_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_C128_MODEL_NL, "Stel C128 model in (c128/c128dcr, pal/ntsc)"},
	/* pl */ {IDCLS_SET_C128_MODEL_PL, "Ustaw model C128 (c128/c128dcr, pal/ntsc)"},
	/* ru */ {IDCLS_SET_C128_MODEL_RU, "Set C128 model (c128/c128dcr, pal/ntsc)"},
	/* sv */ {IDCLS_SET_C128_MODEL_SV, "V‰lj C128-modell (c128/c128dcr, pal/ntsc)"},
	/* tr */ {IDCLS_SET_C128_MODEL_TR, "C128 modelini belirle (c128/c128dcr, pal/ntsc)"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_SET_C128_MACHINE_TYPE,    N_("Set C128 machine type (0: International, 1: Finnish, 2: French, 3: German, 4: Italian, 5: Norwegian, 6: Swedish, 7: Swiss)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_C128_MACHINE_TYPE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_C128_MACHINE_TYPE_DE, "W‰hle C128 Modell (0: International, 1: Finnisch, 2: Franzˆsisch, 3: Deutsch, 4: Italienisch , 5: Norwegisch, 6: Schwedisch, 7: Schweiz)"},
	/* es */ {IDCLS_SET_C128_MACHINE_TYPE_ES, "Seleccionar tipo de m·quina C128 (0: Internacional, 1: Finlandesa, 2: Francesa, 3: Alemana, 4: Italiana, 5: Noruega, 6: Sueca, 7: Suiza)"},
	/* fr */ {IDCLS_SET_C128_MACHINE_TYPE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_C128_MACHINE_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_C128_MACHINE_TYPE_IT, "Imposta tipo C128 (0: Internazionale, 1: Finlandese, 2: Francese, 3: Tedesco, 4: Italiano, 5: Norvegese, 6: Svedese, 7: Svizzero)"},
	/* ko */ {IDCLS_SET_C128_MACHINE_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_C128_MACHINE_TYPE_NL, "Stel C128 machine soort in (0: Internationaal, 1: Fins, 2: Frans, 3: Duits, 4: Italiaans, 5: Noors, 6: Zweeds, 7: Zwitsers)"},
	/* pl */ {IDCLS_SET_C128_MACHINE_TYPE_PL, "Wybierz typ maszyny C128 (0: MiÍdzynarodowy, 1: FiÒski, 2: Francuski, 3: Niemiecki, 4: W≥oski, 5: Norweski, 6: Szwedzki, 7: Szwajcarski)"},
	/* ru */ {IDCLS_SET_C128_MACHINE_TYPE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_C128_MACHINE_TYPE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_C128_MACHINE_TYPE_TR, ""},  /* fuzzy */
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_C128_FULL_BANKS,    N_("Enable RAM banks 2 and 3")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_C128_FULL_BANKS_DA, "AktivÈr RAM-bank 2 og 3"},
	/* de */ {IDCLS_ENABLE_C128_FULL_BANKS_DE, "RAM B‰nke 2 und 3 aktivieren"},
	/* es */ {IDCLS_ENABLE_C128_FULL_BANKS_ES, "Permitir bancos 2 y 3"},
	/* fr */ {IDCLS_ENABLE_C128_FULL_BANKS_FR, "Activer les banques RAM 2 et 3"},
	/* hu */ {IDCLS_ENABLE_C128_FULL_BANKS_HU, "2-es Ès 3-as RAM bankok engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_C128_FULL_BANKS_IT, "Attiva banchi RAM 2 e 3"},
	/* ko */ {IDCLS_ENABLE_C128_FULL_BANKS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_C128_FULL_BANKS_NL, "RAM bank 2 en 3 inschakelen"},
	/* pl */ {IDCLS_ENABLE_C128_FULL_BANKS_PL, "W≥±cz banki RAM 2 i 3"},
	/* ru */ {IDCLS_ENABLE_C128_FULL_BANKS_RU, "Enable RAM banks 2 and 3"},
	/* sv */ {IDCLS_ENABLE_C128_FULL_BANKS_SV, "Aktivera RAM-bank 2 och 3"},
	/* tr */ {IDCLS_ENABLE_C128_FULL_BANKS_TR, "RAM banklar˝ 2 ve 3'¸ aktif et"},
#endif
	
	/* c128/c128-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_C128_FULL_BANKS,    N_("Disable RAM banks 2 and 3")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_C128_FULL_BANKS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_C128_FULL_BANKS_DE, "RAM B‰nke 2 und 3 deaktivieren"},
	/* es */ {IDCLS_DISABLE_C128_FULL_BANKS_ES, "Inhabilitar bancos 2 y 3"},
	/* fr */ {IDCLS_DISABLE_C128_FULL_BANKS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_C128_FULL_BANKS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_C128_FULL_BANKS_IT, "Disattiva banchi RAM 2 e 3"},
	/* ko */ {IDCLS_DISABLE_C128_FULL_BANKS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_C128_FULL_BANKS_NL, "RAM bank 2 en 3 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_C128_FULL_BANKS_PL, "Wy≥±cz banki RAM 2 i 3"},
	/* ru */ {IDCLS_DISABLE_C128_FULL_BANKS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_C128_FULL_BANKS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_C128_FULL_BANKS_TR, ""},  /* fuzzy */
#endif
	
	/* plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_SET_PLUS4_MODEL,    N_("Set Plus4 model (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_PLUS4_MODEL_DA, "Angiv Plus4-modem (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* de */ {IDCLS_SET_PLUS4_MODEL_DE, "Setze Plus4 Modell (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* es */ {IDCLS_SET_PLUS4_MODEL_ES, "Seleccionar Plus4 model (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* fr */ {IDCLS_SET_PLUS4_MODEL_FR, "DÈfinir le modËle Plus4 (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* hu */ {IDCLS_SET_PLUS4_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_PLUS4_MODEL_IT, "Imposta modello Plus4 (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* ko */ {IDCLS_SET_PLUS4_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_PLUS4_MODEL_NL, "Stel Plus4 model in (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* pl */ {IDCLS_SET_PLUS4_MODEL_PL, "Ustaw model Plus4 (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* ru */ {IDCLS_SET_PLUS4_MODEL_RU, "Set Plus4 model (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* sv */ {IDCLS_SET_PLUS4_MODEL_SV, "V‰lj Plus4-modell (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
	/* tr */ {IDCLS_SET_PLUS4_MODEL_TR, "Plus4 modelini ayarla (c16/c16pal/c16ntsc, plus4/plus4pal/plus4ntsc, v364/cv364, c232)"},
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_SET_VIC20_MODEL,    N_("Set VIC20 model (vic20/vic20pal/vic20ntsc, vic21)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_VIC20_MODEL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_VIC20_MODEL_DE, "W‰hle VIC20 Modell (vic20/vic20pal/vic20ntsc, vic21)"},
	/* es */ {IDCLS_SET_VIC20_MODEL_ES, "Seleccionar modelo  VIC20 (vic20/vic20pal/vic20ntsc, vic21)"},
	/* fr */ {IDCLS_SET_VIC20_MODEL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_VIC20_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_VIC20_MODEL_IT, "Imposta modello VIC20 (vic20/vic20pal/vic20ntsc, vic21)"},
	/* ko */ {IDCLS_SET_VIC20_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_VIC20_MODEL_NL, "Stel VIC20 model in (vic20/vic20pal/vic20ntsc, vic21)"},
	/* pl */ {IDCLS_SET_VIC20_MODEL_PL, "Wybierz model VIC20 (vic20/vic20pal/vic20ntsc, vic21)"},
	/* ru */ {IDCLS_SET_VIC20_MODEL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_VIC20_MODEL_SV, "V‰lj VIC20-modell (vic20/vic20pal/vic20ntsc, vic21)"},
	/* tr */ {IDCLS_SET_VIC20_MODEL_TR, ""},  /* fuzzy */
#endif
	
	/* c64dtv/c64dtv-cmdline-options.c */
	/* en */ {IDCLS_SET_DTV_MODEL,    N_("Set DTV model (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DTV_MODEL_DA, "Angiv DTV-model (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* de */ {IDCLS_SET_DTV_MODEL_DE, "Setze DTV Modell (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* es */ {IDCLS_SET_DTV_MODEL_ES, "Seleccionar DTV model (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* fr */ {IDCLS_SET_DTV_MODEL_FR, "DÈfinir le modËle DTV (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* hu */ {IDCLS_SET_DTV_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DTV_MODEL_IT, "Imposta modello DTV (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* ko */ {IDCLS_SET_DTV_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DTV_MODEL_NL, "Stel DTV model in (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* pl */ {IDCLS_SET_DTV_MODEL_PL, "Ustaw model DTV (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* ru */ {IDCLS_SET_DTV_MODEL_RU, "Set DTV model (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* sv */ {IDCLS_SET_DTV_MODEL_SV, "V‰lj DTV-modell (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
	/* tr */ {IDCLS_SET_DTV_MODEL_TR, "DTV modelini ayarla (v2/v2pal/v2ntsc, v3/v3pal/v3ntsc, hummer)"},
#endif
	
	/* c64/c64-cmdline-options.c, cbm2/cbm2-cmdline-options.c,
	 pet/pet-cmdline-options.c, plus4/plus4-cmdline-options.c,
	 vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_KERNAL_ROM_NAME,    N_("Specify name of Kernal ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_DA, "Angiv navn pÂ kernal-ROM-image"},
	/* de */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_DE, "Name von Kernal ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_ES, "Especificar nombre de la imagen ROM Kernal"},
	/* fr */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_FR, "SpÈcifier le nom de l'image ROM Kernal"},
	/* hu */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_HU, "Adja meg a Kernal ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_IT, "Specifica il nome immagine della ROM del Kernal"},
	/* ko */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_NL, "Geef de naam van het Kernal ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Kernala"},
	/* ru */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_RU, "Specify name of Kernal ROM image"},
	/* sv */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_SV, "Ange namn pÂ kernal-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_KERNAL_ROM_NAME_TR, "Kernal ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SCPU64_ROM_NAME,    N_("Specify name of SCPU64 ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_DE, "Name von SCPU64 ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_ES, "Especificar nombre imagen ROM SCPU64"},
	/* fr */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_IT, "Specifica il nome immagine della ROM della SCPU64"},
	/* ko */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_NL, "Geef de naam van het SCPU64 ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM SCPU64"},
	/* ru */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_SCPU64_ROM_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* c64/c64-cmdline-options.c, cbm2/cbm2-cmdline-options.c,
	 pet/pet-cmdline-options.c, plus4/plus4-cmdline-options.c,
	 vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_BASIC_ROM_NAME,    N_("Specify name of BASIC ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_BASIC_ROM_NAME_DA, "Angiv navn pÂ BASIC-ROM-image"},
	/* de */ {IDCLS_SPECIFY_BASIC_ROM_NAME_DE, "Name von Basic ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_BASIC_ROM_NAME_ES, "Especificar nombre de la imagen ROM BASIC"},
	/* fr */ {IDCLS_SPECIFY_BASIC_ROM_NAME_FR, "SpÈcifier le nom de l'image BASIC ROM"},
	/* hu */ {IDCLS_SPECIFY_BASIC_ROM_NAME_HU, "Adja meg a BASIC ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_BASIC_ROM_NAME_IT, "Specifica il nome immagine della ROM del BASIC"},
	/* ko */ {IDCLS_SPECIFY_BASIC_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_BASIC_ROM_NAME_NL, "Geef de naam van het BASIC ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_BASIC_ROM_NAME_PL, "Okre∂l nazwÍ obrazu BASIC ROM"},
	/* ru */ {IDCLS_SPECIFY_BASIC_ROM_NAME_RU, "Specify name of BASIC ROM image"},
	/* sv */ {IDCLS_SPECIFY_BASIC_ROM_NAME_SV, "Ange namn pÂ BASIC-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_BASIC_ROM_NAME_TR, "BASIC ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/c64-cmdline-options.c, cbm2/cbm2-cmdline-options.c,
	 pet/pet-cmdline-options.c, vic20/vic20-cmdline-options.c,
	 scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME,    N_("Specify name of character generator ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_DA, "Angiv navn pÂ BASIC-ROM-image"},
	/* de */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_DE, "Name von Character ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_ES, "Especificar el nombre de la imagen ROM generador de caracteres"},
	/* fr */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_FR, "SpÈcifier le nom de l'image du gÈnÈrateur de caractËres"},
	/* hu */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_HU, "Adja meg a karaktergener·tor ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_IT, "Specifica il nome della ROM del generatore di caratteri"},
	/* ko */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_NL, "Geef de naam van het CHARGEN ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM tworzenia znakÛw"},
	/* ru */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_RU, "Specify name of character generator ROM image"},
	/* sv */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_SV, "Ange namn pÂ teckengenerator-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_CHARGEN_ROM_NAME_TR, "Karakter Olu˛turucu ROM imaj˝n˝n ismini belirt"},
#endif
	
#ifdef COMMON_KBD
	/* c64/c64-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2,    N_("Specify index of keymap file (0=sym, 1=symDE, 2=pos)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_DA, "Angiv indeks for tastaturindstillingsfil (0=symbolsk, 1=symbolsk tysk, 2=positionsbestemt)"},
	/* de */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_DE, "Index f¸r Keymap Datei festlegen (0=symbol, 1=symDE, 2=positional)"},
	/* es */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_ES, "Especificar Ìndice fichero mapa teclado (0=sim, 1=simAL, 2=pos)"},
	/* fr */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_FR, "SpÈcifier l'index du fichier keymap (0=sym, 1=symDE, 2=pos)"},
	/* hu */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_HU, "Adja meg a billenty˚zet lekÈpzÈsi f·jl tÌpus·t (0=szimbolikus, 1=nÈmet szimbolikus, 2=pozÌciÛ szerinti)"},
	/* it */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_IT, "Specifica l'indice del file della mappa della tastiera (0=sim, 1=simGER, 2=pos)"},
	/* ko */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_NL, "Geef de index van het toetstoewijzingsbestand (0=sym, 1=symDE, 2=pos)"},
	/* pl */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_PL, "Podaj indeks dla uk≥adu klawiatury (0=symbol, 1=symbolDE, 2=pozycja)"},
	/* ru */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_RU, "Specify index of keymap file (0=sym, 1=symDE, 2=pos)"},
	/* sv */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_SV, "Ange index fˆr fˆr tangentbordsinst‰llningsfil (0=symbolisk, 1=symbolisk tysk, 2=positionsriktig)"},
	/* tr */ {IDCLS_SPECIFY_INDEX_KEYMAP_FILE_0_2_TR, "Tu˛ haritas˝ dosyas˝n˝n indeksini belirt (0=sembol, 1=sembol Almanca, 2=konumsal)"},
#endif
	
	/* c64/c64-cmdline-options.c, scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP,    N_("Specify name of symbolic German keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_DA, "Angiv fil for tysk symbolsk tastaturindstilling"},
	/* de */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_DE, "Name von symbolischer Keymap Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_ES, "Especificar nombre del fichero teclado simbÛlico alem·n"},
	/* fr */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_FR, "SpÈcifier le nom du fichier symbolique de mappage clavier"},
	/* hu */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_HU, "Adja meg a nevÈt a nÈmet billenty˚zet lekÈpzÈs f·jlnak"},
	/* it */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_IT, "Specifica il nome del file della mappa simbolica della tastiera tedesca"},
	/* ko */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_NL, "Geef de naam van het symbolische Duitse toetstoewijzingsbestand"},
	/* pl */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_PL, "Okre∂l nazwÍ pliku symbolicznego niemieckiego uk≥adu klawiatury"},
	/* ru */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_RU, "Specify name of symbolic German keymap file"},
	/* sv */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_SV, "Ange fil fˆr tysk symbolisk tangentbordsemulering"},
	/* tr */ {IDCLS_SPECIFY_NAME_SYM_DE_KEYMAP_TR, "Sembolik Almanca tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
#endif
	
	/* c64/c64gluelogic.c */
	/* en */ {IDCLS_SET_GLUE_LOGIC_TYPE,    N_("Set glue logic type (0 = discrete, 1 = 252535-01)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_GLUE_LOGIC_TYPE_DA, "VÊlg glue logic type (0=diskret, 1=252535-01)"},
	/* de */ {IDCLS_SET_GLUE_LOGIC_TYPE_DE, "Setze Glue-Lokig Typ (0 = diskret, 1 = 252535-01)"},
	/* es */ {IDCLS_SET_GLUE_LOGIC_TYPE_ES, "Seleccionar tipo de 'glue' lÛgico (0 = discrete, 1 = 252535-01)"},
	/* fr */ {IDCLS_SET_GLUE_LOGIC_TYPE_FR, "DÈfinir le type de logique glue (0 = discrËte, 1 = 252535-01)"},
	/* hu */ {IDCLS_SET_GLUE_LOGIC_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_GLUE_LOGIC_TYPE_IT, "Imposta il tipo di glue logic (0 = discreta, 1 = 252535-01)"},
	/* ko */ {IDCLS_SET_GLUE_LOGIC_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_GLUE_LOGIC_TYPE_NL, "Stel glue logic type in (0 = discreet, 1 = 252535-01)"},
	/* pl */ {IDCLS_SET_GLUE_LOGIC_TYPE_PL, "Ustaw typ glue logic (0 = dyskretna, 1 = 252535-01)"},
	/* ru */ {IDCLS_SET_GLUE_LOGIC_TYPE_RU, "Set glue logic type (0 = discrete, 1 = 252535-01)"},
	/* sv */ {IDCLS_SET_GLUE_LOGIC_TYPE_SV, "V‰lj typ av klisterlogik (0 = diskret, 1 = 252535-01)"},
	/* tr */ {IDCLS_SET_GLUE_LOGIC_TYPE_TR, "Glue logic tipini ayarla (0 = balant˝y˝ kes, 1 = 252535-01)"},
#endif
	
	/* c64dtv/c64dtv-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_HUMMER_ADC,    N_("Enable Hummer ADC")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_HUMMER_ADC_DA, "AktivÈr Hummer ADC"},
	/* de */ {IDCLS_ENABLE_HUMMER_ADC_DE, "Hummer ADC Aktivieren"},
	/* es */ {IDCLS_ENABLE_HUMMER_ADC_ES, "Permitir Hummer ADC"},
	/* fr */ {IDCLS_ENABLE_HUMMER_ADC_FR, "Activer Hummer ADC"},
	/* hu */ {IDCLS_ENABLE_HUMMER_ADC_HU, "Hummer ADC engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_HUMMER_ADC_IT, "Attiva Hummer ADC"},
	/* ko */ {IDCLS_ENABLE_HUMMER_ADC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_HUMMER_ADC_NL, "Hummer ADC inschakelen"},
	/* pl */ {IDCLS_ENABLE_HUMMER_ADC_PL, "W≥±cz Hummer ADC"},
	/* ru */ {IDCLS_ENABLE_HUMMER_ADC_RU, "Enable Hummer ADC"},
	/* sv */ {IDCLS_ENABLE_HUMMER_ADC_SV, "Aktivera Hummer-A/D-omvandlare"},
	/* tr */ {IDCLS_ENABLE_HUMMER_ADC_TR, "Hummer ADC'yi aktif et"},
#endif
	
	/* c64dtv/c64dtv-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_HUMMER_ADC,    N_("Disable Hummer ADC")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_HUMMER_ADC_DA, "DeaktivÈr Hummer ADC"},
	/* de */ {IDCLS_DISABLE_HUMMER_ADC_DE, "Hummer ADC deaktivieren"},
	/* es */ {IDCLS_DISABLE_HUMMER_ADC_ES, "Deshabilitar Hummer ADC"},
	/* fr */ {IDCLS_DISABLE_HUMMER_ADC_FR, "DÈsactiver Hummer ADC"},
	/* hu */ {IDCLS_DISABLE_HUMMER_ADC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_HUMMER_ADC_IT, "Disattiva Hummer ADC"},
	/* ko */ {IDCLS_DISABLE_HUMMER_ADC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_HUMMER_ADC_NL, "Hummer ADC uitschakelen"},
	/* pl */ {IDCLS_DISABLE_HUMMER_ADC_PL, "Wy≥±cz Hummer ADC"},
	/* ru */ {IDCLS_DISABLE_HUMMER_ADC_RU, "Disable Hummer ADC"},
	/* sv */ {IDCLS_DISABLE_HUMMER_ADC_SV, "Inaktivera Hummer-A/D-omvandlare"},
	/* tr */ {IDCLS_DISABLE_HUMMER_ADC_TR, "Hummer ADC'yi pasifle˛tir"},
#endif
	
	/* c64/c64io.c, vic20/vic20io.c */
	/* en */ {IDCLS_SELECT_CONFLICT_HANDLING,    N_("Select the way the I/O collisions should be handled, (0: error message and detach all involved carts, 1: error message and detach last attached involved carts, 2: warning in log and 'AND' the valid return values")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_CONFLICT_HANDLING_DA, "VÊlg hvordan I/O-kollisioner skal hÂndteres (0: fejlbesked og afbryd alle involverede cartridges, 1: fejlbesked og afbryd senest tilsluttede cartridge, 2: Advarsel i log og returner 'logisk OG' af de gyldige returvÊrdier)"},
	/* de */ {IDCLS_SELECT_CONFLICT_HANDLING_DE, "W‰hle, wie I/O Kollisionen behandelt werden sollen, (0: Fehler Meldung\nund Entfernen s‰mtlicher betroffener Module, 1: Fehler Meldung und\nEntfernen des letzten Moduls, 2: Warnung im Log und liefere logische\nUND-Verkn¸pfung g¸ltiger R¸ckgabewerte"},
	/* es */ {IDCLS_SELECT_CONFLICT_HANDLING_ES, "Seleccionar la forma en que las colisiones deben ser manejadas, (0: mensaje de error y desconectar todas los cartuchos involucrados, 1: mensaje de error y desconectar el ˙ltimo cartucho involucrado, 2: advertencia en  log y 'AND' los valores v·lidos de retorno"},
	/* fr */ {IDCLS_SELECT_CONFLICT_HANDLING_FR, "SÈlectionner la faÁon que les collisions E/S devraient Ítre traitÈes, (0: message d'erreur et dÈtachement de toutes les cartouches impliquÈes, 1: message d'erreur et dÈtachement de la derniËre cartouches impliquÈes ‡ avoir ÈtÈ attachÈe, 2: message d'avertissement avec journalisation et 'ET' les valeurs de retour valides"},
	/* hu */ {IDCLS_SELECT_CONFLICT_HANDLING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SELECT_CONFLICT_HANDLING_IT, "Seleziona come gestire le collisioni di I/O, (0: messaggio di errore e rimozione di tutte le cartucce coinvolte, 1: messaggio di errore e rimozione delle ultima cartucce coinvolte, 2: warning nei log e 'AND' il valore di ritorno valido"},
	/* ko */ {IDCLS_SELECT_CONFLICT_HANDLING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_CONFLICT_HANDLING_NL, "Selecteer de manier waarop I/O conflicten worden afgehandeld, (0: foutmelding en ontkoppel alle betrokken cartridges, 1: foutmelding en ontkoppel alle cartridges behalve de eerste cartridge, 2: waarschuwing in het logbestand en doe een logische 'AND' met de teruggegeven waarden"},
	/* pl */ {IDCLS_SELECT_CONFLICT_HANDLING_PL, "Wybierz metodÍ obs≥ugi konfliktÛw we/wy, (0: komunikat b≥Ídu i od≥±czenie wszystkich konfliktuj±cych kartridøÛw, 1: komunikat b≥Ídu i od≥±czenie ostatnio pod≥±czonych konfliktuj±cych kartridøÛw, 2: ostrzeøenie w logu i 'AND' w≥a∂ciwe warto∂ci)"},
	/* ru */ {IDCLS_SELECT_CONFLICT_HANDLING_RU, "Select the way the I/O collisions should be handled, (0: error message and detach all involved carts, 1: error message and detach last attached involved carts, 2: warning in log and 'AND' the valid return values"},
	/* sv */ {IDCLS_SELECT_CONFLICT_HANDLING_SV, "V‰ljer hur I/O-kollisioner skall hanteras, (0: felmeddelande och koppla frÂn alla inblandade moduler, 1: felmeddelande och koppla frÂn sist anslutna inblandade modul, 2: varning i logg och logisk OCH pÂ giltiga returv‰rden)"},
	/* tr */ {IDCLS_SELECT_CONFLICT_HANDLING_TR, "G/« Áak˝˛malar˝n˝n deerlendirilme yˆntemini seÁ, (0: hata mesaj˝ ve t¸m ilgili cihazlar˝n Á˝kar˝lmas˝, 1: hata mesaj˝ ve en son tak˝lm˝˛ t¸m ilgili cihazlar˝n Á˝kar˝lmas˝, 2: log'a uyar˝ yaz˝lmas˝ ve geÁerli dˆn¸˛ deerlerinin 'VE'lenmesi"},
#endif
	
	/* userport/userport.c */
	/* en */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING,    N_("Select the way the Userport collisions should be handled, (0: error message and detach all involved devices, 1: error message and detach last attached involved device, 2: warning in log and 'AND' the valid return values")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_DE, "W‰hle, wie Userport Kollisionen behandelt werden sollen, (0: Fehler Meldung\nund Entfernen s‰mtlicher betroffener Ger‰te, 1: Fehler Meldung und\nEntfernen des letzten Ger‰ts, 2: Warnung im Log und liefere logische\nUND-Verkn¸pfung g¸ltiger R¸ckgabewerte"},
	/* es */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_NL, "Selecteer de manier waarop userport conflicten worden afgehandeld, (0: foutmelding en ontkoppel alle betrokken apparaten, 1: foutmelding en ontkoppel alle apparaten behalve het eerste apparaat, 2: waarschuwing in het logbestand en doe een logische 'AND' met de teruggegeven waarden"},
	/* pl */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SELECT_USERPORT_CONFLICT_HANDLING_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/c64tpi.c */
	/* en */ {IDCLS_ENABLE_IEEE488_INTERFACE,    N_("Enable the IEEE488 interface emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEEE488_INTERFACE_DA, "AktivÈr emulering af IEEE488-interface"},
	/* de */ {IDCLS_ENABLE_IEEE488_INTERFACE_DE, "IEEE488 Schnittstellen-Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_IEEE488_INTERFACE_ES, "Permitir emulaciÛn interface IEEE488"},
	/* fr */ {IDCLS_ENABLE_IEEE488_INTERFACE_FR, "Activer l'interface d'Èmulation IEEE488"},
	/* hu */ {IDCLS_ENABLE_IEEE488_INTERFACE_HU, "IEEE488 interfÈsz emul·ciÛ engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_IEEE488_INTERFACE_IT, "Attiva emulazione interfaccia IEEE488"},
	/* ko */ {IDCLS_ENABLE_IEEE488_INTERFACE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEEE488_INTERFACE_NL, "IEEE488 interface emulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEEE488_INTERFACE_PL, "W≥±cz emulacjÍ interfejsu IEEE488"},
	/* ru */ {IDCLS_ENABLE_IEEE488_INTERFACE_RU, "Enable the IEEE488 interface emulation"},
	/* sv */ {IDCLS_ENABLE_IEEE488_INTERFACE_SV, "Aktivera emulering av IEEE488-gr‰nssnitt"},
	/* tr */ {IDCLS_ENABLE_IEEE488_INTERFACE_TR, "IEEE488 arabirim em¸lasyonunu aktif et"},
#endif
	
	/* c64/cart/c64tpi.c */
	/* en */ {IDCLS_DISABLE_IEEE488_INTERFACE,    N_("Disable the IEEE488 interface emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEEE488_INTERFACE_DA, "DeaktivÈr emulering af IEEE488-interface"},
	/* de */ {IDCLS_DISABLE_IEEE488_INTERFACE_DE, "IEEE488 Schnittstellen-Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEEE488_INTERFACE_ES, "Deshabilitar emulaciÛn interface IEEE488"},
	/* fr */ {IDCLS_DISABLE_IEEE488_INTERFACE_FR, "DÈsactiver l'interface d'Èmulation IEEE488"},
	/* hu */ {IDCLS_DISABLE_IEEE488_INTERFACE_HU, "IEEE488 interfÈsz emul·ciÛ tilt·sa"},
	/* it */ {IDCLS_DISABLE_IEEE488_INTERFACE_IT, "Disattiva emulazione interfaccia IEEE488"},
	/* ko */ {IDCLS_DISABLE_IEEE488_INTERFACE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEEE488_INTERFACE_NL, "IEEE488 interface emulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEEE488_INTERFACE_PL, "Wy≥±cz emulacjÍ interfejsu IEEE488"},
	/* ru */ {IDCLS_DISABLE_IEEE488_INTERFACE_RU, "Disable the IEEE488 interface emulation"},
	/* sv */ {IDCLS_DISABLE_IEEE488_INTERFACE_SV, "Inaktivera emulering av IEEE488-gr‰nssnitt"},
	/* tr */ {IDCLS_DISABLE_IEEE488_INTERFACE_TR, "IEEE488 arabirim em¸lasyonunu pasifle˛tir"},
#endif
	
	/* c64/cart/c64tpi.c */
	/* en */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME,    N_("specify IEEE488 interface image name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_DA, "angiv IEEE488-interface image navn"},
	/* de */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_DE, "IEEE488 Schnittstellenmodul Imagename"},
	/* es */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_ES, "Especificar nombre imagen interface IEEE488"},
	/* fr */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_FR, "spÈcifier le nom de l'image d'interface IEEE-488"},
	/* hu */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_IT, "specifica il nome immagine dell'interfaccia IEEE488"},
	/* ko */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_NL, "Geef de naam van het IEEE488 interface bestand"},
	/* pl */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_PL, "Okre∂l nazwÍ obrazu interfejsu IEEE488"},
	/* ru */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_RU, "specify IEEE488 interface image name"},
	/* sv */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_SV, "ange namn pÂ IEEE488-gr‰nssnittsfil"},
	/* tr */ {IDCLS_SPECIFY_IEEE488_INTERFACE_NAME_TR, "IEEE488 arabirimi imaj ismini belirleyin"},
#endif
	
	/* samplerdrv/file_drv.c */
	/* en */ {IDCLS_SPECIFY_SAMPLE_NAME,    N_("Specify name of sample file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SAMPLE_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_SAMPLE_NAME_DE, "Namen f¸r Sample Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_SAMPLE_NAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_SAMPLE_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_SAMPLE_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_SAMPLE_NAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_SAMPLE_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SAMPLE_NAME_NL, "Geef naam van het samplebestand"},
	/* pl */ {IDCLS_SPECIFY_SAMPLE_NAME_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_SAMPLE_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_SAMPLE_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_SAMPLE_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tape_diag_586220_harness.c */
	/* en */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS,    N_("Enable the tape part of the 586220 diagnostics harness")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_DE, "586220 Diagnose Harness f¸r Bandteil aktivieren"},
	/* es */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_NL, "Tape onderdeel van de 586220 diagnostics gareel inschakelen"},
	/* pl */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TAPE_DIAG_586220_HARNESS_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tape_diag_586220_harness.c */
	/* en */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS,    N_("Disable the tape part of the 586220 diagnostics harness")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_DE, "586220 Diagnose Harness f¸r Bandteil deaktivieren"},
	/* es */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_NL, "Tape onderdeel van de 586220 diagnostics gareel uitschakelen"},
	/* pl */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_TAPE_DIAG_586220_HARNESS_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapelog.c */
	/* en */ {IDCLS_ENABLE_TAPELOG,    N_("Enable the tape log device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TAPELOG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TAPELOG_DE, "Bandlog Ger‰t aktivieren"},
	/* es */ {IDCLS_ENABLE_TAPELOG_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TAPELOG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TAPELOG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TAPELOG_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TAPELOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TAPELOG_NL, "Tape log apparaat inschakelen"},
	/* pl */ {IDCLS_ENABLE_TAPELOG_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TAPELOG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TAPELOG_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TAPELOG_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapelog.c */
	/* en */ {IDCLS_DISABLE_TAPELOG,    N_("Disable the tape log device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TAPELOG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_TAPELOG_DE, "Bandlog Ger‰t deaktivieren"},
	/* es */ {IDCLS_DISABLE_TAPELOG_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_TAPELOG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_TAPELOG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_TAPELOG_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_TAPELOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TAPELOG_NL, "Tape log apparaat uitschakelen"},
	/* pl */ {IDCLS_DISABLE_TAPELOG_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_TAPELOG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_TAPELOG_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_TAPELOG_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapelog.c */
	/* en */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE,    N_("Enable logging to a file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_DE, "Logging auf Datei aktivieren"},
	/* es */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_NL, "Logboekvermeldingen schrijven naar een bestand inschakelen"},
	/* pl */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TAPELOG_LOG_TO_FILE_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapelog.c */
	/* en */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG,    N_("Enable logging to the emulator log file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_DE, "Logging auf Emaulatorlogdatei aktivieren"},
	/* es */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_NL, "Schrijven naar het emulator log bestand inschakelen"},
	/* pl */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TAPELOG_LOG_TO_LOG_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapelog.c */
	/* en */ {IDCLS_SPECIFY_TAPELOG_NAME,    N_("Specify tape log file name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TAPELOG_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_TAPELOG_NAME_DE, "Bandlogdateiname ausw‰hlen"},
	/* es */ {IDCLS_SPECIFY_TAPELOG_NAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_TAPELOG_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_TAPELOG_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_TAPELOG_NAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_TAPELOG_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TAPELOG_NAME_NL, "Geef de naam van het tape logbestand"},
	/* pl */ {IDCLS_SPECIFY_TAPELOG_NAME_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_TAPELOG_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_TAPELOG_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_TAPELOG_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/georam.c */
	/* en */ {IDCLS_ENABLE_GEORAM,    N_("Enable the GEO-RAM expansion unit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_GEORAM_DA, "AktivÈr GEO-RAM-udviddelsesenheden"},
	/* de */ {IDCLS_ENABLE_GEORAM_DE, "GEO-RAM Erweiterung aktivieren"},
	/* es */ {IDCLS_ENABLE_GEORAM_ES, "Permitir unidad expansiÛn GEO-RAM"},
	/* fr */ {IDCLS_ENABLE_GEORAM_FR, "Activer l'unitÈ d'expansion GEO-RAM"},
	/* hu */ {IDCLS_ENABLE_GEORAM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_GEORAM_IT, "Attiva l'espansione GEO-RAM"},
	/* ko */ {IDCLS_ENABLE_GEORAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_GEORAM_NL, "GEO-RAM uitbreidingseenheid inschakelen"},
	/* pl */ {IDCLS_ENABLE_GEORAM_PL, "W≥±cz jednostkÍ rozszerzenia GEO-RAM"},
	/* ru */ {IDCLS_ENABLE_GEORAM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_GEORAM_SV, "Aktivera GEORAM-expansionsenhet"},
	/* tr */ {IDCLS_ENABLE_GEORAM_TR, "GEO-RAM geni˛letme birimini aktif et"},
#endif
	
	/* c64/cart/georam.c */
	/* en */ {IDCLS_DISABLE_GEORAM,    N_("Disable the GEO-RAM expansion unit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_GEORAM_DA, "DeaktivÈr GEO-RAM-udviddelsesenhed"},
	/* de */ {IDCLS_DISABLE_GEORAM_DE, "GEO-RAM Erweiterung deaktivieren"},
	/* es */ {IDCLS_DISABLE_GEORAM_ES, "Deshabilitar unidad expansiÛn GEO-RAM"},
	/* fr */ {IDCLS_DISABLE_GEORAM_FR, "DÈsactiver l'unitÈ d'expansion GEO-RAM"},
	/* hu */ {IDCLS_DISABLE_GEORAM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_GEORAM_IT, "Disattiva l'espansione GEO-RAM"},
	/* ko */ {IDCLS_DISABLE_GEORAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_GEORAM_NL, "GEO-RAM uitbreidingseenheid uitschakelen"},
	/* pl */ {IDCLS_DISABLE_GEORAM_PL, "Wy≥±cz jednostkÍ rozszerzenia GEO-RAM"},
	/* ru */ {IDCLS_DISABLE_GEORAM_RU, "¿–·Ëÿ‡’›ÿ’ ﬂ–‹Ô‚ÿ GEORAM"},
	/* sv */ {IDCLS_DISABLE_GEORAM_SV, "Inaktivera GEO RAM-expansionsenhet"},
	/* tr */ {IDCLS_DISABLE_GEORAM_TR, "GEO-RAM geni˛letme birimini pasifle˛tir"},
#endif
	
	/* c64/cart/georam.c */
	/* en */ {IDCLS_SPECIFY_GEORAM_NAME,    N_("Specify name of GEORAM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_GEORAM_NAME_DA, "Angiv navn pÂ GEORAM-image"},
	/* de */ {IDCLS_SPECIFY_GEORAM_NAME_DE, "GEORAM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_GEORAM_NAME_ES, "Especificar nombre imagen GEORAM"},
	/* fr */ {IDCLS_SPECIFY_GEORAM_NAME_FR, "SpÈcifier le nom de l'image GEO-RAM"},
	/* hu */ {IDCLS_SPECIFY_GEORAM_NAME_HU, "Adja meg a GEORAM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_GEORAM_NAME_IT, "Specifica il nome immagine GEORAM"},
	/* ko */ {IDCLS_SPECIFY_GEORAM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_GEORAM_NAME_NL, "Geef de naam van het GEORAM bestand"},
	/* pl */ {IDCLS_SPECIFY_GEORAM_NAME_PL, "Okre∂l nazwÍ obrazu GEORAM"},
	/* ru */ {IDCLS_SPECIFY_GEORAM_NAME_RU, "Specify name of GEORAM image"},
	/* sv */ {IDCLS_SPECIFY_GEORAM_NAME_SV, "Ange namn pÂ GEORAM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_GEORAM_NAME_TR, "GEORAM imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/cart/georam.c */
	/* en */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE,    N_("Allow writing to GEORAM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_DA, "Tillad skrivning til GEORAM-image."},
	/* de */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_DE, "Schreibzugriff auf GEORAM Image erlauben"},
	/* es */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_ES, "Permitir grabaciÛn sobre imagen GEORAM"},
	/* fr */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_FR, "Permettre l'Ècriture sur l'image GEO-RAM"},
	/* hu */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_IT, "Attiva la scrittura su immagine GEORAM"},
	/* ko */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_NL, "Schrijven naar GEORAM bestand toestaan"},
	/* pl */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_PL, "PozwÛl na zapis do obrazu GEORAM"},
	/* ru */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_RU, "Allow writing to GEORAM image"},
	/* sv */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_SV, "TillÂt skrivning till GEORAM-avbildning"},
	/* tr */ {IDCLS_ALLOW_WRITING_TO_GEORAM_IMAGE_TR, "GEORAM imaj˝na yazmay˝ aktif et"},
#endif
	
	/* c64/cart/georam.c */
	/* en */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE,    N_("Do not write to GEORAM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_DA, "Skriv ikke til GEORAM-image"},
	/* de */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_DE, "Kein Schreibzugriff auf GEORAM Image"},
	/* es */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_ES, "No grabar en imagen GEORAM"},
	/* fr */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_FR, "Ne pas Ècrire sur l'image GEO-RAM"},
	/* hu */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_IT, "Disattiva la scrittura su immagine GEORAM"},
	/* ko */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_NL, "Schrijf niet naar GEORAM bestand"},
	/* pl */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_PL, "Nie zapisuj do obrazu GEORAM"},
	/* ru */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_RU, "Do not write to GEORAM image"},
	/* sv */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_SV, "Skriv inte till GEORAM-avbildning"},
	/* tr */ {IDCLS_DO_NOT_WRITE_TO_GEORAM_IMAGE_TR, "GEORAM imaj˝na yazma"},
#endif
	
	/* c64/cart/georam.c */
	/* en */ {IDCLS_SWAP_CART_IO,    N_("Swap io mapping (map cart I/O-1 to VIC20 I/O-3 and cart I/O-2 to VIC20 I/O-2)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SWAP_CART_IO_DA, "Ombyt I/O-forbindelse (forbind cart I/O-1 til VIC20 I/O-3 og cart I/O-2 til VIC20 I/O-2)"},
	/* de */ {IDCLS_SWAP_CART_IO_DE, "Vertausche IO Zuordnung (cart I/O-1 auf VIC20 I/O-3 und cart I/O-2 auf VIC20 I/O-2)"},
	/* es */ {IDCLS_SWAP_CART_IO_ES, "Intercambiar mapeado IO (map cart I/O-1 to VIC20 I/O-3 and cart I/O-2 to VIC20 I/O-2)"},
	/* fr */ {IDCLS_SWAP_CART_IO_FR, "…changer le mappage E/S (mapper I/O-1 cartouche ‡ I/O-3 VIC20 et I/O-2 cartouche ‡ I/O-2 VIC20)"},
	/* hu */ {IDCLS_SWAP_CART_IO_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SWAP_CART_IO_IT, "Scambia mapping di I/O (mappa I/O-1 cart su I/O-3 VIC20 e I/O-2 cart su I/O-2 VIC20)"},
	/* ko */ {IDCLS_SWAP_CART_IO_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SWAP_CART_IO_NL, "Verwissel de io toewijzing (verwijs cart I/O-1 naar VIC20 I/O-3 en cart I/O-2 naar VIC20 I/O-2)"},
	/* pl */ {IDCLS_SWAP_CART_IO_PL, "ZamieÒ mapowanie we/wy (mapuj  we/wy-1  kartridøa do we/wy-3 VIC20 i we/wy-2 kartidøa do we/wy-2 VIC20)"},
	/* ru */ {IDCLS_SWAP_CART_IO_RU, "Swap io mapping (map cart I/O-1 to VIC20 I/O-3 and cart I/O-2 to VIC20 I/O-2)"},
	/* sv */ {IDCLS_SWAP_CART_IO_SV, "V‰xla io-mappning (mappa modul-I/O 1 till VIC20-I/O 3 och modul-I/O 2 till VIC20-I/O 2)"},
	/* tr */ {IDCLS_SWAP_CART_IO_TR, "G/« haritalamas˝n˝ yer dei˛tir (harita kart˝ G/«-1 ile VIC20 G/«-3 ve kart G/«-2 ile VIC20 G/«-2)"},
#endif
	
	/* c64/cart/georam.c */
	/* en */ {IDCLS_DONT_SWAP_CART_IO,    N_("Don't swap io mapping (map cart I/O-1 to VIC20 I/O-2 and cart I/O-2 to VIC20 I/O-3)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DONT_SWAP_CART_IO_DA, "Ombyt ikke I/O-forbindelse (forbind cart I/O-1 til VIC20 I/O-2 og cart I/O-2 til VIC20 I/O-3)"},
	/* de */ {IDCLS_DONT_SWAP_CART_IO_DE, "Keine Vertauschung der IO Zuordnung (cart I/O-1 auf VIC20 I/O-2 und cart I/O-2 auf VIC20 I/O-3)"},
	/* es */ {IDCLS_DONT_SWAP_CART_IO_ES, "No intercambiar mapeado IO (map cart I/O-1 to VIC20 I/O-2 and cart I/O-2 to VIC20 I/O-3)"},
	/* fr */ {IDCLS_DONT_SWAP_CART_IO_FR, "Ne pas Èchanger le mappage E/S (mapper I/O-1 cartouche ‡ I/O-2 VIC20 et I/O-2 cartouche ‡ I/O-3 VIC20)"},
	/* hu */ {IDCLS_DONT_SWAP_CART_IO_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DONT_SWAP_CART_IO_IT, "Non scambiare mapping di I/O (mappa I/O-1 cart su I/O-2 VIC20 e I/O-2 cart su I/O-3 VIC20)"},
	/* ko */ {IDCLS_DONT_SWAP_CART_IO_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DONT_SWAP_CART_IO_NL, "Verwissel de io toewijzing niet (verwijs cart I/O-1 naar VIC20 I/O-2 en cart I/O-2 naar VIC20 I/O-3)"},
	/* pl */ {IDCLS_DONT_SWAP_CART_IO_PL, "Nie zamieniaj mapowania we/wy (mapuj  we/wy-1  kartridøa do we/wy-2 VIC20 i we/wy-2 kartidøa do we/wy-3 VIC20)"},
	/* ru */ {IDCLS_DONT_SWAP_CART_IO_RU, "Don't swap io mapping (map cart I/O-1 to VIC20 I/O-2 and cart I/O-2 to VIC20 I/O-3)"},
	/* sv */ {IDCLS_DONT_SWAP_CART_IO_SV, "V‰xla inte io-mappning (mappa modul-I/O 1 till VIC20-I/O 2 och modul-I/O 2 till VIC20-I/O 3)"},
	/* tr */ {IDCLS_DONT_SWAP_CART_IO_TR, "G/« haritalamas˝n˝ yer dei˛tirme (harita kart˝ G/«-1 ile VIC20 G/«-2 ve kart G/«-2 ile VIC20 G/«-3)"},
#endif
	
	/* c64/cart/sfx_soundexpander.c, c64/cart/sfx_soundsampler.c */
	/* en */ {IDCLS_MAP_CART_IO_2,    N_("Swap io mapping (map cart I/O to VIC20 I/O-2)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MAP_CART_IO_2_DA, "Ombyt I/O-forbindelse (forbind cart I/O til VIC20 I/O-2)"},
	/* de */ {IDCLS_MAP_CART_IO_2_DE, "Vertausche IO Zuordnung (cart I/O auf VIC20 I/O-2)"},
	/* es */ {IDCLS_MAP_CART_IO_2_ES, "Intercambiar mapeado IO (map cart I/O to VIC20 I/O-2)"},
	/* fr */ {IDCLS_MAP_CART_IO_2_FR, "…changer le mappage E/S (mapper I/O cartouche ‡ I/O-2 VIC20)"},
	/* hu */ {IDCLS_MAP_CART_IO_2_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MAP_CART_IO_2_IT, "Scambia mapping di I/O (mappa I/O cart su I/O-2 VIC20)"},
	/* ko */ {IDCLS_MAP_CART_IO_2_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MAP_CART_IO_2_NL, "Verwissel de io toewijzing (verwijs cart I/O naar VIC20 I/O-2)"},
	/* pl */ {IDCLS_MAP_CART_IO_2_PL, "ZamieÒ mapowanie we/wy (mapuj  we/wy  kartridøa do we/wy-2 VIC20)"},
	/* ru */ {IDCLS_MAP_CART_IO_2_RU, "Swap io mapping (map cart I/O to VIC20 I/O-2)"},
	/* sv */ {IDCLS_MAP_CART_IO_2_SV, "V‰xla io-mappning (mappa modul-I/O till VIC20-I/O 2)"},
	/* tr */ {IDCLS_MAP_CART_IO_2_TR, "G/« haritalamas˝n˝ yer dei˛tir (harita kart˝ G/« ile VIC20 G/«-2)"},
#endif
	
	/* c64/cart/sfx_soundexpander.c, c64/cart/sfx_soundsampler.c */
	/* en */ {IDCLS_MAP_CART_IO_3,    N_("Don't swap io mapping (map cart I/O to VIC20 I/O-3)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MAP_CART_IO_3_DA, "Ombyt ikke I/O-forbindelse (forbind cart I/O til VIC20 I/O-3)"},
	/* de */ {IDCLS_MAP_CART_IO_3_DE, "Keine Vertauschung der IO Zuordnung (cart I/O auf VIC20 I/O-3)"},
	/* es */ {IDCLS_MAP_CART_IO_3_ES, "No intercambia mapeado IO (map cart I/O to VIC20 I/O-3)"},
	/* fr */ {IDCLS_MAP_CART_IO_3_FR, "Ne pas Èchanger le mappage E/S (mapper I/O cartouche ‡ I/O-3 VIC20)"},
	/* hu */ {IDCLS_MAP_CART_IO_3_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MAP_CART_IO_3_IT, "Non scambiare mapping di I/O (mappa I/O cart su I/O-3 VIC20)"},
	/* ko */ {IDCLS_MAP_CART_IO_3_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MAP_CART_IO_3_NL, "Verwissel de io toewijzing niet (verwijs cart I/O naar VIC20 I/O-3)"},
	/* pl */ {IDCLS_MAP_CART_IO_3_PL, "Nie zamieniaj mapowania we/wy (mapuj  we/wy  kartridøa do we/wy-3 VIC20)"},
	/* ru */ {IDCLS_MAP_CART_IO_3_RU, "Don't swap io mapping (map cart I/O to VIC20 I/O-3)"},
	/* sv */ {IDCLS_MAP_CART_IO_3_SV, "V‰xla io-mappning (mappa modul-I/O till VIC20-I/O 3)"},
	/* tr */ {IDCLS_MAP_CART_IO_3_TR, "G/« haritalamas˝n˝ yer dei˛tirme (harita kart˝ G/« ile VIC20 G/«-3)"},
#endif
	
	/* c64/cart/georam.c, c64/cart/ramcart.c, c64/cart/reu.c, pet/petreu.c */
	/* en */ {IDCLS_P_SIZE_IN_KB,    N_("<size in KB>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_SIZE_IN_KB_DA, "<st¯rrelse i kB>"},
	/* de */ {IDCLS_P_SIZE_IN_KB_DE, "<Grˆﬂe in KB>"},
	/* es */ {IDCLS_P_SIZE_IN_KB_ES, "<tamaÒo en KB>"},
	/* fr */ {IDCLS_P_SIZE_IN_KB_FR, "<taille en ko>"},
	/* hu */ {IDCLS_P_SIZE_IN_KB_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_SIZE_IN_KB_IT, "<dimensione in KB>"},
	/* ko */ {IDCLS_P_SIZE_IN_KB_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_SIZE_IN_KB_NL, "<grootte in KB>"},
	/* pl */ {IDCLS_P_SIZE_IN_KB_PL, "<rozmiar w KB>"},
	/* ru */ {IDCLS_P_SIZE_IN_KB_RU, "<‡–◊‹’‡ “ ∫±>"},
	/* sv */ {IDCLS_P_SIZE_IN_KB_SV, "<storlek i KB>"},
	/* tr */ {IDCLS_P_SIZE_IN_KB_TR, "<KB cinsinden boyut>"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_P_SIZE,    N_("<size>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_SIZE_DE, "<Grˆﬂe>"},
	/* es */ {IDCLS_P_SIZE_ES, "<TamaÒo>"},
	/* fr */ {IDCLS_P_SIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_SIZE_IT, "<dimensione>"},
	/* ko */ {IDCLS_P_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_SIZE_NL, "<grootte>"},
	/* pl */ {IDCLS_P_SIZE_PL, "<wielko∂Ê>"},
	/* ru */ {IDCLS_P_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_SIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_SIZE_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-sidcart.c */
	/* en */ {IDCLS_P_ADDRESS,    N_("<address>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_ADDRESS_DA, "<adresse>"},
	/* de */ {IDCLS_P_ADDRESS_DE, "<Adresse>"},
	/* es */ {IDCLS_P_ADDRESS_ES, "<direcciÛn>"},
	/* fr */ {IDCLS_P_ADDRESS_FR, "<adresse>"},
	/* hu */ {IDCLS_P_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_ADDRESS_IT, "<indirizzo>"},
	/* ko */ {IDCLS_P_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_ADDRESS_NL, "<adres>"},
	/* pl */ {IDCLS_P_ADDRESS_PL, "<address>"},
	/* ru */ {IDCLS_P_ADDRESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_ADDRESS_SV, "<adress>"},
	/* tr */ {IDCLS_P_ADDRESS_TR, "<adres>"},
#endif
	
	/* pet/pet-sidcart.c */
	/* en */ {IDCLS_P_CLOCK,    N_("<clock>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_CLOCK_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_CLOCK_DE, "<Takt>"},
	/* es */ {IDCLS_P_CLOCK_ES, "<Reloj>"},
	/* fr */ {IDCLS_P_CLOCK_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_CLOCK_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_CLOCK_IT, "<clock>"},
	/* ko */ {IDCLS_P_CLOCK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_CLOCK_NL, "<klok>"},
	/* pl */ {IDCLS_P_CLOCK_PL, "<zegar>"},
	/* ru */ {IDCLS_P_CLOCK_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_CLOCK_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_CLOCK_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/georam.c */
	/* en */ {IDCLS_GEORAM_SIZE,    N_("Size of the GEORAM expansion unit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_GEORAM_SIZE_DA, "St¯rrelse af GEORAM-udviddelsesenhed"},
	/* de */ {IDCLS_GEORAM_SIZE_DE, "Grˆﬂe der GEORAM Erweiterung"},
	/* es */ {IDCLS_GEORAM_SIZE_ES, "TamaÒo de la unidad expansiÛn GEORAM"},
	/* fr */ {IDCLS_GEORAM_SIZE_FR, "Taille de l'unitÈ d'expansion GEO-RAM"},
	/* hu */ {IDCLS_GEORAM_SIZE_HU, "GEORAM Expansion Unit mÈrete"},
	/* it */ {IDCLS_GEORAM_SIZE_IT, "Dimensione dell'espansione GEORAM"},
	/* ko */ {IDCLS_GEORAM_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_GEORAM_SIZE_NL, "Grootte van de GEORAM uitbreidingseenheid"},
	/* pl */ {IDCLS_GEORAM_SIZE_PL, "Rozmiar jednostki rozszerzenia GEORAM"},
	/* ru */ {IDCLS_GEORAM_SIZE_RU, "Size of the GEORAM expansion unit"},
	/* sv */ {IDCLS_GEORAM_SIZE_SV, "Storlek pÂ GEORAM-expansionsenhet"},
	/* tr */ {IDCLS_GEORAM_SIZE_TR, "GEORAM geni˛letme biriminin boyutu"},
#endif
	
	/* pet/petreu.c */
	/* en */ {IDCLS_ENABLE_PETREU,    N_("Enable the PET Ram and Expansion Unit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_PETREU_DA, "AktivÈr PET REU"},
	/* de */ {IDCLS_ENABLE_PETREU_DE, "PET RAM Speicher und Erweiterungsmodul aktivieren"},
	/* es */ {IDCLS_ENABLE_PETREU_ES, "Permitir PET ram y Unidad ExpansiÛn"},
	/* fr */ {IDCLS_ENABLE_PETREU_FR, "Activer l'expansion d'unitÈ et de RAM du PET"},
	/* hu */ {IDCLS_ENABLE_PETREU_HU, "PET RAM and Expansion Unit engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_PETREU_IT, "Attiva la RAM e l'unit‡ di espansione del PET"},
	/* ko */ {IDCLS_ENABLE_PETREU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_PETREU_NL, "PET RAM en uitbreidingseenheid inschakelen"},
	/* pl */ {IDCLS_ENABLE_PETREU_PL, "W≥±cz PET Ram i Expansion Unit"},
	/* ru */ {IDCLS_ENABLE_PETREU_RU, "Enable the PET Ram and Expansion Unit"},
	/* sv */ {IDCLS_ENABLE_PETREU_SV, "Aktivera PET Ram- och -expansionsenhet"},
	/* tr */ {IDCLS_ENABLE_PETREU_TR, "PET Ram ve Geni˛letme Birimi'ni aktif et"},
#endif
	
	/* pet/petreu.c */
	/* en */ {IDCLS_DISABLE_PETREU,    N_("Disable the PET Ram and Expansion Unit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_PETREU_DA, "DeaktivÈr PET REU"},
	/* de */ {IDCLS_DISABLE_PETREU_DE, "PET RAM Speicher und Erweiterungsmodul dektivieren"},
	/* es */ {IDCLS_DISABLE_PETREU_ES, "Deshabilitar PET RAM y Unidad ExpansiÛn"},
	/* fr */ {IDCLS_DISABLE_PETREU_FR, "DÈsactiver l'expansion d'unitÈ et de RAM du PET"},
	/* hu */ {IDCLS_DISABLE_PETREU_HU, "PET RAM and Expansion Unit tilt·sa"},
	/* it */ {IDCLS_DISABLE_PETREU_IT, "Disattiva la RAM e l'unit‡ di espansione del PET"},
	/* ko */ {IDCLS_DISABLE_PETREU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_PETREU_NL, "PET RAM en uitbreidingseenheid uitschakelen"},
	/* pl */ {IDCLS_DISABLE_PETREU_PL, "Wy≥±cz PET Ram i Expansion Unit"},
	/* ru */ {IDCLS_DISABLE_PETREU_RU, "Disable the PET Ram and Expansion Unit"},
	/* sv */ {IDCLS_DISABLE_PETREU_SV, "Inaktivera PET Ram- och -expansionsenhet"},
	/* tr */ {IDCLS_DISABLE_PETREU_TR, "PET Ram ve Geni˛letme Birimi'ni pasifle˛tir"},
#endif
	
	/* pet/petreu.c */
	/* en */ {IDCLS_SPECIFY_PETREU_NAME,    N_("Specify name of PET Ram and Expansion Unit image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PETREU_NAME_DA, "Angiv navn pÂ PET REU-image"},
	/* de */ {IDCLS_SPECIFY_PETREU_NAME_DE, "Namen f¸r PET RAM Speicher und Erweiterungsmodul w‰hlen"},
	/* es */ {IDCLS_SPECIFY_PETREU_NAME_ES, "Especificar nombre imagen PET Ram y Unidad ExpansiÛn"},
	/* fr */ {IDCLS_SPECIFY_PETREU_NAME_FR, "SpÈcifier le nom de l'image díunitÈ d'expansion RAM"},
	/* hu */ {IDCLS_SPECIFY_PETREU_NAME_HU, "Adja meg a PET RAM and Expansion Unit kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_PETREU_NAME_IT, "Specifica il nome immagine della RAM e dell'unit‡ di espansione del PET"},
	/* ko */ {IDCLS_SPECIFY_PETREU_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PETREU_NAME_NL, "Geef de naam van het PET RAM en uitbreidingseenheid bestand"},
	/* pl */ {IDCLS_SPECIFY_PETREU_NAME_PL, "Okre∂l nazwÍ obrazu PET Ram i Expansion Unit"},
	/* ru */ {IDCLS_SPECIFY_PETREU_NAME_RU, "Specify name of PET Ram and Expansion Unit image"},
	/* sv */ {IDCLS_SPECIFY_PETREU_NAME_SV, "Ange namn pÂ PET Ram- och -expansionsenhet"},
	/* tr */ {IDCLS_SPECIFY_PETREU_NAME_TR, "PET Ram ve Geni˛letme Birimi'nin ismini belirt"},
#endif
	
	/* pet/petreu.c */
	/* en */ {IDCLS_PETREU_SIZE,    N_("Size of the PET Ram and Expansion Unit. (128/512/1024/2048)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PETREU_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PETREU_SIZE_DE, "Grˆﬂe des PET RAM Speicher und Erweiterungsmodul. (128/512/1024/2048)"},
	/* es */ {IDCLS_PETREU_SIZE_ES, "TamaÒo de PET Ram y Unidad ExpansiÛn. (128/512/1024/2048)"},
	/* fr */ {IDCLS_PETREU_SIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PETREU_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PETREU_SIZE_IT, "Dimensione della RAM e dell'unit‡ di espansione del PET. (128/512/1024/2048)"},
	/* ko */ {IDCLS_PETREU_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PETREU_SIZE_NL, "Grootte van de PET RAM en uitbreidingseenheid. (128/512/1024/2048)"},
	/* pl */ {IDCLS_PETREU_SIZE_PL, "Rozmiar PET Ram i Expansion Unit. (128/512/1024/2048)"},
	/* ru */ {IDCLS_PETREU_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PETREU_SIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PETREU_SIZE_TR, ""},  /* fuzzy */
#endif
	
	/* pet/petdww.c */
	/* en */ {IDCLS_SPECIFY_PETDWW_NAME,    N_("Specify name of PET DWW image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PETDWW_NAME_DA, "Angiv navn pÂ PET DWW-image"},
	/* de */ {IDCLS_SPECIFY_PETDWW_NAME_DE, "Namen f¸r PET DWW Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_PETDWW_NAME_ES, "Especificar nombre imagen PET DWW"},
	/* fr */ {IDCLS_SPECIFY_PETDWW_NAME_FR, "SpÈcifier le nom de l'image PET DWW"},
	/* hu */ {IDCLS_SPECIFY_PETDWW_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_PETDWW_NAME_IT, "Specifica il nome immagine PET DWW"},
	/* ko */ {IDCLS_SPECIFY_PETDWW_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PETDWW_NAME_NL, "Geef de naam van het PET DWW bestand"},
	/* pl */ {IDCLS_SPECIFY_PETDWW_NAME_PL, "Okre∂l nazwÍ obrazu PET DWW"},
	/* ru */ {IDCLS_SPECIFY_PETDWW_NAME_RU, "Specify name of PET DWW image"},
	/* sv */ {IDCLS_SPECIFY_PETDWW_NAME_SV, "Ange namn pÂ PET DWW-avbildning"},
	/* tr */ {IDCLS_SPECIFY_PETDWW_NAME_TR, "PET DWW imaj˝n˝n ismini belirt"},
#endif
	
	/* pet/petdww.c */
	/* en */ {IDCLS_ENABLE_PETDWW,    N_("Enable the PET DWW hi-res board")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_PETDWW_DA, "AktivÈr PET DWW h¯jopl¯sningskort"},
	/* de */ {IDCLS_ENABLE_PETDWW_DE, "PET DWW hi-res Karte aktivieren"},
	/* es */ {IDCLS_ENABLE_PETDWW_ES, "Permitir tablero de alta resoluciÛn PET DWW"},
	/* fr */ {IDCLS_ENABLE_PETDWW_FR, "Activer la carte haute-rÈs PET DWW"},
	/* hu */ {IDCLS_ENABLE_PETDWW_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_PETDWW_IT, "Attiva la scheda hi-res PET DWW"},
	/* ko */ {IDCLS_ENABLE_PETDWW_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_PETDWW_NL, "PET DWW hi-res bord inschakelen"},
	/* pl */ {IDCLS_ENABLE_PETDWW_PL, "W≥±cz uk≥ad PET DWW hi-res"},
	/* ru */ {IDCLS_ENABLE_PETDWW_RU, "Enable the PET DWW hi-res board"},
	/* sv */ {IDCLS_ENABLE_PETDWW_SV, "Aktivera PET DWW-hˆgupplˆsningskort"},
	/* tr */ {IDCLS_ENABLE_PETDWW_TR, "PET DWW hi-res boardunu aktif et"},
#endif
	
	/* pet/petdww.c */
	/* en */ {IDCLS_DISABLE_PETDWW,    N_("Disable the PET DWW hi-res board")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_PETDWW_DA, "DeaktivÈr PET DWW h¯jopl¯sningskort"},
	/* de */ {IDCLS_DISABLE_PETDWW_DE, "PET DWW Hi-Res Karte deaktivieren"},
	/* es */ {IDCLS_DISABLE_PETDWW_ES, "Deshabilitar tablero de alta resoluciÛn PET DWW"},
	/* fr */ {IDCLS_DISABLE_PETDWW_FR, "DÈsactiver la carte haute-rÈs PET DWW"},
	/* hu */ {IDCLS_DISABLE_PETDWW_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_PETDWW_IT, "Disattiva la scheda hi-res PET DWW"},
	/* ko */ {IDCLS_DISABLE_PETDWW_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_PETDWW_NL, "PET DWW hi-res bord uitschakelen"},
	/* pl */ {IDCLS_DISABLE_PETDWW_PL, "Wy≥±cz p≥ytÍ PET DWW hi-res"},
	/* ru */ {IDCLS_DISABLE_PETDWW_RU, "Disable the PET DWW hi-res board"},
	/* sv */ {IDCLS_DISABLE_PETDWW_SV, "Inaktivera PET DWW-hˆgupplˆsningskort"},
	/* tr */ {IDCLS_DISABLE_PETDWW_TR, "PET DWW hi-res boardu pasifle˛tir"},
#endif
	
	/* userport/userport_dac.c */
	/* en */ {IDCLS_ENABLE_USERPORT_DAC,    N_("Enable Userport DAC for sound output")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_DAC_DA, "AktivÈr brugerport DAC for lydoutput"},
	/* de */ {IDCLS_ENABLE_USERPORT_DAC_DE, "Userport DAC Sound aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_DAC_ES, "Permitir puerto DAC para salida de sonido"},
	/* fr */ {IDCLS_ENABLE_USERPORT_DAC_FR, "Activer DAC port utilisateur comme sortie de son"},
	/* hu */ {IDCLS_ENABLE_USERPORT_DAC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_DAC_IT, "Attiva DAC su userport per la riproduzione audio"},
	/* ko */ {IDCLS_ENABLE_USERPORT_DAC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_DAC_NL, "Userport DAC voor geluidsuitvoer inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_DAC_PL, "W≥±cz userport DAC jako wyj∂cie dºwiÍku"},
	/* ru */ {IDCLS_ENABLE_USERPORT_DAC_RU, "Enable Userport DAC for sound output"},
	/* sv */ {IDCLS_ENABLE_USERPORT_DAC_SV, "Aktivera D/A-omvandlare pÂ anv‰ndarporten fˆr ljudutdata"},
	/* tr */ {IDCLS_ENABLE_USERPORT_DAC_TR, "Ses Á˝k˝˛˝ iÁin Userport DAC'˝ aktif et"},
#endif
	
	/* userport/userport_dac.c */
	/* en */ {IDCLS_DISABLE_USERPORT_DAC,    N_("Disable Userport DAC for sound output")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_DAC_DA, "DeaktivÈr brugerport DAC for lydoutput"},
	/* de */ {IDCLS_DISABLE_USERPORT_DAC_DE, "Userport DAC Sound deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_DAC_ES, "Deshabilitar puerto DAC para salida de sonido"},
	/* fr */ {IDCLS_DISABLE_USERPORT_DAC_FR, "DÈsactiver DAC port utilisateur comme sortie de son"},
	/* hu */ {IDCLS_DISABLE_USERPORT_DAC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_DAC_IT, "Disattiva DAC su userport per la riproduzione audio"},
	/* ko */ {IDCLS_DISABLE_USERPORT_DAC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_DAC_NL, "Userport DAC voor geluidsuitvoer uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_DAC_PL, "Wy≥±cz userport DAC jako wy∂cie dºwiÍku"},
	/* ru */ {IDCLS_DISABLE_USERPORT_DAC_RU, "Disable Userport DAC for sound output"},
	/* sv */ {IDCLS_DISABLE_USERPORT_DAC_SV, "Inaktivera D/A-omvandlare pÂ anv‰ndarporten fˆr ljudutdata"},
	/* tr */ {IDCLS_DISABLE_USERPORT_DAC_TR, "Ses Á˝k˝˛˝ iÁin Userport DAC'˝ pasifle˛tir"},
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDCLS_ENABLE_USERPORT_JOY,    N_("Enable Userport joystick adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_JOY_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_JOY_DE, "Userport Joystick Adapter aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_JOY_ES, "Permitir adaptador puerto joystick"},
	/* fr */ {IDCLS_ENABLE_USERPORT_JOY_FR, "Activer l'adaptateur joystick port utilisateur"},
	/* hu */ {IDCLS_ENABLE_USERPORT_JOY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_JOY_IT, "Attiva adattatore joystick su userport"},
	/* ko */ {IDCLS_ENABLE_USERPORT_JOY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_JOY_NL, "Userport joystick adapter inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_JOY_PL, "W≥±cz z≥±cze userportu joysticka"},
	/* ru */ {IDCLS_ENABLE_USERPORT_JOY_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_JOY_SV, "Aktivera adapter fˆr anv‰ndarportstyrspakar"},
	/* tr */ {IDCLS_ENABLE_USERPORT_JOY_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDCLS_DISABLE_USERPORT_JOY,    N_("Disable Userport joystick adapter")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_JOY_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_JOY_DE, "Userport Joystick Adapter deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_JOY_ES, "Deshabilitar adaptador puerto joystick"},
	/* fr */ {IDCLS_DISABLE_USERPORT_JOY_FR, "DÈsactiver l'adaptateur Joystick port utilisateur"},
	/* hu */ {IDCLS_DISABLE_USERPORT_JOY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_JOY_IT, "Disattiva adattatore joystick su userport"},
	/* ko */ {IDCLS_DISABLE_USERPORT_JOY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_JOY_NL, "Userport joystick adapter uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_JOY_PL, "Wy≥±cz z≥±cze userportu joysticka"},
	/* ru */ {IDCLS_DISABLE_USERPORT_JOY_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_JOY_SV, "Inativera adapter fˆr anv‰ndarportstyrspakar"},
	/* tr */ {IDCLS_DISABLE_USERPORT_JOY_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_joystick.c */
	/* en */ {IDCLS_SET_USERPORT_JOY_TYPE,    N_("Set Userport joystick adapter type (0: CGA/Protovision, 1: PET, 2: Hummer, 3: OEM, 4: DXS/HIT, 5: Kingsoft, 6: Starbyte)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_USERPORT_JOY_TYPE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_USERPORT_JOY_TYPE_DE, "Setze Userport Joystick adapter Typ (0: CGA/Protovision, 1: PET, 2: Hummer, 3: OEM, 4: DXS/HIT, 5: Kingsoft, 6: Starbyte)"},
	/* es */ {IDCLS_SET_USERPORT_JOY_TYPE_ES, "Seleccionar tipo adaptador para puerto joystick  (0: CGA/Protovision, 1: PET, 2: Hummer, 3: OEM, 4: DXS/HIT, 5: Kingsoft, 6: Starbyte)"},
	/* fr */ {IDCLS_SET_USERPORT_JOY_TYPE_FR, "DÈfinir le type de joystick port utilisateur (0: CGA/Protovision, 1: PET, 2: Hummer, 3: OEM, 4: DXS/HIT, 5: Kingsoft, 6: Starbyte)"},
	/* hu */ {IDCLS_SET_USERPORT_JOY_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_USERPORT_JOY_TYPE_IT, "Imposta il tipo di adattatore joystick su userport (0: CGA/Protovision, 1: PET, 2: Hummer, 3: OEM, 4: DXS/HIT, 5: Kingsoft, 6: Starbyte)"},
	/* ko */ {IDCLS_SET_USERPORT_JOY_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_USERPORT_JOY_TYPE_NL, "Stel Userport joystick adapter soort in (0: CGA/Protovision, 1: PET, 2: Hummer, 3: OEM, 4: DXS/HIT, 5: Kingsoft, 6: Starbyte)"},
	/* pl */ {IDCLS_SET_USERPORT_JOY_TYPE_PL, "Okre∂l typ z≥±cza userportu joysticka (0: CGA/Protovision, 1: PET, 2: Hummer, 3: OEM, 4: DXS/HIT, 5: Kingsoft, 6: Starbyte)"},
	/* ru */ {IDCLS_SET_USERPORT_JOY_TYPE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_USERPORT_JOY_TYPE_SV, "V‰lj typ av styrspakadapter fˆr anv‰ndarporten (0: CGA/Protovision, 1: PET, 2: Hummer, 3: OEM, 4: DXS/HIT, 5: Kingsoft, 6: Starbyte)"},
	/* tr */ {IDCLS_SET_USERPORT_JOY_TYPE_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_58321a.c */
	/* en */ {IDCLS_ENABLE_USERPORT_RTC_58321A,    N_("Enable Userport RTC (58321a)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_RTC_58321A_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_RTC_58321A_DE, "Userport RTC (58321a) aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_RTC_58321A_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_USERPORT_RTC_58321A_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_USERPORT_RTC_58321A_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_RTC_58321A_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_USERPORT_RTC_58321A_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_RTC_58321A_NL, "Userport RTC (58321a) inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_RTC_58321A_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_USERPORT_RTC_58321A_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_USERPORT_RTC_58321A_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_58321a.c */
	/* en */ {IDCLS_DISABLE_USERPORT_RTC_58321A,    N_("Disable Userport RTC (58321a)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_RTC_58321A_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_RTC_58321A_DE, "Userport RTC (58321a) deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_RTC_58321A_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_USERPORT_RTC_58321A_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_USERPORT_RTC_58321A_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_RTC_58321A_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_USERPORT_RTC_58321A_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_RTC_58321A_NL, "Userport RTC (58321a) uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_RTC_58321A_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_USERPORT_RTC_58321A_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_USERPORT_RTC_58321A_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_58321a.c */
	/* en */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE,    N_("Enable saving of the Userport RTC (58321a) data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_DE, "Userport RTC (58321a) Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_NL, "Opslaan van de Userport RTC (58321a) gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_USERPORT_RTC_58321A_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_58321a.c */
	/* en */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE,    N_("Disable saving of the Userport RTC (58321a) data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_DE, "Userport RTC (58321a) Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_NL, "Opslaan van de Userport RTC (58321a) gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_USERPORT_RTC_58321A_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_ds1307.c */
	/* en */ {IDCLS_ENABLE_USERPORT_RTC_DS1307,    N_("Enable Userport RTC (DS1307)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_DE, "Userport RTC (DS1307) aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_NL, "Userport RTC (DS1307) inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_ds1307.c */
	/* en */ {IDCLS_DISABLE_USERPORT_RTC_DS1307,    N_("Disable Userport RTC (DS1307)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_DE, "Userport RTC (DS1307) deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_NL, "Userport RTC (DS1307) uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_ds1307.c */
	/* en */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE,    N_("Enable saving of the Userport RTC (DS1307) data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_DE, "Userport RTC (DS1307) Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_NL, "Opslaan van de Userport RTC (DS1307) gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_USERPORT_RTC_DS1307_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_rtc_ds1307.c */
	/* en */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE,    N_("Disable saving of the Userport RTC (DS1307) data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_DE, "Userport RTC (DS1307) Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_NL, "Opslaan van de Userport RTC (DS1307) gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_USERPORT_RTC_DS1307_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_diag_586220_harness.c */
	/* en */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS,    N_("Enable Userport diag 586220 harness module")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_DE, "Userport Diag 586220 Harnessmodul aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_NL, "Userport diag 586220 gareel module inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_USERPORT_DIAG_586220_HARNESS_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_diag_586220_harness.c */
	/* en */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS,    N_("Disable Userport diag 586220 harness module")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_DE, "Userport Diag 586220 Harnessmodul deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_NL, "Userport diag 586220 gareel module uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_USERPORT_DIAG_586220_HARNESS_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/bbrtc.c */
	/* en */ {IDCLS_ENABLE_BBRTC_SAVE,    N_("Enable saving of the BBRTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_BBRTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_BBRTC_SAVE_DE, "BBRTC Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_BBRTC_SAVE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_BBRTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_BBRTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_BBRTC_SAVE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_BBRTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_BBRTC_SAVE_NL, "Opslaan van de BBRTC gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_BBRTC_SAVE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_BBRTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_BBRTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_BBRTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* joyport/bbrtc.c */
	/* en */ {IDCLS_DISABLE_BBRTC_SAVE,    N_("Disable saving of the BBRTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_BBRTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_BBRTC_SAVE_DE, "BBRTC Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_BBRTC_SAVE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_BBRTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_BBRTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_BBRTC_SAVE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_BBRTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_BBRTC_SAVE_NL, "Opslaan van de BBRTC gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_BBRTC_SAVE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_BBRTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_BBRTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_BBRTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/sense-dongle.c */
	/* en */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE,    N_("Enable tape sense dongle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_DE, "Band Sense Dongle aktivieren"},
	/* es */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_NL, "Tape sense dongle inschakelen"},
	/* pl */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TAPE_SENSE_DONGLE_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/sense-dongle.c */
	/* en */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE,    N_("Disable tape sense dongle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_DE, "Band Sense Dongle deaktivieren"},
	/* es */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_NL, "Tape sense dongle uitschakelen"},
	/* pl */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_TAPE_SENSE_DONGLE_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/dtl-basic-dongle.c */
	/* en */ {IDCLS_ENABLE_DTL_BASIC_DONGLE,    N_("Enable DTL Basic dongle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_DE, "DTL Basic Dongle aktivieren"},
	/* es */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_NL, "DTL Basic dongle inschakelen"},
	/* pl */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_DTL_BASIC_DONGLE_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/dtl-basic-dongle.c */
	/* en */ {IDCLS_DISABLE_DTL_BASIC_DONGLE,    N_("Enable DTL Basic dongle")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_DE, "DTL Basic Dongle aktivieren"},
	/* es */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_NL, "DTL Basic dongle inschakelen"},
	/* pl */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_DTL_BASIC_DONGLE_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapertc.c */
	/* en */ {IDCLS_ENABLE_TAPERTC,    N_("Enable CP Clock F83 (PCF8583 RTC)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TAPERTC_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TAPERTC_DE, "CP Clock F83 (PCF8583 RTC) aktivieren"},
	/* es */ {IDCLS_ENABLE_TAPERTC_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TAPERTC_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TAPERTC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TAPERTC_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TAPERTC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TAPERTC_NL, "CP Clock F83 (PCF8583 RTC) inschakelen"},
	/* pl */ {IDCLS_ENABLE_TAPERTC_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TAPERTC_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TAPERTC_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TAPERTC_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapertc.c */
	/* en */ {IDCLS_DISABLE_TAPERTC,    N_("Disable CP Clock F83 (PCF8583 RTC)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TAPERTC_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_TAPERTC_DE, "CP Clock F83 (PCF8583 RTC) deaktivieren"},
	/* es */ {IDCLS_DISABLE_TAPERTC_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_TAPERTC_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_TAPERTC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_TAPERTC_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_TAPERTC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TAPERTC_NL, "CP Clock F83 (PCF8583 RTC) uitschakelen"},
	/* pl */ {IDCLS_DISABLE_TAPERTC_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_TAPERTC_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_TAPERTC_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_TAPERTC_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapertc.c */
	/* en */ {IDCLS_ENABLE_TAPERTC_SAVE,    N_("Enable saving of the CP Clock F83 (PCF8583 RTC) data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TAPERTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TAPERTC_SAVE_DE, "CP Clock F83 (PCF8583 RTC) Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_TAPERTC_SAVE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TAPERTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TAPERTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TAPERTC_SAVE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TAPERTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TAPERTC_SAVE_NL, "Opslaan van de CP Clock F83 (PCF8583 RTC) gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_TAPERTC_SAVE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TAPERTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TAPERTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TAPERTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* tapeport/tapertc.c */
	/* en */ {IDCLS_DISABLE_TAPERTC_SAVE,    N_("Disable saving of the CP Clock F83 (PCF8583 RTC) data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TAPERTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_TAPERTC_SAVE_DE, "CP Clock F83 (PCF8583 RTC) Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_TAPERTC_SAVE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_TAPERTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_TAPERTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_TAPERTC_SAVE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_TAPERTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TAPERTC_SAVE_NL, "Opslaan van de CP Clock F83 (PCF8583 RTC) gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_TAPERTC_SAVE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_TAPERTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_TAPERTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_TAPERTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_4bit_sampler.c */
	/* en */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER,    N_("Enable Userport 4bit sampler")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_DE, "Userport 4bit Sampler aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_NL, "Userport 4bit sampler inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_USERPORT_4BIT_SAMPLER_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_4bit_sampler.c */
	/* en */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER,    N_("Disable Userport 4bit sampler")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_DE, "Userport 4bit Sampler deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_NL, "Userport 4bit sampler uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_USERPORT_4BIT_SAMPLER_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_8bss.c */
	/* en */ {IDCLS_ENABLE_USERPORT_8BSS,    N_("Enable Userport 8bit stereo sampler")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_8BSS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_8BSS_DE, "Userport 8bit Sampler aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_8BSS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_USERPORT_8BSS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_USERPORT_8BSS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_8BSS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_USERPORT_8BSS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_8BSS_NL, "Userport 8bit stereo sampler inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_8BSS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_USERPORT_8BSS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_8BSS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_USERPORT_8BSS_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_8bss.c */
	/* en */ {IDCLS_DISABLE_USERPORT_8BSS,    N_("Disable Userport 8bit stereo sampler")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_8BSS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_8BSS_DE, "Userport 8bit Sampler deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_8BSS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_USERPORT_8BSS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_USERPORT_8BSS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_8BSS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_USERPORT_8BSS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_8BSS_NL, "Userport 8bit stereo sampler uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_8BSS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_USERPORT_8BSS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_8BSS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_USERPORT_8BSS_TR, ""},  /* fuzzy */
#endif
	
	/* c64/psid.c */
	/* en */ {IDCLS_OVERWRITE_PSID_SETTINGS,    N_("Override PSID settings for Video standard and SID model")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_OVERWRITE_PSID_SETTINGS_DA, "Overstyr PSID-indstillinger for Video-standard og SID-model"},
	/* de */ {IDCLS_OVERWRITE_PSID_SETTINGS_DE, "PSID Einstellung f¸r Video Standard und SID Modell ver‰ndern"},
	/* es */ {IDCLS_OVERWRITE_PSID_SETTINGS_ES, "Sobrescribir ajustes para vÌdeo standard y modelo SID"},
	/* fr */ {IDCLS_OVERWRITE_PSID_SETTINGS_FR, "…craser les paramËtres PSID pour le standard vidÈo et le modËle SID"},
	/* hu */ {IDCLS_OVERWRITE_PSID_SETTINGS_HU, "A videÛ szabv·ny Ès a SID modell PSID be·llÌt·sainak fel¸lbÌr·l·sa"},
	/* it */ {IDCLS_OVERWRITE_PSID_SETTINGS_IT, "Non tener conto delle impostazioni del PSID per lo standard video e il modello di SID"},
	/* ko */ {IDCLS_OVERWRITE_PSID_SETTINGS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_OVERWRITE_PSID_SETTINGS_NL, "Overschrijf PSID instellingen voor videostandaard en SID model"},
	/* pl */ {IDCLS_OVERWRITE_PSID_SETTINGS_PL, "Zast±p ustawienia PSID standardu wideo i modelu SID"},
	/* ru */ {IDCLS_OVERWRITE_PSID_SETTINGS_RU, "Override PSID settings for Video standard and SID model"},
	/* sv */ {IDCLS_OVERWRITE_PSID_SETTINGS_SV, "Ers‰tt PSID-inst‰llningar fˆr videostandard och SID-modell"},
	/* tr */ {IDCLS_OVERWRITE_PSID_SETTINGS_TR, "Video standard˝ ve SID modeli iÁin PSID ayarlar˝n˝ geÁersiz k˝l"},
#endif
	
	/* c64/psid.c */
	/* en */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER,    N_("Specify PSID tune <number>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_DA, "VÊlg PSID-nummer <nummer>"},
	/* de */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_DE, "PSID St¸ck <Nummer> w‰hlen"},
	/* es */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_ES, "Especificar PSID tono <n˙mero>"},
	/* fr */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_FR, "SpÈcifier le <numÈro> de piste PSID"},
	/* hu */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_HU, "Adja meg a PSID hangzatok <sz·m>"},
	/* it */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_IT, "Specifica la melodia <numero> del PSID"},
	/* ko */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_NL, "Geef PSID melodie <nummer>"},
	/* pl */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_PL, "Okre∂l ton PSID <numer>"},
	/* ru */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_RU, "Specify PSID tune <number>"},
	/* sv */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_SV, "Ange PSID-lÂt <nummer>"},
	/* tr */ {IDCLS_SPECIFY_PSID_TUNE_NUMBER_TR, "PSID parÁa <numara>'s˝n˝ belirt"},
#endif
	
	/* c64/cart/ramcart.c */
	/* en */ {IDCLS_ENABLE_RAMCART,    N_("Enable the RamCart expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RAMCART_DA, "AktivÈr RamCart-udviddelsen"},
	/* de */ {IDCLS_ENABLE_RAMCART_DE, "RamCart Erweiterung aktivieren"},
	/* es */ {IDCLS_ENABLE_RAMCART_ES, "Permitir expansiÛn RAMCART"},
	/* fr */ {IDCLS_ENABLE_RAMCART_FR, "Activer l'expansion RAMCART"},
	/* hu */ {IDCLS_ENABLE_RAMCART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_RAMCART_IT, "Attiva l'espansione RamCart"},
	/* ko */ {IDCLS_ENABLE_RAMCART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RAMCART_NL, "RamCart uitbreiding inschakelen"},
	/* pl */ {IDCLS_ENABLE_RAMCART_PL, "W≥±cz rozszerzenie RamCart"},
	/* ru */ {IDCLS_ENABLE_RAMCART_RU, "Enable the RamCart expansion"},
	/* sv */ {IDCLS_ENABLE_RAMCART_SV, "Aktivera RamCart-expansion"},
	/* tr */ {IDCLS_ENABLE_RAMCART_TR, "RamCart geni˛lemesini aktif et"},
#endif
	
	/* c64/cart/ramcart.c */
	/* en */ {IDCLS_DISABLE_RAMCART,    N_("Disable the RamCart expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_RAMCART_DA, "DeaktivÈr RamCart-udviddelse"},
	/* de */ {IDCLS_DISABLE_RAMCART_DE, "RamCart Erweiterung deaktivieren"},
	/* es */ {IDCLS_DISABLE_RAMCART_ES, "Deshabilitar expansiÛn RamCart"},
	/* fr */ {IDCLS_DISABLE_RAMCART_FR, "DÈsactiver l'expansion RAMCART"},
	/* hu */ {IDCLS_DISABLE_RAMCART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_RAMCART_IT, "Disattiva l'espansione RamCart"},
	/* ko */ {IDCLS_DISABLE_RAMCART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_RAMCART_NL, "RamCart uitbreiding uitschakelen"},
	/* pl */ {IDCLS_DISABLE_RAMCART_PL, "Wy≥±cz rozszerzenie RamCart"},
	/* ru */ {IDCLS_DISABLE_RAMCART_RU, "Disable the RamCart expansion"},
	/* sv */ {IDCLS_DISABLE_RAMCART_SV, "Inaktivera RamCart-expansion"},
	/* tr */ {IDCLS_DISABLE_RAMCART_TR, "RamCart geni˛lemesini pasifle˛tir"},
#endif
	
	/* c64/cart/ramcart.c */
	/* en */ {IDCLS_SPECIFY_RAMCART_NAME,    N_("Specify name of RAMCART image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_RAMCART_NAME_DA, "Angiv navn pÂ RAMCART-image"},
	/* de */ {IDCLS_SPECIFY_RAMCART_NAME_DE, "Name f¸r RAMCART Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_RAMCART_NAME_ES, "Especificar nombre imagen RAMCART"},
	/* fr */ {IDCLS_SPECIFY_RAMCART_NAME_FR, "SpÈcifier le nom de l'image RAMCART"},
	/* hu */ {IDCLS_SPECIFY_RAMCART_NAME_HU, "Adja meg a RAMCART kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_RAMCART_NAME_IT, "Specifica il nome immagine RAMCART"},
	/* ko */ {IDCLS_SPECIFY_RAMCART_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_RAMCART_NAME_NL, "Geef de naam van het RAMCART bestand"},
	/* pl */ {IDCLS_SPECIFY_RAMCART_NAME_PL, "Okre∂l nazwÍ obrazu RAMCART"},
	/* ru */ {IDCLS_SPECIFY_RAMCART_NAME_RU, "Specify name of RAMCART image"},
	/* sv */ {IDCLS_SPECIFY_RAMCART_NAME_SV, "Ange namn pÂ RAMCART-avbildning"},
	/* tr */ {IDCLS_SPECIFY_RAMCART_NAME_TR, "RAMCART imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/cart/ramcart.c */
	/* en */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE,    N_("Allow writing to RAMCart image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_DA, "Tillad skrivning til RAMCart-image."},
	/* de */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_DE, "Schreibzugriff auf RAMCart Image erlauben"},
	/* es */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_ES, "Permitir grabar en imagen RAMCart"},
	/* fr */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_FR, "Permetre l'Ècriture ‡ l'image RAMCart"},
	/* hu */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_IT, "Attiva scrittura su immagine RAMCart"},
	/* ko */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_NL, "Sta schrijven naar RAMCart bestand toe"},
	/* pl */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_PL, "PozwÛl na zapis do obrazu RAMCart"},
	/* ru */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_RU, "Allow writing to RAMCart image"},
	/* sv */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_SV, "TillÂt skrivning till RAMCart-avbildning"},
	/* tr */ {IDCLS_ALLOW_WRITING_TO_RAMCART_IMAGE_TR, "RAMCart imaj˝na yazmay˝ aktif et"},
#endif
	
	/* c64/cart/ramcart.c */
	/* en */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE,    N_("Do not write to RAMCart image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_DA, "Skriv ikke til RAMcart-image"},
	/* de */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_DE, "Kein Schreibzugriff auf RAMCart Image"},
	/* es */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_ES, "No permitir grabar en imagen RAMCart"},
	/* fr */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_FR, "Ne pas Ècrire sur l'image RAMCart"},
	/* hu */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_IT, "Disattiva scrittura su immagine RAMCart"},
	/* ko */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_NL, "Schrijf niet naar RAMCart bestand"},
	/* pl */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_PL, "Nie zapisuj do obrazu RAMCart"},
	/* ru */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_RU, "Do not write to RAMCart image"},
	/* sv */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_SV, "Skriv inte till RAMCart-avbildning"},
	/* tr */ {IDCLS_DO_NOT_WRITE_TO_RAMCART_IMAGE_TR, "RAMCart imaj˝na yazma"},
#endif
	
	/* c64/cart/ramcart.c */
	/* en */ {IDCLS_RAMCART_READ_ONLY,    N_("Set the RAMCart switch to read-only")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RAMCART_READ_ONLY_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_RAMCART_READ_ONLY_DE, "RAMCart Schalter auf Schreibschutz"},
	/* es */ {IDCLS_RAMCART_READ_ONLY_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_RAMCART_READ_ONLY_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_RAMCART_READ_ONLY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RAMCART_READ_ONLY_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_RAMCART_READ_ONLY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RAMCART_READ_ONLY_NL, "Stel de RAMCart schakelaar in op alleen-lezen"},
	/* pl */ {IDCLS_RAMCART_READ_ONLY_PL, "Ustaw RAMCart na tryb tylko do odczytu"},
	/* ru */ {IDCLS_RAMCART_READ_ONLY_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_RAMCART_READ_ONLY_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_RAMCART_READ_ONLY_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ramcart.c */
	/* en */ {IDCLS_RAMCART_READ_WRITE,    N_("Set the RAMCart switch to read/write")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RAMCART_READ_WRITE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_RAMCART_READ_WRITE_DE, "RAMCart Schalter auf Schreibzugriff"},
	/* es */ {IDCLS_RAMCART_READ_WRITE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_RAMCART_READ_WRITE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_RAMCART_READ_WRITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RAMCART_READ_WRITE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_RAMCART_READ_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RAMCART_READ_WRITE_NL, "Stel de RAMCart schakelaar in op lezen/schrijven"},
	/* pl */ {IDCLS_RAMCART_READ_WRITE_PL, "Ustaw RAMCart na tryb zapisu i odczytu"},
	/* ru */ {IDCLS_RAMCART_READ_WRITE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_RAMCART_READ_WRITE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_RAMCART_READ_WRITE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ramcart.c */
	/* en */ {IDCLS_RAMCART_SIZE,    N_("Size of the RAMCART expansion. (64/128)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RAMCART_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_RAMCART_SIZE_DE, "Grˆﬂe der RAMCART Erweiterung. (64/128)"},
	/* es */ {IDCLS_RAMCART_SIZE_ES, "TamaÒo expansiÛn RAMCart. (64/128)"},
	/* fr */ {IDCLS_RAMCART_SIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_RAMCART_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RAMCART_SIZE_IT, "Dimensione dell'espansione RAMCART. (64/128)"},
	/* ko */ {IDCLS_RAMCART_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RAMCART_SIZE_NL, "Grootte van de RAMCART uitbreiding. (64/128)"},
	/* pl */ {IDCLS_RAMCART_SIZE_PL, "Rozmiar rozszerzenia RAMCART. (64/128)"},
	/* ru */ {IDCLS_RAMCART_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_RAMCART_SIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_RAMCART_SIZE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/debugcart.c */
	/* en */ {IDCLS_ENABLE_DEBUGCART,    N_("Enable Debug cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DEBUGCART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_DEBUGCART_DE, "Debug Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_DEBUGCART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_DEBUGCART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_DEBUGCART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DEBUGCART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_DEBUGCART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DEBUGCART_NL, "Debug cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_DEBUGCART_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_DEBUGCART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_DEBUGCART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_DEBUGCART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/debugcart.c */
	/* en */ {IDCLS_DISABLE_DEBUGCART,    N_("Disable Debug cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DEBUGCART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_DEBUGCART_DE, "Debug Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_DEBUGCART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_DEBUGCART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_DEBUGCART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DEBUGCART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_DEBUGCART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DEBUGCART_NL, "Debug cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DEBUGCART_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_DEBUGCART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_DEBUGCART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_DEBUGCART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/dqbb.c */
	/* en */ {IDCLS_ENABLE_DQBB,    N_("Enable Double Quick Brown Box")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DQBB_DA, "AktivÈr Double Quick Brown Box"},
	/* de */ {IDCLS_ENABLE_DQBB_DE, "Double Quick Brown Box aktivieren"},
	/* es */ {IDCLS_ENABLE_DQBB_ES, "Permitir Double Quick Brown Box"},
	/* fr */ {IDCLS_ENABLE_DQBB_FR, "Activer Double Quick Brown Box"},
	/* hu */ {IDCLS_ENABLE_DQBB_HU, "Double Quick Brown Box engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_DQBB_IT, "Attiva Double Quick Brown Box"},
	/* ko */ {IDCLS_ENABLE_DQBB_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DQBB_NL, "Double Quick Brown Box cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_DQBB_PL, "W≥±cz Double Quick Brown Box"},
	/* ru */ {IDCLS_ENABLE_DQBB_RU, "Enable Double Quick Brown Box"},
	/* sv */ {IDCLS_ENABLE_DQBB_SV, "Aktivera Double Quick Brown Box"},
	/* tr */ {IDCLS_ENABLE_DQBB_TR, "Double Quick Brown Box'˝ aktif et"},
#endif
	
	/* c64/cart/dqbb.c */
	/* en */ {IDCLS_DISABLE_DQBB,    N_("Disable Double Quick Brown Box")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DQBB_DA, "DeaktivÈr Double Quick Brown Box"},
	/* de */ {IDCLS_DISABLE_DQBB_DE, "Double Quick Brown Box Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_DQBB_ES, "Deshabilitar Double Quick Brown Box"},
	/* fr */ {IDCLS_DISABLE_DQBB_FR, "DÈsactiver Double Quick Brown Box"},
	/* hu */ {IDCLS_DISABLE_DQBB_HU, "Double Quick Brown Box tilt·sa"},
	/* it */ {IDCLS_DISABLE_DQBB_IT, "Disattiva Double Quick Brown Box"},
	/* ko */ {IDCLS_DISABLE_DQBB_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DQBB_NL, "Double Quick Brown Box cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DQBB_PL, "Wy≥±cz Double Quick Brown Box"},
	/* ru */ {IDCLS_DISABLE_DQBB_RU, "Disable Double Quick Brown Box"},
	/* sv */ {IDCLS_DISABLE_DQBB_SV, "Inaktivera Double Quick Brown Box"},
	/* tr */ {IDCLS_DISABLE_DQBB_TR, "Double Quick Brown Box'˝ pasifle˛tir"},
#endif
	
	/* c64/cart/dqbb.c */
	/* en */ {IDCLS_SPECIFY_DQBB_NAME,    N_("Specify Double Quick Brown Box filename")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_DQBB_NAME_DA, "Angiv filnavn for Double Quick Brown Box"},
	/* de */ {IDCLS_SPECIFY_DQBB_NAME_DE, "Double Quick Brown Box Dateiname spezifizieren"},
	/* es */ {IDCLS_SPECIFY_DQBB_NAME_ES, "Especificar fichero imagen Double Quick Brown Box"},
	/* fr */ {IDCLS_SPECIFY_DQBB_NAME_FR, "SpÈcifier nom de fichier Double Quick Brown Box"},
	/* hu */ {IDCLS_SPECIFY_DQBB_NAME_HU, "Adja meg a Double Quick Brown Box f·jlnevet"},
	/* it */ {IDCLS_SPECIFY_DQBB_NAME_IT, "Specifica il nome del file Double Quick Brown Box"},
	/* ko */ {IDCLS_SPECIFY_DQBB_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_DQBB_NAME_NL, "Geef de naam van het Double Quick Brown Box cartridge bestand"},
	/* pl */ {IDCLS_SPECIFY_DQBB_NAME_PL, "Okre∂l nazwÍ pliku Double Quick Brown Box"},
	/* ru */ {IDCLS_SPECIFY_DQBB_NAME_RU, "Specify Double Quick Brown Box filename"},
	/* sv */ {IDCLS_SPECIFY_DQBB_NAME_SV, "Ange filnamn fˆr Double Quick Brown Box"},
	/* tr */ {IDCLS_SPECIFY_DQBB_NAME_TR, "Double Quick Brown Box dosya ismi belirt"},
#endif
	
	/* c64/cart/dqbb.c */
	/* en */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE,    N_("Allow writing to DQBB image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_DA, "Tillad skrivning til DQBB-image."},
	/* de */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_DE, "Schreibzugriff auf DQBB Image erlauben"},
	/* es */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_ES, "Permitir grabaciÛn en imagen DQBB "},
	/* fr */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_FR, "Permettre l'Ècriture ‡ l'image DQBB"},
	/* hu */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_IT, "Attiva scrittura su immagine DQBB"},
	/* ko */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_NL, "Sta schrijven naar DQBB bestand toe"},
	/* pl */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_PL, "PozwÛl na zapis do obrazu DQBB"},
	/* ru */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_RU, "Allow writing to DQBB image"},
	/* sv */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_SV, "TillÂt skrivning till DQBB-avbildning"},
	/* tr */ {IDCLS_ALLOW_WRITING_TO_DQBB_IMAGE_TR, "DQBB imaj˝na yazmay˝ aktif et"},
#endif
	
	/* c64/cart/dqbb.c */
	/* en */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE,    N_("Do not write to DQBB image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_DA, "Skriv ikke til DQBB-image"},
	/* de */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_DE, "Kein Schreibzugriff auf DQBB Image"},
	/* es */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_ES, "No permitir grabaciÛn en imagen DQBB "},
	/* fr */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_FR, "Ne pas Ècrire sur l'image  DQBB"},
	/* hu */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_IT, "Disattiva scrittura su immagine DQBB"},
	/* ko */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_NL, "Schrijf niet naar DQBB bestand"},
	/* pl */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_PL, "Nie zapisuj do obrazu DQBB"},
	/* ru */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_RU, "Do not write to DQBB image"},
	/* sv */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_SV, "Skriv inte till DQBB-avbildning"},
	/* tr */ {IDCLS_DO_NOT_WRITE_TO_DQBB_IMAGE_TR, "DQBB imaj˝na yazma"},
#endif
	
	/* c64/cart/isepic.c */
	/* en */ {IDCLS_ENABLE_ISEPIC,    N_("Enable the ISEPIC cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_ISEPIC_DA, "AktivÈr ISEPIC-cartridge"},
	/* de */ {IDCLS_ENABLE_ISEPIC_DE, "ISEPIC Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_ISEPIC_ES, "Permitir cartucho ISEPIC"},
	/* fr */ {IDCLS_ENABLE_ISEPIC_FR, "Activer la cartouche ISEPIC"},
	/* hu */ {IDCLS_ENABLE_ISEPIC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_ISEPIC_IT, "Attiva la cartuccia ISEPIC"},
	/* ko */ {IDCLS_ENABLE_ISEPIC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_ISEPIC_NL, "ISEPIC cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_ISEPIC_PL, "W≥±cz kartridø ISEPIC"},
	/* ru */ {IDCLS_ENABLE_ISEPIC_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_ISEPIC_SV, "Aktivera ISEPIC-modul"},
	/* tr */ {IDCLS_ENABLE_ISEPIC_TR, "ISEPIC kartu˛unu aktif et"},
#endif
	
	/* c64/cart/isepic.c */
	/* en */ {IDCLS_DISABLE_ISEPIC,    N_("Disable the ISEPIC cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_ISEPIC_DA, "DeaktivÈr ISEPIC-cartridget"},
	/* de */ {IDCLS_DISABLE_ISEPIC_DE, "ISEPIC Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_ISEPIC_ES, "Deshabilitar cartucho ISEPIC"},
	/* fr */ {IDCLS_DISABLE_ISEPIC_FR, "DÈsactiver la cartouche ISEPIC"},
	/* hu */ {IDCLS_DISABLE_ISEPIC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_ISEPIC_IT, "Disattiva la cartuccia ISEPIC"},
	/* ko */ {IDCLS_DISABLE_ISEPIC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_ISEPIC_NL, "ISEPIC cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_ISEPIC_PL, "Wy≥±cz kartridø ISEPIC"},
	/* ru */ {IDCLS_DISABLE_ISEPIC_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_ISEPIC_SV, "Inaktivera ISEPIC-modul"},
	/* tr */ {IDCLS_DISABLE_ISEPIC_TR, "ISEPIC kartu˛unu pasifle˛tir"},
#endif
	
	/* c64/cart/isepic.c */
	/* en */ {IDCLS_ENABLE_ISEPIC_SWITCH,    N_("Enable the ISEPIC switch")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_ISEPIC_SWITCH_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_ISEPIC_SWITCH_DE, "ISEPIC Schalter aktivieren"},
	/* es */ {IDCLS_ENABLE_ISEPIC_SWITCH_ES, "Permitir conmutaciÛn ISEPIC"},
	/* fr */ {IDCLS_ENABLE_ISEPIC_SWITCH_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_ISEPIC_SWITCH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_ISEPIC_SWITCH_IT, "Attiva l'interrutore  ISEPIC"},
	/* ko */ {IDCLS_ENABLE_ISEPIC_SWITCH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_ISEPIC_SWITCH_NL, "Activeer de ISEPIC schakelaar"},
	/* pl */ {IDCLS_ENABLE_ISEPIC_SWITCH_PL, "Ustaw prze≥±cznik ISEPIC"},
	/* ru */ {IDCLS_ENABLE_ISEPIC_SWITCH_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_ISEPIC_SWITCH_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_ISEPIC_SWITCH_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/isepic.c */
	/* en */ {IDCLS_DISABLE_ISEPIC_SWITCH,    N_("Disable the ISEPIC switch")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_ISEPIC_SWITCH_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_ISEPIC_SWITCH_DE, "ISEPIC Schalter deaktivieren"},
	/* es */ {IDCLS_DISABLE_ISEPIC_SWITCH_ES, "Deshabilitar conmutaciÛn ISEPIC"},
	/* fr */ {IDCLS_DISABLE_ISEPIC_SWITCH_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_ISEPIC_SWITCH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_ISEPIC_SWITCH_IT, "Disattiva l'interrutore  ISEPIC"},
	/* ko */ {IDCLS_DISABLE_ISEPIC_SWITCH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_ISEPIC_SWITCH_NL, "Deactiveer de ISEPIC schakelaar"},
	/* pl */ {IDCLS_DISABLE_ISEPIC_SWITCH_PL, "Wy≥±cz prze≥±cznik ISEPIC"},
	/* ru */ {IDCLS_DISABLE_ISEPIC_SWITCH_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_ISEPIC_SWITCH_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_ISEPIC_SWITCH_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/isepic.c */
	/* en */ {IDCLS_SET_ISEPIC_FILENAME,    N_("Set ISEPIC image name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_ISEPIC_FILENAME_DA, "VÊlg ISEPIC image-navn"},
	/* de */ {IDCLS_SET_ISEPIC_FILENAME_DE, "ISEPIC Imagedateiname setzen"},
	/* es */ {IDCLS_SET_ISEPIC_FILENAME_ES, "Seleccionar nombre imagen ISEPIC"},
	/* fr */ {IDCLS_SET_ISEPIC_FILENAME_FR, "Nom de l'image ISEPIC"},
	/* hu */ {IDCLS_SET_ISEPIC_FILENAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_ISEPIC_FILENAME_IT, "Imposta nome immagine ISEPIC"},
	/* ko */ {IDCLS_SET_ISEPIC_FILENAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_ISEPIC_FILENAME_NL, "Geef de naam op van het ISEPIC cartridge bestand"},
	/* pl */ {IDCLS_SET_ISEPIC_FILENAME_PL, "Okre∂l nazwÍ obrazu ISEPIC"},
	/* ru */ {IDCLS_SET_ISEPIC_FILENAME_RU, "Set ISEPIC image name"},
	/* sv */ {IDCLS_SET_ISEPIC_FILENAME_SV, "Namn pÂ ISEPIC-avbildning"},
	/* tr */ {IDCLS_SET_ISEPIC_FILENAME_TR, "ISEPIC imaj ismini belirle"},
#endif
	
	/* c64/cart/isepic.c */
	/* en */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE,    N_("Allow writing to ISEPIC image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_DA, "Tillad skrivning til ISEPIC-image"},
	/* de */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_DE, "Schreibzugriff auf ISEPIC Image erlauben"},
	/* es */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_ES, "Permitir grabaciÛn en imagen ISEPIC"},
	/* fr */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_FR, "Permettre l'Ècriture sur l'image ISEPIC"},
	/* hu */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_IT, "Attiva scrittura su immagine ISEPIC"},
	/* ko */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_NL, "Sta schrijven naar ISEPIC bestand toe"},
	/* pl */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_PL, "PozwÛl na zapis do obrazu ISEPIC"},
	/* ru */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_RU, "Allow writing to ISEPIC image"},
	/* sv */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_SV, "TillÂt skrivning till ISEPIC-avbildning"},
	/* tr */ {IDCLS_ALLOW_WRITING_TO_ISEPIC_IMAGE_TR, "ISEPIC imaj˝na yazmay˝ aktif et"},
#endif
	
	/* c64/cart/isepic.c */
	/* en */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE,    N_("Do not write to ISEPIC image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_DA, "Skriv ikke til ISEPIC-image"},
	/* de */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_DE, "Kein Schreibzugriff auf ISEPIC Image"},
	/* es */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_ES, "No permitir grabaciÛn en imagen ISEPIC"},
	/* fr */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_FR, "Ne pas Ècrire sur l'image ISEPIC"},
	/* hu */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_IT, "Disattiva scrittura su immagine ISEPIC"},
	/* ko */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_NL, "Schrijf niet naar ISEPIC bestand"},
	/* pl */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_PL, "Nie zapisuj do obrazu ISEPIC"},
	/* ru */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_RU, "Do not write to ISEPIC image"},
	/* sv */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_SV, "Skriv inte till ISEPIC-avbildning"},
	/* tr */ {IDCLS_DO_NOT_WRITE_TO_ISEPIC_IMAGE_TR, "ISEPIC imaj˝na yazma"},
#endif
	
	/* c64/cart/magicvoice.c */
	/* en */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME,    N_("Specify name of Magic Voice ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_DE, "Name f¸r Magic Voice ROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_ES, "Especificar nombre imagen Magic Voice ROM"},
	/* fr */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_IT, "Specifica il nome immagine della ROM Magic Voice"},
	/* ko */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_NL, "Geef de naam van het Magic Voice ROM bestand op"},
	/* pl */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_PL, "Okre∂l nazwÍ obrazu ROM Magic Voice"},
	/* ru */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_MAGICVOICE_IMAGE_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_ENABLE_MAGICVOICE,    N_("Enable the Magic Voice cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_MAGICVOICE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_MAGICVOICE_DE, "DigiMAX Cartridge aktivieren"},
	/* es */ {IDCLS_ENABLE_MAGICVOICE_ES, "Permitir cartucho Magic Voice"},
	/* fr */ {IDCLS_ENABLE_MAGICVOICE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_MAGICVOICE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_MAGICVOICE_IT, "Attiva la cartuccia Magic Voice"},
	/* ko */ {IDCLS_ENABLE_MAGICVOICE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_MAGICVOICE_NL, "Magic Voice cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_MAGICVOICE_PL, "W≥±cz kartridø Magic Voice"},
	/* ru */ {IDCLS_ENABLE_MAGICVOICE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_MAGICVOICE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_MAGICVOICE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_DISABLE_MAGICVOICE,    N_("Disable the Magic Voice cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_MAGICVOICE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_MAGICVOICE_DE, "DigiMAX Cartridge deaktivieren"},
	/* es */ {IDCLS_DISABLE_MAGICVOICE_ES, "Deshabilitar cartucho Magic Voice"},
	/* fr */ {IDCLS_DISABLE_MAGICVOICE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_MAGICVOICE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_MAGICVOICE_IT, "Disattiva la  cartuccia Magic Voice"},
	/* ko */ {IDCLS_DISABLE_MAGICVOICE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_MAGICVOICE_NL, "Magic Voice cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_MAGICVOICE_PL, "Wy≥±cz kartridø Magic Voice"},
	/* ru */ {IDCLS_DISABLE_MAGICVOICE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_MAGICVOICE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_MAGICVOICE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_ENABLE_MMC64,    N_("Enable the MMC64 expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_MMC64_DA, "AktivÈr MMC64-udviddelse"},
	/* de */ {IDCLS_ENABLE_MMC64_DE, "MMC64 Erweiterung aktivieren"},
	/* es */ {IDCLS_ENABLE_MMC64_ES, "Permitir expansiÛn MMC64"},
	/* fr */ {IDCLS_ENABLE_MMC64_FR, "Activer l'expansion MMC64"},
	/* hu */ {IDCLS_ENABLE_MMC64_HU, "MMC64 bıvÌtÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_MMC64_IT, "Attiva l'espansione MMC64"},
	/* ko */ {IDCLS_ENABLE_MMC64_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_MMC64_NL, "MMC64 uitbreiding inschakelen"},
	/* pl */ {IDCLS_ENABLE_MMC64_PL, "W≥±cz rozszerzenie MMC64"},
	/* ru */ {IDCLS_ENABLE_MMC64_RU, "Enable the MMC64 expansion"},
	/* sv */ {IDCLS_ENABLE_MMC64_SV, "Aktivera MMC64-expansion"},
	/* tr */ {IDCLS_ENABLE_MMC64_TR, "MMC64 geni˛lemesini aktif et"},
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_DISABLE_MMC64,    N_("Disable the MMC64 expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_MMC64_DA, "DeaktivÈr MMC64-udviddelse"},
	/* de */ {IDCLS_DISABLE_MMC64_DE, "MMC64 Erweiterung deaktivieren"},
	/* es */ {IDCLS_DISABLE_MMC64_ES, "Deshabilitar expansiÛn MMC64"},
	/* fr */ {IDCLS_DISABLE_MMC64_FR, "DÈsactiver l'expansion MMC64"},
	/* hu */ {IDCLS_DISABLE_MMC64_HU, "MMC64 bıvÌtÈs tilt·sa"},
	/* it */ {IDCLS_DISABLE_MMC64_IT, "Disattiva l'espansione MMC64"},
	/* ko */ {IDCLS_DISABLE_MMC64_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_MMC64_NL, "MMC64 uitbreiding uitschakelen"},
	/* pl */ {IDCLS_DISABLE_MMC64_PL, "Wy≥±cz rozszerzenie MMC64"},
	/* ru */ {IDCLS_DISABLE_MMC64_RU, "Disable the MMC64 expansion"},
	/* sv */ {IDCLS_DISABLE_MMC64_SV, "Inaktivera MMC64-expansion"},
	/* tr */ {IDCLS_DISABLE_MMC64_TR, "MMC64 geni˛lemesini pasifle˛tir"},
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_SPECIFY_MMC64_BIOS_NAME,    N_("Specify name of MMC64 BIOS image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_DA, "Angiv navn pÂ MMC64-BIOS-image"},
	/* de */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_DE, "Name f¸r MMC64 BIOS Datei spezifizieren"},
	/* es */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_ES, "Especificar nombre imagen BIOS MMC64"},
	/* fr */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_FR, "SpÈcifier le nom de l'image MMC64 BIOS"},
	/* hu */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_HU, "Adja meg az MMC64 BIOS kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_IT, "Specifica il nome immagine del BIOS MMC64"},
	/* ko */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_NL, "Geef de naam van het MMC64 BIOS bestand"},
	/* pl */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_PL, "Okre∂l nazwÍ obrazu BIOS MMC64"},
	/* ru */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_RU, "Specify name of MMC64 BIOS image"},
	/* sv */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_SV, "Ange namn pÂ MMC64-BIOS-avbildning"},
	/* tr */ {IDCLS_SPECIFY_MMC64_BIOS_NAME_TR, "MMC64 BIOS imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME,    N_("Specify name of MMC64 image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_DA, "Angiv navn pÂ MMC64-image"},
	/* de */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_DE, "Name der MMC64 Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_ES, "Especificar nombre imagen MMC64"},
	/* fr */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_FR, "SpÈcifier le nom de l'image MMC64"},
	/* hu */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_HU, "Adja meg az MMC64 kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_IT, "Specifica il nome immagine MMC64"},
	/* ko */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_NL, "Geef de naam van het MMC64 bestand"},
	/* pl */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_PL, "Okre∂l nazwÍ obrazu MMC64"},
	/* ru */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_RU, "Specify name of MMC64 image"},
	/* sv */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_SV, "Ange namn pÂ MMC64-avbildning"},
	/* tr */ {IDCLS_SPECIFY_MMC64_IMAGE_NAME_TR, "MMC64 imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_SPECIFY_MMC64_REVISION,    N_("Specify MMC64 revision (0: Rev A, 1: Rev B)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_MMC64_REVISION_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_MMC64_REVISION_DE, "Setze MMC64 Revision (0: Rev A, 1: Rev B)"},
	/* es */ {IDCLS_SPECIFY_MMC64_REVISION_ES, "Especificar revisiÛn MMC64 (0: Rev A, 1: Rev B)"},
	/* fr */ {IDCLS_SPECIFY_MMC64_REVISION_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_MMC64_REVISION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_MMC64_REVISION_IT, "Specifica la revisione MMC64 (0: Rev A, 1: Rev B)"},
	/* ko */ {IDCLS_SPECIFY_MMC64_REVISION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_MMC64_REVISION_NL, "Geef de MMC64 revisie (0: Rev A, 1: Rev B)"},
	/* pl */ {IDCLS_SPECIFY_MMC64_REVISION_PL, "Okre∂l wersjÍ MMC64 (0: Rev A, 1: Rev B)"},
	/* ru */ {IDCLS_SPECIFY_MMC64_REVISION_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_MMC64_REVISION_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_MMC64_REVISION_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_SPECIFY_MMC64_SD_TYPE,    N_("Specify MMC64 SD type (0: auto, 1: MMC, 2: SD, 3: SDHC)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_MMC64_SD_TYPE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_MMC64_SD_TYPE_DE, "Setze MMC64 SD Typ (0: Auto, 1: MMC, 2: SD, 3: SDHC)"},
	/* es */ {IDCLS_SPECIFY_MMC64_SD_TYPE_ES, "Especificar tipo MMC64 SD (0: auto, 1: MMC, 2: SD, 3: SDHC)"},
	/* fr */ {IDCLS_SPECIFY_MMC64_SD_TYPE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_MMC64_SD_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_MMC64_SD_TYPE_IT, "Specifica il tipo di SD dell'MMC64 (0: auto, 1: MMC, 2: SD, 3: SDHC)"},
	/* ko */ {IDCLS_SPECIFY_MMC64_SD_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_MMC64_SD_TYPE_NL, "Geef het MMC64 SD soort op (0: automatisch, 1: MMC, 2: SD, 3: SDHC)"},
	/* pl */ {IDCLS_SPECIFY_MMC64_SD_TYPE_PL, "Okre∂l typ SD MMC64 (0: Auto, 1: MMC, 2: SD, 3: SDHC)"},
	/* ru */ {IDCLS_SPECIFY_MMC64_SD_TYPE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_MMC64_SD_TYPE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_MMC64_SD_TYPE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_MMC64_READONLY,    N_("Set the MMC64 card to read-only")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC64_READONLY_DA, "Skrivebeskyt MMC64-kort"},
	/* de */ {IDCLS_MMC64_READONLY_DE, "MMC64 Karte Schreibschutz"},
	/* es */ {IDCLS_MMC64_READONLY_ES, "Seleccionar tarjeta MMC64 en modo sÛlo lectura"},
	/* fr */ {IDCLS_MMC64_READONLY_FR, "DÈfinir la carte MMC64 en lecture seule"},
	/* hu */ {IDCLS_MMC64_READONLY_HU, "MMC64 k·rtya csak olvashatÛ"},
	/* it */ {IDCLS_MMC64_READONLY_IT, "Imposta la card MMC64 in sola lettura"},
	/* ko */ {IDCLS_MMC64_READONLY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC64_READONLY_NL, "Stel de MMC64 in als alleen-lezen"},
	/* pl */ {IDCLS_MMC64_READONLY_PL, "Ustaw kartÍ MMC64 na tryb tylko do odczytu"},
	/* ru */ {IDCLS_MMC64_READONLY_RU, "Set the MMC64 card to read-only"},
	/* sv */ {IDCLS_MMC64_READONLY_SV, "Skrivskydda MMC64-kort"},
	/* tr */ {IDCLS_MMC64_READONLY_TR, "MMC64 kart˝n˝ salt okunura Áevir"},
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_MMC64_READWRITE,    N_("Set the MMC64 card to read/write")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC64_READWRITE_DA, "Tillad skrivning pÂ MMC64-kort"},
	/* de */ {IDCLS_MMC64_READWRITE_DE, "MMC64 Karte Schreibzugriff"},
	/* es */ {IDCLS_MMC64_READWRITE_ES, "Seleccionar tarjeta MMC64 en modo lectura/grabaciÛn"},
	/* fr */ {IDCLS_MMC64_READWRITE_FR, "DÈfinir la carte MMC64 en lecture/Ècriture"},
	/* hu */ {IDCLS_MMC64_READWRITE_HU, "MMC64 k·rtya ÌrhatÛ/olvashatÛ"},
	/* it */ {IDCLS_MMC64_READWRITE_IT, "Imposta la card MMC64 in lettura/scrittura"},
	/* ko */ {IDCLS_MMC64_READWRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC64_READWRITE_NL, "Stel de MMC64 in als lezen/schrijven"},
	/* pl */ {IDCLS_MMC64_READWRITE_PL, "Ustaw kartÍ MMC64 na tryb zapisu i odczytu"},
	/* ru */ {IDCLS_MMC64_READWRITE_RU, "Set the MMC64 card to read/write"},
	/* sv */ {IDCLS_MMC64_READWRITE_SV, "TillÂt skrivning pÂ MMC64-kort"},
	/* tr */ {IDCLS_MMC64_READWRITE_TR, "MMC64 kart˝n˝ okunur/yaz˝l˝ra Áevir"},
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_MMC64_BIOS_WRITE,    N_("Save the MMC64 bios when changed")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC64_BIOS_WRITE_DA, "Gem MMC64-BIOS ved Êndring"},
	/* de */ {IDCLS_MMC64_BIOS_WRITE_DE, "MMC64 BIOS speichern bei ƒnderung"},
	/* es */ {IDCLS_MMC64_BIOS_WRITE_ES, "Grabar BIOS MMC64 cuando cambie"},
	/* fr */ {IDCLS_MMC64_BIOS_WRITE_FR, "Sauvegarder le BIOS MMC64 lorsque modifiÈ"},
	/* hu */ {IDCLS_MMC64_BIOS_WRITE_HU, "MMC64 BIOS mentÈse v·ltoz·s ut·n"},
	/* it */ {IDCLS_MMC64_BIOS_WRITE_IT, "Salva il bios MMC64 alla modifica"},
	/* ko */ {IDCLS_MMC64_BIOS_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC64_BIOS_WRITE_NL, "Sla de MMC64 BIOS op bij wijziging"},
	/* pl */ {IDCLS_MMC64_BIOS_WRITE_PL, "Zapisuj bios MMC64 przy zmianie"},
	/* ru */ {IDCLS_MMC64_BIOS_WRITE_RU, "Save the MMC64 bios when changed"},
	/* sv */ {IDCLS_MMC64_BIOS_WRITE_SV, "Spara MMC64-BIOS vid ‰ndring"},
	/* tr */ {IDCLS_MMC64_BIOS_WRITE_TR, "MMC64 BIOS'unu dei˛tiinde kaydet"},
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_MMC64_BIOS_READ_ONLY,    N_("Do not save the MMC64 bios when changed")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC64_BIOS_READ_ONLY_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_MMC64_BIOS_READ_ONLY_DE, "MMC64 BIOS nicht speichern bei ƒnderung"},
	/* es */ {IDCLS_MMC64_BIOS_READ_ONLY_ES, "No grabar BIOS MMC64 cuando cambie"},
	/* fr */ {IDCLS_MMC64_BIOS_READ_ONLY_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_MMC64_BIOS_READ_ONLY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC64_BIOS_READ_ONLY_IT, "Non salvare il bios MMC64 alla modifica"},
	/* ko */ {IDCLS_MMC64_BIOS_READ_ONLY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC64_BIOS_READ_ONLY_NL, "Sla de MMC64 BIOS niet op bij wijziging"},
	/* pl */ {IDCLS_MMC64_BIOS_READ_ONLY_PL, "Nie zapisuj bios-a MMC64 przy zmianie"},
	/* ru */ {IDCLS_MMC64_BIOS_READ_ONLY_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_MMC64_BIOS_READ_ONLY_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_MMC64_BIOS_READ_ONLY_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_MMC64_SET_FLASH_JUMPER,    N_("Set the MMC64 Flash Jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC64_SET_FLASH_JUMPER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_MMC64_SET_FLASH_JUMPER_DE, "Aktiviere MMC64 Flash Jumper"},
	/* es */ {IDCLS_MMC64_SET_FLASH_JUMPER_ES, "Seleccionar el  MMC64 Flash Jumper"},
	/* fr */ {IDCLS_MMC64_SET_FLASH_JUMPER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_MMC64_SET_FLASH_JUMPER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC64_SET_FLASH_JUMPER_IT, "Imposta il ponticello flash della MMC64"},
	/* ko */ {IDCLS_MMC64_SET_FLASH_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC64_SET_FLASH_JUMPER_NL, "Stel de MMC64 Flash Jumper in"},
	/* pl */ {IDCLS_MMC64_SET_FLASH_JUMPER_PL, "Ustaw zworkÍ MMC64 Flash"},
	/* ru */ {IDCLS_MMC64_SET_FLASH_JUMPER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_MMC64_SET_FLASH_JUMPER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_MMC64_SET_FLASH_JUMPER_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmc64.c */
	/* en */ {IDCLS_MMC64_UNSET_FLASH_JUMPER,    N_("Remove the MMC64 Flash Jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_DE, "Deaktiviere MMC64 Flash Jumper"},
	/* es */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_ES, "Quitar el  MMC64 Flash Jumper"},
	/* fr */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_IT, "Rimuovi il ponticello flash della MMC64"},
	/* ko */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_NL, "Verwijder de MMC64 Flash Jumper"},
	/* pl */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_PL, "UsuÒ zworkÍ RR Flash"},
	/* ru */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_MMC64_UNSET_FLASH_JUMPER_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/rrnetmk3.c */
	/* en */ {IDCLS_RRNETMK3_BIOS_WRITE,    N_("Save the RRNETMK3 bios when changed")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RRNETMK3_BIOS_WRITE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_RRNETMK3_BIOS_WRITE_DE, "RRNETMK3 BIOS speichern bei ƒnderung"},
	/* es */ {IDCLS_RRNETMK3_BIOS_WRITE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_RRNETMK3_BIOS_WRITE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_RRNETMK3_BIOS_WRITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RRNETMK3_BIOS_WRITE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_RRNETMK3_BIOS_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RRNETMK3_BIOS_WRITE_NL, "Sla de RRNET MK3 bios op bij wijziging"},
	/* pl */ {IDCLS_RRNETMK3_BIOS_WRITE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_RRNETMK3_BIOS_WRITE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_RRNETMK3_BIOS_WRITE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_RRNETMK3_BIOS_WRITE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/rrnetmk3.c */
	/* en */ {IDCLS_RRNETMK3_BIOS_READ_ONLY,    N_("Do not save the RRNETMK3 bios when changed")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_DE, "RRNETMK3 BIOS nicht speichern bei ƒnderung"},
	/* es */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_NL, "Sla de RRNET MK3 bios niet op bij wijziging"},
	/* pl */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_RRNETMK3_BIOS_READ_ONLY_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/rrnetmk3.c */
	/* en */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER,    N_("Set the RRNETMK3 Flash Jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_DE, "Aktiviere RRNETMK3 Flash Jumper"},
	/* es */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_NL, "Stel de RRNET MK3 Flash Jumper in"},
	/* pl */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_RRNETMK3_SET_FLASH_JUMPER_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/rrnetmk3.c */
	/* en */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER,    N_("Remove the RRNETMK3 Flash Jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_DE, "Deaktiviere RRNETMK3 Flash Jumper"},
	/* es */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_NL, "Verwijder de RRNET MK3 Flash Jumper"},
	/* pl */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_RRNETMK3_UNSET_FLASH_JUMPER_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME,    N_("Specify MMC Replay card image filename")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_DA, "Angiv filnavn for MMC Replay kort-image"},
	/* de */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_DE, "MMC Replay Modul imeage datei w‰hlen"},
	/* es */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_ES, "Especificar fichero imagen tarjeta MMC Replay"},
	/* fr */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_FR, "SpÈcifier le nom du fichier de l'image carte MMC Replay"},
	/* hu */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_IT, "Specifica il nome file immagine della card MMC Replay"},
	/* ko */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_NL, "Geef de naam van het MMC Replay card bestand"},
	/* pl */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_PL, "Okre∂l nazwÍ pliku obrazu karty MMC Replay"},
	/* ru */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_RU, "Specify MMC Replay card image filename"},
	/* sv */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_SV, "Ange namn pÂ avbildningsfil fˆr MMC Replay-kort"},
	/* tr */ {IDCLS_SELECT_MMC_REPLAY_CARD_IMAGE_FILENAME_TR, "MMC Replay kart imaj dosyaismini belirt"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE,    N_("Specify MMC Replay SD type (0: auto, 1: MMC, 2: SD, 3: SDHC)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_DE, "Setze MMC Replay SD type (0: Auto, 1: MMC, 2: SD, 3: SDHC)"},
	/* es */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_ES, "Especificar tipo MMC Replay SD (0: auto, 1: MMC, 2: SD, 3: SDHC)"},
	/* fr */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_IT, "Specifica il tipo di SD dell'MMC Replay (0: auto, 1: MMC, 2: SD, 3: SDHC)"},
	/* ko */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_NL, "Geef de MMC Replay SD soort op (0: automatisch, 1: MMC, 2: SD, 3: SDHC)"},
	/* pl */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_PL, "Okre∂l typ SD MMC Replay (0: Auto, 1: MMC, 2: SD, 3: SDHC)"},
	/* ru */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SELECT_MMC_REPLAY_SD_TYPE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE,    N_("Enable writes to MMC Replay card image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_DA, "Tillad skrivning til MMC Replay kort-image"},
	/* de */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_DE, "Erlaube Schreibzugriff auf MMC Replay Imagedatei"},
	/* es */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_ES, "Permitir grabaciÛn en imagen tarjeta MMC Replay"},
	/* fr */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_FR, "Permetre l'Ècriture ‡ l'image MMC Replay"},
	/* hu */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_IT, "Attiva scrittura su immagine card MMC Replay"},
	/* ko */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_NL, "Schrijven naar MMC Replay card bestand inschakelen"},
	/* pl */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_PL, "W≥±cz zapis do obrazu karty MMC Replay"},
	/* ru */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_RU, "Enable writes to MMC Replay card image"},
	/* sv */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_SV, "Aktivera skrivning till MMC Replay-kortavbildning"},
	/* tr */ {IDCLS_MMC_REPLAY_CARD_WRITE_ENABLE_TR, "MMC Replay kart imaj˝nda yazmalar˝ aktif et"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE,    N_("Disable writes to MMC Replay card image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_DA, "Skriv ikke til MMC Replay kort-image"},
	/* de */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_DE, "Schreibzugriff auf MMC Replay Imagedatei verhindern"},
	/* es */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_ES, "Deshabilitar grabaciÛn en imagen tarjeta MMC Replay"},
	/* fr */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_FR, "Interdire l'Ècriture ‡ l'image MMC Replay"},
	/* hu */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_IT, "Disattiva scrittura su immagine card MMC Replay"},
	/* ko */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_NL, "Schrijven naar het MMC Replay card bestand uitschakelen"},
	/* pl */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_PL, "Wy≥±cz zapis do obrazu karty MMC Replay"},
	/* ru */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_RU, "Disable writes to MMC Replay card image"},
	/* sv */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_SV, "Inaktivera skrivning till MMC Replay-kortavbildning"},
	/* tr */ {IDCLS_MMC_REPLAY_CARD_WRITE_DISABLE_TR, "MMC Replay kart imaj˝nda yazmalar˝ pasifle˛tir"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE,    N_("Specify MMC Replay EEPROM image filename")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_DA, "Angiv filnavn for MMC Replay EEPROM-image"},
	/* de */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_DE, "MMC Replay EEPROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_ES, "Especificar nombre fichero imagen EEPROM MMC Replay"},
	/* fr */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_FR, "SpÈcifier le nom du fichier de l'image EEPROM MMC Replay"},
	/* hu */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_IT, "Specifica il nome file immagine della EEPROM MMC Replay"},
	/* ko */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_NL, "Geef de naam van het MMC Replay EEPROM bestand"},
	/* pl */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_PL, "Okre∂l nazwÍ pliku obrazu EEPROM MMC Replay"},
	/* ru */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_RU, "Specify MMC Replay EEPROM image filename"},
	/* sv */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_SV, "Ange filnamn fˆr MMC Replay EEPROM-avbildning"},
	/* tr */ {IDCLS_SELECT_MMC_REPLAY_EEPROM_IMAGE_TR, "MMC Replay EEPROM imaj dosyaismini belirt"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE,    N_("Enable writes to MMC Replay EEPROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_DA, "Tillad skrivning til MMC Replay EEPROM-image"},
	/* de */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_DE, "Erlaube Schreibzugriff auf MMC Replay EEPROM Imagedatei"},
	/* es */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_ES, "Permitir grabaciÛn en imagen EEPROM MMC Replay"},
	/* fr */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_FR, "Permetre l'Ècriture ‡ l'image MMC Replay EEPROM"},
	/* hu */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_IT, "Attiva scrittura su immagine EEPROM MMC Replay"},
	/* ko */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_KO, "MMC Replay EEPROM ¿ÃπÃ¡ˆ ø° æ≤±‚ ∞°¥…«œ∞‘ «œ±‚"},
	/* nl */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_NL, "Schrijven naar MMC Replay EEPROM bestand inschakelen"},
	/* pl */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_PL, "W≥±cz zapis do obrazu EEPROM MMC Replay"},
	/* ru */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_RU, "Enable writes to MMC Replay EEPROM image"},
	/* sv */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_SV, "Aktivera skrivning till MMC Replay EEPROM-avbildning"},
	/* tr */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_ENABLE_TR, "MMC Replay EEPROM imaj˝nda yazmalar˝ aktif et"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE,    N_("Disable writes to MMC Replay EEPROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_DA, "Skriv ikke til MMC Replay EEPROM-image"},
	/* de */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_DE, "Schreibzugriff auf MMC Replay EEPROM Imagedatei verhindern"},
	/* es */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_ES, "Deshabilitar grabaciÛn en imagen EEPROM MMC Replay"},
	/* fr */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_FR, "Interdire l'Ècriture ‡ l'image MMC Replay EEPROM"},
	/* hu */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_IT, "Disattiva scrittura su immagine EEPROM MMC Replay"},
	/* ko */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_NL, "Schrijven naar MMC Replay EEPROM bestand uitschakelen"},
	/* pl */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_PL, "Wy≥±cz zapis do obrazu EEPROM MMC Replay"},
	/* ru */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_RU, "Disable writes to MMC Replay EEPROM image"},
	/* sv */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_SV, "Inaktivera skrivning till MMC Replay EEPROM-avbildning"},
	/* tr */ {IDCLS_MMC_REPLAY_EEPROM_WRITE_DISABLE_TR, "MMC Replay EEPROM imaj˝nda yazmalar˝ pasifle˛tir"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE,    N_("Enable MMC Replay rescue mode")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_DA, "AktivÈr MMC Replay fejlsikret tilstand"},
	/* de */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_DE, "MMC Replay Rescue Modus aktivieren"},
	/* es */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_ES, "Permitir modo rescate MMC Replay "},
	/* fr */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_FR, "Activer le mode de secours MMC Replay"},
	/* hu */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_IT, "Attiva modalit‡ ripristino MMC Replay"},
	/* ko */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_NL, "MMC Replay reddingsmodus inschakelen"},
	/* pl */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_PL, "W≥±cz tryb ratunkowy MMC Replay"},
	/* ru */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_RU, "Enable MMC Replay rescue mode"},
	/* sv */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_SV, "Aktivera MMC Replay-r‰ddningsl‰ge"},
	/* tr */ {IDCLS_MMC_REPLAY_RESCUE_MODE_ENABLE_TR, "MMC Replay kurtarma modunu aktif et"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE,    N_("Disable MMC Replay rescue mode")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_DA, "DeaktivÈr MMC Replay fejlsikret tilstand"},
	/* de */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_DE, "MMC Replay Rescue Modus deaktivieren"},
	/* es */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_ES, "Deshabilitar modo rescate MMC Replay "},
	/* fr */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_FR, "DÈsactiver le mode de secours MMC Replay"},
	/* hu */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_IT, "Disattiva modalit‡ ripristino MMC Replay"},
	/* ko */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_NL, "MMC Replay reddingsmodus uitschakelen"},
	/* pl */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_PL, "Wy≥±cz tryb ratunkowy MMC Replay"},
	/* ru */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_RU, "Disable MMC Replay rescue mode"},
	/* sv */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_SV, "Inaktivera MMC Replay-r‰ddningsl‰ge"},
	/* tr */ {IDCLS_MMC_REPLAY_RESCUE_MODE_DISABLE_TR, "MMC Replay kurtarma modunu pasifle˛tir"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE,    N_("Allow writing to MMC Replay image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_DA, "Tillad skrivning til MMC Replay-image"},
	/* de */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_DE, "Erlaube Schreibzugriff auf MMC Replay Image"},
	/* es */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_ES, "Permitir grabaciÛn en imagen MMC Replay"},
	/* fr */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_FR, "Permetre l'Ècriture ‡ l'image MMC Replay"},
	/* hu */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_IT, "Attiva scrittura su immagine MMC Replay"},
	/* ko */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_NL, "Sta schrijven naar MMC Replay bestand toe"},
	/* pl */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_PL, "PozwÛl na zapis do obrazu MMC Replay"},
	/* ru */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_RU, "Allow writing to MMC Replay image"},
	/* sv */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_SV, "TillÂt skrivning till MMC Replay-avbildning"},
	/* tr */ {IDCLS_ALLOW_WRITING_TO_MMC_REPLAY_IMAGE_TR, "MMC Replay imaj˝na yazmay˝ aktif et"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE,    N_("Do not write to MMC Replay image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_DA, "Skriv ikke til MMC Replay-image"},
	/* de */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_DE, "Kein Schreibzugriff auf MMC Replay Image"},
	/* es */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_ES, "No grabar en imagen MMC Replay"},
	/* fr */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_FR, "Interdire l'Ècriture ‡ l'image MMC Replay"},
	/* hu */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_IT, "Disattiva scrittura su immagine MMC Replay"},
	/* ko */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_NL, "Schrijf niet naar MMC Replay bestand"},
	/* pl */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_PL, "Nie zapisuj do obrazu MMC Replay"},
	/* ru */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_RU, "Do not write to MMC Replay image"},
	/* sv */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_SV, "Skriv inte till MMC Replay-avbildning"},
	/* tr */ {IDCLS_DO_NOT_WRITE_TO_MMC_REPLAY_IMAGE_TR, "MMC Replay imaj˝na yazma"},
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE,    N_("Specify GMod2 EEPROM image filename")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_DE, "GMod2 EEPROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_NL, "Geef de naam van het GMod2 EEPROM bestand"},
	/* pl */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SELECT_GMOD2_EEPROM_IMAGE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE,    N_("Enable writes to GMod2 EEPROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_DE, "Erlaube Schreibzugriff auf GMod2 EEPROM Image"},
	/* es */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_NL, "Schrijven naar GMod2 EEPROM bestand inschakelen"},
	/* pl */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_GMOD2_EEPROM_WRITE_ENABLE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/mmcreplay.c */
	/* en */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE,    N_("Disable writes to GMod2 EEPROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_DE, "Schreibzugriff auf GMod2 EEPROM Imagedatei verhindern"},
	/* es */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_NL, "Schrijven naar GMod2 EEPROM bestand uitschakelen"},
	/* pl */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_GMOD2_EEPROM_WRITE_DISABLE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/gmod2.c */
	/* en */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT,    N_("Enable saving of the GMod2 ROM at exit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_DE, "Speichern des GMod2 ROMs beim Beenden aktivieren"},
	/* es */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_NL, "Opslaan van het GMod2 ROM bestand bij afsluiten inschakelen"},
	/* pl */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_SAVE_GMOD2_ROM_AT_EXIT_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/gmod2.c */
	/* en */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT,    N_("Disable saving of the GMod2 ROM at exit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_DE, "Speichern des GMod2 ROMs beim Beenden deaktivieren"},
	/* es */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_NL, "Opslaan van het GMod2 ROM bestand bij afsluiten uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_SAVE_GMOD2_ROM_AT_EXIT_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/retroreplay.c */
	/* en */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT,    N_("Enable saving of the RR ROM at exit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_DA, "AktivÈr skrivning af RR ROM ved afslutning"},
	/* de */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_DE, "Speichern des RR ROMs beim Beenden aktivieren"},
	/* es */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_ES, "Permitir grabar ROM RR al salir"},
	/* fr */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_FR, "Activer la sauvegarde du ROM RR ‡ la fin du programme"},
	/* hu */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_IT, "Attiva salvataggio ROM della RR all'uscita"},
	/* ko */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_NL, "Opslaan van het RR ROM bestand bij afsluiten inschakelen"},
	/* pl */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_PL, "W≥±cz zapis ROM RR przy wyj∂ciu"},
	/* ru */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_RU, "Enable saving of the RR ROM at exit"},
	/* sv */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_SV, "Aktivera sparande av RR-ROM vid avslutning"},
	/* tr */ {IDCLS_ENABLE_SAVE_RR_ROM_AT_EXIT_TR, "«˝k˝˛ta RR ROM'un kaydedilmesini aktif et"},
#endif
	
	/* c64/cart/retroreplay.c */
	/* en */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT,    N_("Disable saving of the RR ROM at exit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_DA, "DektivÈr skrivning af RR ROM ved afslutning"},
	/* de */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_DE, "Speichern des RR ROMs beim Beenden deaktivieren"},
	/* es */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_ES, "Deshabilitar grabar ROM RR al salir"},
	/* fr */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_FR, "DÈsactiver la sauvegarde du ROM RR ‡ la fin du programme"},
	/* hu */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_IT, "Disattiva salvataggio ROM della RR all'uscita"},
	/* ko */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_NL, "Opslaan van het RR ROM bestand bij afsluiten uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_PL, "Wy≥±cz zapis ROM RR przy wyj∂ciu"},
	/* ru */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_RU, "Disable saving of the RR ROM at exit"},
	/* sv */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_SV, "Inaktivera sparande av RR-ROM vid avslutning"},
	/* tr */ {IDCLS_DISABLE_SAVE_RR_ROM_AT_EXIT_TR, "«˝k˝˛ta RR ROM'un kaydedilmesini pasifle˛tir"},
#endif
	
	/* c64/cart/retroreplay.c */
	/* en */ {IDCLS_SET_RR_BANK_JUMPER,    N_("Set RR Bank Jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_RR_BANK_JUMPER_DA, "Tilslut RR Bank Jumper"},
	/* de */ {IDCLS_SET_RR_BANK_JUMPER_DE, "Aktiviere RR Bank Jumper"},
	/* es */ {IDCLS_SET_RR_BANK_JUMPER_ES, "Seleccionar conmutador banco RR"},
	/* fr */ {IDCLS_SET_RR_BANK_JUMPER_FR, "Activer le jumper Banque RR"},
	/* hu */ {IDCLS_SET_RR_BANK_JUMPER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_RR_BANK_JUMPER_IT, "Imposta ponticello bank della RR"},
	/* ko */ {IDCLS_SET_RR_BANK_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_RR_BANK_JUMPER_NL, "Stel RR bank jumper in"},
	/* pl */ {IDCLS_SET_RR_BANK_JUMPER_PL, "Ustaw zworkÍ RR Bank"},
	/* ru */ {IDCLS_SET_RR_BANK_JUMPER_RU, "Set RR Bank Jumper"},
	/* sv */ {IDCLS_SET_RR_BANK_JUMPER_SV, "Aktivera RR-bankbygel"},
	/* tr */ {IDCLS_SET_RR_BANK_JUMPER_TR, "RR Bank Jumper'˝ tak"},
#endif
	
	/* c64/cart/retroreplay.c */
	/* en */ {IDCLS_UNSET_RR_BANK_JUMPER,    N_("Unset RR Bank Jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_UNSET_RR_BANK_JUMPER_DA, "Afbryd RR Bank Jumper"},
	/* de */ {IDCLS_UNSET_RR_BANK_JUMPER_DE, "Deaktiviere RR Bank Jumper"},
	/* es */ {IDCLS_UNSET_RR_BANK_JUMPER_ES, "Quitar conmutador banco RR"},
	/* fr */ {IDCLS_UNSET_RR_BANK_JUMPER_FR, "Ne pas activer le jumper Banque RR"},
	/* hu */ {IDCLS_UNSET_RR_BANK_JUMPER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_UNSET_RR_BANK_JUMPER_IT, "Rimuovi ponticello bank della RR"},
	/* ko */ {IDCLS_UNSET_RR_BANK_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_UNSET_RR_BANK_JUMPER_NL, "Verwijder RR bank jumper"},
	/* pl */ {IDCLS_UNSET_RR_BANK_JUMPER_PL, "UsuÒ zworkÍ RR Bank"},
	/* ru */ {IDCLS_UNSET_RR_BANK_JUMPER_RU, "Unset RR Bank Jumper"},
	/* sv */ {IDCLS_UNSET_RR_BANK_JUMPER_SV, "Inaktivera RR-bankbygel"},
	/* tr */ {IDCLS_UNSET_RR_BANK_JUMPER_TR, "RR Bank Jumper'˝ Á˝kart"},
#endif
	
	/* c64/cart/retroreplay.c */
	/* en */ {IDCLS_SET_RR_FLASH_JUMPER,    N_("Set RR Flash Jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_RR_FLASH_JUMPER_DA, "Tilslut RR Flash jumper"},
	/* de */ {IDCLS_SET_RR_FLASH_JUMPER_DE, "Aktviere RR Flash Jumper"},
	/* es */ {IDCLS_SET_RR_FLASH_JUMPER_ES, "Seleccionar conmutador RR Flash"},
	/* fr */ {IDCLS_SET_RR_FLASH_JUMPER_FR, "Activer le Jumper RR Flash"},
	/* hu */ {IDCLS_SET_RR_FLASH_JUMPER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_RR_FLASH_JUMPER_IT, "Imposta ponticello flash della RR"},
	/* ko */ {IDCLS_SET_RR_FLASH_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_RR_FLASH_JUMPER_NL, "Stel RR Flash Jumper in"},
	/* pl */ {IDCLS_SET_RR_FLASH_JUMPER_PL, "Ustaw prze≥±cznik RR Flash"},
	/* ru */ {IDCLS_SET_RR_FLASH_JUMPER_RU, "Set RR Flash Jumper"},
	/* sv */ {IDCLS_SET_RR_FLASH_JUMPER_SV, "S‰tt RR-flashbygel"},
	/* tr */ {IDCLS_SET_RR_FLASH_JUMPER_TR, "RR Flash Jumper'˝ tak"},
#endif
	
	/* c64/cart/retroreplay.c */
	/* en */ {IDCLS_UNSET_RR_FLASH_JUMPER,    N_("Unset RR Bank Jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_UNSET_RR_FLASH_JUMPER_DA, "Afbryd RR Bank Jumper"},
	/* de */ {IDCLS_UNSET_RR_FLASH_JUMPER_DE, "Deaktiviere RR Bank Jumper"},
	/* es */ {IDCLS_UNSET_RR_FLASH_JUMPER_ES, "Quitar conmutador banco RR"},
	/* fr */ {IDCLS_UNSET_RR_FLASH_JUMPER_FR, "Ne pas activer le jumper Banque RR"},
	/* hu */ {IDCLS_UNSET_RR_FLASH_JUMPER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_UNSET_RR_FLASH_JUMPER_IT, "Rimuovi ponticello bank della RR"},
	/* ko */ {IDCLS_UNSET_RR_FLASH_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_UNSET_RR_FLASH_JUMPER_NL, "Verwijder RR bank jumper"},
	/* pl */ {IDCLS_UNSET_RR_FLASH_JUMPER_PL, "UsuÒ zworkÍ RR Bank"},
	/* ru */ {IDCLS_UNSET_RR_FLASH_JUMPER_RU, "Unset RR Bank Jumper"},
	/* sv */ {IDCLS_UNSET_RR_FLASH_JUMPER_SV, "Inaktivera RR-bankbygel"},
	/* tr */ {IDCLS_UNSET_RR_FLASH_JUMPER_TR, "RR Bank Jumper'˝ Á˝kart"},
#endif
	
	/* c64/cart/retroreplay.c */
	/* en */ {IDCLS_RR_REVISION,    N_("Set RR Revision (0: Retro Replay, 1: Nordic Replay)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RR_REVISION_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_RR_REVISION_DE, "RR Revision (0: Retro Replay, 1: Nordic Replay)"},
	/* es */ {IDCLS_RR_REVISION_ES, "Seleccionar RR Revision (0: Retro Replay, 1: Nordic Replay)"},
	/* fr */ {IDCLS_RR_REVISION_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_RR_REVISION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RR_REVISION_IT, "Imposta revisione RR (0: Retro Replay, 1: Nordic Replay)"},
	/* ko */ {IDCLS_RR_REVISION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RR_REVISION_NL, "Stel RR Revisie in (0: Retro Replay, 1: Nordic Replay)"},
	/* pl */ {IDCLS_RR_REVISION_PL, "Okre∂l wersjÍ RR (0: Retro Replay, 1: Nordic Replay)"},
	/* ru */ {IDCLS_RR_REVISION_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_RR_REVISION_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_RR_REVISION_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/supersnapshot.c */
	/* en */ {IDCLS_ENABLE_SS_RAM_EXPANSION,    N_("Enable SS 32k RAM expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SS_RAM_EXPANSION_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_SS_RAM_EXPANSION_DE, "SS 32k RAM Erweiterung aktivieren"},
	/* es */ {IDCLS_ENABLE_SS_RAM_EXPANSION_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_SS_RAM_EXPANSION_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_SS_RAM_EXPANSION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SS_RAM_EXPANSION_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_SS_RAM_EXPANSION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SS_RAM_EXPANSION_NL, "SS 32k RAM uitbreiding inschakelen"},
	/* pl */ {IDCLS_ENABLE_SS_RAM_EXPANSION_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_SS_RAM_EXPANSION_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SS_RAM_EXPANSION_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_SS_RAM_EXPANSION_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/supersnapshot.c */
	/* en */ {IDCLS_DISABLE_SS_RAM_EXPANSION,    N_("Disable SS 32k RAM expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SS_RAM_EXPANSION_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_SS_RAM_EXPANSION_DE, "SS 32k RAM Erweiterung deaktivieren"},
	/* es */ {IDCLS_DISABLE_SS_RAM_EXPANSION_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_SS_RAM_EXPANSION_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_SS_RAM_EXPANSION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SS_RAM_EXPANSION_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_SS_RAM_EXPANSION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SS_RAM_EXPANSION_NL, "SS 32k RAM uitbreiding uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SS_RAM_EXPANSION_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_SS_RAM_EXPANSION_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SS_RAM_EXPANSION_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_SS_RAM_EXPANSION_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/reu.c */
	/* en */ {IDCLS_ENABLE_REU,    N_("Enable the RAM Expansion Unit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_REU_DA, "AktivÈr RAM-udviddelsesenheden"},
	/* de */ {IDCLS_ENABLE_REU_DE, "RAM Speichererweiterung aktivieren"},
	/* es */ {IDCLS_ENABLE_REU_ES, "Permitir unidad RAM expansion"},
	/* fr */ {IDCLS_ENABLE_REU_FR, "Activer l'unitÈ d'expansion RAM"},
	/* hu */ {IDCLS_ENABLE_REU_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_REU_IT, "Attiva l'espansione di RAM"},
	/* ko */ {IDCLS_ENABLE_REU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_REU_NL, "RAM uitbreidingseenheid (REU) inschakelen"},
	/* pl */ {IDCLS_ENABLE_REU_PL, "W≥±cz RAM Expansion Unit"},
	/* ru */ {IDCLS_ENABLE_REU_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_REU_SV, "Aktivera RAM-expansionsenhet"},
	/* tr */ {IDCLS_ENABLE_REU_TR, "RAM Geni˛letme Birimi'ni aktif et"},
#endif
	
	/* c64/cart/reu.c */
	/* en */ {IDCLS_DISABLE_REU,    N_("Disable the RAM Expansion Unit")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_REU_DA, "DeaktivÈr RAM-udviddelsesenhed"},
	/* de */ {IDCLS_DISABLE_REU_DE, "RAM Speichererweiterung deaktivieren"},
	/* es */ {IDCLS_DISABLE_REU_ES, "Deshabilitar unidad RAM Expansion"},
	/* fr */ {IDCLS_DISABLE_REU_FR, "DÈsactiver l'unitÈ d'expansion RAM"},
	/* hu */ {IDCLS_DISABLE_REU_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_REU_IT, "Disattiva l'espansione di RAM"},
	/* ko */ {IDCLS_DISABLE_REU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_REU_NL, "RAM uitbreidingseenheid (REU) uitschakelen"},
	/* pl */ {IDCLS_DISABLE_REU_PL, "Wy≥±cz RAM Expansion Unit"},
	/* ru */ {IDCLS_DISABLE_REU_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_REU_SV, "Inaktivera RAM-expansionsenhet"},
	/* tr */ {IDCLS_DISABLE_REU_TR, "RAM Geni˛letme Birimi'ni pasifle˛tir"},
#endif
	
	/* c64/cart/reu.c */
	/* en */ {IDCLS_SPECIFY_REU_NAME,    N_("Specify name of REU image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_REU_NAME_DA, "Angiv navn pÂ REU-image"},
	/* de */ {IDCLS_SPECIFY_REU_NAME_DE, "Namen f¸r REU Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_REU_NAME_ES, "Especificar nombre imagen REU"},
	/* fr */ {IDCLS_SPECIFY_REU_NAME_FR, "SpÈcifier le nom de l'image REU"},
	/* hu */ {IDCLS_SPECIFY_REU_NAME_HU, "Adja meg a REU kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_REU_NAME_IT, "Specifica il nome immagine REU"},
	/* ko */ {IDCLS_SPECIFY_REU_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_REU_NAME_NL, "Geef de naam van het REU bestand"},
	/* pl */ {IDCLS_SPECIFY_REU_NAME_PL, "Okre∂l nazwÍ obrazu REU"},
	/* ru */ {IDCLS_SPECIFY_REU_NAME_RU, "Specify name of REU image"},
	/* sv */ {IDCLS_SPECIFY_REU_NAME_SV, "Ange namn pÂ REU-avbildning"},
	/* tr */ {IDCLS_SPECIFY_REU_NAME_TR, "REU imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/cart/reu.c */
	/* en */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE,    N_("Allow writing to REU image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_DA, "Tillad skrivning til REU-image"},
	/* de */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_DE, "Schreibzugriff auf REU Image erlauben"},
	/* es */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_ES, "Permitir grabar en imagen REU"},
	/* fr */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_FR, "Permetre l'Ècriture ‡ l'image REU"},
	/* hu */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_IT, "Attiva scrittura su immagine REU"},
	/* ko */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_NL, "Sta schrijven naar REU bestand toe"},
	/* pl */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_PL, "PozwÛl na zapis do obrazu REU"},
	/* ru */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_RU, "Allow writing to REU image"},
	/* sv */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_SV, "TillÂt skrivning till REU-avbildning"},
	/* tr */ {IDCLS_ALLOW_WRITING_TO_REU_IMAGE_TR, "REU imaj˝na yazmay˝ aktif et"},
#endif
	
	/* c64/cart/reu.c */
	/* en */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE,    N_("Do not write to REU image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_DA, "Skriv ikke til REU-image"},
	/* de */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_DE, "Kein Schreibzugriff auf REU Image"},
	/* es */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_ES, "No grabar en imagen REU"},
	/* fr */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_FR, "Interdire l'Ècriture ‡ l'image REU"},
	/* hu */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_IT, "Disattiva scrittura su immagine REU"},
	/* ko */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_NL, "Schrijf niet naar REU bestand"},
	/* pl */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_PL, "Nie zapisuj do obrazu REU"},
	/* ru */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_RU, "Do not write to REU image"},
	/* sv */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_SV, "Skriv inte till REU-avbildning"},
	/* tr */ {IDCLS_DO_NOT_WRITE_TO_REU_IMAGE_TR, "REU imaj˝na yazma"},
#endif
	
	/* c64/cart/reu.c */
	/* en */ {IDCLS_REU_SIZE,    N_("Size of the RAM expansion unit. (128/256/512/1024/2048/4096/8192/16384)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_REU_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_REU_SIZE_DE, "Grˆﬂe der RAM Erweiterung. (128/256/512/1024/2048/4096/8192/16384)"},
	/* es */ {IDCLS_REU_SIZE_ES, "TamaÒo de la unidad de expansiÛn RAM. (128/256/512/1024/2048/4096/8192/16384)"},
	/* fr */ {IDCLS_REU_SIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_REU_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_REU_SIZE_IT, "Dimensione dell'espansione di RAM. (128/256/512/1024/2048/4096/8192/16384)"},
	/* ko */ {IDCLS_REU_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_REU_SIZE_NL, "Grootte van de RAM uitbreidingseenheid. (128/256/512/1024/2048/4096/8192/16384)"},
	/* pl */ {IDCLS_REU_SIZE_PL, "Wielko∂Ê jednostki rozszerzenia RAM. (128/256/512/1024/2048/4096/8192/16384)"},
	/* ru */ {IDCLS_REU_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_REU_SIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_REU_SIZE_TR, ""},  /* fuzzy */
#endif
	
#ifdef HAVE_PCAP
	/* c64/cart/ethernetcart.c */
	/* en */ {IDCLS_ENABLE_TFE,    N_("Enable the Ethernet Cartridge in TFE (\"The Final Ethernet\") compatible mode and set default I/O address")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TFE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TFE_DE, "Aktiviere Ethernet Erweiterungsmodul in TFE (\"The Final Ethernet\") kompatiblen Modus und setze die Standard I/O Adresse"},
	/* es */ {IDCLS_ENABLE_TFE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TFE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TFE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TFE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TFE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TFE_NL, "Ethernet Cartridge inschakelen in TFE (\"The Final Ethernet\") compatibel modus en zet het standaard I/O adres"},
	/* pl */ {IDCLS_ENABLE_TFE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TFE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TFE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TFE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ethernetcart.c */
	/* en */ {IDCLS_ENABLE_RRNET,    N_("Enable the Ethernet Cartridge in RR-Net compatible mode and set default I/O address")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RRNET_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_RRNET_DE, "Aktiviere Ethernet Erweiterungsmodul in RR-Net kompatiblen Modus uns\nsetze die Standard I/O Adresse"},
	/* es */ {IDCLS_ENABLE_RRNET_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_RRNET_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_RRNET_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_RRNET_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_RRNET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RRNET_NL, "Ethernet Cartridge inschakelen in RR-Net compatibel modus en zet het standaard I/O adres"},
	/* pl */ {IDCLS_ENABLE_RRNET_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_RRNET_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_RRNET_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_RRNET_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ethernetcart.c */
	/* en */ {IDCLS_ENABLE_ETHERNETCART,    N_("Enable the Ethernet Cartridge (TFE/RR-Net/64NIC/FB-NET)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_ETHERNETCART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_ETHERNETCART_DE, "Ethernet Cartridge (TFE/RR-Net/64NIC/FB-NET) aktivieren"},
	/* es */ {IDCLS_ENABLE_ETHERNETCART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_ETHERNETCART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_ETHERNETCART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_ETHERNETCART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_ETHERNETCART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_ETHERNETCART_NL, "Ethernet Cartridge (TFE/RR-Net/64NIC/FB-NET) inschakelen"},
	/* pl */ {IDCLS_ENABLE_ETHERNETCART_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_ETHERNETCART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_ETHERNETCART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_ETHERNETCART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ethernetcart.c */
	/* en */ {IDCLS_DISABLE_ETHERNETCART,    N_("Disable the Ethernet Cartridge (TFE/RR-Net/64NIC/FB-NET)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_ETHERNETCART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_ETHERNETCART_DE, "Ethernet Cartridge  (TFE/RR-Net/64NIC/FB-NET)deaktivieren"},
	/* es */ {IDCLS_DISABLE_ETHERNETCART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_ETHERNETCART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_ETHERNETCART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_ETHERNETCART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_ETHERNETCART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_ETHERNETCART_NL, "Ethernet Cartridge (TFE/RR-Net/64NIC/FB-NET) uitschakelen"},
	/* pl */ {IDCLS_DISABLE_ETHERNETCART_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_ETHERNETCART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_ETHERNETCART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_ETHERNETCART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ethernetcart.c */
	/* en */ {IDCLS_ETHERNETCART_BASE,    N_("Base address of the Ethernet Cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ETHERNETCART_BASE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ETHERNETCART_BASE_DE, "Basis Adresse f¸r Ethernet Erweiterungmodul"},
	/* es */ {IDCLS_ETHERNETCART_BASE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ETHERNETCART_BASE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ETHERNETCART_BASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ETHERNETCART_BASE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ETHERNETCART_BASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ETHERNETCART_BASE_NL, "Basisadres van de Ethernet Cartridge"},
	/* pl */ {IDCLS_ETHERNETCART_BASE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ETHERNETCART_BASE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ETHERNETCART_BASE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ETHERNETCART_BASE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ethernetcart.c */
	/* en */ {IDCLS_ETHERNETCART_MODE,    N_("Mode of Ethernet Cartridge (0: TFE, 1: RR-Net)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ETHERNETCART_MODE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ETHERNETCART_MODE_DE, "Modus des Ethernet Erweiterungsmoduls (0: TFE, 1: RR-Net)"},
	/* es */ {IDCLS_ETHERNETCART_MODE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ETHERNETCART_MODE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ETHERNETCART_MODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ETHERNETCART_MODE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ETHERNETCART_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ETHERNETCART_MODE_NL, "Modus van den Ethernet Cartridge (0: TFE, 1: RR-Net)"},
	/* pl */ {IDCLS_ETHERNETCART_MODE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ETHERNETCART_MODE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ETHERNETCART_MODE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ETHERNETCART_MODE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/cs8900io.c */
	/* en */ {IDCLS_ETHERNET_INTERFACE,    N_("Set the system ethernet interface")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ETHERNET_INTERFACE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ETHERNET_INTERFACE_DE, "Setze die System Ethernet Schnittstelle"},
	/* es */ {IDCLS_ETHERNET_INTERFACE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ETHERNET_INTERFACE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ETHERNET_INTERFACE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ETHERNET_INTERFACE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ETHERNET_INTERFACE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ETHERNET_INTERFACE_NL, "Stel de systeem ethernetinterface in"},
	/* pl */ {IDCLS_ETHERNET_INTERFACE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ETHERNET_INTERFACE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ETHERNET_INTERFACE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ETHERNET_INTERFACE_TR, ""},  /* fuzzy */
#endif
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_CART_ATTACH_DETACH_RESET,    N_("Reset machine if a cartridge is attached or detached")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_CART_ATTACH_DETACH_RESET_DA, "Reset maskinen hvis et cartridge tilsluttes eller frakobles"},
	/* de */ {IDCLS_CART_ATTACH_DETACH_RESET_DE, "Reset der Maschine wenn ein Erweiterungsmodul eingelegt oder entfernt wird"},
	/* es */ {IDCLS_CART_ATTACH_DETACH_RESET_ES, "Reiniciar m·quina si un cartucho es insertado o extraido"},
	/* fr */ {IDCLS_CART_ATTACH_DETACH_RESET_FR, "RÈinitialiser si une cartouche est attachÈe ou dÈtachÈe"},
	/* hu */ {IDCLS_CART_ATTACH_DETACH_RESET_HU, "Reset a cartridge csatol·sakorgy lev·laszt·sakor"},
	/* it */ {IDCLS_CART_ATTACH_DETACH_RESET_IT, "Fare il reset del computer se una cartuccia Ë collegata o rimossa"},
	/* ko */ {IDCLS_CART_ATTACH_DETACH_RESET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_CART_ATTACH_DETACH_RESET_NL, "Reset machine als een cartridge wordt gekoppeld of ontkoppeld"},
	/* pl */ {IDCLS_CART_ATTACH_DETACH_RESET_PL, "Resetuj maszynÍ przy zamontowaniu i odmontowaniu kartridøa"},
	/* ru */ {IDCLS_CART_ATTACH_DETACH_RESET_RU, "Reset machine if a cartridge is attached or detached"},
	/* sv */ {IDCLS_CART_ATTACH_DETACH_RESET_SV, "Nollst‰ll maskinen om en insticksmodul kopplas till eller frÂn"},
	/* tr */ {IDCLS_CART_ATTACH_DETACH_RESET_TR, "Bir kartu˛ tak˝l˝r ya da Á˝kart˝l˝rsa makineyi resetle"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_CART_ATTACH_DETACH_NO_RESET,    N_("Do not reset machine if a cartridge is attached or detached")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_DA, "Reset ikke maskinen hvis et cartridge tilsluttes eller frakobles"},
	/* de */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_DE, "Kein Reset der Maschine wenn ein Erweiterungsmodul eingelegt oder entfernt wird"},
	/* es */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_ES, "No reiniciar m·quina si un cartucho es insertado o extraido"},
	/* fr */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_FR, "Ne pas rÈinitialiser si une cartouche est attachÈe ou dÈtachÈe"},
	/* hu */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_HU, "Nincs reset a cartridge csatol·sakorgy lev·laszt·sakor"},
	/* it */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_IT, "Non fare il reset del computer se una cartuccia Ë collegata o rimossa"},
	/* ko */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_NL, "Reset machine niet als een cartridge wordt gekoppeld of ontkoppeld"},
	/* pl */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_PL, "Nie resetuj maszyny przy zamontowaniu i odmontowaniu kartridøa"},
	/* ru */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_RU, "Do not reset machine if a cartridge is attached or detached"},
	/* sv */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_SV, "Nollst‰ll inte maskinen om en insticksmodul kopplas till eller frÂn"},
	/* tr */ {IDCLS_CART_ATTACH_DETACH_NO_RESET_TR, "Bir kartu˛ tak˝l˝r ya da Á˝kart˝l˝rsa makineyi resetleme"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_CRT_CART,    N_("Attach CRT cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_CRT_CART_DA, "Tilslut CRT-cartridge image"},
	/* de */ {IDCLS_ATTACH_CRT_CART_DE, "CRT Erweiterungsmodul Image einlegen"},
	/* es */ {IDCLS_ATTACH_CRT_CART_ES, "Insertar imagen de cartucho CRT"},
	/* fr */ {IDCLS_ATTACH_CRT_CART_FR, "InsÈrer une cartouche CRT"},
	/* hu */ {IDCLS_ATTACH_CRT_CART_HU, "CRT cartridge kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_CRT_CART_IT, "Seleziona immagine cartuccia CRT"},
	/* ko */ {IDCLS_ATTACH_CRT_CART_KO, "CRT ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿø©≥÷¿∏Ω√ø¿"},
	/* nl */ {IDCLS_ATTACH_CRT_CART_NL, "Koppel CRT cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_CRT_CART_PL, "Zamontuj obraz kartridøa CRT"},
	/* ru */ {IDCLS_ATTACH_CRT_CART_RU, "Attach CRT cartridge image"},
	/* sv */ {IDCLS_ATTACH_CRT_CART_SV, "Anslut CRT-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_CRT_CART_TR, "CRT kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_GENERIC_8KB_CART,    N_("Attach raw 8KB cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_GENERIC_8KB_CART_DA, "Tilslut rÂt 8kB-cartridge image"},
	/* de */ {IDCLS_ATTACH_GENERIC_8KB_CART_DE, "8KB Erweiterungsmodul (raw) Image einlegen"},
	/* es */ {IDCLS_ATTACH_GENERIC_8KB_CART_ES, "Insertar imagen de cartucho 8KB"},
	/* fr */ {IDCLS_ATTACH_GENERIC_8KB_CART_FR, "InsÈrer une cartouche 8KO"},
	/* hu */ {IDCLS_ATTACH_GENERIC_8KB_CART_HU, "Nyers 8KB-os cartridge kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_GENERIC_8KB_CART_IT, "Seleziona immagine cartuccia di 8KB"},
	/* ko */ {IDCLS_ATTACH_GENERIC_8KB_CART_KO, "8kb ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_GENERIC_8KB_CART_NL, "Koppel binair 8KB cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_GENERIC_8KB_CART_PL, "Zamontuj surowy obraz kartridøa 8KB"},
	/* ru */ {IDCLS_ATTACH_GENERIC_8KB_CART_RU, "ø‡ÿ⁄‡’ﬂÿ‚Ï ﬂ‡ﬁ’⁄‚ ﬁ—‡–◊– 8∫— ⁄–‡‚‡ÿ‘÷–"},
	/* sv */ {IDCLS_ATTACH_GENERIC_8KB_CART_SV, "Anslut rÂ 8KB-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_GENERIC_8KB_CART_TR, "D¸z 8KB kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_GENERIC_16KB_CART,    N_("Attach raw 16KB cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_GENERIC_16KB_CART_DA, "Tilslut rÂt 16kB-cartridge image"},
	/* de */ {IDCLS_ATTACH_GENERIC_16KB_CART_DE, "16KB Erweiterungsmodul (raw) Image einlegen"},
	/* es */ {IDCLS_ATTACH_GENERIC_16KB_CART_ES, "Insertar imagen de cartucho 16KB"},
	/* fr */ {IDCLS_ATTACH_GENERIC_16KB_CART_FR, "InsÈrer une cartouche 16KO"},
	/* hu */ {IDCLS_ATTACH_GENERIC_16KB_CART_HU, "Nyers 16KB-os cartridge kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_GENERIC_16KB_CART_IT, "Seleziona immagine cartuccia di 16KB"},
	/* ko */ {IDCLS_ATTACH_GENERIC_16KB_CART_KO, "16kb ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_GENERIC_16KB_CART_NL, "Koppel binair 16KB cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_GENERIC_16KB_CART_PL, "Zamontuj surowy obraz kartridøa 16KB"},
	/* ru */ {IDCLS_ATTACH_GENERIC_16KB_CART_RU, "ø‡ÿ⁄‡’ﬂÿ‚Ï ﬂ‡ﬁ’⁄‚ ﬁ—‡–◊– 16∫— ⁄–‡‚‡ÿ‘÷–"},
	/* sv */ {IDCLS_ATTACH_GENERIC_16KB_CART_SV, "Anslut rÂ 16KB-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_GENERIC_16KB_CART_TR, "D¸z 16KB kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART,    N_("Attach generic 16kB Ultimax cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_DA, "Tilslut standard 16kB Ultimax cartridge image"},
	/* de */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_DE, "Generisches 16kB Ultimax Erweiterungsmodul Image einlegen"},
	/* es */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_ES, "Insertar imagen cartucho genÈrico Ultimax 16KB"},
	/* fr */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_FR, "Attacher une cartouche gÈnÈrique 16 ko Ultimax"},
	/* hu */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_IT, "Seleziona immagine cartuccia Ultimax generica di 16kB"},
	/* ko */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_NL, "Koppel algemeen 16KB Ultimax cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_PL, "Zamontuj standardowy obraz kartridøa 16kB Ultimax"},
	/* ru */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_RU, "Attach generic 16kB Ultimax cartridge image"},
	/* sv */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_SV, "Anslut vanlig 16KB Ultimax-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_GENERIC_16KB_ULTIMAX_CART_TR, "Jenerik 16KB Ultimax kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_STARDOS_CART,    N_("Attach raw 16KB Stardos cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_STARDOS_CART_DA, "Tilslut rÂt 16KB StarDOS cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_STARDOS_CART_DE, "16kB Stardos Erweiterungsmodul Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_STARDOS_CART_ES, "Insertar imagen cartucho Stardos 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_STARDOS_CART_FR, "Attacher une image de cartouche 16 ko Stardos"},
	/* hu */ {IDCLS_ATTACH_RAW_STARDOS_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_STARDOS_CART_IT, "Seleziona immagine cartuccia Stardos di 16KB"},
	/* ko */ {IDCLS_ATTACH_RAW_STARDOS_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_STARDOS_CART_NL, "Koppel binair 16KB Stardos cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_STARDOS_CART_PL, "Zamontuj surowy obraz kartridøa 16KB Stardos"},
	/* ru */ {IDCLS_ATTACH_RAW_STARDOS_CART_RU, "Attach raw 16KB Stardos cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_STARDOS_CART_SV, "Anslut rÂ 16KB Stardos-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_STARDOS_CART_TR, "D¸z 16KB Stardos kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART,    N_("Attach raw 32KB Action Replay MK4 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_DA, "Tilslut rÂt 32KB Action Replay MK4 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_DE, "32kB Action Replay MK4 Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_ES, "Insertar imagen cartucho Action Replay MK4 32KB"},
	/* fr */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_FR, "Attacher une image de cartouche 2 ko Action Replay MK4"},
	/* hu */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_IT, "Seleziona immagine cartuccia Action Replay MK4 di 326KB"},
	/* ko */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_NL, "Koppel binair 32KB Action Replay MK4 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_PL, "Zamontuj surowy obraz kartridøa 32KB Action Replay MK4"},
	/* ru */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_RU, "Attach raw 32KB Action Replay MK4 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_SV, "Anslut rÂ 32KB-Action Replay MK4-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_ACTION_REPLAY4_CART_TR, "D¸z 32KB Action Replay MK4 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART,    N_("Attach raw 16kB Action Replay MK2 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_DA, "Tilslut rÂt 16kB Action Replay MK2 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_DE, "16kB Action Replay MK2 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_ES, "Insertar imagen cartucho Action Replay MK2 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_FR, "Attacher une image de cartouche 16 ko Action Replay MK2"},
	/* hu */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_IT, "Seleziona immagine cartuccia Action Replay MK2 di 16KB"},
	/* ko */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_NL, "Koppel binair 16kB Action Replay MK2 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_PL, "Zamontuj surowy obraz kartridøa 16kB Action Replay MK2"},
	/* ru */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_RU, "Attach raw 16kB Action Replay MK2 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_SV, "Anslut rÂ 16KB-Action Replay MK2-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_ACTION_REPLAY2_CART_TR, "D¸z 16kB Action Replay MK2 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART,    N_("Attach raw 16KB Action Replay MK3 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_DA, "Tilslut rÂt 16KB Action Replay MK3 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_DE, "16kB Action Replay MK3 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_ES, "Insertar imagen cartucho Action Replay MK3 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_FR, "Attacher une image de cartouche 16 ko Action Replay MK3"},
	/* hu */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_IT, "Seleziona immagine cartuccia Action Replay MK3 di 16KB"},
	/* ko */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_NL, "Koppel binair 16KB Action Replay MK3 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_PL, "Zamontuj surowy obraz kartridøa 16KB Action Replay MK3"},
	/* ru */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_RU, "Attach raw 16KB Action Replay MK3 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_SV, "Anslut rÂ 16KB-Action Replay MK3-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_ACTION_REPLAY3_CART_TR, "D¸z 16KB Action Replay MK3 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART,    N_("Attach raw 32KB Action Replay cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_DA, "Tilslut rÂt 32KB Action Replay cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_DE, "32kB Action Replay Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_ES, "Insertar imagen cartucho Action Replay 32KB"},
	/* fr */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_FR, "InsÈrer une cartouche Action Replay 32KO"},
	/* hu */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_HU, "32KB-os Action Replay cartridge kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_IT, "Seleziona immagine cartuccia Action Replay di 32KB"},
	/* ko */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_NL, "Koppel binair 32KB Action Replay cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_PL, "Zamontuj surowy obraz kartidøa 32KB Action Replay"},
	/* ru */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_RU, "Attach raw 32KB Action Replay cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_SV, "Anslut rÂ 32KB-Action Replay-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_ACTION_REPLAY_CART_TR, "D¸z 32KB Action Replay kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_CAPTURE_CART,    N_("Attach raw 8kB Capture cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_CAPTURE_CART_DA, "Tilslut rÂt 8kB Capture cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_CAPTURE_CART_DE, "8kB Capture Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_CAPTURE_CART_ES, "Insertar imagen cartucho Capture 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_CAPTURE_CART_FR, "Attacher une image de cartouche 8 ko Capture"},
	/* hu */ {IDCLS_ATTACH_RAW_CAPTURE_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_CAPTURE_CART_IT, "Seleziona immagine cartuccia Capture di 8KB"},
	/* ko */ {IDCLS_ATTACH_RAW_CAPTURE_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_CAPTURE_CART_NL, "Koppel binair 8kB Capture cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_CAPTURE_CART_PL, "Zamontuj surowy obraz kartridøa 8kB Capture"},
	/* ru */ {IDCLS_ATTACH_RAW_CAPTURE_CART_RU, "Attach raw 8kB Capture cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_CAPTURE_CART_SV, "Anslut rÂ 8KB Capture-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_CAPTURE_CART_TR, "D¸z 8kB Capture kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_COMAL80_CART,    N_("Attach raw 64kB Comal 80 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_COMAL80_CART_DA, "Tilslut rÂt 64kB Comal 80 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_COMAL80_CART_DE, "64kB Comal 80 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_COMAL80_CART_ES, "Insertar imagen cartucho Comal 80 64KB"},
	/* fr */ {IDCLS_ATTACH_RAW_COMAL80_CART_FR, "Attacher une image de cartouche 64 ko Comal 80"},
	/* hu */ {IDCLS_ATTACH_RAW_COMAL80_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_COMAL80_CART_IT, "Seleziona immagine cartuccia Comal 80 di 64kB"},
	/* ko */ {IDCLS_ATTACH_RAW_COMAL80_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_COMAL80_CART_NL, "Koppel binair 64kB Comal 80 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_COMAL80_CART_PL, "Zamontuj surowy obraz kartridøa 64kB Comal 80"},
	/* ru */ {IDCLS_ATTACH_RAW_COMAL80_CART_RU, "Attach raw 64kB Comal 80 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_COMAL80_CART_SV, "Anslut rÂ 64kB Comal 80-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_COMAL80_CART_TR, "D¸z 64kB Comal 80 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_DELA_EP256_CART,    N_("Attach raw Dela EP256 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_DA, "Tilslut rÂt Dela EP256 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_DE, "Dela EP256 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_ES, "Insertar imagen cartucho Dela EP256"},
	/* fr */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_FR, "Attacher une image de cartouche RAW EP256"},
	/* hu */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_IT, "Seleziona immagine cartuccia Dela EP256"},
	/* ko */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_NL, "Koppel binair Dela EP256 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_PL, "Zamontuj surowy obraz kartridøa Dela EP256"},
	/* ru */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_RU, "Attach raw Dela EP256 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_SV, "Anslut rÂ dela EP256-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_DELA_EP256_CART_TR, "D¸z Dela EP256 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_DELA_EP64_CART,    N_("Attach raw Dela EP64 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_DA, "Tilslut rÂt Dela EP64 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_DE, "Dela EP64 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_ES, "Insertar imagen cartucho Dela EP64"},
	/* fr */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_FR, "Attacher une image de cartouche Dela EP64"},
	/* hu */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_IT, "Seleziona immagine cartuccia Dela EP64"},
	/* ko */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_NL, "Koppel binair Dela EO64 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_PL, "Zamontuj surowy obraz kartridøa EP64"},
	/* ru */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_RU, "Attach raw Dela EP64 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_SV, "Anslut rÂ Dela EP64-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_DELA_EP64_CART_TR, "D¸z Dela EP64 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART,    N_("Attach raw Dela EP7x8 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_DA, "Tilslut rÂt Dela EP7x8 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_DE, "Dela EP7x8 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_ES, "Insertar imagen cartucho Dela EP 7x8"},
	/* fr */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_FR, "Attacher une image de cartouche Dela EP7x8"},
	/* hu */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_IT, "Seleziona immagine cartuccia Dela EP7x8"},
	/* ko */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_NL, "Koppel binair Dela ep7x8 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_PL, "Zamontuj surowy obraz kartridøa Dela EP7x8"},
	/* ru */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_RU, "Attach raw Dela EP7x8 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_SV, "Anslut rÂ Dela EP7x8-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_DELA_EP7X8_CART_TR, "D¸z Dela EP7x8 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_DINAMIC_CART,    N_("Attach raw 128kB Dinamic cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_DINAMIC_CART_DA, "Tilslut rÂt 128kB Dinamic cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_DINAMIC_CART_DE, "128kB Dinamic Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_DINAMIC_CART_ES, "Insertar imagen cartucho Dinamic 128KB"},
	/* fr */ {IDCLS_ATTACH_RAW_DINAMIC_CART_FR, "Attacher une image RAW 128 ko Dinamic"},
	/* hu */ {IDCLS_ATTACH_RAW_DINAMIC_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_DINAMIC_CART_IT, "Seleziona immagine cartuccia Dinamic di 128kB"},
	/* ko */ {IDCLS_ATTACH_RAW_DINAMIC_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_DINAMIC_CART_NL, "Koppel binair 128kB Dinamic cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_DINAMIC_CART_PL, "Zamontuj surowy obraz kartridøa 128kB Dinamic"},
	/* ru */ {IDCLS_ATTACH_RAW_DINAMIC_CART_RU, "Attach raw 128kB Dinamic cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_DINAMIC_CART_SV, "Anslut rÂ 12kB Dinamic-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_DINAMIC_CART_TR, "D¸z 128kB Dinamic kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART,    N_("Attach raw 8kB Diashow-Maker cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_DA, "Tilslut rÂt 8kB Diashow-Maker cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_DE, "8kB Diashow-Maker Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_ES, "Insertar imagen cartucho Diashow-Maker 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_FR, "Attacher image RAW 8 ko Diashow-Maker"},
	/* hu */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_IT, "Seleziona immagine cartuccia Diashow-Maker di 8KB"},
	/* ko */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_NL, "Koppel binair 8kB Diashow-Maker cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_PL, "Zamontuj surowy obraz kartridøa 8kB Diashow-Maker"},
	/* ru */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_RU, "Attach raw 8kB Diashow-Maker cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_SV, "Anslut rÂ 8kB Diashow-Maker-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_DIASHOW_MAKER_CART_TR, "D¸z 8kB Diashow-Maker kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_DQBB_CART,    N_("Attach raw 16kB Double Quick Brown Box cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_DQBB_CART_DA, "Tilslut rÂt 16kB Double Quick Brown Box cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_DQBB_CART_DE, "16kB Double Quick Brown Box Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_DQBB_CART_ES, "Insertar imagen cartucho DBQQ 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_DQBB_CART_FR, "Attacher image RAW 16 ko Cartouche Double Quick Brown Box"},
	/* hu */ {IDCLS_ATTACH_RAW_DQBB_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_DQBB_CART_IT, "Seleziona immagine cartuccia Double Quick Brown Box di 16kB"},
	/* ko */ {IDCLS_ATTACH_RAW_DQBB_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_DQBB_CART_NL, "Koppel binair 16kB Double Quick Brown Box cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_DQBB_CART_PL, "Zamontuj surowy obraz kartridøa Double Quick Brown Box"},
	/* ru */ {IDCLS_ATTACH_RAW_DQBB_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_DQBB_CART_SV, "Anslut rÂ 16KB Double Quick Brown Box-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_DQBB_CART_TR, "D¸z 16kB Double Quick Brown Box kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART,    N_("Attach raw EasyFlash cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_DA, "Tilslut rÂt EasyFlash cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_DE, "Easyflash Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_ES, "Insertar imagen cartucho  EasyFlash"},
	/* fr */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_FR, "Attacher une image de cartouche EasyFlash"},
	/* hu */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_IT, "Seleziona immagine cartuccia EasyFlash"},
	/* ko */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_KO, "EasyFlash ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_NL, "Koppel binair EasyFlash cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_PL, "Zamontuj surowy obraz kartridøa EasyFlash"},
	/* ru */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_SV, "Anslut rÂ EasyFlash-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_EASY_FLASH_CART_TR, "D¸z EasyFlash kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART,    N_("Attach raw 64KB Retro Replay cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_DA, "Tilslut rÂt 64kB-Retro Replay-cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_DE, "64kB Retro Replay Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_ES, "Insertar imagen cartucho Retro Replay 64KB"},
	/* fr */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_FR, "InsÈrer une cartouche Retro Replay 64KO"},
	/* hu */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_HU, "64KB-os Retro Replay cartridge kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_IT, "Seleziona immagine cartuccia Retro Replay di 64KB"},
	/* ko */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_KO, "64KB Retro Replay ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_NL, "Koppel binair 64KB Retro Replay cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_PL, "Zamontuj surowy obraz kartridøa 64KB Retro Replay"},
	/* ru */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_RU, "Attach raw 64KB Retro Replay cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_SV, "Anslut rÂ 64KB-Retro Replay-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_RETRO_REPLAY_CART_TR, "D¸z 64KB Retro Replay kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART,    N_("Attach raw 8kB REX Utility cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_DA, "Tilslut rÂt 8kB REX Utility cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_DE, "8kB REX Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_ES, "Insertar imagen cartucho REX Utility 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_FR, "Attacher une image de cartouche 8 ko REX Utility"},
	/* hu */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_IT, "Seleziona immagine cartuccia REX Utility di 8kB"},
	/* ko */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_NL, "Koppel binair 8kB REX Utility cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_PL, "Zamontuj surowy obraz kartridøa 8kB REX Utility"},
	/* ru */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_RU, "Attach raw 8kB REX Utility cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_SV, "Anslut rÂ 8kB REX Utility-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_REX_UTILITY_CART_TR, "D¸z 8kB REX Utility kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_SS64_CART,    N_("Attach raw 4kB Snapshot 64 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_SS64_CART_DA, "Tilslut rÂt 4kB Snapshot 64 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_SS64_CART_DE, "4kB Snapshot 64 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_SS64_CART_ES, "Insertar imagen cartucho Snapshot 64 4KB"},
	/* fr */ {IDCLS_ATTACH_RAW_SS64_CART_FR, "Attacher une image de cartouche 4 ko Snapshot 64"},
	/* hu */ {IDCLS_ATTACH_RAW_SS64_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_SS64_CART_IT, "Seleziona immagine cartuccia Snapshot 64 di 4kB"},
	/* ko */ {IDCLS_ATTACH_RAW_SS64_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_SS64_CART_NL, "Koppel binair 4kB Snapshot 64 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_SS64_CART_PL, "Zamontuj surowy obraz kartridøa 4kB Snapshot 64"},
	/* ru */ {IDCLS_ATTACH_RAW_SS64_CART_RU, "Attach raw 4kB Snapshot 64 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_SS64_CART_SV, "Anslut rÂ 4kB Snapshot 64-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_SS64_CART_TR, "D¸z 4kB Snapshot 64 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_IDE64_CART,    N_("Attach raw 64KB IDE64 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_IDE64_CART_DA, "Tilslut rÂt 64kB IDE64-cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_IDE64_CART_DE, "64kB IDE64 Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_IDE64_CART_ES, "Insertar imagen cartucho IDE64 64KB"},
	/* fr */ {IDCLS_ATTACH_RAW_IDE64_CART_FR, "InsÈrer une cartouche IDE64 64KO"},
	/* hu */ {IDCLS_ATTACH_RAW_IDE64_CART_HU, "64KB-os IDE64 cartridge kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_RAW_IDE64_CART_IT, "Seleziona immagine cartuccia IDE64 di 64KB"},
	/* ko */ {IDCLS_ATTACH_RAW_IDE64_CART_KO, "64kb IDE64 ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_RAW_IDE64_CART_NL, "Koppel binair 64KB IDE64 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_IDE64_CART_PL, "Zamontuj surowy obraz kartridøa 64KB IDE64"},
	/* ru */ {IDCLS_ATTACH_RAW_IDE64_CART_RU, "Attach raw 64KB IDE64 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_IDE64_CART_SV, "Anslut rÂ 64KB IDE64-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_IDE64_CART_TR, "D¸z 64KB IDE64 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART,    N_("Attach raw 32KB Atomic Power cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_DA, "Tilslut rÂt 32kB Atomic Power-cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_DE, "32kB Atomic Power Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_ES, "Insertar imagen cartucho Atomic Power 32KB"},
	/* fr */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_FR, "InsÈrer une cartouche Atomic Power 32KO"},
	/* hu */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_HU, "32KB-os Atomic Power cartridge kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_IT, "Seleziona immagine cartuccia Atomic Power di 32KB"},
	/* ko */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_KO, "32kb Atomic Power ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_NL, "Koppel binair 32KB Atomic Power cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_PL, "Zamontuj surowy obraz kartridøa 32KB Atomic Power"},
	/* ru */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_RU, "Attach raw 32KB Atomic Power cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_SV, "Anslut rÂ 32KB Atomic Power-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_ATOMIC_POWER_CART_TR, "D¸z 32KB Atomic Power kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART,    N_("Attach raw 8KB Epyx FastLoad cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_DA, "Tilslut rÂt 8kB Epyx FastLoad cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_DE, "8kB Epyx Fastload Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_ES, "Insertar imagen cartucho Epyx Fastload 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_FR, "Attacher une image de cartouche 8 ko Epyx FastLoad"},
	/* hu */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_IT, "Seleziona immagine cartuccia Epyx FastLoad di 8KB"},
	/* ko */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_KO, "8kb Epyx FastLoad ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_NL, "Koppel binair 8KB Epyx FastLoad cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_PL, "Zamontuj surowy obraz kartridøa 8KB Epyx FastLoad"},
	/* ru */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_RU, "Attach raw 8KB Epyx FastLoad cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_SV, "Anslut rÂ 8KB Epyx FastLoad-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_EPYX_FASTLOAD_CART_TR, "D¸z 8KB Epyx FastLoad kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_EXOS_CART,    N_("Attach raw 8kB EXOS cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_EXOS_CART_DA, "Tilslut rÂt 8kB EXOS cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_EXOS_CART_DE, "8kB EXOS Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_EXOS_CART_ES, "Insertar imagen cartucho EXOS 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_EXOS_CART_FR, "Attacher une image de cartouche EXOS"},
	/* hu */ {IDCLS_ATTACH_RAW_EXOS_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_EXOS_CART_IT, "Seleziona immagine cartuccia EXOS di 8KB"},
	/* ko */ {IDCLS_ATTACH_RAW_EXOS_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_EXOS_CART_NL, "Koppel binair 8kB EXOS cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_EXOS_CART_PL, "Zamontuj surowy obraz kartridøa 8kB EXOS"},
	/* ru */ {IDCLS_ATTACH_RAW_EXOS_CART_RU, "Attach raw 8kB EXOS cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_EXOS_CART_SV, "Anslut rÂ 8kB EXOS-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_EXOS_CART_TR, "D¸z 8kB EXOS kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_EXPERT_CART,    N_("Attach raw 8kB Expert Cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_EXPERT_CART_DA, "Tilslut rÂt 8kB Expert Cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_EXPERT_CART_DE, "8kB Expert Cartridge Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_EXPERT_CART_ES, "Insertar imagen cartucho Expert 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_EXPERT_CART_FR, "InsÈrer une image de Cartouche Expert 8 ko"},
	/* hu */ {IDCLS_ATTACH_RAW_EXPERT_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_EXPERT_CART_IT, "Seleziona immagine cartuccia Expert Cartridge di 8kB"},
	/* ko */ {IDCLS_ATTACH_RAW_EXPERT_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_EXPERT_CART_NL, "Koppel binair 8kB Expert Cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_EXPERT_CART_PL, "Zamontuj surowy obraz 8kB Expert Cartridge"},
	/* ru */ {IDCLS_ATTACH_RAW_EXPERT_CART_RU, "Attach raw 8kB Expert Cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_EXPERT_CART_SV, "Anslut rÂ 8kB Expert Cartridge-avbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_EXPERT_CART_TR, "D¸z 8kB Expert Kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_F64_CART,    N_("Attach raw 32kB Formel 64 image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_F64_CART_DA, "Tilslut rÂt 32kB Formel 64 image"},
	/* de */ {IDCLS_ATTACH_RAW_F64_CART_DE, "32kB Formel 64 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_F64_CART_ES, "Insertar imagen cartucho Formel 64 de 32KB"},
	/* fr */ {IDCLS_ATTACH_RAW_F64_CART_FR, "Attacher une image de cartouche 32 ko Formel 64"},
	/* hu */ {IDCLS_ATTACH_RAW_F64_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_F64_CART_IT, "Seleziona immagine cartuccia Formel 64 di 32kb"},
	/* ko */ {IDCLS_ATTACH_RAW_F64_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_F64_CART_NL, "Koppel binair 32kB Formel 64 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_F64_CART_PL, "Zamontuj surowy obraz 32kB Formel 64"},
	/* ru */ {IDCLS_ATTACH_RAW_F64_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_F64_CART_SV, "Anslut rÂ 32kB Formel 64-avbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_F64_CART_TR, "D¸z 32kB Formel 64 imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_FC1_CART,    N_("Attach raw 16kB Final Cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_FC1_CART_DA, "Tilslut rÂt 16kB Final Cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_FC1_CART_DE, "16kB Final Cartridge Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_FC1_CART_ES, "Insertar imagen cartucho Final Cartridge 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_FC1_CART_FR, "Attacher une image de cartouche 16 ko Final Cartridge"},
	/* hu */ {IDCLS_ATTACH_RAW_FC1_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_FC1_CART_IT, "Seleziona immagine cartuccia Final Cartridge di 16kB"},
	/* ko */ {IDCLS_ATTACH_RAW_FC1_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_FC1_CART_NL, "Koppel binair 16kB Final Cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_FC1_CART_PL, "Zamontuj surowy obraz 16kB Final Cartridge"},
	/* ru */ {IDCLS_ATTACH_RAW_FC1_CART_RU, "Attach raw 16kB Final Cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_FC1_CART_SV, "Anslut rÂ 16kB Final Cartridge-avbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_FC1_CART_TR, "D¸z 16kB Final kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_FC3_CART,    N_("Attach raw 64kB Final Cartridge III image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_FC3_CART_DA, "Tilslut rÂt 64kB Final Cartridge III image"},
	/* de */ {IDCLS_ATTACH_RAW_FC3_CART_DE, "64kB Final Cartridge Image III (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_FC3_CART_ES, "Insertar imagen cartucho Final Cartridge III 64KB"},
	/* fr */ {IDCLS_ATTACH_RAW_FC3_CART_FR, "Attacher une image de cartouche 64 ko Final Cartridge III"},
	/* hu */ {IDCLS_ATTACH_RAW_FC3_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_FC3_CART_IT, "Seleziona immagine cartuccia Final Cartridge III di 64kB"},
	/* ko */ {IDCLS_ATTACH_RAW_FC3_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_FC3_CART_NL, "Koppel binair 16kB Final Cartridge III bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_FC3_CART_PL, "Zamontuj surowy obraz Final Cartridge III"},
	/* ru */ {IDCLS_ATTACH_RAW_FC3_CART_RU, "Attach raw 64kB Final Cartridge III image"},
	/* sv */ {IDCLS_ATTACH_RAW_FC3_CART_SV, "Anslut rÂ 64kB Final Cartridge III-avbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_FC3_CART_TR, "D¸z 64kB Final Cartridge III imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_FCPLUS_CART,    N_("Attach raw 32kB Final Cartridge Plus image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_FCPLUS_CART_DA, "Tilslut rÂt 32kB Final Cartridge Plus image"},
	/* de */ {IDCLS_ATTACH_RAW_FCPLUS_CART_DE, "32kB Final Cartridge Plus Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_FCPLUS_CART_ES, "Insertar imagen cartucho Final Cartridge PLUS 32KB"},
	/* fr */ {IDCLS_ATTACH_RAW_FCPLUS_CART_FR, "Attacher une image de cartouche 32 ko Final Cartridge Plus"},
	/* hu */ {IDCLS_ATTACH_RAW_FCPLUS_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_FCPLUS_CART_IT, "Seleziona immagine cartuccia Final Cartridge Plus di 32kB"},
	/* ko */ {IDCLS_ATTACH_RAW_FCPLUS_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_FCPLUS_CART_NL, "Koppel binair 32kB Final Cartridge Plus bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_FCPLUS_CART_PL, "Zamontuj surowy obraz 32kB Final Cartridge Plus"},
	/* ru */ {IDCLS_ATTACH_RAW_FCPLUS_CART_RU, "Attach raw 32kB Final Cartridge Plus image"},
	/* sv */ {IDCLS_ATTACH_RAW_FCPLUS_CART_SV, "Anslut rÂ 32kB Final Cartridge Plus-fil"},
	/* tr */ {IDCLS_ATTACH_RAW_FCPLUS_CART_TR, "D¸z 32kB Final Cartridge Plus imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART,    N_("Attach raw 8kB Freeze Frame image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_DA, "Tilslut rÂt 8kB Freeze Frame image"},
	/* de */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_DE, "8kB Freeze Frame Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_ES, "Insertar imagen cartucho Freeze Frame 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_FR, "Attacher une image de cartouche 8 ko Freeze Frame"},
	/* hu */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_IT, "Seleziona immagine cartuccia Freeze Frame di 8kb"},
	/* ko */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_NL, "Koppel binair 8kB Freeze Frame cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_PL, "Zamontuj surowy obraz 8kB Freeze Frame"},
	/* ru */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_RU, "Attach raw 8kB Freeze Frame image"},
	/* sv */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_SV, "Anslut rÂ 8kB Freeze Frame-avbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_FREEZE_FRAME_CART_TR, "D¸z 8kB Freeze Frame imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART,    N_("Attach raw 32kB Freeze Machine image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_DA, "Tilslut rÂt 32kB Freeze Machine image"},
	/* de */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_DE, "8kB Freeze Machine Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_ES, "Insertar imagen cartucho Freeze Machine 32KB"},
	/* fr */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_FR, "Attacher une image de cartouche 32 ko Freeze Machine"},
	/* hu */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_IT, "Seleziona immagine cartuccia Freeze Machine di 32kb"},
	/* ko */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_NL, "Koppel binair 32kB Freeze Machine cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_PL, "Zamontuj surowy obraz 32kB Freeze Machine"},
	/* ru */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_RU, "Attach raw 32kB Freeze Machine image"},
	/* sv */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_SV, "Anslut rÂ 32kB Freeze Machine-avbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_FREEZE_MACHINE_CART_TR, "D¸z 32kB Freeze Machine imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_FP_PP_CART,    N_("Attach raw 128kB Fun Play/Power Play cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_FP_PP_CART_DA, "Tilslut rÂt 128kB Fun Play/Power Play cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_FP_PP_CART_DE, "128kB Fun Play/Power Play Cartridge  Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_FP_PP_CART_ES, "Insertar imagen cartucho Fun Play/Power Play 128KB"},
	/* fr */ {IDCLS_ATTACH_RAW_FP_PP_CART_FR, "Attacher une image de cartouche 128 ko Fun Play / Power Play"},
	/* hu */ {IDCLS_ATTACH_RAW_FP_PP_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_FP_PP_CART_IT, "Seleziona immagine cartuccia Fun Play/Power Play di 128kB"},
	/* ko */ {IDCLS_ATTACH_RAW_FP_PP_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_FP_PP_CART_NL, "Koppel binair 128kB Fun Play/Power Play cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_FP_PP_CART_PL, "Zamontuj surowy obraz kartridøa 128kB Fun Play/Power Play"},
	/* ru */ {IDCLS_ATTACH_RAW_FP_PP_CART_RU, "Attach raw 128kB Fun Play/Power Play cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_FP_PP_CART_SV, "Anslut rÂ 128kB Fun Play/Power Play-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_FP_PP_CART_TR, "D¸z 128kB Fun Play/Power Play kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_SS4_CART,    N_("Attach raw 32KB Super Snapshot V4 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_SS4_CART_DA, "Tilslut rÂt 32kB Super Snapshot V4 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_SS4_CART_DE, "32kB Super Snapshot V4 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_SS4_CART_ES, "Insertar imagen cartucho Super Snapshot V4 32KB"},
	/* fr */ {IDCLS_ATTACH_RAW_SS4_CART_FR, "Attacher une image de cartouche 32 ko Super Snapshot V4"},
	/* hu */ {IDCLS_ATTACH_RAW_SS4_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_SS4_CART_IT, "Seleziona immagine cartuccia Super Snapshot V4 di 32KB"},
	/* ko */ {IDCLS_ATTACH_RAW_SS4_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_SS4_CART_NL, "Koppel binair 32KB Super Snapshot V4 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_SS4_CART_PL, "Zamontuj surowy obraz kartridøa 32KB Super Snapshot V4"},
	/* ru */ {IDCLS_ATTACH_RAW_SS4_CART_RU, "Attach raw 32KB Super Snapshot V4 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_SS4_CART_SV, "Anslut rÂ 32KB Super Snapshot V4-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_SS4_CART_TR, "D¸z 32KB Super Snapshot V4 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_SS5_CART,    N_("Attach raw 64KB Super Snapshot V5 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_SS5_CART_DA, "Tilslut rÂt 64kB Super Snapshot V5 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_SS5_CART_DE, "64kB Super Snapshot V5 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_SS5_CART_ES, "Insertar imagen cartucho Super Snapshot V5 64KB"},
	/* fr */ {IDCLS_ATTACH_RAW_SS5_CART_FR, "Attacher une image de cartouche 64 ko Super Snapshot V5"},
	/* hu */ {IDCLS_ATTACH_RAW_SS5_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_SS5_CART_IT, "Seleziona immagine cartuccia Super Snapshot V5 di 64KB"},
	/* ko */ {IDCLS_ATTACH_RAW_SS5_CART_KO, "64KB Super Snapshot V5 ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_RAW_SS5_CART_NL, "Koppel binair 64KB Super Snapshot V5 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_SS5_CART_PL, "Zamontuj surowy obraz kartridøa Super Snapshot V5"},
	/* ru */ {IDCLS_ATTACH_RAW_SS5_CART_RU, "Attach raw 64KB Super Snapshot V5 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_SS5_CART_SV, "Anslut rÂ 64KB-Super Snapshot V5-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_SS5_CART_TR, "D¸z 64KB Super Snapshot V5 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_CBM_IEEE488_CART,    N_("Attach IEEE-488 Interface cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_CBM_IEEE488_CART_DA, "Tilslut IEEE-488 interface cartridge image"},
	/* de */ {IDCLS_ATTACH_CBM_IEEE488_CART_DE, "IEEE Schnittstellen Erweiterungsmodul Image einlegen"},
	/* es */ {IDCLS_ATTACH_CBM_IEEE488_CART_ES, "Insertar imagen de cartucho Interface IEEE-488"},
	/* fr */ {IDCLS_ATTACH_CBM_IEEE488_CART_FR, "Attacher une image de cartouche Interface IEEE-488"},
	/* hu */ {IDCLS_ATTACH_CBM_IEEE488_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_CBM_IEEE488_CART_IT, "Seleziona immagine cartuccia interfaccia IEEE"},
	/* ko */ {IDCLS_ATTACH_CBM_IEEE488_CART_KO, "IEEE-488 ¿Œ≈Õ∆–¿ÃΩ∫ ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿø©≥÷¿∏ººø‰"},
	/* nl */ {IDCLS_ATTACH_CBM_IEEE488_CART_NL, "Koppel IEEE-488 Interface cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_CBM_IEEE488_CART_PL, "Zamontuj obraz kartridøa IEEE-488 Interface"},
	/* ru */ {IDCLS_ATTACH_CBM_IEEE488_CART_RU, "Attach IEEE-488 Interface cartridge image"},
	/* sv */ {IDCLS_ATTACH_CBM_IEEE488_CART_SV, "Anslut IEEE-488-gr‰nssnittsavbildningsfil"},
	/* tr */ {IDCLS_ATTACH_CBM_IEEE488_CART_TR, "IEEE-488 Arabirimi kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_ISEPIC_CART,    N_("Attach raw 2kB ISEPIC cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_ISEPIC_CART_DA, "Tilslut rÂt 2kB ISEPIC-cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_ISEPIC_CART_DE, "2kB ISEPIC Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_ISEPIC_CART_ES, "Insertar imagen cartucho ISEPIC 2KB"},
	/* fr */ {IDCLS_ATTACH_RAW_ISEPIC_CART_FR, "Attacher une image de cartouche 2 ko ISEPIC"},
	/* hu */ {IDCLS_ATTACH_RAW_ISEPIC_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_ISEPIC_CART_IT, "Seleziona immagine cartuccia ISEPIC di 2kB"},
	/* ko */ {IDCLS_ATTACH_RAW_ISEPIC_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_ISEPIC_CART_NL, "Koppel binair 2kB ISEPIC cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_ISEPIC_CART_PL, "Zamontuj surowy obraz kartridøa 2kB ISEPIC"},
	/* ru */ {IDCLS_ATTACH_RAW_ISEPIC_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_ISEPIC_CART_SV, "Anslut rÂ 2kB ISEPIC-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_ISEPIC_CART_TR, "D¸z 2kB ISEPIC kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_KCS_CART,    N_("Attach raw 16kB KCS Power cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_KCS_CART_DA, "Tilslut rÂt 16kB KCS Power Cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_KCS_CART_DE, "16kB KCS Power Cartridge Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_KCS_CART_ES, "Insertar imagen cartucho KCS Power Cartridge 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_KCS_CART_FR, "Attacher une image de cartouche 16 ko KCS Power"},
	/* hu */ {IDCLS_ATTACH_RAW_KCS_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_KCS_CART_IT, "Seleziona immagine cartuccia KCS Power di 16kB"},
	/* ko */ {IDCLS_ATTACH_RAW_KCS_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_KCS_CART_NL, "Koppel binair 16kB KSC Power cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_KCS_CART_PL, "Zamontuj surowy obraz kartridøa 16kB KCS Power"},
	/* ru */ {IDCLS_ATTACH_RAW_KCS_CART_RU, "Attach raw 16kB KCS Power cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_KCS_CART_SV, "Anslut rÂ 16KB KCS Power-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_KCS_CART_TR, "D¸z 16kB KCS Power kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_KINGSOFT_CART,    N_("Attach raw 24kB Kingsoft cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_DA, "Tilslut rÂt 24kB Snapshot 64 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_DE, "24kB Kingsoft Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_ES, "Insertar imagen cartucho 24KB Kingsoft"},
	/* fr */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_FR, "Attacher une image de cartouche 24 ko Kingsoft"},
	/* hu */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_IT, "Seleziona immagine cartuccia Kingsoft di 24kB"},
	/* ko */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_NL, "Koppel binair 24kB Kingsoft cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_PL, "Zamontuj surowy obraz kartridøa 24kB Kingsoft"},
	/* ru */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_SV, "Anslut rÂ 24kB-Kingsoft-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_KINGSOFT_CART_TR, "D¸z 24kB Kingsoft kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_MACH5_CART,    N_("Attach raw 8kB MACH 5 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_MACH5_CART_DA, "Tilslut rÂt 8kB MACH 5 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_MACH5_CART_DE, "8kB MAC 5 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_MACH5_CART_ES, "Insertar imagen cartucho MACH 5 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_MACH5_CART_FR, "Attacher une image de cartouche 8 ko MACH 5"},
	/* hu */ {IDCLS_ATTACH_RAW_MACH5_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_MACH5_CART_IT, "Seleziona immagine cartuccia MACH 5 di 8kB"},
	/* ko */ {IDCLS_ATTACH_RAW_MACH5_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_MACH5_CART_NL, "Koppel binair 8kB MACH 5 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_MACH5_CART_PL, "Zamontuj surowy obraz kartridøa 8kB MACH 5"},
	/* ru */ {IDCLS_ATTACH_RAW_MACH5_CART_RU, "Attach raw 8kB MACH 5 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_MACH5_CART_SV, "Anslut rÂ 8kB MACH 5-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_MACH5_CART_TR, "D¸z 8kB MACH 5 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART,    N_("Attach raw 32/64/128kB Magic Desk cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_DA, "Tilslut rÂt 32/64/128kB Magic Desk cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_DE, "32/64/128kB Magic Desk Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_ES, "Insertar imagen cartucho Magic Desc 32/64/128KB"},
	/* fr */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_FR, "Attacher une image de cartouche 32/64/128 ko Magic Desk"},
	/* hu */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_IT, "Seleziona immagine cartuccia Magic Desk di 32/64/128kB"},
	/* ko */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_NL, "Koppel binair 32/64/128kB Magic Desk cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_PL, "Zamontuj surowy obraz kartridøa 32/64/128kB Magic Desk"},
	/* ru */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_RU, "Attach raw 32/64/128kB Magic Desk cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_SV, "Anslut rÂ 32/64/128kB Magic Desk-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_MAGIC_DESK_CART_TR, "D¸z 32/64/128kB Magic Desk kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART,    N_("Attach raw Magic Formel cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_DA, "Tilslut rÂt Magic Formel cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_DE, "Magic Formel Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_ES, "Insertar imagen cartucho Magic Formel"},
	/* fr */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_FR, "Attacher une image de cartouche Magic Formel"},
	/* hu */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_IT, "Seleziona immagine cartuccia Magic Formel"},
	/* ko */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_KO, "Magic Formel ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_NL, "Koppel binair Magic Formel cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_PL, "Zamontuj surowy obraz kartridøa Magic Formel"},
	/* ru */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_RU, "Attach raw Magic Formel cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_SV, "Anslut rÂ Magic Romel-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_MAGIC_FORMEL_CART_TR, "D¸z Magic Formel kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART,    N_("Attach raw 8kB Mikro Assembler cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_DA, "Tilslut rÂt 8kB Mikro Assembler cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_DE, "8kB Mikro Assembler Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_ES, "Insertar imagen cartucho Mikro Assembler 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_FR, "Attacher une image de cartouche 8 ko Mikor Assembler"},
	/* hu */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_IT, "Seleziona immagine cartuccia Mikro Assembler di 8kB"},
	/* ko */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_NL, "Koppel binair 8KB Mikro Assembler cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_PL, "Zamontuj surowy obraz kartridøa 8kB Mikro Assembler"},
	/* ru */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_RU, "Attach raw 8kB Mikro Assembler cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_SV, "Anslut rÂ 8kB Mikro Assembler-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_MIKRO_ASSEMBLER_CART_TR, "D¸z 8kB Mikro Assembler kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_MMC64_CART,    N_("Attach raw 8kB MMC64 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_MMC64_CART_DA, "Tilslut rÂt 8kB MMC64 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_MMC64_CART_DE, "8kB MMC64 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_MMC64_CART_ES, "Insertar imagen cartucho MMC64 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_MMC64_CART_FR, "Attacher une image de cartouche 8 ko MMC64"},
	/* hu */ {IDCLS_ATTACH_RAW_MMC64_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_MMC64_CART_IT, "Seleziona immagine cartuccia MMC64 di 8kB"},
	/* ko */ {IDCLS_ATTACH_RAW_MMC64_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_MMC64_CART_NL, "Koppel binair 8kB MMC64 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_MMC64_CART_PL, "Zamontuj surowy obraz kartridøa 8kB MMC64"},
	/* ru */ {IDCLS_ATTACH_RAW_MMC64_CART_RU, "Attach raw 8kB MMC64 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_MMC64_CART_SV, "Anslut rÂ 8kB MMC64-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_MMC64_CART_TR, "D¸z 8kB MMC64 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART,    N_("Attach raw 512kB MMC Replay cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_DA, "Tilslut rÂt 512kB MMC Replay cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_DE, "512kB MMC Replay Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_ES, "Insertar imagen cartucho MMC Replay 512KB"},
	/* fr */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_FR, "Attacher une image de cartouche 512 ko MMC Replay"},
	/* hu */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_IT, "Seleziona immagine cartuccia MMC Replay di 512kB"},
	/* ko */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_KO, "512KB MMC Replay ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ ∫Ÿ¿Ãººø‰"},
	/* nl */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_NL, "Koppel binair 512kB MMC Replay cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_PL, "Zamontuj surowy obraz kartridøa 512kB MMC Replay"},
	/* ru */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_RU, "Attach raw 512kB MMC Replay cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_SV, "Anslut rÂ 512kB MMC Replay-avbildningsfil"},
	/* tr */ {IDCLS_ATTACH_RAW_MMC_REPLAY_CART_TR, "D¸z 512kB MMC Replay kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART,    N_("Attach raw 16kB Magic Voice cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_DA, "Tilslut rÂt 16kB Magic Voice cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_DE, "16kB Magic Voice Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_ES, "Insertar imagen cartucho Magic Voice 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_FR, "Attacher une image de cartouche 16 ko Magic Voice"},
	/* hu */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_IT, "Seleziona immagine cartuccia Magic Voice di 16kB"},
	/* ko */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_NL, "Koppel binair 16kB Magic Voice cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_PL, "Zamontuj surowy obraz kartridøa 16kB Magic Voice"},
	/* ru */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_RU, "Attach raw 16kB Magic Voice cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_SV, "Anslut rÂ 16kB Magic Voice-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_MAGIC_VOICE_CART_TR, "D¸z 16kB Magic Voice kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_OCEAN_CART,    N_("Attach raw Ocean cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_OCEAN_CART_DA, "Tilslut rÂt Ocean cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_OCEAN_CART_DE, "Ocean Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_OCEAN_CART_ES, "Insertar imagen cartucho Ocean"},
	/* fr */ {IDCLS_ATTACH_RAW_OCEAN_CART_FR, "Attacher une image de cartouche RAW Ocean"},
	/* hu */ {IDCLS_ATTACH_RAW_OCEAN_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_OCEAN_CART_IT, "Seleziona immagine cartuccia Ocean"},
	/* ko */ {IDCLS_ATTACH_RAW_OCEAN_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_OCEAN_CART_NL, "Koppel binair Ocean cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_OCEAN_CART_PL, "Zamontuj surowy obraz kartridøa Ocean"},
	/* ru */ {IDCLS_ATTACH_RAW_OCEAN_CART_RU, "Attach raw Ocean cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_OCEAN_CART_SV, "Anslut rÂ Ocean-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_OCEAN_CART_TR, "D¸z Ocean kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_WESTERMANN_CART,    N_("Attach raw 16KB Westermann Learning cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_DA, "Tilslut rÂt 16kB Westermann Learning cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_DE, "16kB Westermann learning Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_ES, "Insertar imagen cartucho Wstermann Learning 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_FR, "Attacher une image de cartouche 16 ko Westermann Leaning"},
	/* hu */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_IT, "Seleziona immagine cartuccia Westermann Learning di 16KB"},
	/* ko */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_NL, "Koppel binair 16kB Westermann Learning cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_PL, "Zamontuj surowy obraz kartridøa 16KB Westermann Learning"},
	/* ru */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_RU, "Attach raw 16KB Westermann Learning cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_SV, "Anslut rÂ 16KB Westermann Learning-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_WESTERMANN_CART_TR, "D¸z 16KB Westermann Learning kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_WARPSPEED_CART,    N_("Attach raw 8kB Warp Speed cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_DA, "Tilslut rÂt 8kB Warp Speed cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_DE, "8kB Warp Speed Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_ES, "Insertar imagen cartucho Warp Speed 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_FR, "Attacher une image de cartouche 8 ko Warp Speed"},
	/* hu */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_IT, "Seleziona immagine cartuccia Warp Speed di 8kB"},
	/* ko */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_NL, "Koppel binair 8kB Warp Speed cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_PL, "Zamontuj surowy obraz kartridøa 8kB Warp Speed"},
	/* ru */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_RU, "Attach raw 8kB Warp Speed cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_SV, "Anslut rÂ 8kB-Warp Speed-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_WARPSPEED_CART_TR, "D¸z 8kB Warp Speed kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_ZAXXON_CART,    N_("Attach raw 16kB Zaxxon cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_ZAXXON_CART_DA, "Tilslut rÂt 16kB Zaxxon cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_ZAXXON_CART_DE, "16kB Zaxxon Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_ZAXXON_CART_ES, "Insertar imagen cartucho Zaxxon 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_ZAXXON_CART_FR, "Attacher une image de cartouche 16 ko Zaxxon"},
	/* hu */ {IDCLS_ATTACH_RAW_ZAXXON_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_ZAXXON_CART_IT, "Seleziona immagine cartuccia Zaxxon di 16kB"},
	/* ko */ {IDCLS_ATTACH_RAW_ZAXXON_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_ZAXXON_CART_NL, "Koppel binair 16kB Zaxxon cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_ZAXXON_CART_PL, "Zamontuj surowy obraz kartridøa 16kB Zaxxon"},
	/* ru */ {IDCLS_ATTACH_RAW_ZAXXON_CART_RU, "Attach raw 16kB Zaxxon cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_ZAXXON_CART_SV, "Anslut rÂ 16kB Zaxxon-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_ZAXXON_CART_TR, "D¸z 16kB Zaxxon kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_EASYCALC_CART,    N_("Attach raw 24kB Easy Calc Result cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_EASYCALC_CART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ATTACH_RAW_EASYCALC_CART_DE, "Easy Calc Result Erweiterungsmodul Image (24kb raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_EASYCALC_CART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ATTACH_RAW_EASYCALC_CART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ATTACH_RAW_EASYCALC_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_EASYCALC_CART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ATTACH_RAW_EASYCALC_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_EASYCALC_CART_NL, "Koppel binair 24kB Easy Calc Result cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_EASYCALC_CART_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ATTACH_RAW_EASYCALC_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_EASYCALC_CART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ATTACH_RAW_EASYCALC_CART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_PAGEFOX_CART,    N_("Attach raw 64KB Pagefox cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_DA, "Tilslut rÂt 64kB Pagefox cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_DE, "Pagefox Erweiterungsmodul Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_ES, "Insertar imagen cartucho 64KB Pagefox"},
	/* fr */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_FR, "Attacher une image de cartouche 64 ko Pagefox"},
	/* hu */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_IT, "Seleziona immagine cartuccia Pagefox di 64KB"},
	/* ko */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_NL, "Koppel binair 64KB Pagefox cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_PL, "Zamontuj surowy obraz kartridøa 64KB Pagefox"},
	/* ru */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_SV, "Anslut rÂ 64KB-Pagefox-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_PAGEFOX_CART_TR, "D¸z 64KB Pagefox kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_P64_CART,    N_("Attach raw 256KB Prophet 64 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_P64_CART_DA, "Tilslut rÂt 256kB Prophet 64 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_P64_CART_DE, "256KB Prophet 64 Erweiterungsmodul Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_P64_CART_ES, "Insertar imagen cartucho Prohet 64 256KB"},
	/* fr */ {IDCLS_ATTACH_RAW_P64_CART_FR, "Attacher une image de cartouche 256 ko Prophet 64"},
	/* hu */ {IDCLS_ATTACH_RAW_P64_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_P64_CART_IT, "Seleziona immagine cartuccia Prophet 64 di 256KB"},
	/* ko */ {IDCLS_ATTACH_RAW_P64_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_P64_CART_NL, "Koppel binair 256KB Prophet 64 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_P64_CART_PL, "Zamontuj surowy obraz kartridøa 256KB Prophet 64"},
	/* ru */ {IDCLS_ATTACH_RAW_P64_CART_RU, "Attach raw 256KB Prophet 64 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_P64_CART_SV, "Anslut rÂ 256KB Prophet 64-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_P64_CART_TR, "D¸z 256KB Prophet 64 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_RAMCART_CART,    N_("Attach raw RamCart cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_RAMCART_CART_DA, "Tilslut rÂt RamCart cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_RAMCART_CART_DE, "RamCart Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_RAMCART_CART_ES, "Insertar imagen cartucho RamCart"},
	/* fr */ {IDCLS_ATTACH_RAW_RAMCART_CART_FR, "Attacher une image de cartouche RAW RAMCART"},
	/* hu */ {IDCLS_ATTACH_RAW_RAMCART_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_RAMCART_CART_IT, "Seleziona immagine cartuccia RamCart"},
	/* ko */ {IDCLS_ATTACH_RAW_RAMCART_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_RAMCART_CART_NL, "Koppel binair RamCart cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_RAMCART_CART_PL, "Zamontuj surowy obraz kartridøa RamCart"},
	/* ru */ {IDCLS_ATTACH_RAW_RAMCART_CART_RU, "Attach raw RamCart cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_RAMCART_CART_SV, "Anslut rÂ RamCart-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_RAMCART_CART_TR, "D¸z RamCart kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_REU_CART,    N_("Attach raw REU cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_REU_CART_DA, "Tilslut rÂt REU cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_REU_CART_DE, "REU Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_REU_CART_ES, "Insertar imagen cartucho REU"},
	/* fr */ {IDCLS_ATTACH_RAW_REU_CART_FR, "Attacher une image de cartouche RAW REU"},
	/* hu */ {IDCLS_ATTACH_RAW_REU_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_REU_CART_IT, "Seleziona immagine cartuccia REU"},
	/* ko */ {IDCLS_ATTACH_RAW_REU_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_REU_CART_NL, "Koppel binair REU cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_REU_CART_PL, "Zamontuj surowy obraz kartridøa REU"},
	/* ru */ {IDCLS_ATTACH_RAW_REU_CART_RU, "Attach raw REU cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_REU_CART_SV, "Anslut rÂ REU-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_REU_CART_TR, "D¸z REU kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_REX_EP256_CART,    N_("Attach raw REX EP256 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_REX_EP256_CART_DA, "Tilslut rÂt REX EP256 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_REX_EP256_CART_DE, "REX EP256 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_REX_EP256_CART_ES, "Insertar imagen cartucho REX EP256"},
	/* fr */ {IDCLS_ATTACH_RAW_REX_EP256_CART_FR, "Attacher une image de cartouche REX EP256"},
	/* hu */ {IDCLS_ATTACH_RAW_REX_EP256_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_REX_EP256_CART_IT, "Seleziona immagine cartuccia REX EP256"},
	/* ko */ {IDCLS_ATTACH_RAW_REX_EP256_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_REX_EP256_CART_NL, "Koppel binair REX EP256 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_REX_EP256_CART_PL, "Zamontuj surowy obraz kartridøa REX EP256"},
	/* ru */ {IDCLS_ATTACH_RAW_REX_EP256_CART_RU, "Attach raw REX EP256 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_REX_EP256_CART_SV, "Anslut rÂ REX EP256-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_REX_EP256_CART_TR, "D¸z REX EP256 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_RGCD_CART,    N_("Attach raw 64kB RGCD cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_RGCD_CART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ATTACH_RAW_RGCD_CART_DE, "64kB RGBD Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_RGCD_CART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ATTACH_RAW_RGCD_CART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ATTACH_RAW_RGCD_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_RGCD_CART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ATTACH_RAW_RGCD_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_RGCD_CART_NL, "Koppel binair 64KB RGCD cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_RGCD_CART_PL, "Zamontuj surowy 64KB obraz kartridøa  RGCD"},
	/* ru */ {IDCLS_ATTACH_RAW_RGCD_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_RGCD_CART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ATTACH_RAW_RGCD_CART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_RRNETMK3_CART,    N_("Attach raw 8kB RR-Net MK3 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_DE, "8kB RR-Net MK3 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_NL, "Koppel binair 8kB RRNET MK3 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ATTACH_RAW_RRNETMK3_CART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_ROSS_CART,    N_("Attach raw 16/32kB ROSS cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_ROSS_CART_DA, "Tilslut rÂt 16/32kB ROSS cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_ROSS_CART_DE, "16/32kB ROSS Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_ROSS_CART_ES, "Insertar imagen cartucho ROSS 16/32KB"},
	/* fr */ {IDCLS_ATTACH_RAW_ROSS_CART_FR, "Attacher une image de cartouche 16/32 ko ROSS"},
	/* hu */ {IDCLS_ATTACH_RAW_ROSS_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_ROSS_CART_IT, "Seleziona immagine cartuccia ROSS di 16/32kB"},
	/* ko */ {IDCLS_ATTACH_RAW_ROSS_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_ROSS_CART_NL, "Koppel binair 16/32kB ROSS cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_ROSS_CART_PL, "Zamontuj surowy obraz kartridøa 12/32kB ROSS"},
	/* ru */ {IDCLS_ATTACH_RAW_ROSS_CART_RU, "Attach raw 16/32kB ROSS cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_ROSS_CART_SV, "Anslut rÂ 16/32kB ROSS-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_ROSS_CART_TR, "D¸z 16/32kB ROSS kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART,    N_("Attach raw 8KB Game Killer cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_DA, "Tilslut rÂt 8kB Game Killer cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_DE, "8KB Game Killer Erweiterungsmodul (raw) Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_ES, "Insertar imagen cartucho Game Killer 8KB"},
	/* fr */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_FR, "Attacher une image de cartouche 8 ko Game Killer"},
	/* hu */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_IT, "Seleziona immagine cartuccia Game Killer di 8KB"},
	/* ko */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_NL, "Koppel binair 8KB Game Killer cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_PL, "Zamontuj surowy obraz kartridøa 8KB Game Killer"},
	/* ru */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_RU, "Attach raw 8KB Game Killer cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_SV, "Anslut rÂ 8KB Game Killer-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_GAME_KILLER_CART_TR, "D¸z 8KB Game Killer kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_GEORAM_CART,    N_("Attach raw GEO-RAM cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_GEORAM_CART_DA, "Tilslut rÂt GEO-RAM cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_GEORAM_CART_DE, "GEO-RAM Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_GEORAM_CART_ES, "Insertar imagen cartucho GEO-RAM"},
	/* fr */ {IDCLS_ATTACH_RAW_GEORAM_CART_FR, "Attacher une image de cartouche RAW GEO-RAM"},
	/* hu */ {IDCLS_ATTACH_RAW_GEORAM_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_GEORAM_CART_IT, "Seleziona immagine cartuccia GEO-RAM"},
	/* ko */ {IDCLS_ATTACH_RAW_GEORAM_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_GEORAM_CART_NL, "Koppel binair GEO-RAM cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_GEORAM_CART_PL, "Zamontuj surowy obraz kartridøa GEO-RAM"},
	/* ru */ {IDCLS_ATTACH_RAW_GEORAM_CART_RU, "Attach raw GEO-RAM cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_GEORAM_CART_SV, "Anslut rÂ GEO-RAM-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_GEORAM_CART_TR, "D¸z GEO-RAM kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART,    N_("Attach raw 512kB Game System cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_DA, "Tilslut rÂt 512kB Game System cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_DE, "512kB Game System Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_ES, "Insertar imagen cartucho Game System 512KB"},
	/* fr */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_FR, "Attacher une image de cartouche 512 ko Game System"},
	/* hu */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_IT, "Seleziona immagine cartuccia Game System di 512kB"},
	/* ko */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_NL, "Koppel binair 512kB Game System cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_PL, "Zamontuj surowy obraz kartridøa 512kB Game System"},
	/* ru */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_RU, "Attach raw 512kB Game System cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_SV, "Anslut rÂ 512kB Game System-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_GAME_SYSTEM_CART_TR, "D¸z 512kB Game System kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c, c64/expert.c */
	/* en */ {IDCLS_ENABLE_EXPERT_CART,    N_("Enable the Expert Cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_EXPERT_CART_DA, "AktivÈr Expert Cartridge"},
	/* de */ {IDCLS_ENABLE_EXPERT_CART_DE, "Expert Cartridge aktivieren"},
	/* es */ {IDCLS_ENABLE_EXPERT_CART_ES, "Permitir Expert Cartridge"},
	/* fr */ {IDCLS_ENABLE_EXPERT_CART_FR, "Activer Cartouche Expert"},
	/* hu */ {IDCLS_ENABLE_EXPERT_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_EXPERT_CART_IT, "Attiva Expert Cartridge"},
	/* ko */ {IDCLS_ENABLE_EXPERT_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_EXPERT_CART_NL, "Expert Cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_EXPERT_CART_PL, "W≥±cz Expert Cartridge"},
	/* ru */ {IDCLS_ENABLE_EXPERT_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_EXPERT_CART_SV, "Aktivera Expert Cartridge"},
	/* tr */ {IDCLS_ENABLE_EXPERT_CART_TR, "Expert Kartu˛u'nu aktif et"},
#endif
	
	/* c64/expert.c */
	/* en */ {IDCLS_DISABLE_EXPERT_CART,    N_("Disable the Expert Cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_EXPERT_CART_DA, "DeaktivÈr Expert Cartridge"},
	/* de */ {IDCLS_DISABLE_EXPERT_CART_DE, "Expert Cartridge deaktivieren"},
	/* es */ {IDCLS_DISABLE_EXPERT_CART_ES, "Deshabilitar Expert Cartridge"},
	/* fr */ {IDCLS_DISABLE_EXPERT_CART_FR, "DÈsactiver la Cartouche Expert"},
	/* hu */ {IDCLS_DISABLE_EXPERT_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_EXPERT_CART_IT, "Disattiva Expert Cartridge"},
	/* ko */ {IDCLS_DISABLE_EXPERT_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_EXPERT_CART_NL, "Expert Cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_EXPERT_CART_PL, "Wy≥±cz Expert Cartridge"},
	/* ru */ {IDCLS_DISABLE_EXPERT_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_EXPERT_CART_SV, "Inaktivera Expert Cartridge"},
	/* tr */ {IDCLS_DISABLE_EXPERT_CART_TR, "Expert Kartu˛u'nu pasifle˛tir"},
#endif
	
	/* c64/expert.c */
	/* en */ {IDCLS_SET_EXPERT_FILENAME,    N_("Set Expert Cartridge image name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_EXPERT_FILENAME_DA, "VÊlg Expert Cartridge image-navn"},
	/* de */ {IDCLS_SET_EXPERT_FILENAME_DE, "Expert Cartridge Imagedateiname setzen"},
	/* es */ {IDCLS_SET_EXPERT_FILENAME_ES, "Seleccionar nombre imagen Expert Cartridge"},
	/* fr */ {IDCLS_SET_EXPERT_FILENAME_FR, "Choisir le nom de l'image de la Cartouche Expert"},
	/* hu */ {IDCLS_SET_EXPERT_FILENAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_EXPERT_FILENAME_IT, "Imposta nome immagine Expert Cartridge"},
	/* ko */ {IDCLS_SET_EXPERT_FILENAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_EXPERT_FILENAME_NL, "Stel Expert Cartridge bestandsnaam in"},
	/* pl */ {IDCLS_SET_EXPERT_FILENAME_PL, "Okre∂l nazwÍ obrazu Expert Cartridge"},
	/* ru */ {IDCLS_SET_EXPERT_FILENAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_EXPERT_FILENAME_SV, "V‰lj Expert Cartridge-modulfil"},
	/* tr */ {IDCLS_SET_EXPERT_FILENAME_TR, "Expert Kartu˛ imaj ismini belirle"},
#endif
	
	/* c64/expert.c */
	/* en */ {IDCLS_SET_EXPERT_MODE,    N_("Set Expert Cartridge mode (0: Off, 1: Prg, 2: On)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_EXPERT_MODE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_EXPERT_MODE_DE, "Setze Expert Cartridge Modus (0: Off, 1: Prg, 2: On)"},
	/* es */ {IDCLS_SET_EXPERT_MODE_ES, "Seleccionar modo Expert Cartridge (0: Apagado, 1: Prg, 2: Encendido)"},
	/* fr */ {IDCLS_SET_EXPERT_MODE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_EXPERT_MODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_EXPERT_MODE_IT, "Imposta modalit‡ Expert Cartridge (0: Off, 1: Prg, 2: On)"},
	/* ko */ {IDCLS_SET_EXPERT_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_EXPERT_MODE_NL, "Stel de Expert Cartridge modus in (0: uit, 1: prg, 2: aan)"},
	/* pl */ {IDCLS_SET_EXPERT_MODE_PL, "Wybierz tryb Expert Cartridge (0: Wy≥±czony, 1: Prg, 2: W≥±czony)"},
	/* ru */ {IDCLS_SET_EXPERT_MODE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_EXPERT_MODE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_EXPERT_MODE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/expert.c */
	/* en */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE,    N_("Allow writing to Expert Cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_DA, "Tillad skrivning til Expert Cartridge image"},
	/* de */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_DE, "Schreibzugriff auf Expert Cartridge Image erlauben"},
	/* es */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_ES, "Permitir grabaciÛn en Expert Cartridge"},
	/* fr */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_FR, "Permettre l'Ècriture sur l'image Cartouche Expert"},
	/* hu */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_IT, "Attiva scrittura su immagine Expert Cartridge"},
	/* ko */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_NL, "Sta schrijven naar Expert Cartridge bestand toe"},
	/* pl */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_PL, "PozwÛl na zapis do obrazu Expert Cartridge"},
	/* ru */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_RU, "Allow writing to Expert Cartridge image"},
	/* sv */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_SV, "TillÂt skrivning till Expert Cartridge-avbildning"},
	/* tr */ {IDCLS_ALLOW_WRITING_TO_EXPERT_IMAGE_TR, "Expert Kartu˛ imaj˝na yazmaya izin ver"},
#endif
	
	/* c64/expert.c */
	/* en */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE,    N_("Do not write to Expert Cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_DA, "Skriv ikke til Expert Cartridge image"},
	/* de */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_DE, "Kein Schreibzugriff auf Expert Cartridge Image"},
	/* es */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_ES, "Deshabilitar grabaciÛn en Expert Cartridge"},
	/* fr */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_FR, "NE PAS permettre l'Ècriture sur l'image de Cartouche Expert"},
	/* hu */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_IT, "Disattiva scrittura su immagine Expert Cartridge"},
	/* ko */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_NL, "Schrijf niet naar Expert Cartridge bestand"},
	/* pl */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_PL, "Nie zapisuj do obrazu Expert Cartridge"},
	/* ru */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_RU, "Do not write to Expert Cartridge image"},
	/* sv */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_SV, "Skriv inte till Expert Cartridge-avbildning"},
	/* tr */ {IDCLS_DO_NOT_WRITE_TO_EXPERT_IMAGE_TR, "Expert Kartu˛ imaj˝na yazma"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_DISABLE_CART,    N_("Disable default cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_CART_DA, "DeaktivÈr standard cartridge"},
	/* de */ {IDCLS_DISABLE_CART_DE, "Standard Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_CART_ES, "Deshabilitar cartucho por defecto"},
	/* fr */ {IDCLS_DISABLE_CART_FR, "DÈsactiver la cartouche par dÈfaut"},
	/* hu */ {IDCLS_DISABLE_CART_HU, "Alap cartridge tilt·sa"},
	/* it */ {IDCLS_DISABLE_CART_IT, "Disattiva cartuccia predefinita"},
	/* ko */ {IDCLS_DISABLE_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_CART_NL, "Standaard cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_CART_PL, "Wy≥±cz domy∂lny kartridø"},
	/* ru */ {IDCLS_DISABLE_CART_RU, "Disable default cartridge"},
	/* sv */ {IDCLS_DISABLE_CART_SV, "Inaktivera standardinsticksmodul"},
	/* tr */ {IDCLS_DISABLE_CART_TR, "Varsay˝lan kartu˛u pasifle˛tir"},
#endif
	
	/* plus4/plus4cart.c */
	/* en */ {IDCLS_SMART_ATTACH_CART,    N_("Smart-attach cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SMART_ATTACH_CART_DA, "Smart-tilslut cartridge-image"},
	/* de */ {IDCLS_SMART_ATTACH_CART_DE, "Smart-Einlegen Modul Image"},
	/* es */ {IDCLS_SMART_ATTACH_CART_ES, "Insertar imagen cartucho inteligentemente"},
	/* fr */ {IDCLS_SMART_ATTACH_CART_FR, "Insertion intelligente d'une image de cartouche"},
	/* hu */ {IDCLS_SMART_ATTACH_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SMART_ATTACH_CART_IT, "Seleziona una cartuccia"},
	/* ko */ {IDCLS_SMART_ATTACH_CART_KO, "ƒ´∆Æ∏Æ¡ˆ ¿ÃπÃ¡ˆ∏¶ Ω∫∏∂∆Æ-∫Ÿ¿Ã±‚ «œ±‚"},
	/* nl */ {IDCLS_SMART_ATTACH_CART_NL, "Slim-koppel cartridge bestand"},
	/* pl */ {IDCLS_SMART_ATTACH_CART_PL, "Inteligentnie zamontuj obraz kartridøa"},
	/* ru */ {IDCLS_SMART_ATTACH_CART_RU, "Smart-attach cartridge image"},
	/* sv */ {IDCLS_SMART_ATTACH_CART_SV, "Smartanslut insticksmodulfil"},
	/* tr */ {IDCLS_SMART_ATTACH_CART_TR, "Ak˝ll˝ kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_SPECIFY_IDE64_NAME,    N_("Specify name of IDE64 image file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_IDE64_NAME_DA, "Angiv navn pÂ IDE64-cartridge image"},
	/* de */ {IDCLS_SPECIFY_IDE64_NAME_DE, "Namen f¸r IDE64 Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_IDE64_NAME_ES, "Especificar nombre fichero imagen IDE64"},
	/* fr */ {IDCLS_SPECIFY_IDE64_NAME_FR, "SpÈcifier le nom de l'image IDE64"},
	/* hu */ {IDCLS_SPECIFY_IDE64_NAME_HU, "Adja meg az IDE64 kÈpm·s f·jlnevÈt"},
	/* it */ {IDCLS_SPECIFY_IDE64_NAME_IT, "Specifica il nome del file immagine dell'IDE64"},
	/* ko */ {IDCLS_SPECIFY_IDE64_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_IDE64_NAME_NL, "Geef de naam van het IDE64 bestand"},
	/* pl */ {IDCLS_SPECIFY_IDE64_NAME_PL, "Okre∂l nazwÍ pliku obrazu IDE64"},
	/* ru */ {IDCLS_SPECIFY_IDE64_NAME_RU, "Specify name of IDE64 image file"},
	/* sv */ {IDCLS_SPECIFY_IDE64_NAME_SV, "Ange namn pÂ IDE64-insticksmodulfil"},
	/* tr */ {IDCLS_SPECIFY_IDE64_NAME_TR, "IDE64 imaj dosyas˝n˝n ismini belirt"},
#endif
	
	/* c64/cart/ide64.c, gfxoutputdrv/ffmpegdrv.c, datasette.c,
	 debug.c, mouse.c, ram.c, sound.c, vsync.c */
	/* en */ {IDCLS_P_VALUE,    N_("<value>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_VALUE_DA, "<vÊrdi>"},
	/* de */ {IDCLS_P_VALUE_DE, "<Wert>"},
	/* es */ {IDCLS_P_VALUE_ES, "<valor>"},
	/* fr */ {IDCLS_P_VALUE_FR, "<valeur>"},
	/* hu */ {IDCLS_P_VALUE_HU, "<ÈrtÈk>"},
	/* it */ {IDCLS_P_VALUE_IT, "<valore>"},
	/* ko */ {IDCLS_P_VALUE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_VALUE_NL, "<waarde>"},
	/* pl */ {IDCLS_P_VALUE_PL, "<warto∂Ê>"},
	/* ru */ {IDCLS_P_VALUE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_VALUE_SV, "<v‰rde>"},
	/* tr */ {IDCLS_P_VALUE_TR, "<deer>"},
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64,    N_("Set number of cylinders for the IDE64 emulation. (1..65535)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_DE, "Anzahl der Zylinder f¸r IDE64 Emulation. (1..65535)"},
	/* es */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_ES, "Seleccionar n˙mero de cilindros para emulaciÛn IDE64 (1..65535)"},
	/* fr */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_IT, "Imposta il numero di cilindri dell'emulazione IDE64. (1..65535)"},
	/* ko */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_NL, "Stel het aantal cylinders voor de IDE64 emulatie in. (1..65535)"},
	/* pl */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_PL, "Okre∂l liczbÍ cylindrÛw w emulacji IDE64. (1..65535)"},
	/* ru */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_AMOUNT_CYLINDERS_IDE64_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_SET_AMOUNT_HEADS_IDE64,    N_("Set number of heads for the IDE64 emulation. (1..16)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_AMOUNT_HEADS_IDE64_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_AMOUNT_HEADS_IDE64_DE, "Anzahl der Laufwerkskˆpfe f¸r IDE64 Emulation. (1..16)"},
	/* es */ {IDCLS_SET_AMOUNT_HEADS_IDE64_ES, "Seleccionar n˙mero de cabezas para emulaciÛn IDE64 (1..16)"},
	/* fr */ {IDCLS_SET_AMOUNT_HEADS_IDE64_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_AMOUNT_HEADS_IDE64_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_AMOUNT_HEADS_IDE64_IT, "Imposta il numero di testine dell'emulazione IDE64. (1..16)"},
	/* ko */ {IDCLS_SET_AMOUNT_HEADS_IDE64_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_AMOUNT_HEADS_IDE64_NL, "Stel het aantal koppen voor de IDE64 emulatie in. (1..16)"},
	/* pl */ {IDCLS_SET_AMOUNT_HEADS_IDE64_PL, "Okre∂l liczbÍ g≥owic w emulacji IDE64. (1..16)"},
	/* ru */ {IDCLS_SET_AMOUNT_HEADS_IDE64_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_AMOUNT_HEADS_IDE64_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_AMOUNT_HEADS_IDE64_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_SET_AMOUNT_SECTORS_IDE64,    N_("Set number of sectors for the IDE64 emulation. (1..63)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_DE, "Anzahl der Sektoren f¸r IDE64 Emulation. (1..63)"},
	/* es */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_ES, "Seleccionar n˙mero de sectores para emulaciÛn IDE64 (1..63)"},
	/* fr */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_IT, "Imposta il numero di settori dell'emulazione IDE64. (1..63)"},
	/* ko */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_NL, "Stel het aantal sectors voor de IDE64 emulatie in. (1..63)"},
	/* pl */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_PL, "Okre∂l liczbÍ sektorÛw w emulacji IDE64. (1..63)"},
	/* ru */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_AMOUNT_SECTORS_IDE64_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_AUTODETECT_IDE64_GEOMETRY,    N_("Autodetect image size")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_DA, "AutodetektÈr st¯rrelse pÂ image"},
	/* de */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_DE, "Image Grˆﬂe automatisch erkennen"},
	/* es */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_ES, "AutodetecciÛn del tamaÒo de la imagen"},
	/* fr */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_FR, "AutodÈtection de la taille de l'image"},
	/* hu */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_IT, "Rileva automaticamente immagine"},
	/* ko */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_KO, "¿ÃπÃ¡ˆ ≈©±‚ ¿⁄µø¿∏∑Œ √£±‚"},
	/* nl */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_NL, "Automatisch detecteren bestandsgrootte"},
	/* pl */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_PL, "Automatycznie wykrywaj rozmiar obrazu"},
	/* ru */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_RU, "∞“‚ﬁ‹–‚ÿÁ’·⁄ﬁ’ ﬁﬂ‡’‘’€’›ÿ’ ‡–◊‹’‡– ÿ◊ﬁ—‡–÷’›ÿÔ"},
	/* sv */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_SV, "Autodetektera storlek pÂ avbildning"},
	/* tr */ {IDCLS_AUTODETECT_IDE64_GEOMETRY_TR, "Imaj boyutunu otomatik alg˝la"},
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY,    N_("Do not autodetect geometry of formatted images")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_DA, "AutodetektÈr ikke geometri fra formatterede images"},
	/* de */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_DE, "Geometrie von formatierten Imagedateien nicht automatisch erkennen"},
	/* es */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_ES, "No autodetectar formato geometrÌa im·genes"},
	/* fr */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_FR, "Ne pas dÈtecter automatiquement la gÈomÈtrie des images formattÈes"},
	/* hu */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_HU, "Form·zott kÈpm·sok geometri·j·t ne ismerje fel automatikusan"},
	/* it */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_IT, "Non rilevare automaticamente la geometrica delle immagini formattate"},
	/* ko */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_NL, "Niet automatisch de grootte van de geformatteerde bestanden detecteren"},
	/* pl */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_PL, "Nie wykrywaj geometrii sformatowanych obrazÛw"},
	/* ru */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_RU, "Do not autodetect geometry of formatted images"},
	/* sv */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_SV, "Autodetektera inte geometri frÂn formaterad avbildningsfil"},
	/* tr */ {IDCLS_NO_AUTODETECT_IDE64_GEOMETRY_TR, "Formatlanm˝˛ imajlar˝n yap˝lar˝n˝ otomatik alg˝lama"},
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_IDE64_VERSION,    N_("IDE64 cartridge version")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_IDE64_VERSION_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_IDE64_VERSION_DE, "IDE64 Modul Version"},
	/* es */ {IDCLS_IDE64_VERSION_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_IDE64_VERSION_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_IDE64_VERSION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_IDE64_VERSION_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_IDE64_VERSION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_IDE64_VERSION_NL, "IDE64 cartridge versie"},
	/* pl */ {IDCLS_IDE64_VERSION_PL, "Wersja kartridøa IDE64"},
	/* ru */ {IDCLS_IDE64_VERSION_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_IDE64_VERSION_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_IDE64_VERSION_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_ENABLE_IDE64_USB_SERVER,    N_("Enable IDE64 USB server")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IDE64_USB_SERVER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_IDE64_USB_SERVER_DE, "IDE64 USB Server aktivieren"},
	/* es */ {IDCLS_ENABLE_IDE64_USB_SERVER_ES, "Permitir IDE64 USB server"},
	/* fr */ {IDCLS_ENABLE_IDE64_USB_SERVER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_IDE64_USB_SERVER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_IDE64_USB_SERVER_IT, "Attiva server IDE64 USB"},
	/* ko */ {IDCLS_ENABLE_IDE64_USB_SERVER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IDE64_USB_SERVER_NL, "IDE64 USB server inschakelen"},
	/* pl */ {IDCLS_ENABLE_IDE64_USB_SERVER_PL, "W≥±cz serwer USB IDE64"},
	/* ru */ {IDCLS_ENABLE_IDE64_USB_SERVER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_IDE64_USB_SERVER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_IDE64_USB_SERVER_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_DISABLE_IDE64_USB_SERVER,    N_("Disable IDE64 USB server")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IDE64_USB_SERVER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_IDE64_USB_SERVER_DE, "IDE64 USB Server deaktivieren"},
	/* es */ {IDCLS_DISABLE_IDE64_USB_SERVER_ES, "Deshabilitar servidor IDE64 USB"},
	/* fr */ {IDCLS_DISABLE_IDE64_USB_SERVER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_IDE64_USB_SERVER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_IDE64_USB_SERVER_IT, "Disattiva server IDE64 USB"},
	/* ko */ {IDCLS_DISABLE_IDE64_USB_SERVER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IDE64_USB_SERVER_NL, "IDE64 USB server uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IDE64_USB_SERVER_PL, "Wy≥±cz serwer USB IDE64"},
	/* ru */ {IDCLS_DISABLE_IDE64_USB_SERVER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_IDE64_USB_SERVER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_IDE64_USB_SERVER_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_IDE64_USB_SERVER_ADDRESS,    N_("IDE64 USB server address")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_IDE64_USB_SERVER_ADDRESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_IDE64_USB_SERVER_ADDRESS_DE, "IDE64 USB Server Adresse"},
	/* es */ {IDCLS_IDE64_USB_SERVER_ADDRESS_ES, "Seleccionar direcciÛn servidor IDE64 USB"},
	/* fr */ {IDCLS_IDE64_USB_SERVER_ADDRESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_IDE64_USB_SERVER_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_IDE64_USB_SERVER_ADDRESS_IT, "Indirizzo server IDE64 USB"},
	/* ko */ {IDCLS_IDE64_USB_SERVER_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_IDE64_USB_SERVER_ADDRESS_NL, "IDE64 USB serveradres"},
	/* pl */ {IDCLS_IDE64_USB_SERVER_ADDRESS_PL, "Adres serwera USB IDE64"},
	/* ru */ {IDCLS_IDE64_USB_SERVER_ADDRESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_IDE64_USB_SERVER_ADDRESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_IDE64_USB_SERVER_ADDRESS_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_ENABLE_IDE64_RTC_SAVE,    N_("Enable saving of IDE64 RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IDE64_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_IDE64_RTC_SAVE_DE, "IDE64 RTC Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_IDE64_RTC_SAVE_ES, "Permitir grabar datos IDE64 RTC cuando cambien"},
	/* fr */ {IDCLS_ENABLE_IDE64_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_IDE64_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_IDE64_RTC_SAVE_IT, "Attiva il salvataggio dei dati RTC dell'IDE64 alla modifica."},
	/* ko */ {IDCLS_ENABLE_IDE64_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IDE64_RTC_SAVE_NL, "Opslaan van de IDE64 RTC gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_IDE64_RTC_SAVE_PL, "Zapisuj dane zegara czasu rzeczywistego IDE64 przy zmianie"},
	/* ru */ {IDCLS_ENABLE_IDE64_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_IDE64_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_IDE64_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ide64.c */
	/* en */ {IDCLS_DISABLE_IDE64_RTC_SAVE,    N_("Disable saving of IDE64 RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IDE64_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_IDE64_RTC_SAVE_DE, "IDE64 RTC Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_IDE64_RTC_SAVE_ES, "Deshabilitar grabar datos IDE64 RTC cuando cambien"},
	/* fr */ {IDCLS_DISABLE_IDE64_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_IDE64_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_IDE64_RTC_SAVE_IT, "Disattiva il salvataggio dei dati RTC dell'IDE64 alla modifica."},
	/* ko */ {IDCLS_DISABLE_IDE64_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IDE64_RTC_SAVE_NL, "Opslaan van de IDE64 RTC gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_IDE64_RTC_SAVE_PL, "Nie zapisuj danych zegara czasu rzeczywistego IDE64 przy zmianie"},
	/* ru */ {IDCLS_DISABLE_IDE64_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_IDE64_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_IDE64_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* cbm2/cbm2-cmdline-options.c, pet/pet-cmdline-options.c */
	/* en */ {IDCLS_P_MODELNUMBER,    N_("<modelnumber>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_MODELNUMBER_DA, "<modelnummer>"},
	/* de */ {IDCLS_P_MODELNUMBER_DE, "<Modell Nummer>"},
	/* es */ {IDCLS_P_MODELNUMBER_ES, "<n˙mero modelo>"},
	/* fr */ {IDCLS_P_MODELNUMBER_FR, "<numerodemodele>"},
	/* hu */ {IDCLS_P_MODELNUMBER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_MODELNUMBER_IT, "<numero di modello>"},
	/* ko */ {IDCLS_P_MODELNUMBER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_MODELNUMBER_NL, "<modelnummer>"},
	/* pl */ {IDCLS_P_MODELNUMBER_PL, "<modelnumber>"},
	/* ru */ {IDCLS_P_MODELNUMBER_RU, "<modelnumber>"},
	/* sv */ {IDCLS_P_MODELNUMBER_SV, "<modellnummer>"},
	/* tr */ {IDCLS_P_MODELNUMBER_TR, "<modelnumaras˝>"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CBM2_MODEL,    N_("Specify CBM-II model to emulate. (610, 620, 620+, 710, 720, 720+)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CBM2_MODEL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_CBM2_MODEL_DE, "CBM-II Modell f¸r Emulation w‰hlen. (610, 620, 620+, 710, 720, 720+)"},
	/* es */ {IDCLS_SPECIFY_CBM2_MODEL_ES, "Especificar modelo CBM-II para emular. (610, 620, 620+, 710, 720, 720+)"},
	/* fr */ {IDCLS_SPECIFY_CBM2_MODEL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_CBM2_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_CBM2_MODEL_IT, "Specifica il modello di CBM II da emulare. (610, 620, 620+, 710, 720, 720+)"},
	/* ko */ {IDCLS_SPECIFY_CBM2_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CBM2_MODEL_NL, "Geef CBM-II model om te emuleren. (610, 620, 620+, 710, 720, 720+)"},
	/* pl */ {IDCLS_SPECIFY_CBM2_MODEL_PL, "Okre∂l emulowany model CBM-II. (610, 620, 620+, 710, 720, 720+)"},
	/* ru */ {IDCLS_SPECIFY_CBM2_MODEL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_CBM2_MODEL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_CBM2_MODEL_TR, ""},  /* fuzzy */
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CBM5X0_MODEL,    N_("Specify CBM-II model to emulate. (510)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CBM5X0_MODEL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_CBM5X0_MODEL_DE, "CBM-II Modell f¸r Emulation w‰hlen. (510)"},
	/* es */ {IDCLS_SPECIFY_CBM5X0_MODEL_ES, "Especificar modelo CBM-II para emular. (510)"},
	/* fr */ {IDCLS_SPECIFY_CBM5X0_MODEL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_CBM5X0_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_CBM5X0_MODEL_IT, "Specifica il modello di CBM II da emulare. (510)"},
	/* ko */ {IDCLS_SPECIFY_CBM5X0_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CBM5X0_MODEL_NL, "Geef CBM-II model om te emuleren. (510)"},
	/* pl */ {IDCLS_SPECIFY_CBM5X0_MODEL_PL, "Okre∂l emulowany model CBM-II. (510)"},
	/* ru */ {IDCLS_SPECIFY_CBM5X0_MODEL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_CBM5X0_MODEL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_CBM5X0_MODEL_TR, ""},  /* fuzzy */
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_TO_USE_VIC_II,    N_("Specify to use VIC-II")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TO_USE_VIC_II_DA, "Brug VIC-II"},
	/* de */ {IDCLS_SPECIFY_TO_USE_VIC_II_DE, "VIC-II Unterst¸tzung aktivieren"},
	/* es */ {IDCLS_SPECIFY_TO_USE_VIC_II_ES, "Especificar usar CBM-II"},
	/* fr */ {IDCLS_SPECIFY_TO_USE_VIC_II_FR, "SpÈcifier l'utilisation de VIC-II"},
	/* hu */ {IDCLS_SPECIFY_TO_USE_VIC_II_HU, "VIC-II haszn·lata"},
	/* it */ {IDCLS_SPECIFY_TO_USE_VIC_II_IT, "Specifica di utilizzare il VIC-II"},
	/* ko */ {IDCLS_SPECIFY_TO_USE_VIC_II_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TO_USE_VIC_II_NL, "Gebruik de VIC-II"},
	/* pl */ {IDCLS_SPECIFY_TO_USE_VIC_II_PL, "Okre∂l uøycie VIC-II"},
	/* ru */ {IDCLS_SPECIFY_TO_USE_VIC_II_RU, "Specify to use VIC-II"},
	/* sv */ {IDCLS_SPECIFY_TO_USE_VIC_II_SV, "Ange fˆr att anv‰nda VIC-II"},
	/* tr */ {IDCLS_SPECIFY_TO_USE_VIC_II_TR, "VIC-II kullan˝m˝ iÁin seÁin"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_TO_USE_CRTC,    N_("Specify to use CRTC")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TO_USE_CRTC_DA, "Brug CRTC"},
	/* de */ {IDCLS_SPECIFY_TO_USE_CRTC_DE, "CRTC Unterst¸tzung aktivieren"},
	/* es */ {IDCLS_SPECIFY_TO_USE_CRTC_ES, "Especificar usar CRT"},
	/* fr */ {IDCLS_SPECIFY_TO_USE_CRTC_FR, "SpÈcifier l'utilisation de CRTC"},
	/* hu */ {IDCLS_SPECIFY_TO_USE_CRTC_HU, "CRTC haszn·lata"},
	/* it */ {IDCLS_SPECIFY_TO_USE_CRTC_IT, "Specifica di utilizzare il CRTC"},
	/* ko */ {IDCLS_SPECIFY_TO_USE_CRTC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TO_USE_CRTC_NL, "Gebruik de CRTC"},
	/* pl */ {IDCLS_SPECIFY_TO_USE_CRTC_PL, "Okre∂l uøycie CRTC"},
	/* ru */ {IDCLS_SPECIFY_TO_USE_CRTC_RU, "Specify to use CRTC"},
	/* sv */ {IDCLS_SPECIFY_TO_USE_CRTC_SV, "Ange fˆr att anv‰nda CRTC"},
	/* tr */ {IDCLS_SPECIFY_TO_USE_CRTC_TR, "CRTC kullan˝m˝ iÁin seÁin"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_P_LINENUMBER,    N_("<linenumber>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_LINENUMBER_DA, "<linjenummer>"},
	/* de */ {IDCLS_P_LINENUMBER_DE, "<Zeilennummer>"},
	/* es */ {IDCLS_P_LINENUMBER_ES, "<n˙mero linea>"},
	/* fr */ {IDCLS_P_LINENUMBER_FR, "<numÈrodeligne>"},
	/* hu */ {IDCLS_P_LINENUMBER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_LINENUMBER_IT, "<numero di linea>"},
	/* ko */ {IDCLS_P_LINENUMBER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_LINENUMBER_NL, "<lijnnummer>"},
	/* pl */ {IDCLS_P_LINENUMBER_PL, "<linenumber>"},
	/* ru */ {IDCLS_P_LINENUMBER_RU, "<linenumber>"},
	/* sv */ {IDCLS_P_LINENUMBER_SV, "<linjenummer>"},
	/* tr */ {IDCLS_P_LINENUMBER_TR, "<sat˝rnumaras˝>"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE,    N_("Specify CBM-II model hardware (0=6x0, 1=7x0)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_DA, "Angiv hardware for CBM-II-model (0=6x0, 1=7x0)"},
	/* de */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_DE, "CBM-II Hardware Modell w‰hlen (0=6x0, 1=7x0)"},
	/* es */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_ES, "Especificar modelo hardware CBM-II (0=6x0, 1=7x0)"},
	/* fr */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_FR, "SpÈcifier le modËle hardware CBM-II (0=6x0, 1=7x0)"},
	/* hu */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_HU, "Adja meg a CBM-II hardver modellt (0=6x0, 1=7x0)"},
	/* it */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_IT, "Specifica il modello hardware del CBM-II (0=6x0, 1=7x0)"},
	/* ko */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_NL, "Geef CBM-II hardwaremodel (0=6x0, 1=7x0)"},
	/* pl */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_PL, "Okre∂l model sprzÍtu CBM-II (0=6x0, 1=7x0)"},
	/* ru */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_RU, "Specify CBM-II model hardware (0=6x0, 1=7x0)"},
	/* sv */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_SV, "Ange maskinvara fˆr CBM-II-modell (0=6x0, 1=7x0)"},
	/* tr */ {IDCLS_SPECIFY_CBM2_MODEL_HARDWARE_TR, "CBM-II modeli donan˝m˝n˝ belirt (0=6x0, 1=7x0)"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c, plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_P_RAMSIZE,    N_("<RAM size>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_RAMSIZE_DA, "<RAM-st¯rrelse>"},
	/* de */ {IDCLS_P_RAMSIZE_DE, "<RAM Grˆﬂe>"},
	/* es */ {IDCLS_P_RAMSIZE_ES, "<TamaÒo RAM>"},
	/* fr */ {IDCLS_P_RAMSIZE_FR, "<Taille de la RAM>"},
	/* hu */ {IDCLS_P_RAMSIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_RAMSIZE_IT, "<Dimensione RAM>"},
	/* ko */ {IDCLS_P_RAMSIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_RAMSIZE_NL, "<RAM grootte>"},
	/* pl */ {IDCLS_P_RAMSIZE_PL, "<Rozmiar RAM>"},
	/* ru */ {IDCLS_P_RAMSIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_RAMSIZE_SV, "<RAM-storlek>"},
	/* tr */ {IDCLS_P_RAMSIZE_TR, "<RAM boyutu>"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SIZE_OF_RAM,    N_("Specify size of RAM (64/128/256/512/1024 kByte)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SIZE_OF_RAM_DA, "Angiv st¯rrelse pÂ RAM (64/128/256/512/1024 kByte)"},
	/* de */ {IDCLS_SPECIFY_SIZE_OF_RAM_DE, "RAM Grˆﬂe w‰hlen (64/128/256/512/1024 kByte)"},
	/* es */ {IDCLS_SPECIFY_SIZE_OF_RAM_ES, "Especificar tamaÒo RAM  (64/128/256/512/1024 KB)"},
	/* fr */ {IDCLS_SPECIFY_SIZE_OF_RAM_FR, "SpÈcifier la taille de la RAM (64/128/256/512/1024 KO)"},
	/* hu */ {IDCLS_SPECIFY_SIZE_OF_RAM_HU, "Adja meg a RAM mÈretÈt (64/128/256/512/1024 kB·jt)"},
	/* it */ {IDCLS_SPECIFY_SIZE_OF_RAM_IT, "Specifica la dimensione della RAM (4/128/256/512/1024 kByte)"},
	/* ko */ {IDCLS_SPECIFY_SIZE_OF_RAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SIZE_OF_RAM_NL, "Geef geheugengrootte (64/128/256/512/1024 kByte)"},
	/* pl */ {IDCLS_SPECIFY_SIZE_OF_RAM_PL, "Okre∂l rozmiar RAM-u (64/128/256/512/1024 kBajty)"},
	/* ru */ {IDCLS_SPECIFY_SIZE_OF_RAM_RU, "Specify size of RAM (64/128/256/512/1024 kByte)"},
	/* sv */ {IDCLS_SPECIFY_SIZE_OF_RAM_SV, "Ange storlek pÂ RAM (64/128/256/512/1024 kByte)"},
	/* tr */ {IDCLS_SPECIFY_SIZE_OF_RAM_TR, "RAM boyutunu belirt (64/128/256/512/1024 kByte)"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2,    N_("Specify size of RAM (128/256/512/1024 kByte)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_DE, "RAM Grˆﬂe w‰hlen (128/256/512/1024 kByte)"},
	/* es */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_ES, "Especificar tamaÒo RAM  (128/256/512/1024 kByte)"},
	/* fr */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_IT, "Specifica la dimensione della RAM (128/256/512/1024 kByte)"},
	/* ko */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_NL, "Geef geheugengrootte (128/256/512/1024 kByte)"},
	/* pl */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_PL, "Okre∂l rozmiar RAM-u (128/256/512/1024 kB)"},
	/* ru */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_SIZE_OF_RAM_CBM2_TR, ""},  /* fuzzy */
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CART_ROM_1000_NAME,    N_("Specify name of cartridge ROM image for $1000")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_DA, "Angiv navn pÂ cartridge-ROM-image for $1000"},
	/* de */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_DE, "Name f¸r ROM Module Imagedatei ($1000)"},
	/* es */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_ES, "Especificar nombre imagen cartucho ROM para $1000"},
	/* fr */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_FR, "SpÈcifier le nom de líimage de cartouche ROM pour $1000"},
	/* hu */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_HU, "Adja meg a $1000 cÌm˚ cartridge ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_IT, "Specifica il nome immagine della cartuccia ROM a $1000"},
	/* ko */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_NL, "Geef de naam van het cartridge ROM bestand voor $1000"},
	/* pl */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_PL, "Okre∂l nazwÍ obrazu ROM kartridøa w $1000"},
	/* ru */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_RU, "Specify name of cartridge ROM image for $1000"},
	/* sv */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_SV, "Ange namn pÂ insticksmodul-ROM-avbildning fˆr $1000"},
	/* tr */ {IDCLS_SPECIFY_CART_ROM_1000_NAME_TR, "$1000 adresi iÁin kartu˛ ROM imaj ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CART_ROM_2000_NAME,    N_("Specify name of cartridge ROM image for $2000-$3fff")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_DA, "Angiv navn pÂ cartridge-ROM-image for $2000-3fff"},
	/* de */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_DE, "Name f¸r ROM Module Imagedatei ($2000-3fff)"},
	/* es */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_ES, "Especificar nombre imagen cartucho ROM para $2000-$3FFF"},
	/* fr */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_FR, "SpÈcifier le nom de líimage de cartouche ROM pour $2000-$3fff"},
	/* hu */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_HU, "Adja meg a $2000-$3fff cÌm˚ cartridge ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_IT, "Specifica il nome immagine della ROM della cartuccia per $2000-$3fff"},
	/* ko */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_NL, "Geef de naam van het cartridge ROM bestand voor $2000-$3fff"},
	/* pl */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_PL, "Okre∂l nazwÍ obrazu ROM kartridøa w $2000-$3fff"},
	/* ru */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_RU, "Specify name of cartridge ROM image for $2000-$3fff"},
	/* sv */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_SV, "Ange namn pÂ insticksmodul-ROM-avbildning fˆr $2000-3fff"},
	/* tr */ {IDCLS_SPECIFY_CART_ROM_2000_NAME_TR, "$2000-$3fff adres aral˝˝ iÁin kartu˛ ROM imaj ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CART_ROM_4000_NAME,    N_("Specify name of cartridge ROM image for $4000-$5fff")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_DA, "Angiv navn pÂ cartridge-ROM-image for $4000-$5fff"},
	/* de */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_DE, "Name f¸r ROM Module Imagedatei ($4000-5fff)"},
	/* es */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_ES, "Especificar nombre imagen cartucho ROM para $4000-$5FFF"},
	/* fr */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_FR, "SpÈcifier le nom de líimage de cartouche ROM pour $4000-$5fff"},
	/* hu */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_HU, "Adja meg a $4000-$5fff cÌm˚ cartridge ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_IT, "Specifica il nome immagine della ROM della cartuccia per $4000-$5fff"},
	/* ko */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_NL, "Geef de naam van het cartridge ROM bestand voor $4000-$5fff"},
	/* pl */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_PL, "Okre∂l nazwÍ obrazu ROM kartridøa w $4000-$5fff"},
	/* ru */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_RU, "Specify name of cartridge ROM image for $4000-$5fff"},
	/* sv */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_SV, "Ange namn pÂ insticksmodul-ROM-avbildning fˆr $4000-$5fff"},
	/* tr */ {IDCLS_SPECIFY_CART_ROM_4000_NAME_TR, "$4000-$5fff adres aral˝˝ iÁin kartu˛ ROM imaj ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CART_ROM_6000_NAME,    N_("Specify name of cartridge ROM image for $6000-$7fff")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_DA, "Angiv navn pÂ cartridge-ROM-image for $6000-$7fff"},
	/* de */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_DE, "Name f¸r ROM Module Imagedatei ($6000-7fff)"},
	/* es */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_ES, "Especificar nombre imagen cartucho ROM para $6000-$7FFF"},
	/* fr */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_FR, "SpÈcifier le nom de líimage de cartouche ROM pour $6000-$7fff"},
	/* hu */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_HU, "Adja meg a $6000-$7fff cÌm˚ cartridge ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_IT, "Specifica il nome immagine della ROM della cartuccia per $6000-$7fff"},
	/* ko */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_NL, "Geef de naam van het cartridge ROM bestand voor $6000-$7fff"},
	/* pl */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_PL, "Okre∂l nazwÍ obrazu ROM kartridøa w $6000-$7fff"},
	/* ru */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_RU, "Specify name of cartridge ROM image for $6000-$7fff"},
	/* sv */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_SV, "Ange namn pÂ insticksmodul-ROM-avbildning fˆr $6000-$7fff"},
	/* tr */ {IDCLS_SPECIFY_CART_ROM_6000_NAME_TR, "$6000-$7fff adres aral˝˝ iÁin kartu˛ ROM imaj ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800,    N_("Enable RAM mapping in $0800-$0FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_DA, "AktivÈr RAM-mappning pÂ $0800-$0FFF"},
	/* de */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_DE, "RAM Zuordnung in $0800-$0FFF aktivieren"},
	/* es */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_ES, "Permitir mapeado RAM en $0800-$0FFF"},
	/* fr */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_FR, "Activer le mappage RAM dans $0800-$0FFF"},
	/* hu */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_HU, "$0800-$0FFF RAM lekÈpzÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_IT, "Attiva mappatura della RAM a $0800-$0FFF"},
	/* ko */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_NL, "RAM op adres $0800-$0FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_PL, "W≥±cz mapowanie RAM w $0800-$0FFF"},
	/* ru */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_RU, "Enable RAM mapping in $0800-$0FFF"},
	/* sv */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_SV, "Aktivera RAM-mappning pÂ $0800-$0FFF"},
	/* tr */ {IDCLS_ENABLE_RAM_MAPPING_IN_0800_TR, "$0800-$0FFF aral˝˝nda RAM e˛le˛tirmeyi aktif et"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800,    N_("Disable RAM mapping in $0800-$0FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_DA, "DeaktivÈr RAM-mappning pÂ $0800-$0FFF"},
	/* de */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_DE, "RAM Zuordnung in $0800-$0FFF deaktivieren"},
	/* es */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_ES, "Deshabilitar mapeado RAM en $0800-$0FFF"},
	/* fr */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_FR, "DÈsactiver le mappage RAM dans $0800-$0FFF"},
	/* hu */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_HU, "$0800-$0FFF RAM lekÈpzÈs tilt·sa"},
	/* it */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_IT, "Disattiva mappatura della RAM a $0800-$0FFF"},
	/* ko */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_NL, "RAM op adres $0800-$0FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_PL, "Wy≥±cz mapowanie RAM w $0800-$0FFF"},
	/* ru */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_RU, "Disable RAM mapping in $0800-$0FFF"},
	/* sv */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_SV, "Inaktivera RAM-mappning pÂ $0800-$0FFF"},
	/* tr */ {IDCLS_DISABLE_RAM_MAPPING_IN_0800_TR, "$0800-$0FFF aral˝˝nda RAM e˛le˛tirmeyi pasifle˛tir"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000,    N_("Enable RAM mapping in $1000-$1FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_DA, "AktivÈr RAM-mappning pÂ $1000-$1FFF"},
	/* de */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_DE, "RAM Zuordnung in $1000-$1FFF aktivieren"},
	/* es */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_ES, "Permitir mapeado RAM en $1000-$1FFF"},
	/* fr */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_FR, "Activer le mappage RAM dans $1000-$1FFF"},
	/* hu */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_HU, "$1000-$1FFF RAM lekÈpzÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_IT, "Attiva mappatura della RAM a $1000-$1FFF"},
	/* ko */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_NL, "RAM op adres $1000-$1FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_PL, "W≥±cz mapowanie RAM w $1000-$1FFF"},
	/* ru */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_RU, "Enable RAM mapping in $1000-$1FFF"},
	/* sv */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_SV, "Aktivera RAM-mappning pÂ $1000-$1FFF"},
	/* tr */ {IDCLS_ENABLE_RAM_MAPPING_IN_1000_TR, "$1000-$1FFF aral˝˝nda RAM e˛le˛tirmeyi aktif et"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000,    N_("Disable RAM mapping in $1000-$1FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_DA, "DeaktivÈr RAM-mappning pÂ $1000-$1FFF"},
	/* de */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_DE, "RAM Zuordnung in $1000-$1FFF deaktivieren"},
	/* es */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_ES, "Deshabilitar mapeado RAM en $1000-$1FFF"},
	/* fr */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_FR, "DÈsactiver le mappage RAM dans $1000-$1FFF"},
	/* hu */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_HU, "$1000-$1FFF RAM lekÈpzÈs tilt·sa"},
	/* it */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_IT, "Disattiva mappatura della RAM a $1000-$1FFF"},
	/* ko */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_NL, "RAM op adres $1000-$1FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_PL, "Wy≥±cz mapowanie RAM w $1000-$1FFF"},
	/* ru */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_RU, "Disable RAM mapping in $1000-$1FFF"},
	/* sv */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_SV, "Inaktivera RAM-mappning pÂ $1000-$1FFF"},
	/* tr */ {IDCLS_DISABLE_RAM_MAPPING_IN_1000_TR, "$1000-$1FFF aral˝˝nda RAM e˛le˛tirmeyi pasifle˛tir"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000,    N_("Enable RAM mapping in $2000-$3FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_DA, "AktivÈr RAM-mappning pÂ $2000-$3FFF"},
	/* de */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_DE, "RAM Zuordnung in $2000-3FFF aktivieren"},
	/* es */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_ES, "Permitir mapeado RAM en $2000-$3FFF"},
	/* fr */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_FR, "Activer le mappage RAM dans $2000-$3FFF"},
	/* hu */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_HU, "$2000-$3FFF RAM lekÈpzÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_IT, "Attiva mappatura della RAM a $2000-$3FFF"},
	/* ko */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_NL, "RAM op adres $2000-$3FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_PL, "W≥±cz mapowanie RAM w $2000-$3FFF"},
	/* ru */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_RU, "Enable RAM mapping in $2000-$3FFF"},
	/* sv */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_SV, "Aktivera RAM-mappning pÂ $2000-$3FFF"},
	/* tr */ {IDCLS_ENABLE_RAM_MAPPING_IN_2000_TR, "$2000-$3FFF aral˝˝nda RAM e˛le˛tirmeyi aktif et"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000,    N_("Disable RAM mapping in $2000-$3FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_DA, "DeaktivÈr RAM-mappning pÂ $2000-$3FFF"},
	/* de */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_DE, "RAM Zuordnung in $2000-$3FFF deaktivieren"},
	/* es */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_ES, "Deshabilitar mapeado RAM en $2000-$3FFF"},
	/* fr */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_FR, "DÈsactiver le mappage RAM dans $2000-$3FFF"},
	/* hu */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_HU, "$2000-$3FFF RAM lekÈpzÈs tilt·sa"},
	/* it */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_IT, "Disattiva mappatura della RAM a $2000-$3FFF"},
	/* ko */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_NL, "RAM op adres $2000-$3FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_PL, "Wy≥±cz mapowanie RAM w $2000-$3FFF"},
	/* ru */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_RU, "Disable RAM mapping in $2000-$3FFF"},
	/* sv */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_SV, "Inaktivera RAM-mappning pÂ $2000-$3FFF"},
	/* tr */ {IDCLS_DISABLE_RAM_MAPPING_IN_2000_TR, "$2000-$3FFF aral˝˝nda RAM e˛le˛tirmeyi pasifle˛tir"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000,    N_("Enable RAM mapping in $4000-$5FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_DA, "AktivÈr RAM-mappning pÂ $4000-$5FFF"},
	/* de */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_DE, "RAM Zuordnung in $4000-$5FFF aktivieren"},
	/* es */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_ES, "Permitir mapeado RAM en $4000-$5FFF"},
	/* fr */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_FR, "Activer le mappage RAM dans $4000-$5FFF"},
	/* hu */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_HU, "$4000-$5FFF RAM lekÈpzÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_IT, "Attiva mappatura della RAM a $4000-$5FFF"},
	/* ko */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_NL, "RAM op adres $4000-$5FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_PL, "W≥±cz mapowanie RAM w $4000-$5FFF"},
	/* ru */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_RU, "Enable RAM mapping in $4000-$5FFF"},
	/* sv */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_SV, "Aktivera RAM-mappning pÂ $4000-$5FFF"},
	/* tr */ {IDCLS_ENABLE_RAM_MAPPING_IN_4000_TR, "$4000-$5FFF aral˝˝nda RAM e˛le˛tirmeyi aktif et"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000,    N_("Disable RAM mapping in $4000-$5FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_DA, "DeaktivÈr RAM-mappning pÂ $4000-$5FFF"},
	/* de */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_DE, "RAM Zuordnung in $4000-$5FFF deaktivieren"},
	/* es */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_ES, "Deshabilitar mapeado RAM en $4000-$5FFF"},
	/* fr */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_FR, "DÈsactiver le mappage RAM dans $4000-$5FFF"},
	/* hu */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_HU, "$4000-$5FFF RAM lekÈpzÈs tilt·sa"},
	/* it */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_IT, "Disattiva mappatura della RAM a $4000-$5FFF"},
	/* ko */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_NL, "RAM op adres $4000-$5FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_PL, "Wy≥±cz mapowanie RAM w $4000-$5FFF"},
	/* ru */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_RU, "Disable RAM mapping in $4000-$5FFF"},
	/* sv */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_SV, "Inaktivera RAM-mappning pÂ $4000-$5FFF"},
	/* tr */ {IDCLS_DISABLE_RAM_MAPPING_IN_4000_TR, "$4000-$5FFF aral˝˝nda RAM e˛le˛tirmeyi pasifle˛tir"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000,    N_("Enable RAM mapping in $6000-$7FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_DA, "AktivÈr RAM-mappning pÂ $6000-$7FFF"},
	/* de */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_DE, "RAM Zuordnung in $6000-$7FFF aktivieren"},
	/* es */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_ES, "Permitir mapeado RAM en $6000-$7FFF"},
	/* fr */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_FR, "Activer le mappage RAM dans $6000-$7FFF"},
	/* hu */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_HU, "$6000-$7FFF RAM lekÈpzÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_IT, "Attiva mappatura della RAM a $6000-$7FFF"},
	/* ko */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_NL, "RAM op adres $6000-$7FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_PL, "W≥±cz mapowanie RAM w $6000-$7FFF"},
	/* ru */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_RU, "Enable RAM mapping in $6000-$7FFF"},
	/* sv */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_SV, "Aktivera RAM-mappning pÂ $6000-$7FFF"},
	/* tr */ {IDCLS_ENABLE_RAM_MAPPING_IN_6000_TR, "$6000-$7FFF aral˝˝nda RAM e˛le˛tirmeyi aktif et"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000,    N_("Disable RAM mapping in $6000-$7FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_DA, "DeaktivÈr RAM-mappning pÂ $6000-$7FFF"},
	/* de */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_DE, "RAM Zuordnung in $6000-$7FFF deaktivieren"},
	/* es */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_ES, "Deshabilitar mapeado RAM en $6000-$7FFF"},
	/* fr */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_FR, "DÈsactiver le mappage RAM dans $6000-$7FFF"},
	/* hu */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_HU, "$6000-$7FFF RAM lekÈpzÈs tilt·sa"},
	/* it */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_IT, "Disattiva mappatura della RAM a $6000-$7FFF"},
	/* ko */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_NL, "RAM op adres $6000-$7FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_PL, "Wy≥±cz mapowanie RAM w $6000-$7FFF"},
	/* ru */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_RU, "Disable RAM mapping in $6000-$7FFF"},
	/* sv */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_SV, "Inaktivera RAM-mappning pÂ $6000-$7FFF"},
	/* tr */ {IDCLS_DISABLE_RAM_MAPPING_IN_6000_TR, "$6000-$7FFF aral˝˝nda RAM e˛le˛tirmeyi pasifle˛tir"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000,    N_("Enable RAM mapping in $C000-$CFFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_DA, "AktivÈr RAM-mappning pÂ $C000-$CFFF"},
	/* de */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_DE, "RAM Zuordnung in $C000-$CFFF aktivieren"},
	/* es */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_ES, "Permitir mapeado RAM en $C000-$CFFF"},
	/* fr */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_FR, "Activer le mappage RAM dans $C000-$CFFF"},
	/* hu */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_HU, "$C000-$CFFF RAM lekÈpzÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_IT, "Attiva mappatura della RAM a $C000-$CFFF"},
	/* ko */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_NL, "RAM op adres $C000-$CFFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_PL, "W≥±cz mapowanie RAM w $C000-$CFFF"},
	/* ru */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_RU, "Enable RAM mapping in $C000-$CFFF"},
	/* sv */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_SV, "Aktivera RAM-mappning pÂ $C000-$CFFF"},
	/* tr */ {IDCLS_ENABLE_RAM_MAPPING_IN_C000_TR, "$C000-$CFFF aral˝˝nda RAM e˛le˛tirmeyi aktif et"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000,    N_("Disable RAM mapping in $C000-$CFFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_DA, "DeaktivÈr RAM-mappning pÂ $C000-$CFFF"},
	/* de */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_DE, "RAM Zuordnung in $C000-$CFFF deaktivieren"},
	/* es */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_ES, "Deshabilitar mapeado RAM en $C000-$CFFF"},
	/* fr */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_FR, "DÈsactiver le mappage RAM dans $C000-$CFFF"},
	/* hu */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_HU, "$C000-$CFFF RAM lekÈpzÈs tilt·sa"},
	/* it */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_IT, "Disattiva mappatura della RAM a $C000-$CFFF"},
	/* ko */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_NL, "RAM op adres $C000-$CFFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_PL, "Wy≥±cz mapowanie RAM w $C000-$CFFF"},
	/* ru */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_RU, "Disable RAM mapping in $C000-$CFFF"},
	/* sv */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_SV, "Inaktivera RAM-mappning pÂ $C000-$CFFF"},
	/* tr */ {IDCLS_DISABLE_RAM_MAPPING_IN_C000_TR, "$C000-$CFFF aral˝˝nda RAM e˛le˛tirmeyi pasifle˛tir"},
#endif
	
#ifdef COMMON_KBD
	/* cbm2/cbm2-cmdline-options.c, pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_KEYMAP_INDEX,    N_("Specify index of keymap file (0: UK business symbolic, 1: UK business positional, 2: Graphics symbolic, 3: Graphics positional, 4: German business symbolic, 5: German business positional)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_KEYMAP_INDEX_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_KEYMAP_INDEX_DE, "Index der Keymap Datei w‰hlen (0: UK business symbolic, 1: UK business positional, 2: Graphics symbolic, 3: Graphics positional, 4: German business symbolic, 5: German business positional)"},
	/* es */ {IDCLS_SPECIFY_KEYMAP_INDEX_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_KEYMAP_INDEX_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_KEYMAP_INDEX_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_KEYMAP_INDEX_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_KEYMAP_INDEX_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_KEYMAP_INDEX_NL, "Geef index van het toetstoewijzingsbestand (0: UK business symbolisch, 1: UK business positioneel, 2: grafisch symbolisch, 3: grafisch positioneel, 4: Duits business symbolisch, 5: Duits business positioneel)"},
	/* pl */ {IDCLS_SPECIFY_KEYMAP_INDEX_PL, "Podaj indeks uk≥adu klawiatury (0: UK biznesowa - symbol, 1: UK biznesowa - pozycja, 2: Graficzna - symbol, 3: Graficzna - pozycja, 4: Niemiecka biznesowa - symbol, 5: Niemiecka biznesowa - pozycja)"},
	/* ru */ {IDCLS_SPECIFY_KEYMAP_INDEX_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_KEYMAP_INDEX_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_KEYMAP_INDEX_TR, ""},  /* fuzzy */
#endif
	
	/* cbm2/cbm2-cmdline-options.c, pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME,    N_("Specify name of graphics keyboard symbolic keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_DA, "Angiv fil for symbolsk tastaturemulering for \"graphics\"-tastatur"},
	/* de */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_DE, "Keymap Datei f¸r graphics keyboard symbolic w‰hlen"},
	/* es */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_ES, "Especificar nombre fichero gr·ficos teclado simbÛlico"},
	/* fr */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_FR, "SpÈcifier le nom du fichier de mappage clavier symbolique"},
	/* hu */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_HU, "Adja meg a grafikus Ès szimbolikus billenty˚zet lekÈpzÈsi f·jl nevÈt."},
	/* it */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_IT, "Specifica il nome del file della mappa simbolica della tastiera grafica"},
	/* ko */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_NL, "Geef de naam van het symbolisch toetstoewijzingsbestand voor het grafische toetsenbord"},
	/* pl */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_PL, "Okre∂l nazwÍ pliku symbolicznego uk≥adu klawiatury graficznej"},
	/* ru */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_RU, "Specify name of graphics keyboard symbolic keymap file"},
	/* sv */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_SV, "Ange fil fˆr symbolisk tangentbordsemulering fˆr \"graphics\"-tangentbord"},
	/* tr */ {IDCLS_SPECIFY_GFX_SYM_KEYMAP_NAME_TR, "Grafik klavyesi sembolik tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c, pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME,    N_("Specify name of graphics keyboard positional keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_DA, "Angiv fil for positionsbestemt tastaturemulering for \"graphics\"-tastatur"},
	/* de */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_DE, "Keymap Datei f¸r graphics keyboard positional w‰hlen"},
	/* es */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_ES, "Especificar nombre fichero gr·ficos teclado posicional"},
	/* fr */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_FR, "SpÈcifier le nom du fichier de mappage clavier positionnel"},
	/* hu */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_HU, "Adja meg a grafikus Ès pozÌciÛ szerinti billenty˚zet lekÈpzÈsi f·jl nevÈt."},
	/* it */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_IT, "Specifica il nome del file della mappa posizionale della tastiera grafica"},
	/* ko */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_NL, "Geef de naam van het positioneel toetstoewijzingsbestand voor het grafische toetsenbord"},
	/* pl */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_PL, "Okre∂l nazwÍ pliku pozycyjnego uk≥adu klawiatury graficznej"},
	/* ru */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_RU, "Specify name of graphics keyboard positional keymap file"},
	/* sv */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_SV, "Ange fil fˆr positionsriktig tangentbordsemulering fˆr \"graphics\"-tangentbord"},
	/* tr */ {IDCLS_SPECIFY_GFX_POS_KEYMAP_NAME_TR, "Grafik klavyesi konumsal tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c, pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME,    N_("Specify name of UK business keyboard symbolic keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_DA, "Angiv fil for symbolsk tastaturemulering for Britisk \"business\"-tastatur"},
	/* de */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_DE, "Keymap Datei f¸r UK business keyboard symbolic w‰hlen"},
	/* es */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_ES, "Especificar nombre fichero teclado UK business simbÛlico"},
	/* fr */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_FR, "SpÈcifier le nom du fichier de mappage clavier symbolique UK"},
	/* hu */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_HU, "Adja meg az angol, hivatalos, szimbolikus billenty˚zet lekÈpzÈsi f·jl nevÈt."},
	/* it */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_IT, "Specifica il nome del file della mappa simbolica della tastiera UK business"},
	/* ko */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_NL, "Geef de naam van het symbolisch toetstoewijzingsbestand voor het UK zakelijk toetsenbord"},
	/* pl */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_PL, "Okre∂l nazwÍ pliku symbolicznego uk≥adu angielskiej klawiatury biznesowej"},
	/* ru */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_RU, "Specify name of UK business keyboard symbolic keymap file"},
	/* sv */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_SV, "Ange fil fˆr symbolisk tangentbordsemulering fˆr brittiskt \"business\"-tangentbord"},
	/* tr */ {IDCLS_SPECIFY_BUK_SYM_KEYMAP_NAME_TR, "UK business klavyesi sembolik tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c, pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME,    N_("Specify name of UK business keyboard positional keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_DA, "Angiv fil for positionsbestemt tastaturemulering for Britisk \"business\"-tastatur"},
	/* de */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_DE, "Keymap Datei f¸r UK business keyboard positional w‰hlen"},
	/* es */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_ES, "Especificar nombre fichero teclado UK business posicional"},
	/* fr */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_FR, "SpÈcifier le nom du fichier de mappage clavier positionnel UK"},
	/* hu */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_HU, "Adja meg az angol, hivatalos, pozÌciÛ szerinti billenty˚zet lekÈpzÈsi f·jl nevÈt."},
	/* it */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_IT, "Specifica il nome del file della mappa posizionale della tastiera UK business"},
	/* ko */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_NL, "Geef de naam van het positioneel toetstoewijzingsbestand voor het UK zakelijk toetsenbord"},
	/* pl */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_PL, "Okre∂l nazwÍ pliku pozycyjnego uk≥adu angielskiej klawiatury biznesowej"},
	/* ru */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_RU, "Specify name of UK business keyboard positional keymap file"},
	/* sv */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_SV, "Ange fil fˆr positionsriktig tangentbordsemulering fˆr brittiskt \"business\"-tangentbord"},
	/* tr */ {IDCLS_SPECIFY_BUK_POS_KEYMAP_NAME_TR, "UK business klavyesi konumsal tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c, pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME,    N_("Specify name of German business keyboard symbolic keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_DA, "Angiv fil for symbolsk tastaturemulering for Tysk \"business\"-tastatur"},
	/* de */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_DE, "Keymap Datei f¸r German business keyboard symbolic w‰hlen"},
	/* es */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_ES, "Especificar nombre fichero teclado Alem·n business simbÛlico"},
	/* fr */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_FR, "SpÈcifier le nom du fichier de mappage clavier symbolique allemand"},
	/* hu */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_HU, "Adja meg a nÈmet, hivatalos, szimbolikus billenty˚zet lekÈpzÈsi f·jl nevÈt."},
	/* it */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_IT, "Specifica il nome del file della mappa simbolica della tastiera business tedesca"},
	/* ko */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_NL, "Geef de naam van het symbolisch toetstoewijzingsbestand voor het Duitse zakelijk toetsenbord"},
	/* pl */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_PL, "Okre∂l nazwÍ pliku symbolicznego uk≥adu niemieckiej klawiatury biznesowej"},
	/* ru */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_RU, "Specify name of German business keyboard symbolic keymap file"},
	/* sv */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_SV, "Ange fil fˆr symbolisk tangentbordsemulering fˆr tyskt \"business\"-tangentbord"},
	/* tr */ {IDCLS_SPECIFY_BDE_SYM_KEYMAP_NAME_TR, "Alman business klavyesi sembolik tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
	
	/* cbm2/cbm2-cmdline-options.c, pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME,    N_("Specify name of German business keyboard positional keymap file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_DA, "Angiv fil for positionsbestemt tastaturemulering for Tysk \"business\"-tastatur"},
	/* de */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_DE, "Keymap Datei f¸r German business keyboard positional w‰hlen"},
	/* es */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_ES, "Especificar nombre fichero teclado Alem·n business posicional"},
	/* fr */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_FR, "SpÈcifier le nom du fichier de mappage clavier positionnel allemand"},
	/* hu */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_HU, "Adja meg a nÈmet, hivatalos, pozÌciÛ szerinti billenty˚zet lekÈpzÈsi f·jl nevÈt."},
	/* it */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_IT, "Specifica il nome del file della mappa posizionale della tastiera business tedesca"},
	/* ko */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_NL, "Geef de naam van het positioneel toetstoewijzingsbestand voor het Duitse zakelijk toetsenbord"},
	/* pl */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_PL, "Okre∂l nazwÍ pliku pozycyjnego uk≥adu niemieckiej klawiatury biznesowej"},
	/* ru */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_RU, "Specify name of German business keyboard positional keymap file"},
	/* sv */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_SV, "Ange fil fˆr positionsriktig tangentbordsemulering fˆr tyskt \"business\"-tangentbord"},
	/* tr */ {IDCLS_SPECIFY_BDE_POS_KEYMAP_NAME_TR, "Alman business klavyesi konumsal tu˛ haritas˝ dosyas˝n˝n ismini belirt"},
#endif
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_TRUE_DRIVE,    N_("Enable hardware-level emulation of disk drives")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TRUE_DRIVE_DA, "AktivÈr emulering af diskettedrev pÂ hardware-niveau"},
	/* de */ {IDCLS_ENABLE_TRUE_DRIVE_DE, "Hardware Emulation f¸r Disklaufwerke aktivieren"},
	/* es */ {IDCLS_ENABLE_TRUE_DRIVE_ES, "Permitir EmulaciÛn a nivel hardware unidades de disco"},
	/* fr */ {IDCLS_ENABLE_TRUE_DRIVE_FR, "Activer l'Èmulation de disques au niveau physique"},
	/* hu */ {IDCLS_ENABLE_TRUE_DRIVE_HU, "LemezegysÈgek hardver szint˚ emul·ciÛj·nak engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_TRUE_DRIVE_IT, "Attiva emulazione hardware dei disk drive"},
	/* ko */ {IDCLS_ENABLE_TRUE_DRIVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TRUE_DRIVE_NL, "Hardwarematige emulatie van diskdrives inschakelen"},
	/* pl */ {IDCLS_ENABLE_TRUE_DRIVE_PL, "W≥±cz sprzÍtow± emulacjÍ stacji dyskÛw"},
	/* ru */ {IDCLS_ENABLE_TRUE_DRIVE_RU, "Enable hardware-level emulation of disk drives"},
	/* sv */ {IDCLS_ENABLE_TRUE_DRIVE_SV, "Aktivera emulering av diskettstationer pÂ maskinvarunivÂ"},
	/* tr */ {IDCLS_ENABLE_TRUE_DRIVE_TR, "Disk s¸r¸c¸lerinin donan˝m seviyesinde em¸lasyonunu aktif et"},
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_TRUE_DRIVE,    N_("Disable hardware-level emulation of disk drives")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TRUE_DRIVE_DA, "DeaktivÈr emulering af diskettedrev pÂ hardware-niveau"},
	/* de */ {IDCLS_DISABLE_TRUE_DRIVE_DE, "Hardware Emulation f¸r Disklaufwerke deaktivieren"},
	/* es */ {IDCLS_DISABLE_TRUE_DRIVE_ES, "No permitir EmulaciÛn a nivel hardware unidades de disco"},
	/* fr */ {IDCLS_DISABLE_TRUE_DRIVE_FR, "DÈsactiver l'Èmulation de disques au niveau physique"},
	/* hu */ {IDCLS_DISABLE_TRUE_DRIVE_HU, "LemezegysÈgek hardver szint˚ emul·ciÛj·nak tilt·sa"},
	/* it */ {IDCLS_DISABLE_TRUE_DRIVE_IT, "Disattiva emulazione hardware dei disk drive"},
	/* ko */ {IDCLS_DISABLE_TRUE_DRIVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TRUE_DRIVE_NL, "Hardwarematige emulatie van diskdrives uitschakelen"},
	/* pl */ {IDCLS_DISABLE_TRUE_DRIVE_PL, "Wy≥±cz sprzÍtow± emulacjÍ stacji dyskÛw"},
	/* ru */ {IDCLS_DISABLE_TRUE_DRIVE_RU, "Disable hardware-level emulation of disk drives"},
	/* sv */ {IDCLS_DISABLE_TRUE_DRIVE_SV, "Inaktivera emulering av diskettstationer pÂ maskinvarunivÂ"},
	/* tr */ {IDCLS_DISABLE_TRUE_DRIVE_TR, "Disk s¸r¸c¸lerinin donan˝m seviyesinde em¸lasyonunu pasifle˛tir"},
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DRIVE_SOUND,    N_("Enable sound emulation of disk drives")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DRIVE_SOUND_DA, "AktivÈr emulering af lyd for diskettedrev"},
	/* de */ {IDCLS_ENABLE_DRIVE_SOUND_DE, "Sound Emulation f¸r Disklaufwerke aktivieren"},
	/* es */ {IDCLS_ENABLE_DRIVE_SOUND_ES, "Permitir emulaciÛn de sonido en unidad de discos"},
	/* fr */ {IDCLS_ENABLE_DRIVE_SOUND_FR, "Activer l'Èmulation du son des lecteurs de disque"},
	/* hu */ {IDCLS_ENABLE_DRIVE_SOUND_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DRIVE_SOUND_IT, "Attiva emulazione audio dei disk drive"},
	/* ko */ {IDCLS_ENABLE_DRIVE_SOUND_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DRIVE_SOUND_NL, "Geluidsemulatie van diskdrives inschakelen"},
	/* pl */ {IDCLS_ENABLE_DRIVE_SOUND_PL, "W≥±cz emulacjÍ odg≥osÛw stacji dyskÛw"},
	/* ru */ {IDCLS_ENABLE_DRIVE_SOUND_RU, "Enable sound emulation of disk drives"},
	/* sv */ {IDCLS_ENABLE_DRIVE_SOUND_SV, "Aktivera ljudemulering fˆr diskettstationer"},
	/* tr */ {IDCLS_ENABLE_DRIVE_SOUND_TR, "Disk s¸r¸c¸lerinin ses em¸lasyonunu aktif et"},
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DRIVE_SOUND,    N_("Disable sound emulation of disk drives")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DRIVE_SOUND_DA, "DeaktivÈr emulering af lyd for diskettedrev"},
	/* de */ {IDCLS_DISABLE_DRIVE_SOUND_DE, "Sound Emulation f¸r Disklaufwerke deaktivieren"},
	/* es */ {IDCLS_DISABLE_DRIVE_SOUND_ES, "Inhabilitar emulaciÛn de sonido wn unidad de discos"},
	/* fr */ {IDCLS_DISABLE_DRIVE_SOUND_FR, "DÈsactiver l'Èmulation du son des lecteurs disque"},
	/* hu */ {IDCLS_DISABLE_DRIVE_SOUND_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DRIVE_SOUND_IT, "Disattiva emulazione audio dei disk drive"},
	/* ko */ {IDCLS_DISABLE_DRIVE_SOUND_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DRIVE_SOUND_NL, "Geluidsemulatie van diskdrives uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DRIVE_SOUND_PL, "Wy≥±cz emulacjÍ odg≥osÛw stacji dyskÛw"},
	/* ru */ {IDCLS_DISABLE_DRIVE_SOUND_RU, "Disable sound emulation of disk drives"},
	/* sv */ {IDCLS_DISABLE_DRIVE_SOUND_SV, "Inaktivera ljudemulering fˆr diskettstationer"},
	/* tr */ {IDCLS_DISABLE_DRIVE_SOUND_TR, "Disk s¸r¸c¸lerinin ses em¸lasyonunu pasifle˛tir"},
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_SET_DRIVE_SOUND_VOLUME,    N_("Set volume for disk drive sound emulation (0-4000)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DRIVE_SOUND_VOLUME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DRIVE_SOUND_VOLUME_DE, "Lautst‰rke f¸r Floppyger‰usch Emulation (0-4000)"},
	/* es */ {IDCLS_SET_DRIVE_SOUND_VOLUME_ES, "Ajustar volumen para emulaciÛn sonido disquetera (0-400)"},
	/* fr */ {IDCLS_SET_DRIVE_SOUND_VOLUME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DRIVE_SOUND_VOLUME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DRIVE_SOUND_VOLUME_IT, "Imposta il volume per l'emulazione audio del disk drive (0-4000)"},
	/* ko */ {IDCLS_SET_DRIVE_SOUND_VOLUME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DRIVE_SOUND_VOLUME_NL, "Stel het volume in van de diskdrive geluidemulatie (0-4000)"},
	/* pl */ {IDCLS_SET_DRIVE_SOUND_VOLUME_PL, "Ustaw g≥o∂no∂Ê emulacji odg≥osÛw stacji dyskÛw (0-4000)"},
	/* ru */ {IDCLS_SET_DRIVE_SOUND_VOLUME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DRIVE_SOUND_VOLUME_SV, "V‰lj volym fˆr ljudemulering fˆr diskettstation (0-4000)"},
	/* tr */ {IDCLS_SET_DRIVE_SOUND_VOLUME_TR, ""},  /* fuzzy */
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_P_VOLUME,    N_("<Volume>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_VOLUME_DA, "<Lydstyrke>"},
	/* de */ {IDCLS_P_VOLUME_DE, "<Lautst‰rke>"},
	/* es */ {IDCLS_P_VOLUME_ES, "<Volumen>"},
	/* fr */ {IDCLS_P_VOLUME_FR, "<Volume>"},
	/* hu */ {IDCLS_P_VOLUME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_VOLUME_IT, "<Volume>"},
	/* ko */ {IDCLS_P_VOLUME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_VOLUME_NL, "<Volume>"},
	/* pl */ {IDCLS_P_VOLUME_PL, "<G≥o∂no∂Ê>"},
	/* ru */ {IDCLS_P_VOLUME_RU, "<Volume>"},
	/* sv */ {IDCLS_P_VOLUME_SV, "<Volym>"},
	/* tr */ {IDCLS_P_VOLUME_TR, "<Ses D¸zeyi>"},
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_P_RPM,    N_("<RPM>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_RPM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_RPM_DE, "<RPM>"},
	/* es */ {IDCLS_P_RPM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_P_RPM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_RPM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_RPM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_P_RPM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_RPM_NL, "<RPM>"},
	/* pl */ {IDCLS_P_RPM_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_P_RPM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_RPM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_RPM_TR, ""},  /* fuzzy */
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_P_WOBBLE,    N_("<Wobble>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_WOBBLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_WOBBLE_DE, "<Schwankung>"},
	/* es */ {IDCLS_P_WOBBLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_P_WOBBLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_WOBBLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_WOBBLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_P_WOBBLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_WOBBLE_NL, "<Wiebel>"},
	/* pl */ {IDCLS_P_WOBBLE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_P_WOBBLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_WOBBLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_WOBBLE_TR, ""},  /* fuzzy */
#endif
	
	/* drive/drive-cmdline-options.c, drive/iec/plus4exp/plus4exp-cmdline-options.c,
	 drive/iec/c64exp/c64exp-cmdline-options.c, printerdrv/interface-serial.c,
	 attach.c, lightpen.c, c128/functionrom.c, userport/userport_joystick.c,
	 c64/c64gluelogic.c, c64/cart/sfx_soundexpander.c */
	/* en */ {IDCLS_P_TYPE,    N_("<Type>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_TYPE_DA, "<Type>"},
	/* de */ {IDCLS_P_TYPE_DE, "<Typ>"},
	/* es */ {IDCLS_P_TYPE_ES, "<Tipo>"},
	/* fr */ {IDCLS_P_TYPE_FR, "<type>"},
	/* hu */ {IDCLS_P_TYPE_HU, "<TÌpus>"},
	/* it */ {IDCLS_P_TYPE_IT, "<Tipo>"},
	/* ko */ {IDCLS_P_TYPE_KO, "<¡æ∑˘>"},
	/* nl */ {IDCLS_P_TYPE_NL, "<Soort>"},
	/* pl */ {IDCLS_P_TYPE_PL, "<Typ>"},
	/* ru */ {IDCLS_P_TYPE_RU, "<Type>"},
	/* sv */ {IDCLS_P_TYPE_SV, "<Typ>"},
	/* tr */ {IDCLS_P_TYPE_TR, "<Tip>"},
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_SET_DRIVE_TYPE,    N_("Set drive type (0: no drive")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DRIVE_TYPE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DRIVE_TYPE_DE, "Setze Laufwerkstyp (0: kein Laufwerk"},
	/* es */ {IDCLS_SET_DRIVE_TYPE_ES, "Seleccionar tipo unidad disco (0: sin unidad disco)"},
	/* fr */ {IDCLS_SET_DRIVE_TYPE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DRIVE_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DRIVE_TYPE_IT, "Imposta tipo drive (0: nessun drive"},
	/* ko */ {IDCLS_SET_DRIVE_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DRIVE_TYPE_NL, "Stel drive soort in (0: geen drive"},
	/* pl */ {IDCLS_SET_DRIVE_TYPE_PL, "Ustaw typ stacji (0: brak stacji"},
	/* ru */ {IDCLS_SET_DRIVE_TYPE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DRIVE_TYPE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DRIVE_TYPE_TR, ""},  /* fuzzy */
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_SET_DRIVE_RPM,    N_("Set drive rpm (30000 = 300rpm)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DRIVE_RPM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DRIVE_RPM_DE, "Setze Laufwerk RPM (30000 = 300rpm)"},
	/* es */ {IDCLS_SET_DRIVE_RPM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_DRIVE_RPM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DRIVE_RPM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DRIVE_RPM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_DRIVE_RPM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DRIVE_RPM_NL, "Stel drive rpm in (30000 = 300rpm)"},
	/* pl */ {IDCLS_SET_DRIVE_RPM_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SET_DRIVE_RPM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DRIVE_RPM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DRIVE_RPM_TR, ""},  /* fuzzy */
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_SET_DRIVE_WOBBLE,    N_("Set drive wobble (100 = +/-0.5rpm)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DRIVE_WOBBLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DRIVE_WOBBLE_DE, "Setze Laufwaerkschwankung (100 = +/-0.5rpm)"},
	/* es */ {IDCLS_SET_DRIVE_WOBBLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_DRIVE_WOBBLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DRIVE_WOBBLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DRIVE_WOBBLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_DRIVE_WOBBLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DRIVE_WOBBLE_NL, "Stel drive wiebel in (100 = +/-0.5rpm)"},
	/* pl */ {IDCLS_SET_DRIVE_WOBBLE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SET_DRIVE_WOBBLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DRIVE_WOBBLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DRIVE_WOBBLE_TR, ""},  /* fuzzy */
#endif
	
	/* drive/drive-cmdline-options.c,
	 drive/iec/iec-cmdline-options.c,
	 gfxoutputdrv/doodledrv.c */
	/* en */ {IDCLS_P_METHOD,    N_("<method>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_METHOD_DA, "<metode>"},
	/* de */ {IDCLS_P_METHOD_DE, "<Methode>"},
	/* es */ {IDCLS_P_METHOD_ES, "<mÈtodo>"},
	/* fr */ {IDCLS_P_METHOD_FR, "<mÈthode>"},
	/* hu */ {IDCLS_P_METHOD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_METHOD_IT, "<metodo>"},
	/* ko */ {IDCLS_P_METHOD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_METHOD_NL, "<methode>"},
	/* pl */ {IDCLS_P_METHOD_PL, "<metoda>"},
	/* ru */ {IDCLS_P_METHOD_RU, "<method>"},
	/* sv */ {IDCLS_P_METHOD_SV, "<metod>"},
	/* tr */ {IDCLS_P_METHOD_TR, "<metod>"},
#endif
	
	/* gfxoutputdrv/doodledrv.c */
	/* en */ {IDCLS_P_COLOR,    N_("<color>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_COLOR_DA, "<farve>"},
	/* de */ {IDCLS_P_COLOR_DE, "<Farbe>"},
	/* es */ {IDCLS_P_COLOR_ES, "<color>"},
	/* fr */ {IDCLS_P_COLOR_FR, "<couleur>"},
	/* hu */ {IDCLS_P_COLOR_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_COLOR_IT, "<colore>"},
	/* ko */ {IDCLS_P_COLOR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_COLOR_NL, "<kleur>"},
	/* pl */ {IDCLS_P_COLOR_PL, "<kolor>"},
	/* ru */ {IDCLS_P_COLOR_RU, "<Ê“’‚>"},
	/* sv */ {IDCLS_P_COLOR_SV, "<f‰rg>"},
	/* tr */ {IDCLS_P_COLOR_TR, "<renk>"},
#endif
	
	/* gfxoutputdrv/doodledrv.c, gfxoutputdrv/koaladrv.c */
	/* en */ {IDCLS_OVERSIZED_HANDLING,    N_("Select the way the oversized input should be handled, (0: scale down, 1: crop left top, 2: crop center top,  3: crop right top, 4: crop left center, 5: crop center, 6: crop right center, 7: crop left bottom, 8: crop center bottom, 9:  crop right bottom)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_OVERSIZED_HANDLING_DA, "Angiv hÂndtering af for stort input (0: skalÈr ned, 1: klip venstre top, 2: klip midt top,  3: klip h¯jre top, 4: klip venstre midt, 5: klip centreret, 6: klip h¯jre midt, 7: klip venstre bund, 8: klip midt bund, 9: klip h¯jre bund)"},
	/* de */ {IDCLS_OVERSIZED_HANDLING_DE, "W‰hle, wie ¸bergroﬂer Input behandelt werden soll, (0: skaliere, 1: schneide links oben, 2: schneide mitte oben,  3: schneide rechts oben, 4: schneide links mitte, 5: schneide mitte, 6: schneide rechts mitte, 7: schneide links unten, 8: schneide mitte unten, 9:  schneide rechts unten)"},
	/* es */ {IDCLS_OVERSIZED_HANDLING_ES, "Seleccionar cÛmo se manejar·n las entradas sobredimensionadas, (0: escalar, 1:recorte sup izq, 2:recorte sup centro, 3:recorte sup dcha, 4:recorte centro izq, 5:recorte centro, 6:recorte centro dcha, 7:recorte abajo izq, 8:recorte centro abajo, 9:recorte abajo dcha)"},
	/* fr */ {IDCLS_OVERSIZED_HANDLING_FR, "SÈlectionner la faÁon de traiter les entrÈes trop longues (0: rÈduire, 1-9: Rogner ‡ partir du coin: 1: haut-gauche, 2: haut-centre, 3: haut-droite, 4: centre-gauche, 5: centre, 6: centre-droite, 7: bas-gauche, 8: bas-centre, 9: bas-droite)"},
	/* hu */ {IDCLS_OVERSIZED_HANDLING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_OVERSIZED_HANDLING_IT, "Seleziona come gestire il sovradimensionamento dell'input, (0: scale down, 1: ritaglia sinistra alto, 2: ritaglia centro alto,  3: ritaglia destra alto, 4: ritaglia sinistra centro, 5: ritaglia centro, 6: ritaglia destra centro, 7: ritaglia sinistra basso, 8: ritaglia centro basso, 9:  ritaglia destra basso)"},
	/* ko */ {IDCLS_OVERSIZED_HANDLING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_OVERSIZED_HANDLING_NL, "Selecteer de manier waarop de te grote invoer moet worden afgehandeld, (0: maak kleiner, 1: knip linker bovenkant, 2: knip midden bovenkant, 3: knip rechter bovenkant, 4: knip linker midden, 5: knip midden, 6: knip rechter midden, 7: knip linker onderkant, 8: knip midden onderkant, 9: knip rechter onderkant)"},
	/* pl */ {IDCLS_OVERSIZED_HANDLING_PL, "Wybierz sposÛb, w jaki ma byÊ obs≥ugiwany za duøy obraz, (0: zmniejszenie, 1: ∂ciÍcie z gÛry z lewej, 2: ∂ciÍcie z gÛry, 3: ∂ciÍcie z prawej z gÛry, 4: ∂ciÍcie z lewej, 5: ∂ciÍcie od ∂rodka, 6: ∂ciÍcie z prawej, 7: ∂ciÍcie z lewej z do≥u, 8: ∂ciÍcie z do≥u, 9: ∂ciÍcie z prawej z do≥u)"},
	/* ru */ {IDCLS_OVERSIZED_HANDLING_RU, "Select the way the oversized input should be handled, (0: scale down, 1: crop left top, 2: crop center top,  3: crop right top, 4: crop left center, 5: crop center, 6: crop right center, 7: crop left bottom, 8: crop center bottom, 9:  crop right bottom)"},
	/* sv */ {IDCLS_OVERSIZED_HANDLING_SV, "V‰lj hur fˆr stor indata skall hanteras, (0: skala ned, 1: visa ˆvre v‰nstra, 2: visa ˆvre mitten, 3: visa ˆvre hˆgra, 4: visa v‰nstra mitten, 5: visa mitten, 6: visa hˆgra mitten, 7: visa nedre v‰nstra, 8: visa nedre mitten, 9: visa nedre hˆgre)"},
	/* tr */ {IDCLS_OVERSIZED_HANDLING_TR, "B¸y¸k girdilerin ele al˝nma yˆntemini seÁin, (0: k¸Á¸lt, 1: sol ¸stten k˝rp, 2: ortal˝ ¸stten k˝rp,  3: sa ¸stten k˝rp, 4: soldan ortal˝ k˝rp, 5: ortadan k˝rp, 6: sadan ortal˝ k˝rp, 7: sol alttan k˝rp, 8: ortal˝ alttan k˝rp, 9:  sa alttan k˝rp)"},
#endif
	
	/* gfxoutputdrv/doodledrv.c, gfxoutputdrv/koaladrv.c */
	/* en */ {IDCLS_UNDERSIZED_HANDLING,    N_("Select the way the undersized input should be handled, (0: scale up, 1: borderize)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_UNDERSIZED_HANDLING_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_UNDERSIZED_HANDLING_DE, "Wie soll ungen¸gende Eingabe behandelt werden (0: skaliere, 1: borderize)"},
	/* es */ {IDCLS_UNDERSIZED_HANDLING_ES, "Seleccionar cÛmo se manejar· la entrada fuera de rango,  (0: scale up, 1: borderize)"},
	/* fr */ {IDCLS_UNDERSIZED_HANDLING_FR, "SÈlectionner la faÁon de traiter les entrÈe trop courtes, (0: agrandir, 1: ajouter bordure)"},
	/* hu */ {IDCLS_UNDERSIZED_HANDLING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_UNDERSIZED_HANDLING_IT, "Seleziona come gestire il sottodimensionamento dell'input, (0: scale up, 1: borderize)"},
	/* ko */ {IDCLS_UNDERSIZED_HANDLING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_UNDERSIZED_HANDLING_NL, "Selecteer de manier waarop de te kleine invoer moet worden afgehandeld, (0: maak groter, 1: maak border)"},
	/* pl */ {IDCLS_UNDERSIZED_HANDLING_PL, "Wybierz sposÛb obs≥ugi ma≥ego obrazu (0: skaluj, 1: dodaj ramkÍ)"},
	/* ru */ {IDCLS_UNDERSIZED_HANDLING_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_UNDERSIZED_HANDLING_SV, "V‰lj hur fˆr liten indata skall hanteras (0: skala upp, 1: inramning)"},
	/* tr */ {IDCLS_UNDERSIZED_HANDLING_TR, ""},  /* fuzzy */
#endif
	
	/* gfxoutputdrv/doodledrv.c */
	/* en */ {IDCLS_MULTICOLOR_HANDLING,    N_("Select the way the multicolor to hires should be handled, (0: b&w, 1: 2 colors, 2: 4 colors, 3: gray scale,  4: best cell colors)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_MULTICOLOR_HANDLING_DA, "Angiv mÂden multicolor til h¯jopl¯sning skal hÂndteres pÂ, (0: sort/hvid, 1: 2 farver, 2: 4 farver, 3: grÂtoner,  4: bedste cellefarver)"},
	/* de */ {IDCLS_MULTICOLOR_HANDLING_DE, "W‰hle, wie Multicolor zu Hochauflˆsung behandelte werden soll, (0: s&w, 1: 2 Farben, 2: 4 Farben, 4 Graustufen, 4: Beste Zellenfarben)"},
	/* es */ {IDCLS_MULTICOLOR_HANDLING_ES, "Seleccionar cÛmo se manejar·n el cambio multicolor-alta resoluciÛn (0:ByN, 1:2 colores, 2:4 colores, 3:escala grises, 4:mejores colores)"},
	/* fr */ {IDCLS_MULTICOLOR_HANDLING_FR, "SÈlectionner la faÁon de traiter multicolore ‡ haute rÈsolution, (0: noir et blanc, 1: 2 couleurs, 2: 4 couleurs, 3: Èchelle de gris, 4: meilleures couleurs)"},
	/* hu */ {IDCLS_MULTICOLOR_HANDLING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_MULTICOLOR_HANDLING_IT, "Seleziona come gestire il multicolor in hires, (0: b&w, 1: 2 colori, 2: 4 colori, 3: scala di grigi,  4: migliori colori cella)"},
	/* ko */ {IDCLS_MULTICOLOR_HANDLING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_MULTICOLOR_HANDLING_NL, "Selecteer de manier waarop 'multicolor' naar 'hires' conversie moet worden afgehandeld, (0: zwart/wit, 1: 2 kleuren, 2: 4 kleuren, 3: grijs, 4: best mogelijke kleuren in de cel)"},
	/* pl */ {IDCLS_MULTICOLOR_HANDLING_PL, "Wybiera sposÛb obs≥ugi wielu kolorÛw w wysokiej rozdzielczo∂ci (0: czarno-bia≥y, 1: 2 kolory, 2: 4 kolory, 3: skala szaro∂ci, 4: najlepsze kolory komÛrek)"},
	/* ru */ {IDCLS_MULTICOLOR_HANDLING_RU, "Select the way the multicolor to hires should be handled, (0: b&w, 1: 2 colors, 2: 4 colors, 3: gray scale,  4: best cell colors)"},
	/* sv */ {IDCLS_MULTICOLOR_HANDLING_SV, "V‰lj hur flerf‰rgs-till-hˆgupplˆst skall hanteras, (0: svartvit, 1: 2 f‰rger, 2: fyra f‰rger, 3: grÂskala, 4: b‰sta cellf‰rger)"},
	/* tr */ {IDCLS_MULTICOLOR_HANDLING_TR, "«ok renkliden y¸ksek Áˆz¸n¸rl¸e geÁi˛in ele al˝nma yˆntemini seÁin, (0: siyah&beyaz, 1: 2 renk, 2: 4 renk, 3: gri tonlu,  4: en iyi h¸cre renkleri)"},
#endif
	
	/* gfxoutputdrv/doodledrv.c */
	/* en */ {IDCLS_TED_LUM_HANDLING,    N_("Select the way the TED luminosity should be handled, (0: ignore, 1: dither)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TED_LUM_HANDLING_DA, "Angiv mÂden TED-lysstyrker skal hÂndteres pÂ (0: ignorÈr, 1: blanding)"},
	/* de */ {IDCLS_TED_LUM_HANDLING_DE, "W‰hle, wie TED Leuchtst‰rke behandelt werden soll, (0: ignorieren, 1: dither)"},
	/* es */ {IDCLS_TED_LUM_HANDLING_ES, "Seleccionar cÛmo se manejar·n la luminosidad TED (0:ignorar, 1:dither)"},
	/* fr */ {IDCLS_TED_LUM_HANDLING_FR, "SÈlectionner la faÁon de traiter la luminositÈ TED, (0: ignorer, 1: tramer)"},
	/* hu */ {IDCLS_TED_LUM_HANDLING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_TED_LUM_HANDLING_IT, "Seleziona come gestire la luminosit‡ TED, (0: ignora, 1: dither)"},
	/* ko */ {IDCLS_TED_LUM_HANDLING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TED_LUM_HANDLING_NL, "Selecteer de manier waarop de TED kleuren moeten worden afgehandeld, (0: negeren, 1: dither)"},
	/* pl */ {IDCLS_TED_LUM_HANDLING_PL, "WybÛr sposobu obs≥ugi jaskrawo∂ci TED (0: ignoruj, 1: ditheruj)"},
	/* ru */ {IDCLS_TED_LUM_HANDLING_RU, "Select the way the TED luminosity should be handled, (0: ignore, 1: dither)"},
	/* sv */ {IDCLS_TED_LUM_HANDLING_SV, "V‰lj hur TED-luminositet skall hanteras, (0: ignorera, 1: f‰rgutj‰mna)"},
	/* tr */ {IDCLS_TED_LUM_HANDLING_TR, "TED l¸minozitesinin ele al˝nma yˆntemini seÁin, (0: yok say, 1: dither)"},
#endif
	
	/* gfxoutputdrv/doodledrv.c */
	/* en */ {IDCLS_CRTC_TEXT_COLOR,    N_("Select the CRTC text color (0: white, 1: amber, 2: green)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_CRTC_TEXT_COLOR_DA, "VÊlg CRTC tekstfarve (0: hvid, 1: ravgul, 2: gr¯n)"},
	/* de */ {IDCLS_CRTC_TEXT_COLOR_DE, "W‰hle CRTC Textfarbe (0: weiﬂ, 1: Bernstein, 2: gr¸n)"},
	/* es */ {IDCLS_CRTC_TEXT_COLOR_ES, "Seleccionar color texto CRTC (0:blanco, 1:·mbar, 2:verde)"},
	/* fr */ {IDCLS_CRTC_TEXT_COLOR_FR, "SÈlectionner la couleur du texte CRTC (0: blanc, 1: ambre, 2: vert)"},
	/* hu */ {IDCLS_CRTC_TEXT_COLOR_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_CRTC_TEXT_COLOR_IT, "Seleziona il colore del testo CRTC (0: bianco, 1: ambra, 2: verde)"},
	/* ko */ {IDCLS_CRTC_TEXT_COLOR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_CRTC_TEXT_COLOR_NL, "Selecteer de CRTC tekst kleur (0: wit, 1: amber, 2: groen)"},
	/* pl */ {IDCLS_CRTC_TEXT_COLOR_PL, "Wybierz kolor tekstu CRTC (0: bia≥y, 1: bursztynowy, 2: zielony)"},
	/* ru */ {IDCLS_CRTC_TEXT_COLOR_RU, "Select the CRTC text color (0: white, 1: amber, 2: green)"},
	/* sv */ {IDCLS_CRTC_TEXT_COLOR_SV, "V‰lj CRTC-textf‰rg (0: vit, 1: gul, 2: grˆn)"},
	/* tr */ {IDCLS_CRTC_TEXT_COLOR_TR, "CRTC yaz˝ rengini seÁin (0: beyaz, 1: kehribar, 2: ye˛il)"},
#endif
	
	/* drive/drive-cmdline-options.c */
	/* en */ {IDCLS_SET_DRIVE_EXTENSION_POLICY,    N_("Set drive 40 track extension policy (0: never, 1: ask, 2: on access)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_DA, "Angiv drevets regel for 40-spors udviddelse (0: aldrig, 1: sp¯rg, 2: ved tilgang)"},
	/* de */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_DE, "40 Spur Erweiterungsmethode (0: nie, 1: R¸ckfrage, 2: bei Bedarf)"},
	/* es */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_ES, "Seleccionar polÌtica unidad disco extensiÛn 40 pistas (0:nunca, 1:preguntar, 2:al acceder)"},
	/* fr */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_FR, "SpÈcifier la rËgle d'extention 40 pistes (0: jamais, 1: demander, 2: ‡ l'accËs)"},
	/* hu */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_HU, "Adja meg a 40 s·vra kiterjesztÈs terjesztÈs mÛdj·t (0: soha, 1: kÈrdÈs, 2: hozz·fÈrÈskor)"},
	/* it */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_IT, "Imposta la politica di estensione a 40 tracce del drive (0: mai, 1: a richiesta, 2: all'accesso)"},
	/* ko */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_NL, "Stel beleid voor 40-sporenuitbreiding drive in (0: nooit, 1: vraag, 2: bij toegang)"},
	/* pl */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_PL, "Okre∂l zasadÍ rozszerzania do 40 ∂cieøek (0: nigdy, 1: pytaj, 2: przy dostÍpie)"},
	/* ru */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_RU, "Set drive 40 track extension policy (0: never, 1: ask, 2: on access)"},
	/* sv */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_SV, "Ange regel fˆr 40-spÂrsutˆkning (0: aldrig, 1: frÂga, 2: vid Âtkomst)"},
	/* tr */ {IDCLS_SET_DRIVE_EXTENSION_POLICY_TR, "S¸r¸c¸ 40 track b¸y¸me kural˝n˝ ayarla (0: asla, 1: sor, 2: eri˛ildiinde)"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME,    N_("Specify name of 1540 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_DE, "Name der 1540 DOS ROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_ES, "Especificar nombre imagen ROM de 1540 DOS "},
	/* fr */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1540"},
	/* ko */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_NL, "Geef de naam van het 1540 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1540"},
	/* ru */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_1540_DOS_ROM_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME,    N_("Specify name of 1541 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_DA, "Angiv navn pÂ 1541-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_DE, "Name der 1541 DOS ROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_ES, "Especificar nombre imagen ROM de 1541 DOS "},
	/* fr */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 1541 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_HU, "Adja meg az 1541 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1541"},
	/* ko */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_NL, "Geef de naam van het 1541 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1541"},
	/* ru */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_RU, "Specify name of 1541 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_SV, "Ange namn pÂ 1541-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_1541_DOS_ROM_NAME_TR, "1541 DOS ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME,    N_("Specify name of 1541-II DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_DA, "Angivn navn pÂ 1541-II-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_DE, "Name der 1541-II DOS ROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_ES, "Especificar nombre imagen ROM de 1541-II DOS"},
	/* fr */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 1541-II DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_HU, "Adja meg az 1541-II DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1541-II"},
	/* ko */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_NL, "Geef de naam van het 1541-II DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1541-II"},
	/* ru */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_RU, "Specify name of 1541-II DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_SV, "Ange namn pÂ 1541-II-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_1541_II_DOS_ROM_NAME_TR, "1541-II DOS ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME,    N_("Specify name of 1570 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_DA, "Angivn navn pÂ 1570-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_DE, "Name der 1570 DOS ROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_ES, "Especificar nombre imagen ROM de 1570 DOS"},
	/* fr */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 1570 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_HU, "Adja meg az 1570 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1570"},
	/* ko */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_NL, "Geef de naam van het 1570 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1570"},
	/* ru */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_RU, "Specify name of 1570 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_SV, "Ange namn pÂ 1570-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_1570_DOS_ROM_NAME_TR, "1570 DOS ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME,    N_("Specify name of 1571 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_DA, "Angivn navn pÂ 1571-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_DE, "Name der 1571 DOS ROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_ES, "Especificar nombre imagen ROM de 1571 DOS "},
	/* fr */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 1571 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_HU, "Adja meg az 1571 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1571"},
	/* ko */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_NL, "Geef de naam van het 1571 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1571"},
	/* ru */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_RU, "Specify name of 1571 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_SV, "Ange namn pÂ 1571-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_1571_DOS_ROM_NAME_TR, "1571 DOS ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME,    N_("Specify name of 1581 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_DA, "Angivn navn pÂ 1581-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_DE, "Name der 1581 DOS ROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_ES, "Especificar nombre imagen ROM de 1581 DOS "},
	/* fr */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 1581 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_HU, "Adja meg az 1581 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1581"},
	/* ko */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_NL, "Geef de naam van het 1581 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1581"},
	/* ru */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_RU, "Specify name of 1581 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_SV, "Ange namn pÂ 1581-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_1581_DOS_ROM_NAME_TR, "1581 DOS ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME,    N_("Specify name of 2000 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_DA, "Angiv navn pÂ 2000-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_DE, "Name der 2000 DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_ES, "Especificar nombre imagen 2000 DOS ROM"},
	/* fr */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 2000 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del 2000 DOS"},
	/* ko */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_NL, "Geef de naam van het 2000 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 2000"},
	/* ru */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_RU, "Specify name of 2000 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_SV, "Ange namn pÂ 2000-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_2000_DOS_ROM_NAME_TR, "2000 DOS ROM imaj ismini belirt"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME,    N_("Specify name of 4000 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_DA, "Angiv navn pÂ 4000-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_DE, "Name der 4000 DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_ES, "Especificar nombre imagen 4000 DOS ROM"},
	/* fr */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 4000 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del 4000 DOS"},
	/* ko */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_NL, "Geef de naam van het 4000 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 4000"},
	/* ru */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_RU, "Specify name of 4000 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_SV, "Ange namn pÂ 4000-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_4000_DOS_ROM_NAME_TR, "4000 DOS ROM imaj ismini belirt"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_SET_IDLE_METHOD,    N_("Set drive idling method (0: no traps, 1: skip cycles, 2: trap idle)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_IDLE_METHOD_DA, "VÊlg tomgangsmetode for diskettedrev (0: ingen traps, 1: spring over cykler, 2: trap ledig)"},
	/* de */ {IDCLS_SET_IDLE_METHOD_DE, "Laufwerks idling Methode (0: kein Traps, 1: Zyklen verwerfen, 2: trap idle)"},
	/* es */ {IDCLS_SET_IDLE_METHOD_ES, "Seleccionar mÈtodo tiempo inactivo (0: sin intercepciÛn, 1:saltar ciclos, 2:interceptar tiempos inactivos)"},
	/* fr */ {IDCLS_SET_IDLE_METHOD_FR, "SpÈcifier la mÈthode d'idle (0: pas de traps, 1: sauter des cycles, 2: trap idle)"},
	/* hu */ {IDCLS_SET_IDLE_METHOD_HU, "LemezegysÈg ¸resj·r·si mÛdja (0: folytonos emul·ciÛ, 1: ciklusok kihagy·sa, 2: ¸res DOS ciklusok kihagy·sa)"},
	/* it */ {IDCLS_SET_IDLE_METHOD_IT, "Imposta il metodo per rilevare l'inattivit‡ del drive (0: non rilevare, 1: salta cicli, 2: rileva inattivit‡)"},
	/* ko */ {IDCLS_SET_IDLE_METHOD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_IDLE_METHOD_NL, "Stel de drive idlemethode in (0: geen traps, 1: sla cycli over, 2: trap idle)"},
	/* pl */ {IDCLS_SET_IDLE_METHOD_PL, "Okre∂l metodÍ bezczynno∂ci napÍdu (0: no traps, 1: opuszczaj cykle, 2: trap idle)"},
	/* ru */ {IDCLS_SET_IDLE_METHOD_RU, "Set drive idling method (0: no traps, 1: skip cycles, 2: trap idle)"},
	/* sv */ {IDCLS_SET_IDLE_METHOD_SV, "St‰ll in v‰ntemetod fˆr diskettstation (0: fÂnta inge, 1: hoppa cykler, 2: fÂnga v‰ntan)"},
	/* tr */ {IDCLS_SET_IDLE_METHOD_TR, "S¸r¸c¸ bo˛a vakit geÁirme metodunu ayarlay˝n (0: trap yok, 1: cyclelar˝ atla, 2: trap bo˛a vakit geÁirme)"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DRIVE_RTC_SAVE,    N_("Enable saving of FD2000/4000 RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_DE, "FD2000/4000 RTC Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_ES, "Permitir grabaciÛn de datos cuando cambien de  FD2000/4000 RTC."},
	/* fr */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_IT, "Attiva il salvataggio dei dati RTC del FD2000/4000 alla modifica."},
	/* ko */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_NL, "Opslaan van de FD2000/4000 RTC gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_PL, "Zapisuj dane zegara czasu rzeczywistego FD2000/4000 przy zmianie"},
	/* ru */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_DRIVE_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DRIVE_RTC_SAVE,    N_("Disable saving of FD2000/4000 RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_DE, "FD2000/4000 RTC Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_ES, "Deshabilitar grabaciÛn de datos cuando cambien de  FD2000/4000 RTC."},
	/* fr */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_IT, "Disattiva il salvataggio dei dati RTC del FD2000/4000 alla modifica."},
	/* ko */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_NL, "Opslaan van de FD2000/4000 RTC gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_PL, "Nie zapisuj danych zegara czasu rzeczywistego FD2000/4000 przy zmianie"},
	/* ru */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_DRIVE_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DRIVE_RAM_2000,    N_("Enable 8KB RAM expansion at $2000-$3FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DRIVE_RAM_2000_DA, "AktivÈr 8kB RAM-udviddelse pÂ $2000-$3FFF"},
	/* de */ {IDCLS_ENABLE_DRIVE_RAM_2000_DE, "8KB RAM Erweiterung bei $2000-3fff aktivieren"},
	/* es */ {IDCLS_ENABLE_DRIVE_RAM_2000_ES, "Permitir expansiÛn RAM de 8KB en $2000-$3FFF"},
	/* fr */ {IDCLS_ENABLE_DRIVE_RAM_2000_FR, "Activer l'exansion de 8KB RAM ‡ $2000-$3FFF"},
	/* hu */ {IDCLS_ENABLE_DRIVE_RAM_2000_HU, "8KB RAM kiterjeszÈs engedÈlyezÈse a $2000-$3FFF cÌmen"},
	/* it */ {IDCLS_ENABLE_DRIVE_RAM_2000_IT, "Attiva l'espansione di RAM di 8KB a $2000-$3FFF"},
	/* ko */ {IDCLS_ENABLE_DRIVE_RAM_2000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DRIVE_RAM_2000_NL, "8KB RAM uitbreiding op adres $2000-$3FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_DRIVE_RAM_2000_PL, "W≥±cz rozszerzenie 8KB RAM w $2000-$3FFF"},
	/* ru */ {IDCLS_ENABLE_DRIVE_RAM_2000_RU, "Enable 8KB RAM expansion at $2000-$3FFF"},
	/* sv */ {IDCLS_ENABLE_DRIVE_RAM_2000_SV, "Aktivera 8KB RAM-expansion pÂ $2000-$3FFF"},
	/* tr */ {IDCLS_ENABLE_DRIVE_RAM_2000_TR, "$2000-$3FFF aras˝nda 8KB RAM geni˛letmeyi aktif et"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DRIVE_RAM_2000,    N_("Disable 8KB RAM expansion at $2000-$3FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DRIVE_RAM_2000_DA, "DeaktivÈr 8kB RAM-udviddelse pÂ $2000-$3FFF"},
	/* de */ {IDCLS_DISABLE_DRIVE_RAM_2000_DE, "8KB RAM Erweiterung bei $2000-3fff deaktivieren"},
	/* es */ {IDCLS_DISABLE_DRIVE_RAM_2000_ES, "Deshabilitar expansiÛn RAM de 8KB en $2000-$3FFF"},
	/* fr */ {IDCLS_DISABLE_DRIVE_RAM_2000_FR, "DÈsactiver l'exansion de 8KB RAM ‡ $2000-$3FFF"},
	/* hu */ {IDCLS_DISABLE_DRIVE_RAM_2000_HU, "8KB RAM kiterjeszÈs tilt·sa a $2000-$3FFF cÌmen"},
	/* it */ {IDCLS_DISABLE_DRIVE_RAM_2000_IT, "Disattiva l'espansione di RAM di 8KB a $2000-$3FFF"},
	/* ko */ {IDCLS_DISABLE_DRIVE_RAM_2000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DRIVE_RAM_2000_NL, "8KB RAM uitbreiding op adres $2000-$3FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DRIVE_RAM_2000_PL, "Wy≥±cz rozszerzenie 8KB RAM w $2000-$3FFF"},
	/* ru */ {IDCLS_DISABLE_DRIVE_RAM_2000_RU, "Disable 8KB RAM expansion at $2000-$3FFF"},
	/* sv */ {IDCLS_DISABLE_DRIVE_RAM_2000_SV, "Inaktivera 8KB RAM-expansion pÂ $2000-$3FFF"},
	/* tr */ {IDCLS_DISABLE_DRIVE_RAM_2000_TR, "$2000-$3FFF aras˝nda 8KB RAM geni˛letmeyi pasifle˛tir"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DRIVE_RAM_4000,    N_("Enable 8KB RAM expansion at $4000-$5FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DRIVE_RAM_4000_DA, "AktivÈr 8kB RAM-udviddelse pÂ $4000-$5FFF"},
	/* de */ {IDCLS_ENABLE_DRIVE_RAM_4000_DE, "8KB RAM Erweiterung bei $4000-5fff aktivieren"},
	/* es */ {IDCLS_ENABLE_DRIVE_RAM_4000_ES, "Permitir expansiÛn RAM de 8KB en $4000-$5FFF"},
	/* fr */ {IDCLS_ENABLE_DRIVE_RAM_4000_FR, "Activer l'exansion de 8KB RAM ‡ $4000-$5FFF"},
	/* hu */ {IDCLS_ENABLE_DRIVE_RAM_4000_HU, "8KB RAM kiterjeszÈs engedÈlyezÈse a $4000-$5FFF cÌmen"},
	/* it */ {IDCLS_ENABLE_DRIVE_RAM_4000_IT, "Attiva l'espansione di RAM di 8KB a $4000-$5FFF"},
	/* ko */ {IDCLS_ENABLE_DRIVE_RAM_4000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DRIVE_RAM_4000_NL, "8KB RAM uitbreiding op adres $4000-$5FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_DRIVE_RAM_4000_PL, "W≥±cz rozszerzenie 8KB RAM w $4000-$5FFF"},
	/* ru */ {IDCLS_ENABLE_DRIVE_RAM_4000_RU, "Enable 8KB RAM expansion at $4000-$5FFF"},
	/* sv */ {IDCLS_ENABLE_DRIVE_RAM_4000_SV, "Aktivera 8KB RAM-expansion pÂ $4000-$5FFF"},
	/* tr */ {IDCLS_ENABLE_DRIVE_RAM_4000_TR, "$4000-$5FFF aras˝nda 8KB RAM geni˛letmeyi aktif et"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DRIVE_RAM_4000,    N_("Disable 8KB RAM expansion at $4000-$5FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DRIVE_RAM_4000_DA, "DeaktivÈr 8kB RAM-udviddelse pÂ $4000-$5FFF"},
	/* de */ {IDCLS_DISABLE_DRIVE_RAM_4000_DE, "8KB RAM Erweiterung bei $4000-5fff deaktivieren"},
	/* es */ {IDCLS_DISABLE_DRIVE_RAM_4000_ES, "Deshabilitar expansiÛn RAM de 8KB en $4000-$5FFF"},
	/* fr */ {IDCLS_DISABLE_DRIVE_RAM_4000_FR, "DÈsactiver l'exansion de 8KB RAM ‡ $4000-$5FFF"},
	/* hu */ {IDCLS_DISABLE_DRIVE_RAM_4000_HU, "8KB RAM kiterjeszÈs tilt·sa a $4000-$5FFF cÌmen"},
	/* it */ {IDCLS_DISABLE_DRIVE_RAM_4000_IT, "Disattiva l'espansione di RAM di 8KB a $4000-$5FFF"},
	/* ko */ {IDCLS_DISABLE_DRIVE_RAM_4000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DRIVE_RAM_4000_NL, "8KB RAM uitbreiding op adres $4000-$5FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DRIVE_RAM_4000_PL, "Wy≥±cz rozszerzenie 8KB RAM w $4000-$5FFF"},
	/* ru */ {IDCLS_DISABLE_DRIVE_RAM_4000_RU, "Disable 8KB RAM expansion at $4000-$5FFF"},
	/* sv */ {IDCLS_DISABLE_DRIVE_RAM_4000_SV, "Inaktivera 8KB RAM-expansion pÂ $4000-$5FFF"},
	/* tr */ {IDCLS_DISABLE_DRIVE_RAM_4000_TR, "$4000-$5FFF aras˝nda 8KB RAM geni˛letmeyi pasifle˛tir"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DRIVE_RAM_6000,    N_("Enable 8KB RAM expansion at $6000-$7FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DRIVE_RAM_6000_DA, "AktivÈr 8kB RAM-udviddelse pÂ $6000-$7FFF"},
	/* de */ {IDCLS_ENABLE_DRIVE_RAM_6000_DE, "8KB RAM Erweiterung bei $6000-7fff aktivieren"},
	/* es */ {IDCLS_ENABLE_DRIVE_RAM_6000_ES, "Permitir expansiÛn RAM de 8KB en $6000-$7FFF"},
	/* fr */ {IDCLS_ENABLE_DRIVE_RAM_6000_FR, "Activer l'exansion de 8KB RAM ‡ $6000-$7FFF"},
	/* hu */ {IDCLS_ENABLE_DRIVE_RAM_6000_HU, "8KB RAM kiterjeszÈs engedÈlyezÈse a $6000-$7FFF cÌmen"},
	/* it */ {IDCLS_ENABLE_DRIVE_RAM_6000_IT, "Attiva l'espansione di RAM di 8KB a $6000-$7FFF"},
	/* ko */ {IDCLS_ENABLE_DRIVE_RAM_6000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DRIVE_RAM_6000_NL, "8KB RAM uitbreiding op adres $6000-$7FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_DRIVE_RAM_6000_PL, "W≥±cz rozszerzenie 8KB RAM w $6000-$7FFF"},
	/* ru */ {IDCLS_ENABLE_DRIVE_RAM_6000_RU, "Enable 8KB RAM expansion at $6000-$7FFF"},
	/* sv */ {IDCLS_ENABLE_DRIVE_RAM_6000_SV, "Aktivera 8KB RAM-expansion pÂ $6000-$7FFF"},
	/* tr */ {IDCLS_ENABLE_DRIVE_RAM_6000_TR, "$6000-$7FFF aras˝nda 8KB RAM geni˛letmeyi aktif et"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DRIVE_RAM_6000,    N_("Disable 8KB RAM expansion at $6000-$7FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DRIVE_RAM_6000_DA, "DeaktivÈr 8kB RAM-udviddelse pÂ $6000-$7FFF"},
	/* de */ {IDCLS_DISABLE_DRIVE_RAM_6000_DE, "8KB RAM Erweiterung bei $6000-7fff deaktivieren"},
	/* es */ {IDCLS_DISABLE_DRIVE_RAM_6000_ES, "Deshabilitar expansiÛn RAM de 8KB en $6000-$7FFF"},
	/* fr */ {IDCLS_DISABLE_DRIVE_RAM_6000_FR, "DÈsactiver l'exansion de 8KB RAM ‡ $6000-$7FFF"},
	/* hu */ {IDCLS_DISABLE_DRIVE_RAM_6000_HU, "8KB RAM kiterjeszÈs tilt·sa a $6000-$7FFF cÌmen"},
	/* it */ {IDCLS_DISABLE_DRIVE_RAM_6000_IT, "Disattiva l'espansione di RAM di 8KB a $6000-$7FFF"},
	/* ko */ {IDCLS_DISABLE_DRIVE_RAM_6000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DRIVE_RAM_6000_NL, "8KB RAM uitbreiding op adres $6000-$7FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DRIVE_RAM_6000_PL, "Wy≥±cz rozszerzenie 8KB RAM w $6000-$7FFF"},
	/* ru */ {IDCLS_DISABLE_DRIVE_RAM_6000_RU, "Disable 8KB RAM expansion at $6000-$7FFF"},
	/* sv */ {IDCLS_DISABLE_DRIVE_RAM_6000_SV, "Inaktivera 8KB RAM-expansion pÂ $6000-$7FFF"},
	/* tr */ {IDCLS_DISABLE_DRIVE_RAM_6000_TR, "$6000-$7FFF aras˝nda 8KB RAM geni˛letmeyi pasifle˛tir"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DRIVE_RAM_8000,    N_("Enable 8KB RAM expansion at $8000-$9FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DRIVE_RAM_8000_DA, "AktivÈr 8kB RAM-udviddelse pÂ $8000-$9FFF"},
	/* de */ {IDCLS_ENABLE_DRIVE_RAM_8000_DE, "8KB RAM Erweiterung bei $8000-9fff aktivieren"},
	/* es */ {IDCLS_ENABLE_DRIVE_RAM_8000_ES, "Permitir expansiÛn RAM de 8KB en $8000-$9FFF"},
	/* fr */ {IDCLS_ENABLE_DRIVE_RAM_8000_FR, "Activer l'exansion de 8KB RAM ‡ $8000-$9FFF"},
	/* hu */ {IDCLS_ENABLE_DRIVE_RAM_8000_HU, "8KB RAM kiterjeszÈs engedÈlyezÈse a $8000-$9FFF cÌmen"},
	/* it */ {IDCLS_ENABLE_DRIVE_RAM_8000_IT, "Attiva l'espansione di RAM di 8KB a $8000-$9FFF"},
	/* ko */ {IDCLS_ENABLE_DRIVE_RAM_8000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DRIVE_RAM_8000_NL, "8KB RAM uitbreiding op adres $8000-$9FFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_DRIVE_RAM_8000_PL, "W≥±cz rozszerzenie 8KB RAM w $8000-$9FFF"},
	/* ru */ {IDCLS_ENABLE_DRIVE_RAM_8000_RU, "Enable 8KB RAM expansion at $8000-$9FFF"},
	/* sv */ {IDCLS_ENABLE_DRIVE_RAM_8000_SV, "Aktivera 8KB RAM-expansion pÂ $8000-$9FFF"},
	/* tr */ {IDCLS_ENABLE_DRIVE_RAM_8000_TR, "$8000-$9FFF aras˝nda 8KB RAM geni˛letmeyi aktif et"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DRIVE_RAM_8000,    N_("Disable 8KB RAM expansion at $8000-$9FFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DRIVE_RAM_8000_DA, "DeaktivÈr 8kB RAM-udviddelse pÂ $8000-$9FFF"},
	/* de */ {IDCLS_DISABLE_DRIVE_RAM_8000_DE, "8KB RAM Erweiterung bei $8000-9fff deaktivieren"},
	/* es */ {IDCLS_DISABLE_DRIVE_RAM_8000_ES, "Deshabilitar expansiÛn RAM de 8KB en $8000-$9FFF"},
	/* fr */ {IDCLS_DISABLE_DRIVE_RAM_8000_FR, "DÈsactiver l'exansion de 8KB RAM ‡ $8000-$9FFF"},
	/* hu */ {IDCLS_DISABLE_DRIVE_RAM_8000_HU, "8KB RAM kiterjeszÈs tilt·sa a $8000-$9FFF cÌmen"},
	/* it */ {IDCLS_DISABLE_DRIVE_RAM_8000_IT, "Disattiva l'espansione di RAM di 8KB a $8000-$9FFF"},
	/* ko */ {IDCLS_DISABLE_DRIVE_RAM_8000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DRIVE_RAM_8000_NL, "8KB RAM uitbreiding op adres $8000-$9FFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DRIVE_RAM_8000_PL, "Wy≥±cz rozszerzenie 8KB RAM w $8000-$9FFF"},
	/* ru */ {IDCLS_DISABLE_DRIVE_RAM_8000_RU, "Disable 8KB RAM expansion at $8000-$9FFF"},
	/* sv */ {IDCLS_DISABLE_DRIVE_RAM_8000_SV, "Inaktivera 8KB RAM-expansion pÂ $8000-$9FFF"},
	/* tr */ {IDCLS_DISABLE_DRIVE_RAM_8000_TR, "$8000-$9FFF aras˝nda 8KB RAM geni˛letmeyi pasifle˛tir"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DRIVE_RAM_A000,    N_("Enable 8KB RAM expansion at $A000-$BFFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DRIVE_RAM_A000_DA, "AktivÈr 8kB RAM-udviddelse pÂ $A000-$BFFF"},
	/* de */ {IDCLS_ENABLE_DRIVE_RAM_A000_DE, "8KB RAM Erweiterung bei $A000-Bfff aktivieren"},
	/* es */ {IDCLS_ENABLE_DRIVE_RAM_A000_ES, "Permitir expansiÛn RAM de 8KB en $A000-$BFFF"},
	/* fr */ {IDCLS_ENABLE_DRIVE_RAM_A000_FR, "Activer l'exansion de 8KB RAM ‡ $A000-$BFFF"},
	/* hu */ {IDCLS_ENABLE_DRIVE_RAM_A000_HU, "8KB RAM kiterjeszÈs engedÈlyezÈse a $A000-$BFFF cÌmen"},
	/* it */ {IDCLS_ENABLE_DRIVE_RAM_A000_IT, "Attiva l'espansione di RAM di 8KB a $A000-$BFFF"},
	/* ko */ {IDCLS_ENABLE_DRIVE_RAM_A000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DRIVE_RAM_A000_NL, "8KB RAM uitbreiding op adres $A000-$BFFF inschakelen"},
	/* pl */ {IDCLS_ENABLE_DRIVE_RAM_A000_PL, "W≥±cz rozszerzenie 8KB RAM w $A000-$BFFF"},
	/* ru */ {IDCLS_ENABLE_DRIVE_RAM_A000_RU, "Enable 8KB RAM expansion at $A000-$BFFF"},
	/* sv */ {IDCLS_ENABLE_DRIVE_RAM_A000_SV, "Aktivera 8KB RAM-expansion pÂ $A000-$BFFF"},
	/* tr */ {IDCLS_ENABLE_DRIVE_RAM_A000_TR, "$A000-$BFFF aras˝nda 8KB RAM geni˛letmeyi aktif et"},
#endif
	
	/* drive/iec/iec-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DRIVE_RAM_A000,    N_("Disable 8KB RAM expansion at $A000-$BFFF")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DRIVE_RAM_A000_DA, "DeaktivÈr 8kB RAM-udviddelse pÂ $A000-$BFFF"},
	/* de */ {IDCLS_DISABLE_DRIVE_RAM_A000_DE, "8KB RAM Erweiterung bei $A000-Bfff deaktivieren"},
	/* es */ {IDCLS_DISABLE_DRIVE_RAM_A000_ES, "Deshabilitar expansiÛn RAM de 8KB en $A000-$BFFF"},
	/* fr */ {IDCLS_DISABLE_DRIVE_RAM_A000_FR, "DÈsactiver l'exansion de 8KB RAM ‡ $A000-$BFFF"},
	/* hu */ {IDCLS_DISABLE_DRIVE_RAM_A000_HU, "8KB RAM kiterjeszÈs tilt·sa a $A000-$BFFF cÌmen"},
	/* it */ {IDCLS_DISABLE_DRIVE_RAM_A000_IT, "Disattiva l'espansione di RAM di 8KB a $A000-$BFFF"},
	/* ko */ {IDCLS_DISABLE_DRIVE_RAM_A000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DRIVE_RAM_A000_NL, "8KB RAM uitbreiding op adres $A000-$BFFF uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DRIVE_RAM_A000_PL, "Wy≥±cz rozszerzenie 8KB RAM w $A000-$BFFF"},
	/* ru */ {IDCLS_DISABLE_DRIVE_RAM_A000_RU, "Disable 8KB RAM expansion at $A000-$BFFF"},
	/* sv */ {IDCLS_DISABLE_DRIVE_RAM_A000_SV, "Inaktivera 8KB RAM-expansion pÂ $A000-$BFFF"},
	/* tr */ {IDCLS_DISABLE_DRIVE_RAM_A000_TR, "$A000-$BFFF aras˝nda 8KB RAM geni˛letmeyi pasifle˛tir"},
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_PAR_CABLE_C64EXP_TYPE,    N_("Set parallel cable type (0: none, 1: standard, 2: Dolphin DOS 3, 3: Formel 64)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PAR_CABLE_C64EXP_TYPE_DA, "VÊlg typen af parallelkabel (0: ingen, 1: standard, 2: Dolphin DOS 3 3: Formel 64)"},
	/* de */ {IDCLS_PAR_CABLE_C64EXP_TYPE_DE, "Parallel Kabel Typ (0: Kein, 1: Standard, 2: Dolphin DOS 3, 3: Formel 64)"},
	/* es */ {IDCLS_PAR_CABLE_C64EXP_TYPE_ES, "Seleccionar tipo de cable paralelo (0:no, 1:est·ndar, 2:Dolphin DOS 3, 3:Formel 64)"},
	/* fr */ {IDCLS_PAR_CABLE_C64EXP_TYPE_FR, "DÈfinir le type de c‚ble parallËle (0: aucun, 1: standard, 2: Dolphin DOS 3, 3: Formel 64)"},
	/* hu */ {IDCLS_PAR_CABLE_C64EXP_TYPE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PAR_CABLE_C64EXP_TYPE_IT, "Imposta il tipo di cavo parallelo (0: none, 1: standard, 2: Dolphin DOS 3, 3: Formel 64)"},
	/* ko */ {IDCLS_PAR_CABLE_C64EXP_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PAR_CABLE_C64EXP_TYPE_NL, "Stel parallelle kabelsoort in (0: geen, 1: standaard, 2: Dolphin DOS 3, 3: Formel 64)"},
	/* pl */ {IDCLS_PAR_CABLE_C64EXP_TYPE_PL, "Okre∂l typ ≥±cza rÛwnoleg≥ego (0: brak, 1: zwyk≥e, 2: Dolphin DOS 3, 3: Formel 64)"},
	/* ru */ {IDCLS_PAR_CABLE_C64EXP_TYPE_RU, "Set parallel cable type (0: none, 1: standard, 2: Dolphin DOS 3, 3: Formel 64)"},
	/* sv */ {IDCLS_PAR_CABLE_C64EXP_TYPE_SV, "V‰lj typ av parallellkabel (0: ingen, 1: standard, 2: Dolphin DOS 3, 3: Formel 64)"},
	/* tr */ {IDCLS_PAR_CABLE_C64EXP_TYPE_TR, "Paralel kablo tipini belirle (0: yok, 1: standart, 2: Dolphin DOS 3, 3: Formel 64)"},
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_PROFDOS,    N_("Enable Professional DOS")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_PROFDOS_DA, "AktivÈr Professional DOS"},
	/* de */ {IDCLS_ENABLE_PROFDOS_DE, "Professional DOS aktivieren"},
	/* es */ {IDCLS_ENABLE_PROFDOS_ES, "Permitir DOS profesional"},
	/* fr */ {IDCLS_ENABLE_PROFDOS_FR, "Activer le DOS Professionnel"},
	/* hu */ {IDCLS_ENABLE_PROFDOS_HU, "Professional DOS engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_PROFDOS_IT, "Attiva Professional DOS"},
	/* ko */ {IDCLS_ENABLE_PROFDOS_KO, "Professional DOS ªÁøÎ∞°¥…«œ∞‘ «œ±‚"},
	/* nl */ {IDCLS_ENABLE_PROFDOS_NL, "Activeer Professional DOS"},
	/* pl */ {IDCLS_ENABLE_PROFDOS_PL, "W≥±cz Professional DOS"},
	/* ru */ {IDCLS_ENABLE_PROFDOS_RU, "Enable Professional DOS"},
	/* sv */ {IDCLS_ENABLE_PROFDOS_SV, "Aktivera Professional DOS"},
	/* tr */ {IDCLS_ENABLE_PROFDOS_TR, "Professional DOS'u aktif et"},
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_PROFDOS,    N_("Disable Professional DOS")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_PROFDOS_DA, "DeaktivÈr Professional DOS"},
	/* de */ {IDCLS_DISABLE_PROFDOS_DE, "Professional DOS deaktivieren"},
	/* es */ {IDCLS_DISABLE_PROFDOS_ES, "Deshabilitar Professional DOS"},
	/* fr */ {IDCLS_DISABLE_PROFDOS_FR, "DÈsactiver le DOS Professionnel"},
	/* hu */ {IDCLS_DISABLE_PROFDOS_HU, "Professional DOS tilt·sa"},
	/* it */ {IDCLS_DISABLE_PROFDOS_IT, "Disattiva Professional DOS"},
	/* ko */ {IDCLS_DISABLE_PROFDOS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_PROFDOS_NL, "Professional DOS uitschakelen"},
	/* pl */ {IDCLS_DISABLE_PROFDOS_PL, "Wy≥±cz Professional DOS"},
	/* ru */ {IDCLS_DISABLE_PROFDOS_RU, "Disable Professional DOS"},
	/* sv */ {IDCLS_DISABLE_PROFDOS_SV, "Inaktivera Professional DOS"},
	/* tr */ {IDCLS_DISABLE_PROFDOS_TR, "Professional DOS'u pasifle˛tir"},
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME,    N_("Specify name of Professional DOS 1571 ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_DA, "Angiv navn pÂ Professional DOS 1571-ROM-image"},
	/* de */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_DE, "Name von Professional DOS 1571 Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_ES, "Especificar nombre imagen ROM Professional DOS 1571"},
	/* fr */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_FR, "SpÈcifier le nom du nouveau ROM Professional DOS 1571"},
	/* hu */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_HU, "Adja meg a Professional DOS 1571 ROM kÈpm·st"},
	/* it */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_IT, "Specifica il nome immagine della ROM del Professional DOS 1571"},
	/* ko */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_NL, "Geef de naam van het Professional DOS 1571 ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Professional DOS 1571"},
	/* ru */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_RU, "Specify name of Professional DOS 1571 ROM image"},
	/* sv */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_SV, "Ange namn pÂ Professional DOS 1571-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_PROFDOS_1571_ROM_NAME_TR, "Professional DOS 1571 ROM imaj ismini belirt"},
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_SUPERCARD,    N_("Enable SuperCard+")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SUPERCARD_DA, "AktivÈr SuperCard+"},
	/* de */ {IDCLS_ENABLE_SUPERCARD_DE, "SuperCard+ aktivieren"},
	/* es */ {IDCLS_ENABLE_SUPERCARD_ES, "Permitir SuperCard+"},
	/* fr */ {IDCLS_ENABLE_SUPERCARD_FR, "Activer SuperCard+"},
	/* hu */ {IDCLS_ENABLE_SUPERCARD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SUPERCARD_IT, "Attiva SuperCard+"},
	/* ko */ {IDCLS_ENABLE_SUPERCARD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SUPERCARD_NL, "Activeer SuperCard+"},
	/* pl */ {IDCLS_ENABLE_SUPERCARD_PL, "W≥±cz SuperCard+"},
	/* ru */ {IDCLS_ENABLE_SUPERCARD_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SUPERCARD_SV, "Aktivera SuperCard+"},
	/* tr */ {IDCLS_ENABLE_SUPERCARD_TR, "SuperCard+'˝ aktif et"},
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_SUPERCARD,    N_("Disable SuperCard+")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SUPERCARD_DA, "DeaktivÈr SuperCard+"},
	/* de */ {IDCLS_DISABLE_SUPERCARD_DE, "SuperCard+ deaktivieren"},
	/* es */ {IDCLS_DISABLE_SUPERCARD_ES, "Deshabilitar SuperCard+"},
	/* fr */ {IDCLS_DISABLE_SUPERCARD_FR, "DÈsactiver SuperCard+"},
	/* hu */ {IDCLS_DISABLE_SUPERCARD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SUPERCARD_IT, "Disattiva SuperCard+"},
	/* ko */ {IDCLS_DISABLE_SUPERCARD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SUPERCARD_NL, "SuperCard+ uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SUPERCARD_PL, "Wy≥±cz SuperCard+"},
	/* ru */ {IDCLS_DISABLE_SUPERCARD_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SUPERCARD_SV, "Inaktivera SuperCard+"},
	/* tr */ {IDCLS_DISABLE_SUPERCARD_TR, "SuperCard+'˝ pasifle˛tir"},
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME,    N_("Specify name of SuperCard+ ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_DA, "Angiv navn pÂ SuperCard+ ROM-image"},
	/* de */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_DE, "Name von SuperCard+ ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_ES, "Especificar nombre de Imagen ROM SuperCard+"},
	/* fr */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_FR, "SpÈcifier le nom de l'image ROM SuperCard+"},
	/* hu */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_IT, "Specifica il nome immagine della ROM del SuperCard+"},
	/* ko */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_NL, "Geef de naam van het SuperCard+ ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM SuperCard+"},
	/* ru */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_SV, "Ange namn pÂ SuperCard+-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_SUPERCARD_ROM_NAME_TR, "SuperCard+ ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_STARDOS,    N_("Enable StarDOS")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_STARDOS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_STARDOS_DE, ""},  /* fuzzy */
	/* es */ {IDCLS_ENABLE_STARDOS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_STARDOS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_STARDOS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_STARDOS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_STARDOS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_STARDOS_NL, "Activeer StarDOS"},
	/* pl */ {IDCLS_ENABLE_STARDOS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_STARDOS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_STARDOS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_STARDOS_TR, ""},  /* fuzzy */
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_STARDOS,    N_("Disable StarDOS")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_STARDOS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_STARDOS_DE, ""},  /* fuzzy */
	/* es */ {IDCLS_DISABLE_STARDOS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_STARDOS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_STARDOS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_STARDOS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_STARDOS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_STARDOS_NL, "StarDOS uitschakelen"},
	/* pl */ {IDCLS_DISABLE_STARDOS_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_STARDOS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_STARDOS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_STARDOS_TR, ""},  /* fuzzy */
#endif
	
	/* drive/iec/c64exp/c64exp-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_STARDOS_ROM_NAME,    N_("Specify name of StarDOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_DE, ""},  /* fuzzy */
	/* es */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_NL, "Geef de naam van het StarDOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_STARDOS_ROM_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* drive/iec/plus4exp/plus4exp-cmdline-options.c */
	/* en */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE,    N_("Set parallel cable type (0: none, 1: standard)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_DA, "VÊlg typen af parallelkabel parallellkabel (0: ingen, 1: standard)"},
	/* de */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_DE, "Parallelkabel Typ (0: kein, 1: Standard)"},
	/* es */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_ES, "Seleccionar tipo de cable paralelo (0:no, 1:standard)"},
	/* fr */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_FR, "DÈfinir le type de c‚ble parallËle (0: aucun, 1: standard)"},
	/* hu */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_HU, "Adja meg a p·rhuzamos k·bel tÌpus·t (0: nincs, 1: standard)"},
	/* it */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_IT, "Imposta il tipo di cavo parallelo (0: nessuno, 1: standard)"},
	/* ko */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_NL, "Stel parallelle kabelsoort in (0: geen, 1: standaard)"},
	/* pl */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_PL, "Ustaw typ kabla portu rÛwnoleg≥ego (0: brak, 1: typowy)"},
	/* ru */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_RU, "Set parallel cable type (0: none, 1: standard)"},
	/* sv */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_SV, "V‰lj typ av parallellkabel (0: ingen, 1: standard)"},
	/* tr */ {IDCLS_PAR_CABLE_PLUS4EXP_TYPE_TR, "Paralel kablo tipini ayarlay˝n (0: yok, 1: standart)"},
#endif
	
	/* drive/iec128dcr/iec128dcr-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME,    N_("Specify name of 1571CR DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_DA, "Angiv navn pÂ 1571CR-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_DE, "Name der 1571CR DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_ES, "Especificar nombre imagen DOS ROM 1571CR"},
	/* fr */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 1571CR DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_HU, "Adja meg az 1571CR DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1571CR"},
	/* ko */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_NL, "Geef de naam van het 1571CR DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1571CR"},
	/* ru */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_RU, "Specify name of 1571CR DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_SV, "Ange namn pÂ 1571CR-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_1571CR_DOS_ROM_NAME_TR, "1571CR DOS ROM imaj ismini belirt"},
#endif
	
	/* drive/ieee/ieee-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME,    N_("Specify name of 2031 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_DA, "Angiv navn pÂ 2031-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_DE, "Name der 2031 DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_ES, "Especificar nombre imagen DOS ROM 2031"},
	/* fr */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 2031 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_HU, "Adja meg a 2031 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 2031"},
	/* ko */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_NL, "Geef de naam van het 2031 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 2031"},
	/* ru */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_RU, "Specify name of 2031 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_SV, "Ange namn pÂ 2031-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_2031_DOS_ROM_NAME_TR, "2031 DOS ROM imaj ismini belirt"},
#endif
	
	/* drive/ieee/ieee-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME,    N_("Specify name of 2040 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_DA, "Angiv navn pÂ 2040-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_DE, "Name der 2040 DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_ES, "Especificar nombre imagen DOS ROM 2040"},
	/* fr */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 2040 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_HU, "Adja meg a 2040 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 2040"},
	/* ko */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_NL, "Geef de naam van het 2040 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 2040"},
	/* ru */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_RU, "Specify name of 2040 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_SV, "Ange namn pÂ 2040-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_2040_DOS_ROM_NAME_TR, "2040 DOS ROM imaj ismini belirt"},
#endif
	
	/* drive/ieee/ieee-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME,    N_("Specify name of 3040 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_DA, "Angiv navn pÂ 3040-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_DE, "Name der 3040 DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_ES, "Especificar nombre imagen DOS ROM 3040"},
	/* fr */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 3040 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_HU, "Adja meg a 3040 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 3040"},
	/* ko */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_NL, "Geef de naam van het 3040 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 3040"},
	/* ru */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_RU, "Specify name of 3040 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_SV, "Ange namn pÂ 3040-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_3040_DOS_ROM_NAME_TR, "3040 DOS ROM imaj ismini belirt"},
#endif
	
	/* drive/ieee/ieee-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME,    N_("Specify name of 4040 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_DA, "Angiv navn pÂ 4040-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_DE, "Name der 4040 DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_ES, "Especificar nombre imagen DOS ROM 4040"},
	/* fr */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 4040 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_HU, "Adja meg a 4040 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 4040"},
	/* ko */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_NL, "Geef de naam van het 4040 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 4040"},
	/* ru */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_RU, "Specify name of 4040 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_SV, "Ange namn pÂ 4040-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_4040_DOS_ROM_NAME_TR, "4040 DOS ROM imaj ismini belirt"},
#endif
	
	/* drive/ieee/ieee-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME,    N_("Specify name of 1001/8050/8250 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_DA, "Angiv navn pÂ 1001/8050/8250-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_DE, "Name der 1001/8040/8205 DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_ES, "Especificar nombre imagen DOS ROM 1001/8050/8250"},
	/* fr */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 1001/8050/8250 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_HU, "Adja meg az 1001/8050/8250 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1001/8050/8250"},
	/* ko */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_NL, "Geef de naam van het 1001/8050/8250 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1001/8050/8250"},
	/* ru */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_RU, "Specify name of 1001/8050/8250 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_SV, "Ange namn pÂ 1001/8050/8250-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_1001_DOS_ROM_NAME_TR, "1001/8050/8250 DOS ROM imaj ismini belirt"},
#endif
	
	/* drive/tcbm/tcbm-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME,    N_("Specify name of 1551 DOS ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_DA, "Angiv navn pÂ 1551-DOS-ROM-image"},
	/* de */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_DE, "Name der 1551 DOS ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_ES, "Especificar nombre imagen DOS ROM 1551"},
	/* fr */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_FR, "SpÈcifier le nom de l'image 1551 DOS ROM"},
	/* hu */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_HU, "Adja meg az 1551 DOS ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_IT, "Specifica il nome immagine della ROM del DOS del 1551"},
	/* ko */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_NL, "Geef de naam van het 1551 DOS ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM DOS-a 1551"},
	/* ru */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_RU, "Specify name of 1551 DOS ROM image"},
	/* sv */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_SV, "Ange namn pÂ 1551-DOS-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_1551_DOS_ROM_NAME_TR, "1551 DOS ROM imaj ismini belirt"},
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8,    N_("Use <name> as directory for file system device #8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_DA, "Brug <navn> som katalog for filsystembaseret drev #8"},
	/* de */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_DE, "Benutze <Name> f¸r Verzeichnis beim Verzeichniszugriff f¸r Ger‰t #8"},
	/* es */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_ES, "Usar <nombre> como directorio fichero sistema perifÈrico #8"},
	/* fr */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_FR, "Utiliser le rÈpertoire <nom> comme systËme de fichiers pour le lecteur #8"},
	/* hu */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_HU, "<nÈv> kˆnyvt·r haszn·lata az #8-as egysÈg f·jlrendszerÈnek"},
	/* it */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_IT, "Usa <nome> come directory per il file system della periferica #8"},
	/* ko */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_NL, "Gebruik <naam> als directory voor bestandssysteemapparaat #8"},
	/* pl */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_PL, "Uzyj <nazwa> jako katalog urz±dzenia systemu plikÛw #8"},
	/* ru */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_RU, "Use <name> as directory for file system device #8"},
	/* sv */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_SV, "Ange <namn> som katalog fˆr filsystemsbaserad enhet 8"},
	/* tr */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_8_TR, "<isim>'i ayg˝t #8'in dosya sistemi dizini olarak kullan"},
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9,    N_("Use <name> as directory for file system device #9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_DA, "Brug <navn> som katalog for filsystembaseret drev #9"},
	/* de */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_DE, "Benutze <Name> f¸r Verzeichnis beim Verzeichniszugriff f¸r Ger‰t #9"},
	/* es */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_ES, "Usar <nombre> como directorio fichero sistema perifÈrico #9"},
	/* fr */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_FR, "Utiliser le rÈpertoire <nom> comme systËme de fichiers pour le lecteur #9"},
	/* hu */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_HU, "<nÈv> kˆnyvt·r haszn·lata az #9-es egysÈg f·jlrendszerÈnek"},
	/* it */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_IT, "Usa <nome> come directory per il file system della periferica #9"},
	/* ko */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_NL, "Gebruik <naam> als directory voor bestandssysteemapparaat #9"},
	/* pl */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_PL, "Uzyj <nazwa> jako katalog urz±dzenia systemu plikÛw #9"},
	/* ru */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_RU, "Use <name> as directory for file system device #9"},
	/* sv */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_SV, "Ange <namn> som katalog fˆr filsystemsbaserad enhet 9"},
	/* tr */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_9_TR, "<isim>'i ayg˝t #9'un dosya sistemi dizini olarak kullan"},
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10,    N_("Use <name> as directory for file system device #10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_DA, "Brug <navn> som katalog for filsystembaseret drev #10"},
	/* de */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_DE, "Benutze <Name> f¸r Verzeichnis beim Verzeichniszugriff f¸r Ger‰t #10"},
	/* es */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_ES, "Usar <nombre> como directorio fichero sistema perifÈrico #10"},
	/* fr */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_FR, "Utiliser le rÈpertoire <nom> comme systËme de fichiers pour le lecteur #10"},
	/* hu */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_HU, "<nÈv> kˆnyvt·r haszn·lata az #10-es egysÈg f·jlrendszerÈnek"},
	/* it */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_IT, "Usa <nome> come directory per il file system della periferica #10"},
	/* ko */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_NL, "Gebruik <naam> als directory voor bestandssysteemapparaat #10"},
	/* pl */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_PL, "Uzyj <nazwa> jako katalog urz±dzenia systemu plikÛw #10"},
	/* ru */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_RU, "Use <name> as directory for file system device #10"},
	/* sv */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_SV, "Ange <namn> som katalog fˆr filsystemsbaserad enhet 10"},
	/* tr */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_10_TR, "<isim>'i ayg˝t #10'un dosya sistemi dizini olarak kullan"},
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11,    N_("Use <name> as directory for file system device #11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_DA, "Brug <navn> som katalog for filsystembaseret drev #11"},
	/* de */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_DE, "Benutze <Name> f¸r Verzeichnis beim Verzeichniszugriff f¸r Ger‰t #11"},
	/* es */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_ES, "Usar <nombre> como directorio fichero sistema perifÈrico #10"},
	/* fr */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_FR, "Utiliser le rÈpertoire <nom> comme systËme de fichiers pour le lecteur #11"},
	/* hu */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_HU, "<nÈv> kˆnyvt·r haszn·lata az #11-es egysÈg f·jlrendszerÈnek"},
	/* it */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_IT, "Usa <nome> come directory per il file system della periferica #11"},
	/* ko */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_NL, "Gebruik <naam> als directory voor bestandssysteemapparaat #11"},
	/* pl */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_PL, "Uzyj <nazwa> jako katalog urz±dzenia systemu plikÛw #11"},
	/* ru */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_RU, "Use <name> as directory for file system device #11"},
	/* sv */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_SV, "Ange <namn> som katalog fˆr filsystemsbaserad enhet 11"},
	/* tr */ {IDCLS_USE_AS_DIRECTORY_FSDEVICE_11_TR, "<isim>'i ayg˝t #11'in dosya sistemi dizini olarak kullan"},
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00,    N_("Enable on-read support for P00 files on drive 8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_DE, "On-read Unterst¸tzung f¸r P00 Dateien auf Laufwerk 8 aktivieren"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_NL, "Ondersteuning voor het lezen van P00 bestanden voor drive 8 inschakelen"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_PL, "W≥±cz obs≥ugÍ plikÛw P00 przy odczycie z napÍdu 8"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_8_CONVERT_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00,    N_("Disable on-read support for P00 files on drive 8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_DE, "On-read Unterst¸tzung f¸r P00 Dateien auf Laufwerk 8 deaktivieren"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_NL, "Ondersteuning voor het lezen van P00 bestanden voor drive 8 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_PL, "Wy≥±cz obs≥ugÍ plikÛw P00 przy odczycie z napÍdu 8"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_8_CONVERT_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00,    N_("Enable on-read support for P00 files on drive 9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_DE, "On-read Unterst¸tzung f¸r P00 Dateien auf Laufwerk 9 aktivieren"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_NL, "Ondersteuning voor het lezen van P00 bestanden voor drive 9 inschakelen"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_PL, "W≥±cz obs≥ugÍ plikÛw P00 przy odczycie z napÍdu 9"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_9_CONVERT_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00,    N_("Disable on-read support for P00 files on drive 9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_DE, "On-read Unterst¸tzung f¸r P00 Dateien auf Laufwerk 9 deaktivieren"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_NL, "Ondersteuning voor het lezen van P00 bestanden voor drive 9 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_PL, "Wy≥±cz obs≥ugÍ plikÛw P00 przy odczycie z napÍdu 9"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_9_CONVERT_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00,    N_("Enable on-read support for P00 files on drive 10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_DE, "On-read Unterst¸tzung f¸r P00 Dateien auf Laufwerk 10 aktivieren"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_NL, "Ondersteuning voor het lezen van P00 bestanden voor drive 10 inschakelen"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_PL, "W≥±cz obs≥ugÍ plikÛw P00 przy odczycie z napÍdu 10"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_10_CONVERT_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00,    N_("Disable on-read support for P00 files on drive 10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_DE, "On-read Unterst¸tzung f¸r P00 Dateien auf Laufwerk 10 deaktivieren"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_NL, "Ondersteuning voor het lezen van P00 bestanden voor drive 10 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_PL, "Wy≥±cz obs≥ugÍ plikÛw P00 przy odczycie z napÍdu 10"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_10_CONVERT_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00,    N_("Enable on-read support for P00 files on drive 11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_DE, "On-read Unterst¸tzung f¸r P00 Dateien auf Laufwerk 11 aktivieren"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_NL, "Ondersteuning voor het lezen van P00 bestanden voor drive 11 inschakelen"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_PL, "W≥±cz obs≥ugÍ plikÛw P00 przy odczycie z napÍdu 11"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_11_CONVERT_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00,    N_("Disable on-read support for P00 files on drive 11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_DE, "On-read Unterst¸tzung f¸r P00 Dateien auf Laufwerk 11 deaktivieren"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_NL, "Ondersteuning voor het lezen van P00 bestanden voor drive 11 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_PL, "Wy≥±cz obs≥ugÍ plikÛw P00 przy odczycie z napÍdu 11"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_11_CONVERT_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00,    N_("Enable saving of P00 files on drive 8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_DE, "Speichern von P00 Dateien auf Laufwerk 8 aktivieren"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_NL, "Opslaan van P00 bestanden voor drive 8 inschakelen"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_PL, "W≥±cz zapis plikÛw P00 w napÍdzie 8"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_8_SAVE_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00,    N_("Disable saving of P00 files on drive 8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_DE, "Speichern von P00 Dateien auf Laufwerk 8 deaktivieren"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_NL, "Opslaan van P00 bestanden voor drive 8 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_PL, "Wy≥±cz zapis plikÛw P00 w napÍdzie 8"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_8_SAVE_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00,    N_("Enable saving of P00 files on drive 9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_DE, "Speichern von P00 Dateien auf Laufwerk 9 aktivieren"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_NL, "Opslaan van P00 bestanden voor drive 9 inschakelen"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_PL, "W≥±cz zapis plikÛw P00 w napÍdzie 9"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_9_SAVE_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00,    N_("Disable saving of P00 files on drive 9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_DE, "Speichern von P00 Dateien auf Laufwerk 9 deaktivieren"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_NL, "Opslaan van P00 bestanden voor drive 9 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_PL, "Wy≥±cz zapis plikÛw P00 w napÍdzie 9"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_9_SAVE_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00,    N_("Enable saving of P00 files on drive 10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_DE, "Speichern von P00 Dateien auf Laufwerk 10 aktivieren"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_NL, "Opslaan van P00 bestanden voor drive 10 inschakelen"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_PL, "W≥±cz zapis plikÛw P00 w napÍdzie 10"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_10_SAVE_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00,    N_("Disable saving of P00 files on drive 10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_DE, "Speichern von P00 Dateien auf Laufwerk 10 deaktivieren"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_NL, "Opslaan van P00 bestanden voor drive 10 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_PL, "Wy≥±cz zapis plikÛw P00 w napÍdzie 10"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_10_SAVE_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00,    N_("Enable saving of P00 files on drive 11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_DE, "Speichern von P00 Dateien auf Laufwerk 11 aktivieren"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_NL, "Opslaan van P00 bestanden voor drive 11 inschakelen"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_PL, "W≥±cz zapis plikÛw P00 w napÍdzie 11"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_11_SAVE_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00,    N_("Disable saving of P00 files on drive 11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_DE, "Speichern von P00 Dateien auf Laufwerk 11 deaktivieren"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_NL, "Opslaan van P00 bestanden voor drive 11 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_PL, "Wy≥±cz zapis plikÛw P00 w napÍdzie 11"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_11_SAVE_P00_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM,    N_("Hide CBM files (only show P00 files) for drive 8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_DE, "Laufwerk 8 CBM Dateien ausblenden (nur P00 Dateien zeigen)"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_NL, "Verberg CBM bestanden (toon alleen P00 bestanden) voor drive 8"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_PL, "Ukryj pliki CBM (pokazuj tylko pliki P00) dla napÍdu 8"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_8_HIDE_CBM_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM,    N_("Do not hide CBM files (show all files) for drive 8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_DE, "Laufwerk 8 alle Dateien zeigen (CBM Dateien nicht ausblenden)"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_NL, "Verberg CBM bestanden niet (toon alle bestanden) voor drive 8"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_PL, "Nie ukrywaj plikÛw CBM (pokazuj wszystkie pliki) dla napÍdu 8"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_8_HIDE_CBM_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM,    N_("Hide CBM files (only show P00 files) for drive 9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_DE, "Laufwerk 9 CBM Dateien ausblenden (nur P00 Dateien zeigen)"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_NL, "Verberg CBM bestanden (toon alleen P00 bestanden) voor drive 9"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_PL, "Ukryj pliki CBM (pokazuj tylko pliki P00) dla napÍdu 9"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_9_HIDE_CBM_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM,    N_("Do not hide CBM files (show all files) for drive 9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_DE, "Laufwerk 9 alle Dateien zeigen (CBM Dateien nicht ausblenden)"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_NL, "Verberg CBM bestanden niet (toon alle bestanden) voor drive 9"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_PL, "Nie ukrywaj plikÛw CBM (pokazuj wszystkie pliki) dla napÍdu 9"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_9_HIDE_CBM_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM,    N_("Hide CBM files (only show P00 files) for drive 10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_DE, "Laufwerk 10 CBM Dateien ausblenden (nur P00 Dateien zeigen)"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_NL, "Verberg CBM bestanden (toon alleen P00 bestanden) voor drive 10"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_PL, "Ukryj pliki CBM (pokazuj tylko pliki P00) dla napÍdu 10"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_10_HIDE_CBM_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM,    N_("Do not hide CBM files (show all files) for drive 10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_DE, "Laufwerk 10 alle Dateien zeigen (CBM Dateien nicht ausblenden)"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_NL, "Verberg CBM bestanden niet (toon alle bestanden) voor drive 10"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_PL, "Nie ukrywaj plikÛw CBM (pokazuj wszystkie pliki) dla napÍdu 10"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_10_HIDE_CBM_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM,    N_("Hide CBM files (only show P00 files) for drive 11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_DE, "Laufwerk 11 CBM Dateien ausblenden (nur P00 Dateien zeigen)"},
	/* es */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_NL, "Verberg CBM bestanden (toon alleen P00 bestanden) voor drive 11"},
	/* pl */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_PL, "Ukryj pliki CBM (pokazuj tylko pliki P00) dla napÍdu 11"},
	/* ru */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_FSDEVICE_11_HIDE_CBM_TR, ""},  /* fuzzy */
#endif
	
	/* fsdevice/fsdevice-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM,    N_("Do not hide CBM files (show all files) for drive 11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_DE, "Laufwerk 11 alle Dateien zeigen (CBM Dateien nicht ausblenden)"},
	/* es */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_NL, "Verberg CBM bestanden niet (toon alle bestanden) voor drive 11"},
	/* pl */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_PL, "Nie ukrywaj plikÛw CBM (pokazuj wszystkie pliki) dla napÍdu 11"},
	/* ru */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_FSDEVICE_11_HIDE_CBM_TR, ""},  /* fuzzy */
#endif
	
	/* gfxoutputdrv/ffmpegdrv.c */
	/* en */ {IDCLS_SET_AUDIO_STREAM_BITRATE,    N_("Set bitrate for audio stream in media file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_AUDIO_STREAM_BITRATE_DA, "Angiv bitrate for lydstr¯m i mediefil"},
	/* de */ {IDCLS_SET_AUDIO_STREAM_BITRATE_DE, "Bitrate f¸r Audio Stream f¸r Media Datei setzen"},
	/* es */ {IDCLS_SET_AUDIO_STREAM_BITRATE_ES, "Seleccionar velocidad muestreo para flujo audio en fichero media"},
	/* fr */ {IDCLS_SET_AUDIO_STREAM_BITRATE_FR, "RÈgler le bitrate audio du fichier mÈdia"},
	/* hu */ {IDCLS_SET_AUDIO_STREAM_BITRATE_HU, "Adja meg az audiÛ bitr·t·t a mÈdia f·jlhoz"},
	/* it */ {IDCLS_SET_AUDIO_STREAM_BITRATE_IT, "Imposta il bitrate del flusso audio nel file multimediale"},
	/* ko */ {IDCLS_SET_AUDIO_STREAM_BITRATE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_AUDIO_STREAM_BITRATE_NL, "Zet de bitrate voor het audiogedeelte van het mediabestand"},
	/* pl */ {IDCLS_SET_AUDIO_STREAM_BITRATE_PL, "Okre∂l szybko∂Ê transmisji strumienia audio w pliku multimediÛw"},
	/* ru */ {IDCLS_SET_AUDIO_STREAM_BITRATE_RU, "Set bitrate for audio stream in media file"},
	/* sv */ {IDCLS_SET_AUDIO_STREAM_BITRATE_SV, "Ange bithastighet fˆr ljudstrˆm i mediafil"},
	/* tr */ {IDCLS_SET_AUDIO_STREAM_BITRATE_TR, "Ses ak˝˛˝ iÁin ortam dosyas˝ndaki bith˝z˝'n˝ ayarla"},
#endif
	
	/* gfxoutputdrv/ffmpegdrv.c */
	/* en */ {IDCLS_SET_VIDEO_STREAM_BITRATE,    N_("Set bitrate for video stream in media file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_VIDEO_STREAM_BITRATE_DA, "Angiv bitrate for videostr¯m i mediefil"},
	/* de */ {IDCLS_SET_VIDEO_STREAM_BITRATE_DE, "Bitrate f¸r Video Stream f¸r Media Datei setzen"},
	/* es */ {IDCLS_SET_VIDEO_STREAM_BITRATE_ES, "Seleccionar velocidad muestreo para flujo vÌdeo en fichero media"},
	/* fr */ {IDCLS_SET_VIDEO_STREAM_BITRATE_FR, "RÈgler le bitrate vidÈo du fichier mÈdia"},
	/* hu */ {IDCLS_SET_VIDEO_STREAM_BITRATE_HU, "Adja meg az videÛ bitr·t·t a mÈdia f·jlhoz"},
	/* it */ {IDCLS_SET_VIDEO_STREAM_BITRATE_IT, "Imposta il bitrate per il flusso video nel file multimediale"},
	/* ko */ {IDCLS_SET_VIDEO_STREAM_BITRATE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_VIDEO_STREAM_BITRATE_NL, "Zet de bitrate voor het videogedeelte van het mediabestand"},
	/* pl */ {IDCLS_SET_VIDEO_STREAM_BITRATE_PL, "Okre∂l szybko∂Ê transmisji strumienia wideo w pliku multimediÛw"},
	/* ru */ {IDCLS_SET_VIDEO_STREAM_BITRATE_RU, "Set bitrate for video stream in media file"},
	/* sv */ {IDCLS_SET_VIDEO_STREAM_BITRATE_SV, "Ange bithastighet fˆr videostrˆm i mediafil"},
	/* tr */ {IDCLS_SET_VIDEO_STREAM_BITRATE_TR, "Gˆr¸nt¸ ak˝˛˝ iÁin ortam dosyas˝ndaki bith˝z˝'n˝ ayarla"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_PET_MODEL,    N_("Specify PET model to emulate. (2001/3008/3016/3032/3032B/4016/4032/4032B/8032/8096/8296/SuperPET)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PET_MODEL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_PET_MODEL_DE, "PET model f¸r Emulation w‰hlen. (2001/3008/3016/3032/3032B/4016/4032/4032B/8032/8096/8296/SuperPET)"},
	/* es */ {IDCLS_SPECIFY_PET_MODEL_ES, "Especificar modelo PET a emular. (2001/3008/3016/3032/3032B/4016/4032/4032B/8032/8096/8296/SuperPET)"},
	/* fr */ {IDCLS_SPECIFY_PET_MODEL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_PET_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_PET_MODEL_IT, "Specifica il modello PET da emulare. (2001/3008/3016/3032/3032B/4016/4032/4032B/8032/8096/8296/SuperPET)"},
	/* ko */ {IDCLS_SPECIFY_PET_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PET_MODEL_NL, "Geef PET model om te emuleren. (2001/3008/3016/3032/3032B/4016/4032/4032B/8032/8096/8296/SuperPET)"},
	/* pl */ {IDCLS_SPECIFY_PET_MODEL_PL, "Wybierz emulowany model PET. (2001/3008/3016/3032/3032B/4016/4032/4032B/8032/8096/8296/SuperPET)"},
	/* ru */ {IDCLS_SPECIFY_PET_MODEL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_PET_MODEL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_PET_MODEL_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_EDITOR_ROM_NAME,    N_("Specify name of Editor ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_DA, "Angiv navn pÂ Editor-ROM-image"},
	/* de */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_DE, "Name f¸r Editor ROM Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_ES, "Especificar nombre imagen editor ROM"},
	/* fr */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_FR, "SpÈcifier le nom de líimage ROM Editor"},
	/* hu */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_HU, "Adja meg a szerkesztı ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_IT, "Specifica il nome immagine della Editor ROM"},
	/* ko */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_NL, "Geef naam van het Editor ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Editor"},
	/* ru */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_RU, "Specify name of Editor ROM image"},
	/* sv */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_SV, "Ange namn pÂ Editor-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_EDITOR_ROM_NAME_TR, "Editor ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME,    N_("Specify 4K extension ROM name at $9***")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_DA, "Angiv navn pÂ 4K-udviddelses-ROM pÂ $9***"},
	/* de */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_DE, "4K Erweiterungs ROM Dateiname bei $9*** w‰hlen"},
	/* es */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_ES, "Especificar nombre extensiÛn ROM 4KB en $9***"},
	/* fr */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_FR, "SpÈcifier le nom de l'extension ROM 4K ‡ $9***"},
	/* hu */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_HU, "Adja meg a $9*** cÌm˚ 4K ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_IT, "Specifica il nome della ROM di estensione di 4k a $9***"},
	/* ko */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_NL, "Geef naam van het 4K uitbreiding ROM bestand op adres $9***"},
	/* pl */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_PL, "Okre∂l nazwÍ rozszerzenia 4K ROM w $9***"},
	/* ru */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_RU, "Specify 4K extension ROM name at $9***"},
	/* sv */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_SV, "Ange namn pÂ 4K-utˆknings-ROM pÂ $9***"},
	/* tr */ {IDCLS_SPECIFY_4K_ROM_9XXX_NAME_TR, "$9*** adresindeki 4K uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME,    N_("Specify 4K extension ROM name at $A***")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_DA, "Angiv navn pÂ 4K-udviddelses-ROM pÂ $A***"},
	/* de */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_DE, "4K Erweiterungs ROM Dateiname bei $A*** w‰hlen"},
	/* es */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_ES, "Especificar nombre extensiÛn ROM 4KB en $A***"},
	/* fr */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_FR, "SpÈcifier le nom de l'extension ROM 4K ‡ $A***"},
	/* hu */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_HU, "Adja meg a $A*** cÌm˚ 4K ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_IT, "Specifica il nome della ROM di estensione di 4k a $A***"},
	/* ko */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_NL, "Geef naam van het 4K uitbreiding ROM bestand op adres $A***"},
	/* pl */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_PL, "Okre∂l nazwÍ rozszerzenia 4K ROM w $A***"},
	/* ru */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_RU, "Specify 4K extension ROM name at $A***"},
	/* sv */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_SV, "Ange namn pÂ 4K-utˆknings-ROM pÂ $A***"},
	/* tr */ {IDCLS_SPECIFY_4K_ROM_AXXX_NAME_TR, "$A*** adresindeki 4K uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME,    N_("Specify 4K extension ROM name at $B***")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_DA, "Angiv navn pÂ 4K-udviddelses-ROM pÂ $B***"},
	/* de */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_DE, "4K Erweiterungs ROM Dateiname bei $B*** w‰hlen"},
	/* es */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_ES, "Especificar nombre extensiÛn ROM 4KB en $B***"},
	/* fr */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_FR, "SpÈcifier le nom de l'extension ROM 4K ‡ $B***"},
	/* hu */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_HU, "Adja meg a $B*** cÌm˚ 4K ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_IT, "Specifica il nome della ROM di estensione di 4k a $B***"},
	/* ko */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_NL, "Geef naam van het 4K uitbreiding ROM bestand op adres $B***"},
	/* pl */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_PL, "Okre∂l nazwÍ rozszerzenia 4K ROM w $B***"},
	/* ru */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_RU, "Specify 4K extension ROM name at $B***"},
	/* sv */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_SV, "Ange namn pÂ 4K-utˆknings-ROM pÂ $B***"},
	/* tr */ {IDCLS_SPECIFY_4K_ROM_BXXX_NAME_TR, "$B*** adresindeki 4K uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000,    N_("Enable PET8296 4K RAM mapping at $9***")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_DA, "AktivÈr PET8296 4K-RAM-mapping pÂ $9***"},
	/* de */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_DE, "PET8296 4K RAM Zuordnung bei $9*** aktivieren"},
	/* es */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_ES, "Permitir mapeado 4KB de RAM PET8296 en $9***"},
	/* fr */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_FR, "Activer l'extension PET8296 4K ‡ $9***"},
	/* hu */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_HU, "PET8296 4K RAM engedÈlyezÈse a $9*** cÌmen"},
	/* it */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_IT, "Attiva la mappatura della RAM di 4K del PET8296 a $9***"},
	/* ko */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_NL, "PET8296 4K RAM op adres $9*** inschakelen"},
	/* pl */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_PL, "W≥±cz mapowanie 4K RAM PET8296 w $9***"},
	/* ru */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_RU, "Enable PET8296 4K RAM mapping at $9***"},
	/* sv */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_SV, "Aktivera PET8296 4K-RAM-mappning pÂ $9***"},
	/* tr */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_9000_TR, "$9*** adresindeki PET8296 4K RAM e˛le˛tirmesini aktif et"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000,    N_("Disable PET8296 4K RAM mapping at $9***")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_DA, "DeaktivÈr PET8296 4K-RAM-mapping pÂ $9***"},
	/* de */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_DE, "PET8296 4K RAM Zuordnung bei $9*** deaktivieren"},
	/* es */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_ES, "Deshabilitar mapeado 4KB de RAM PET8296 en $9***"},
	/* fr */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_FR, "DÈsactiver l'extension PET8296 4K ‡ $9***"},
	/* hu */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_HU, "PET8296 4K RAM tilt·sa a $9*** cÌmen"},
	/* it */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_IT, "Disattiva la mappatura della RAM di 4K del PET8296 a $9***"},
	/* ko */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_NL, "PET8296 4K RAM op adres $9*** uitschakelen"},
	/* pl */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_PL, "Wy≥±cz mapowanie 4K RAM PET8296 w $9***"},
	/* ru */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_RU, "Disable PET8296 4K RAM mapping at $9***"},
	/* sv */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_SV, "Inaktivera PET8296 4K-RAM-mappning pÂ $9***"},
	/* tr */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_9000_TR, "$9*** adresindeki PET8296 4K RAM e˛le˛tirmesini pasifle˛tir"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000,    N_("Enable PET8296 4K RAM mapping at $A***")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_DA, "AktivÈr PET8296 4K-RAM-mapping pÂ $A***"},
	/* de */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_DE, "PET8296 4K RAM Zuordnung bei $A*** aktivieren"},
	/* es */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_ES, "Permitir mapeado 4KB de RAM PET8296 en $A***"},
	/* fr */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_FR, "Activer l'extension PET8296 4K ‡ $A***"},
	/* hu */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_HU, "PET8296 4K RAM engedÈlyezÈse a $A*** cÌmen"},
	/* it */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_IT, "Attiva la mappatura della RAM di 4K del PET8296 a $A***"},
	/* ko */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_NL, "PET8296 4K RAM op adres $A*** inschakelen"},
	/* pl */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_PL, "W≥±cz mapowanie 4K RAM PET8296 w $A***"},
	/* ru */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_RU, "Enable PET8296 4K RAM mapping at $A***"},
	/* sv */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_SV, "Aktivera PET8296 4K-RAM-mappning pÂ $A***"},
	/* tr */ {IDCLS_PET8296_ENABLE_4K_RAM_AT_A000_TR, "$A*** adresindeki PET8296 4K RAM e˛le˛tirmesini aktif et"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000,    N_("Disable PET8296 4K RAM mapping at $A***")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_DA, "DeaktivÈr PET8296 4K-RAM-mapping pÂ $A***"},
	/* de */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_DE, "PET8296 4K RAM Zuordnung bei $A*** deaktivieren"},
	/* es */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_ES, "Deshabilitar mapeado 4KB de RAM PET8296 en $A***"},
	/* fr */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_FR, "DÈsactiver l'extension PET8296 4K ‡ $A***"},
	/* hu */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_HU, "PET8296 4K RAM tilt·sa a $A*** cÌmen"},
	/* it */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_IT, "Disattiva la mappatura della RAM di 4K del PET8296 a $A***"},
	/* ko */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_NL, "PET8296 4K RAM op adres $A*** uitschakelen"},
	/* pl */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_PL, "Wy≥±cz mapowanie 4K RAM PET8296 w $A***"},
	/* ru */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_RU, "Disable PET8296 4K RAM mapping at $A***"},
	/* sv */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_SV, "Inaktivera PET8296 4K-RAM-mappning pÂ $A***"},
	/* tr */ {IDCLS_PET8296_DISABLE_4K_RAM_AT_A000_TR, "$A*** adresindeki PET8296 4K RAM e˛le˛tirmesini pasifle˛tir"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_SUPERPET_IO,    N_("Enable SuperPET I/O")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SUPERPET_IO_DA, "AktivÈr SuperPET-I/O"},
	/* de */ {IDCLS_ENABLE_SUPERPET_IO_DE, "SuperPET I/O aktivieren"},
	/* es */ {IDCLS_ENABLE_SUPERPET_IO_ES, "Permitir entrada/salida SuperPET"},
	/* fr */ {IDCLS_ENABLE_SUPERPET_IO_FR, "Activer les E/S SuperPET"},
	/* hu */ {IDCLS_ENABLE_SUPERPET_IO_HU, "SuperPET I/O engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_SUPERPET_IO_IT, "Attiva l'I/O del SuperPET"},
	/* ko */ {IDCLS_ENABLE_SUPERPET_IO_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SUPERPET_IO_NL, "SuperPET I/O inschakelen"},
	/* pl */ {IDCLS_ENABLE_SUPERPET_IO_PL, "W≥±cz we/wy SuperPET"},
	/* ru */ {IDCLS_ENABLE_SUPERPET_IO_RU, "Enable SuperPET I/O"},
	/* sv */ {IDCLS_ENABLE_SUPERPET_IO_SV, "Aktivera SuperPET-I/O"},
	/* tr */ {IDCLS_ENABLE_SUPERPET_IO_TR, "SuperPET G/«'˝ aktif et"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_SUPERPET_IO,    N_("Disable SuperPET I/O")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SUPERPET_IO_DA, "DeaktivÈr SuperPET-I/O"},
	/* de */ {IDCLS_DISABLE_SUPERPET_IO_DE, "SuperPET I/O deaktivieren"},
	/* es */ {IDCLS_DISABLE_SUPERPET_IO_ES, "Deshabilitar entrada/salida SuperPET"},
	/* fr */ {IDCLS_DISABLE_SUPERPET_IO_FR, "DÈsactiver les E/S SuperPET"},
	/* hu */ {IDCLS_DISABLE_SUPERPET_IO_HU, "SuperPET I/O tilt·sa"},
	/* it */ {IDCLS_DISABLE_SUPERPET_IO_IT, "Disattiva l'I/O del SuperPET"},
	/* ko */ {IDCLS_DISABLE_SUPERPET_IO_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SUPERPET_IO_NL, "SuperPET I/O uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SUPERPET_IO_PL, "Wy≥±cz we/wy SuperPET"},
	/* ru */ {IDCLS_DISABLE_SUPERPET_IO_RU, "Disable SuperPET I/O"},
	/* sv */ {IDCLS_DISABLE_SUPERPET_IO_SV, "Inaktivera SuperPET-I/O"},
	/* tr */ {IDCLS_DISABLE_SUPERPET_IO_TR, "SuperPET G/«'˝ pasifle˛tir"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES,    N_("Enable ROM 1 Kernal patches")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_DA, "AktivÈr ROM 1 kerneÊndringer"},
	/* de */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_DE, "ROM 1 Kernal patches aktivieren"},
	/* es */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_ES, "Permitir parches ROM 1 Kernal"},
	/* fr */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_FR, "Activer les patches ROM 1 Kernal"},
	/* hu */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_HU, "ROM 1 Kernal foltoz·s engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_IT, "Attiva le patch del Kernal della ROM 1"},
	/* ko */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_NL, "ROM 1 Kernalverbeteringen inschakelen"},
	/* pl */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_PL, "W≥±cz ≥atki Kernala ROM 1"},
	/* ru */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_RU, "Enable ROM 1 Kernal patches"},
	/* sv */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_SV, "Aktivera ROM 1-Kernalpatchar"},
	/* tr */ {IDCLS_ENABLE_ROM_1_KERNAL_PATCHES_TR, "ROM 1 Kernal yamalar˝n˝ aktif et"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES,    N_("Disable ROM 1 Kernal patches")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_DA, "DeaktivÈr ROM 1 kerneÊndringer"},
	/* de */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_DE, "ROM 1 Kernal patches deaktivieren"},
	/* es */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_ES, "Deshabilitar parches ROM 1 Kernal"},
	/* fr */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_FR, "DÈsactiver les patches ROM 1 Kernal"},
	/* hu */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_HU, "ROM 1 Kernal foltoz·s tilt·sa"},
	/* it */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_IT, "Disattiva le patch del Kernal della ROM 1"},
	/* ko */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_NL, "ROM 1 Kernalverbeteringen uitschakelen"},
	/* pl */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_PL, "Wy≥±cz ≥atki Kernala ROM 1"},
	/* ru */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_RU, "Disable ROM 1 Kernal patches"},
	/* sv */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_SV, "Inaktivera ROM 1-Kernalpatchar"},
	/* tr */ {IDCLS_DISABLE_ROM_1_KERNAL_PATCHES_TR, "ROM 1 Kernal yamalar˝n˝ pasifle˛tir"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET,    N_("Switch upper/lower case charset")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_DA, "Skift mellem store og smÂ bogstaver"},
	/* de */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_DE, "Wechsel Groﬂ-/Kleinschreibung Zeichensatz"},
	/* es */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_ES, "Intercambiar may˙sculas/min˙scula"},
	/* fr */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_FR, "Inverser le charset majuscule et minuscule"},
	/* hu */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_HU, "V·lt·s nagy-/kisbet˚s karakterkÈszletre"},
	/* it */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_IT, "Scambia il set di caratteri maiuscolo/minuscolo"},
	/* ko */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_NL, "Schakel tussen grote/kleine letters"},
	/* pl */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_PL, "Prze≥±cz zestaw wielkich/ma≥ych znakÛw"},
	/* ru */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_RU, "≤⁄€ÓÁÿ‚Ï “’‡Â›ÿŸ/›ÿ÷›ÿŸ ‡’”ÿ·‚‡"},
	/* sv */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_SV, "V‰xla smÂ/stora bokst‰ver i teckengeneratorn"},
	/* tr */ {IDCLS_SWITCH_UPPER_LOWER_CHARSET_TR, "B¸y¸k/k¸Á¸k harf karakter seti dei˛tir"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET,    N_("Do not switch upper/lower case charset")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_DA, "Skift ikke mellem store og smÂ bogstaver"},
	/* de */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_DE, "Kein Wechsel Groﬂ-/Kleinschreibung Zeichensatz"},
	/* es */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_ES, "No intercambiar may˙sculas/min˙sculas"},
	/* fr */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_FR, "Ne pas inverser le charset majuscule et minuscule"},
	/* hu */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_HU, "Ne v·ltson a nagy-/kisbet˚s karakterkÈszletre"},
	/* it */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_IT, "Non scambiare il set di caratteri maiuscolo/minuscolo"},
	/* ko */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_NL, "Schakel niet tussen grote/kleine letters"},
	/* pl */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_PL, "Nie prze≥±czaj zestawu wielkich/ma≥ych znakÛw"},
	/* ru */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_RU, "Ω’ “⁄€ÓÁ–‚Ï “’‡Â›ÿŸ/›ÿ÷›ÿŸ ‡’”ÿ·‚‡"},
	/* sv */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_SV, "V‰xla inte smÂ/stora bokst‰ver i teckengeneratorn"},
	/* tr */ {IDCLS_NO_SWITCH_UPPER_LOWER_CHARSET_TR, "B¸y¸k/k¸Á¸k harf karakter seti dei˛tirme"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_EOI_BLANKS_SCREEN,    N_("EOI blanks screen")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_EOI_BLANKS_SCREEN_DA, "EOI sletter skÊrmen"},
	/* de */ {IDCLS_EOI_BLANKS_SCREEN_DE, "EOI lˆscht Bildschirm"},
	/* es */ {IDCLS_EOI_BLANKS_SCREEN_ES, "EOI pantalla en blanco"},
	/* fr */ {IDCLS_EOI_BLANKS_SCREEN_FR, "EOI vide l'Ècran"},
	/* hu */ {IDCLS_EOI_BLANKS_SCREEN_HU, "EOI tˆrli a kÈpernyıt"},
	/* it */ {IDCLS_EOI_BLANKS_SCREEN_IT, "EOI oscura lo schermo"},
	/* ko */ {IDCLS_EOI_BLANKS_SCREEN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_EOI_BLANKS_SCREEN_NL, "EOI maakt het scherm leeg"},
	/* pl */ {IDCLS_EOI_BLANKS_SCREEN_PL, "Koniec przerwania czy∂ci ekran"},
	/* ru */ {IDCLS_EOI_BLANKS_SCREEN_RU, "EOI blanks screen"},
	/* sv */ {IDCLS_EOI_BLANKS_SCREEN_SV, "EOI tˆmmer sk‰rmen"},
	/* tr */ {IDCLS_EOI_BLANKS_SCREEN_TR, "EOI ekran˝ bo˛alt˝r"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN,    N_("EOI does not blank screen")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_DA, "EOI sletter ikke skÊrmen"},
	/* de */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_DE, "EIO lˆscht Bildschirm nicht"},
	/* es */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_ES, "EOI pantalla no en blanco"},
	/* fr */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_FR, "EOI ne vide pas l'Ècran"},
	/* hu */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_HU, "EOI nem tˆrli a kÈpernyıt"},
	/* it */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_IT, "EOI non oscura lo schermo"},
	/* ko */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_NL, "EOI maakt het scherm niet leeg"},
	/* pl */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_PL, "Koniec przerwania nie czy∂ci ekranu"},
	/* ru */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_RU, "EOI does not blank screen"},
	/* sv */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_SV, "EOI tˆmmer inte sk‰rmen"},
	/* tr */ {IDCLS_EOI_DOES_NOT_BLANK_SCREEN_TR, "EOI ekran˝ bo˛altmaz"},
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SET_CPU_SWITCH_TO_6502,    N_("Set SuperPET CPU switch to '6502'")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_CPU_SWITCH_TO_6502_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_CPU_SWITCH_TO_6502_DE, "Setze SuperPET CPU Schalter auf '6502'"},
	/* es */ {IDCLS_SET_CPU_SWITCH_TO_6502_ES, "Ajustar interruptor SuperPET CPU"},
	/* fr */ {IDCLS_SET_CPU_SWITCH_TO_6502_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_CPU_SWITCH_TO_6502_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_CPU_SWITCH_TO_6502_IT, "Imposta pulsante CPU SuperPET a '6502'"},
	/* ko */ {IDCLS_SET_CPU_SWITCH_TO_6502_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_CPU_SWITCH_TO_6502_NL, "Zet SuperPET CPU schakelaar naar '6502'"},
	/* pl */ {IDCLS_SET_CPU_SWITCH_TO_6502_PL, "Ustaw prze≥±cznik CPU SuperPET na '6502\""},
	/* ru */ {IDCLS_SET_CPU_SWITCH_TO_6502_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_CPU_SWITCH_TO_6502_SV, "S‰tt SuperPET-processoromkopplaren till \"6502\""},
	/* tr */ {IDCLS_SET_CPU_SWITCH_TO_6502_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SET_CPU_SWITCH_TO_6809,    N_("Set SuperPET CPU switch to '6809'")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_CPU_SWITCH_TO_6809_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_CPU_SWITCH_TO_6809_DE, "Setze SuperPET CPU Schalter auf '6809'"},
	/* es */ {IDCLS_SET_CPU_SWITCH_TO_6809_ES, "Ajustar interruptor SuperPET CPU para '6809'"},
	/* fr */ {IDCLS_SET_CPU_SWITCH_TO_6809_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_CPU_SWITCH_TO_6809_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_CPU_SWITCH_TO_6809_IT, "Imposta pulsante CPU SuperPET a '6809'"},
	/* ko */ {IDCLS_SET_CPU_SWITCH_TO_6809_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_CPU_SWITCH_TO_6809_NL, "Zet SuperPET CPU schakelaar naar '6809'"},
	/* pl */ {IDCLS_SET_CPU_SWITCH_TO_6809_PL, "Ustaw prze≥±cznik CPU SuperPET na '6809\""},
	/* ru */ {IDCLS_SET_CPU_SWITCH_TO_6809_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_CPU_SWITCH_TO_6809_SV, "S‰tt SuperPET-processoromkopplaren till \"6809\""},
	/* tr */ {IDCLS_SET_CPU_SWITCH_TO_6809_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SET_CPU_SWITCH_TO_PROG,    N_("Set SuperPET CPU switch to 'Prog'")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_CPU_SWITCH_TO_PROG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_CPU_SWITCH_TO_PROG_DE, "Setze SuperPET CPU Schalter auf 'Prog'"},
	/* es */ {IDCLS_SET_CPU_SWITCH_TO_PROG_ES, "Ajustar interruptor SuperPET CPU para 'Prog'"},
	/* fr */ {IDCLS_SET_CPU_SWITCH_TO_PROG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_CPU_SWITCH_TO_PROG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_CPU_SWITCH_TO_PROG_IT, "Imposta pulsante CPU SuperPET a 'Prog'"},
	/* ko */ {IDCLS_SET_CPU_SWITCH_TO_PROG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_CPU_SWITCH_TO_PROG_NL, "Zet SuperPET CPU schakelaar naar 'Prog'"},
	/* pl */ {IDCLS_SET_CPU_SWITCH_TO_PROG_PL, "Ustaw prze≥±cznik CPU SuperPET na 'Prog\""},
	/* ru */ {IDCLS_SET_CPU_SWITCH_TO_PROG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_CPU_SWITCH_TO_PROG_SV, "S‰tt SuperPET-processoromkopplaren till \"Prog\""},
	/* tr */ {IDCLS_SET_CPU_SWITCH_TO_PROG_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_6809_ROM_AT_A000,    N_("Specify 4K to 24K ROM file name at $A000 for 6809")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_6809_ROM_AT_A000_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_6809_ROM_AT_A000_DE, "4K-24K Erweiterungs ROM Dateiname f¸r $A000 f¸r 6809 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_6809_ROM_AT_A000_ES, "Especificar nombre ROM 4K a 24K  en $A000 para 6809"},
	/* fr */ {IDCLS_SPECIFY_6809_ROM_AT_A000_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_6809_ROM_AT_A000_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_6809_ROM_AT_A000_IT, "Specifica il nome della ROM da 4K a 24K a $A000 per il 6809"},
	/* ko */ {IDCLS_SPECIFY_6809_ROM_AT_A000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_6809_ROM_AT_A000_NL, "Geef de naam van het bestand voor de 4K tot 24K ROM op $A000 voor de 6809"},
	/* pl */ {IDCLS_SPECIFY_6809_ROM_AT_A000_PL, "Okre∂l nazwÍ pliku ROM-u 4K do 24K w $A000 dla 6809"},
	/* ru */ {IDCLS_SPECIFY_6809_ROM_AT_A000_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_6809_ROM_AT_A000_SV, "Ange namn fˆr 4-24K-utˆknings-ROM pÂ $A000 fˆr 6809"},
	/* tr */ {IDCLS_SPECIFY_6809_ROM_AT_A000_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_6809_ROM_AT_B000,    N_("Specify 4K to 20K ROM file name at $B000 for 6809")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_6809_ROM_AT_B000_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_6809_ROM_AT_B000_DE, "4K-20K Erweiterungs ROM Dateiname f¸r $B000 f¸r 6809 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_6809_ROM_AT_B000_ES, "Especificar nombre ROM 4K a 20K  en $B000 para 6809"},
	/* fr */ {IDCLS_SPECIFY_6809_ROM_AT_B000_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_6809_ROM_AT_B000_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_6809_ROM_AT_B000_IT, "Specifica il nome della ROM da 4K a 20K a $B000 per il 6809"},
	/* ko */ {IDCLS_SPECIFY_6809_ROM_AT_B000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_6809_ROM_AT_B000_NL, "Geef de naam van het bestand voor de 4K tot 20K ROM op $B000 voor de 6809"},
	/* pl */ {IDCLS_SPECIFY_6809_ROM_AT_B000_PL, "Okre∂l nazwÍ pliku ROM-u 4K do 20K w $B000 dla 6809"},
	/* ru */ {IDCLS_SPECIFY_6809_ROM_AT_B000_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_6809_ROM_AT_B000_SV, "Ange namn fˆr 4-20K-utˆknings-ROM pÂ $B000 fˆr 6809"},
	/* tr */ {IDCLS_SPECIFY_6809_ROM_AT_B000_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_6809_ROM_AT_C000,    N_("Specify 4K to 16K ROM file name at $C000 for 6809")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_6809_ROM_AT_C000_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_6809_ROM_AT_C000_DE, "4K-16K Erweiterungs ROM Dateiname f¸r $C000 f¸r 6809 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_6809_ROM_AT_C000_ES, "Especificar nombre ROM 4K a 16K  en $C000 para 6809"},
	/* fr */ {IDCLS_SPECIFY_6809_ROM_AT_C000_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_6809_ROM_AT_C000_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_6809_ROM_AT_C000_IT, "Specifica il nome della ROM da 4K a 16K a $C000 per il 6809"},
	/* ko */ {IDCLS_SPECIFY_6809_ROM_AT_C000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_6809_ROM_AT_C000_NL, "Geef de naam van het bestand voor de 4K tot 16K ROM op $C000 voor de 6809"},
	/* pl */ {IDCLS_SPECIFY_6809_ROM_AT_C000_PL, "Okre∂l nazwÍ pliku ROM-u 4K do 16K w $C000 dla 6809"},
	/* ru */ {IDCLS_SPECIFY_6809_ROM_AT_C000_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_6809_ROM_AT_C000_SV, "Ange namn fˆr 4-16K-utˆknings-ROM pÂ $C000 fˆr 6809"},
	/* tr */ {IDCLS_SPECIFY_6809_ROM_AT_C000_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_6809_ROM_AT_D000,    N_("Specify 4K to 12K ROM file name at $D000 for 6809")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_6809_ROM_AT_D000_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_6809_ROM_AT_D000_DE, "4K-12K Erweiterungs ROM Dateiname f¸r $D000 f¸r 6809 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_6809_ROM_AT_D000_ES, "Especificar nombre ROM 4K a 12K  en $D000 para 6809"},
	/* fr */ {IDCLS_SPECIFY_6809_ROM_AT_D000_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_6809_ROM_AT_D000_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_6809_ROM_AT_D000_IT, "Specifica il nome della ROM da 4K a 12K a $D000 per il 6809"},
	/* ko */ {IDCLS_SPECIFY_6809_ROM_AT_D000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_6809_ROM_AT_D000_NL, "Geef de naam van het bestand voor de 4K tot 12K ROM op $D000 voor de 6809"},
	/* pl */ {IDCLS_SPECIFY_6809_ROM_AT_D000_PL, "Okre∂l nazwÍ pliku ROM-u 4K do 12K w $D000 dla 6809"},
	/* ru */ {IDCLS_SPECIFY_6809_ROM_AT_D000_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_6809_ROM_AT_D000_SV, "Ange namn fˆr 4-12K-utˆknings-ROM pÂ $D000 fˆr 6809"},
	/* tr */ {IDCLS_SPECIFY_6809_ROM_AT_D000_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_6809_ROM_AT_E000,    N_("Specify 2K or 8K ROM file name at $E000 for 6809")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_6809_ROM_AT_E000_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_6809_ROM_AT_E000_DE, "4K-8K Erweiterungs ROM Dateiname f¸r $E000 f¸r 6809 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_6809_ROM_AT_E000_ES, "Especificar nombre ROM 4K a 8K  en $E000 para 6809"},
	/* fr */ {IDCLS_SPECIFY_6809_ROM_AT_E000_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_6809_ROM_AT_E000_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_6809_ROM_AT_E000_IT, "Specifica il nome della ROM da 2K a 8K a $E000 per il 6809"},
	/* ko */ {IDCLS_SPECIFY_6809_ROM_AT_E000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_6809_ROM_AT_E000_NL, "Geef de naam van het bestand voor de 2K of 8K ROM op $E000 voor de 6809"},
	/* pl */ {IDCLS_SPECIFY_6809_ROM_AT_E000_PL, "Okre∂l nazwÍ pliku ROM-u 2K lub 8K w $E000 dla 6809"},
	/* ru */ {IDCLS_SPECIFY_6809_ROM_AT_E000_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_6809_ROM_AT_E000_SV, "Ange namn fˆr 2/8K-utˆknings-ROM pÂ $E000 fˆr 6809"},
	/* tr */ {IDCLS_SPECIFY_6809_ROM_AT_E000_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_6809_ROM_AT_F000,    N_("Specify 4K ROM file name at $F000 for 6809")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_6809_ROM_AT_F000_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_6809_ROM_AT_F000_DE, "4K Erweiterungs ROM Dateiname f¸r $F000 f¸r 6809 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_6809_ROM_AT_F000_ES, "Especificar nombre ROM 4K en $F000 para 6809"},
	/* fr */ {IDCLS_SPECIFY_6809_ROM_AT_F000_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_6809_ROM_AT_F000_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_6809_ROM_AT_F000_IT, "Specifica il nome della ROM da 4K a $F000 per il 6809"},
	/* ko */ {IDCLS_SPECIFY_6809_ROM_AT_F000_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_6809_ROM_AT_F000_NL, "Geef de naam van het bestand voor de 4K ROM op $F000 voor de 6809"},
	/* pl */ {IDCLS_SPECIFY_6809_ROM_AT_F000_PL, "Okre∂l nazwÍ pliku ROM-u 4K w $F000 dla 6809"},
	/* ru */ {IDCLS_SPECIFY_6809_ROM_AT_F000_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_6809_ROM_AT_F000_SV, "Ange namn fˆr 4K-utˆknings-ROM pÂ $F000 fˆr 6809"},
	/* tr */ {IDCLS_SPECIFY_6809_ROM_AT_F000_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET4032_COLOUR_RGBI,    N_("RGBI colour extension to PET 4032")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET4032_COLOUR_RGBI_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PET4032_COLOUR_RGBI_DE, "RGBI Farberweiterung f¸r PET 4032"},
	/* es */ {IDCLS_PET4032_COLOUR_RGBI_ES, "Extension color RGBI para PET 4032"},
	/* fr */ {IDCLS_PET4032_COLOUR_RGBI_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PET4032_COLOUR_RGBI_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PET4032_COLOUR_RGBI_IT, "Estensione colore RGBI per PET 4032"},
	/* ko */ {IDCLS_PET4032_COLOUR_RGBI_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET4032_COLOUR_RGBI_NL, "RGBI kleur uitbreiding voor de PET 4032"},
	/* pl */ {IDCLS_PET4032_COLOUR_RGBI_PL, "Rozszerzenie barw RGBI do PET 4032"},
	/* ru */ {IDCLS_PET4032_COLOUR_RGBI_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PET4032_COLOUR_RGBI_SV, "RGBI-f‰rgutˆkning fˆr PET 4032"},
	/* tr */ {IDCLS_PET4032_COLOUR_RGBI_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET4032_COLOUR_ANALOG,    N_("Analog colour extension to PET 4032")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET4032_COLOUR_ANALOG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PET4032_COLOUR_ANALOG_DE, "Analog Farberweiterung f¸r PET 4032"},
	/* es */ {IDCLS_PET4032_COLOUR_ANALOG_ES, "Extension analÛgica de color para PET 4032"},
	/* fr */ {IDCLS_PET4032_COLOUR_ANALOG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PET4032_COLOUR_ANALOG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PET4032_COLOUR_ANALOG_IT, "Estensione colore analogica per PET 4032"},
	/* ko */ {IDCLS_PET4032_COLOUR_ANALOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET4032_COLOUR_ANALOG_NL, "Analoge kleur uitbreiding voor de PET 4032"},
	/* pl */ {IDCLS_PET4032_COLOUR_ANALOG_PL, "Rozszerzenie analogowych barw do PET 4032"},
	/* ru */ {IDCLS_PET4032_COLOUR_ANALOG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PET4032_COLOUR_ANALOG_SV, "Analog f‰rgutˆkning fˆr PET 4032"},
	/* tr */ {IDCLS_PET4032_COLOUR_ANALOG_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_P_COLOUR_0_255,    N_("<Colour 0-255>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_COLOUR_0_255_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_COLOUR_0_255_DE, "<Farbe 0-255>"},
	/* es */ {IDCLS_P_COLOUR_0_255_ES, "<Color 0-255>"},
	/* fr */ {IDCLS_P_COLOUR_0_255_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_COLOUR_0_255_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_COLOUR_0_255_IT, "<Colore 0-255>"},
	/* ko */ {IDCLS_P_COLOUR_0_255_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_COLOUR_0_255_NL, "<Kleur 0-255>"},
	/* pl */ {IDCLS_P_COLOUR_0_255_PL, "<Kolor 0-255>"},
	/* ru */ {IDCLS_P_COLOUR_0_255_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_COLOUR_0_255_SV, "<F‰rg 0-255>"},
	/* tr */ {IDCLS_P_COLOUR_0_255_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET4032_COLOUR_BACKGROUND,    N_("Analog colour background on PET 4032")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET4032_COLOUR_BACKGROUND_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PET4032_COLOUR_BACKGROUND_DE, "Analog Hintergrundfarbe f¸r PET 4032"},
	/* es */ {IDCLS_PET4032_COLOUR_BACKGROUND_ES, "Color del fondo analÛgico para PET 4032"},
	/* fr */ {IDCLS_PET4032_COLOUR_BACKGROUND_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PET4032_COLOUR_BACKGROUND_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PET4032_COLOUR_BACKGROUND_IT, "Coloro sfondo analogico su PET 4032"},
	/* ko */ {IDCLS_PET4032_COLOUR_BACKGROUND_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET4032_COLOUR_BACKGROUND_NL, "Analoge kleuren achtergrond voor de PET 4032"},
	/* pl */ {IDCLS_PET4032_COLOUR_BACKGROUND_PL, "Analogowy kolor t≥a w PET 4032"},
	/* ru */ {IDCLS_PET4032_COLOUR_BACKGROUND_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PET4032_COLOUR_BACKGROUND_SV, "Analog f‰rgbakgrund pÂ PET 4032"},
	/* tr */ {IDCLS_PET4032_COLOUR_BACKGROUND_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET_RAM_SIZE,    N_("PET RAM size (4/8/16/32/96/128)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET_RAM_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PET_RAM_SIZE_DE, "PET RAM Grˆﬂe (4/8/16/32/96/128)"},
	/* es */ {IDCLS_PET_RAM_SIZE_ES, "TamaÒo de la RAM en PET  (4/8/16/32/96/128)"},
	/* fr */ {IDCLS_PET_RAM_SIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PET_RAM_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PET_RAM_SIZE_IT, "Dimensione RAM PET (4/8/16/32/96/128)"},
	/* ko */ {IDCLS_PET_RAM_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET_RAM_SIZE_NL, "PET RAM grootte (4/8/16/32/96/128)"},
	/* pl */ {IDCLS_PET_RAM_SIZE_PL, "Wielko∂Ê RAM-u PET (4/8/16/32/96/128)"},
	/* ru */ {IDCLS_PET_RAM_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PET_RAM_SIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PET_RAM_SIZE_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_PET_IO_SIZE,    N_("PET I/O size (256/2048)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET_IO_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PET_IO_SIZE_DE, "PET I/O Grˆﬂe (256/2048)"},
	/* es */ {IDCLS_PET_IO_SIZE_ES, "TamaÒo PET I/O (256/2048)"},
	/* fr */ {IDCLS_PET_IO_SIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PET_IO_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PET_IO_SIZE_IT, "Dimensione I/O PET (256/2048)"},
	/* ko */ {IDCLS_PET_IO_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET_IO_SIZE_NL, "PET I/O grootte (256/2048)"},
	/* pl */ {IDCLS_PET_IO_SIZE_PL, "Rozmiar we/wy PET (256/2048)"},
	/* ru */ {IDCLS_PET_IO_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PET_IO_SIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PET_IO_SIZE_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_CRTC,    N_("Enable CRTC")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_CRTC_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_CRTC_DE, "CRTC aktivieren"},
	/* es */ {IDCLS_ENABLE_CRTC_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_CRTC_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_CRTC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_CRTC_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_CRTC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_CRTC_NL, "CRTC inschakelen"},
	/* pl */ {IDCLS_ENABLE_CRTC_PL, "W≥±cz CRTC"},
	/* ru */ {IDCLS_ENABLE_CRTC_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_CRTC_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_CRTC_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_CRTC,    N_("Disable CRTC")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_CRTC_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_CRTC_DE, "CRTC deaktivieren"},
	/* es */ {IDCLS_DISABLE_CRTC_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_CRTC_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_CRTC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_CRTC_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_CRTC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_CRTC_NL, "CRTC uitschakelen"},
	/* pl */ {IDCLS_DISABLE_CRTC_PL, "Wy≥±cz CRTC"},
	/* ru */ {IDCLS_DISABLE_CRTC_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_CRTC_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_CRTC_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pet-cmdline-options.c */
	/* en */ {IDCLS_SET_VIDEO_SIZE,    N_("Set video size (0: Automatic, 40: 40 Columns, 80: 80 Columns)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_VIDEO_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_VIDEO_SIZE_DE, "Video Grˆﬂe (0: Automatisch, 40: 40 Spalten, 80: 80 Spalten)"},
	/* es */ {IDCLS_SET_VIDEO_SIZE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_VIDEO_SIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_VIDEO_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_VIDEO_SIZE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_VIDEO_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_VIDEO_SIZE_NL, "Stel video grootte in (0: Automatisch, 40: 40 Kolommen, 80: 80 Kolommen)"},
	/* pl */ {IDCLS_SET_VIDEO_SIZE_PL, "Okre∂l wielko∂Ê obrazu (0: Automatyczna, 40: 40 kolumn, 80: 80 kolumn)"},
	/* ru */ {IDCLS_SET_VIDEO_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_VIDEO_SIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_VIDEO_SIZE_TR, ""},  /* fuzzy */
#endif
	
	/* pet/petpia1.c */
	/* en */ {IDCLS_ENABLE_USERPORT_DIAG_PIN,    N_("Enable userport diagnostic pin")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_DA, "AktivÈr diagnostik pÂ brugerporten"},
	/* de */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_DE, "Userport Diagnose Pin aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_ES, "Permitir pin puerto diagnostico"},
	/* fr */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_FR, "Activer la pin de diagnostique userport"},
	/* hu */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_HU, "Userport diagnosztikai l·b engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_IT, "Attiva pin diagnostico su userport"},
	/* ko */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_NL, "Userport diagnostic pin inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_PL, "W≥±cz pin diagnostyczny userportu"},
	/* ru */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_RU, "Enable userport diagnostic pin"},
	/* sv */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_SV, "Aktivera diagnostik pÂ anv‰ndarporten"},
	/* tr */ {IDCLS_ENABLE_USERPORT_DIAG_PIN_TR, "Userport diyagnostik pinini aktif et"},
#endif
	
	/* pet/petpia1.c */
	/* en */ {IDCLS_DISABLE_USERPORT_DIAG_PIN,    N_("Disable userport diagnostic pin")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_DA, "DeaktivÈr diagnostik pÂ brugerporten"},
	/* de */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_DE, "Userport Diagnose Pin deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_ES, "Deshabilitar pin puerto diagnostico"},
	/* fr */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_FR, "DÈsactiver la pin de diagnostique userport"},
	/* hu */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_HU, "Userport diagnosztikai l·b tilt·sa"},
	/* it */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_IT, "Disattiva pin diagnostico su userport"},
	/* ko */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_NL, "Userport diagnostic pin uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_PL, "Wy≥±cz pin diagnostyczny userportu"},
	/* ru */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_RU, "Disable userport diagnostic pin"},
	/* sv */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_SV, "Inaktivera diagnostik pÂ anv‰ndarporten"},
	/* tr */ {IDCLS_DISABLE_USERPORT_DIAG_PIN_TR, "Userport diyagnostik pinini pasifle˛tir"},
#endif
	
	/* plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME,    N_("Specify name of Function low ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_DA, "Angiv navn pÂ image for lav funktions-ROM"},
	/* de */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_DE, "Namen f¸r Funktions ROM Datei (unterer Bereich) w‰hlen"},
	/* es */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_ES, "Especificar nombre imagen ROM funciÛn baja"},
	/* fr */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_FR, "SpÈcifier le nom de l'image basse ROM Fonction"},
	/* hu */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_HU, "Adja meg az alsÛ Function ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_IT, "Specifica il nome immagine della Function ROM bassa"},
	/* ko */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_NL, "Geef de naam van het Function low ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Function low"},
	/* ru */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_RU, "Specify name of Function low ROM image"},
	/* sv */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_SV, "Ange namn pÂ ROM-avbildning fˆr lÂgt funktions-ROM"},
	/* tr */ {IDCLS_SPECIFY_FUNCTION_LOW_ROM_NAME_TR, "Function low ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME,    N_("Specify name of Function high ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_DA, "Angiv navn pÂ image for h¯j funktions-ROM"},
	/* de */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_DE, "Namen f¸r Funktions ROM Datei (oberer Bereich) w‰hlen"},
	/* es */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_ES, "Especificar nombre imagen ROM funciÛn alta"},
	/* fr */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_FR, "SpÈcifier le nom de l'image haute ROM Fonction"},
	/* hu */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_HU, "Adja meg a felsı Function ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_IT, "Specifica il nome immagine della Function ROM alta"},
	/* ko */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_NL, "Geef de naam van het Function high ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Function high"},
	/* ru */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_RU, "Specify name of Function high ROM image"},
	/* sv */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_SV, "Ange namn pÂ ROM-avbildning fˆr hˆgt funktions-ROM"},
	/* tr */ {IDCLS_SPECIFY_FUNCTION_HIGH_ROM_NAME_TR, "Function high ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME,    N_("Specify name of Cartridge 1 low ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_DA, "Angiv navn pÂ image for lav cartridge 1 ROM"},
	/* de */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_DE, "Name f¸r Erweiterungsmodul 1 ROM Datei (unterer Bereich) w‰hlen"},
	/* es */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_ES, "Especificar nombre imagen ROM cartucho 1 bajo"},
	/* fr */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_FR, "SpÈcifier le nom de l'image basse ROM Cartouche 1"},
	/* hu */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_HU, "Adja meg az alsÛ Cartridge 1 ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_IT, "Specifica il nome immagine della ROM Cartridge 1 bassa"},
	/* ko */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_NL, "Geef de naam van het Cartridge 1 low ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Cartridge 1 low"},
	/* ru */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_RU, "Specify name of Cartridge 1 low ROM image"},
	/* sv */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_SV, "Ange namn pÂ ROM-avbildning fˆr lÂgt insticksmodul 1-ROM"},
	/* tr */ {IDCLS_SPECIFY_CART_1_LOW_ROM_NAME_TR, "Cartridge 1 low ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME,    N_("Specify name of Cartridge 1 high ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_DA, "Angiv navn pÂ image for h¯j cartridge 1 ROM"},
	/* de */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_DE, "Name f¸r Erweiterungsmodul 1 ROM Datei (oberer Bereich) w‰hlen"},
	/* es */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_ES, "Especificar nombre imagen ROM cartucho 1 alto"},
	/* fr */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_FR, "SpÈcifier le nom de l'image haute ROM Cartouche 1"},
	/* hu */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_HU, "Adja meg a felsı Cartridge 1 ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_IT, "Specifica il nome immagine della ROM Cartridge 1 alta"},
	/* ko */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_NL, "Geef de naam van het Cartridge 1 high ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Cartridge 1 high"},
	/* ru */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_RU, "Specify name of Cartridge 1 high ROM image"},
	/* sv */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_SV, "Ange namn pÂ ROM-avbildning fˆr hˆgt insticksmodul 1-ROM"},
	/* tr */ {IDCLS_SPECIFY_CART_1_HIGH_ROM_NAME_TR, "Cartridge 1 high ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME,    N_("Specify name of Cartridge 2 low ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_DA, "Angiv navn pÂ image for lav cartridge 2 ROM"},
	/* de */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_DE, "Name f¸r Erweiterungsmodul 2 ROM Datei (unterer Bereich) w‰hlen"},
	/* es */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_ES, "Especificar nombre imagen ROM cartucho 2 bajo"},
	/* fr */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_FR, "SpÈcifier le nom de l'image basse ROM Cartouche 2"},
	/* hu */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_HU, "Adja meg az alsÛ Cartridge 2 ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_IT, "Specifica il nome immagine della ROM Cartridge 2 bassa"},
	/* ko */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_NL, "Geef de naam van het Cartridge 2 low ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Cartridge 2 low"},
	/* ru */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_RU, "Specify name of Cartridge 2 low ROM image"},
	/* sv */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_SV, "Ange namn pÂ ROM-avbildning fˆr lÂgt insticksmodul 2-ROM"},
	/* tr */ {IDCLS_SPECIFY_CART_2_LOW_ROM_NAME_TR, "Cartridge 2 low ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME,    N_("Specify name of Cartridge 2 high ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_DA, "Angiv navn pÂ image for h¯j cartridge 1 ROM"},
	/* de */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_DE, "Name f¸r Erweiterungsmodul 2 ROM Datei (oberer Bereich) w‰hlen"},
	/* es */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_ES, "Especificar nombre imagen ROM cartucho 2 alto"},
	/* fr */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_FR, "SpÈcifier le nom de l'image haute ROM Cartouche 2"},
	/* hu */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_HU, "Adja meg a felsı Cartridge 2 ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_IT, "Specifica il nome immagine della ROM Cartridge 2 alta"},
	/* ko */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_NL, "Geef de naam van het Cartridge 2 high ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM Cartridge 2 high"},
	/* ru */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_RU, "Specify name of Cartridge 2 high ROM image"},
	/* sv */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_SV, "Ange namn pÂ ROM-avbildning fˆr hˆgt insticksmodul 2-ROM"},
	/* tr */ {IDCLS_SPECIFY_CART_2_HIGH_ROM_NAME_TR, "Cartridge 2 high ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* plus4/plus4-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_RAM_INSTALLED,    N_("Specify size of RAM installed in kb (16/32/64)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_RAM_INSTALLED_DA, "Angiv st¯rrelse pÂ installeret RAM i kB (16/32/64)"},
	/* de */ {IDCLS_SPECIFY_RAM_INSTALLED_DE, "Grˆﬂe des emulierten RAM Speichers in kb (16/32/64) spezifizieren"},
	/* es */ {IDCLS_SPECIFY_RAM_INSTALLED_ES, "Especificar tamaÒo RAM instalada en KB (16/32/64)"},
	/* fr */ {IDCLS_SPECIFY_RAM_INSTALLED_FR, "SpÈcifier la taille du RAM installÈ (16/32/64)"},
	/* hu */ {IDCLS_SPECIFY_RAM_INSTALLED_HU, "Adja meg a telepÌtett RAM mÈretÈt kb-ban (16/32/64)"},
	/* it */ {IDCLS_SPECIFY_RAM_INSTALLED_IT, "Specifica la dimensione della RAM installata in kb (16/32/64)"},
	/* ko */ {IDCLS_SPECIFY_RAM_INSTALLED_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_RAM_INSTALLED_NL, "Geef de hoeveelheid RAM in kb (16/32/64)"},
	/* pl */ {IDCLS_SPECIFY_RAM_INSTALLED_PL, "Okre∂l ilo∂Ê zainstalowanego RAM-u w kb (16/32/64)"},
	/* ru */ {IDCLS_SPECIFY_RAM_INSTALLED_RU, "Specify size of RAM installed in kb (16/32/64)"},
	/* sv */ {IDCLS_SPECIFY_RAM_INSTALLED_SV, "Ange storlek pÂ installerat RAM i kb (16/32/64)"},
	/* tr */ {IDCLS_SPECIFY_RAM_INSTALLED_TR, "Kurulmu˛ olan RAM boyutunu kb cinsinden belirt (16/32/64)"},
#endif
	
	/* plus4/ted-cmdline-options.c, vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_SATURATION,    N_("Set saturation of internal calculated palette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_SATURATION_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_SATURATION_DE, "S‰ttigung f¸r intern berechneten Palette setzen"},
	/* es */ {IDCLS_SET_SATURATION_ES, "Seleccionar saturaciÛn calculada en paleta interna"},
	/* fr */ {IDCLS_SET_SATURATION_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_SATURATION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_SATURATION_IT, "Imposta la saturazione della palette calcolata internamente"},
	/* ko */ {IDCLS_SET_SATURATION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_SATURATION_NL, "Stel de verzadiging in van het intern berekend kleurenpalet"},
	/* pl */ {IDCLS_SET_SATURATION_PL, "Okre∂l nasycenie wewnÍtrznie obliczonej palety kolorystycznej"},
	/* ru */ {IDCLS_SET_SATURATION_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_SATURATION_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_SATURATION_TR, ""},  /* fuzzy */
#endif
	
	/* plus4/ted-cmdline-options.c, vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_CONTRAST,    N_("Set contrast of internal calculated palette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_CONTRAST_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_CONTRAST_DE, "Kontrast f¸r intern berechnete Palette setzen"},
	/* es */ {IDCLS_SET_CONTRAST_ES, "Seleccionar contraste calculado en paleta interna [1000]"},
	/* fr */ {IDCLS_SET_CONTRAST_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_CONTRAST_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_CONTRAST_IT, "Imposta il constrasto della palette calcolata internamente"},
	/* ko */ {IDCLS_SET_CONTRAST_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_CONTRAST_NL, "Stel het contrast in van het intern berekend kleurenpalet"},
	/* pl */ {IDCLS_SET_CONTRAST_PL, "Okre∂l kontrast wewnÍtrznie obliczonej palety kolorystycznej"},
	/* ru */ {IDCLS_SET_CONTRAST_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_CONTRAST_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_CONTRAST_TR, ""},  /* fuzzy */
#endif
	
	/* plus4/ted-cmdline-options.c, vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_BRIGHTNESS,    N_("Set brightness of internal calculated palette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_BRIGHTNESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_BRIGHTNESS_DE, "Helligkeit f¸r intern berechnete Palette setzen"},
	/* es */ {IDCLS_SET_BRIGHTNESS_ES, "Seleccionar brillo calculado en paleta interna"},
	/* fr */ {IDCLS_SET_BRIGHTNESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_BRIGHTNESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_BRIGHTNESS_IT, "Imposta la luminanza della palette calcolata internamente"},
	/* ko */ {IDCLS_SET_BRIGHTNESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_BRIGHTNESS_NL, "Stel de helderheid in van het intern berekend kleurenpalet"},
	/* pl */ {IDCLS_SET_BRIGHTNESS_PL, "Okre∂l jasno∂Ê wewnÍtrznie obliczonej palety kolorystycznej"},
	/* ru */ {IDCLS_SET_BRIGHTNESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_BRIGHTNESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_BRIGHTNESS_TR, ""},  /* fuzzy */
#endif
	
	/* plus4/ted-cmdline-options.c`, vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_GAMMA,    N_("Set gamma of internal calculated palette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_GAMMA_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_GAMMA_DE, "Gamma f¸r intern berechnete Palette setzen"},
	/* es */ {IDCLS_SET_GAMMA_ES, "Seleccionar gamma calculada en paleta interna"},
	/* fr */ {IDCLS_SET_GAMMA_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_GAMMA_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_GAMMA_IT, "Imposta la gamma della palette calcolata internamente"},
	/* ko */ {IDCLS_SET_GAMMA_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_GAMMA_NL, "Stel de gamma in van het intern berekend kleurenpalet"},
	/* pl */ {IDCLS_SET_GAMMA_PL, "Okre∂l jaskrawo∂Ê wewnÍtrznie obliczonej palety kolorystycznej"},
	/* ru */ {IDCLS_SET_GAMMA_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_GAMMA_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_GAMMA_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_TINT,    N_("Set tint of internal calculated palette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_TINT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_TINT_DE, "Farbton f¸r intern berechneten Palette setzen"},
	/* es */ {IDCLS_SET_TINT_ES, "Seleccionar tinta calculada en paleta interna"},
	/* fr */ {IDCLS_SET_TINT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_TINT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_TINT_IT, "Imposta la sfumatura della palette calcolata internamente"},
	/* ko */ {IDCLS_SET_TINT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_TINT_NL, "Stel de tint in van het intern berekend kleuren palette"},
	/* pl */ {IDCLS_SET_TINT_PL, "Okre∂l odcieÒ wewnÍtrznie obliczonej palety kolorystycznej"},
	/* ru */ {IDCLS_SET_TINT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_TINT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_TINT_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_ODDLINES_PHASE,    N_("Set phase for color carrier in odd lines")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_ODDLINES_PHASE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_ODDLINES_PHASE_DE, "Phase f¸r Farbtr‰ger in ungeraden Zeilen setzen"},
	/* es */ {IDCLS_SET_ODDLINES_PHASE_ES, "Seleccionar fase trasporte color en lineas impares"},
	/* fr */ {IDCLS_SET_ODDLINES_PHASE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_ODDLINES_PHASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_ODDLINES_PHASE_IT, "Imposta la fase per la portante colore nelle linee dispari"},
	/* ko */ {IDCLS_SET_ODDLINES_PHASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_ODDLINES_PHASE_NL, "Stel de fase in van de kleuren drager in oneven lijnen"},
	/* pl */ {IDCLS_SET_ODDLINES_PHASE_PL, "Okre∂l fazÍ no∂nika koloru w liniach nieparzystych"},
	/* ru */ {IDCLS_SET_ODDLINES_PHASE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_ODDLINES_PHASE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_ODDLINES_PHASE_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_ODDLINES_OFFSET,    N_("Set phase offset for color carrier in odd lines")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_ODDLINES_OFFSET_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_ODDLINES_OFFSET_DE, "Phasenoffset f¸r Farbtr‰ger in ungeraden Zeilen setzen"},
	/* es */ {IDCLS_SET_ODDLINES_OFFSET_ES, "Seleccionar aÒadido fase trasporte color en lineas impares"},
	/* fr */ {IDCLS_SET_ODDLINES_OFFSET_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_ODDLINES_OFFSET_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_ODDLINES_OFFSET_IT, "Imposta l'offset della fase per la portate colore nelle linee dispari"},
	/* ko */ {IDCLS_SET_ODDLINES_OFFSET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_ODDLINES_OFFSET_NL, "Stel de fase in compensatie van de kleuren drager in oneven lijnen"},
	/* pl */ {IDCLS_SET_ODDLINES_OFFSET_PL, "Okre∂l wyrÛwnanie fazy no∂nika koloru w liniach nieparzystych"},
	/* ru */ {IDCLS_SET_ODDLINES_OFFSET_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_ODDLINES_OFFSET_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_ODDLINES_OFFSET_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_BLUR,    N_("Amount of horizontal blur for the CRT emulation.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_BLUR_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_BLUR_DE, "Wert f¸r horizontale Unsch‰rfe f¸r CRT Emulation."},
	/* es */ {IDCLS_SET_BLUR_ES, "Cantidad de borrosidad para EmulaciÛn CRT."},
	/* fr */ {IDCLS_SET_BLUR_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_BLUR_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_BLUR_IT, "Valore della sfuocatura orizzontale per l'emulazione CRT"},
	/* ko */ {IDCLS_SET_BLUR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_BLUR_NL, "Horizontale wazigheid hoeveelheid voor de CRT emulatie."},
	/* pl */ {IDCLS_SET_BLUR_PL, "Warto∂Ê rozmycia w poziomie emulacji CRT"},
	/* ru */ {IDCLS_SET_BLUR_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_BLUR_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_BLUR_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_SCANLINE_SHADE,    N_("Amount of scan line shading for the CRT emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_SCANLINE_SHADE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_SCANLINE_SHADE_DE, "Wert f¸r Scanline Schatten f¸r CRT Emulation"},
	/* es */ {IDCLS_SET_SCANLINE_SHADE_ES, "Cantidad de lineas sombra escaneo para emulaciÛn CRT"},
	/* fr */ {IDCLS_SET_SCANLINE_SHADE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_SCANLINE_SHADE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_SCANLINE_SHADE_IT, "Valore dell'ombreggiatura della scan line per l'emulazione CRT"},
	/* ko */ {IDCLS_SET_SCANLINE_SHADE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_SCANLINE_SHADE_NL, "Hoeveelheid van de scan lijn schaduw voor de CRT emulatie"},
	/* pl */ {IDCLS_SET_SCANLINE_SHADE_PL, "Warto∂Ê cieniowania linii wybierania emulacji CRT"},
	/* ru */ {IDCLS_SET_SCANLINE_SHADE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_SCANLINE_SHADE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_SCANLINE_SHADE_TR, ""},  /* fuzzy */
#endif
	
	/* viciisc/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_VICII_MODEL,    N_("Set VIC-II model (6569/6569r1/8565/6567/8562/6567r56a/6572)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_VICII_MODEL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_VICII_MODEL_DE, "W‰hle VIC-II Modell (6569/6569r1/8565/6567/8562/6567r56a/6572)"},
	/* es */ {IDCLS_SET_VICII_MODEL_ES, "Seleccionar modelo VIC-II (6569/6569r1/8565/6567/8562/6567r56a/6572)"},
	/* fr */ {IDCLS_SET_VICII_MODEL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_VICII_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_VICII_MODEL_IT, "Imposta modello VIC-II (6569/6569r1/8565/6567/8562/6567r56a/6572)"},
	/* ko */ {IDCLS_SET_VICII_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_VICII_MODEL_NL, "Zet VIC-II model (6569/6569r1/8565/6567/8562/6567r56a/6572)"},
	/* pl */ {IDCLS_SET_VICII_MODEL_PL, "Ustaw model VIC-II (6569/6569r1/8565/6567/8562/6567r56a/6572)"},
	/* ru */ {IDCLS_SET_VICII_MODEL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_VICII_MODEL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_VICII_MODEL_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/driver-select.c */
	/* en */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME,    N_("Specify name of printer driver for device #4. (ascii/mps803/nl10/raw)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_DE, "Name f¸r Druckertreiber f¸r Ger‰t #4 w‰hlen. (ascii/mps803/nl10/raw)"},
	/* es */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_ES, "Especificar nombre controlador impresora perifÈrico #4. (ascii/mps803/nl10/raw)"},
	/* fr */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_IT, "Specifica il nome del driver di stampa per la periferica #4. (ascii/mps803/nl10/raw)"},
	/* ko */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_NL, "Geef de naam van het stuurprogramma voor apparaat #4. (ascii/mps803/nl10/raw)"},
	/* pl */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_PL, "Okre∂l nazwÍ sterownika drukarki urz±dzenia #4. (ascii/mps803/nl10/raw)"},
	/* ru */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_PRT_DRIVER_4_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/driver-select.c */
	/* en */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME,    N_("Specify name of printer driver for device #5. (ascii/mps803/nl10/raw)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_DE, "Name f¸r Druckertreiber f¸r Ger‰t #5 w‰hlen. (ascii/mps803/nl10/raw)"},
	/* es */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_ES, "Especificar nombre controlador impresora perifÈrico #5. (ascii/mps803/nl10/raw)"},
	/* fr */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_IT, "Specifica il nome del driver di stampa per la periferica #5. (ascii/mps803/nl10/raw)"},
	/* ko */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_NL, "Geef de naam van het stuurprogramma voor apparaat #5. (ascii/mps803/nl10/raw)"},
	/* pl */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_PL, "Okre∂l nazwÍ sterownika drukarki urz±dzenia #5. (ascii/mps803/nl10/raw)"},
	/* ru */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_PRT_DRIVER_5_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/driver-select.c */
	/* en */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME,    N_("Specify name of printer driver for device #6. (1520/raw)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_DE, "Name f¸r Druckertreiber f¸r Ger‰t #6 w‰hlen. (1520/raw)"},
	/* es */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_ES, "Especificar nombre controlador impresora perifÈrico #6. (ascii/mps803/nl10/raw)"},
	/* fr */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_IT, "Specifica il nome del driver di stampa per la periferica #4. (1520/raw)"},
	/* ko */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_NL, "Geef de naam van het stuurprogramma voor apparaat #6. (1520/raw)"},
	/* pl */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_PL, "Okre∂l nazwÍ sterownika drukarki urz±dzenia #6. (1520/raw)"},
	/* ru */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_PRT_DRIVER_6_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/driver-select.c */
	/* en */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME,    N_("Specify name of printer driver for the userport printer. (ascii/nl10/raw)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_DE, "Name f¸r Userport Druckertreiber w‰hlen. (ascii/nl10/raw)"},
	/* es */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_ES, "Especificar nombre controlador impresiÛn para puerto impresora. (ascii/nl10/raw)"},
	/* fr */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_IT, "Specifica il nome del driver di stampa per la stampante su userport. (ascii/nl10/raw)"},
	/* ko */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_NL, "Geef de naam van het stuurprogramma voor de userport printer. (ascii/nl10/raw)"},
	/* pl */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_PL, "Okre∂l nazwÍ sterownika drukarki dla drukarki w userporcie. (ascii/nl10/raw)"},
	/* ru */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_PRT_DRIVER_USR_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/interface-serial.c */
	/* en */ {IDCLS_SET_DEVICE_TYPE_4,    N_("Set device type for device #4 (0: None, 1: Filesystem, 2: Real)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DEVICE_TYPE_4_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DEVICE_TYPE_4_DE, "Ger‰tetyp f¸r Ger‰t #4 (0: Kein, 1: Dateisystem, 2: Real) setzen"},
	/* es */ {IDCLS_SET_DEVICE_TYPE_4_ES, "Seleccionar tipo para el perifÈrico #4 (0:Ninguno 1: Filesystem, 2:Real)"},
	/* fr */ {IDCLS_SET_DEVICE_TYPE_4_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DEVICE_TYPE_4_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DEVICE_TYPE_4_IT, "Specifica il tipo di periferica #4 (0: Nessuna, 1: Filesystem, 2: Reale)"},
	/* ko */ {IDCLS_SET_DEVICE_TYPE_4_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DEVICE_TYPE_4_NL, "Stel apparaatsoort invoor apparaat #4 (0: Geen, 1: Bestandssysteem, 2: Echt)"},
	/* pl */ {IDCLS_SET_DEVICE_TYPE_4_PL, "Okre∂l typ urz±dzenia #4 (0: Brak, 1: System plikÛw, 2: Rzeczywiste)"},
	/* ru */ {IDCLS_SET_DEVICE_TYPE_4_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DEVICE_TYPE_4_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DEVICE_TYPE_4_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/interface-serial.c */
	/* en */ {IDCLS_SET_DEVICE_TYPE_5,    N_("Set device type for device #5 (0: None, 1: Filesystem, 2: Real)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DEVICE_TYPE_5_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DEVICE_TYPE_5_DE, "Ger‰tetyp f¸r Ger‰t #5 (0: Kein, 1: Dateisystem, 2: Real) setzen"},
	/* es */ {IDCLS_SET_DEVICE_TYPE_5_ES, "Seleccionar tipo para el perifÈrico #5 (0:Ninguno 1: Filesystem, 2:Real)"},
	/* fr */ {IDCLS_SET_DEVICE_TYPE_5_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DEVICE_TYPE_5_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DEVICE_TYPE_5_IT, "Specifica il tipo di periferica #5 (0: Nessuna, 1: Filesystem, 2: Reale)"},
	/* ko */ {IDCLS_SET_DEVICE_TYPE_5_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DEVICE_TYPE_5_NL, "Stel apparaatsoort in voor apparaat #5 (0: Geen, 1: Bestandssysteem, 2: Echt)"},
	/* pl */ {IDCLS_SET_DEVICE_TYPE_5_PL, "Okre∂l typ urz±dzenia #5 (0: Brak, 1: System plikÛw, 2: Rzeczywiste)"},
	/* ru */ {IDCLS_SET_DEVICE_TYPE_5_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DEVICE_TYPE_5_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DEVICE_TYPE_5_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/interface-serial.c */
	/* en */ {IDCLS_SET_DEVICE_TYPE_6,    N_("Set device type for device #6 (0: None, 1: Filesystem, 2: Real)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DEVICE_TYPE_6_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DEVICE_TYPE_6_DE, "Ger‰tetyp f¸r Ger‰t #6 (0: Kein, 1: Dateisystem, 2: Real) setzen"},
	/* es */ {IDCLS_SET_DEVICE_TYPE_6_ES, "Seleccionar tipo para el perifÈrico #6 (0:Ninguno 1: Filesystem, 2:Real)"},
	/* fr */ {IDCLS_SET_DEVICE_TYPE_6_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DEVICE_TYPE_6_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DEVICE_TYPE_6_IT, "Specifica il tipo di periferica #6 (0: Nessuna, 1: Filesystem, 2: Reale)"},
	/* ko */ {IDCLS_SET_DEVICE_TYPE_6_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DEVICE_TYPE_6_NL, "Stel apparaatsoort in voor apparaat #6 (0: Geen, 1: Bestandssysteem, 2: Echt)"},
	/* pl */ {IDCLS_SET_DEVICE_TYPE_6_PL, "Okre∂l typ urz±dzenia #6 (0: Brak, 1: System plikÛw, 2: Rzeczywiste)"},
	/* ru */ {IDCLS_SET_DEVICE_TYPE_6_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DEVICE_TYPE_6_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DEVICE_TYPE_6_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/interface-serial.c */
	/* en */ {IDCLS_SET_DEVICE_TYPE_7,    N_("Set device type for device #7 (0: None, 2: Real)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DEVICE_TYPE_7_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DEVICE_TYPE_7_DE, "Ger‰tetyp f¸r Ger‰t #7 (0: Kein, 2: Real) setzen"},
	/* es */ {IDCLS_SET_DEVICE_TYPE_7_ES, "Seleccionar tipo para el perifÈrico #7 (0:Ninguno 1: Filesystem, 2:Real)"},
	/* fr */ {IDCLS_SET_DEVICE_TYPE_7_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DEVICE_TYPE_7_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DEVICE_TYPE_7_IT, "Specifica il tipo di periferica #7 (0: Nessuna, 1: Filesystem, 2: Reale)"},
	/* ko */ {IDCLS_SET_DEVICE_TYPE_7_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DEVICE_TYPE_7_NL, "Stel apparaatsoort in voor apparaat #7 (0: Geen, 2: Echt)"},
	/* pl */ {IDCLS_SET_DEVICE_TYPE_7_PL, "Okre∂l typ urz±dzenia #7 (0: Brak, 2: Rzeczywiste)"},
	/* ru */ {IDCLS_SET_DEVICE_TYPE_7_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DEVICE_TYPE_7_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DEVICE_TYPE_7_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/interface-userport.c */
	/* en */ {IDCLS_ENABLE_USERPORT_PRINTER,    N_("Enable the userport printer emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_PRINTER_DA, "AktivÈr emulering af printer pÂ brugerporten"},
	/* de */ {IDCLS_ENABLE_USERPORT_PRINTER_DE, "Userport Drucker Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_PRINTER_ES, "Permitir emulaciÛn puerto impresora"},
	/* fr */ {IDCLS_ENABLE_USERPORT_PRINTER_FR, "Acviver l'Èmulation de l'imprimante userport"},
	/* hu */ {IDCLS_ENABLE_USERPORT_PRINTER_HU, "Userport nyomtatÛ emul·ciÛj·nak engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_USERPORT_PRINTER_IT, "Attiva l'emulazione della stampante su userport"},
	/* ko */ {IDCLS_ENABLE_USERPORT_PRINTER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_PRINTER_NL, "Userport printeremulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_PRINTER_PL, "W≥±cz emulacjÍ drukarki w userporcie"},
	/* ru */ {IDCLS_ENABLE_USERPORT_PRINTER_RU, "Enable the userport printer emulation"},
	/* sv */ {IDCLS_ENABLE_USERPORT_PRINTER_SV, "Aktivera emulering av skrivare pÂ anv‰ndarporten"},
	/* tr */ {IDCLS_ENABLE_USERPORT_PRINTER_TR, "Userport yaz˝c˝ em¸lasyonunu aktif et"},
#endif
	
	/* printerdrv/interface-userport.c */
	/* en */ {IDCLS_DISABLE_USERPORT_PRINTER,    N_("Disable the userport printer emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_PRINTER_DA, "DeaktivÈr emulering af printer pÂ brugerporten"},
	/* de */ {IDCLS_DISABLE_USERPORT_PRINTER_DE, "Userport Drucker Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_PRINTER_ES, "Deshabilitar emulaciÛn puerto impresora"},
	/* fr */ {IDCLS_DISABLE_USERPORT_PRINTER_FR, "DÈsactiver l'Èmulation de l'imprimante userport"},
	/* hu */ {IDCLS_DISABLE_USERPORT_PRINTER_HU, "Userport nyomtatÛ emul·ciÛj·nak tilt·sa"},
	/* it */ {IDCLS_DISABLE_USERPORT_PRINTER_IT, "Disattiva l'emulazione della stampante su userport"},
	/* ko */ {IDCLS_DISABLE_USERPORT_PRINTER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_PRINTER_NL, "Userport printeremulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_PRINTER_PL, "Wy≥±cz emulacjÍ drukarki w userporcie"},
	/* ru */ {IDCLS_DISABLE_USERPORT_PRINTER_RU, "Disable the userport printer emulation"},
	/* sv */ {IDCLS_DISABLE_USERPORT_PRINTER_SV, "Inaktivera emulering av skrivare pÂ anv‰ndarporten"},
	/* tr */ {IDCLS_DISABLE_USERPORT_PRINTER_TR, "Userport yaz˝c˝ em¸lasyonunu pasifle˛tir"},
#endif
	
	/* printerdrv/output-select.c */
	/* en */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME,    N_("Specify name of output device for device #4")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_DA, "Angiv navn pÂ uddataenhed for enhed #4"},
	/* de */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_DE, "Name f¸r Druckerausgabe Datei Ger‰t #4 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_ES, "Especificar nombre perifÈrico de salida para perifÈrico #4"},
	/* fr */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_FR, "SpÈcifiez le nom du pÈriphÈrique de sortie pour le pÈriphÈrique #4"},
	/* hu */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_HU, "Adja meg a kimeneti eszkˆz nevÈt a #4-es egysÈghez"},
	/* it */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_IT, "Specifica il nome del dispositivo di output per la periferica #4"},
	/* ko */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_NL, "Geef de naam van het uitvoerapparaat voor apparaat #4"},
	/* pl */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_PL, "Okre∂l nazwÍ urz±dzenia wyj∂cia #4"},
	/* ru */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_RU, "Specify name of output device for device #4"},
	/* sv */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_SV, "Ange namn pÂ utskriftsenhet fˆr enhet 4"},
	/* tr */ {IDCLS_SPECIFY_OUTPUT_DEVICE_4_NAME_TR, "Ayg˝t #4 iÁin Á˝k˝˛ ayg˝t˝ ismini belirt"},
#endif
	
	/* printerdrv/output-select.c */
	/* en */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME,    N_("Specify name of output device for device #5")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_DA, "Angiv navn pÂ uddataenhed for enhed #5"},
	/* de */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_DE, "Name f¸r Druckerausgabe Datei Ger‰t #5 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_ES, "Especificar nombre perifÈrico de salida para perifÈrico #5"},
	/* fr */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_FR, "SpÈcifiez le nom du pÈriphÈrique de sortie pour le pÈriphÈrique #5"},
	/* hu */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_HU, "Adja meg a kimeneti eszkˆz nevÈt a #5-ˆs egysÈghez"},
	/* it */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_IT, "Specifica il nome del dispositivo di output per la periferica #5"},
	/* ko */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_NL, "Geef de naam van het uitvoerapparaat voor apparaat #5"},
	/* pl */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_PL, "Okre∂l nazwÍ urz±dzenia wyj∂cia #5"},
	/* ru */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_RU, "Specify name of output device for device #5"},
	/* sv */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_SV, "Ange namn pÂ utskriftsenhet fˆr enhet 5"},
	/* tr */ {IDCLS_SPECIFY_OUTPUT_DEVICE_5_NAME_TR, "Ayg˝t #5 iÁin Á˝k˝˛ ayg˝t˝ ismini belirt"},
#endif
	
	/* printerdrv/output-select.c */
	/* en */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME,    N_("Specify name of output device for device #6")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_DE, "Name f¸r Druckerausgabe Datei Ger‰t #6 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_NL, "Geef de naam van het uitvoerapparaat voor apparaat #6"},
	/* pl */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_PL, "Okre∂l nazwÍ urz±dzenia wyj∂cia #6"},
	/* ru */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_OUTPUT_DEVICE_6_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/output-select.c */
	/* en */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME,    N_("Specify name of output device for the userport printer")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_DA, "Angiv navn pÂ uddataenhed for brugerports-printer"},
	/* de */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_DE, "Name f¸r Druckerausgabe Datei Userport w‰hlen"},
	/* es */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_ES, "Especificar nombre perifÈrico de salida para puerto usuario impresora"},
	/* fr */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_FR, "SpÈcifiez le nom du pÈriphÈrique de sortie pour l'imprimante userport"},
	/* hu */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_HU, "Adja meg a kimeneti eszkˆz nevÈt a userport nyomtatÛhoz"},
	/* it */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_IT, "Specifica il nome del dispositivo di output per la stampante su userport"},
	/* ko */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_NL, "Geef de naam van het uitvoerapparaat voor de userport printer"},
	/* pl */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_PL, "Okre∂l nazwÍ urz±dzenia wyj∂ciowego dla drukarki w userporcie"},
	/* ru */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_RU, "Specify name of output device for the userport printer"},
	/* sv */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_SV, "Ange namn pÂ utskriftsenhet fˆr anv‰ndarporten"},
	/* tr */ {IDCLS_SPECIFY_OUTPUT_DEVICE_USR_NAME_TR, "Userport yaz˝c˝s˝ iÁin Á˝k˝˛ ayg˝t˝ ismini belirt"},
#endif
	
	/* printerdrv/output-text.c */
	/* en */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME,    N_("Specify name of printer text device or dump file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_DA, "Angiv navn pÂ printer-tekst-enhed eller dump-fil"},
	/* de */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_DE, "Name f¸r Druckerausgabe Textger‰t oder Dump Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_ES, "Especificar nombre del perifÈrico para imprimir texto o fichero de volcado"},
	/* fr */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_FR, "SpÈcifier le nom de l'imprimante texte ou \"dump file\""},
	/* hu */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_HU, "Adja meg a szˆvegnyomtatÛ eszkˆz nevÈt, vagy a kimeneti f·jlt"},
	/* it */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_IT, "Specifica il nome del dispositivo testuale di stampa o salva su file"},
	/* ko */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_NL, "Geef de naam van het printer tekst apparaat of het dumpbestand"},
	/* pl */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_PL, "Okre∂l nazwÍ urz±dzenia tekstowego drukarki lub pliku zrzutu"},
	/* ru */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_RU, "Specify name of printer text device or dump file"},
	/* sv */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_SV, "Ange namn pÂ textenhet fˆr utskrift eller dumpfil"},
	/* tr */ {IDCLS_SPECIFY_TEXT_DEVICE_DUMP_NAME_TR, "Metin ayg˝t˝ yaz˝c˝s˝ ya da dˆk¸m dosyas˝n˝n ismini belirt"},
#endif
	
	/* printerdrv/output-text.c */
	/* en */ {IDCLS_SPECIFY_TEXT_DEVICE_4,    N_("Specify printer text output device for printer #4")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TEXT_DEVICE_4_DA, "Angiv printer tekst-uddata enhed for printer #4"},
	/* de */ {IDCLS_SPECIFY_TEXT_DEVICE_4_DE, "Textausgabe Ger‰t f¸r Drucker #4 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_TEXT_DEVICE_4_ES, "Especificar perifÈrico salida impresiÛn texto impresora #4"},
	/* fr */ {IDCLS_SPECIFY_TEXT_DEVICE_4_FR, "SpÈcifier le pÈriphÈrique de sortie texte pour l'imprimante 4"},
	/* hu */ {IDCLS_SPECIFY_TEXT_DEVICE_4_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_TEXT_DEVICE_4_IT, "Specifica il dispositivo di uscita testuale per la stampante #4"},
	/* ko */ {IDCLS_SPECIFY_TEXT_DEVICE_4_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TEXT_DEVICE_4_NL, "Geef het printer tekst uitvoer apparaat voor printer #4"},
	/* pl */ {IDCLS_SPECIFY_TEXT_DEVICE_4_PL, "Okre∂l urz±dzenie wyj∂cia tekstowego drukarki #4"},
	/* ru */ {IDCLS_SPECIFY_TEXT_DEVICE_4_RU, "Specify printer text output device for printer #4"},
	/* sv */ {IDCLS_SPECIFY_TEXT_DEVICE_4_SV, "Ange skrivartextutenhet fˆr skrivare 4"},
	/* tr */ {IDCLS_SPECIFY_TEXT_DEVICE_4_TR, "Yaz˝c˝ #4 iÁin metin Á˝kt˝ ayg˝t˝ yaz˝c˝s˝ belirt"},
#endif
	
	/* printerdrv/output-text.c */
	/* en */ {IDCLS_SPECIFY_TEXT_DEVICE_5,    N_("Specify printer text output device for printer #5")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TEXT_DEVICE_5_DA, "Angiv printer tekst-uddata enhed for printer #5"},
	/* de */ {IDCLS_SPECIFY_TEXT_DEVICE_5_DE, "Textausgabe Ger‰t f¸r Drucker #5 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_TEXT_DEVICE_5_ES, "Especificar perifÈrico salida impresiÛn texto impresora #5"},
	/* fr */ {IDCLS_SPECIFY_TEXT_DEVICE_5_FR, "SpÈcifier le pÈriphÈrique de sortie texte pour l'imprimante 5"},
	/* hu */ {IDCLS_SPECIFY_TEXT_DEVICE_5_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_TEXT_DEVICE_5_IT, "Specifica il dispositivo di uscita testuale per la stampante #5"},
	/* ko */ {IDCLS_SPECIFY_TEXT_DEVICE_5_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TEXT_DEVICE_5_NL, "Geef het printer tekst uitvoer apparaat voor printer #5"},
	/* pl */ {IDCLS_SPECIFY_TEXT_DEVICE_5_PL, "Okre∂l urz±dzenie wyj∂cia tekstowego drukarki #5"},
	/* ru */ {IDCLS_SPECIFY_TEXT_DEVICE_5_RU, "Specify printer text output device for printer #5"},
	/* sv */ {IDCLS_SPECIFY_TEXT_DEVICE_5_SV, "Ange skrivartextutenhet fˆr skrivare 5"},
	/* tr */ {IDCLS_SPECIFY_TEXT_DEVICE_5_TR, "Yaz˝c˝ #5 iÁin metin Á˝kt˝ ayg˝t˝ yaz˝c˝s˝ belirt"},
#endif
	
	/* printerdrv/output-text.c */
	/* en */ {IDCLS_SPECIFY_TEXT_DEVICE_6,    N_("Specify printer text output device for printer #6")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TEXT_DEVICE_6_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_TEXT_DEVICE_6_DE, "Textausgabe Ger‰t f¸r Drucker #6 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_TEXT_DEVICE_6_ES, "Especificar perifÈrico salida impresiÛn texto impresora #6"},
	/* fr */ {IDCLS_SPECIFY_TEXT_DEVICE_6_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_TEXT_DEVICE_6_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_TEXT_DEVICE_6_IT, "Specifica il dispositivo di uscita testuale per la stampante #6"},
	/* ko */ {IDCLS_SPECIFY_TEXT_DEVICE_6_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TEXT_DEVICE_6_NL, "Geef het printer tekst uitvoer apparaat voor printer #6"},
	/* pl */ {IDCLS_SPECIFY_TEXT_DEVICE_6_PL, "Okre∂l urz±dzenie wyj∂cia tekstowego drukarki #6"},
	/* ru */ {IDCLS_SPECIFY_TEXT_DEVICE_6_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_TEXT_DEVICE_6_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_TEXT_DEVICE_6_TR, ""},  /* fuzzy */
#endif
	
	/* printerdrv/output-text.c */
	/* en */ {IDCLS_SPECIFY_TEXT_USERPORT,    N_("Specify printer text output device for userport printer")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TEXT_USERPORT_DA, "Angiv printer tekst-uddata enhed for brugerports-printer"},
	/* de */ {IDCLS_SPECIFY_TEXT_USERPORT_DE, "Textausgabe Ger‰t f¸r Userport Drucker w‰hlen"},
	/* es */ {IDCLS_SPECIFY_TEXT_USERPORT_ES, "Especificar perifÈrico de salida para imprimir texto impresora puerto usuario"},
	/* fr */ {IDCLS_SPECIFY_TEXT_USERPORT_FR, "SpÈcifier le pÈriphÈrique de sortie text pour l'imprimante userport"},
	/* hu */ {IDCLS_SPECIFY_TEXT_USERPORT_HU, "Adja meg a szˆvegnyomtatÛ eszkˆzt a userport nyomtatÛhoz"},
	/* it */ {IDCLS_SPECIFY_TEXT_USERPORT_IT, "Specifica il nome del dispositivo di output testuale di stampa per la stampante su userport"},
	/* ko */ {IDCLS_SPECIFY_TEXT_USERPORT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TEXT_USERPORT_NL, "Geef het printer tekst uitvoer apparaat voor de userport printer"},
	/* pl */ {IDCLS_SPECIFY_TEXT_USERPORT_PL, "Okre∂l urz±dzenie wyj∂cia tekstu dla drukarki w userporcie"},
	/* ru */ {IDCLS_SPECIFY_TEXT_USERPORT_RU, "Specify printer text output device for userport printer"},
	/* sv */ {IDCLS_SPECIFY_TEXT_USERPORT_SV, "Ange skrivartextutenhet fˆr anv‰ndarportsskrivare"},
	/* tr */ {IDCLS_SPECIFY_TEXT_USERPORT_TR, "Userport yaz˝c˝s˝ iÁin metin Á˝kt˝ ayg˝t˝ yaz˝c˝s˝ belirt"},
#endif
	
	/* raster/raster-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_VIDEO_CACHE,    N_("Enable the video cache")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_VIDEO_CACHE_DA, "AktivÈr videocache"},
	/* de */ {IDCLS_ENABLE_VIDEO_CACHE_DE, "Video Cache aktivieren"},
	/* es */ {IDCLS_ENABLE_VIDEO_CACHE_ES, "Permitir cachÈ de video"},
	/* fr */ {IDCLS_ENABLE_VIDEO_CACHE_FR, "Activer le cache vidÈo"},
	/* hu */ {IDCLS_ENABLE_VIDEO_CACHE_HU, "VideÛ gyorsÌtÛt·r engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_VIDEO_CACHE_IT, "Attiva la cache video"},
	/* ko */ {IDCLS_ENABLE_VIDEO_CACHE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_VIDEO_CACHE_NL, "Videocache inschakelen"},
	/* pl */ {IDCLS_ENABLE_VIDEO_CACHE_PL, "W≥±cz cache wideo"},
	/* ru */ {IDCLS_ENABLE_VIDEO_CACHE_RU, "Enable the video cache"},
	/* sv */ {IDCLS_ENABLE_VIDEO_CACHE_SV, "Aktivera videocache"},
	/* tr */ {IDCLS_ENABLE_VIDEO_CACHE_TR, "Gˆr¸nt¸ ˆnbelleini aktif et"},
#endif
	
	/* raster/raster-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_VIDEO_CACHE,    N_("Disable the video cache")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_VIDEO_CACHE_DA, "DeaktivÈr videocache"},
	/* de */ {IDCLS_DISABLE_VIDEO_CACHE_DE, "Video Cache deaktivieren"},
	/* es */ {IDCLS_DISABLE_VIDEO_CACHE_ES, "Deshabilitar cachÈ de vÌdeo"},
	/* fr */ {IDCLS_DISABLE_VIDEO_CACHE_FR, "DÈsactiver le cache vidÈo"},
	/* hu */ {IDCLS_DISABLE_VIDEO_CACHE_HU, "VideÛ gyorsÌtÛt·r tilt·sa"},
	/* it */ {IDCLS_DISABLE_VIDEO_CACHE_IT, "Disattiva la cache video"},
	/* ko */ {IDCLS_DISABLE_VIDEO_CACHE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_VIDEO_CACHE_NL, "Videocache uitschakelen"},
	/* pl */ {IDCLS_DISABLE_VIDEO_CACHE_PL, "Wy≥±cz cache wideo"},
	/* ru */ {IDCLS_DISABLE_VIDEO_CACHE_RU, "Disable the video cache"},
	/* sv */ {IDCLS_DISABLE_VIDEO_CACHE_SV, "Inaktivera videocache"},
	/* tr */ {IDCLS_DISABLE_VIDEO_CACHE_TR, "Gˆr¸nt¸ ˆnbelleini pasifle˛tir"},
#endif
	
#if defined(HAVE_RS232DEV) || defined(HAVE_RS232NET)
	/* rs232drv/rs232drv.c */
	/* en */ {IDCLS_SPECIFY_RS232_1_NAME,    N_("Specify name of first RS232 device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_RS232_1_NAME_DA, "Angiv navn pÂ f¯rste RS232-enhed"},
	/* de */ {IDCLS_SPECIFY_RS232_1_NAME_DE, "Name f¸r erstes RS232 Ger‰t w‰hlen"},
	/* es */ {IDCLS_SPECIFY_RS232_1_NAME_ES, "Especificar el nombre del primer perifÈrico RS232"},
	/* fr */ {IDCLS_SPECIFY_RS232_1_NAME_FR, "SpÈcifier le nom du 1er pÈriphÈrique RS232"},
	/* hu */ {IDCLS_SPECIFY_RS232_1_NAME_HU, "Adja meg az elsı RS232-es eszkˆz nevÈt"},
	/* it */ {IDCLS_SPECIFY_RS232_1_NAME_IT, "Specifica il nome della prima RS232"},
	/* ko */ {IDCLS_SPECIFY_RS232_1_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_RS232_1_NAME_NL, "Geef de naam van het eerste RS232 apparaat"},
	/* pl */ {IDCLS_SPECIFY_RS232_1_NAME_PL, "Okre∂l nazwÍ pierwszego urz±dzenia RS232"},
	/* ru */ {IDCLS_SPECIFY_RS232_1_NAME_RU, "Specify name of first RS232 device"},
	/* sv */ {IDCLS_SPECIFY_RS232_1_NAME_SV, "Ange namn pÂ fˆrsta RS232-enheten"},
	/* tr */ {IDCLS_SPECIFY_RS232_1_NAME_TR, "›lk RS232 ayg˝t˝n˝n ismini belirt"},
#endif
	
	/* rs232drv/rs232drv.c */
	/* en */ {IDCLS_SPECIFY_RS232_2_NAME,    N_("Specify name of second RS232 device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_RS232_2_NAME_DA, "Angiv navn pÂ anden RS232-enhed"},
	/* de */ {IDCLS_SPECIFY_RS232_2_NAME_DE, "Name f¸r zweites RS232 Ger‰t w‰hlen"},
	/* es */ {IDCLS_SPECIFY_RS232_2_NAME_ES, "Especificar el nombre del segundo perifÈrico RS232"},
	/* fr */ {IDCLS_SPECIFY_RS232_2_NAME_FR, "SpÈcifier le nom du 2e pÈriphÈrique RS232"},
	/* hu */ {IDCLS_SPECIFY_RS232_2_NAME_HU, "Adja meg a m·sodik RS232-es eszkˆz nevÈt"},
	/* it */ {IDCLS_SPECIFY_RS232_2_NAME_IT, "Specifica il nome della seconda RS232"},
	/* ko */ {IDCLS_SPECIFY_RS232_2_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_RS232_2_NAME_NL, "Geef de naam van het tweede RS232 apparaat"},
	/* pl */ {IDCLS_SPECIFY_RS232_2_NAME_PL, "Okre∂l nazwÍ drugiego urz±dzenia RS232"},
	/* ru */ {IDCLS_SPECIFY_RS232_2_NAME_RU, "Specify name of second RS232 device"},
	/* sv */ {IDCLS_SPECIFY_RS232_2_NAME_SV, "Ange namn pÂ andra RS232-enheten"},
	/* tr */ {IDCLS_SPECIFY_RS232_2_NAME_TR, "›kinci RS232 ayg˝t˝n˝n ismini belirt"},
#endif
	
	/* rs232drv/rs232drv.c */
	/* en */ {IDCLS_SPECIFY_RS232_3_NAME,    N_("Specify name of third RS232 device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_RS232_3_NAME_DA, "Angiv navn pÂ tredje RS232-enhed"},
	/* de */ {IDCLS_SPECIFY_RS232_3_NAME_DE, "Name f¸r drittes RS232 Ger‰t w‰hlen"},
	/* es */ {IDCLS_SPECIFY_RS232_3_NAME_ES, "Especificar el nombre del tercer perifÈrico RS232"},
	/* fr */ {IDCLS_SPECIFY_RS232_3_NAME_FR, "SpÈcifier le nom du 3e pÈriphÈrique RS232"},
	/* hu */ {IDCLS_SPECIFY_RS232_3_NAME_HU, "Adja meg a harmadik RS232-es eszkˆz nevÈt"},
	/* it */ {IDCLS_SPECIFY_RS232_3_NAME_IT, "Specifica il nome della terza RS232"},
	/* ko */ {IDCLS_SPECIFY_RS232_3_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_RS232_3_NAME_NL, "Geef de naam van het derde RS232 apparaat"},
	/* pl */ {IDCLS_SPECIFY_RS232_3_NAME_PL, "Okre∂l nazwÍ trzeciego urz±dzenia RS232"},
	/* ru */ {IDCLS_SPECIFY_RS232_3_NAME_RU, "Specify name of third RS232 device"},
	/* sv */ {IDCLS_SPECIFY_RS232_3_NAME_SV, "Ange namn pÂ tredje RS232-enheten"},
	/* tr */ {IDCLS_SPECIFY_RS232_3_NAME_TR, "‹Á¸nc¸ RS232 ayg˝t˝n˝n ismini belirt"},
#endif
	
	/* rs232drv/rs232drv.c */
	/* en */ {IDCLS_SPECIFY_RS232_4_NAME,    N_("Specify name of fourth RS232 device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_RS232_4_NAME_DA, "Angiv navn pÂ fjerde RS232-enhed"},
	/* de */ {IDCLS_SPECIFY_RS232_4_NAME_DE, "Name f¸r viertes RS232 Ger‰t w‰hlen"},
	/* es */ {IDCLS_SPECIFY_RS232_4_NAME_ES, "Especificar el nombre del cuarto perifÈrico RS232"},
	/* fr */ {IDCLS_SPECIFY_RS232_4_NAME_FR, "SpÈcifier le nom du 4e pÈriphÈrique RS232"},
	/* hu */ {IDCLS_SPECIFY_RS232_4_NAME_HU, "Adja meg a negyedik RS232-es eszkˆz nevÈt"},
	/* it */ {IDCLS_SPECIFY_RS232_4_NAME_IT, "Specifica il nome della quarta RS232"},
	/* ko */ {IDCLS_SPECIFY_RS232_4_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_RS232_4_NAME_NL, "Geef de naam van het vierde RS232 apparaat"},
	/* pl */ {IDCLS_SPECIFY_RS232_4_NAME_PL, "Okre∂l nazwÍ czwartego urz±dzenia RS232"},
	/* ru */ {IDCLS_SPECIFY_RS232_4_NAME_RU, "Specify name of fourth RS232 device"},
	/* sv */ {IDCLS_SPECIFY_RS232_4_NAME_SV, "Ange namn pÂ fj‰rde RS232-enheten"},
	/* tr */ {IDCLS_SPECIFY_RS232_4_NAME_TR, "Dˆrd¸nc¸ RS232 ayg˝t˝n˝n ismini belirt"},
#endif
#endif
	
	/* rs232drv/rsuser.c */
	/* en */ {IDCLS_ENABLE_RS232_USERPORT,    N_("Enable RS232 userport emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_RS232_USERPORT_DA, "AktivÈr RS232-emulering pÂ brugerporten"},
	/* de */ {IDCLS_ENABLE_RS232_USERPORT_DE, "RS232 Userport Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_RS232_USERPORT_ES, "Permitir EmulaciÛn puerto RS232"},
	/* fr */ {IDCLS_ENABLE_RS232_USERPORT_FR, "Activer l'Èmulation de pÈriphÈrique RS232 userport"},
	/* hu */ {IDCLS_ENABLE_RS232_USERPORT_HU, "RS232 userport emul·ciÛ engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_RS232_USERPORT_IT, "Attiva l'emulazione della RS232 su userport"},
	/* ko */ {IDCLS_ENABLE_RS232_USERPORT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_RS232_USERPORT_NL, "RS232 userportemulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_RS232_USERPORT_PL, "W≥±cz emulacjÍ userportu RS232"},
	/* ru */ {IDCLS_ENABLE_RS232_USERPORT_RU, "Enable RS232 userport emulation"},
	/* sv */ {IDCLS_ENABLE_RS232_USERPORT_SV, "Aktivera RS232-emulering pÂ anv‰ndarporten"},
	/* tr */ {IDCLS_ENABLE_RS232_USERPORT_TR, "RS232 userport em¸lasyonunu aktif et"},
#endif
	
	/* rs232drv/rsuser.c */
	/* en */ {IDCLS_DISABLE_RS232_USERPORT,    N_("Disable RS232 userport emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_RS232_USERPORT_DA, "DeaktivÈr RS232-emulering pÂ brugerporten"},
	/* de */ {IDCLS_DISABLE_RS232_USERPORT_DE, "RS232 Userport Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_RS232_USERPORT_ES, "Deshabilitar EmulaciÛn puerto RS232"},
	/* fr */ {IDCLS_DISABLE_RS232_USERPORT_FR, "DÈsactiver l'Èmulation de pÈriphÈrique RS232 userport"},
	/* hu */ {IDCLS_DISABLE_RS232_USERPORT_HU, "RS232 userport emul·ciÛ tilt·sa"},
	/* it */ {IDCLS_DISABLE_RS232_USERPORT_IT, "Disattiva l'emulazione della RS232 su userport"},
	/* ko */ {IDCLS_DISABLE_RS232_USERPORT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_RS232_USERPORT_NL, "RS232 userportemulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_RS232_USERPORT_PL, "Wy≥±cz emulacjÍ userportu RS232"},
	/* ru */ {IDCLS_DISABLE_RS232_USERPORT_RU, "Disable RS232 userport emulation"},
	/* sv */ {IDCLS_DISABLE_RS232_USERPORT_SV, "Inaktivera RS232-emulering pÂ anv‰ndarporten"},
	/* tr */ {IDCLS_DISABLE_RS232_USERPORT_TR, "RS232 userport em¸lasyonunu pasifle˛tir"},
#endif
	
	/* rs232drv/rsuser.c */
	/* en */ {IDCLS_P_BAUD,    N_("<baud>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_BAUD_DA, "<baud>"},
	/* de */ {IDCLS_P_BAUD_DE, "<Baud>"},
	/* es */ {IDCLS_P_BAUD_ES, "<baudios>"},
	/* fr */ {IDCLS_P_BAUD_FR, "<baud>"},
	/* hu */ {IDCLS_P_BAUD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_BAUD_IT, "<baud>"},
	/* ko */ {IDCLS_P_BAUD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_BAUD_NL, "<baud>"},
	/* pl */ {IDCLS_P_BAUD_PL, "<bod>"},
	/* ru */ {IDCLS_P_BAUD_RU, "<baud>"},
	/* sv */ {IDCLS_P_BAUD_SV, "<baud>"},
	/* tr */ {IDCLS_P_BAUD_TR, "<baud>"},
#endif
	
	/* rs232drv/rsuser.c */
	/* en */ {IDCLS_SET_BAUD_RS232_USERPORT,    N_("Set the baud rate of the RS232 userport emulation.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_BAUD_RS232_USERPORT_DA, "Angiv baudhastighet for RS232-emuleringen pÂ brugerporten."},
	/* de */ {IDCLS_SET_BAUD_RS232_USERPORT_DE, "Baudrate f¸r RS232 Userport Emulation setzen"},
	/* es */ {IDCLS_SET_BAUD_RS232_USERPORT_ES, "Seleccionar velocidad (baudios) para EmulaciÛn puerto RS232"},
	/* fr */ {IDCLS_SET_BAUD_RS232_USERPORT_FR, "SpÈcifiez la vitesse en baud du pÈriphÈrique RS232 userport"},
	/* hu */ {IDCLS_SET_BAUD_RS232_USERPORT_HU, "Bitr·ta megad·sa az RS232 userport emul·ciÛhoz."},
	/* it */ {IDCLS_SET_BAUD_RS232_USERPORT_IT, "Imposta la velocit‡ della RS232 su userport (in baud)"},
	/* ko */ {IDCLS_SET_BAUD_RS232_USERPORT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_BAUD_RS232_USERPORT_NL, "Stel de baud rate in van de RS232 userport emulatie."},
	/* pl */ {IDCLS_SET_BAUD_RS232_USERPORT_PL, "Okre∂l prÍdko∂Ê transmisji emulacji userportu RS232."},
	/* ru */ {IDCLS_SET_BAUD_RS232_USERPORT_RU, "Set the baud rate of the RS232 userport emulation."},
	/* sv */ {IDCLS_SET_BAUD_RS232_USERPORT_SV, "Ange baudhastighet fˆr RS232-emuleringen pÂ anv‰ndarporten."},
	/* tr */ {IDCLS_SET_BAUD_RS232_USERPORT_TR, "RS232 userport em¸lasyonunun baud rate'ini ayarla."},
#endif
	
	/* rs232drv/rsuser.c */
	/* en */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT,    N_("Specify VICE RS232 device for userport")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_DA, "Angiv VICE RS232-enhed for brugerporten"},
	/* de */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_DE, "VICE RS232 Ger‰t f¸r Userport w‰hlen"},
	/* es */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_ES, "Especificar perifÈrico VICE RS232 para puerto usuario"},
	/* fr */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_FR, "SpÈcifier un pÈriphÈrique VICE RS232 pour le userport"},
	/* hu */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_HU, "VICE RS232-es userport eszkˆz megad·sa"},
	/* it */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_IT, "Specifica il dispositivo RS232 di VICE per userport"},
	/* ko */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_NL, "Geef VICE RS232 apparaat voor userport"},
	/* pl */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_PL, "Okre∂l urz±dzenie VICE RS232 w userporcie"},
	/* ru */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_RU, "Specify VICE RS232 device for userport"},
	/* sv */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_SV, "Ange VICE RS232-enhet fˆr anv‰ndarporten"},
	/* tr */ {IDCLS_SPECIFY_RS232_DEVICE_USERPORT_TR, "Userport iÁin VICE RS232 ayg˝t˝n˝ belirt"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_ENABLE_IEC_4,    N_("Enable IEC device emulation for device #4")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEC_4_DA, "AktivÈr IEC-enhedsemulering for enhed #4"},
	/* de */ {IDCLS_ENABLE_IEC_4_DE, "IEC Ger‰te Emulation f¸r Ger‰t #4 aktivieren"},
	/* es */ {IDCLS_ENABLE_IEC_4_ES, "Permitir emulaciÛn IEC para perifÈrico #4 "},
	/* fr */ {IDCLS_ENABLE_IEC_4_FR, "Activer l'Èmulation IEC pour le pÈriphÈrique #4"},
	/* hu */ {IDCLS_ENABLE_IEC_4_HU, "IEC eszkˆz #4 emul·ciÛj·nak engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_IEC_4_IT, "Attiva emulazione IEC per la periferica #4"},
	/* ko */ {IDCLS_ENABLE_IEC_4_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEC_4_NL, "IEC apparaat emulatie voor apparaat #4 inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEC_4_PL, "W≥±cz emulacjÍ IEC urz±dzenia #4"},
	/* ru */ {IDCLS_ENABLE_IEC_4_RU, "Enable IEC device emulation for device #4"},
	/* sv */ {IDCLS_ENABLE_IEC_4_SV, "Aktivera IEC-enhetsemulering fˆr enhet 4"},
	/* tr */ {IDCLS_ENABLE_IEC_4_TR, "Ayg˝t #4 iÁin IEC ayg˝t em¸lasyonunu aktif et"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_DISABLE_IEC_4,    N_("Disable IEC device emulation for device #4")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEC_4_DA, "DeaktivÈr IEC-enhedsemulering for enhed #4"},
	/* de */ {IDCLS_DISABLE_IEC_4_DE, "IEC Ger‰te Emulation f¸r Ger‰t #4 deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEC_4_ES, "Deshabilitar emulaciÛn IEC para perifÈrico #4 "},
	/* fr */ {IDCLS_DISABLE_IEC_4_FR, "DÈsactiver l'Èmulation IEC pour le pÈriphÈrique #4"},
	/* hu */ {IDCLS_DISABLE_IEC_4_HU, "IEC eszkˆz #4 emul·ciÛj·nak tilt·sa"},
	/* it */ {IDCLS_DISABLE_IEC_4_IT, "Disattiva emulazione IEC per la periferica #4"},
	/* ko */ {IDCLS_DISABLE_IEC_4_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEC_4_NL, "IEC apparaat emulatie voor apparaat #4 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEC_4_PL, "Wy≥±cz emulacjÍ urz±dzenia IEC dla urz±dzenia #4"},
	/* ru */ {IDCLS_DISABLE_IEC_4_RU, "Disable IEC device emulation for device #4"},
	/* sv */ {IDCLS_DISABLE_IEC_4_SV, "Inaktivera IEC-enhetsemulering fˆr enhet 4"},
	/* tr */ {IDCLS_DISABLE_IEC_4_TR, "Ayg˝t #4 iÁin IEC ayg˝t em¸lasyonunu pasifle˛tir"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_ENABLE_IEC_5,    N_("Enable IEC device emulation for device #5")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEC_5_DA, "AktivÈr IEC-enhedsemulering for enhed #5"},
	/* de */ {IDCLS_ENABLE_IEC_5_DE, "IEC Ger‰te Emulation f¸r Ger‰t #5 aktivieren"},
	/* es */ {IDCLS_ENABLE_IEC_5_ES, "Permitir emulaciÛn IEC para perifÈrico #5"},
	/* fr */ {IDCLS_ENABLE_IEC_5_FR, "Activer l'Èmulation IEC pour le pÈriphÈrique #5"},
	/* hu */ {IDCLS_ENABLE_IEC_5_HU, "IEC eszkˆz #5 emul·ciÛj·nak engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_IEC_5_IT, "Attiva emulazione IEC per la periferica #5"},
	/* ko */ {IDCLS_ENABLE_IEC_5_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEC_5_NL, "IEC apparaat emulatie voor apparaat #5 inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEC_5_PL, "W≥±cz emulacjÍ IEC urz±dzenia #5"},
	/* ru */ {IDCLS_ENABLE_IEC_5_RU, "Enable IEC device emulation for device #5"},
	/* sv */ {IDCLS_ENABLE_IEC_5_SV, "Aktivera IEC-enhetsemulering fˆr enhet 5"},
	/* tr */ {IDCLS_ENABLE_IEC_5_TR, "Ayg˝t #5 iÁin IEC ayg˝t em¸lasyonunu aktif et"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_DISABLE_IEC_5,    N_("Disable IEC device emulation for device #5")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEC_5_DA, "DeaktivÈr IEC-enhedsemulering for enhed #5"},
	/* de */ {IDCLS_DISABLE_IEC_5_DE, "IEC Ger‰te Emulation f¸r Ger‰t #5 deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEC_5_ES, "Deshabilitar emulaciÛn IEC para perifÈrico #5"},
	/* fr */ {IDCLS_DISABLE_IEC_5_FR, "DÈsactiver l'Èmulation IEC pour le pÈriphÈrique #5"},
	/* hu */ {IDCLS_DISABLE_IEC_5_HU, "IEC eszkˆz #5 emul·ciÛj·nak tilt·sa"},
	/* it */ {IDCLS_DISABLE_IEC_5_IT, "Disattiva emulazione IEC per la periferica #5"},
	/* ko */ {IDCLS_DISABLE_IEC_5_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEC_5_NL, "IEC apparaat emulatie voor apparaat #5 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEC_5_PL, "Wy≥±cz emulacjÍ urz±dzenia IEC dla urz±dzenia #5"},
	/* ru */ {IDCLS_DISABLE_IEC_5_RU, "Disable IEC device emulation for device #5"},
	/* sv */ {IDCLS_DISABLE_IEC_5_SV, "Inaktivera IEC-enhetsemulering fˆr enhet 5"},
	/* tr */ {IDCLS_DISABLE_IEC_5_TR, "Ayg˝t #5 iÁin IEC ayg˝t em¸lasyonunu pasifle˛tir"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_ENABLE_IEC_6,    N_("Enable IEC device emulation for device #6")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEC_6_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_IEC_6_DE, "IEC Ger‰te Emulation f¸r Ger‰t #6 aktivieren"},
	/* es */ {IDCLS_ENABLE_IEC_6_ES, "Permitir emulaciÛn IEC para perifÈrico #6"},
	/* fr */ {IDCLS_ENABLE_IEC_6_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_IEC_6_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_IEC_6_IT, "Attiva emulazione IEC per la periferica #6"},
	/* ko */ {IDCLS_ENABLE_IEC_6_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEC_6_NL, "IEC apparaat emulatie voor apparaat #6 inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEC_6_PL, "W≥±cz emulacjÍ IEC urz±dzenia #6"},
	/* ru */ {IDCLS_ENABLE_IEC_6_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_IEC_6_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_IEC_6_TR, ""},  /* fuzzy */
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_DISABLE_IEC_6,    N_("Disable IEC device emulation for device #6")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEC_6_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_IEC_6_DE, "IEC Ger‰te Emulation f¸r Ger‰t #6 deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEC_6_ES, "Deshabilitar emulaciÛn IEC para perifÈrico #6"},
	/* fr */ {IDCLS_DISABLE_IEC_6_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_IEC_6_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_IEC_6_IT, "Disattiva emulazione IEC per la periferica #6"},
	/* ko */ {IDCLS_DISABLE_IEC_6_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEC_6_NL, "IEC apparaat emulatie voor apparaat #6 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEC_6_PL, "Wy≥±cz emulacjÍ urz±dzenia IEC dla urz±dzenia #6"},
	/* ru */ {IDCLS_DISABLE_IEC_6_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_IEC_6_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_IEC_6_TR, ""},  /* fuzzy */
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_ENABLE_IEC_7,    N_("Enable IEC device emulation for device #7")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEC_7_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_IEC_7_DE, "IEC Ger‰te Emulation f¸r Ger‰t #7 aktivieren"},
	/* es */ {IDCLS_ENABLE_IEC_7_ES, "Permitir emulaciÛn IEC para perifÈrico #7"},
	/* fr */ {IDCLS_ENABLE_IEC_7_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_IEC_7_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_IEC_7_IT, "Attiva emulazione IEC per la periferica #7"},
	/* ko */ {IDCLS_ENABLE_IEC_7_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEC_7_NL, "IEC apparaat emulatie voor apparaat #7 inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEC_7_PL, "W≥±cz emulacjÍ IEC urz±dzenia #7"},
	/* ru */ {IDCLS_ENABLE_IEC_7_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_IEC_7_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_IEC_7_TR, ""},  /* fuzzy */
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_DISABLE_IEC_7,    N_("Disable IEC device emulation for device #7")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEC_7_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_IEC_7_DE, "IEC Ger‰te Emulation f¸r Ger‰t #7 deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEC_7_ES, "Deshabilitar emulaciÛn IEC para perifÈrico #7"},
	/* fr */ {IDCLS_DISABLE_IEC_7_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_IEC_7_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_IEC_7_IT, "Disattiva emulazione IEC per la periferica #7"},
	/* ko */ {IDCLS_DISABLE_IEC_7_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEC_7_NL, "IEC apparaat emulatie voor apparaat #7 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEC_7_PL, "Wy≥±cz emulacjÍ urz±dzenia IEC dla urz±dzenia #7"},
	/* ru */ {IDCLS_DISABLE_IEC_7_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_IEC_7_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_IEC_7_TR, ""},  /* fuzzy */
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_ENABLE_IEC_8,    N_("Enable IEC device emulation for device #8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEC_8_DA, "AktivÈr IEC-enhedsemulering for enhed #8"},
	/* de */ {IDCLS_ENABLE_IEC_8_DE, "IEC Ger‰te Emulation f¸r Ger‰t #8 aktivieren"},
	/* es */ {IDCLS_ENABLE_IEC_8_ES, "Permitir emulaciÛn IEC para perifÈrico #8"},
	/* fr */ {IDCLS_ENABLE_IEC_8_FR, "Activer l'Èmulation IEC pour le lecteur #8"},
	/* hu */ {IDCLS_ENABLE_IEC_8_HU, "IEC eszkˆz #8 emul·ciÛj·nak engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_IEC_8_IT, "Attiva emulazione IEC per la periferica #8"},
	/* ko */ {IDCLS_ENABLE_IEC_8_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEC_8_NL, "IEC apparaat emulatie voor apparaat #8 inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEC_8_PL, "W≥±cz emulacjÍ IEC urz±dzenia #8"},
	/* ru */ {IDCLS_ENABLE_IEC_8_RU, "Enable IEC device emulation for device #8"},
	/* sv */ {IDCLS_ENABLE_IEC_8_SV, "Aktivera IEC-enhetsemulering fˆr enhet 8"},
	/* tr */ {IDCLS_ENABLE_IEC_8_TR, "Ayg˝t #8 iÁin IEC ayg˝t em¸lasyonunu aktif et"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_DISABLE_IEC_8,    N_("Disable IEC device emulation for device #8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEC_8_DA, "DeaktivÈr IEC-enhedsemulering for enhed #8"},
	/* de */ {IDCLS_DISABLE_IEC_8_DE, "IEC Ger‰te Emulation f¸r Ger‰t #8 deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEC_8_ES, "Deshabilitar emulaciÛn IEC para perifÈrico #8"},
	/* fr */ {IDCLS_DISABLE_IEC_8_FR, "DÈsactiver l'Èmulation IEC pour le lecteur #8"},
	/* hu */ {IDCLS_DISABLE_IEC_8_HU, "IEC eszkˆz #8 emul·ciÛj·nak tilt·sa"},
	/* it */ {IDCLS_DISABLE_IEC_8_IT, "Disattiva emulazione IEC per la periferica #8"},
	/* ko */ {IDCLS_DISABLE_IEC_8_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEC_8_NL, "IEC apparaat emulatie voor apparaat #8 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEC_8_PL, "Wy≥±cz emulacjÍ urz±dzenia IEC dla urz±dzenia #8"},
	/* ru */ {IDCLS_DISABLE_IEC_8_RU, "Disable IEC device emulation for device #8"},
	/* sv */ {IDCLS_DISABLE_IEC_8_SV, "Inaktivera IEC-enhetsemulering fˆr enhet 8"},
	/* tr */ {IDCLS_DISABLE_IEC_8_TR, "Ayg˝t #8 iÁin IEC ayg˝t em¸lasyonunu pasifle˛tir"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_ENABLE_IEC_9,    N_("Enable IEC device emulation for device #9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEC_9_DA, "AktivÈr IEC-enhedsemulering for enhed #9"},
	/* de */ {IDCLS_ENABLE_IEC_9_DE, "IEC Ger‰te Emulation f¸r Ger‰t #9 aktivieren"},
	/* es */ {IDCLS_ENABLE_IEC_9_ES, "Permitir emulaciÛn IEC para perifÈrico #9"},
	/* fr */ {IDCLS_ENABLE_IEC_9_FR, "Activer l'Èmulation IEC pour le lecteur #9"},
	/* hu */ {IDCLS_ENABLE_IEC_9_HU, "IEC eszkˆz #9 emul·ciÛj·nak engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_IEC_9_IT, "Attiva emulazione IEC per la periferica #9"},
	/* ko */ {IDCLS_ENABLE_IEC_9_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEC_9_NL, "IEC apparaat emulatie voor apparaat #9 inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEC_9_PL, "W≥±cz emulacjÍ IEC urz±dzenia #9"},
	/* ru */ {IDCLS_ENABLE_IEC_9_RU, "Enable IEC device emulation for device #9"},
	/* sv */ {IDCLS_ENABLE_IEC_9_SV, "Aktivera IEC-enhetsemulering fˆr enhet 9"},
	/* tr */ {IDCLS_ENABLE_IEC_9_TR, "Ayg˝t #9 iÁin IEC ayg˝t em¸lasyonunu aktif et"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_DISABLE_IEC_9,    N_("Disable IEC device emulation for device #9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEC_9_DA, "DeaktivÈr IEC-enhedsemulering for enhed #9"},
	/* de */ {IDCLS_DISABLE_IEC_9_DE, "IEC Ger‰te Emulation f¸r Ger‰t #9 deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEC_9_ES, "Deshabilitar emulaciÛn IEC para perifÈrico #9"},
	/* fr */ {IDCLS_DISABLE_IEC_9_FR, "DÈsactiver l'Èmulation IEC pour le lecteur #9"},
	/* hu */ {IDCLS_DISABLE_IEC_9_HU, "IEC eszkˆz #9 emul·ciÛj·nak tilt·sa"},
	/* it */ {IDCLS_DISABLE_IEC_9_IT, "Disattiva emulazione IEC per la periferica #9"},
	/* ko */ {IDCLS_DISABLE_IEC_9_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEC_9_NL, "IEC apparaat emulatie voor apparaat #9 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEC_9_PL, "Wy≥±cz emulacjÍ urz±dzenia IEC dla urz±dzenia #9"},
	/* ru */ {IDCLS_DISABLE_IEC_9_RU, "Disable IEC device emulation for device #9"},
	/* sv */ {IDCLS_DISABLE_IEC_9_SV, "Inaktivera IEC-enhetsemulering fˆr enhet 9"},
	/* tr */ {IDCLS_DISABLE_IEC_9_TR, "Ayg˝t #9 iÁin IEC ayg˝t em¸lasyonunu pasifle˛tir"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_ENABLE_IEC_10,    N_("Enable IEC device emulation for device #10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEC_10_DA, "AktivÈr IEC-enhedsemulering for enhed #10"},
	/* de */ {IDCLS_ENABLE_IEC_10_DE, "IEC Ger‰te Emulation f¸r Ger‰t #10 aktivieren"},
	/* es */ {IDCLS_ENABLE_IEC_10_ES, "Permitir emulaciÛn IEC para perifÈrico #10"},
	/* fr */ {IDCLS_ENABLE_IEC_10_FR, "Activer l'Èmulation IEC pour le lecteur #10"},
	/* hu */ {IDCLS_ENABLE_IEC_10_HU, "IEC eszkˆz #10 emul·ciÛj·nak engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_IEC_10_IT, "Attiva emulazione IEC per la periferica #10"},
	/* ko */ {IDCLS_ENABLE_IEC_10_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEC_10_NL, "IEC apparaat emulatie voor apparaat #10 inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEC_10_PL, "W≥±cz emulacjÍ IEC urz±dzenia #10"},
	/* ru */ {IDCLS_ENABLE_IEC_10_RU, "Enable IEC device emulation for device #10"},
	/* sv */ {IDCLS_ENABLE_IEC_10_SV, "Aktivera IEC-enhetsemulering fˆr enhet 10"},
	/* tr */ {IDCLS_ENABLE_IEC_10_TR, "Ayg˝t #10 iÁin IEC ayg˝t em¸lasyonunu aktif et"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_DISABLE_IEC_10,    N_("Disable IEC device emulation for device #10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEC_10_DA, "DeaktivÈr IEC-enhedsemulering for enhed #10"},
	/* de */ {IDCLS_DISABLE_IEC_10_DE, "IEC Ger‰te Emulation f¸r Ger‰t #10 deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEC_10_ES, "Deshabilitar emulaciÛn IEC para perifÈrico #10"},
	/* fr */ {IDCLS_DISABLE_IEC_10_FR, "DÈsactiver l'Èmulation IEC pour le lecteur #10"},
	/* hu */ {IDCLS_DISABLE_IEC_10_HU, "IEC eszkˆz #10 emul·ciÛj·nak tilt·sa"},
	/* it */ {IDCLS_DISABLE_IEC_10_IT, "Disattiva emulazione IEC per la periferica #10"},
	/* ko */ {IDCLS_DISABLE_IEC_10_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEC_10_NL, "IEC apparaat emulatie voor apparaat #10 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEC_10_PL, "Wy≥±cz emulacjÍ urz±dzenia IEC dla urz±dzenia #10"},
	/* ru */ {IDCLS_DISABLE_IEC_10_RU, "Disable IEC device emulation for device #10"},
	/* sv */ {IDCLS_DISABLE_IEC_10_SV, "Inaktivera IEC-enhetsemulering fˆr enhet 10"},
	/* tr */ {IDCLS_DISABLE_IEC_10_TR, "Ayg˝t #10 iÁin IEC ayg˝t em¸lasyonunu pasifle˛tir"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_ENABLE_IEC_11,    N_("Enable IEC device emulation for device #11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IEC_11_DA, "AktivÈr IEC-enhedsemulering for enhed #11"},
	/* de */ {IDCLS_ENABLE_IEC_11_DE, "IEC Ger‰te Emulation f¸r Ger‰t #11 aktivieren"},
	/* es */ {IDCLS_ENABLE_IEC_11_ES, "Permitir emulaciÛn IEC para perifÈrico #11"},
	/* fr */ {IDCLS_ENABLE_IEC_11_FR, "Activer l'Èmulation IEC pour le lecteur #11"},
	/* hu */ {IDCLS_ENABLE_IEC_11_HU, "IEC eszkˆz #11 emul·ciÛj·nak engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_IEC_11_IT, "Attiva emulazione IEC per la periferica #11"},
	/* ko */ {IDCLS_ENABLE_IEC_11_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IEC_11_NL, "IEC apparaat emulatie voor apparaat #11 inschakelen"},
	/* pl */ {IDCLS_ENABLE_IEC_11_PL, "W≥±cz emulacjÍ IEC urz±dzenia #11"},
	/* ru */ {IDCLS_ENABLE_IEC_11_RU, "Enable IEC device emulation for device #11"},
	/* sv */ {IDCLS_ENABLE_IEC_11_SV, "Aktivera IEC-enhetsemulering fˆr enhet 11"},
	/* tr */ {IDCLS_ENABLE_IEC_11_TR, "Ayg˝t #11 iÁin IEC ayg˝t em¸lasyonunu aktif et"},
#endif
	
	/* serial/serial-iec-device.c */
	/* en */ {IDCLS_DISABLE_IEC_11,    N_("Disable IEC device emulation for device #11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IEC_11_DA, "DeaktivÈr IEC-enhedsemulering for enhed #11"},
	/* de */ {IDCLS_DISABLE_IEC_11_DE, "IEC Ger‰te Emulation f¸r Ger‰t #11 deaktivieren"},
	/* es */ {IDCLS_DISABLE_IEC_11_ES, "Deshabilitar emulaciÛn IEC para perifÈrico #11"},
	/* fr */ {IDCLS_DISABLE_IEC_11_FR, "DÈsactiver l'Èmulation IEC pour le lecteur #11"},
	/* hu */ {IDCLS_DISABLE_IEC_11_HU, "IEC eszkˆz #11 emul·ciÛj·nak tilt·sa"},
	/* it */ {IDCLS_DISABLE_IEC_11_IT, "Disattiva emulazione IEC per la periferica #11"},
	/* ko */ {IDCLS_DISABLE_IEC_11_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IEC_11_NL, "IEC apparaat emulatie voor apparaat #11 uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IEC_11_PL, "Wy≥±cz emulacjÍ urz±dzenia IEC dla urz±dzenia #11"},
	/* ru */ {IDCLS_DISABLE_IEC_11_RU, "Disable IEC device emulation for device #11"},
	/* sv */ {IDCLS_DISABLE_IEC_11_SV, "Inaktivera IEC-enhetsemulering fˆr enhet 11"},
	/* tr */ {IDCLS_DISABLE_IEC_11_TR, "Ayg˝t #11 iÁin IEC ayg˝t em¸lasyonunu pasifle˛tir"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_P_ENGINE_MODEL,    N_("<engine and model>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_ENGINE_MODEL_DA, "<motor og model>"},
	/* de */ {IDCLS_P_ENGINE_MODEL_DE, "<Engine und Modell>"},
	/* es */ {IDCLS_P_ENGINE_MODEL_ES, "<motor y modelo>"},
	/* fr */ {IDCLS_P_ENGINE_MODEL_FR, "<engin et modËle>"},
	/* hu */ {IDCLS_P_ENGINE_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_ENGINE_MODEL_IT, "<motore e modello>"},
	/* ko */ {IDCLS_P_ENGINE_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_ENGINE_MODEL_NL, "<kern en model>"},
	/* pl */ {IDCLS_P_ENGINE_MODEL_PL, "<silnik i model>"},
	/* ru */ {IDCLS_P_ENGINE_MODEL_RU, "<engine and model>"},
	/* sv */ {IDCLS_P_ENGINE_MODEL_SV, "<motor och modell>"},
	/* tr */ {IDCLS_P_ENGINE_MODEL_TR, "<motor ve model>"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_P_AMOUNT,    N_("<amount>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_AMOUNT_DA, "<antal>"},
	/* de */ {IDCLS_P_AMOUNT_DE, "<Anzahl>"},
	/* es */ {IDCLS_P_AMOUNT_ES, "<cantidad>"},
	/* fr */ {IDCLS_P_AMOUNT_FR, "<quantitÈ>"},
	/* hu */ {IDCLS_P_AMOUNT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_AMOUNT_IT, "<numero>"},
	/* ko */ {IDCLS_P_AMOUNT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_AMOUNT_NL, "<hoeveelheid>"},
	/* pl */ {IDCLS_P_AMOUNT_PL, "<ilo∂Ê>"},
	/* ru */ {IDCLS_P_AMOUNT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_AMOUNT_SV, "<antal>"},
	/* tr */ {IDCLS_P_AMOUNT_TR, "<adet>"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_AMOUNT_EXTRA_SIDS,    N_("amount of extra SID chips. (0..2)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_AMOUNT_EXTRA_SIDS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_AMOUNT_EXTRA_SIDS_DE, "Anzahl der extra SID Chips. (0..2)"},
	/* es */ {IDCLS_AMOUNT_EXTRA_SIDS_ES, "Cantidad de chips SID extra (0..2)"},
	/* fr */ {IDCLS_AMOUNT_EXTRA_SIDS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_AMOUNT_EXTRA_SIDS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_AMOUNT_EXTRA_SIDS_IT, "numero di SID chip aggiuntivi. (0..2)"},
	/* ko */ {IDCLS_AMOUNT_EXTRA_SIDS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_AMOUNT_EXTRA_SIDS_NL, "Aantal extra SID chips. (0..2)"},
	/* pl */ {IDCLS_AMOUNT_EXTRA_SIDS_PL, "liczba dodatkowych czipÛw SID (0..2)"},
	/* ru */ {IDCLS_AMOUNT_EXTRA_SIDS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_AMOUNT_EXTRA_SIDS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_AMOUNT_EXTRA_SIDS_TR, ""},  /* fuzzy */
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_FASTSID_ENGINE_MODEL,    N_("0: FastSID 6581, 1: FastSID 8580")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_FASTSID_ENGINE_MODEL_DA, "0: FastSID 6581, 1: FastSID 8580"},
	/* de */ {IDCLS_FASTSID_ENGINE_MODEL_DE, "0: FastSID 6581, 1: FastSID 8580"},
	/* es */ {IDCLS_FASTSID_ENGINE_MODEL_ES, "0: FastSID 6581, 1: FastSID 8580"},
	/* fr */ {IDCLS_FASTSID_ENGINE_MODEL_FR, "0: FastSID 6581, 1: FastSID 8580"},
	/* hu */ {IDCLS_FASTSID_ENGINE_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_FASTSID_ENGINE_MODEL_IT, "0: FastSID 6581, 1: FastSID 8580"},
	/* ko */ {IDCLS_FASTSID_ENGINE_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_FASTSID_ENGINE_MODEL_NL, "0: FastSID 6581, 1: FastSID 8580"},
	/* pl */ {IDCLS_FASTSID_ENGINE_MODEL_PL, "0: FastSID 6581, 1: FastSID 8580"},
	/* ru */ {IDCLS_FASTSID_ENGINE_MODEL_RU, "0: FastSID 6581, 1: FastSID 8580"},
	/* sv */ {IDCLS_FASTSID_ENGINE_MODEL_SV, "0: FastSID 6581, 1: FastSID 8580"},
	/* tr */ {IDCLS_FASTSID_ENGINE_MODEL_TR, "0: FastSID 6581, 1: FastSID 8580"},
#endif
	
#ifdef HAVE_RESID
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_RESID_ENGINE_MODEL,    N_("256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RESID_ENGINE_MODEL_DA, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digitalforstÊrkning"},
	/* de */ {IDCLS_RESID_ENGINE_MODEL_DE, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost"},
	/* es */ {IDCLS_RESID_ENGINE_MODEL_ES, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost"},
	/* fr */ {IDCLS_RESID_ENGINE_MODEL_FR, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost"},
	/* hu */ {IDCLS_RESID_ENGINE_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RESID_ENGINE_MODEL_IT, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost"},
	/* ko */ {IDCLS_RESID_ENGINE_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RESID_ENGINE_MODEL_NL, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost"},
	/* pl */ {IDCLS_RESID_ENGINE_MODEL_PL, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost"},
	/* ru */ {IDCLS_RESID_ENGINE_MODEL_RU, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost"},
	/* sv */ {IDCLS_RESID_ENGINE_MODEL_SV, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digifˆrst‰rkning"},
	/* tr */ {IDCLS_RESID_ENGINE_MODEL_TR, "256: ReSID 6581, 257: ReSID 8580, 258: ReSID 8580 + digiboost"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_RESIDDTV_ENGINE_MODEL,    N_("260: DTVSID")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RESIDDTV_ENGINE_MODEL_DA, "260: DTVSID"},
	/* de */ {IDCLS_RESIDDTV_ENGINE_MODEL_DE, "260: DTVSID"},
	/* es */ {IDCLS_RESIDDTV_ENGINE_MODEL_ES, "260: DTVSID"},
	/* fr */ {IDCLS_RESIDDTV_ENGINE_MODEL_FR, "260: DTVSID"},
	/* hu */ {IDCLS_RESIDDTV_ENGINE_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RESIDDTV_ENGINE_MODEL_IT, "260: DTVSID"},
	/* ko */ {IDCLS_RESIDDTV_ENGINE_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RESIDDTV_ENGINE_MODEL_NL, "260: DTVSID"},
	/* pl */ {IDCLS_RESIDDTV_ENGINE_MODEL_PL, "260: DTVSID"},
	/* ru */ {IDCLS_RESIDDTV_ENGINE_MODEL_RU, "260: DTVSID"},
	/* sv */ {IDCLS_RESIDDTV_ENGINE_MODEL_SV, "260: DTVSID"},
	/* tr */ {IDCLS_RESIDDTV_ENGINE_MODEL_TR, "260: DTVSID"},
#endif
#endif
	
#ifdef HAVE_CATWEASELMKIII
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL,    N_("512: Catweasel")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_DA, "512: Catweasel"},
	/* de */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_DE, "512: Catweasel"},
	/* es */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_ES, "512: Catweasel"},
	/* fr */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_FR, "512: Catweasel"},
	/* hu */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_IT, "512: Catweasel"},
	/* ko */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_NL, "512: Catweasel"},
	/* pl */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_PL, "512: Catweasel"},
	/* ru */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_RU, "512: Catweasel"},
	/* sv */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_SV, "512: Catweasel"},
	/* tr */ {IDCLS_CATWEASELMKIII_ENGINE_MODEL_TR, "512: Catweasel"},
#endif
#endif
	
#ifdef HAVE_HARDSID
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_HARDSID_ENGINE_MODEL,    N_("768: HardSID")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_HARDSID_ENGINE_MODEL_DA, "768: HardSID"},
	/* de */ {IDCLS_HARDSID_ENGINE_MODEL_DE, "768: HardSID"},
	/* es */ {IDCLS_HARDSID_ENGINE_MODEL_ES, "768: HardSID"},
	/* fr */ {IDCLS_HARDSID_ENGINE_MODEL_FR, "768: HardSID"},
	/* hu */ {IDCLS_HARDSID_ENGINE_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_HARDSID_ENGINE_MODEL_IT, "768: HardSID"},
	/* ko */ {IDCLS_HARDSID_ENGINE_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_HARDSID_ENGINE_MODEL_NL, "768: HardSID"},
	/* pl */ {IDCLS_HARDSID_ENGINE_MODEL_PL, "768: HardSID"},
	/* ru */ {IDCLS_HARDSID_ENGINE_MODEL_RU, "768: HardSID"},
	/* sv */ {IDCLS_HARDSID_ENGINE_MODEL_SV, "768: HardSID"},
	/* tr */ {IDCLS_HARDSID_ENGINE_MODEL_TR, "768: HardSID"},
#endif
#endif
	
#ifdef HAVE_PARSID
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_PARSID_ENGINE_MODEL,    N_("1024: ParSID in par port 1, 1280: ParSID in par port 2, 1536: ParSID in par port 3")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PARSID_ENGINE_MODEL_DA, "1024: ParSID pÂ port 1, 1280: ParSID pÂ port 2, 1536: ParSID pÂ port 3"},
	/* de */ {IDCLS_PARSID_ENGINE_MODEL_DE, "1024: ParSID in par port 1, 1280: ParSID in par port 2, 1536: ParSID in par port 3"},
	/* es */ {IDCLS_PARSID_ENGINE_MODEL_ES, "1024: ParSID in par port 1, 1280: ParSID in par port 2, 1536: ParSID in par port 3"},
	/* fr */ {IDCLS_PARSID_ENGINE_MODEL_FR, "1024: ParSID sur port.par 1, 1280: ParSID sur port.par 2, 1536: ParSID sur port.par 3"},
	/* hu */ {IDCLS_PARSID_ENGINE_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PARSID_ENGINE_MODEL_IT, "1024: ParSID in par port 1, 1280: ParSID in par port 2, 1536: ParSID in par port 3"},
	/* ko */ {IDCLS_PARSID_ENGINE_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PARSID_ENGINE_MODEL_NL, "1024: ParSID in par port 1, 1280: ParSID in par port 2, 1536: ParSID in par port 3"},
	/* pl */ {IDCLS_PARSID_ENGINE_MODEL_PL, "1024: ParSID in porcie par 1, 1280: ParSID w porcie par 2, 1536: ParSID in porcie par 3"},
	/* ru */ {IDCLS_PARSID_ENGINE_MODEL_RU, "1024: ParSID in par port 1, 1280: ParSID in par port 2, 1536: ParSID in par port 3"},
	/* sv */ {IDCLS_PARSID_ENGINE_MODEL_SV, "1024: ParSID i parport 1, 1280: ParSID i parport 2, 1536: ParSID i parport 3"},
	/* tr */ {IDCLS_PARSID_ENGINE_MODEL_TR, "1024: Par port 1'de ParSID, 1280: Par port 2'de ParSID, 1536: Par port 3'de ParSID"},
#endif
#endif
	
#ifdef HAVE_SSI2001
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_SSI2001_ENGINE_MODEL,    N_("1792: SSI2001")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SSI2001_ENGINE_MODEL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SSI2001_ENGINE_MODEL_DE, "1792: SSI2001"},
	/* es */ {IDCLS_SSI2001_ENGINE_MODEL_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SSI2001_ENGINE_MODEL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SSI2001_ENGINE_MODEL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SSI2001_ENGINE_MODEL_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SSI2001_ENGINE_MODEL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SSI2001_ENGINE_MODEL_NL, "1792: SSI2001"},
	/* pl */ {IDCLS_SSI2001_ENGINE_MODEL_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SSI2001_ENGINE_MODEL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SSI2001_ENGINE_MODEL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SSI2001_ENGINE_MODEL_TR, ""},  /* fuzzy */
#endif
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_SIDCART,    N_("Enable the SID cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SIDCART_DA, "AktivÈr SID-cartridget"},
	/* de */ {IDCLS_ENABLE_SIDCART_DE, "SID Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_SIDCART_ES, "Permitir cartucho SID"},
	/* fr */ {IDCLS_ENABLE_SIDCART_FR, "Activer la cartouche SID"},
	/* hu */ {IDCLS_ENABLE_SIDCART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SIDCART_IT, "Attiva cartuccia SID"},
	/* ko */ {IDCLS_ENABLE_SIDCART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SIDCART_NL, "SID cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_SIDCART_PL, "W≥±cz kartridø SID"},
	/* ru */ {IDCLS_ENABLE_SIDCART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SIDCART_SV, "Aktivera SID-insticksmodul"},
	/* tr */ {IDCLS_ENABLE_SIDCART_TR, "SID kartu˛unu aktif et"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_SIDCART,    N_("Disable the SID cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SIDCART_DA, "DeaktivÈr SID-cartridge"},
	/* de */ {IDCLS_DISABLE_SIDCART_DE, "SID Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_SIDCART_ES, "Deshabilitar cartucho SID"},
	/* fr */ {IDCLS_DISABLE_SIDCART_FR, "DÈsactiver la cartouche SID"},
	/* hu */ {IDCLS_DISABLE_SIDCART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SIDCART_IT, "Disattiva la cartuccia SID"},
	/* ko */ {IDCLS_DISABLE_SIDCART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SIDCART_NL, "SID cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SIDCART_PL, "Wy≥±cz kartridø SID"},
	/* ru */ {IDCLS_DISABLE_SIDCART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SIDCART_SV, "Inaktivera SID-insticksmodul"},
	/* tr */ {IDCLS_DISABLE_SIDCART_TR, "SID kartu˛unu pasifle˛tir"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_PET_SIDCART_ADDRESS,    N_("SID cartridge address (0x8F00/0xE900)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET_SIDCART_ADDRESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PET_SIDCART_ADDRESS_DE, "SID Modul Adresse (0x8F00/0xE900)"},
	/* es */ {IDCLS_PET_SIDCART_ADDRESS_ES, "DirecciÛn del cartucho SID (0x8F00/0xE900)"},
	/* fr */ {IDCLS_PET_SIDCART_ADDRESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PET_SIDCART_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PET_SIDCART_ADDRESS_IT, "Indirizzo cartuccia SID (0x8F00/0xE900)"},
	/* ko */ {IDCLS_PET_SIDCART_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET_SIDCART_ADDRESS_NL, "SID cartridge adres (0x8F00/0xE900)"},
	/* pl */ {IDCLS_PET_SIDCART_ADDRESS_PL, "Adres kartridøa SID (0x8F00/0xE900)"},
	/* ru */ {IDCLS_PET_SIDCART_ADDRESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PET_SIDCART_ADDRESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PET_SIDCART_ADDRESS_TR, ""},  /* fuzzy */
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_PLUS4_SIDCART_ADDRESS,    N_("SID cartridge address (0xFD40/0xFE80)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PLUS4_SIDCART_ADDRESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PLUS4_SIDCART_ADDRESS_DE, "SID Modul Adresse (0xFD40/0xFE80)"},
	/* es */ {IDCLS_PLUS4_SIDCART_ADDRESS_ES, "DirecciÛn del cartucho SID (0xFD40/0xFE80)"},
	/* fr */ {IDCLS_PLUS4_SIDCART_ADDRESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PLUS4_SIDCART_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PLUS4_SIDCART_ADDRESS_IT, "Indirizzo cartuccia SID (0xFD40/0xFE80)"},
	/* ko */ {IDCLS_PLUS4_SIDCART_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PLUS4_SIDCART_ADDRESS_NL, "SID cartridge adres (0xFD40/0xFE80)"},
	/* pl */ {IDCLS_PLUS4_SIDCART_ADDRESS_PL, "Adres kartridøa SID (0xFD40/0xFE80)"},
	/* ru */ {IDCLS_PLUS4_SIDCART_ADDRESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PLUS4_SIDCART_ADDRESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PLUS4_SIDCART_ADDRESS_TR, ""},  /* fuzzy */
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_VIC20_SIDCART_ADDRESS,    N_("SID cartridge address (0x9800/0x9C00)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_VIC20_SIDCART_ADDRESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_VIC20_SIDCART_ADDRESS_DE, "SID Module Adresse (0x9800/0x9C00)"},
	/* es */ {IDCLS_VIC20_SIDCART_ADDRESS_ES, "DirecciÛn del cartucho SID (0x9800/0x9C00)"},
	/* fr */ {IDCLS_VIC20_SIDCART_ADDRESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_VIC20_SIDCART_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_VIC20_SIDCART_ADDRESS_IT, "Indirizzo cartuccia SID (0x9800/0x9C00)"},
	/* ko */ {IDCLS_VIC20_SIDCART_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_VIC20_SIDCART_ADDRESS_NL, "SID cartridge adres (0x9800/0x9C00)"},
	/* pl */ {IDCLS_VIC20_SIDCART_ADDRESS_PL, "Adres kartridøa SID (0x9800/0x9C00)"},
	/* ru */ {IDCLS_VIC20_SIDCART_ADDRESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_VIC20_SIDCART_ADDRESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_VIC20_SIDCART_ADDRESS_TR, ""},  /* fuzzy */
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_PET_SIDCART_CLOCK,    N_("SID cartridge clock (0: C64 clock, 1: PET clock)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PET_SIDCART_CLOCK_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PET_SIDCART_CLOCK_DE, "SID Module Takt (0: C64 Takt, 1: PET Takt)"},
	/* es */ {IDCLS_PET_SIDCART_CLOCK_ES, "Reloj del cartucho SID (0: Reloj C64, 1: Reloj PET)"},
	/* fr */ {IDCLS_PET_SIDCART_CLOCK_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PET_SIDCART_CLOCK_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PET_SIDCART_CLOCK_IT, "Clock cartuccia SID (0: clock C64 , 1: clock PET)"},
	/* ko */ {IDCLS_PET_SIDCART_CLOCK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PET_SIDCART_CLOCK_NL, "SID cartridge klok (0: C64 klok, 1: PET klok)"},
	/* pl */ {IDCLS_PET_SIDCART_CLOCK_PL, "Zegar kartridøa SID (0: Zegar C64, 1: Zegar PET)"},
	/* ru */ {IDCLS_PET_SIDCART_CLOCK_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PET_SIDCART_CLOCK_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PET_SIDCART_CLOCK_TR, ""},  /* fuzzy */
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_PLUS4_SIDCART_CLOCK,    N_("SID cartridge clock (0: C64 clock, 1: PLUS4 clock)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PLUS4_SIDCART_CLOCK_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PLUS4_SIDCART_CLOCK_DE, "SID Modul Takt (0: C64 Takt, 1: PLUS4 Takt)"},
	/* es */ {IDCLS_PLUS4_SIDCART_CLOCK_ES, "Reloj del cartucho SID (0: Reloj C64, 1: Reloj PLUS4)"},
	/* fr */ {IDCLS_PLUS4_SIDCART_CLOCK_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PLUS4_SIDCART_CLOCK_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PLUS4_SIDCART_CLOCK_IT, "Clock cartuccia SID (0: clock C64 , 1: clock PLUS4)"},
	/* ko */ {IDCLS_PLUS4_SIDCART_CLOCK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PLUS4_SIDCART_CLOCK_NL, "SID cartridge klok (0: C64 klok, 1: PLUS4 klok)"},
	/* pl */ {IDCLS_PLUS4_SIDCART_CLOCK_PL, "Zegar kartridøa SID (0: Zegar C64, 1: Zegar PLUS4)"},
	/* ru */ {IDCLS_PLUS4_SIDCART_CLOCK_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PLUS4_SIDCART_CLOCK_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PLUS4_SIDCART_CLOCK_TR, ""},  /* fuzzy */
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_VIC20_SIDCART_CLOCK,    N_("SID cartridge clock (0: C64 clock, 1: VIC20 clock)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_VIC20_SIDCART_CLOCK_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_VIC20_SIDCART_CLOCK_DE, "SID Modul Takt (0: C64 Takt, 1: VIC20 Takt)"},
	/* es */ {IDCLS_VIC20_SIDCART_CLOCK_ES, "Reloj del cartucho SID (0: Reloj C64, 1: Reloj VIC20)"},
	/* fr */ {IDCLS_VIC20_SIDCART_CLOCK_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_VIC20_SIDCART_CLOCK_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_VIC20_SIDCART_CLOCK_IT, "Clock cartuccia SID (0: clock C64 , 1: clock VIC20)"},
	/* ko */ {IDCLS_VIC20_SIDCART_CLOCK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_VIC20_SIDCART_CLOCK_NL, "SID cartridge klok (0: C64 klok, 1: VIC20 klok)"},
	/* pl */ {IDCLS_VIC20_SIDCART_CLOCK_PL, "Zegar kartridøa SID (0: Zegar C64, 1: Zegar VIC20)"},
	/* ru */ {IDCLS_VIC20_SIDCART_CLOCK_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_VIC20_SIDCART_CLOCK_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_VIC20_SIDCART_CLOCK_TR, ""},  /* fuzzy */
#endif
	
	/* sid/sid-cmdline-options.c, c64/plus60k, c64/c64_256k.c */
	/* en */ {IDCLS_P_BASE_ADDRESS,    N_("<Base address>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_BASE_ADDRESS_DA, "<Baseadresse>"},
	/* de */ {IDCLS_P_BASE_ADDRESS_DE, "<Basis Adresse>"},
	/* es */ {IDCLS_P_BASE_ADDRESS_ES, "<DirecciÛn base>"},
	/* fr */ {IDCLS_P_BASE_ADDRESS_FR, "<Adresse de base>"},
	/* hu */ {IDCLS_P_BASE_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_BASE_ADDRESS_IT, "<Indirizzo base>"},
	/* ko */ {IDCLS_P_BASE_ADDRESS_KO, "<∫£¿ÃΩ∫ ¡÷º“>"},
	/* nl */ {IDCLS_P_BASE_ADDRESS_NL, "<Basisadres>"},
	/* pl */ {IDCLS_P_BASE_ADDRESS_PL, "<Adres bazowy>"},
	/* ru */ {IDCLS_P_BASE_ADDRESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_BASE_ADDRESS_SV, "<Basadress>"},
	/* tr */ {IDCLS_P_BASE_ADDRESS_TR, "<Taban adresi>"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SID_2_ADDRESS,    N_("Specify base address for 2nd SID")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SID_2_ADDRESS_DA, "Angiv baseadresse for sekundÊr SID"},
	/* de */ {IDCLS_SPECIFY_SID_2_ADDRESS_DE, "Basis Adresse f¸r zweiten SID definieren"},
	/* es */ {IDCLS_SPECIFY_SID_2_ADDRESS_ES, "Especificar direcciÛn base para el segundo ISD"},
	/* fr */ {IDCLS_SPECIFY_SID_2_ADDRESS_FR, "SpÈcifier l'adresse de base pour le second SID"},
	/* hu */ {IDCLS_SPECIFY_SID_2_ADDRESS_HU, "Adja meg a b·ziscÌmÈt a m·sodik SID-nek"},
	/* it */ {IDCLS_SPECIFY_SID_2_ADDRESS_IT, "Specifica l'indirizzo di base per il secondo SID"},
	/* ko */ {IDCLS_SPECIFY_SID_2_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SID_2_ADDRESS_NL, "Geef het basisadres van de 2e SID"},
	/* pl */ {IDCLS_SPECIFY_SID_2_ADDRESS_PL, "Okre∂l adres bazowy drugiego SID-a"},
	/* ru */ {IDCLS_SPECIFY_SID_2_ADDRESS_RU, "Specify base address for 2nd SID"},
	/* sv */ {IDCLS_SPECIFY_SID_2_ADDRESS_SV, "Ange basadress fˆr andra SID"},
	/* tr */ {IDCLS_SPECIFY_SID_2_ADDRESS_TR, "2nci SID iÁin taban adresini belirt"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_SID_3_ADDRESS,    N_("Specify base address for 3rd SID")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SID_3_ADDRESS_DA, "Angiv baseadresse for tredie SID"},
	/* de */ {IDCLS_SPECIFY_SID_3_ADDRESS_DE, "Basis Adresse f¸r dritten SID definieren"},
	/* es */ {IDCLS_SPECIFY_SID_3_ADDRESS_ES, "Especificar direcciÛn base para el tercer SID"},
	/* fr */ {IDCLS_SPECIFY_SID_3_ADDRESS_FR, "SpÈcifier l'adresse de base pour le troisiËme SID"},
	/* hu */ {IDCLS_SPECIFY_SID_3_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_SID_3_ADDRESS_IT, "Specifica l'indirizzo di base per il terzo SID"},
	/* ko */ {IDCLS_SPECIFY_SID_3_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SID_3_ADDRESS_NL, "Geef het basisadres van de 3e SID"},
	/* pl */ {IDCLS_SPECIFY_SID_3_ADDRESS_PL, "Okre∂l adres bazowy trzeciego SID-a"},
	/* ru */ {IDCLS_SPECIFY_SID_3_ADDRESS_RU, "Specify base address for 3rd SID"},
	/* sv */ {IDCLS_SPECIFY_SID_3_ADDRESS_SV, "Ange basadress fˆr tredje SID"},
	/* tr */ {IDCLS_SPECIFY_SID_3_ADDRESS_TR, "3¸nc¸ SID iÁin taban adresini belirt"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_SID_FILTERS,    N_("Emulate SID filters")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SID_FILTERS_DA, "EmulÈr SID-filtre"},
	/* de */ {IDCLS_ENABLE_SID_FILTERS_DE, "SID Filter Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_SID_FILTERS_ES, "Emular filtros SID"},
	/* fr */ {IDCLS_ENABLE_SID_FILTERS_FR, "…muler les filtres SID"},
	/* hu */ {IDCLS_ENABLE_SID_FILTERS_HU, "SID sz˚rık emul·ciÛja"},
	/* it */ {IDCLS_ENABLE_SID_FILTERS_IT, "Emula i filtri del SID"},
	/* ko */ {IDCLS_ENABLE_SID_FILTERS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SID_FILTERS_NL, "Emuleer SID filters"},
	/* pl */ {IDCLS_ENABLE_SID_FILTERS_PL, "Emuluj filtry SID"},
	/* ru */ {IDCLS_ENABLE_SID_FILTERS_RU, "Emulate SID filters"},
	/* sv */ {IDCLS_ENABLE_SID_FILTERS_SV, "Emulera SID-filter"},
	/* tr */ {IDCLS_ENABLE_SID_FILTERS_TR, "SID filtrelerini em¸le et"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_SID_FILTERS,    N_("Do not emulate SID filters")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SID_FILTERS_DA, "EmulÈr ikke SID-filtre"},
	/* de */ {IDCLS_DISABLE_SID_FILTERS_DE, "SID Filter Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_SID_FILTERS_ES, "No emular filtros SID"},
	/* fr */ {IDCLS_DISABLE_SID_FILTERS_FR, "Ne pas Èmuler les filtres SID"},
	/* hu */ {IDCLS_DISABLE_SID_FILTERS_HU, "Nem emul·lja a SID sz˚rıket"},
	/* it */ {IDCLS_DISABLE_SID_FILTERS_IT, "Non emulare i filtri del SID"},
	/* ko */ {IDCLS_DISABLE_SID_FILTERS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SID_FILTERS_NL, "SID filters niet emuleren"},
	/* pl */ {IDCLS_DISABLE_SID_FILTERS_PL, "Nie emuluj filtrÛw SID"},
	/* ru */ {IDCLS_DISABLE_SID_FILTERS_RU, "Do not emulate SID filters"},
	/* sv */ {IDCLS_DISABLE_SID_FILTERS_SV, "Emulera inte SID-filter"},
	/* tr */ {IDCLS_DISABLE_SID_FILTERS_TR, "SID filtrelerini em¸le etme"},
#endif
	
#ifdef HAVE_RESID
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_RESID_SAMPLING_METHOD,    N_("reSID sampling method (0: fast, 1: interpolating, 2: resampling, 3: fast resampling)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RESID_SAMPLING_METHOD_DA, "reSID-samplingmetode (0: hurtig, 1: interpolerende, 2: omsampling, 3: hurtig omsampling)"},
	/* de */ {IDCLS_RESID_SAMPLING_METHOD_DE, "reSID Sample Methode (0: schnell, 1: interpolierend, 2: resampling, 3: schnelles resampling)"},
	/* es */ {IDCLS_RESID_SAMPLING_METHOD_ES, "reSID mÈtodo sampleado (0: r·pido, 1: interpolaciÛn, 2: resampelado, 3: resampleado r·pido)"},
	/* fr */ {IDCLS_RESID_SAMPLING_METHOD_FR, "MÈthode Èchantillonnage reSID (0: rapide, 1: interpolation, 2: rÈÈch., 3: rÈÈch. rapide)"},
	/* hu */ {IDCLS_RESID_SAMPLING_METHOD_HU, "reSID mintavÈtelezÈsi mÛd (0: gyors, 1: interpol·lÛ, 2: ˙jramintavÈtelezı, 3: gyors ˙jramintavÈtelezı)"},
	/* it */ {IDCLS_RESID_SAMPLING_METHOD_IT, "Metodo di campionamento del reSID (0: veloce, 1: interpolato, 2: ricampionato 3: ricampionamento veloce)"},
	/* ko */ {IDCLS_RESID_SAMPLING_METHOD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RESID_SAMPLING_METHOD_NL, "reSID sampling methode (0: snel, 1: interpoleren, 2: resampling, 3: snelle resampling)"},
	/* pl */ {IDCLS_RESID_SAMPLING_METHOD_PL, "Metoda samplowania reSID (0: szybkie, 1: z przeplotem, 2: resamplowanie, 3: szybkie resamplowanie)"},
	/* ru */ {IDCLS_RESID_SAMPLING_METHOD_RU, "reSID sampling method (0: fast, 1: interpolating, 2: resampling, 3: fast resampling)"},
	/* sv */ {IDCLS_RESID_SAMPLING_METHOD_SV, "reSID-samplingsmetod (0: snabb, 1: interpolerande, 2: omsampling, 3: snabb omsampling)"},
	/* tr */ {IDCLS_RESID_SAMPLING_METHOD_TR, "reSID ˆrnekleme metodu (0: h˝zl˝, 1: ara deer bulma, 2: yeniden ˆrnekleme, 3: h˝zl˝ yeniden ˆrnekleme)"},
#endif
	
	/* sid/sid-cmdline-options.c, vsync.c */
	/* en */ {IDCLS_P_PERCENT,    N_("<percent>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_PERCENT_DA, "<procent>"},
	/* de */ {IDCLS_P_PERCENT_DE, "<prozent>"},
	/* es */ {IDCLS_P_PERCENT_ES, "<porcentaje>"},
	/* fr */ {IDCLS_P_PERCENT_FR, "<pourcent>"},
	/* hu */ {IDCLS_P_PERCENT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_PERCENT_IT, "<percento>"},
	/* ko */ {IDCLS_P_PERCENT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_PERCENT_NL, "<procent>"},
	/* pl */ {IDCLS_P_PERCENT_PL, "<procent>"},
	/* ru */ {IDCLS_P_PERCENT_RU, "<percent>"},
	/* sv */ {IDCLS_P_PERCENT_SV, "<procent>"},
	/* tr */ {IDCLS_P_PERCENT_TR, "<y¸zde>"},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_PASSBAND_PERCENTAGE,    N_("reSID resampling passband in percentage of total bandwidth (0 - 90)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PASSBAND_PERCENTAGE_DA, "PasbÂnd for reSID-resampling i procent af total bÂndbredde (0 - 90)"},
	/* de */ {IDCLS_PASSBAND_PERCENTAGE_DE, "reSID Resampling Passband Prozentwert der gesamte Bandbreite (0 - 90)\n(0 - 90, niedrig ist schneller, hˆher ist besser)"},
	/* es */ {IDCLS_PASSBAND_PERCENTAGE_ES, "reSID resampleado pasabanda en porcentaje del total de ancho de banda (0 - 90)"},
	/* fr */ {IDCLS_PASSBAND_PERCENTAGE_FR, "Bande passante pour le resampling reSID en pourcentage de la bande totale (0 - 90)"},
	/* hu */ {IDCLS_PASSBAND_PERCENTAGE_HU, "reSID ˙jramintavÈtelezÈsi s·vnak a teljes s·vszÈlessÈgre vonatkoztatott ar·nya (0 - 90)"},
	/* it */ {IDCLS_PASSBAND_PERCENTAGE_IT, "Banda passante di ricampionamento del reSID in percentuale di quella totale (0 - 90)"},
	/* ko */ {IDCLS_PASSBAND_PERCENTAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PASSBAND_PERCENTAGE_NL, "reSID resampling passband in percentage van de totale bandbreedte (0 - 90)"},
	/* pl */ {IDCLS_PASSBAND_PERCENTAGE_PL, "Pasmo resamplowania reSID w procentach ca≥kowitej przepustowo∂ci (0 - 90)"},
	/* ru */ {IDCLS_PASSBAND_PERCENTAGE_RU, "reSID resampling passband in percentage of total bandwidth (0 - 90)"},
	/* sv */ {IDCLS_PASSBAND_PERCENTAGE_SV, "Passband fˆr reSID-resampling i procent av total bandbredd (0 - 90)"},
	/* tr */ {IDCLS_PASSBAND_PERCENTAGE_TR, "reSID yeniden ˆrnekleme passband'inin toplam bant geni˛liine y¸zdesel oran˝ (0 - 90)"},
#endif
	
	/* sid/sid-cmdline-options.c, vsync.c */
	/* en */ {IDCLS_RESID_FILTER_BIAS,    N_("reSID filter bias setting, which can be used to adjust DAC bias in millivolts.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RESID_FILTER_BIAS_DA, "reSID filter biasindstilling, som kan bruges til at justere DAC bias i millivolt."},
	/* de */ {IDCLS_RESID_FILTER_BIAS_DE, "reSID Filter Vorspannungseinstellung, welche f¸r die Anpassung der DAC\nVorspannung in Millivolt benutzt werden kann"},
	/* es */ {IDCLS_RESID_FILTER_BIAS_ES, "reSID ajuste filtro bias, que puede ser usado para ajustar DAC bias en milivoltios."},
	/* fr */ {IDCLS_RESID_FILTER_BIAS_FR, "ParamËtre de polarisation de filtre reSID, pouvant Ítre utilisÈ pour rÈgler la polarisation du DAC en millivolts."},
	/* hu */ {IDCLS_RESID_FILTER_BIAS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_RESID_FILTER_BIAS_IT, "Impostazione del bias del filtro del reSID, che puÚ essere usato per correggere il bias del DAC in millivolt."},
	/* ko */ {IDCLS_RESID_FILTER_BIAS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RESID_FILTER_BIAS_NL, "reSID filter voorkeur instelling, dit kan gebruikt worden om de DAC voorkeur in millivolts aan te passen."},
	/* pl */ {IDCLS_RESID_FILTER_BIAS_PL, "Ustawienie filtra reSID, uøywane do regulacji nastawienia DAC w miliwoltach"},
	/* ru */ {IDCLS_RESID_FILTER_BIAS_RU, "reSID filter bias setting, which can be used to adjust DAC bias in millivolts."},
	/* sv */ {IDCLS_RESID_FILTER_BIAS_SV, "Bias-inst‰llning fˆr reSID-filter, vilket kan anv‰ndas fˆr att justera D/A-bias i millivolt."},
	/* tr */ {IDCLS_RESID_FILTER_BIAS_TR, "Millivolt cinsinden DAC meyilini ayarlamak iÁin kullan˝labilen reSID filtresi meyil ayar˝."},
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_RESID_GAIN_PERCENTAGE,    N_("reSID gain in percent (90 - 100)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RESID_GAIN_PERCENTAGE_DA, "Gain for reSID i procent (90 - 100)"},
	/* de */ {IDCLS_RESID_GAIN_PERCENTAGE_DE, "reSID Gain in Prozent (90 - 100)"},
	/* es */ {IDCLS_RESID_GAIN_PERCENTAGE_ES, "reSID ganancia en porcentaje (90-100)"},
	/* fr */ {IDCLS_RESID_GAIN_PERCENTAGE_FR, "Gain reSID en pourcent (90 - 100)"},
	/* hu */ {IDCLS_RESID_GAIN_PERCENTAGE_HU, "reSID sz·zalÈkos erısÌtÈs (90 - 100)"},
	/* it */ {IDCLS_RESID_GAIN_PERCENTAGE_IT, "Guadagno del reSID in percentuale (90 - 100)"},
	/* ko */ {IDCLS_RESID_GAIN_PERCENTAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_RESID_GAIN_PERCENTAGE_NL, "reSID versterking procent (90 - 100)"},
	/* pl */ {IDCLS_RESID_GAIN_PERCENTAGE_PL, "Wzmocnienie reSID-a w procentach (90 - 100)"},
	/* ru */ {IDCLS_RESID_GAIN_PERCENTAGE_RU, "reSID gain in percent (90 - 100)"},
	/* sv */ {IDCLS_RESID_GAIN_PERCENTAGE_SV, "Gain fˆr reSID i procent (90 - 100)"},
	/* tr */ {IDCLS_RESID_GAIN_PERCENTAGE_TR, "reSID gain y¸zdesi (90 - 100)"},
#endif
#endif
	
#ifdef HAVE_HARDSID
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_HARDSID_MAIN,    N_("Set the HardSID device for the main SID output")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_HARDSID_MAIN_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_HARDSID_MAIN_DE, "HardSID Ger‰t f¸r SID Ausgabe (main)"},
	/* es */ {IDCLS_HARDSID_MAIN_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_HARDSID_MAIN_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_HARDSID_MAIN_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_HARDSID_MAIN_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_HARDSID_MAIN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_HARDSID_MAIN_NL, "Zet het HardSID apparaat voor de hoofd SID uitvoer"},
	/* pl */ {IDCLS_HARDSID_MAIN_PL, "Okre∂l urz±dzenie HardSID dla g≥Ûwnego wyj∂cia SID"},
	/* ru */ {IDCLS_HARDSID_MAIN_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_HARDSID_MAIN_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_HARDSID_MAIN_TR, ""},  /* fuzzy */
#endif
	
	/* sid/sid-cmdline-options.c */
	/* en */ {IDCLS_HARDSID_RIGHT,    N_("Set the HardSID device for the right SID output")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_HARDSID_RIGHT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_HARDSID_RIGHT_DE, "HardSID Ger‰t f¸r SID Ausgabe (rechts)"},
	/* es */ {IDCLS_HARDSID_RIGHT_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_HARDSID_RIGHT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_HARDSID_RIGHT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_HARDSID_RIGHT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_HARDSID_RIGHT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_HARDSID_RIGHT_NL, "Zet het HardSID apparaat voor de rechter SID uitvoer"},
	/* pl */ {IDCLS_HARDSID_RIGHT_PL, "Okre∂l urz±dzenie SID dla prawego wyj∂cia SID"},
	/* ru */ {IDCLS_HARDSID_RIGHT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_HARDSID_RIGHT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_HARDSID_RIGHT_TR, ""},  /* fuzzy */
#endif
#endif
	
	/* vdc/vdc-cmdline-options.c, crtc/crtc-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_STRETCH_VERTICAL,    N_("Enable additional vertical stretching.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_STRETCH_VERTICAL_DA, "AktivÈr yderligere vertikal udstrÊkning"},
	/* de */ {IDCLS_ENABLE_STRETCH_VERTICAL_DE, "Aktiviere zus‰tzliche Vertikalstreckung."},
	/* es */ {IDCLS_ENABLE_STRETCH_VERTICAL_ES, "Permitir extensiÛn vertical adicional"},
	/* fr */ {IDCLS_ENABLE_STRETCH_VERTICAL_FR, "Activer l'ajout d'Ètirement vertical."},
	/* hu */ {IDCLS_ENABLE_STRETCH_VERTICAL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_STRETCH_VERTICAL_IT, "Attiva allungamento verticale aggiuntivo."},
	/* ko */ {IDCLS_ENABLE_STRETCH_VERTICAL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_STRETCH_VERTICAL_NL, "Additionele verticale rekking inschakelen."},
	/* pl */ {IDCLS_ENABLE_STRETCH_VERTICAL_PL, "W≥±cz dodatkowe rozci±ganie w pionie"},
	/* ru */ {IDCLS_ENABLE_STRETCH_VERTICAL_RU, "Enable additional vertical stretching."},
	/* sv */ {IDCLS_ENABLE_STRETCH_VERTICAL_SV, "Aktivera ytterligare vertikal utdragning."},
	/* tr */ {IDCLS_ENABLE_STRETCH_VERTICAL_TR, "Ek dikey uzatmay˝ aktif et."},
#endif
	
	/* vdc/vdc-cmdline-options.c, crtc/crtc-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_STRETCH_VERTICAL,    N_("Disable additional vertical stretching.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_STRETCH_VERTICAL_DA, "DeaktivÈr yderligere vertikal udstrÊkning"},
	/* de */ {IDCLS_DISABLE_STRETCH_VERTICAL_DE, "Deaktiviere zus‰tzliche Vertikalstreckung."},
	/* es */ {IDCLS_DISABLE_STRETCH_VERTICAL_ES, "Inhabilitar extensiÛn vertical adicional"},
	/* fr */ {IDCLS_DISABLE_STRETCH_VERTICAL_FR, "DÈsactiver l'ajout d'Ètirement vertical."},
	/* hu */ {IDCLS_DISABLE_STRETCH_VERTICAL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_STRETCH_VERTICAL_IT, "Disattiva allungamento verticale aggiuntivo."},
	/* ko */ {IDCLS_DISABLE_STRETCH_VERTICAL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_STRETCH_VERTICAL_NL, "Additionele verticale rekking uitschakelen."},
	/* pl */ {IDCLS_DISABLE_STRETCH_VERTICAL_PL, "Wy≥±cz dodatkowe rozci±ganie w pionie"},
	/* ru */ {IDCLS_DISABLE_STRETCH_VERTICAL_RU, "Disable additional vertical stretching."},
	/* sv */ {IDCLS_DISABLE_STRETCH_VERTICAL_SV, "Inaktivera ytterligare vertikal utdragning."},
	/* tr */ {IDCLS_DISABLE_STRETCH_VERTICAL_TR, "Ek dikey uzatmay˝ pasifle˛tir."},
#endif
	
	/* vdc/vdc-cmdline-options.c */
	/* en */ {IDCLS_SET_VDC_MEMORY_16KB,    N_("Set the VDC memory size to 16KB")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_VDC_MEMORY_16KB_DA, "SÊt VDC-hukommelsesst¯rrelse til 16KB"},
	/* de */ {IDCLS_SET_VDC_MEMORY_16KB_DE, "VDC Speichgrˆﬂe auf 16KB setzen"},
	/* es */ {IDCLS_SET_VDC_MEMORY_16KB_ES, "Seleccionar tamaÒo memoria VDC en 16KB"},
	/* fr */ {IDCLS_SET_VDC_MEMORY_16KB_FR, "RÈgler la taille de la mÈmoire VDC ‡ 16KO"},
	/* hu */ {IDCLS_SET_VDC_MEMORY_16KB_HU, "VDC memÛria mÈret be·llÌt·sa 16KB-ra"},
	/* it */ {IDCLS_SET_VDC_MEMORY_16KB_IT, "Imposta la dimensione della memoria del VDC a 16KB"},
	/* ko */ {IDCLS_SET_VDC_MEMORY_16KB_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_VDC_MEMORY_16KB_NL, "Stel de VDC geheugengrootte in op 16KB"},
	/* pl */ {IDCLS_SET_VDC_MEMORY_16KB_PL, "Ustaw rozmiar pamiÍci VDC na 16 KB"},
	/* ru */ {IDCLS_SET_VDC_MEMORY_16KB_RU, "Set the VDC memory size to 16KB"},
	/* sv */ {IDCLS_SET_VDC_MEMORY_16KB_SV, "S‰tt VDC-minnesstorlek till 16KB"},
	/* tr */ {IDCLS_SET_VDC_MEMORY_16KB_TR, "VDC bellek boyutunu 16KB'a ayarla"},
#endif
	
	/* vdc/vdc-cmdline-options.c */
	/* en */ {IDCLS_SET_VDC_MEMORY_64KB,    N_("Set the VDC memory size to 64KB")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_VDC_MEMORY_64KB_DA, "SÊt VDC-hukommelsesst¯rrelse til 64KB"},
	/* de */ {IDCLS_SET_VDC_MEMORY_64KB_DE, "VDC Speichgrˆﬂe auf 64KB setzen"},
	/* es */ {IDCLS_SET_VDC_MEMORY_64KB_ES, "Seleccionar tamaÒo memoria VDC en 64KB"},
	/* fr */ {IDCLS_SET_VDC_MEMORY_64KB_FR, "RÈgler la taille de la mÈmoire VDC ‡ 64KO"},
	/* hu */ {IDCLS_SET_VDC_MEMORY_64KB_HU, "VDC memÛria mÈret be·llÌt·sa 64KB-ra"},
	/* it */ {IDCLS_SET_VDC_MEMORY_64KB_IT, "Imposta la dimensione della memoria del VDC a 64KB"},
	/* ko */ {IDCLS_SET_VDC_MEMORY_64KB_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_VDC_MEMORY_64KB_NL, "Stel de VDC geheugengrootte in op 64KB"},
	/* pl */ {IDCLS_SET_VDC_MEMORY_64KB_PL, "Ustaw rozmiar pamiÍci VDC na 64 KB"},
	/* ru */ {IDCLS_SET_VDC_MEMORY_64KB_RU, "Set the VDC memory size to 64KB"},
	/* sv */ {IDCLS_SET_VDC_MEMORY_64KB_SV, "S‰tt VDC-minnesstorlek till 64KB"},
	/* tr */ {IDCLS_SET_VDC_MEMORY_64KB_TR, "VDC bellek boyutunu 64KB'a ayarla"},
#endif
	
	/* vdc/vdc-cmdline-options.c */
	/* en */ {IDCLS_SET_VDC_REVISION,    N_("Set VDC revision (0..2)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_VDC_REVISION_DA, "Angiv VDC-revision (0..2)"},
	/* de */ {IDCLS_SET_VDC_REVISION_DE, "VDC Revision setzen (0..2)"},
	/* es */ {IDCLS_SET_VDC_REVISION_ES, "Poner revisiÛn VDC (0..2)"},
	/* fr */ {IDCLS_SET_VDC_REVISION_FR, "RÈgler la rÈvision VDC (0..2)"},
	/* hu */ {IDCLS_SET_VDC_REVISION_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_VDC_REVISION_IT, "Imposta la revisione del VDC (0..2)"},
	/* ko */ {IDCLS_SET_VDC_REVISION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_VDC_REVISION_NL, "Stel de VDC revisie in (0..2)"},
	/* pl */ {IDCLS_SET_VDC_REVISION_PL, "Okre∂l wersjÍ VDC (0..2)"},
	/* ru */ {IDCLS_SET_VDC_REVISION_RU, "Set VDC revision (0..2)"},
	/* sv */ {IDCLS_SET_VDC_REVISION_SV, "Ange VDC-revision (0..2)"},
	/* tr */ {IDCLS_SET_VDC_REVISION_TR, "VDC revizyonunu ayarla (0..2)"},
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_P_SPEC,    N_("<spec>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_SPEC_DA, "<spec>"},
	/* de */ {IDCLS_P_SPEC_DE, "<Spec>"},
	/* es */ {IDCLS_P_SPEC_ES, "<especificaciones>"},
	/* fr */ {IDCLS_P_SPEC_FR, "<spec>"},
	/* hu */ {IDCLS_P_SPEC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_SPEC_IT, "<spec>"},
	/* ko */ {IDCLS_P_SPEC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_SPEC_NL, "<spec>"},
	/* pl */ {IDCLS_P_SPEC_PL, "<spec>"},
	/* ru */ {IDCLS_P_SPEC_RU, "<spec>"},
	/* sv */ {IDCLS_P_SPEC_SV, "<spec>"},
	/* tr */ {IDCLS_P_SPEC_TR, "<˛art>"},
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_MEMORY_CONFIG,    N_("Specify memory configuration. (none/3k/8k/16k/24k/all) / (0/1/2/3/5) / (04/20/40/60/a0)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_MEMORY_CONFIG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_MEMORY_CONFIG_DE, "Speicher Konfiguration w‰hlen. (none/3k/8k/16k/24k/all) / (0/1/2/3/5) / (04/20/40/60/a0)"},
	/* es */ {IDCLS_SPECIFY_MEMORY_CONFIG_ES, "Especificar configuraciÛn de memoria. (none/3k/8k/16k/24k/all) / (0/1/2/3/5) / (04/20/40/60/a0)"},
	/* fr */ {IDCLS_SPECIFY_MEMORY_CONFIG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_MEMORY_CONFIG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_MEMORY_CONFIG_IT, "Specifica la configurazione della memoria. (nessuna/3k/8k/16k/24k/tutte) / (0/1/2/3/5) / (04/20/40/60/a0)"},
	/* ko */ {IDCLS_SPECIFY_MEMORY_CONFIG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_MEMORY_CONFIG_NL, "Geef geheugen configuratie. (geen/3k/8k/16k/24k/allemaal) / (0/1/2/3/5) / (04/20/40/60/a0)"},
	/* pl */ {IDCLS_SPECIFY_MEMORY_CONFIG_PL, "Okre∂l konfiguracjÍ pamiÍci. (none/3k/8k/16k/24k/all) / (0/1/2/3/5) / (04/20/40/60/a0)"},
	/* ru */ {IDCLS_SPECIFY_MEMORY_CONFIG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_MEMORY_CONFIG_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_MEMORY_CONFIG_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_VIC1112_IEEE488,    N_("Enable VIC-1112 IEEE488 interface")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_VIC1112_IEEE488_DA, "AktivÈr VIC-1112-IEEE488-interface"},
	/* de */ {IDCLS_ENABLE_VIC1112_IEEE488_DE, "VIC-1112 IEEE488 Schnittstelle aktivieren"},
	/* es */ {IDCLS_ENABLE_VIC1112_IEEE488_ES, "Permitir interface VIC-1112 IEEE488"},
	/* fr */ {IDCLS_ENABLE_VIC1112_IEEE488_FR, "Activer l'interface VIC-1112 IEEE488"},
	/* hu */ {IDCLS_ENABLE_VIC1112_IEEE488_HU, "VIC-1112 IEEE488 interfÈsz engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_VIC1112_IEEE488_IT, "Attiva interfaccia IEEE488 VIC-1112"},
	/* ko */ {IDCLS_ENABLE_VIC1112_IEEE488_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_VIC1112_IEEE488_NL, "VIC-1112 IEEE488 interface inschakelen"},
	/* pl */ {IDCLS_ENABLE_VIC1112_IEEE488_PL, "W≥±cz interfejs IEEE488 VIC-1112"},
	/* ru */ {IDCLS_ENABLE_VIC1112_IEEE488_RU, "Enable VIC-1112 IEEE488 interface"},
	/* sv */ {IDCLS_ENABLE_VIC1112_IEEE488_SV, "Aktivera VIC-1112-IEEE488-gr‰nssnitt"},
	/* tr */ {IDCLS_ENABLE_VIC1112_IEEE488_TR, "VIC-1112 IEEE488 arabirimini aktif et"},
#endif
	
	/* vic20/vic20-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_VIC1112_IEEE488,    N_("Disable VIC-1112 IEEE488 interface")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_VIC1112_IEEE488_DA, "DeaktivÈr VIC-1112-IEEE488-interface"},
	/* de */ {IDCLS_DISABLE_VIC1112_IEEE488_DE, "VIC-1112 IEEE488 Schnittstelle deaktivieren"},
	/* es */ {IDCLS_DISABLE_VIC1112_IEEE488_ES, "Deshabilitar interface VIC-1112 IEEE488"},
	/* fr */ {IDCLS_DISABLE_VIC1112_IEEE488_FR, "DÈsactiver l'interface VIC-1112 IEEE488"},
	/* hu */ {IDCLS_DISABLE_VIC1112_IEEE488_HU, "VIC-1112 IEEE488 interfÈsz tilt·sa"},
	/* it */ {IDCLS_DISABLE_VIC1112_IEEE488_IT, "Disattiva interfaccia IEEE488 VIC-1112"},
	/* ko */ {IDCLS_DISABLE_VIC1112_IEEE488_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_VIC1112_IEEE488_NL, "VIC-1112 IEEE488 interface uitschakelen"},
	/* pl */ {IDCLS_DISABLE_VIC1112_IEEE488_PL, "Wy≥±cz interfejs IEEE488 VIC-1112"},
	/* ru */ {IDCLS_DISABLE_VIC1112_IEEE488_RU, "Disable VIC-1112 IEEE488 interface"},
	/* sv */ {IDCLS_DISABLE_VIC1112_IEEE488_SV, "Inaktivera VIC-1112-IEEE488-gr‰nssnitt"},
	/* tr */ {IDCLS_DISABLE_VIC1112_IEEE488_TR, "VIC-1112 IEEE488 arabirimini pasifle˛tir"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME,    N_("Specify 4/8/16K extension ROM name at $2000")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_DA, "Angiv navn for 4/8/16k-udviddelses-ROM pÂ $2000"},
	/* de */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_DE, "4/8/16K Erweiterungs ROM Dateiname f¸r $2000 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_ES, "Especificar nombre ROM extendida de 4/8/16KB en $2000"},
	/* fr */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_FR, "SpÈcifier le nom de l'extension ROM 4/8/16K ‡ $2000"},
	/* hu */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_HU, "Adja meg a $2000 cÌm˚ 4/8/16K ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_IT, "Specifica il nome della ROM di estensione di 4/8/16K a $2000"},
	/* ko */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_NL, "Geef de naam van het bestand voor de 4/8/16K ROM op $2000"},
	/* pl */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_PL, "Okre∂l nazwÍ ROM rozszerzenia 4/8/16K w $2000"},
	/* ru */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_RU, "Specify 4/8/16K extension ROM name at $2000"},
	/* sv */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_SV, "Ange namn fˆr 4/8/16K-utˆknings-ROM pÂ $2000"},
	/* tr */ {IDCLS_SPECIFY_EXT_ROM_2000_NAME_TR, "$2000 adresindeki 4/8/16K uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME,    N_("Specify 4/8/16K extension ROM name at $4000")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_DA, "Angiv navn for 4/8/16k-udviddelses-ROM pÂ $4000"},
	/* de */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_DE, "4/8/16K Erweiterungs ROM Dateiname f¸r $4000 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_ES, "Especificar nombre ROM extendida de 4/8/16KB en $4000"},
	/* fr */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_FR, "SpÈcifier le nom de l'extension ROM 4/8/16K ‡ $4000"},
	/* hu */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_HU, "Adja meg a $4000 cÌm˚ 4/8/16K ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_IT, "Specifica il nome della ROM di estensione di 4/8/16K a $4000"},
	/* ko */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_NL, "Geef de naam van het bestand voor de 4/8/16K ROM op $4000"},
	/* pl */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_PL, "Okre∂l nazwÍ ROM rozszerzenia 4/8/16K w $4000"},
	/* ru */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_RU, "Specify 4/8/16K extension ROM name at $4000"},
	/* sv */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_SV, "Ange namn fˆr 4/8/16K-utˆknings-ROM pÂ $4000"},
	/* tr */ {IDCLS_SPECIFY_EXT_ROM_4000_NAME_TR, "$4000 adresindeki 4/8/16K uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME,    N_("Specify 4/8/16K extension ROM name at $6000")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_DA, "Angiv navn for 4/8/16k-udviddelses-ROM pÂ $6000"},
	/* de */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_DE, "4/8/16K Erweiterungs ROM Dateiname f¸r $6000 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_ES, "Especificar nombre ROM extendida de 4/8/16KB en $6000"},
	/* fr */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_FR, "SpÈcifier le nom de l'extension ROM 4/8/16K ‡ $6000"},
	/* hu */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_HU, "Adja meg a $6000 cÌm˚ 4/8/16K ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_IT, "Specifica il nome della ROM di estensione di 4/8/16K a $6000"},
	/* ko */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_NL, "Geef de naam van het bestand voor de 4/8/16K ROM op $6000"},
	/* pl */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_PL, "Okre∂l nazwÍ ROM rozszerzenia 4/8/16K w $6000"},
	/* ru */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_RU, "Specify 4/8/16K extension ROM name at $6000"},
	/* sv */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_SV, "Ange namn fˆr 4/8/16K-utˆknings-ROM pÂ $6000"},
	/* tr */ {IDCLS_SPECIFY_EXT_ROM_6000_NAME_TR, "$6000 adresindeki 4/8/16K uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME,    N_("Specify 4/8K extension ROM name at $A000")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_DA, "Angiv navn for 4/8k-udviddelses-ROM pÂ $A000"},
	/* de */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_DE, "4/8/16K Erweiterungs ROM Dateiname f¸r $A000 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_ES, "Especificar nombre ROM extendida de 4/8KB en $A000"},
	/* fr */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_FR, "SpÈcifier le nom de l'extension ROM 4/8K ‡ $A000"},
	/* hu */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_HU, "Adja meg a $A000 cÌm˚ 4/8K ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_IT, "Specifica il nome della ROM di estensione di 4/8K a $A000"},
	/* ko */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_NL, "Geef de naam van het bestand voor de 4/8K ROM op $A000"},
	/* pl */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_PL, "Okre∂l nazwÍ ROM rozszerzenia 4/8/16K w $A000"},
	/* ru */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_RU, "Specify 4/8K extension ROM name at $A000"},
	/* sv */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_SV, "Ange namn fˆr 4/8K-utˆknings-ROM pÂ $A000"},
	/* tr */ {IDCLS_SPECIFY_EXT_ROM_A000_NAME_TR, "$A000 adresindeki 4/8K uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME,    N_("Specify 4K extension ROM name at $B000")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_DA, "Angiv navn for 4k-udviddelses-ROM pÂ $B000"},
	/* de */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_DE, "4/8/16K Erweiterungs ROM Dateiname f¸r $B000 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_ES, "Especificar nombre ROM extendida de 4KB en $B000"},
	/* fr */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_FR, "SpÈcifier le nom de l'extension ROM 4K ‡ $B000"},
	/* hu */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_HU, "Adja meg a $B000 cÌm˚ 4K ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_IT, "Specifica il nome della ROM di estensione di 4 a $B000"},
	/* ko */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_NL, "Geef de naam van het bestand voor de 4K ROM op $B000"},
	/* pl */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_PL, "Okre∂l nazwÍ ROM rozszerzenia 4/8/16K w $B000"},
	/* ru */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_RU, "Specify 4K extension ROM name at $B000"},
	/* sv */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_SV, "Ange namn fˆr 4K-utˆknings-ROM pÂ $B000"},
	/* tr */ {IDCLS_SPECIFY_EXT_ROM_B000_NAME_TR, "$B000 adresindeki 4K uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/behrbonz.c */
	/* en */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME,    N_("Specify Behr Bonz extension ROM name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_DE, "Behr Bonz Erweiterungs ROM Dateiname w‰hlen"},
	/* es */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_NL, "Geef de naam van het bestand voor de Behr Bonz uitbreidings ROM"},
	/* pl */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_BEHRBONZ_ROM_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_GENERIC_ROM_NAME,    N_("Specify generic extension ROM name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_DA, "Angiv navn pÂ standard-udviddelses-ROM"},
	/* de */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_DE, "Generisches Erweiterungs ROM w‰hlen"},
	/* es */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_ES, "Especificar nombre ROM extendida generica"},
	/* fr */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_FR, "SpÈcifier le nom de l'extension ROM gÈnÈrique"},
	/* hu */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_HU, "Adja meg az ·ltal·nos ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_IT, "Specifica il nome dell'estensione ROM generica"},
	/* ko */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_NL, "Geef de naam van het bestand voor de algemene ROM"},
	/* pl */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_PL, "Okre∂l nazwÍ typowego ROM-u rozszerzeÒ"},
	/* ru */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_RU, "Specify generic extension ROM name"},
	/* sv */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_SV, "Ange namn pÂ generiskt utˆknings-ROM"},
	/* tr */ {IDCLS_SPECIFY_GENERIC_ROM_NAME_TR, "Jenerik uzant˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME,    N_("Specify Mega-Cart extension ROM name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_DA, "Angiv navn pÂ MegaCart-udviddelses-ROM"},
	/* de */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_DE, "Mega-Cart Erweiterungs ROM w‰hlen"},
	/* es */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_ES, "Especificar nombre ROM extendida Mega Cart"},
	/* fr */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_FR, "SpÈcifier le nom de l'extension ROM Mega-Cart"},
	/* hu */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_HU, "Adja meg a Mega-Cart ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_IT, "Specifica il nome dell'estensione ROM Mega-Cart"},
	/* ko */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_NL, "Geef de naam van het bestand voor de Mega-Cart ROM"},
	/* pl */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_PL, "Okre∂l nazwÍ ROM rozszerzenia Mega-Cart"},
	/* ru */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_RU, "Specify Mega-Cart extension ROM name"},
	/* sv */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_SV, "Ange namn pÂ Mega-Cart-utˆknings-ROM"},
	/* tr */ {IDCLS_SPECIFY_MEGA_CART_ROM_NAME_TR, "Mega-Cart uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME,    N_("Specify Final Expansion extension ROM name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_DA, "Angiv navn pÂ Final Expansion udviddelses-ROM"},
	/* de */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_DE, "Name f¸r Final Expansion Erweiterungs ROM w‰hlen"},
	/* es */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_ES, "Especificar nombre ROM extendida Final Expansion"},
	/* fr */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_FR, "SpÈcifier le nom de l'extension ROM Final Expansion"},
	/* hu */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_HU, "Adja meg a Final Expansion ROM kiterjesztÈs nevÈt"},
	/* it */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_IT, "Specifica il nome dell'estensione ROM Final Expansion"},
	/* ko */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_NL, "Geef de naam van het bestand voor de Final Expansion ROM"},
	/* pl */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_PL, "Okre∂l nazwÍ rozszerzenia ROM Final Expansion"},
	/* ru */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_RU, "Specify Final Expansion extension ROM name"},
	/* sv */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_SV, "Ange namn pÂ Final Expansion utˆknings-ROM"},
	/* tr */ {IDCLS_SPECIFY_FINAL_EXPANSION_ROM_NAME_TR, "Final Expansion uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME,    N_("Specify Vic Flash Plugin extension ROM name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_DA, "Angiv navn pÂ Vic Flash Plugin udviddelses-ROM"},
	/* de */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_DE, "Name f¸r Vic Flash Plugin Erweiterungs ROM w‰hlen"},
	/* es */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_ES, "Especificar nombre ROM extendida Vic Flash Plugin"},
	/* fr */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_FR, "SpÈcifier le nom de l'extension ROM Vic Flash Plugin"},
	/* hu */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_IT, "Specifica il nome dell'estensione ROM Vic Flash Plugin"},
	/* ko */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_NL, "Geef de naam van het bestand voor de Vic Flash Plugin ROM"},
	/* pl */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_PL, "Okre∂l nazwÍ rozszerzenia ROM Vic Flash Plugin"},
	/* ru */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_RU, "Specify Vic Flash Plugin extension ROM name"},
	/* sv */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_SV, "Ange namn pÂ Vic Flash-insticksmodulutˆknings-ROM"},
	/* tr */ {IDCLS_SPECIFY_VIC_FP_ROM_NAME_TR, "Vic Flash Plugin uzant˝s˝ ROM ismini belirt"},
#endif
	
	/* vic20/cart/vic20cart.c */
	/* en */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME,    N_("Specify UltiMem extension ROM name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_DE, "UltiMem Erweiterungs ROM w‰hlen"},
	/* es */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_NL, "Geef de naam van het bestand voor de UltiMem ROM"},
	/* pl */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_VIC_UM_ROM_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/cart/finalexpansion.c */
	/* en */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK,    N_("Enable Final Expansion write back to ROM file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_DA, "AktivÈr Final Expansion tilbageskrivning til ROM-fil"},
	/* de */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_DE, "Final Expansion ROM Schreibzugriff aktivieren"},
	/* es */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_ES, "Permitir grabaciÛn en fichero ROM Final Expansion"},
	/* fr */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_FR, "Activer la rÈÈcriture du fichier ROM Final Expansion"},
	/* hu */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_HU, "Final Expansion ROM f·jlba visszaÌr·s engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_IT, "Attiva write back per Final Expansion su file ROM"},
	/* ko */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_NL, "Terugschrijven naar ROM bestand voor Final Expansion inschakelen"},
	/* pl */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_PL, "W≥±cz zapis do pliku ROM Final Expansion"},
	/* ru */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_RU, "Enable Final Expansion write back to ROM file"},
	/* sv */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_SV, "Aktivera Âterskrivning till ROM-fil fˆr Final Expansion"},
	/* tr */ {IDCLS_ENABLE_FINALEXPANSION_WRITEBACK_TR, "Final Expansion ROM dosyas˝na geri yazmay˝ aktif et"},
#endif
	
	/* vic20/cart/finalexpansion.c */
	/* en */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK,    N_("Disable Final Expansion write back to ROM file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_DA, "DeaktivÈr Final Expansion tilbageskrivning til ROM-fil"},
	/* de */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_DE, "Final Expansion ROM Schreibzugriff deaktivieren"},
	/* es */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_ES, "Deshabilitar grabaciÛn en fichero ROM Final Expansion"},
	/* fr */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_FR, "DÈsactiver la rÈÈcriture du fichier ROM Final Expansion"},
	/* hu */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_HU, "Final Expansion ROM f·jlba visszaÌr·s tilt·sa"},
	/* it */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_IT, "Disattiva write back per Final Expansion su file ROM"},
	/* ko */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_NL, "Deactiveer terug schrijven naar ROM bestand voor Final Expansion"},
	/* pl */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_PL, "Wy≥±cz zapis do pliku ROM Final Expansion"},
	/* ru */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_RU, "Disable Final Expansion write back to ROM file"},
	/* sv */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_SV, "Inaktivera Âterskrivning till ROM-fil fˆr Final Expansion"},
	/* tr */ {IDCLS_DISABLE_FINALEXPANSION_WRITEBACK_TR, "Final Expansion ROM dosyas˝na geri yazmay˝ pasifle˛tir"},
#endif
	
	/* vic20/cart/megacart.c */
	/* en */ {IDCLS_SET_MEGACART_NVRAM_NAME,    N_("Set Mega-Cart NvRAM filename")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_MEGACART_NVRAM_NAME_DA, "VÊlg Mega-Cart NvRAM filnavn"},
	/* de */ {IDCLS_SET_MEGACART_NVRAM_NAME_DE, "Mega-Cart NvRAM Imagename w‰hlen"},
	/* es */ {IDCLS_SET_MEGACART_NVRAM_NAME_ES, "Seleccionar nombre fichero  Mega-Cart NvRAM"},
	/* fr */ {IDCLS_SET_MEGACART_NVRAM_NAME_FR, "Nom du fichier NvRAM Mega-Cart"},
	/* hu */ {IDCLS_SET_MEGACART_NVRAM_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_MEGACART_NVRAM_NAME_IT, "Imposta nome file NvRAM Mega-Cart"},
	/* ko */ {IDCLS_SET_MEGACART_NVRAM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_MEGACART_NVRAM_NAME_NL, "Zet Mega-Cart NvRAM bestandsnaam"},
	/* pl */ {IDCLS_SET_MEGACART_NVRAM_NAME_PL, "Okre∂l nazwÍ pliku NvRAM Mega-Cart"},
	/* ru */ {IDCLS_SET_MEGACART_NVRAM_NAME_RU, "Set Mega-Cart NvRAM filename"},
	/* sv */ {IDCLS_SET_MEGACART_NVRAM_NAME_SV, "V‰lj  Mega-Cart NvRAM-filnamn"},
	/* tr */ {IDCLS_SET_MEGACART_NVRAM_NAME_TR, "Mega-Cart NvRAM dosyaismini gir"},
#endif
	
	/* vic20/cart/megacart.c */
	/* en */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE,    N_("Enable Mega-Cart NvRAM writeback")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_DA, "AktivÈr Mega-Cart NvRAM tilbageskrivning"},
	/* de */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_DE, "Mega-Cart NvRAM R¸ckschreiben aktivieren"},
	/* es */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_ES, "Permitir respuesta Mega-Cart NvRAM"},
	/* fr */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_FR, "Permettre la rÈÈcriture de l'image Mega-Cart"},
	/* hu */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_IT, "Write back NvRAM Mega-Cart"},
	/* ko */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_NL, "Mega-Cart NvRAM terugschrijven inschakelem"},
	/* pl */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_PL, "W≥±cz zapis do NvRAM Mega-Cart"},
	/* ru */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_RU, "Enable Mega-Cart NvRAM writeback"},
	/* sv */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_SV, "Aktivera Mega-Cart NvRAM-Âterskrivning"},
	/* tr */ {IDCLS_ENABLE_MEGACART_NVRAM_WRITE_TR, "Mega-Cart NvRAM geri yazmay˝ aktif et"},
#endif
	
	/* vic20/cart/megacart.c */
	/* en */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE,    N_("Disable Mega-Cart NvRAM writeback")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_DA, "DeaktivÈr Mega-Cart NvRAM tilbageskrivning"},
	/* de */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_DE, "Mega-Cart NvRAM R¸ckschreiben deaktivieren"},
	/* es */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_ES, "Deshabilitar grabaciÛn en Mega-Cart NvRAM"},
	/* fr */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_FR, "Interdire la rÈÈcriture NvRAM Mega-Cart"},
	/* hu */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_IT, "Disattiva write back su NvRAM Mega-Cart"},
	/* ko */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_NL, "Mega-Cart NvRAM terugschrijven uitschakelen"},
	/* pl */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_PL, "Wy≥±cz zapis do NvRAM Mega-Cart"},
	/* ru */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_RU, "Disable Mega-Cart NvRAM writeback"},
	/* sv */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_SV, "Inaktivera Mega-Cart NvRAM-Âterskrivning"},
	/* tr */ {IDCLS_DISABLE_MEGACART_NVRAM_WRITE_TR, "Mega-Cart NvRAM geri yazmay˝ pasifle˛tir"},
#endif
	
	/* vic20/cart/vic-fp.c */
	/* en */ {IDCLS_ENABLE_VICFP_ROM_WRITE,    N_("Enable Vic Flash Plugin write back to ROM file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_VICFP_ROM_WRITE_DA, "AktivÈr Vic Flash Plugin tilbageskrivning til ROM-fil"},
	/* de */ {IDCLS_ENABLE_VICFP_ROM_WRITE_DE, "Vic Flash Plugin R¸ckschreiben auf ROM Datei aktivieren"},
	/* es */ {IDCLS_ENABLE_VICFP_ROM_WRITE_ES, "Permitir grabaciÛn en ROM Vic Flash Plugin"},
	/* fr */ {IDCLS_ENABLE_VICFP_ROM_WRITE_FR, "Activer la rÈÈcriture du fichier ROM Vic Flash Plugin"},
	/* hu */ {IDCLS_ENABLE_VICFP_ROM_WRITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_VICFP_ROM_WRITE_IT, "Attiva write back per Vic Flash Plugin su file ROM"},
	/* ko */ {IDCLS_ENABLE_VICFP_ROM_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_VICFP_ROM_WRITE_NL, "Terugschrijven naar ROM bestand voor Vic Flash Plugin inschakelen"},
	/* pl */ {IDCLS_ENABLE_VICFP_ROM_WRITE_PL, "W≥±cz zapis do pliku ROM Vic Flash Plugin"},
	/* ru */ {IDCLS_ENABLE_VICFP_ROM_WRITE_RU, "Enable Vic Flash Plugin write back to ROM file"},
	/* sv */ {IDCLS_ENABLE_VICFP_ROM_WRITE_SV, "Aktivera Âterskrivning till ROM-fil fˆr Vic Flash-insticksmodul"},
	/* tr */ {IDCLS_ENABLE_VICFP_ROM_WRITE_TR, "Flash Plugin ROM dosyas˝na geri yazmay˝ aktif et"},
#endif
	
	/* vic20/cart/vic-fp.c */
	/* en */ {IDCLS_DISABLE_VICFP_ROM_WRITE,    N_("Disable Vic Flash Plugin write back to ROM file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_VICFP_ROM_WRITE_DA, "DeaktivÈr Vic Flash Plugin tilbageskrivning til ROM-fil"},
	/* de */ {IDCLS_DISABLE_VICFP_ROM_WRITE_DE, "Vic Flash Plugin R¸ckschreiben auf ROM Datei deaktivieren"},
	/* es */ {IDCLS_DISABLE_VICFP_ROM_WRITE_ES, "Deshabilitar grabaciÛn en ROM Vic Flash Plugin"},
	/* fr */ {IDCLS_DISABLE_VICFP_ROM_WRITE_FR, "DÈsactiver la rÈÈcriture du fichier ROM Vic Flash Plugin"},
	/* hu */ {IDCLS_DISABLE_VICFP_ROM_WRITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_VICFP_ROM_WRITE_IT, "Disattiva write back per Vic Flash Plugin su file ROM"},
	/* ko */ {IDCLS_DISABLE_VICFP_ROM_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_VICFP_ROM_WRITE_NL, "Terugschrijven naar ROM bestand voor Vic Flash Plugin uitschakelen"},
	/* pl */ {IDCLS_DISABLE_VICFP_ROM_WRITE_PL, "Wy≥±cz zapis do pliku ROM Vic Flash Plugin"},
	/* ru */ {IDCLS_DISABLE_VICFP_ROM_WRITE_RU, "Disable Vic Flash Plugin write back to ROM file"},
	/* sv */ {IDCLS_DISABLE_VICFP_ROM_WRITE_SV, "Inaktivera Âterskrivning till ROM-fil fˆr Vic Flash-insticksmodul"},
	/* tr */ {IDCLS_DISABLE_VICFP_ROM_WRITE_TR, "Flash Plugin ROM dosyas˝na geri yazmay˝ pasifle˛tir"},
#endif
	
	/* vic20/cart/ultimem.c */
	/* en */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE,    N_("Enable UltiMem write back to ROM file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_DE, "UltiMem ROM Schreibzugriff aktivieren"},
	/* es */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_NL, "Terugschrijven naar ROM bestand voor UltiMem inschakelen"},
	/* pl */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_VIC_UM_ROM_WRITE_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/cart/ultimem.c */
	/* en */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE,    N_("Disable UltiMem write back to ROM file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_DE, "UltiMem ROM Schreibzugriff deaktivieren"},
	/* es */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_NL, "Terugschrijven naar ROM bestand voor UltiMem uitschakelen"},
	/* pl */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_VIC_UM_ROM_WRITE_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/cart/ioramcart.c */
	/* en */ {IDCLS_ENABLE_IO2_RAM,    N_("Enable I/O-2 RAM")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IO2_RAM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_IO2_RAM_DE, "I/O-2 RAM aktivieren"},
	/* es */ {IDCLS_ENABLE_IO2_RAM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_IO2_RAM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_IO2_RAM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_IO2_RAM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_IO2_RAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IO2_RAM_NL, "I/O-2 RAM inschakelen"},
	/* pl */ {IDCLS_ENABLE_IO2_RAM_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_IO2_RAM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_IO2_RAM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_IO2_RAM_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/cart/ioramcart.c */
	/* en */ {IDCLS_DISABLE_IO2_RAM,    N_("Disable I/O-2 RAM")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IO2_RAM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_IO2_RAM_DE, "I/O-2 RAM deaktivieren"},
	/* es */ {IDCLS_DISABLE_IO2_RAM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_IO2_RAM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_IO2_RAM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_IO2_RAM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_IO2_RAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IO2_RAM_NL, "I/O-2 RAM uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IO2_RAM_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_IO2_RAM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_IO2_RAM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_IO2_RAM_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/cart/ioramcart.c */
	/* en */ {IDCLS_ENABLE_IO3_RAM,    N_("Enable I/O-3 RAM")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_IO3_RAM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_IO3_RAM_DE, "I/O-3 RAM aktivieren"},
	/* es */ {IDCLS_ENABLE_IO3_RAM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_IO3_RAM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_IO3_RAM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_IO3_RAM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_IO3_RAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_IO3_RAM_NL, "I/O-3 RAM inschakelen"},
	/* pl */ {IDCLS_ENABLE_IO3_RAM_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_IO3_RAM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_IO3_RAM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_IO3_RAM_TR, ""},  /* fuzzy */
#endif
	
	/* vic20/cart/ioramcart.c */
	/* en */ {IDCLS_DISABLE_IO3_RAM,    N_("Disable I/O-3 RAM")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_IO3_RAM_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_IO3_RAM_DE, "I/O-3 RAM deaktivieren"},
	/* es */ {IDCLS_DISABLE_IO3_RAM_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_IO3_RAM_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_IO3_RAM_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_IO3_RAM_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_IO3_RAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_IO3_RAM_NL, "I/O-3 RAM uitschakelen"},
	/* pl */ {IDCLS_DISABLE_IO3_RAM_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_IO3_RAM_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_IO3_RAM_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_IO3_RAM_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_SET_BORDER_MODE,    N_("Set border display mode (0: normal, 1: full, 2: debug)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_BORDER_MODE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_BORDER_MODE_DE, "Rahmen Darstellung Modus (0: Normal, 1: Voll, 2: Debug)"},
	/* es */ {IDCLS_SET_BORDER_MODE_ES, "Seleccionar tipo de borde (0: normal, 1: completo, 2: depuraciÛn)"},
	/* fr */ {IDCLS_SET_BORDER_MODE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_BORDER_MODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_BORDER_MODE_IT, "Imposta la modalit‡ di visualizzazione del bordo (0: normale, 1: intero, 2: debug)"},
	/* ko */ {IDCLS_SET_BORDER_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_BORDER_MODE_NL, "Stel border weergavemodus in (0: normaal, 1: volledig, 2: debug)"},
	/* pl */ {IDCLS_SET_BORDER_MODE_PL, "Okre∂l tryb wy∂wietlania ramki (0: zwyk≥a, 1: pe≥na, 2: debug)"},
	/* ru */ {IDCLS_SET_BORDER_MODE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_BORDER_MODE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_BORDER_MODE_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_SPRITE_BACKGROUND,    N_("Enable sprite-background collision registers")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SPRITE_BACKGROUND_DA, "AktivÈr sprite-til-baggrunds-kollisionsregistre"},
	/* de */ {IDCLS_ENABLE_SPRITE_BACKGROUND_DE, "Sprite-Hintergrund Kollisionen aktivieren"},
	/* es */ {IDCLS_ENABLE_SPRITE_BACKGROUND_ES, "Habilitar registros colisiÛn sprite-fondo"},
	/* fr */ {IDCLS_ENABLE_SPRITE_BACKGROUND_FR, "Activer les registres de collisions de sprite avec arriËre-plan"},
	/* hu */ {IDCLS_ENABLE_SPRITE_BACKGROUND_HU, "Sprite-h·ttÈr ¸tkˆzÈsi regiszterek engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_SPRITE_BACKGROUND_IT, "Attiva registri di collisione sprite-sfondo"},
	/* ko */ {IDCLS_ENABLE_SPRITE_BACKGROUND_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SPRITE_BACKGROUND_NL, "Sprite-achtergrond botsing registers inschakelen"},
	/* pl */ {IDCLS_ENABLE_SPRITE_BACKGROUND_PL, "W≥±cz rejestry kolizji duszkÛw z t≥em"},
	/* ru */ {IDCLS_ENABLE_SPRITE_BACKGROUND_RU, "Enable sprite-background collision registers"},
	/* sv */ {IDCLS_ENABLE_SPRITE_BACKGROUND_SV, "Aktivera sprite-till-bakgrund-kollisionsregister"},
	/* tr */ {IDCLS_ENABLE_SPRITE_BACKGROUND_TR, "Yarat˝k-arka plan Áarp˝˛ma registerlar˝n˝ aktif et"},
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_SPRITE_BACKGROUND,    N_("Disable sprite-background collision registers")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SPRITE_BACKGROUND_DA, "DeaktivÈr sprite-til-baggrunds-kollisionsregistre"},
	/* de */ {IDCLS_DISABLE_SPRITE_BACKGROUND_DE, "Sprite-Hintergrund Kollisionen deaktivieren"},
	/* es */ {IDCLS_DISABLE_SPRITE_BACKGROUND_ES, "Deshabilitar registros colisiÛn sprite-fondo"},
	/* fr */ {IDCLS_DISABLE_SPRITE_BACKGROUND_FR, "DÈsactiver les registres de collisions de sprite avec arriËre-plan"},
	/* hu */ {IDCLS_DISABLE_SPRITE_BACKGROUND_HU, "Sprite-h·ttÈr ¸tkˆzÈsi regiszterek tilt·sa"},
	/* it */ {IDCLS_DISABLE_SPRITE_BACKGROUND_IT, "Disattiva registri di collisione sprite-sfondo"},
	/* ko */ {IDCLS_DISABLE_SPRITE_BACKGROUND_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SPRITE_BACKGROUND_NL, "Sprite-achtergrond botsing registers uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SPRITE_BACKGROUND_PL, "Wy≥±cz rejestry kolizji duszkÛw z t≥em"},
	/* ru */ {IDCLS_DISABLE_SPRITE_BACKGROUND_RU, "Disable sprite-background collision registers"},
	/* sv */ {IDCLS_DISABLE_SPRITE_BACKGROUND_SV, "Inaktivera sprite-till-bakgrund-kollisionsregister"},
	/* tr */ {IDCLS_DISABLE_SPRITE_BACKGROUND_TR, "Yarat˝k-arka plan Áarp˝˛ma registerlar˝n˝ pasifle˛tir"},
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_SPRITE_SPRITE,    N_("Enable sprite-sprite collision registers")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SPRITE_SPRITE_DA, "AktivÈr sprite-til-sprite-kollisionsregistre"},
	/* de */ {IDCLS_ENABLE_SPRITE_SPRITE_DE, "Sprite-Sprite Kollisionen aktivieren"},
	/* es */ {IDCLS_ENABLE_SPRITE_SPRITE_ES, "Habilitar registros colisiÛn sprite-sprite"},
	/* fr */ {IDCLS_ENABLE_SPRITE_SPRITE_FR, "Activer les registres de collisions de sprite avec sprite"},
	/* hu */ {IDCLS_ENABLE_SPRITE_SPRITE_HU, "Sprite-sprite ¸tkˆzÈsi regiszterek engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_SPRITE_SPRITE_IT, "Attiva registri di collisione sprite-sprite"},
	/* ko */ {IDCLS_ENABLE_SPRITE_SPRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SPRITE_SPRITE_NL, "Sprite-sprite botsing registers inschakelen"},
	/* pl */ {IDCLS_ENABLE_SPRITE_SPRITE_PL, "W≥±cz rejestry kolizji duszkÛw"},
	/* ru */ {IDCLS_ENABLE_SPRITE_SPRITE_RU, "Enable sprite-sprite collision registers"},
	/* sv */ {IDCLS_ENABLE_SPRITE_SPRITE_SV, "Aktivera sprite-till-sprite-kollisionsregister"},
	/* tr */ {IDCLS_ENABLE_SPRITE_SPRITE_TR, "Yarat˝k-yarat˝k Áarp˝˛ma registerlar˝n˝ aktif et"},
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_SPRITE_SPRITE,    N_("Disable sprite-sprite collision registers")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SPRITE_SPRITE_DA, "DeaktivÈr sprite-til-sprite-kollisionsregistre"},
	/* de */ {IDCLS_DISABLE_SPRITE_SPRITE_DE, "Sprite-Sprite Kollisionen deaktivieren"},
	/* es */ {IDCLS_DISABLE_SPRITE_SPRITE_ES, "Deshabilitar registros colisiÛn sprite-sprite"},
	/* fr */ {IDCLS_DISABLE_SPRITE_SPRITE_FR, "DÈsactiver les registres de collisions de sprite avec sprite"},
	/* hu */ {IDCLS_DISABLE_SPRITE_SPRITE_HU, "Sprite-sprite ¸tkˆzÈsi regiszterek tilt·sa"},
	/* it */ {IDCLS_DISABLE_SPRITE_SPRITE_IT, "Disattiva registri di collisione sprite-sprite"},
	/* ko */ {IDCLS_DISABLE_SPRITE_SPRITE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SPRITE_SPRITE_NL, "Sprite-sprite botsing registers uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SPRITE_SPRITE_PL, "Wy≥±cz rejestry kolizji duszkÛw"},
	/* ru */ {IDCLS_DISABLE_SPRITE_SPRITE_RU, "Disable sprite-sprite collision registers"},
	/* sv */ {IDCLS_DISABLE_SPRITE_SPRITE_SV, "Inaktivera sprite-till-sprite-kollisionsregister"},
	/* tr */ {IDCLS_DISABLE_SPRITE_SPRITE_TR, "Yarat˝k-yarat˝k Áarp˝˛ma registerlar˝n˝ pasifle˛tir"},
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_VSPBUG,    N_("Enable VSP bug emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_VSPBUG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_VSPBUG_DE, "VSP Fehler Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_VSPBUG_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_VSPBUG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_VSPBUG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_VSPBUG_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_VSPBUG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_VSPBUG_NL, "VSP bug emulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_VSPBUG_PL, "W≥±cz emulacjÍ blÍdu VSP"},
	/* ru */ {IDCLS_ENABLE_VSPBUG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_VSPBUG_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_VSPBUG_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_VSPBUG,    N_("Disable VSP bug emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_VSPBUG_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_VSPBUG_DE, "VSP Fehler Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_VSPBUG_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_VSPBUG_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_VSPBUG_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_VSPBUG_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_VSPBUG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_VSPBUG_NL, "VSP bug emulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_VSPBUG_PL, "Wy≥±cz emulacjÍ b≥Ídu VSP"},
	/* ru */ {IDCLS_DISABLE_VSPBUG_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_VSPBUG_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_VSPBUG_TR, ""},  /* fuzzy */
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_USE_NEW_LUMINANCES,    N_("Use new luminances")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_NEW_LUMINANCES_DA, "Brug nye lysstyrker"},
	/* de */ {IDCLS_USE_NEW_LUMINANCES_DE, "Neue Helligkeitsemulation"},
	/* es */ {IDCLS_USE_NEW_LUMINANCES_ES, "Usar luminancias nuevas"},
	/* fr */ {IDCLS_USE_NEW_LUMINANCES_FR, "Utiliser les nouvelles luminescences"},
	/* hu */ {IDCLS_USE_NEW_LUMINANCES_HU, "Az ˙j fÈnyerıket haszn·lja"},
	/* it */ {IDCLS_USE_NEW_LUMINANCES_IT, "Usa nuove luminanze"},
	/* ko */ {IDCLS_USE_NEW_LUMINANCES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_NEW_LUMINANCES_NL, "Gebruik nieuwe luminantie"},
	/* pl */ {IDCLS_USE_NEW_LUMINANCES_PL, "Uøyj nowego typu jaskrawo∂ci"},
	/* ru */ {IDCLS_USE_NEW_LUMINANCES_RU, "Use new luminances"},
	/* sv */ {IDCLS_USE_NEW_LUMINANCES_SV, "Anv‰nd nya ljusstyrkor"},
	/* tr */ {IDCLS_USE_NEW_LUMINANCES_TR, "Yeni parlakl˝klar˝ kullan"},
#endif
	
	/* vicii/vicii-cmdline-options.c */
	/* en */ {IDCLS_USE_OLD_LUMINANCES,    N_("Use old luminances")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_OLD_LUMINANCES_DA, "Brug gamle lysstyrker"},
	/* de */ {IDCLS_USE_OLD_LUMINANCES_DE, "Alte Helligkeitsemulation"},
	/* es */ {IDCLS_USE_OLD_LUMINANCES_ES, "Usar luminancias antiguas"},
	/* fr */ {IDCLS_USE_OLD_LUMINANCES_FR, "Utiliser les anciennes luminescences"},
	/* hu */ {IDCLS_USE_OLD_LUMINANCES_HU, "A rÈgi fÈnyerıket haszn·lja"},
	/* it */ {IDCLS_USE_OLD_LUMINANCES_IT, "Usa vecchie luminanze"},
	/* ko */ {IDCLS_USE_OLD_LUMINANCES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_OLD_LUMINANCES_NL, "Gebruik oude luminantie"},
	/* pl */ {IDCLS_USE_OLD_LUMINANCES_PL, "Uøyj starego typu jaskrawo∂ci"},
	/* ru */ {IDCLS_USE_OLD_LUMINANCES_RU, "Use old luminances"},
	/* sv */ {IDCLS_USE_OLD_LUMINANCES_SV, "Anv‰nd gamla ljusstyrkor"},
	/* tr */ {IDCLS_USE_OLD_LUMINANCES_TR, "Eski parlakl˝klar˝ kullan"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_HWSCALE_POSSIBLE,    N_("Enable the possibility of hardware scaling")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_DE, "Hardwareunterst¸tzung f¸r Skalierung aktivieren"},
	/* es */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_ES, "Permitir escalado hardware"},
	/* fr */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_IT, "Attiva la possibilit‡ di ridimensionamento a livello hardware"},
	/* ko */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_NL, "Mogelijkheid van hardwareschaling inschakelen"},
	/* pl */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_PL, "W≥±cz moøliwo∂Ê skalowania sprzÍtowego"},
	/* ru */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_HWSCALE_POSSIBLE_TR, ""},  /* fuzzy */
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_HWSCALE_POSSIBLE,    N_("Disable the possibility of hardware scaling")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_DE, "Hardwareunterst¸tzung f¸r Skalierung deaktivieren"},
	/* es */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_ES, "Deshabilitar escalado hardware"},
	/* fr */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_IT, "Disattiva la possibilit‡ di ridimensionamento a livello hardware"},
	/* ko */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_NL, "Mogelijkheid van hardwareschaling uitschakelen"},
	/* pl */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_PL, "Wy≥±cz moøliwo∂Ê skalowania sprzÍtowego"},
	/* ru */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_HWSCALE_POSSIBLE_TR, ""},  /* fuzzy */
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DOUBLE_SIZE,    N_("Enable double size")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DOUBLE_SIZE_DA, "AktivÈr dobbelt st¯rrelse"},
	/* de */ {IDCLS_ENABLE_DOUBLE_SIZE_DE, "Doppelte Grˆﬂe aktivieren"},
	/* es */ {IDCLS_ENABLE_DOUBLE_SIZE_ES, "Permitir tamaÒo doble"},
	/* fr */ {IDCLS_ENABLE_DOUBLE_SIZE_FR, "Taille double"},
	/* hu */ {IDCLS_ENABLE_DOUBLE_SIZE_HU, "Dupla mÈret engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_DOUBLE_SIZE_IT, "Attiva dimensione doppia"},
	/* ko */ {IDCLS_ENABLE_DOUBLE_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DOUBLE_SIZE_NL, "Dubbele grootte inschakelen"},
	/* pl */ {IDCLS_ENABLE_DOUBLE_SIZE_PL, "W≥±cz podwÛjny rozmiar"},
	/* ru */ {IDCLS_ENABLE_DOUBLE_SIZE_RU, "Enable double size"},
	/* sv */ {IDCLS_ENABLE_DOUBLE_SIZE_SV, "Aktivera dubbel storlek"},
	/* tr */ {IDCLS_ENABLE_DOUBLE_SIZE_TR, "«ift boyutu aktif et"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DOUBLE_SIZE,    N_("Disable double size")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DOUBLE_SIZE_DA, "DeaktivÈr dobbelt st¯rrelse"},
	/* de */ {IDCLS_DISABLE_DOUBLE_SIZE_DE, "Doppelte Grˆﬂe deaktivieren"},
	/* es */ {IDCLS_DISABLE_DOUBLE_SIZE_ES, "Deshabilitar tamaÒo doble"},
	/* fr */ {IDCLS_DISABLE_DOUBLE_SIZE_FR, "Taille normale"},
	/* hu */ {IDCLS_DISABLE_DOUBLE_SIZE_HU, "Dupla mÈret tilt·sa"},
	/* it */ {IDCLS_DISABLE_DOUBLE_SIZE_IT, "Disattiva dimensione doppia"},
	/* ko */ {IDCLS_DISABLE_DOUBLE_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DOUBLE_SIZE_NL, "Dubbele grootte uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DOUBLE_SIZE_PL, "Wy≥±cz podwÛjny rozmiar"},
	/* ru */ {IDCLS_DISABLE_DOUBLE_SIZE_RU, "Disable double size"},
	/* sv */ {IDCLS_DISABLE_DOUBLE_SIZE_SV, "Inaktivera dubbel storlek"},
	/* tr */ {IDCLS_DISABLE_DOUBLE_SIZE_TR, "«ift boyutu pasifle˛tir"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_DOUBLE_SCAN,    N_("Enable double scan")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DOUBLE_SCAN_DA, "AktivÈr dobbelt-skan"},
	/* de */ {IDCLS_ENABLE_DOUBLE_SCAN_DE, "Doppelt Scan aktivieren"},
	/* es */ {IDCLS_ENABLE_DOUBLE_SCAN_ES, "Permitir doble escaneado"},
	/* fr */ {IDCLS_ENABLE_DOUBLE_SCAN_FR, "Activer le mode double scan"},
	/* hu */ {IDCLS_ENABLE_DOUBLE_SCAN_HU, "Dupla p·szt·z·s engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_DOUBLE_SCAN_IT, "Attiva scansione doppia"},
	/* ko */ {IDCLS_ENABLE_DOUBLE_SCAN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DOUBLE_SCAN_NL, "Dubbele scan inschakelen"},
	/* pl */ {IDCLS_ENABLE_DOUBLE_SCAN_PL, "W≥±cz podwÛjne skanowanie"},
	/* ru */ {IDCLS_ENABLE_DOUBLE_SCAN_RU, "Enable double scan"},
	/* sv */ {IDCLS_ENABLE_DOUBLE_SCAN_SV, "Aktivera dubbelskanning"},
	/* tr */ {IDCLS_ENABLE_DOUBLE_SCAN_TR, "«ift taramay˝ aktif et"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_DOUBLE_SCAN,    N_("Disable double scan")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DOUBLE_SCAN_DA, "DeaktivÈr dobbelt-skan"},
	/* de */ {IDCLS_DISABLE_DOUBLE_SCAN_DE, "Doppelt Scan deaktivieren"},
	/* es */ {IDCLS_DISABLE_DOUBLE_SCAN_ES, "Deshabilitar doble escaneado"},
	/* fr */ {IDCLS_DISABLE_DOUBLE_SCAN_FR, "DÈsactiver le mode double scan"},
	/* hu */ {IDCLS_DISABLE_DOUBLE_SCAN_HU, "Dupla p·szt·z·s tilt·sa"},
	/* it */ {IDCLS_DISABLE_DOUBLE_SCAN_IT, "Disattiva scansione doppia"},
	/* ko */ {IDCLS_DISABLE_DOUBLE_SCAN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DOUBLE_SCAN_NL, "Dubbele scan uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DOUBLE_SCAN_PL, "Wy≥±cz podwÛjne skanowanie"},
	/* ru */ {IDCLS_DISABLE_DOUBLE_SCAN_RU, "Disable double scan"},
	/* sv */ {IDCLS_DISABLE_DOUBLE_SCAN_SV, "Inaktivera dubbelskanning"},
	/* tr */ {IDCLS_DISABLE_DOUBLE_SCAN_TR, "«ift taramay˝ pasifle˛tir"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_AUDIO_LEAK,    N_("Enable audio leak emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_AUDIO_LEAK_DA, "AktivÈr audiolÊkage-emulering"},
	/* de */ {IDCLS_ENABLE_AUDIO_LEAK_DE, "Audio Leak Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_AUDIO_LEAK_ES, "Permitir emulaciÛn audio leak"},
	/* fr */ {IDCLS_ENABLE_AUDIO_LEAK_FR, "Activer l'Èmulation de fuite audio"},
	/* hu */ {IDCLS_ENABLE_AUDIO_LEAK_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_AUDIO_LEAK_IT, "Attiva emulazione audio leak"},
	/* ko */ {IDCLS_ENABLE_AUDIO_LEAK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_AUDIO_LEAK_NL, "Audio lek emulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_AUDIO_LEAK_PL, "W≥±cz emulacjÍ brudÛw w dºwiÍku"},
	/* ru */ {IDCLS_ENABLE_AUDIO_LEAK_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_AUDIO_LEAK_SV, "Aktivera emulering av audiol‰ckage"},
	/* tr */ {IDCLS_ENABLE_AUDIO_LEAK_TR, "Ses kaÁa˝ em¸lasyonunu aktif et"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_AUDIO_LEAK,    N_("Disable audio leak emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_AUDIO_LEAK_DA, "DeaktivÈr audiolÊkage-emulering"},
	/* de */ {IDCLS_DISABLE_AUDIO_LEAK_DE, "Audio Leak Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_AUDIO_LEAK_ES, "Deshabilitar emulaciÛn audio leak"},
	/* fr */ {IDCLS_DISABLE_AUDIO_LEAK_FR, "DÈsactiver l'Èmulation de fuite audio"},
	/* hu */ {IDCLS_DISABLE_AUDIO_LEAK_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_AUDIO_LEAK_IT, "Disattiva emulazione audio leak"},
	/* ko */ {IDCLS_DISABLE_AUDIO_LEAK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_AUDIO_LEAK_NL, "Audio lek emulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_AUDIO_LEAK_PL, "Wy≥±cz emulacjÍ brudÛw w dºwiÍku"},
	/* ru */ {IDCLS_DISABLE_AUDIO_LEAK_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_AUDIO_LEAK_SV, "Inaktivera emulering av audiol‰ckage"},
	/* tr */ {IDCLS_DISABLE_AUDIO_LEAK_TR, "Ses kaÁa˝ em¸lasyonunu pasifle˛tir"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_HARDWARE_SCALING,    N_("Enable hardware scaling")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_HARDWARE_SCALING_DA, "AktivÈr hardware-skalering"},
	/* de */ {IDCLS_ENABLE_HARDWARE_SCALING_DE, "Hardwareunterst¸tzung f¸r Skalierung aktivieren"},
	/* es */ {IDCLS_ENABLE_HARDWARE_SCALING_ES, "Permitir escalado hardware"},
	/* fr */ {IDCLS_ENABLE_HARDWARE_SCALING_FR, "Activer le \"scaling\" matÈriel"},
	/* hu */ {IDCLS_ENABLE_HARDWARE_SCALING_HU, "Hardver ·tmÈretezÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_HARDWARE_SCALING_IT, "Attiva ridimensionamento a livello hardware"},
	/* ko */ {IDCLS_ENABLE_HARDWARE_SCALING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_HARDWARE_SCALING_NL, "Hardwareschaling inschakelen"},
	/* pl */ {IDCLS_ENABLE_HARDWARE_SCALING_PL, "W≥±cz skalowanie sprzÍtowe"},
	/* ru */ {IDCLS_ENABLE_HARDWARE_SCALING_RU, "Enable hardware scaling"},
	/* sv */ {IDCLS_ENABLE_HARDWARE_SCALING_SV, "Aktivera maskinvaruskalning"},
	/* tr */ {IDCLS_ENABLE_HARDWARE_SCALING_TR, "Donan˝m destekli ˆlÁeklendirmeyi aktif et"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_HARDWARE_SCALING,    N_("Disable hardware scaling")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_HARDWARE_SCALING_DA, "DeaktivÈr hardware-skalering"},
	/* de */ {IDCLS_DISABLE_HARDWARE_SCALING_DE, "Hardwareunterst¸tzung f¸r Skalierung deaktivieren"},
	/* es */ {IDCLS_DISABLE_HARDWARE_SCALING_ES, "Deshabilitar escalado hardware"},
	/* fr */ {IDCLS_DISABLE_HARDWARE_SCALING_FR, "DÈsactiver le \"scaling\" matÈriel"},
	/* hu */ {IDCLS_DISABLE_HARDWARE_SCALING_HU, "Hardver ·tmÈretezÈs tilt·sa"},
	/* it */ {IDCLS_DISABLE_HARDWARE_SCALING_IT, "Disattiva ridimensionamento a livello hardware"},
	/* ko */ {IDCLS_DISABLE_HARDWARE_SCALING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_HARDWARE_SCALING_NL, "Hardwareschaling uitschakelen"},
	/* pl */ {IDCLS_DISABLE_HARDWARE_SCALING_PL, "Wy≥±cz skalowanie sprzÍtowe"},
	/* ru */ {IDCLS_DISABLE_HARDWARE_SCALING_RU, "Disable hardware scaling"},
	/* sv */ {IDCLS_DISABLE_HARDWARE_SCALING_SV, "Inaktivera maskinvaruskalning"},
	/* tr */ {IDCLS_DISABLE_HARDWARE_SCALING_TR, "Donan˝m destekli ˆlÁeklendirmeyi pasifle˛tir"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_SET_VIDEO_FILTER,    N_("Select rendering filter: (0: none, 1: CRT emulation, 2: scale2x)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_VIDEO_FILTER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_VIDEO_FILTER_DE, "W‰hle Rendering Filter: (0: kein, 1: CRT Emulation, 2: Scale2x)"},
	/* es */ {IDCLS_SET_VIDEO_FILTER_ES, "Seleccionar filtro de renderizado: ( 0: no, 1: emulaciÛn CRT, 2: scale2x)"},
	/* fr */ {IDCLS_SET_VIDEO_FILTER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_VIDEO_FILTER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_VIDEO_FILTER_IT, "Seleziona il filtro di rendering, (0: none, 1: CRT emulation, 2: scale2x)"},
	/* ko */ {IDCLS_SET_VIDEO_FILTER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_VIDEO_FILTER_NL, "Selecteer rendering filter: (0: geen, 1: CRT emulatie, 2: Scale2x)"},
	/* pl */ {IDCLS_SET_VIDEO_FILTER_PL, "Wybierz filtr renderera: (0:brak, 1: emulacja CRT, 2: scale2x)"},
	/* ru */ {IDCLS_SET_VIDEO_FILTER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_VIDEO_FILTER_SV, "Ange rendreringsfiler: (0:ingen, 1:CRT-emulering, 2: scale2x)"},
	/* tr */ {IDCLS_SET_VIDEO_FILTER_TR, ""},  /* fuzzy */
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_USE_INTERNAL_CALC_PALETTE,    N_("Use an internal calculated palette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_INTERNAL_CALC_PALETTE_DA, "Brug en internt beregnet palette"},
	/* de */ {IDCLS_USE_INTERNAL_CALC_PALETTE_DE, "Benutze intern berechnete Palette"},
	/* es */ {IDCLS_USE_INTERNAL_CALC_PALETTE_ES, "Usar paleta calculada internamente"},
	/* fr */ {IDCLS_USE_INTERNAL_CALC_PALETTE_FR, "Utiliser une palette interne calculÈe"},
	/* hu */ {IDCLS_USE_INTERNAL_CALC_PALETTE_HU, "Belsıleg sz·mÌtott paletta haszn·lata"},
	/* it */ {IDCLS_USE_INTERNAL_CALC_PALETTE_IT, "Usa una palette interna calcolata"},
	/* ko */ {IDCLS_USE_INTERNAL_CALC_PALETTE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_INTERNAL_CALC_PALETTE_NL, "Gebruik een intern berekend kleurenpalet"},
	/* pl */ {IDCLS_USE_INTERNAL_CALC_PALETTE_PL, "Uøywaj wewnÍtrznej obliczonej palety kolorystycznej"},
	/* ru */ {IDCLS_USE_INTERNAL_CALC_PALETTE_RU, "Use an internal calculated palette"},
	/* sv */ {IDCLS_USE_INTERNAL_CALC_PALETTE_SV, "Anv‰nd en internt ber‰knad palett"},
	/* tr */ {IDCLS_USE_INTERNAL_CALC_PALETTE_TR, "Dahili hesaplanm˝˛ bir palet kullan"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_USE_EXTERNAL_FILE_PALETTE,    N_("Use an external palette (file)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_DA, "Brug en ekstern palette (fil)"},
	/* de */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_DE, "Benutze externe Palette (Datei)"},
	/* es */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_ES, "Usar paleta externa (fichero)"},
	/* fr */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_FR, "Utiliser une palette externe (fichier)"},
	/* hu */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_HU, "K¸lsı paletta haszn·lata (f·jl)"},
	/* it */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_IT, "Usa una palette esterna (file)"},
	/* ko */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_NL, "Gebruik een extern kleurenpalet (bestand)"},
	/* pl */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_PL, "Uøywaj zewnÍtrznej palety kolorystycznej (z pliku)"},
	/* ru */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_RU, "Use an external palette (file)"},
	/* sv */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_SV, "Anv‰nd en extern palett (fil)"},
	/* tr */ {IDCLS_USE_EXTERNAL_FILE_PALETTE_TR, "Harici bir palet (dosyadan) kullan"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME,    N_("Specify name of file of external palette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_DA, "Angiv filnavn for ekstern palette"},
	/* de */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_DE, "Dateiname f¸r externe Palette w‰hlen"},
	/* es */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_ES, "Especificar nombre fichero de la paleta externa"},
	/* fr */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_FR, "SpÈcifier le nom du fichier de la palette externe"},
	/* hu */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_HU, "Adja meg a k¸lsı paletta nevÈt"},
	/* it */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_IT, "Specifica il nome del file della palette esterna"},
	/* ko */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_NL, "Geef de naam van het externe kleurenpaletbestand"},
	/* pl */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_PL, "Okre∂l nazwÍ pliku zewnÍtrznej palety barw"},
	/* ru */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_RU, "Specify name of file of external palette"},
	/* sv */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_SV, "Ange namn pÂ fil fˆr extern palett"},
	/* tr */ {IDCLS_SPECIFY_EXTERNAL_PALETTE_NAME_TR, "Harici palet dosyas˝n˝n ismini belirt"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_P_DEVICE,    N_("<device>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_DEVICE_DA, "<enhed>"},
	/* de */ {IDCLS_P_DEVICE_DE, "<Ger‰t>"},
	/* es */ {IDCLS_P_DEVICE_ES, "<perifÈrico>"},
	/* fr */ {IDCLS_P_DEVICE_FR, "<pÈriphÈrique>"},
	/* hu */ {IDCLS_P_DEVICE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_DEVICE_IT, "<dispositivo>"},
	/* ko */ {IDCLS_P_DEVICE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_DEVICE_NL, "<apparaat>"},
	/* pl */ {IDCLS_P_DEVICE_PL, "<device>"},
	/* ru */ {IDCLS_P_DEVICE_RU, "<device>"},
	/* sv */ {IDCLS_P_DEVICE_SV, "<enhet>"},
	/* tr */ {IDCLS_P_DEVICE_TR, "<ayg˝t>"},
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_SELECT_FULLSCREEN_DEVICE,    N_("Select fullscreen device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_FULLSCREEN_DEVICE_DA, "Angiv fuldskÊrmsenhed"},
	/* de */ {IDCLS_SELECT_FULLSCREEN_DEVICE_DE, "Vollbild Ger‰t ausw‰hlen"},
	/* es */ {IDCLS_SELECT_FULLSCREEN_DEVICE_ES, "Seleccionar perifÈrico pantalla completa"},
	/* fr */ {IDCLS_SELECT_FULLSCREEN_DEVICE_FR, "SÈlectionner le pÈriphÈrique plein Ècran"},
	/* hu */ {IDCLS_SELECT_FULLSCREEN_DEVICE_HU, "TeljeskÈpernyıs eszkˆz kiv·laszt·sa"},
	/* it */ {IDCLS_SELECT_FULLSCREEN_DEVICE_IT, "Seleziona il dispositivo per la visualizzazione a schermo intero"},
	/* ko */ {IDCLS_SELECT_FULLSCREEN_DEVICE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_FULLSCREEN_DEVICE_NL, "Selecteer volledig schermapparaat"},
	/* pl */ {IDCLS_SELECT_FULLSCREEN_DEVICE_PL, "Wybierz urz±dzenie pe≥nego ekranu"},
	/* ru */ {IDCLS_SELECT_FULLSCREEN_DEVICE_RU, "Select fullscreen device"},
	/* sv */ {IDCLS_SELECT_FULLSCREEN_DEVICE_SV, "Ange fullsk‰rmsenhet"},
	/* tr */ {IDCLS_SELECT_FULLSCREEN_DEVICE_TR, "Tam ekran ayg˝t˝ seÁ"},
#endif
	
	/* video/video-cmdline-options.c, autostart.c */
	/* en */ {IDCLS_P_MODE,    N_("<Mode>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_MODE_DA, "<Tilstand>"},
	/* de */ {IDCLS_P_MODE_DE, "<Modus>"},
	/* es */ {IDCLS_P_MODE_ES, "<Modo>"},
	/* fr */ {IDCLS_P_MODE_FR, "<Mode>"},
	/* hu */ {IDCLS_P_MODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_MODE_IT, "<Modalit‡>"},
	/* ko */ {IDCLS_P_MODE_KO, "<∏µÂ>"},
	/* nl */ {IDCLS_P_MODE_NL, "<Modus>"},
	/* pl */ {IDCLS_P_MODE_PL, "<Tryb>"},
	/* ru */ {IDCLS_P_MODE_RU, "<Mode>"},
	/* sv */ {IDCLS_P_MODE_SV, "<L‰ge>"},
	/* tr */ {IDCLS_P_MODE_TR, "<Mod>"},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_P_SECONDS,    N_("<Seconds>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_SECONDS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_SECONDS_DE, "<Sekunden>"},
	/* es */ {IDCLS_P_SECONDS_ES, "<Segundos>"},
	/* fr */ {IDCLS_P_SECONDS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_SECONDS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_SECONDS_IT, "<Secondi>"},
	/* ko */ {IDCLS_P_SECONDS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_SECONDS_NL, "<Seconden>"},
	/* pl */ {IDCLS_P_SECONDS_PL, "<sekund>"},
	/* ru */ {IDCLS_P_SECONDS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_SECONDS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_SECONDS_TR, ""},  /* fuzzy */
#endif
	
	/* video/video-cmdline-options.c */
	/* en */ {IDCLS_SELECT_FULLSCREEN_MODE,    N_("Select fullscreen mode")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_FULLSCREEN_MODE_DA, "AktivÈr fuldskÊrms-tilstand"},
	/* de */ {IDCLS_SELECT_FULLSCREEN_MODE_DE, "Vollbild Modus w‰hlen"},
	/* es */ {IDCLS_SELECT_FULLSCREEN_MODE_ES, "Seleccionar modo pantalla completa"},
	/* fr */ {IDCLS_SELECT_FULLSCREEN_MODE_FR, "SÈlectionner le mode plein Ècran"},
	/* hu */ {IDCLS_SELECT_FULLSCREEN_MODE_HU, "Teljes kÈpernyıs mÛd kiv·laszt·sa"},
	/* it */ {IDCLS_SELECT_FULLSCREEN_MODE_IT, "Seleziona modalit‡ a schermo intero"},
	/* ko */ {IDCLS_SELECT_FULLSCREEN_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_FULLSCREEN_MODE_NL, "Volledig scherm-modus inschakelen"},
	/* pl */ {IDCLS_SELECT_FULLSCREEN_MODE_PL, "Wybierz tryb pe≥nego ekranu"},
	/* ru */ {IDCLS_SELECT_FULLSCREEN_MODE_RU, "Select fullscreen mode"},
	/* sv */ {IDCLS_SELECT_FULLSCREEN_MODE_SV, "Aktivera fullsk‰rmsl‰ge"},
	/* tr */ {IDCLS_SELECT_FULLSCREEN_MODE_TR, "Tam ekran modu seÁ"},
#endif
	
	/* aciacore.c */
	/* en */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE,    N_("Specify RS232 device this ACIA should work on")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_DA, "Angiv RS232-enhed denne ACIA skal bruge"},
	/* de */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_DE, "RS232 Ger‰t f¸r welches ACIA funktionieren soll ist zu spezifizieren"},
	/* es */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_ES, "Especificar perifÈrico RS232 con el que funcionar· esta ACIA"},
	/* fr */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_FR, "SpÈcifier le pÈriphÈrique RS232 sur lequel cet ACIA doit fonctionner"},
	/* hu */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_HU, "Adja meg az RS232-es egysÈget, amivel az ACIA m˚kˆdni fog"},
	/* it */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_IT, "Specifica il dispositivo RS232 con il quale questa ACIA dovrebbe funzionare"},
	/* ko */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_NL, "Geef het RS232 apparaat waarmee deze ACIA moet werken"},
	/* pl */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_PL, "Okre∂l urz±dzenie RS232, na ktÛrym ma dzia≥aÊ ACIA"},
	/* ru */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_RU, "Specify RS232 device this ACIA should work on"},
	/* sv */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_SV, "Ange RS232-enhet denna ACIA skall arbeta pÂ"},
	/* tr */ {IDCLS_SPECIFY_ACIA_RS232_DEVICE_TR, "Bu ACIA'in Áal˝˛mas˝ gereken RS232 ayg˝t˝n˝ belirt"},
#endif
	
	/* attach.c */
	/* en */ {IDCLS_SET_DEVICE_TYPE_8,    N_("Set device type for device #8 (0: None, 1: Filesystem, 2: OpenCBM, 3: Block device)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DEVICE_TYPE_8_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DEVICE_TYPE_8_DE, "Ger‰tetyp f¸r Ger‰t #8 (0: Kein, 1: Dateisystem, 2: OpenCBM, 3: Block Ger‰t)"},
	/* es */ {IDCLS_SET_DEVICE_TYPE_8_ES, "Seleccionar tipo para perifÈrico #8  (0: Ninguno, 1: Filesystem, 2: OpenCBM, 3: Block device)"},
	/* fr */ {IDCLS_SET_DEVICE_TYPE_8_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DEVICE_TYPE_8_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DEVICE_TYPE_8_IT, "Imposta il tipo di periferica #8 (0: Nessuna, 1: Filesystem, 2: OpenCBM, 3: Dispositivo a blocchi)"},
	/* ko */ {IDCLS_SET_DEVICE_TYPE_8_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DEVICE_TYPE_8_NL, "Stel het apparaatsoort in voor apparaat #8 (0: Geen, 1: Bestandssysteem, 2: OpenCBM, 3: Blok apparaat)"},
	/* pl */ {IDCLS_SET_DEVICE_TYPE_8_PL, "Okre∂l typ urz±dzenia #8 (0: Brak, 1: System plikÛw, 2: OpenCBM, 3: Urz±dzenie blokowe)"},
	/* ru */ {IDCLS_SET_DEVICE_TYPE_8_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DEVICE_TYPE_8_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DEVICE_TYPE_8_TR, ""},  /* fuzzy */
#endif
	
	/* attach.c */
	/* en */ {IDCLS_SET_DEVICE_TYPE_9,    N_("Set device type for device #9 (0: None, 1: Filesystem, 2: OpenCBM, 3: Block device)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DEVICE_TYPE_9_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DEVICE_TYPE_9_DE, "Ger‰tetyp f¸r Ger‰t #9 (0: Kein, 1: Dateisystem, 2: OpenCBM, 3: Block Ger‰t)"},
	/* es */ {IDCLS_SET_DEVICE_TYPE_9_ES, "Seleccionar tipo para perifÈrico #9  (0: Ninguno, 1: Filesystem, 2: OpenCBM, 3: Block device)"},
	/* fr */ {IDCLS_SET_DEVICE_TYPE_9_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DEVICE_TYPE_9_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DEVICE_TYPE_9_IT, "Imposta il tipo di periferica #9 (0: Nessuna, 1: Filesystem, 2: OpenCBM, 3: Dispositivo a blocchi)"},
	/* ko */ {IDCLS_SET_DEVICE_TYPE_9_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DEVICE_TYPE_9_NL, "Stel het apparaatsoort in voor apparaat #9 (0: Geen, 1: Bestandssysteem, 2: OpenCBM, 3: Blok apparaat)"},
	/* pl */ {IDCLS_SET_DEVICE_TYPE_9_PL, "Okre∂l typ urz±dzenia #8 (0: Brak, 1: System plikÛw, 2: OpenCBM, 3: Urz±dzenie blokowe)"},
	/* ru */ {IDCLS_SET_DEVICE_TYPE_9_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DEVICE_TYPE_9_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DEVICE_TYPE_9_TR, ""},  /* fuzzy */
#endif
	
	/* attach.c */
	/* en */ {IDCLS_SET_DEVICE_TYPE_10,    N_("Set device type for device #10 (0: None, 1: Filesystem, 2: OpenCBM, 3: Block device)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DEVICE_TYPE_10_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DEVICE_TYPE_10_DE, "Ger‰tetyp f¸r Ger‰t #10 (0: Kein, 1: Dateisystem, 2: OpenCBM, 3: Block Ger‰t)"},
	/* es */ {IDCLS_SET_DEVICE_TYPE_10_ES, "Seleccionar tipo para perifÈrico #10  (0: Ninguno, 1: Filesystem, 2: OpenCBM, 3: Block device)"},
	/* fr */ {IDCLS_SET_DEVICE_TYPE_10_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DEVICE_TYPE_10_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DEVICE_TYPE_10_IT, "Imposta il tipo di periferica #10 (0: Nessuna, 1: Filesystem, 2: OpenCBM, 3: Dispositivo a blocchi)"},
	/* ko */ {IDCLS_SET_DEVICE_TYPE_10_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DEVICE_TYPE_10_NL, "Stel het apparaatsoort in voor apparaat #10 (0: Geen, 1: Bestandssysteem, 2: OpenCBM, 3: Blok apparaat)"},
	/* pl */ {IDCLS_SET_DEVICE_TYPE_10_PL, "Okre∂l typ urz±dzenia #8 (0: Brak, 1: System plikÛw, 2: OpenCBM, 3: Urz±dzenie blokowe)"},
	/* ru */ {IDCLS_SET_DEVICE_TYPE_10_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DEVICE_TYPE_10_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DEVICE_TYPE_10_TR, ""},  /* fuzzy */
#endif
	
	/* attach.c */
	/* en */ {IDCLS_SET_DEVICE_TYPE_11,    N_("Set device type for device #11 (0: None, 1: Filesystem, 2: OpenCBM, 3: Block device)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_DEVICE_TYPE_11_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_DEVICE_TYPE_11_DE, "Ger‰tetyp f¸r Ger‰t #11 (0: Kein, 1: Dateisystem, 2: OpenCBM, 3: Block Ger‰t)"},
	/* es */ {IDCLS_SET_DEVICE_TYPE_11_ES, "Seleccionar tipo para perifÈrico #11  (0: Ninguno, 1: Filesystem, 2: OpenCBM, 3: Block device)"},
	/* fr */ {IDCLS_SET_DEVICE_TYPE_11_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_DEVICE_TYPE_11_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_DEVICE_TYPE_11_IT, "Imposta il tipo di periferica #11 (0: Nessuna, 1: Filesystem, 2: OpenCBM, 3: Dispositivo a blocchi)"},
	/* ko */ {IDCLS_SET_DEVICE_TYPE_11_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_DEVICE_TYPE_11_NL, "Stel het apparaatsoort in voor apparaat #11 (0: Geen, 1: Bestandssysteem, 2: OpenCBM, 3: Blok apparaat)"},
	/* pl */ {IDCLS_SET_DEVICE_TYPE_11_PL, "Okre∂l typ urz±dzenia #8 (0: Brak, 1: System plikÛw, 2: OpenCBM, 3: Urz±dzenie blokowe)"},
	/* ru */ {IDCLS_SET_DEVICE_TYPE_11_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_DEVICE_TYPE_11_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_DEVICE_TYPE_11_TR, ""},  /* fuzzy */
#endif
	
	/* attach.c */
	/* en */ {IDCLS_ATTACH_READ_ONLY_8,    N_("Attach disk image for drive #8 read only")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_READ_ONLY_8_DA, "Tilslut disk-image til drev #8 skrivebeskyttet"},
	/* de */ {IDCLS_ATTACH_READ_ONLY_8_DE, "Disk Image als Laufwerk #8 benutzen (schreibgesch¸tzt)"},
	/* es */ {IDCLS_ATTACH_READ_ONLY_8_ES, "Insertar imagen disco sÛlo lectura para unidad disco #8"},
	/* fr */ {IDCLS_ATTACH_READ_ONLY_8_FR, "Charger une image de disque pour le lecteur #8 en lecture seule"},
	/* hu */ {IDCLS_ATTACH_READ_ONLY_8_HU, "Csak olvashatÛ kÈpm·s csatol·sa a #8-as lemezegysÈghez"},
	/* it */ {IDCLS_ATTACH_READ_ONLY_8_IT, "Seleziona immagine del disco per il drive #8 in sola lettura"},
	/* ko */ {IDCLS_ATTACH_READ_ONLY_8_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_READ_ONLY_8_NL, "Koppel schijfbestand voor drive #8 als alleen-lezen"},
	/* pl */ {IDCLS_ATTACH_READ_ONLY_8_PL, "Zamontuj obraz dysku w stacji #8 tylko do odczytu"},
	/* ru */ {IDCLS_ATTACH_READ_ONLY_8_RU, "Attach disk image for drive #8 read only"},
	/* sv */ {IDCLS_ATTACH_READ_ONLY_8_SV, "Anslut diskettavbildningsfil fˆr enhet 8 med skrivskydd"},
	/* tr */ {IDCLS_ATTACH_READ_ONLY_8_TR, "S¸r¸c¸ #8 iÁin disk imaj˝n˝ salt okunur olarak yerle˛tir"},
#endif
	
	/* attach.c */
	/* en */ {IDCLS_ATTACH_READ_WRITE_8,    N_("Attach disk image for drive #8 read write (if possible)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_READ_WRITE_8_DA, "Tilslut disk-image til drev #8 skrivbar (hvis muligt)"},
	/* de */ {IDCLS_ATTACH_READ_WRITE_8_DE, "Disk Image als Laufwerk #8 benutzen (Schreibzugriff, wenn mˆglich)"},
	/* es */ {IDCLS_ATTACH_READ_WRITE_8_ES, "Insertar imagen disco sÛlo lectura/grabaciÛn (si es posible) para unidad disco #8"},
	/* fr */ {IDCLS_ATTACH_READ_WRITE_8_FR, "Charger une image de disque pour le lecteur #8 en lecture-Ècriture (si possible)"},
	/* hu */ {IDCLS_ATTACH_READ_WRITE_8_HU, "ÕrhatÛ-olvashatÛ kÈpm·s csatol·sa a #8-as lemezegysÈghez"},
	/* it */ {IDCLS_ATTACH_READ_WRITE_8_IT, "Seleziona immagine del disco per il drive #8 in lettura/scrittura (se possibile)"},
	/* ko */ {IDCLS_ATTACH_READ_WRITE_8_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_READ_WRITE_8_NL, "Koppel schijfbestand voor drive #8 als schrijfbaar (indien mogelijk)"},
	/* pl */ {IDCLS_ATTACH_READ_WRITE_8_PL, "Zamontuj obraz dyskietki w stacji #8 w trybie zapisu i odczytu (o ile to moøliwe)"},
	/* ru */ {IDCLS_ATTACH_READ_WRITE_8_RU, "Attach disk image for drive #8 read write (if possible)"},
	/* sv */ {IDCLS_ATTACH_READ_WRITE_8_SV, "Anslut diskettavbildningsfil fˆr enhet 8 skrivbar (om mˆjligt)"},
	/* tr */ {IDCLS_ATTACH_READ_WRITE_8_TR, "S¸r¸c¸ #8 iÁin disk imaj˝n˝ okunur/yaz˝l˝r olarak yerle˛tir (m¸mk¸nse)"},
#endif
	
	/* attach.c */
	/* en */ {IDCLS_ATTACH_READ_ONLY_9,    N_("Attach disk image for drive #9 read only")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_READ_ONLY_9_DA, "Tilslut disk-image til drev #9 skrivebeskyttet"},
	/* de */ {IDCLS_ATTACH_READ_ONLY_9_DE, "Disk Image als Laufwerk #9 benutzen (schreibgesch¸tzt)"},
	/* es */ {IDCLS_ATTACH_READ_ONLY_9_ES, "Insertar imagen disco sÛlo lectura para unidad disco #9"},
	/* fr */ {IDCLS_ATTACH_READ_ONLY_9_FR, "Charger une image de disque pour le lecteur #9 en lecture seule"},
	/* hu */ {IDCLS_ATTACH_READ_ONLY_9_HU, "Csak olvashatÛ kÈpm·s csatol·sa a #9-es lemezegysÈghez"},
	/* it */ {IDCLS_ATTACH_READ_ONLY_9_IT, "Seleziona immagine del disco per il drive #9 in sola lettura"},
	/* ko */ {IDCLS_ATTACH_READ_ONLY_9_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_READ_ONLY_9_NL, "Koppel schijfbestand voor drive #9 als alleen-lezen"},
	/* pl */ {IDCLS_ATTACH_READ_ONLY_9_PL, "Zamontuj obraz dysku w stacji #9 tylko do odczytu"},
	/* ru */ {IDCLS_ATTACH_READ_ONLY_9_RU, "Attach disk image for drive #9 read only"},
	/* sv */ {IDCLS_ATTACH_READ_ONLY_9_SV, "Anslut diskettavbildningsfil fˆr enhet 9 med skrivskydd"},
	/* tr */ {IDCLS_ATTACH_READ_ONLY_9_TR, "S¸r¸c¸ #9 iÁin disk imaj˝n˝ salt okunur olarak yerle˛tir"},
#endif
	
	/* attach.c */
	/* en */ {IDCLS_ATTACH_READ_WRITE_9,    N_("Attach disk image for drive #9 read write (if possible)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_READ_WRITE_9_DA, "Tilslut disk-image til drev #9 skrivbar (hvis muligt)"},
	/* de */ {IDCLS_ATTACH_READ_WRITE_9_DE, "Disk Image als Laufwerk #9 benutzen (Schreibzugriff, wenn mˆglich)"},
	/* es */ {IDCLS_ATTACH_READ_WRITE_9_ES, "Insertar imagen disco sÛlo lectura/grabaciÛn (si es posible) para unidad disco #9"},
	/* fr */ {IDCLS_ATTACH_READ_WRITE_9_FR, "Charger une image de disque pour le lecteur #9 en lecture-Ècriture (si possible)"},
	/* hu */ {IDCLS_ATTACH_READ_WRITE_9_HU, "ÕrhatÛ-olvashatÛ kÈpm·s csatol·sa a #9-es lemezegysÈghez"},
	/* it */ {IDCLS_ATTACH_READ_WRITE_9_IT, "Seleziona immagine del disco per il drive #9 in lettura/scrittura (se possibile)"},
	/* ko */ {IDCLS_ATTACH_READ_WRITE_9_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_READ_WRITE_9_NL, "Koppel schijfbestand voor drive #9 als schrijfbaar (indien mogelijk)"},
	/* pl */ {IDCLS_ATTACH_READ_WRITE_9_PL, "Zamontuj obraz dyskietki w stacji #9 w trybie zapisu i odczytu (o ile to moøliwe)"},
	/* ru */ {IDCLS_ATTACH_READ_WRITE_9_RU, "Attach disk image for drive #9 read write (if possible)"},
	/* sv */ {IDCLS_ATTACH_READ_WRITE_9_SV, "Anslut diskettavbildningsfil fˆr enhet 9 skrivbar (om mˆjligt)"},
	/* tr */ {IDCLS_ATTACH_READ_WRITE_9_TR, "S¸r¸c¸ #9 iÁin disk imaj˝n˝ okunur/yaz˝l˝r olarak yerle˛tir (m¸mk¸nse)"},
#endif
	
	/* attach.c */
	/* en */ {IDCLS_ATTACH_READ_ONLY_10,    N_("Attach disk image for drive #10 read only")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_READ_ONLY_10_DA, "Tilslut disk-image til drev #10 skrivebeskyttet"},
	/* de */ {IDCLS_ATTACH_READ_ONLY_10_DE, "Disk Image als Laufwerk #10 benutzen (schreibgesch¸tzt)"},
	/* es */ {IDCLS_ATTACH_READ_ONLY_10_ES, "Insertar imagen disco sÛlo lectura para unidad disco #10"},
	/* fr */ {IDCLS_ATTACH_READ_ONLY_10_FR, "Charger une image de disque pour le lecteur #10 en lecture seule"},
	/* hu */ {IDCLS_ATTACH_READ_ONLY_10_HU, "Csak olvashatÛ kÈpm·s csatol·sa a #10-es lemezegysÈghez"},
	/* it */ {IDCLS_ATTACH_READ_ONLY_10_IT, "Seleziona immagine del disco per il drive #10 in sola lettura"},
	/* ko */ {IDCLS_ATTACH_READ_ONLY_10_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_READ_ONLY_10_NL, "Koppel schijfbestand voor drive #10 als alleen-lezen"},
	/* pl */ {IDCLS_ATTACH_READ_ONLY_10_PL, "Zamontuj obraz dysku w stacji #10 tylko do odczytu"},
	/* ru */ {IDCLS_ATTACH_READ_ONLY_10_RU, "Attach disk image for drive #10 read only"},
	/* sv */ {IDCLS_ATTACH_READ_ONLY_10_SV, "Anslut diskettavbildningsfil fˆr enhet 10 med skrivskydd"},
	/* tr */ {IDCLS_ATTACH_READ_ONLY_10_TR, "S¸r¸c¸ #10 iÁin disk imaj˝n˝ salt okunur olarak yerle˛tir"},
#endif
	
	/* attach.c */
	/* en */ {IDCLS_ATTACH_READ_WRITE_10,    N_("Attach disk image for drive #10 read write (if possible)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_READ_WRITE_10_DA, "Tilslut disk-image til drev #10 skrivbar (hvis muligt)"},
	/* de */ {IDCLS_ATTACH_READ_WRITE_10_DE, "Disk Image als Laufwerk #10 benutzen (Schreibzugriff, wenn mˆglich)"},
	/* es */ {IDCLS_ATTACH_READ_WRITE_10_ES, "Insertar imagen disco sÛlo lectura/grabaciÛn (si es posible) para unidad disco #10"},
	/* fr */ {IDCLS_ATTACH_READ_WRITE_10_FR, "Charger une image de disque pour le lecteur #10 en lecture-Ècriture (si possible)"},
	/* hu */ {IDCLS_ATTACH_READ_WRITE_10_HU, "ÕrhatÛ-olvashatÛ kÈpm·s csatol·sa a #10-es lemezegysÈghez"},
	/* it */ {IDCLS_ATTACH_READ_WRITE_10_IT, "Seleziona immagine del disco per il drive #10 in lettura/scrittura (se possibile)"},
	/* ko */ {IDCLS_ATTACH_READ_WRITE_10_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_READ_WRITE_10_NL, "Koppel schijfbestand voor drive #10 als schrijfbaar (indien mogelijk)"},
	/* pl */ {IDCLS_ATTACH_READ_WRITE_10_PL, "Zamontuj obraz dyskietki w stacji #10 w trybie zapisu i odczytu (o ile to moøliwe)"},
	/* ru */ {IDCLS_ATTACH_READ_WRITE_10_RU, "Attach disk image for drive #10 read write (if possible)"},
	/* sv */ {IDCLS_ATTACH_READ_WRITE_10_SV, "Anslut diskettavbildningsfil fˆr enhet 10 skrivbar (om mˆjligt)"},
	/* tr */ {IDCLS_ATTACH_READ_WRITE_10_TR, "S¸r¸c¸ #10 iÁin disk imaj˝n˝ okunur/yaz˝l˝r olarak yerle˛tir (m¸mk¸nse)"},
#endif
	
	/* attach.c */
	/* en */ {IDCLS_ATTACH_READ_ONLY_11,    N_("Attach disk image for drive #11 read only")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_READ_ONLY_11_DA, "Tilslut disk-image til drev #11 skrivebeskyttet"},
	/* de */ {IDCLS_ATTACH_READ_ONLY_11_DE, "Disk Image als Laufwerk #11 benutzen (schreibgesch¸tzt)"},
	/* es */ {IDCLS_ATTACH_READ_ONLY_11_ES, "Insertar imagen disco sÛlo lectura para unidad disco #11"},
	/* fr */ {IDCLS_ATTACH_READ_ONLY_11_FR, "Charger une image de disque pour le lecteur #11 en lecture seule"},
	/* hu */ {IDCLS_ATTACH_READ_ONLY_11_HU, "Csak olvashatÛ kÈpm·s csatol·sa a #11-es lemezegysÈghez"},
	/* it */ {IDCLS_ATTACH_READ_ONLY_11_IT, "Seleziona immagine del disco per il drive #11 in sola lettura"},
	/* ko */ {IDCLS_ATTACH_READ_ONLY_11_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_READ_ONLY_11_NL, "Koppel schijfbestand voor drive #11 als alleen-lezen"},
	/* pl */ {IDCLS_ATTACH_READ_ONLY_11_PL, "Zamontuj obraz dysku w stacji #11 tylko do odczytu"},
	/* ru */ {IDCLS_ATTACH_READ_ONLY_11_RU, "Attach disk image for drive #11 read only"},
	/* sv */ {IDCLS_ATTACH_READ_ONLY_11_SV, "Anslut diskettavbildningsfil fˆr enhet 11 med skrivskydd"},
	/* tr */ {IDCLS_ATTACH_READ_ONLY_11_TR, "S¸r¸c¸ #11 iÁin disk imaj˝n˝ salt okunur olarak yerle˛tir"},
#endif
	
	/* attach.c */
	/* en */ {IDCLS_ATTACH_READ_WRITE_11,    N_("Attach disk image for drive #11 read write (if possible)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_READ_WRITE_11_DA, "Tilslut disk-image til drev #11 skrivbar (hvis muligt)"},
	/* de */ {IDCLS_ATTACH_READ_WRITE_11_DE, "Disk Image als Laufwerk #11 benutzen (Schreibzugriff, wenn mˆglich)"},
	/* es */ {IDCLS_ATTACH_READ_WRITE_11_ES, "Insertar imagen disco sÛlo lectura/grabaciÛn (si es posible) para unidad disco #11"},
	/* fr */ {IDCLS_ATTACH_READ_WRITE_11_FR, "Charger une image de disque pour le lecteur #11 en lecture-Ècriture (si possible)"},
	/* hu */ {IDCLS_ATTACH_READ_WRITE_11_HU, "ÕrhatÛ-olvashatÛ kÈpm·s csatol·sa a #11-es lemezegysÈghez"},
	/* it */ {IDCLS_ATTACH_READ_WRITE_11_IT, "Seleziona immagine del disco per il drive #11 in lettura/scrittura (se possibile)"},
	/* ko */ {IDCLS_ATTACH_READ_WRITE_11_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_READ_WRITE_11_NL, "Koppel schijfbestand voor drive #11 als schrijfbaar (indien mogelijk)"},
	/* pl */ {IDCLS_ATTACH_READ_WRITE_11_PL, "Zamontuj obraz dyskietki w stacji #11 w trybie zapisu i odczytu (o ile to moøliwe)"},
	/* ru */ {IDCLS_ATTACH_READ_WRITE_11_RU, "Attach disk image for drive #11 read write (if possible)"},
	/* sv */ {IDCLS_ATTACH_READ_WRITE_11_SV, "Anslut diskettavbildningsfil fˆr enhet 11 skrivbar (om mˆjligt)"},
	/* tr */ {IDCLS_ATTACH_READ_WRITE_11_TR, "S¸r¸c¸ #11 iÁin disk imaj˝n˝ okunur/yaz˝l˝r olarak yerle˛tir (m¸mk¸nse)"},
#endif
	
	/* datasette.c */
	/* en */ {IDCLS_ENABLE_DATASETTE,    N_("Enable Datasette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DATASETTE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_DATASETTE_DE, "Datasette aktivieren"},
	/* es */ {IDCLS_ENABLE_DATASETTE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_DATASETTE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_DATASETTE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DATASETTE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_DATASETTE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DATASETTE_NL, "Datasette inschakelen"},
	/* pl */ {IDCLS_ENABLE_DATASETTE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_DATASETTE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_DATASETTE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_DATASETTE_TR, ""},  /* fuzzy */
#endif
	
	/* datasette.c */
	/* en */ {IDCLS_DISABLE_DATASETTE,    N_("Disable Datasette")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DATASETTE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_DATASETTE_DE, "Datasette deaktivieren"},
	/* es */ {IDCLS_DISABLE_DATASETTE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_DATASETTE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_DATASETTE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DATASETTE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_DATASETTE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DATASETTE_NL, "Datasette uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DATASETTE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_DATASETTE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_DATASETTE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_DATASETTE_TR, ""},  /* fuzzy */
#endif
	
	/* datasette.c */
	/* en */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET,    N_("Enable automatic Datasette-Reset")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_DA, "AktivÈr automatisk Datasette-reset"},
	/* de */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_DE, "Automatisches Datasette-Reset aktivieren"},
	/* es */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_ES, "Permitir reinicio autom·tico de la unidad de cinta"},
	/* fr */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_FR, "Activer le redÈmarrage automatique du Datasette"},
	/* hu */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_HU, "Automatikus magnÛ Reset engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_IT, "Attiva reset automatico del registratore"},
	/* ko */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_NL, "Automatische Datasette-Reset inschakelen"},
	/* pl */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_PL, "W≥±cz automatyczny reset magnetofonu"},
	/* ru */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_RU, "Enable automatic Datasette-Reset"},
	/* sv */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_SV, "Aktivera automatisk DatasetteÂterst‰llning"},
	/* tr */ {IDCLS_ENABLE_AUTO_DATASETTE_RESET_TR, "Otomatik Teyp-Reset'i aktif et"},
#endif
	
	/* datasette.c */
	/* en */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET,    N_("Disable automatic Datasette-Reset")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_DA, "DeaktivÈr automatisk Datasette-reset"},
	/* de */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_DE, "Automatisches Datasette-Reset deaktivieren"},
	/* es */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_ES, "Deshabilitar reinicio autom·tico de la unidad de cinta"},
	/* fr */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_FR, "DÈsactiver le redÈmarrage automatique du Datasette"},
	/* hu */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_HU, "Automatikus magnÛ Reset tilt·sa"},
	/* it */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_IT, "Disattiva reset automatico del registratore"},
	/* ko */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_NL, "Automatische Datasette-Reset uitschakelen"},
	/* pl */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_PL, "Wy≥±cz automatyczny reset magnetofonu"},
	/* ru */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_RU, "Disable automatic Datasette-Reset"},
	/* sv */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_SV, "Inaktivera automatisk DatasetteÂterst‰llning"},
	/* tr */ {IDCLS_DISABLE_AUTO_DATASETTE_RESET_TR, "Otomatik Teyp-Reset'i pasifle˛tir"},
#endif
	
	/* datasette.c */
	/* en */ {IDCLS_SET_ZERO_TAP_DELAY,    N_("Set delay in cycles for a zero in the tap")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_ZERO_TAP_DELAY_DA, "Angiv forsinkelse i cykler for et nul i tap-filen"},
	/* de */ {IDCLS_SET_ZERO_TAP_DELAY_DE, "Verzˆgerung f¸r Zero in Zyklen im Tap"},
	/* es */ {IDCLS_SET_ZERO_TAP_DELAY_ES, "Seleccionar retardo en ciclos para cero en la cinta"},
	/* fr */ {IDCLS_SET_ZERO_TAP_DELAY_FR, "SpÈcifier le dÈlai en cycles pour un zÈro sur le ruban"},
	/* hu */ {IDCLS_SET_ZERO_TAP_DELAY_HU, "Adja meg ciklusokban a legelsı szalagrÈs hossz·t"},
	/* it */ {IDCLS_SET_ZERO_TAP_DELAY_IT, "Imposta il ritardo in cicli per uno zero nel tap"},
	/* ko */ {IDCLS_SET_ZERO_TAP_DELAY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_ZERO_TAP_DELAY_NL, "Stel de vertraging in cycli in voor een nul in de tap"},
	/* pl */ {IDCLS_SET_ZERO_TAP_DELAY_PL, "Okre∂l opÛºnienie cyklÛw dla pocz±tku ta∂my"},
	/* ru */ {IDCLS_SET_ZERO_TAP_DELAY_RU, "Set delay in cycles for a zero in the tap"},
	/* sv */ {IDCLS_SET_ZERO_TAP_DELAY_SV, "Ange v‰ntecykler fˆr nolla i tap-filen"},
	/* tr */ {IDCLS_SET_ZERO_TAP_DELAY_TR, "Balant˝ an˝nda cycle tipinden bekleme s¸resini ayarla"},
#endif
	
	/* datasette.c */
	/* en */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP,    N_("Set number of cycles added to each gap in the tap")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_DA, "Angiv antal cykler der lÊgges til hvert hul i tap-filen"},
	/* de */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_DE, "Setze Anzahl der Zyklen f¸r jedes Loch im Tap"},
	/* es */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_ES, "Seleccionar n˙mero ciclos aÒadidos en cada espacio en la cinta"},
	/* fr */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_FR, "SpÈcifier le nombre de cycles ajoutÈ a chaque raccord du ruban"},
	/* hu */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_HU, "Adja meg ciklusokban a szalagf·jlok kˆzti ¸res rÈs hossz·t"},
	/* it */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_IT, "Imposta il numero di cicli aggiunti ad ogni gap nel tap"},
	/* ko */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_NL, "Stel aantal extra cycli in voor elk gat in de tap"},
	/* pl */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_PL, "Okre∂l liczbÍ cyklÛw dodanych do kaødej przerwy w ta∂mie"},
	/* ru */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_RU, "Set number of cycles added to each gap in the tap"},
	/* sv */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_SV, "Ange cykler som l‰ggs till varje gap i tap-filen"},
	/* tr */ {IDCLS_SET_CYCLES_ADDED_GAP_TAP_TR, "Balant˝ an˝nda her bo˛luk iÁin eklenecek cycle say˝s˝n˝ ayarla"},
#endif
	
	/* datasette.c */
	/* en */ {IDCLS_SET_TAPE_WOBBLE,    N_("Set maximum random number of cycles added to each gap in the tap")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_TAPE_WOBBLE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_TAPE_WOBBLE_DE, "Setze maximal Zufallszahl der Zyklen f¸r jedes Loch im Tap"},
	/* es */ {IDCLS_SET_TAPE_WOBBLE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_TAPE_WOBBLE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_TAPE_WOBBLE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_TAPE_WOBBLE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_TAPE_WOBBLE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_TAPE_WOBBLE_NL, "Stel maximaal willekeurige extra aantal cycli in voor elk gat in de tap"},
	/* pl */ {IDCLS_SET_TAPE_WOBBLE_PL, "Okre∂l maksymaln± losow± liczbÍ cyklÛw dodanych do kaødej przerwy w ta∂mie"},
	/* ru */ {IDCLS_SET_TAPE_WOBBLE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_TAPE_WOBBLE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_TAPE_WOBBLE_TR, ""},  /* fuzzy */
#endif
	
#ifdef DEBUG
	/* debug.c */
	/* en */ {IDCLS_TRACE_MAIN_CPU,    N_("Trace the main CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TRACE_MAIN_CPU_DA, "Trace hovedprocessoren"},
	/* de */ {IDCLS_TRACE_MAIN_CPU_DE, "Haupt CPU verfolgen"},
	/* es */ {IDCLS_TRACE_MAIN_CPU_ES, "Rastrear CPU principal"},
	/* fr */ {IDCLS_TRACE_MAIN_CPU_FR, "Tracer l'UCT principal"},
	/* hu */ {IDCLS_TRACE_MAIN_CPU_HU, "Fı CPU nyomkˆvetÈse"},
	/* it */ {IDCLS_TRACE_MAIN_CPU_IT, "Traccia la CPU principale"},
	/* ko */ {IDCLS_TRACE_MAIN_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TRACE_MAIN_CPU_NL, "Traceer de hoofd CPU"},
	/* pl */ {IDCLS_TRACE_MAIN_CPU_PL, "¶ledzenie g≥Ûwnego CPU"},
	/* ru */ {IDCLS_TRACE_MAIN_CPU_RU, "Trace the main CPU"},
	/* sv */ {IDCLS_TRACE_MAIN_CPU_SV, "SpÂra huvudprocessorn"},
	/* tr */ {IDCLS_TRACE_MAIN_CPU_TR, "Ana CPU'yu izle"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_DONT_TRACE_MAIN_CPU,    N_("Do not trace the main CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DONT_TRACE_MAIN_CPU_DA, "Trace ikke hovedprocessoren"},
	/* de */ {IDCLS_DONT_TRACE_MAIN_CPU_DE, "Haupt CPU nicht verfolgen"},
	/* es */ {IDCLS_DONT_TRACE_MAIN_CPU_ES, "No rastrear CPU principal"},
	/* fr */ {IDCLS_DONT_TRACE_MAIN_CPU_FR, "Ne pas tracer l'UCT principal"},
	/* hu */ {IDCLS_DONT_TRACE_MAIN_CPU_HU, "Fı CPU nyomkˆvetÈsÈnek tilt·sa"},
	/* it */ {IDCLS_DONT_TRACE_MAIN_CPU_IT, "Non tracciare la CPU principale"},
	/* ko */ {IDCLS_DONT_TRACE_MAIN_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DONT_TRACE_MAIN_CPU_NL, "Traceer de hoofd CPU niet"},
	/* pl */ {IDCLS_DONT_TRACE_MAIN_CPU_PL, "Bez ∂ledzenia g≥Ûwnego CPU"},
	/* ru */ {IDCLS_DONT_TRACE_MAIN_CPU_RU, "Do not trace the main CPU"},
	/* sv */ {IDCLS_DONT_TRACE_MAIN_CPU_SV, "SpÂra inte huvudprocessorn"},
	/* tr */ {IDCLS_DONT_TRACE_MAIN_CPU_TR, "Ana CPU'yu izleme"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_TRACE_DRIVE0_CPU,    N_("Trace the drive0 CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TRACE_DRIVE0_CPU_DA, "Trace processor i drev 0"},
	/* de */ {IDCLS_TRACE_DRIVE0_CPU_DE, "Laufwerk0 CPU verfolgen"},
	/* es */ {IDCLS_TRACE_DRIVE0_CPU_ES, "Rastrear la CPU de la unidad de disco 0"},
	/* fr */ {IDCLS_TRACE_DRIVE0_CPU_FR, "Tracer l'UCT du drive0"},
	/* hu */ {IDCLS_TRACE_DRIVE0_CPU_HU, "A 0-s lemezegysÈg CPU-j·nak nyomkˆvetÈse"},
	/* it */ {IDCLS_TRACE_DRIVE0_CPU_IT, "Traccia la CPU del drive0"},
	/* ko */ {IDCLS_TRACE_DRIVE0_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TRACE_DRIVE0_CPU_NL, "Traceer de CPU van drive 0"},
	/* pl */ {IDCLS_TRACE_DRIVE0_CPU_PL, "¶ledzenie CPU stacji0"},
	/* ru */ {IDCLS_TRACE_DRIVE0_CPU_RU, "Trace the drive0 CPU"},
	/* sv */ {IDCLS_TRACE_DRIVE0_CPU_SV, "SpÂra processor i diskettstation 0"},
	/* tr */ {IDCLS_TRACE_DRIVE0_CPU_TR, "S¸r¸c¸0 CPU'sunu izle"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_DONT_TRACE_DRIVE0_CPU,    N_("Do not trace the drive0 CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DONT_TRACE_DRIVE0_CPU_DA, "Trace ikke processor i drev 0"},
	/* de */ {IDCLS_DONT_TRACE_DRIVE0_CPU_DE, "Laufwerk0 CPU nicht verfolgen"},
	/* es */ {IDCLS_DONT_TRACE_DRIVE0_CPU_ES, "No rastrear la CPU de la unidad de disco 0"},
	/* fr */ {IDCLS_DONT_TRACE_DRIVE0_CPU_FR, "Ne pas tracer l'UCT du drive0"},
	/* hu */ {IDCLS_DONT_TRACE_DRIVE0_CPU_HU, "A 0-s lemezegysÈg CPU-ja nyomkˆvetÈsÈnek tilt·sa"},
	/* it */ {IDCLS_DONT_TRACE_DRIVE0_CPU_IT, "Non tracciare la CPU del drive0"},
	/* ko */ {IDCLS_DONT_TRACE_DRIVE0_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DONT_TRACE_DRIVE0_CPU_NL, "Traceer de CPU van drive 0 niet"},
	/* pl */ {IDCLS_DONT_TRACE_DRIVE0_CPU_PL, "Bez ∂ledzenia CPU stacji0"},
	/* ru */ {IDCLS_DONT_TRACE_DRIVE0_CPU_RU, "Do not trace the drive0 CPU"},
	/* sv */ {IDCLS_DONT_TRACE_DRIVE0_CPU_SV, "SpÂra inte processor i diskettstation 0"},
	/* tr */ {IDCLS_DONT_TRACE_DRIVE0_CPU_TR, "S¸r¸c¸0 CPU'sunu izleme"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_TRACE_DRIVE1_CPU,    N_("Trace the drive1 CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TRACE_DRIVE1_CPU_DA, "Trace processor i drev 1"},
	/* de */ {IDCLS_TRACE_DRIVE1_CPU_DE, "Laufwerk1 CPU verfolgen"},
	/* es */ {IDCLS_TRACE_DRIVE1_CPU_ES, "Rastrear la CPU de la unidad de disco 1"},
	/* fr */ {IDCLS_TRACE_DRIVE1_CPU_FR, "Tracer l'UCT du drive1"},
	/* hu */ {IDCLS_TRACE_DRIVE1_CPU_HU, "Az 1-es lemezegysÈg CPU-j·nak nyomkˆvetÈse"},
	/* it */ {IDCLS_TRACE_DRIVE1_CPU_IT, "Traccia la CPU del drive1"},
	/* ko */ {IDCLS_TRACE_DRIVE1_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TRACE_DRIVE1_CPU_NL, "Traceer de CPU van drive 1"},
	/* pl */ {IDCLS_TRACE_DRIVE1_CPU_PL, "¶ledzenie CPU stacji1"},
	/* ru */ {IDCLS_TRACE_DRIVE1_CPU_RU, "Trace the drive1 CPU"},
	/* sv */ {IDCLS_TRACE_DRIVE1_CPU_SV, "SpÂra processor i diskettstation 1"},
	/* tr */ {IDCLS_TRACE_DRIVE1_CPU_TR, "S¸r¸c¸1 CPU'sunu izle"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_DONT_TRACE_DRIVE1_CPU,    N_("Do not trace the drive1 CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DONT_TRACE_DRIVE1_CPU_DA, "Trace ikke processor i drev 1"},
	/* de */ {IDCLS_DONT_TRACE_DRIVE1_CPU_DE, "Laufwerk1 CPU nicht verfolgen"},
	/* es */ {IDCLS_DONT_TRACE_DRIVE1_CPU_ES, "No rastrear la CPU de la unidad de disco 1"},
	/* fr */ {IDCLS_DONT_TRACE_DRIVE1_CPU_FR, "Ne pas tracer l'UCT du drive1"},
	/* hu */ {IDCLS_DONT_TRACE_DRIVE1_CPU_HU, "Az 1-es lemezegysÈg CPU-ja nyomkˆvetÈsÈnek tilt·sa"},
	/* it */ {IDCLS_DONT_TRACE_DRIVE1_CPU_IT, "Non tracciare la CPU del drive1"},
	/* ko */ {IDCLS_DONT_TRACE_DRIVE1_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DONT_TRACE_DRIVE1_CPU_NL, "Traceer de CPU van drive 1 niet"},
	/* pl */ {IDCLS_DONT_TRACE_DRIVE1_CPU_PL, "Bez ∂ledzenia CPU stacji1"},
	/* ru */ {IDCLS_DONT_TRACE_DRIVE1_CPU_RU, "Do not trace the drive1 CPU"},
	/* sv */ {IDCLS_DONT_TRACE_DRIVE1_CPU_SV, "SpÂra inte processor i diskettstation 1"},
	/* tr */ {IDCLS_DONT_TRACE_DRIVE1_CPU_TR, "S¸r¸c¸1 CPU'sunu izleme"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_TRACE_DRIVE2_CPU,    N_("Trace the drive2 CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TRACE_DRIVE2_CPU_DA, "Trace processor i drev 2"},
	/* de */ {IDCLS_TRACE_DRIVE2_CPU_DE, "Laufwerk2 CPU verfolgen"},
	/* es */ {IDCLS_TRACE_DRIVE2_CPU_ES, "Rastrear la CPU de la unidad de disco 2"},
	/* fr */ {IDCLS_TRACE_DRIVE2_CPU_FR, "Tracer l'UCT du drive2"},
	/* hu */ {IDCLS_TRACE_DRIVE2_CPU_HU, "A 2-es lemezegysÈg CPU-j·nak nyomkˆvetÈse"},
	/* it */ {IDCLS_TRACE_DRIVE2_CPU_IT, "Traccia la CPU del drive2"},
	/* ko */ {IDCLS_TRACE_DRIVE2_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TRACE_DRIVE2_CPU_NL, "Traceer de CPU van drive 2"},
	/* pl */ {IDCLS_TRACE_DRIVE2_CPU_PL, "¶ledzenie CPU stacji2"},
	/* ru */ {IDCLS_TRACE_DRIVE2_CPU_RU, "Trace the drive2 CPU"},
	/* sv */ {IDCLS_TRACE_DRIVE2_CPU_SV, "SpÂra processor i diskettstation 2"},
	/* tr */ {IDCLS_TRACE_DRIVE2_CPU_TR, "S¸r¸c¸2 CPU'sunu izle"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_DONT_TRACE_DRIVE2_CPU,    N_("Do not trace the drive2 CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DONT_TRACE_DRIVE2_CPU_DA, "Trace ikke processor i drev 2"},
	/* de */ {IDCLS_DONT_TRACE_DRIVE2_CPU_DE, "Laufwerk2 CPU nicht verfolgen"},
	/* es */ {IDCLS_DONT_TRACE_DRIVE2_CPU_ES, "No rastrear la CPU de la unidad de disco 2"},
	/* fr */ {IDCLS_DONT_TRACE_DRIVE2_CPU_FR, "Ne pas tracer l'UCT du drive2"},
	/* hu */ {IDCLS_DONT_TRACE_DRIVE2_CPU_HU, "A 2-es lemezegysÈg CPU-ja nyomkˆvetÈsÈnek tilt·sa"},
	/* it */ {IDCLS_DONT_TRACE_DRIVE2_CPU_IT, "Non tracciare la CPU del drive2"},
	/* ko */ {IDCLS_DONT_TRACE_DRIVE2_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DONT_TRACE_DRIVE2_CPU_NL, "Traceer de CPU van drive 2 niet"},
	/* pl */ {IDCLS_DONT_TRACE_DRIVE2_CPU_PL, "Bez ∂ledzenia CPU stacji2"},
	/* ru */ {IDCLS_DONT_TRACE_DRIVE2_CPU_RU, "Do not trace the drive2 CPU"},
	/* sv */ {IDCLS_DONT_TRACE_DRIVE2_CPU_SV, "SpÂra inte processor i diskettstation 2"},
	/* tr */ {IDCLS_DONT_TRACE_DRIVE2_CPU_TR, "S¸r¸c¸2 CPU'sunu izleme"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_TRACE_DRIVE3_CPU,    N_("Trace the drive3 CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TRACE_DRIVE3_CPU_DA, "Trace processor i drev 3"},
	/* de */ {IDCLS_TRACE_DRIVE3_CPU_DE, "Laufwerk3 CPU verfolgen"},
	/* es */ {IDCLS_TRACE_DRIVE3_CPU_ES, "Rastrear la CPU de la unidad de disco 3"},
	/* fr */ {IDCLS_TRACE_DRIVE3_CPU_FR, "Tracer l'UCT du drive3"},
	/* hu */ {IDCLS_TRACE_DRIVE3_CPU_HU, "A 3-as lemezegysÈg CPU-j·nak nyomkˆvetÈse"},
	/* it */ {IDCLS_TRACE_DRIVE3_CPU_IT, "Traccia la CPU del drive3"},
	/* ko */ {IDCLS_TRACE_DRIVE3_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TRACE_DRIVE3_CPU_NL, "Traceer de CPU van drive 3"},
	/* pl */ {IDCLS_TRACE_DRIVE3_CPU_PL, "¶ledzenie CPU stacji3"},
	/* ru */ {IDCLS_TRACE_DRIVE3_CPU_RU, "Trace the drive3 CPU"},
	/* sv */ {IDCLS_TRACE_DRIVE3_CPU_SV, "SpÂra processor i diskettstation 3"},
	/* tr */ {IDCLS_TRACE_DRIVE3_CPU_TR, "S¸r¸c¸3 CPU'sunu izle"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_DONT_TRACE_DRIVE3_CPU,    N_("Do not trace the drive3 CPU")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DONT_TRACE_DRIVE3_CPU_DA, "Trace ikke processor i drev 3"},
	/* de */ {IDCLS_DONT_TRACE_DRIVE3_CPU_DE, "Laufwerk3 CPU nicht verfolgen"},
	/* es */ {IDCLS_DONT_TRACE_DRIVE3_CPU_ES, "No rastrear la CPU de la unidad de disco 3"},
	/* fr */ {IDCLS_DONT_TRACE_DRIVE3_CPU_FR, "Ne pas tracer l'UCT du drive3"},
	/* hu */ {IDCLS_DONT_TRACE_DRIVE3_CPU_HU, "A 3-as lemezegysÈg CPU-ja nyomkˆvetÈsÈnek tilt·sa"},
	/* it */ {IDCLS_DONT_TRACE_DRIVE3_CPU_IT, "Non tracciare la CPU del drive3"},
	/* ko */ {IDCLS_DONT_TRACE_DRIVE3_CPU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DONT_TRACE_DRIVE3_CPU_NL, "Traceer de CPU van drive 3 niet"},
	/* pl */ {IDCLS_DONT_TRACE_DRIVE3_CPU_PL, "Bez ∂ledzenia CPU stacji3"},
	/* ru */ {IDCLS_DONT_TRACE_DRIVE3_CPU_RU, "Do not trace the drive3 CPU"},
	/* sv */ {IDCLS_DONT_TRACE_DRIVE3_CPU_SV, "SpÂra inte processor i diskettstation 3"},
	/* tr */ {IDCLS_DONT_TRACE_DRIVE3_CPU_TR, "S¸r¸c¸3 CPU'sunu izleme"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_TRACE_MODE,    N_("Trace mode (0=normal 1=small 2=history)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TRACE_MODE_DA, "Trace-tilstand (0=normal 1=lille 2=historik)"},
	/* de */ {IDCLS_TRACE_MODE_DE, "Verfolgung Modus (0=normal 1=klein 2=Geschichte)"},
	/* es */ {IDCLS_TRACE_MODE_ES, "Modo de rastreo (0=normal, 1=pequeÒo, 2=historial)"},
	/* fr */ {IDCLS_TRACE_MODE_FR, "Mode de trace (0=normal 1=petit 2=historique)"},
	/* hu */ {IDCLS_TRACE_MODE_HU, "NyomkˆvetÈsi mÛd (0=norm·l 1=kicsi 2=elızmÈnyek)"},
	/* it */ {IDCLS_TRACE_MODE_IT, "Modalit‡ di tracciamento (0=normale 1=piccola 2=storica)"},
	/* ko */ {IDCLS_TRACE_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TRACE_MODE_NL, "Traceermodus (0=normaal 1=klein 2=geschiedenis"},
	/* pl */ {IDCLS_TRACE_MODE_PL, "Tryb ∂ledzenia (0=zwyk≥e 1=ma≥e 2=w historii)"},
	/* ru */ {IDCLS_TRACE_MODE_RU, "Trace mode (0=normal 1=small 2=history)"},
	/* sv */ {IDCLS_TRACE_MODE_SV, "SpÂrningsl‰ge (0=normal 1=liten 2=historik)"},
	/* tr */ {IDCLS_TRACE_MODE_TR, "›zleme modu (0=normal 1=k¸Á¸k 2=tarihÁe)"},
#endif
	
	/* debug.c */
	/* en */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES,    N_("Set the amount of automatic playback frames")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_DE, "Frames f¸r automatische Wiedergabe eingeben"},
	/* es */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_NL, "Stel het aantal automatische weergave frames in"},
	/* pl */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_PL, "Ustaw liczbÍ†klatek do automatycznego odegrania"},
	/* ru */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_AUTO_PLAYBACK_FRAMES_TR, ""},  /* fuzzy */
#endif
	
#endif
	
	/* debug.c */
	/* en */ {IDCLS_P_FRAMES,    N_("<frames>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_FRAMES_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_FRAMES_DE, "<Rahmen>"},
	/* es */ {IDCLS_P_FRAMES_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_P_FRAMES_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_FRAMES_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_FRAMES_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_P_FRAMES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_FRAMES_NL, "<frames>"},
	/* pl */ {IDCLS_P_FRAMES_PL, "<klatki>"},
	/* ru */ {IDCLS_P_FRAMES_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_FRAMES_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_FRAMES_TR, ""},  /* fuzzy */
#endif
	
	/* event.c */
	/* en */ {IDCLS_PLAYBACK_RECORDED_EVENTS,    N_("Playback recorded events")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PLAYBACK_RECORDED_EVENTS_DA, "Afspil hÊndelsesoptagelse"},
	/* de */ {IDCLS_PLAYBACK_RECORDED_EVENTS_DE, "Wiedergabe von aufgenommener Ereignisse"},
	/* es */ {IDCLS_PLAYBACK_RECORDED_EVENTS_ES, "Reproducir eventos grabados"},
	/* fr */ {IDCLS_PLAYBACK_RECORDED_EVENTS_FR, "Lire les ÈvÈnements enregistrÈs"},
	/* hu */ {IDCLS_PLAYBACK_RECORDED_EVENTS_HU, "A felvett esemÈnyek visszaj·tsz·sa"},
	/* it */ {IDCLS_PLAYBACK_RECORDED_EVENTS_IT, "Riproduzione eventi registrati"},
	/* ko */ {IDCLS_PLAYBACK_RECORDED_EVENTS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PLAYBACK_RECORDED_EVENTS_NL, "Afspelen opgenomen gebeurtenissen"},
	/* pl */ {IDCLS_PLAYBACK_RECORDED_EVENTS_PL, "OdtwÛrz zapisane zdarzenia"},
	/* ru */ {IDCLS_PLAYBACK_RECORDED_EVENTS_RU, "Playback recorded events"},
	/* sv */ {IDCLS_PLAYBACK_RECORDED_EVENTS_SV, "Spela upp inspelade h‰ndelser"},
	/* tr */ {IDCLS_PLAYBACK_RECORDED_EVENTS_TR, "Kay˝ttan y¸r¸tme kaydedilmi˛ olaylar˝"},
#endif
	
	/* event.c */
	/* en */ {IDCLS_SET_EVENT_SNAPSHOT_DIR,    N_("Set event snapshot directory")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_DE, "Event Snapshot Verzeichnis w‰hlen."},
	/* es */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_ES, "Seleccionar directorio eventos Instantaneas"},
	/* fr */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_NL, "Stel gebeurtenis momentopname map in"},
	/* pl */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_PL, "Ustaw katalog zrzutÛw zdarzeÒ"},
	/* ru */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_EVENT_SNAPSHOT_DIR_TR, ""},  /* fuzzy */
#endif
	
	/* event.c */
	/* en */ {IDCLS_SET_EVENT_START_SNAPSHOT,    N_("Set event start snapshot")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_EVENT_START_SNAPSHOT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_EVENT_START_SNAPSHOT_DE, "Snapshot speichern starten"},
	/* es */ {IDCLS_SET_EVENT_START_SNAPSHOT_ES, "Seleccionar evento inicio Instantanea"},
	/* fr */ {IDCLS_SET_EVENT_START_SNAPSHOT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_EVENT_START_SNAPSHOT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_EVENT_START_SNAPSHOT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_EVENT_START_SNAPSHOT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_EVENT_START_SNAPSHOT_NL, "Stel start gebeurtenis momentopname in"},
	/* pl */ {IDCLS_SET_EVENT_START_SNAPSHOT_PL, "Ustaw zrzut pocz±tku zdarzenia"},
	/* ru */ {IDCLS_SET_EVENT_START_SNAPSHOT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_EVENT_START_SNAPSHOT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_EVENT_START_SNAPSHOT_TR, ""},  /* fuzzy */
#endif
	
	/* event.c */
	/* en */ {IDCLS_SET_EVENT_END_SNAPSHOT,    N_("Set event end snapshot")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_EVENT_END_SNAPSHOT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_EVENT_END_SNAPSHOT_DE, "Event Snapshot beenden"},
	/* es */ {IDCLS_SET_EVENT_END_SNAPSHOT_ES, "Seleccionar evento fin Instantanea"},
	/* fr */ {IDCLS_SET_EVENT_END_SNAPSHOT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_EVENT_END_SNAPSHOT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_EVENT_END_SNAPSHOT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_EVENT_END_SNAPSHOT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_EVENT_END_SNAPSHOT_NL, "Stel eind gebeurtenis momentopname in"},
	/* pl */ {IDCLS_SET_EVENT_END_SNAPSHOT_PL, "Ustaw zrzut koÒca zdarzenia"},
	/* ru */ {IDCLS_SET_EVENT_END_SNAPSHOT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_EVENT_END_SNAPSHOT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_EVENT_END_SNAPSHOT_TR, ""},  /* fuzzy */
#endif
	
	/* event.c */
	/* en */ {IDCLS_SET_EVENT_START_MODE,    N_("Set event start mode (0: file save, 1: file load, 2: reset, 3: playback)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_EVENT_START_MODE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_EVENT_START_MODE_DE, "Event Start Modus (0: File speichern, 1: File laden, 2: Reset, 3: Playback)"},
	/* es */ {IDCLS_SET_EVENT_START_MODE_ES, "Seleccionar modo inicio eventos (0: grabar fichero, 1: cargar fichero, 2: reinicio, 3: reproducir)"},
	/* fr */ {IDCLS_SET_EVENT_START_MODE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_EVENT_START_MODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_EVENT_START_MODE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_EVENT_START_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_EVENT_START_MODE_NL, "Stel gebeurtenis start modus in (0: bestand opslaan, 1: bestand laden, 2: reset, 4: afspelen)"},
	/* pl */ {IDCLS_SET_EVENT_START_MODE_PL, "Ustaw tryb rozpoczÍcia zdarzenia (0: zapis pliku, 1: wczytanie pliku, 2: reset, 3: odegranie)"},
	/* ru */ {IDCLS_SET_EVENT_START_MODE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_EVENT_START_MODE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_EVENT_START_MODE_TR, ""},  /* fuzzy */
#endif
	
	/* event.c */
	/* en */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE,    N_("Enable including disk images")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_DE, "Disk Image inkludieren"},
	/* es */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_NL, "Insluiten van disk bestanden inschakelen"},
	/* pl */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_PL, "W≥±cz do≥±czanie obrazÛw dyskietek"},
	/* ru */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_EVENT_IMAGE_INCLUDE_TR, ""},  /* fuzzy */
#endif
	
	/* event.c */
	/* en */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE,    N_("Disable including disk images")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_DE, "Disk Image exkludieren"},
	/* es */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_NL, "Insluiten van disk bestanden uitschakelen"},
	/* pl */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_PL, "Wy≥±cz do≥±czanie obrazÛw dyskietek"},
	/* ru */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_EVENT_IMAGE_INCLUDE_TR, ""},  /* fuzzy */
#endif
	
	/* monitor.c */
	/* en */ {IDCLS_EXECUTE_MONITOR_FROM_FILE,    N_("Execute monitor commands from file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_DA, "Udf¯r monitorkommandoer fra fil"},
	/* de */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_DE, "Monitor Kommandos aus Datei ausf¸hren"},
	/* es */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_ES, "Ejecutar comandos del monitor desde fichero"},
	/* fr */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_FR, "ExÈcuter des commandes moniteur depuis un fichier"},
	/* hu */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_HU, "Monitorprogram parancsok futtat·sa f·jlbÛl"},
	/* it */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_IT, "Esegui i comandi del monitor da un file"},
	/* ko */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_NL, "Uitvoeren van monitor commando's uit bestand"},
	/* pl */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_PL, "Wykonaj polecenia monitora z pliku"},
	/* ru */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_RU, "Execute monitor commands from file"},
	/* sv */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_SV, "Exekvera monitorkommandon frÂn fil"},
	/* tr */ {IDCLS_EXECUTE_MONITOR_FROM_FILE_TR, "Dosyadan monitˆr komutlar˝ Áal˝˛t˝r"},
#endif
	
	/* monitor.c */
	/* en */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE,    N_("Write monitor output also to file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_DE, "Monitor Ausgabe auch auf Datei schreiben"},
	/* es */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_ES, "Grabar salida del monitor tambien en fichero"},
	/* fr */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_IT, "Scrivi anche su file l'output del monitor"},
	/* ko */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_NL, "Schrijf uitvoer van de monitor ook naar een bestand"},
	/* pl */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_PL, "Zapisuj wyj∂cie monitora rÛwnieø do pliku"},
	/* ru */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_SV, "Skriv monitorutdata ‰ven till fil"},
	/* tr */ {IDCLS_WRITE_MONITOR_LOG_TO_FILE_TR, ""},  /* fuzzy */
#endif
	
	/* monitor.c */
	/* en */ {IDCLS_SET_INITIAL_BREAKPOINT,    N_("Set an initial breakpoint for the monitor")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_INITIAL_BREAKPOINT_DA, "SÊt et initielt breakpoint for monitoren"},
	/* de */ {IDCLS_SET_INITIAL_BREAKPOINT_DE, "Setze initialen Breakpoint f¸r Monitor"},
	/* es */ {IDCLS_SET_INITIAL_BREAKPOINT_ES, "Seleccionar un punto inicial de ruptura para el monitor"},
	/* fr */ {IDCLS_SET_INITIAL_BREAKPOINT_FR, "RÈgler un point díarrÍt initial pour le moniteur"},
	/* hu */ {IDCLS_SET_INITIAL_BREAKPOINT_HU, "Kezdeti tˆrÈspont megad·sa a monitorprogramnak"},
	/* it */ {IDCLS_SET_INITIAL_BREAKPOINT_IT, "Imposta un breakpoint iniziale per il monitor"},
	/* ko */ {IDCLS_SET_INITIAL_BREAKPOINT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_INITIAL_BREAKPOINT_NL, "Stel een initieel breekpunt in voor de monitor"},
	/* pl */ {IDCLS_SET_INITIAL_BREAKPOINT_PL, "Okre∂l pocz±tkowy punkt wstrzymania w monitorze"},
	/* ru */ {IDCLS_SET_INITIAL_BREAKPOINT_RU, "Set an initial breakpoint for the monitor"},
	/* sv */ {IDCLS_SET_INITIAL_BREAKPOINT_SV, "S‰tt en ursprunglig brytpunkt fˆr monitorn"},
	/* tr */ {IDCLS_SET_INITIAL_BREAKPOINT_TR, "Monitˆr iÁin dahili bir kesme noktas˝ ayarla"},
#endif
	
	/* monitor.c */
	/* en */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN,    N_("Keep the monitor open")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_DE, "Monitor gˆffnet halten"},
	/* es */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_NL, "Laat de monitor geopend blijven"},
	/* pl */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_PL, "Nie zamykaj monitora"},
	/* ru */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_KEEP_MONITOR_OPEN_TR, ""},  /* fuzzy */
#endif
	
	/* monitor.c */
	/* en */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN,    N_("Do not keep the monitor open")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_DE, "Monitor nicht gˆffnet halten"},
	/* es */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_NL, "Laat de monitor niet geopend blijven"},
	/* pl */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_PL, "Zamykaj monitor"},
	/* ru */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_KEEP_MONITOR_OPEN_TR, ""},  /* fuzzy */
#endif
	
	/* monitor_network.c */
	/* en */ {IDCLS_ENABLE_REMOTE_MONITOR,    N_("Enable remote monitor")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_REMOTE_MONITOR_DA, "AktivÈr fjern-monitor"},
	/* de */ {IDCLS_ENABLE_REMOTE_MONITOR_DE, "Monitor Fernsteuerung aktivieren"},
	/* es */ {IDCLS_ENABLE_REMOTE_MONITOR_ES, "Permitir monitor remoto"},
	/* fr */ {IDCLS_ENABLE_REMOTE_MONITOR_FR, "Activer le moniteur distant"},
	/* hu */ {IDCLS_ENABLE_REMOTE_MONITOR_HU, "T·voli monitor engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_REMOTE_MONITOR_IT, "Attiva monitor remoto"},
	/* ko */ {IDCLS_ENABLE_REMOTE_MONITOR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_REMOTE_MONITOR_NL, "Monitor afstandsbediening inschakelen"},
	/* pl */ {IDCLS_ENABLE_REMOTE_MONITOR_PL, "W≥±cz zdalny monitor"},
	/* ru */ {IDCLS_ENABLE_REMOTE_MONITOR_RU, "Enable remote monitor"},
	/* sv */ {IDCLS_ENABLE_REMOTE_MONITOR_SV, "Aktivera fj‰rrmonitor"},
	/* tr */ {IDCLS_ENABLE_REMOTE_MONITOR_TR, "Uzak monitˆr¸ aktif et"},
#endif
	
	/* monitor_network.c */
	/* en */ {IDCLS_DISABLE_REMOTE_MONITOR,    N_("Disable remote monitor")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_REMOTE_MONITOR_DA, "DeaktivÈr fjern-monitor"},
	/* de */ {IDCLS_DISABLE_REMOTE_MONITOR_DE, "Monitor Fernsteuerung deaktivieren"},
	/* es */ {IDCLS_DISABLE_REMOTE_MONITOR_ES, "Deshabilitar monitor remoto"},
	/* fr */ {IDCLS_DISABLE_REMOTE_MONITOR_FR, "DÈsactiver le moniteur distant"},
	/* hu */ {IDCLS_DISABLE_REMOTE_MONITOR_HU, "T·voli monitor tilt·sa"},
	/* it */ {IDCLS_DISABLE_REMOTE_MONITOR_IT, "Disattiva monitor remoto"},
	/* ko */ {IDCLS_DISABLE_REMOTE_MONITOR_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_REMOTE_MONITOR_NL, "Monitor afstandsbediening uitschakelen"},
	/* pl */ {IDCLS_DISABLE_REMOTE_MONITOR_PL, "Wy≥±cz zdalny monitor"},
	/* ru */ {IDCLS_DISABLE_REMOTE_MONITOR_RU, "Disable remote monitor"},
	/* sv */ {IDCLS_DISABLE_REMOTE_MONITOR_SV, "Inaktivera fj‰rrmonitor"},
	/* tr */ {IDCLS_DISABLE_REMOTE_MONITOR_TR, "Uzak monitˆr¸ pasifle˛tir"},
#endif
	
	/* monitor_network.c */
	/* en */ {IDCLS_REMOTE_MONITOR_ADDRESS,    N_("The local address the remote monitor should bind to")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_REMOTE_MONITOR_ADDRESS_DA, "Lokal adresse fjern-monitoren skal binde til"},
	/* de */ {IDCLS_REMOTE_MONITOR_ADDRESS_DE, "Lokale Adresse f¸r die Monitorfernsteuerung"},
	/* es */ {IDCLS_REMOTE_MONITOR_ADDRESS_ES, "DirecciÛn del monitor remoto al que se debe unir"},
	/* fr */ {IDCLS_REMOTE_MONITOR_ADDRESS_FR, "L'adresse locale ‡ laquelle le moniteur distant devrait se brancher"},
	/* hu */ {IDCLS_REMOTE_MONITOR_ADDRESS_HU, "A helyi cÌm, amelyet a monitorprogram lefoglal"},
	/* it */ {IDCLS_REMOTE_MONITOR_ADDRESS_IT, "Indirizzo locale a cui il monitor remoto dovrebbe collegarsi"},
	/* ko */ {IDCLS_REMOTE_MONITOR_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_REMOTE_MONITOR_ADDRESS_NL, "Lokaal adres waar de afstandsbediening voor de monitor aan gebonden moet worden"},
	/* pl */ {IDCLS_REMOTE_MONITOR_ADDRESS_PL, "Lokalny adres, z ktÛrym ma siÍ po≥±czyÊ zdalny monitor"},
	/* ru */ {IDCLS_REMOTE_MONITOR_ADDRESS_RU, "The local address the remote monitor should bind to"},
	/* sv */ {IDCLS_REMOTE_MONITOR_ADDRESS_SV, "Lokal adress fˆr fj‰rrmonitor att ansluta till"},
	/* tr */ {IDCLS_REMOTE_MONITOR_ADDRESS_TR, "Uzak monitˆr¸n balanmas˝ gereken lokal adres"},
#endif
	
	/* fliplist.c */
	/* en */ {IDCLS_SPECIFY_FLIP_LIST_NAME,    N_("Specify name of the flip list file image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_FLIP_LIST_NAME_DA, "Angiv navn pÂ bladreliste"},
	/* de */ {IDCLS_SPECIFY_FLIP_LIST_NAME_DE, "Namen f¸r Fliplist Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_FLIP_LIST_NAME_ES, "Especificar nombre fichero lista de intercambio imagenes"},
	/* fr */ {IDCLS_SPECIFY_FLIP_LIST_NAME_FR, "SpÈcifier le nom de l'image du fichier de groupement de disques"},
	/* hu */ {IDCLS_SPECIFY_FLIP_LIST_NAME_HU, "Adja meg a lemezlista f·jl nevÈt"},
	/* it */ {IDCLS_SPECIFY_FLIP_LIST_NAME_IT, "Specifica il nome immagine del file della flip list"},
	/* ko */ {IDCLS_SPECIFY_FLIP_LIST_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_FLIP_LIST_NAME_NL, "Geef de naam van het fliplijstbestand"},
	/* pl */ {IDCLS_SPECIFY_FLIP_LIST_NAME_PL, "Okre∂l nazwÍ pliku obrazu listy podmiany no∂nikÛw"},
	/* ru */ {IDCLS_SPECIFY_FLIP_LIST_NAME_RU, "Specify name of the flip list file image"},
	/* sv */ {IDCLS_SPECIFY_FLIP_LIST_NAME_SV, "Ange namn pÂ vallistefilavbildningen"},
	/* tr */ {IDCLS_SPECIFY_FLIP_LIST_NAME_TR, "S˝ralama listesi dosya imaj˝ ismi belirt"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS,    N_("Show a list of the available options and exit normally")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_DA, "Vis en liste over tilgÊngelige parametre og afslut"},
	/* de */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_DE, "Liste von verf¸gbaren Optionen zeigen und beenden"},
	/* es */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_ES, "Muestra lista de las opciones disponibles al salir normalmente"},
	/* fr */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_FR, "Montrer une liste des options disponibles et terminer normalement"},
	/* hu */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_HU, "Kilist·zza a lehetsÈges opciÛkat Ès kilÈp"},
	/* it */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_IT, "Mostra un elenco delle opzioni disponibili ed esce"},
	/* ko */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_NL, "Toon een lijst van de beschikbare opties en sluit normaal af"},
	/* pl */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_PL, "Pokaø listÍ dostÍpnych poleceÒ i normalnie wyjdº"},
	/* ru */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_RU, "Show a list of the available options and exit normally"},
	/* sv */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_SV, "Visa lista ˆver tillg‰ngliga flaggor och avsluta normalt"},
	/* tr */ {IDCLS_SHOW_COMMAND_LINE_OPTIONS_TR, "Mevcut seÁeneklerin listesini gˆster ve normal Á˝k"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_SHOW_COMPILETIME_FEATURES,    N_("Show a list of the available compile-time options and their configuration.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SHOW_COMPILETIME_FEATURES_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SHOW_COMPILETIME_FEATURES_DE, "Liste von verf¸gbaren kompilierten Optionen und deren Konfiguration zeigen."},
	/* es */ {IDCLS_SHOW_COMPILETIME_FEATURES_ES, "Muestra lista de las opciones disponibles en tiempo de compilaciÛn y su configuraciÛn."},
	/* fr */ {IDCLS_SHOW_COMPILETIME_FEATURES_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SHOW_COMPILETIME_FEATURES_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SHOW_COMPILETIME_FEATURES_IT, "Mostra un elenco delle opzioni di compilazione disponibili e la loro configurazione."},
	/* ko */ {IDCLS_SHOW_COMPILETIME_FEATURES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SHOW_COMPILETIME_FEATURES_NL, "Toon een lijst van de beschikbare 'compile-time' opties en de bijbehorende configuratie."},
	/* pl */ {IDCLS_SHOW_COMPILETIME_FEATURES_PL, "Pokaø listÍ dostÍpnych opcji kompilacji i ich konfiguracjÍ."},
	/* ru */ {IDCLS_SHOW_COMPILETIME_FEATURES_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SHOW_COMPILETIME_FEATURES_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SHOW_COMPILETIME_FEATURES_TR, ""},  /* fuzzy */
#endif
	
#if (!defined  __OS2__ && !defined __BEOS__)
	/* initcmdline.c */
	/* en */ {IDCLS_CONSOLE_MODE,    N_("Console mode (for music playback)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_CONSOLE_MODE_DA, "Konsoltilstand (for musikafspilning)"},
	/* de */ {IDCLS_CONSOLE_MODE_DE, "Konsolenmodus (f¸r Musikwiedergabe)"},
	/* es */ {IDCLS_CONSOLE_MODE_ES, "Modo consola (para reproducciÛn m˙sica)"},
	/* fr */ {IDCLS_CONSOLE_MODE_FR, "Mode console (pour l'Ècoute de musique)"},
	/* hu */ {IDCLS_CONSOLE_MODE_HU, "Konzol mÛd (zene lej·tsz·shoz)"},
	/* it */ {IDCLS_CONSOLE_MODE_IT, "Modalit‡ console (per la riproduzione musicale)"},
	/* ko */ {IDCLS_CONSOLE_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_CONSOLE_MODE_NL, "Consolemodus (voor afspelen van muziek)"},
	/* pl */ {IDCLS_CONSOLE_MODE_PL, "Tryb konsolowy (do odtwarzania muzyki)"},
	/* ru */ {IDCLS_CONSOLE_MODE_RU, "Console mode (for music playback)"},
	/* sv */ {IDCLS_CONSOLE_MODE_SV, "Konsoll‰ge (fˆr musikspelning)"},
	/* tr */ {IDCLS_CONSOLE_MODE_TR, "Konsol modu (m¸zik Áalmak iÁin)"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_ALLOW_CORE_DUMPS,    N_("Allow production of core dumps")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ALLOW_CORE_DUMPS_DA, "Tillad generering af core-dumps"},
	/* de */ {IDCLS_ALLOW_CORE_DUMPS_DE, "Core Dumps ermˆglichen"},
	/* es */ {IDCLS_ALLOW_CORE_DUMPS_ES, "Permitir producciÛn volcados de nucleo"},
	/* fr */ {IDCLS_ALLOW_CORE_DUMPS_FR, "Permettre la production de \"core dumps\""},
	/* hu */ {IDCLS_ALLOW_CORE_DUMPS_HU, "Core dump kÈszÌtÈs engedÈlyezÈse"},
	/* it */ {IDCLS_ALLOW_CORE_DUMPS_IT, "Consente la produzione di core dump"},
	/* ko */ {IDCLS_ALLOW_CORE_DUMPS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ALLOW_CORE_DUMPS_NL, "Maken van core dumps toestaan"},
	/* pl */ {IDCLS_ALLOW_CORE_DUMPS_PL, "PozwÛl tworzyÊ zrzuty rdzenia"},
	/* ru */ {IDCLS_ALLOW_CORE_DUMPS_RU, "Allow production of core dumps"},
	/* sv */ {IDCLS_ALLOW_CORE_DUMPS_SV, "TillÂt skapa minnesutskrift"},
	/* tr */ {IDCLS_ALLOW_CORE_DUMPS_TR, "Bellek durumunun kay˝tlar˝n˝n ¸retilmesine izin ver"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_DONT_ALLOW_CORE_DUMPS,    N_("Do not produce core dumps")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DONT_ALLOW_CORE_DUMPS_DA, "Lav ikke core-dumps"},
	/* de */ {IDCLS_DONT_ALLOW_CORE_DUMPS_DE, "Core Dumps verhindern"},
	/* es */ {IDCLS_DONT_ALLOW_CORE_DUMPS_ES, "No permitir producciÛn volcados de nucleo"},
	/* fr */ {IDCLS_DONT_ALLOW_CORE_DUMPS_FR, "Ne pas produire de \"core dumps\""},
	/* hu */ {IDCLS_DONT_ALLOW_CORE_DUMPS_HU, "Core dump kÈszÌtÈs tilt·sa"},
	/* it */ {IDCLS_DONT_ALLOW_CORE_DUMPS_IT, "Non produce core dump"},
	/* ko */ {IDCLS_DONT_ALLOW_CORE_DUMPS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DONT_ALLOW_CORE_DUMPS_NL, "Maken van core dumps niet toestaan"},
	/* pl */ {IDCLS_DONT_ALLOW_CORE_DUMPS_PL, "Nie twÛrz zrzutÛw rdzenia"},
	/* ru */ {IDCLS_DONT_ALLOW_CORE_DUMPS_RU, "Do not produce core dumps"},
	/* sv */ {IDCLS_DONT_ALLOW_CORE_DUMPS_SV, "Skapa inte minnesutskrift"},
	/* tr */ {IDCLS_DONT_ALLOW_CORE_DUMPS_TR, "Bellek durumunun kay˝tlar˝n˝ ¸retme"},
#endif
	
#else
	
	/* initcmdline.c */
	/* en */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER,    N_("Don't call exception handler")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_DA, "Kald ikke exception-handler"},
	/* de */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_DE, "Ausnahmebehandlung vermeiden"},
	/* es */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_ES, "No llamar al manipulador de excepciones"},
	/* fr */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_FR, "Ne pas utiliser l'assistant d'exception"},
	/* hu */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_HU, "Ne hÌvja a kivÈtel kezelıt"},
	/* it */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_IT, "Richiama il gestore delle eccezioni"},
	/* ko */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_NL, "Geen gebruik maken van de exception handler"},
	/* pl */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_PL, "Nie zajmuj siÍ wyj±tkami"},
	/* ru */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_RU, "Don't call exception handler"},
	/* sv */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_SV, "Anropa inte undantagshanterare"},
	/* tr */ {IDCLS_DONT_CALL_EXCEPTION_HANDLER_TR, "Kural d˝˛˝ i˛leyiciyi Áa˝rma"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_CALL_EXCEPTION_HANDLER,    N_("Call exception handler (default)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_CALL_EXCEPTION_HANDLER_DA, "Kald exception-handler (standard)"},
	/* de */ {IDCLS_CALL_EXCEPTION_HANDLER_DE, "Ausnahmebehandlung aktivieren (Default)"},
	/* es */ {IDCLS_CALL_EXCEPTION_HANDLER_ES, "Llamar al manipulador de excepciones (por defecto)"},
	/* fr */ {IDCLS_CALL_EXCEPTION_HANDLER_FR, "Utiliser l'assistant d'exception (par dÈfaut)"},
	/* hu */ {IDCLS_CALL_EXCEPTION_HANDLER_HU, "KivÈtel kezelı hÌv·sa (alapÈrtelmezÈs)"},
	/* it */ {IDCLS_CALL_EXCEPTION_HANDLER_IT, "Richiama il gestore delle eccezioni (predefinito)"},
	/* ko */ {IDCLS_CALL_EXCEPTION_HANDLER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_CALL_EXCEPTION_HANDLER_NL, "Gebruik maken van de exception handler (standaard)"},
	/* pl */ {IDCLS_CALL_EXCEPTION_HANDLER_PL, "Zajmij siÍ wyj±tkami (domy∂lnie)"},
	/* ru */ {IDCLS_CALL_EXCEPTION_HANDLER_RU, "Call exception handler (default)"},
	/* sv */ {IDCLS_CALL_EXCEPTION_HANDLER_SV, "Anropa undantagshanterare (standard)"},
	/* tr */ {IDCLS_CALL_EXCEPTION_HANDLER_TR, "Kural d˝˛˝ i˛leyiciyi Áa˝r (varsay˝lan)"},
#endif
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_RESTORE_DEFAULT_SETTINGS,    N_("Restore default settings")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_RESTORE_DEFAULT_SETTINGS_DA, "Gendan standardindstillinger"},
	/* de */ {IDCLS_RESTORE_DEFAULT_SETTINGS_DE, "Standard Einstellungen wiederherstellen"},
	/* es */ {IDCLS_RESTORE_DEFAULT_SETTINGS_ES, "Restaurar valores por defecto"},
	/* fr */ {IDCLS_RESTORE_DEFAULT_SETTINGS_FR, "RÈtablir les paramËtres par dÈfaut"},
	/* hu */ {IDCLS_RESTORE_DEFAULT_SETTINGS_HU, "AlapÈrtelmezÈs vissza·llÌt·sa"},
	/* it */ {IDCLS_RESTORE_DEFAULT_SETTINGS_IT, "Ripristina impostazioni predefinite"},
	/* ko */ {IDCLS_RESTORE_DEFAULT_SETTINGS_KO, "µ∆˙∆Æ º≥¡§ ∫πø¯Ω√≈∞±‚"},
	/* nl */ {IDCLS_RESTORE_DEFAULT_SETTINGS_NL, "Herstel standaard instellingen"},
	/* pl */ {IDCLS_RESTORE_DEFAULT_SETTINGS_PL, "PrzywrÛÊ domy∂lne ustawienia"},
	/* ru */ {IDCLS_RESTORE_DEFAULT_SETTINGS_RU, "≤ﬁ··‚–›ﬁ“ÿ‚Ï ›–·‚‡ﬁŸ⁄ÿ ﬂﬁ „‹ﬁ€Á–›ÿÓ"},
	/* sv */ {IDCLS_RESTORE_DEFAULT_SETTINGS_SV, "≈terst‰ll fˆrvalda inst‰llningar"},
	/* tr */ {IDCLS_RESTORE_DEFAULT_SETTINGS_TR, "Varsay˝lan ayarlar˝ geri y¸kle"},
#endif
	
	/* initcmdline.c, c64/cart/mmcreplay.c */
	/* en */ {IDCLS_P_FILE,    N_("<filename>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_FILE_DA, "<filnavn>"},
	/* de */ {IDCLS_P_FILE_DE, "<Dateiname>"},
	/* es */ {IDCLS_P_FILE_ES, "<Nombre fichero>"},
	/* fr */ {IDCLS_P_FILE_FR, "<nom de fichier>"},
	/* hu */ {IDCLS_P_FILE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_FILE_IT, "<nome file>"},
	/* ko */ {IDCLS_P_FILE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_FILE_NL, "<bestandsnaam>"},
	/* pl */ {IDCLS_P_FILE_PL, "<filename>"},
	/* ru */ {IDCLS_P_FILE_RU, "<ÿ‹Ô ‰–Ÿ€–>"},
	/* sv */ {IDCLS_P_FILE_SV, "<filnamn>"},
	/* tr */ {IDCLS_P_FILE_TR, "<dosyaismi>"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_SPECIFY_CONFIG_FILE,    N_("Specify config file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_CONFIG_FILE_DA, "Angiv konfigurationsfil"},
	/* de */ {IDCLS_SPECIFY_CONFIG_FILE_DE, "Konfigurationsdateiname definieren"},
	/* es */ {IDCLS_SPECIFY_CONFIG_FILE_ES, "Especificar fichero de configuraciÛn"},
	/* fr */ {IDCLS_SPECIFY_CONFIG_FILE_FR, "SpÈcifier le fichier de configuration"},
	/* hu */ {IDCLS_SPECIFY_CONFIG_FILE_HU, "Adja meg a konfigur·ciÛs f·jl nevÈt"},
	/* it */ {IDCLS_SPECIFY_CONFIG_FILE_IT, "Specifica il nome del file di configurazione"},
	/* ko */ {IDCLS_SPECIFY_CONFIG_FILE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_CONFIG_FILE_NL, "Geef configuratie bestandsnaam"},
	/* pl */ {IDCLS_SPECIFY_CONFIG_FILE_PL, "Okre∂l plik konfiguracji"},
	/* ru */ {IDCLS_SPECIFY_CONFIG_FILE_RU, "Specify config file"},
	/* sv */ {IDCLS_SPECIFY_CONFIG_FILE_SV, "Ange inst‰llningsfil"},
	/* tr */ {IDCLS_SPECIFY_CONFIG_FILE_TR, "Konfig¸rasyon dosyas˝n˝ belirt"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_SPECIFY_DUMPCONFIG_FILE,    N_("Dump all resources to specified config file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_DE, "Exportiere s‰mtliche Resourcen in angegebene Datei"},
	/* es */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_NL, "Sla alle huidige instellingen op naar opgegeven configuratiebestand"},
	/* pl */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_DUMPCONFIG_FILE_TR, ""},  /* fuzzy */
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_LIMIT_CYCLES,    N_("Specify number of cycles to run before quitting with an error.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_LIMIT_CYCLES_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_LIMIT_CYCLES_DE, "Definiere die Zyklenanzahl, welche vor dem Beenden der Emulation noch\nausgef¸hrt wird."},
	/* es */ {IDCLS_LIMIT_CYCLES_ES, "Especificar n˙mero de ciclos a ejecutar antes de salir con un error"},
	/* fr */ {IDCLS_LIMIT_CYCLES_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_LIMIT_CYCLES_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_LIMIT_CYCLES_IT, "Specifica il numero di cicli da eseguire prima di uscire con un errore."},
	/* ko */ {IDCLS_LIMIT_CYCLES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_LIMIT_CYCLES_NL, "Geef het aantal uit te voeren cycli voor het afsluiten met een fout."},
	/* pl */ {IDCLS_LIMIT_CYCLES_PL, "Ustal, ile cykli pu∂ciÊ przed zatrzymaniem i wyrzuceniem b≥Ídu."},
	/* ru */ {IDCLS_LIMIT_CYCLES_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_LIMIT_CYCLES_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_LIMIT_CYCLES_TR, ""},  /* fuzzy */
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_ATTACH_AND_AUTOSTART,    N_("Attach and autostart tape/disk image <name>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_AND_AUTOSTART_DA, "Tilslut og autostart bÂnd-/disk-image <navn>"},
	/* de */ {IDCLS_ATTACH_AND_AUTOSTART_DE, "Einlegen und Autostart Disk/Band Imagedatei <Name>"},
	/* es */ {IDCLS_ATTACH_AND_AUTOSTART_ES, "Insertar y autoarrancar imagen de cinta/disco <nombre>"},
	/* fr */ {IDCLS_ATTACH_AND_AUTOSTART_FR, "InsÈrer et autocharger l'image de disque/datassette <nom>"},
	/* hu */ {IDCLS_ATTACH_AND_AUTOSTART_HU, "<nÈv> lemez/szalag kÈpm·s csatol·sa Ès automatikus elindÌt·sa"},
	/* it */ {IDCLS_ATTACH_AND_AUTOSTART_IT, "Seleziona ed avvia l'immagine di una cassetta/disco <nome>"},
	/* ko */ {IDCLS_ATTACH_AND_AUTOSTART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_AND_AUTOSTART_NL, "Koppel en autostart tape/diskbestand <naam>"},
	/* pl */ {IDCLS_ATTACH_AND_AUTOSTART_PL, "Zamontuj i automatycznie uruchom obraz ta∂my lub dysku <nazwa>"},
	/* ru */ {IDCLS_ATTACH_AND_AUTOSTART_RU, "Attach and autostart tape/disk image <name>"},
	/* sv */ {IDCLS_ATTACH_AND_AUTOSTART_SV, "Anslut och starta band-/diskettavbildning <namn>"},
	/* tr */ {IDCLS_ATTACH_AND_AUTOSTART_TR, "Teyp/Disk imaj˝ <isim> yerle˛tir ve otomatik ba˛lat"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_ATTACH_AND_AUTOLOAD,    N_("Attach and autoload tape/disk image <name>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_AND_AUTOLOAD_DA, "Tilslut og lÊs bÂnd-/disk-image <navn>"},
	/* de */ {IDCLS_ATTACH_AND_AUTOLOAD_DE, "Einlegen und Autoload Disk/Band Imagedatei <Name>"},
	/* es */ {IDCLS_ATTACH_AND_AUTOLOAD_ES, "Insertar y autocargar imagen de cinta/disco <nombre>"},
	/* fr */ {IDCLS_ATTACH_AND_AUTOLOAD_FR, "InsÈrer et charger l'image de disque/datassette <nom>"},
	/* hu */ {IDCLS_ATTACH_AND_AUTOLOAD_HU, "<nÈv> lemez/szalag kÈpm·s csatol·sa Ès automatikus betˆltÈse"},
	/* it */ {IDCLS_ATTACH_AND_AUTOLOAD_IT, "Seleziona e carica l'immagine di una cassetta/disco <nome>"},
	/* ko */ {IDCLS_ATTACH_AND_AUTOLOAD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_AND_AUTOLOAD_NL, "Koppel en autolaad tape-/diskbestand <naam>"},
	/* pl */ {IDCLS_ATTACH_AND_AUTOLOAD_PL, "Zamontuj i automatycznie wczytaj obraz ta∂my lub dysku <nazwa>"},
	/* ru */ {IDCLS_ATTACH_AND_AUTOLOAD_RU, "Attach and autoload tape/disk image <name>"},
	/* sv */ {IDCLS_ATTACH_AND_AUTOLOAD_SV, "Anslut och l‰s in band-/diskettavbildning <namn>"},
	/* tr */ {IDCLS_ATTACH_AND_AUTOLOAD_TR, "Teyp/Disk imaj˝ <isim> yerle˛tir ve otomatik y¸kle"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_ATTACH_AS_TAPE,    N_("Attach <name> as a tape image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_AS_TAPE_DA, "Tilslut <navn> som bÂnd-image"},
	/* de */ {IDCLS_ATTACH_AS_TAPE_DE, "Imagedatei <Name> einlegen"},
	/* es */ {IDCLS_ATTACH_AS_TAPE_ES, "Insertar <nombre> como imagen de cinta"},
	/* fr */ {IDCLS_ATTACH_AS_TAPE_FR, "InsÈrer <nom> comme image de datassette"},
	/* hu */ {IDCLS_ATTACH_AS_TAPE_HU, "<nÈv> szalag kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_AS_TAPE_IT, "Seleziona <nome> come un'immagine di una cassetta"},
	/* ko */ {IDCLS_ATTACH_AS_TAPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_AS_TAPE_NL, "Koppel <naam> als een tapebestand"},
	/* pl */ {IDCLS_ATTACH_AS_TAPE_PL, "Zamontuj <nazwa> jako obraz ta∂my"},
	/* ru */ {IDCLS_ATTACH_AS_TAPE_RU, "Attach <name> as a tape image"},
	/* sv */ {IDCLS_ATTACH_AS_TAPE_SV, "Anslut <namn> som bandavbildning"},
	/* tr */ {IDCLS_ATTACH_AS_TAPE_TR, "Teyp imaj˝ olarak <isim> yerle˛tir"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_ATTACH_AS_DISK_8,    N_("Attach <name> as a disk image in drive #8")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_AS_DISK_8_DA, "Tilslut <navn> som disk-image i enhed #8"},
	/* de */ {IDCLS_ATTACH_AS_DISK_8_DE, "Imagedatei <Name> im Laufwerk #8 einlegen"},
	/* es */ {IDCLS_ATTACH_AS_DISK_8_ES, "Insertar <nombre> como imagen disco en unidad disco #8"},
	/* fr */ {IDCLS_ATTACH_AS_DISK_8_FR, "InsÈrer <nom> comme image de disque dans le lecteur #8"},
	/* hu */ {IDCLS_ATTACH_AS_DISK_8_HU, "<nÈv> lemezkÈpm·s csatol·sa #8-as egysÈgkÈnt"},
	/* it */ {IDCLS_ATTACH_AS_DISK_8_IT, "Seleziona <nome> come un'immagine di un disco nel drive #8"},
	/* ko */ {IDCLS_ATTACH_AS_DISK_8_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_AS_DISK_8_NL, "Koppel <naam> aan als een schijfbestand in drive #8"},
	/* pl */ {IDCLS_ATTACH_AS_DISK_8_PL, "Zamontuj <nazwa> jako obraz dyskietki w stacji #8"},
	/* ru */ {IDCLS_ATTACH_AS_DISK_8_RU, "Attach <name> as a disk image in drive #8"},
	/* sv */ {IDCLS_ATTACH_AS_DISK_8_SV, "Anslut <namn> som diskettavbildning i enhet 8"},
	/* tr */ {IDCLS_ATTACH_AS_DISK_8_TR, "S¸r¸c¸ #8 disk imaj˝ olarak <isim> yerle˛tir"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_ATTACH_AS_DISK_9,    N_("Attach <name> as a disk image in drive #9")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_AS_DISK_9_DA, "Tilslut <navn> som disk-image i enhed #9"},
	/* de */ {IDCLS_ATTACH_AS_DISK_9_DE, "Imagedatei <Name> im Laufwerk #9 einlegen"},
	/* es */ {IDCLS_ATTACH_AS_DISK_9_ES, "Insertar <nombre> como imagen disco en unidad disco #9"},
	/* fr */ {IDCLS_ATTACH_AS_DISK_9_FR, "InsÈrer <nom> comme image de disque dans le lecteur #9"},
	/* hu */ {IDCLS_ATTACH_AS_DISK_9_HU, "<nÈv> lemezkÈpm·s csatol·sa #9-es egysÈgkÈnt"},
	/* it */ {IDCLS_ATTACH_AS_DISK_9_IT, "Seleziona <nome> come un'immagine di un disco nel drive #9"},
	/* ko */ {IDCLS_ATTACH_AS_DISK_9_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_AS_DISK_9_NL, "Koppel <naam> aan als een schijfbestand in drive #9"},
	/* pl */ {IDCLS_ATTACH_AS_DISK_9_PL, "Zamontuj <nazwa> jako obraz dyskietki w stacji #9"},
	/* ru */ {IDCLS_ATTACH_AS_DISK_9_RU, "Attach <name> as a disk image in drive #9"},
	/* sv */ {IDCLS_ATTACH_AS_DISK_9_SV, "Anslut <namn> som diskettavbildning i enhet 9"},
	/* tr */ {IDCLS_ATTACH_AS_DISK_9_TR, "S¸r¸c¸ #9 disk imaj˝ olarak <isim> yerle˛tir"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_ATTACH_AS_DISK_10,    N_("Attach <name> as a disk image in drive #10")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_AS_DISK_10_DA, "Tilslut <navn> som disk-image i enhed #10"},
	/* de */ {IDCLS_ATTACH_AS_DISK_10_DE, "Imagedatei <Name> im Laufwerk #10 einlegen"},
	/* es */ {IDCLS_ATTACH_AS_DISK_10_ES, "Insertar <nombre> como imagen disco en unidad disco #10"},
	/* fr */ {IDCLS_ATTACH_AS_DISK_10_FR, "InsÈrer <nom> comme image de disque dans le lecteur #10"},
	/* hu */ {IDCLS_ATTACH_AS_DISK_10_HU, "<nÈv> lemezkÈpm·s csatol·sa #10-es egysÈgkÈnt"},
	/* it */ {IDCLS_ATTACH_AS_DISK_10_IT, "Seleziona <nome> come un'immagine di un disco nel drive #10"},
	/* ko */ {IDCLS_ATTACH_AS_DISK_10_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_AS_DISK_10_NL, "Koppel <naam> aan als een schijfbestand in drive #10"},
	/* pl */ {IDCLS_ATTACH_AS_DISK_10_PL, "Zamontuj <nazwa> jako obraz dyskietki w stacji #10"},
	/* ru */ {IDCLS_ATTACH_AS_DISK_10_RU, "Attach <name> as a disk image in drive #10"},
	/* sv */ {IDCLS_ATTACH_AS_DISK_10_SV, "Anslut <namn> som diskettavbildning i enhet 10"},
	/* tr */ {IDCLS_ATTACH_AS_DISK_10_TR, "S¸r¸c¸ #10 disk imaj˝ olarak <isim> yerle˛tir"},
#endif
	
	/* initcmdline.c */
	/* en */ {IDCLS_ATTACH_AS_DISK_11,    N_("Attach <name> as a disk image in drive #11")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_AS_DISK_11_DA, "Tilslut <navn> som disk-image i enhed #11"},
	/* de */ {IDCLS_ATTACH_AS_DISK_11_DE, "Imagedatei <Name> im Laufwerk #11 einlegen"},
	/* es */ {IDCLS_ATTACH_AS_DISK_11_ES, "Insertar <nombre> como imagen disco en unidad disco #11"},
	/* fr */ {IDCLS_ATTACH_AS_DISK_11_FR, "InsÈrer <nom> comme image de disque dans le lecteur #11"},
	/* hu */ {IDCLS_ATTACH_AS_DISK_11_HU, "<nÈv> lemezkÈpm·s csatol·sa #11-es egysÈgkÈnt"},
	/* it */ {IDCLS_ATTACH_AS_DISK_11_IT, "Seleziona <nome> come un'immagine di un disco nel drive #11"},
	/* ko */ {IDCLS_ATTACH_AS_DISK_11_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_AS_DISK_11_NL, "Koppel <naam> aan als een schijfbestand in drive #11"},
	/* pl */ {IDCLS_ATTACH_AS_DISK_11_PL, "Zamontuj <nazwa> jako obraz dyskietki w stacji #11"},
	/* ru */ {IDCLS_ATTACH_AS_DISK_11_RU, "Attach <name> as a disk image in drive #11"},
	/* sv */ {IDCLS_ATTACH_AS_DISK_11_SV, "Anslut <namn> som diskettavbildning i enhet 11"},
	/* tr */ {IDCLS_ATTACH_AS_DISK_11_TR, "S¸r¸c¸ #11 disk imaj˝ olarak <isim> yerle˛tir"},
#endif
	
	/* kbdbuf.c */
	/* en */ {IDCLS_P_STRING,    N_("<string>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_STRING_DA, "<streng>"},
	/* de */ {IDCLS_P_STRING_DE, "<String>"},
	/* es */ {IDCLS_P_STRING_ES, "<cadena>"},
	/* fr */ {IDCLS_P_STRING_FR, "<chaine>"},
	/* hu */ {IDCLS_P_STRING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_STRING_IT, "<stringa>"},
	/* ko */ {IDCLS_P_STRING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_STRING_NL, "<tekenreeks>"},
	/* pl */ {IDCLS_P_STRING_PL, "<string>"},
	/* ru */ {IDCLS_P_STRING_RU, "<string>"},
	/* sv */ {IDCLS_P_STRING_SV, "<str‰ng>"},
	/* tr */ {IDCLS_P_STRING_TR, "<yaz˝>"},
#endif
	
	/* kbdbuf.c */
	/* en */ {IDCLS_PUT_STRING_INTO_KEYBUF,    N_("Put the specified string into the keyboard buffer.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PUT_STRING_INTO_KEYBUF_DA, "IndsÊt den angivne streng i tastatur-bufferen."},
	/* de */ {IDCLS_PUT_STRING_INTO_KEYBUF_DE, "Definierte Eingabe in Tastaturpuffer bereitstellen."},
	/* es */ {IDCLS_PUT_STRING_INTO_KEYBUF_ES, "Introduce la cadena indicada en el b˙fer de teclado."},
	/* fr */ {IDCLS_PUT_STRING_INTO_KEYBUF_FR, "Placer la chaÓne spÈcifiÈe dans le tampon clavier."},
	/* hu */ {IDCLS_PUT_STRING_INTO_KEYBUF_HU, "A megadott sztring elhelyezÈse a billenty˚zet pufferben."},
	/* it */ {IDCLS_PUT_STRING_INTO_KEYBUF_IT, "Metti la stringa specificata nel buffer di tastiera."},
	/* ko */ {IDCLS_PUT_STRING_INTO_KEYBUF_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PUT_STRING_INTO_KEYBUF_NL, "Plaats de opgegeven tekenreeks in de toetsenbordbuffer."},
	/* pl */ {IDCLS_PUT_STRING_INTO_KEYBUF_PL, "Umie∂Ê dany ci±g znakÛw w buforze klawiatury"},
	/* ru */ {IDCLS_PUT_STRING_INTO_KEYBUF_RU, "Put the specified string into the keyboard buffer."},
	/* sv */ {IDCLS_PUT_STRING_INTO_KEYBUF_SV, "L‰gg den angivna str‰ngen i tangentbordsbufferten."},
	/* tr */ {IDCLS_PUT_STRING_INTO_KEYBUF_TR, "Klavye arabelleine belirtilen yaz˝y˝ koy."},
#endif
	
	/* autostart.c */
	/* en */ {IDCLS_SET_KEYBUF_DELAY,    N_("Set additional keyboard buffer delay (0: use default)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_KEYBUF_DELAY_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_KEYBUF_DELAY_DE, "Setze zus‰tzliche Tastaturbuffer Verzˆgerung (0: benutze Standard)"},
	/* es */ {IDCLS_SET_KEYBUF_DELAY_ES, "Seleccionar retardo adicional buffer de teclado (0: Por defecto)"},
	/* fr */ {IDCLS_SET_KEYBUF_DELAY_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_KEYBUF_DELAY_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_KEYBUF_DELAY_IT, "Imposta il ritardo supplementare del buffer di tastiera (0: usa predefinito)"},
	/* ko */ {IDCLS_SET_KEYBUF_DELAY_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_KEYBUF_DELAY_NL, "Stel extra toetsenbord buffer vertraging in (0: gebruik de standaard waarde)"},
	/* pl */ {IDCLS_SET_KEYBUF_DELAY_PL, "Okre∂l dodatkowe opÛºnienie bufora klawiatury (0: domy∂lne)"},
	/* ru */ {IDCLS_SET_KEYBUF_DELAY_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_KEYBUF_DELAY_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_KEYBUF_DELAY_TR, ""},  /* fuzzy */
#endif
	
	/* log.c */
	/* en */ {IDCLS_SPECIFY_LOG_FILE_NAME,    N_("Specify log file name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_LOG_FILE_NAME_DA, "Angiv navn pÂ logfil"},
	/* de */ {IDCLS_SPECIFY_LOG_FILE_NAME_DE, "Logdateiname w‰hlen"},
	/* es */ {IDCLS_SPECIFY_LOG_FILE_NAME_ES, "Especificar nombre fichero registro"},
	/* fr */ {IDCLS_SPECIFY_LOG_FILE_NAME_FR, "SpÈcifier le nom du fichier log"},
	/* hu */ {IDCLS_SPECIFY_LOG_FILE_NAME_HU, "Adja meg a naplÛf·jl nevÈt"},
	/* it */ {IDCLS_SPECIFY_LOG_FILE_NAME_IT, "Specifica il nome del file di log"},
	/* ko */ {IDCLS_SPECIFY_LOG_FILE_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_LOG_FILE_NAME_NL, "Geef de naam van het logbestand"},
	/* pl */ {IDCLS_SPECIFY_LOG_FILE_NAME_PL, "Okre∂l nazwÍ pliku z logiem"},
	/* ru */ {IDCLS_SPECIFY_LOG_FILE_NAME_RU, "Specify log file name"},
	/* sv */ {IDCLS_SPECIFY_LOG_FILE_NAME_SV, "Ange namn pÂ loggfil"},
	/* tr */ {IDCLS_SPECIFY_LOG_FILE_NAME_TR, "Log dosyas˝ ismini belirt"},
#endif
	
	/* log.c */
	/* en */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT,    N_("Enable verbose log output.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_DA, "AktivÈr omstÊndeligt log-output"},
	/* de */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_DE, "Ausf¸hrliche Diagnose aktivieren."},
	/* es */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_ES, "Permitir salida escrita registro."},
	/* fr */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_FR, "Activer la journalisation verbeuse"},
	/* hu */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_IT, "Attiva output dei log verboso."},
	/* ko */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_NL, "Uitgebreide log uitvoer inschakelen."},
	/* pl */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_PL, "W≥±cz szczegÛ≥owy zapis do loga"},
	/* ru */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_RU, "Enable verbose log output."},
	/* sv */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_SV, "Aktivera pratsam loggutdata."},
	/* tr */ {IDCLS_ENABLE_VERBOSE_LOG_OUTPUT_TR, "Verbose log Á˝kt˝s˝n˝ aktif et."},
#endif
	
	/* mouse.c */
	/* en */ {IDCLS_ENABLE_MOUSE_GRAB,    N_("Enable mouse grab")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_MOUSE_GRAB_DA, "AktivÈr overtagelse af mus"},
	/* de */ {IDCLS_ENABLE_MOUSE_GRAB_DE, "Maus aktivieren"},
	/* es */ {IDCLS_ENABLE_MOUSE_GRAB_ES, "Permitir captura del ratÛn"},
	/* fr */ {IDCLS_ENABLE_MOUSE_GRAB_FR, "Saisir la souris"},
	/* hu */ {IDCLS_ENABLE_MOUSE_GRAB_HU, "EgÈr elkap·s engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_MOUSE_GRAB_IT, "Attiva cattura mouse"},
	/* ko */ {IDCLS_ENABLE_MOUSE_GRAB_KO, "∏∂øÏΩ∫ ±◊∑¶ ∞°¥…«œ∞‘ «œ±‚"},
	/* nl */ {IDCLS_ENABLE_MOUSE_GRAB_NL, "Gebruiken van de muis inschakelen"},
	/* pl */ {IDCLS_ENABLE_MOUSE_GRAB_PL, "W≥±cz przechwytywanie myszy"},
	/* ru */ {IDCLS_ENABLE_MOUSE_GRAB_RU, "Enable mouse grab"},
	/* sv */ {IDCLS_ENABLE_MOUSE_GRAB_SV, "Aktivera fÂngande av mus"},
	/* tr */ {IDCLS_ENABLE_MOUSE_GRAB_TR, "Mouse yakalamay˝ aktif et"},
#endif
	
	/* mouse.c */
	/* en */ {IDCLS_DISABLE_MOUSE_GRAB,    N_("Disable mouse grab")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_MOUSE_GRAB_DA, "DeaktivÈr overtagelse af mus"},
	/* de */ {IDCLS_DISABLE_MOUSE_GRAB_DE, "Maus deaktivieren"},
	/* es */ {IDCLS_DISABLE_MOUSE_GRAB_ES, "Deshabilitar captura de ratÛn"},
	/* fr */ {IDCLS_DISABLE_MOUSE_GRAB_FR, "Ne pas saisir la souris"},
	/* hu */ {IDCLS_DISABLE_MOUSE_GRAB_HU, "EgÈr elkap·s tilt·sa"},
	/* it */ {IDCLS_DISABLE_MOUSE_GRAB_IT, "Disattiva cattura mouse"},
	/* ko */ {IDCLS_DISABLE_MOUSE_GRAB_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_MOUSE_GRAB_NL, "Gebruiken van de muis uitschakelen"},
	/* pl */ {IDCLS_DISABLE_MOUSE_GRAB_PL, "Wy≥±cz przechwytywanie myszy"},
	/* ru */ {IDCLS_DISABLE_MOUSE_GRAB_RU, "Disable mouse grab"},
	/* sv */ {IDCLS_DISABLE_MOUSE_GRAB_SV, "Inaktivera fÂngande av mus"},
	/* tr */ {IDCLS_DISABLE_MOUSE_GRAB_TR, "Mouse yakalamay˝ pasifle˛tir"},
#endif
	
	/* mouse.c */
	/* en */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE,    N_("Enable saving of smart mouse RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_DE, "Smart Mouse RTC Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_ES, "Permitir grabar datos smart mouse RTC cuando cambien."},
	/* fr */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_IT, "Attiva il salvataggio dei dati RTC dello smart mouse alla modifica."},
	/* ko */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_NL, "Opslaan van de smart mouse RTC gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_PL, "Zapisuj dane zegara czasu rzeczywistego Smart Mouse przy zmianie"},
	/* ru */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_SMART_MOUSE_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* mouse.c */
	/* en */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE,    N_("Disable saving of smart mouse RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_DE, "Smart Mouse RTC Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_ES, "Deshabilitar grabar datos Smart Mouse RTC cuand cambien."},
	/* fr */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_IT, "Disattiva il salvataggio dei dati RTC dello smart mouse alla modifica."},
	/* ko */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_NL, "Opslaan van de smart mouse RTC gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_PL, "Nie zapisuj danych zegara czasu rzeczywistego Smart Mouse przy zmianie"},
	/* ru */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_SMART_MOUSE_RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* ps2mouse.c */
	/* en */ {IDCLS_ENABLE_PS2MOUSE,    N_("Enable PS/2 mouse on userport")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_PS2MOUSE_DA, "AktivÈr PS/2-mus pÂ brugerporten"},
	/* de */ {IDCLS_ENABLE_PS2MOUSE_DE, "PS/2 Userport Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_PS2MOUSE_ES, "Permitir ratÛn PS/2 en puerto usuario"},
	/* fr */ {IDCLS_ENABLE_PS2MOUSE_FR, "Activer souris PS/2 sur port utilisateur"},
	/* hu */ {IDCLS_ENABLE_PS2MOUSE_HU, "PS/2 egÈr engedÈlyezÈse a userporton"},
	/* it */ {IDCLS_ENABLE_PS2MOUSE_IT, "Attiva mouse PS/2 su userport"},
	/* ko */ {IDCLS_ENABLE_PS2MOUSE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_PS2MOUSE_NL, "Emulatie van een PS/2 muis op de userport inschakelen"},
	/* pl */ {IDCLS_ENABLE_PS2MOUSE_PL, "W≥±cz mysz PS/2 w userporcie"},
	/* ru */ {IDCLS_ENABLE_PS2MOUSE_RU, "Enable PS/2 mouse on userport"},
	/* sv */ {IDCLS_ENABLE_PS2MOUSE_SV, "Aktivera PS/2-mus pÂ anv‰ndarport"},
	/* tr */ {IDCLS_ENABLE_PS2MOUSE_TR, "Userport ¸zerinde PS/2 mouse'u aktif et"},
#endif
	
	/* ps2mouse.c */
	/* en */ {IDCLS_DISABLE_PS2MOUSE,    N_("Disable PS/2 mouse on userport")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_PS2MOUSE_DA, "DeaktivÈr PS/2-mus pÂ brugerporten"},
	/* de */ {IDCLS_DISABLE_PS2MOUSE_DE, "PS/2 Userport Emulation deaktivieren\""},
	/* es */ {IDCLS_DISABLE_PS2MOUSE_ES, "Deshabilitar ratÛn PS/2 en puerto usuario"},
	/* fr */ {IDCLS_DISABLE_PS2MOUSE_FR, "DÈsactiver la souris PS/2 sur port utilisateur"},
	/* hu */ {IDCLS_DISABLE_PS2MOUSE_HU, "PS/2 egÈr tilt·sa a userporton"},
	/* it */ {IDCLS_DISABLE_PS2MOUSE_IT, "Disattiva mouse PS/2 su userport"},
	/* ko */ {IDCLS_DISABLE_PS2MOUSE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_PS2MOUSE_NL, "Emulatie van een PS/2 muis op de userport uitschakelen"},
	/* pl */ {IDCLS_DISABLE_PS2MOUSE_PL, "Wy≥±cz mysz PS/2 w userporcie"},
	/* ru */ {IDCLS_DISABLE_PS2MOUSE_RU, "Disable PS/2 mouse on userport"},
	/* sv */ {IDCLS_DISABLE_PS2MOUSE_SV, "Inaktivera PS/2-mus pÂ anv‰ndarport"},
	/* tr */ {IDCLS_DISABLE_PS2MOUSE_TR, "Userport ¸zerinde PS/2 mouse'u pasifle˛tir"},
#endif
	
	/* ram.c */
	/* en */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE,    N_("Set the value for the very first RAM address after powerup")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_DA, "Angiv vÊrdien for den allerf¯rste RAM adresse efter power-up"},
	/* de */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_DE, "Wert f¸r erstes Byte im RAM nach Kaltstart setzen"},
	/* es */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_ES, "Seleccionar el valor para la primera direcciÛn RAM despuÈs del encendido"},
	/* fr */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_FR, "SpÈcifier la valeur de la premiËre adresse RAM aprËs la mise sous tension"},
	/* hu */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_HU, "Adja meg a legelsı RAM cÌm ÈrtÈkÈt bekapcsol·s ut·n"},
	/* it */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_IT, "Imposta il valore del primissimo indirizzo della RAM dopo l'accensione"},
	/* ko */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_NL, "Stel de waarde in voor het allereerste RAM adres na koude start"},
	/* pl */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_PL, "Okre∂l warto∂Ê pierwszego adresu RAM po uruchomieniu"},
	/* ru */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_RU, "Set the value for the very first RAM address after powerup"},
	/* sv */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_SV, "Ange v‰rdet pÂ den allra fˆrsta RAM-adressen efter strˆmpÂslag"},
	/* tr */ {IDCLS_SET_FIRST_RAM_ADDRESS_VALUE_TR, "G¸Á verildiinde ba˛lang˝Átaki RAM adreslerine atanacak deeri belirle"},
#endif
	
	/* ram.c */
	/* en */ {IDCLS_P_NUM_OF_BYTES,    N_("<num of bytes>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_NUM_OF_BYTES_DA, "<antal bytes>"},
	/* de */ {IDCLS_P_NUM_OF_BYTES_DE, "<Anzahl an Bytes>"},
	/* es */ {IDCLS_P_NUM_OF_BYTES_ES, "<n˙mero de bytes>"},
	/* fr */ {IDCLS_P_NUM_OF_BYTES_FR, "<nb. d'octets>"},
	/* hu */ {IDCLS_P_NUM_OF_BYTES_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_NUM_OF_BYTES_IT, "<numero di byte>"},
	/* ko */ {IDCLS_P_NUM_OF_BYTES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_NUM_OF_BYTES_NL, "<aantal bytes>"},
	/* pl */ {IDCLS_P_NUM_OF_BYTES_PL, "<num of bytes>"},
	/* ru */ {IDCLS_P_NUM_OF_BYTES_RU, "<num of bytes>"},
	/* sv */ {IDCLS_P_NUM_OF_BYTES_SV, "<antal byte>"},
	/* tr */ {IDCLS_P_NUM_OF_BYTES_TR, "<byte say˝s˝>"},
#endif
	
	/* ram.c */
	/* en */ {IDCLS_LENGTH_BLOCK_SAME_VALUE,    N_("Length of memory block initialized with the same value")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_DA, "LÊngde pÂ hukommelsesblok som initialiseres med samme vÊrdi"},
	/* de */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_DE, "L‰nge des Speicherblocks der mit dem gleichen Wert initialisiert ist"},
	/* es */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_ES, "Longitud del bloque de memoria inicializado con el mismo valor"},
	/* fr */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_FR, "Longeur du premier bloc mÈmoire initialisÈ avec la mÍme valeur"},
	/* hu */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_HU, "Azonos ÈrtÈkkel feltˆltˆtt memÛria blokkoknak a hossza"},
	/* it */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_IT, "Lunghezza del blocco di memoria inizializzato con lo stesso valore"},
	/* ko */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_NL, "Geheugenblokgrootte die dezelfde waarde krijgt bij initialisatie"},
	/* pl */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_PL, "D≥ugo∂Ê bloku pamiÍci zainicjowanego z t± sam± warto∂ci±"},
	/* ru */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_RU, "Length of memory block initialized with the same value"},
	/* sv */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_SV, "L‰ngd pÂ minnesblock som initierats med samma v‰rde"},
	/* tr */ {IDCLS_LENGTH_BLOCK_SAME_VALUE_TR, "Ayn˝ deerle balang˝Á ayarlar˝ yap˝lan bellek blou uzunluu"},
#endif
	
	/* ram.c */
	/* en */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN,    N_("Length of memory block initialized with the same pattern")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_DA, "LÊngde pÂ hukommelsesblok som initialiseres med samme m¯nster"},
	/* de */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_DE, "L‰nge des Speicherblocks der mit dem gleichen Muster initialisiert ist"},
	/* es */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_ES, "Longitud bloque memoria inicializado con el mismo patrÛn"},
	/* fr */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_FR, "Longeur du premier bloc mÈmoire initialisÈ avec le mÍme pattern"},
	/* hu */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_HU, "Azonos mint·val feltˆltˆtt memÛria blokkoknak a hossza"},
	/* it */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_IT, "Lunghezza del blocco di memoria inizializzato con lo stesso pattern"},
	/* ko */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_NL, "Geheugenblokgrootte met hetzelfde patroon bij initialisatie"},
	/* pl */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_PL, "D≥ugo∂Ê bloku pamiÍci zainicjowanego z tym samym wzorcem"},
	/* ru */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_RU, "Length of memory block initialized with the same pattern"},
	/* sv */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_SV, "L‰ngd pÂ minnesblock som initierats med samma mˆnster"},
	/* tr */ {IDCLS_LENGTH_BLOCK_SAME_PATTERN_TR, "Ayn˝ desenle balang˝Á ayarlar˝ yap˝lan bellek blou uzunluu"},
#endif
	
	/* romset.c */
	/* en */ {IDCLS_PB_FILE,    N_("<File>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PB_FILE_DA, "<Fil>"},
	/* de */ {IDCLS_PB_FILE_DE, "<Datei>"},
	/* es */ {IDCLS_PB_FILE_ES, "<Archivo>"},
	/* fr */ {IDCLS_PB_FILE_FR, "<Fichier>"},
	/* hu */ {IDCLS_PB_FILE_HU, "<F·jl>"},
	/* it */ {IDCLS_PB_FILE_IT, "<File>"},
	/* ko */ {IDCLS_PB_FILE_KO, "<∆ƒ¿œ>"},
	/* nl */ {IDCLS_PB_FILE_NL, "<Bestand>"},
	/* pl */ {IDCLS_PB_FILE_PL, "<Plik>"},
	/* ru */ {IDCLS_PB_FILE_RU, "<ƒ–Ÿ€>"},
	/* sv */ {IDCLS_PB_FILE_SV, "<Arkiv>"},
	/* tr */ {IDCLS_PB_FILE_TR, "<Dosya>"},
#endif
	
	/* romset.c */
	/* en */ {IDCLS_LOAD_ROMSET_FILE,    N_("load the given romset file")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_LOAD_ROMSET_FILE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_LOAD_ROMSET_FILE_DE, "Lade angegebene ROM Satz Datei!"},
	/* es */ {IDCLS_LOAD_ROMSET_FILE_ES, "Cargar el fichero romset dado"},
	/* fr */ {IDCLS_LOAD_ROMSET_FILE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_LOAD_ROMSET_FILE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_LOAD_ROMSET_FILE_IT, "carica il file del ROM set specificato"},
	/* ko */ {IDCLS_LOAD_ROMSET_FILE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_LOAD_ROMSET_FILE_NL, "Laad het opgegeven romsetbestand"},
	/* pl */ {IDCLS_LOAD_ROMSET_FILE_PL, "Wczytaj dany plik zestawu ROM-Ûw"},
	/* ru */ {IDCLS_LOAD_ROMSET_FILE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_LOAD_ROMSET_FILE_SV, "l‰s in angiven romset-fil"},
	/* tr */ {IDCLS_LOAD_ROMSET_FILE_TR, ""},  /* fuzzy */
#endif
	
	/* romset.c */
	/* en */ {IDCLS_LOAD_ROMSET_ARCHIVE,    N_("load the given romset archive")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_LOAD_ROMSET_ARCHIVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_LOAD_ROMSET_ARCHIVE_DE, "Lade angegebenes ROM Satz Archiv!"},
	/* es */ {IDCLS_LOAD_ROMSET_ARCHIVE_ES, "Cargar el archivo romset dado"},
	/* fr */ {IDCLS_LOAD_ROMSET_ARCHIVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_LOAD_ROMSET_ARCHIVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_LOAD_ROMSET_ARCHIVE_IT, "carica l'archivio del ROM set specificato"},
	/* ko */ {IDCLS_LOAD_ROMSET_ARCHIVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_LOAD_ROMSET_ARCHIVE_NL, "Laad het opgegeven romsetarchief"},
	/* pl */ {IDCLS_LOAD_ROMSET_ARCHIVE_PL, "Wczytaj dane archiwum zestawu ROM-Ûw"},
	/* ru */ {IDCLS_LOAD_ROMSET_ARCHIVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_LOAD_ROMSET_ARCHIVE_SV, "l‰s in givet romset-arkiv"},
	/* tr */ {IDCLS_LOAD_ROMSET_ARCHIVE_TR, ""},  /* fuzzy */
#endif
	
	/* romset.c */
	/* en */ {IDCLS_P_ITEM_NUMBER,    N_("<Item number>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_ITEM_NUMBER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_ITEM_NUMBER_DE, "<Titel Nummer>"},
	/* es */ {IDCLS_P_ITEM_NUMBER_ES, "<N˙mero Ìtem>"},
	/* fr */ {IDCLS_P_ITEM_NUMBER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_ITEM_NUMBER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_ITEM_NUMBER_IT, "<Numero voce>"},
	/* ko */ {IDCLS_P_ITEM_NUMBER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_ITEM_NUMBER_NL, "<Item nummer>"},
	/* pl */ {IDCLS_P_ITEM_NUMBER_PL, "<Liczba elementu>"},
	/* ru */ {IDCLS_P_ITEM_NUMBER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_ITEM_NUMBER_SV, "<Postnummer>"},
	/* tr */ {IDCLS_P_ITEM_NUMBER_TR, ""},  /* fuzzy */
#endif
	
	/* romset.c */
	/* en */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE,    N_("select the given item from the current romset archive")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_DE, "Auswahl des aktuellen ROM Satz Archivs ausw‰hlen"},
	/* es */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_ES, "Seleccionar del archivo romset actual el Ìtem dado"},
	/* fr */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_IT, "seleziona la voce specificata dall'archivio del ROM set attuale"},
	/* ko */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_NL, "selecteer het opgegeven item van het huidige romsetarchief"},
	/* pl */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_PL, "Wybierz dany element z obecnego zestawu ROM-Ûw."},
	/* ru */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_SV, "v‰lj given post frÂn aktuellt romset-arkiv"},
	/* tr */ {IDCLS_SELECT_ITEM_FROM_ROMSET_ARCHIVE_TR, ""},  /* fuzzy */
#endif
	
	/* sound.c */
	/* en */ {IDCLS_ENABLE_SOUND_PLAYBACK,    N_("Enable sound playback")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SOUND_PLAYBACK_DA, "SlÂ lydemulering til"},
	/* de */ {IDCLS_ENABLE_SOUND_PLAYBACK_DE, "Sound Wiedergabe einschalten"},
	/* es */ {IDCLS_ENABLE_SOUND_PLAYBACK_ES, "Permitir sonido pregrabado"},
	/* fr */ {IDCLS_ENABLE_SOUND_PLAYBACK_FR, "Activer le son"},
	/* hu */ {IDCLS_ENABLE_SOUND_PLAYBACK_HU, "Hangok engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_SOUND_PLAYBACK_IT, "Attiva riproduzione audio"},
	/* ko */ {IDCLS_ENABLE_SOUND_PLAYBACK_KO, "ªÁøÓµÂ «√∑π¿ÃπÈ ¿ª ªÁøÎ∞°¥…«œ∞‘ «œ±‚"},
	/* nl */ {IDCLS_ENABLE_SOUND_PLAYBACK_NL, "Geluidsuitvoer inschakelen"},
	/* pl */ {IDCLS_ENABLE_SOUND_PLAYBACK_PL, "W≥±cz odtwarzanie dºwiÍku"},
	/* ru */ {IDCLS_ENABLE_SOUND_PLAYBACK_RU, "Enable sound playback"},
	/* sv */ {IDCLS_ENABLE_SOUND_PLAYBACK_SV, "Aktivera ljudÂtergivning"},
	/* tr */ {IDCLS_ENABLE_SOUND_PLAYBACK_TR, "Sesi aktif et"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_DISABLE_SOUND_PLAYBACK,    N_("Disable sound playback")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SOUND_PLAYBACK_DA, "SlÂ lydemulering fra"},
	/* de */ {IDCLS_DISABLE_SOUND_PLAYBACK_DE, "Sound Wiedergabe ausschalten"},
	/* es */ {IDCLS_DISABLE_SOUND_PLAYBACK_ES, "Desactivar salida de sonido"},
	/* fr */ {IDCLS_DISABLE_SOUND_PLAYBACK_FR, "DÈsactiver le son"},
	/* hu */ {IDCLS_DISABLE_SOUND_PLAYBACK_HU, "Hangok tilt·sa"},
	/* it */ {IDCLS_DISABLE_SOUND_PLAYBACK_IT, "Disattiva riproduzione audio"},
	/* ko */ {IDCLS_DISABLE_SOUND_PLAYBACK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SOUND_PLAYBACK_NL, "Geluidsuitvoer uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SOUND_PLAYBACK_PL, "Wy≥±cz odtwarzanie dºwiÍku"},
	/* ru */ {IDCLS_DISABLE_SOUND_PLAYBACK_RU, "Disable sound playback"},
	/* sv */ {IDCLS_DISABLE_SOUND_PLAYBACK_SV, "Inaktivera ljudÂtergivning"},
	/* tr */ {IDCLS_DISABLE_SOUND_PLAYBACK_TR, "Sesi pasifle˛tir"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ,    N_("Set sound sample rate to <value> Hz")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_DA, "SÊt lydsamplefrekvens til <vÊrdi> Hz"},
	/* de */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_DE, "Setze Sound Sample Rate zu <Wert> Hz"},
	/* es */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_ES, "Seleccionar velocidad de muestreo de sonido en <valor> Hz"},
	/* fr */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_FR, "RÈgler le taux d'Èchantillonage ‡ <valeur> Hz"},
	/* hu */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_HU, "Hang mintavÈtelezÈsi r·ta be·llÌt·sa <ÈrtÈk> Hz-re"},
	/* it */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_IT, "Imposta la velocit‡ di campionamento audio a <valore> Hz"},
	/* ko */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_NL, "Stel geluid sample rate in op <waarde> Hz"},
	/* pl */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_PL, "Ustaw prÛbkowanie sampli na <warto∂Ê> Hz"},
	/* ru */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_RU, "Set sound sample rate to <value> Hz"},
	/* sv */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_SV, "S‰tt ljudsamplingshastighet till <v‰rde> Hz"},
	/* tr */ {IDCLS_SET_SAMPLE_RATE_VALUE_HZ_TR, "Ses ˆrnekleme h˝z˝n˝ <deer> Hz olarak ayarla"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC,    N_("Set sound buffer size to <value> msec")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_DA, "SÊt lydbufferst¯rrelse til <vÊrdi> ms"},
	/* de */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_DE, "Setze Source Buffer Grˆﬂe zu <Wert> msek"},
	/* es */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_ES, "Seleccionar tamaÒo b˙fer sonido en <valor>mseg"},
	/* fr */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_FR, "RÈgler la taille du tampon son ‡ <valeur> ms"},
	/* hu */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_HU, "A hangpuffer mÈretÈt <ÈrtÈk> mp-re ·llÌtja"},
	/* it */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_IT, "Imposta la dimensione del buffer audio a <valore> msec"},
	/* ko */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_NL, "Stel geluidsbuffergrootte in op <waarde> msec"},
	/* pl */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_PL, "Ustaw rozmiar bufora dºwiÍku na <warto∂Ê> ms"},
	/* ru */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_RU, "Set sound buffer size to <value> msec"},
	/* sv */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_SV, "S‰tt ljudbuffertstorlek till <v‰rde> ms"},
	/* tr */ {IDCLS_SET_SOUND_BUFFER_SIZE_MSEC_TR, "Ses arabellek boyutunu <deer> milisaniye olarak ayarla"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SET_SOUND_FRAGMENT_SIZE,    N_("Set sound fragment size (0: very small, 1: small, 2: medium, 3: large, 4: very large)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_DE, "Setze Sound Fragmentgrˆﬂe (0 = sehr klein, 1 = klein, 2 = mittel, 3 = groﬂ, 4 = sehr groﬂ)"},
	/* es */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_ES, "Seleccionar tamaÒo fragmento  (0: pequeÒo, 1: medio, 2: medio, 3: grande, 4: muy grande)"},
	/* fr */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_IT, "Imposta la dimensione dei frammenti audio (0: molto piccola, 1: piccola, 2: media, 3: grande, 4: molto grande)"},
	/* ko */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_NL, "Stel geluidsfragment grootte in (0: heel klein, 1: klein, 2: matig, 3: groot, 4: heel groot)"},
	/* pl */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_PL, "Ustaw rozmiar fragmentu dºwiÍku (0: bardzo ma≥y, 1: ma≥y, 2: ∂redni, 3: duøy, 4: bardzo duøy)"},
	/* ru */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_SOUND_FRAGMENT_SIZE_TR, ""},  /* fuzzy */
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SPECIFY_SOUND_DRIVER,    N_("Specify sound driver")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SOUND_DRIVER_DA, "Angiv lyddriver"},
	/* de */ {IDCLS_SPECIFY_SOUND_DRIVER_DE, "Sound Treiber spezifieren"},
	/* es */ {IDCLS_SPECIFY_SOUND_DRIVER_ES, "Especificar controlador de sonido"},
	/* fr */ {IDCLS_SPECIFY_SOUND_DRIVER_FR, "SpÈcifier le pilote de son"},
	/* hu */ {IDCLS_SPECIFY_SOUND_DRIVER_HU, "Adja meg a hangmodul nevÈt"},
	/* it */ {IDCLS_SPECIFY_SOUND_DRIVER_IT, "Specifica il driver audio"},
	/* ko */ {IDCLS_SPECIFY_SOUND_DRIVER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SOUND_DRIVER_NL, "Geef geluidsstuurprogramma"},
	/* pl */ {IDCLS_SPECIFY_SOUND_DRIVER_PL, "Okre∂l sterownik dºwiÍku"},
	/* ru */ {IDCLS_SPECIFY_SOUND_DRIVER_RU, "Specify sound driver"},
	/* sv */ {IDCLS_SPECIFY_SOUND_DRIVER_SV, "Ange ljuddrivrutin"},
	/* tr */ {IDCLS_SPECIFY_SOUND_DRIVER_TR, "Ses s¸r¸c¸s¸ belirt"},
#endif
	
	/* sampler/sampler.c */
	/* en */ {IDCLS_SPECIFY_SAMPLER_DEVICE,    N_("Specify sampler device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SAMPLER_DEVICE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_SAMPLER_DEVICE_DE, "Sample Ger‰t ausw‰hlen"},
	/* es */ {IDCLS_SPECIFY_SAMPLER_DEVICE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_SAMPLER_DEVICE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_SAMPLER_DEVICE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_SAMPLER_DEVICE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_SAMPLER_DEVICE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SAMPLER_DEVICE_NL, "Geef samplerapparaat op"},
	/* pl */ {IDCLS_SPECIFY_SAMPLER_DEVICE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_SAMPLER_DEVICE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_SAMPLER_DEVICE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_SAMPLER_DEVICE_TR, ""},  /* fuzzy */
#endif
	
	/* sampler/sampler.c */
	/* en */ {IDCLS_P_GAIN,    N_("<Sampler gain>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_GAIN_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_GAIN_DE, "<Sampler Gain>"},
	/* es */ {IDCLS_P_GAIN_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_P_GAIN_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_GAIN_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_GAIN_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_P_GAIN_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_GAIN_NL, "<Sampler toename>"},
	/* pl */ {IDCLS_P_GAIN_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_P_GAIN_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_GAIN_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_GAIN_TR, ""},  /* fuzzy */
#endif
	
	/* sampler/sampler.c */
	/* en */ {IDCLS_SAMPLER_GAIN_IN_PERCENT,    N_("Specify sampler gain in percent (1%-200%)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_DE, "Sampler Gain in Prozent (1% - 200%)"},
	/* es */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_NL, "Geef sampler versterking in procent (1% - 200%)"},
	/* pl */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SAMPLER_GAIN_IN_PERCENT_TR, ""},  /* fuzzy */
#endif
	
	/* sound.c */
	/* en */ {IDCLS_P_ARGS,    N_("<args>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_ARGS_DA, "<argumenter>"},
	/* de */ {IDCLS_P_ARGS_DE, "<Argumente>"},
	/* es */ {IDCLS_P_ARGS_ES, "<argumentos>"},
	/* fr */ {IDCLS_P_ARGS_FR, "<args>"},
	/* hu */ {IDCLS_P_ARGS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_ARGS_IT, "<argomenti>"},
	/* ko */ {IDCLS_P_ARGS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_ARGS_NL, "<parameters>"},
	/* pl */ {IDCLS_P_ARGS_PL, "<args>"},
	/* ru */ {IDCLS_P_ARGS_RU, "<args>"},
	/* sv */ {IDCLS_P_ARGS_SV, "<flaggor>"},
	/* tr */ {IDCLS_P_ARGS_TR, "<arg¸manlar>"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM,    N_("Specify initialization parameters for sound driver")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_DA, "Angiv initialiseringsparametre for lyddriver"},
	/* de */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_DE, "Initialisierungsparameter des Sound Treibers spezifizieren"},
	/* es */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_ES, "Especificar par·metros de inicializaciÛn para el controlador de sonido"},
	/* fr */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_FR, "SpÈcifier les paramËtres d'initialisation pour le pilote son"},
	/* hu */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_HU, "Adja meg a hangmodul indul·si paramÈtereit"},
	/* it */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_IT, "Specifica i parametri di inizializzazione del driver audio"},
	/* ko */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_NL, "Geef de initialisatieparameters voor het geluidsstuurprogramma"},
	/* pl */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_PL, "Okre∂l parametry inicjalizacji sterownika dºwiÍku"},
	/* ru */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_RU, "Specify initialization parameters for sound driver"},
	/* sv */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_SV, "Ange initieringsflaggor fˆr ljuddrivrutin"},
	/* tr */ {IDCLS_SPECIFY_SOUND_DRIVER_PARAM_TR, "Ses s¸r¸c¸s¸ iÁin ba˛lang˝Á parametrelerini belirt"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER,    N_("Specify recording sound driver")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_DA, "Angiv driver til lydoptagelse"},
	/* de */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_DE, "Sound Treiber f¸r Aufnahme spezifizieren"},
	/* es */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_ES, "Especificar controlador grabaciÛn sonido"},
	/* fr */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_FR, "SpÈcifier le pilote d'enregistrement"},
	/* hu */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_HU, "Adja meg a felvÈtelhez haszn·lt hangmodult"},
	/* it */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_IT, "Specifica il driver audio di registrazione"},
	/* ko */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_NL, "Geef geluidsstuurprogramma voor opname"},
	/* pl */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_PL, "Okre∂l sterownik zapisu dºwiÍku"},
	/* ru */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_RU, "Specify recording sound driver"},
	/* sv */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_SV, "Ange ljuddrivrutin fˆr inspelning"},
	/* tr */ {IDCLS_SPECIFY_RECORDING_SOUND_DRIVER_TR, "Ses kay˝t s¸r¸c¸s¸n¸ belirt"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM,    N_("Specify initialization parameters for recording sound driver")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_DA, "Angiv initialiseringsparametre for driver til lydoptagelse"},
	/* de */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_DE, "Initialisierungsparameter f¸r Aufnahme Sound Treiber spezifieren"},
	/* es */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_ES, "Especificar par·metros inicializaciÛn para el controlador grabaciÛn sonido"},
	/* fr */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_FR, "SpÈcifier les paramËtres d'initialisation pour le pilote d'enregistrement"},
	/* hu */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_HU, "Adja meg a felvÈtelhez haszn·lt hangmodul indul·si paramÈtereit"},
	/* it */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_IT, "Specifica i parametri di inizializzazione per il driver audio di registrazione"},
	/* ko */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_NL, "Geef initialisatieparameters voor het geluidsstuurprogramma voor opname"},
	/* pl */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_PL, "Okre∂l parametry inicjalizacji sterownika zapisu dºwiÍku"},
	/* ru */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_RU, "Specify initialization parameters for recording sound driver"},
	/* sv */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_SV, "Ange initieringsflaggor fˆr ljuddrivrutin fˆr inspelning"},
	/* tr */ {IDCLS_SPECIFY_REC_SOUND_DRIVER_PARAM_TR, "Ses kay˝t s¸r¸c¸s¸ iÁin ba˛lang˝Á parametrelerini belirt"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_P_SYNC,    N_("<sync>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_SYNC_DA, "<sync>"},
	/* de */ {IDCLS_P_SYNC_DE, "<Sync>"},
	/* es */ {IDCLS_P_SYNC_ES, "<sincronÌa>"},
	/* fr */ {IDCLS_P_SYNC_FR, "<sync>"},
	/* hu */ {IDCLS_P_SYNC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_SYNC_IT, "<sync>"},
	/* ko */ {IDCLS_P_SYNC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_SYNC_NL, "<sync>"},
	/* pl */ {IDCLS_P_SYNC_PL, "<sync>"},
	/* ru */ {IDCLS_P_SYNC_RU, "<sync>"},
	/* sv */ {IDCLS_P_SYNC_SV, "<synk>"},
	/* tr */ {IDCLS_P_SYNC_TR, "<senkron>"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SET_SOUND_SPEED_ADJUST,    N_("Set sound speed adjustment (0: flexible, 1: adjusting, 2: exact)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_SOUND_SPEED_ADJUST_DA, "Indstil lydjusteringshastighed (0: fleksibel, 1: justerende, 2: n¯jagtig)"},
	/* de */ {IDCLS_SET_SOUND_SPEED_ADJUST_DE, "Setze Sound Geschwindigkeit Anpassung (0: flexibel, 1: anpassend, 2: exakt)"},
	/* es */ {IDCLS_SET_SOUND_SPEED_ADJUST_ES, "Ajuste velocidad sonido (0: flexible, 1: ajustado, 2: exacto)"},
	/* fr */ {IDCLS_SET_SOUND_SPEED_ADJUST_FR, "Choisir la mÈthode d'ajustement du son (0: flexible, 1: ajustÈ 2: exact)"},
	/* hu */ {IDCLS_SET_SOUND_SPEED_ADJUST_HU, "Adja meg a zene sebessÈg igazÌt·s·t (0: rugalmas. 1: igazodÛ, 2: pontos)"},
	/* it */ {IDCLS_SET_SOUND_SPEED_ADJUST_IT, "Imposta il tipo di adattamento della velocit‡ audio (0: flessibile, 1: adattabile, 2: esatta)"},
	/* ko */ {IDCLS_SET_SOUND_SPEED_ADJUST_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_SOUND_SPEED_ADJUST_NL, "Stel geluidssnelheid aanpassing in (0: flexibel, 1: aanpassend, 2: exact)"},
	/* pl */ {IDCLS_SET_SOUND_SPEED_ADJUST_PL, "Ustaw dopasowanie prÍdko∂ci dºwiÍku (0: elastyczne, 1: dopasowane, 2: dok≥adne)"},
	/* ru */ {IDCLS_SET_SOUND_SPEED_ADJUST_RU, "Set sound speed adjustment (0: flexible, 1: adjusting, 2: exact)"},
	/* sv */ {IDCLS_SET_SOUND_SPEED_ADJUST_SV, "St‰ll in ljudhastighetsjustering (0: flexibel, 1: justerande, 2: exakt)"},
	/* tr */ {IDCLS_SET_SOUND_SPEED_ADJUST_TR, "Ses h˝z˝ ayarlamas˝n˝ yap˝n (0: esnek, 1: d¸zeltme, 2: aynen)"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_P_OUTPUT_MODE,    N_("<output mode>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_OUTPUT_MODE_DA, "<Uddata-tilstand>"},
	/* de */ {IDCLS_P_OUTPUT_MODE_DE, "<Ausgabe Modus>"},
	/* es */ {IDCLS_P_OUTPUT_MODE_ES, "<Modo de salida>"},
	/* fr */ {IDCLS_P_OUTPUT_MODE_FR, "<mode de sortie>"},
	/* hu */ {IDCLS_P_OUTPUT_MODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_OUTPUT_MODE_IT, "<modalit‡ di uscita>"},
	/* ko */ {IDCLS_P_OUTPUT_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_OUTPUT_MODE_NL, "<uitvoer modus>"},
	/* pl */ {IDCLS_P_OUTPUT_MODE_PL, "<tryb wyj∂cia>"},
	/* ru */ {IDCLS_P_OUTPUT_MODE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_OUTPUT_MODE_SV, "<utdatal‰ge>"},
	/* tr */ {IDCLS_P_OUTPUT_MODE_TR, "<Á˝k˝˛ modu>"},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SOUND_OUTPUT_MODE,    N_("Sound output mode: (0: system decides mono/stereo, 1: always mono, 2: always stereo)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SOUND_OUTPUT_MODE_DA, "Lydindstillinger: "},
	/* de */ {IDCLS_SOUND_OUTPUT_MODE_DE, "Ton Ausgabe Modus: "},
	/* es */ {IDCLS_SOUND_OUTPUT_MODE_ES, "Modo salida sonido: "},
	/* fr */ {IDCLS_SOUND_OUTPUT_MODE_FR, "Mode de sortie de son: "},
	/* hu */ {IDCLS_SOUND_OUTPUT_MODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SOUND_OUTPUT_MODE_IT, "Modalit‡ di uscita audio: "},
	/* ko */ {IDCLS_SOUND_OUTPUT_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SOUND_OUTPUT_MODE_NL, "Geluid uitvoer modus: "},
	/* pl */ {IDCLS_SOUND_OUTPUT_MODE_PL, "Tryb wyj∂cia dºwiÍku: "},
	/* ru */ {IDCLS_SOUND_OUTPUT_MODE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SOUND_OUTPUT_MODE_SV, "Utdatal‰ge fˆr ljud: "},
	/* tr */ {IDCLS_SOUND_OUTPUT_MODE_TR, "Ses Á˝k˝˛˝ modu: "},
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SOUND_SUSPEND_TIME,    N_("Specify the pause interval when audio underflows (clicks) happen. 0 means no pause is done.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SOUND_SUSPEND_TIME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SOUND_SUSPEND_TIME_DE, "Pause Interval bei Audio Unterlauf (Klicks) w‰hlen. (0: keine Pause)"},
	/* es */ {IDCLS_SOUND_SUSPEND_TIME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SOUND_SUSPEND_TIME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SOUND_SUSPEND_TIME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SOUND_SUSPEND_TIME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SOUND_SUSPEND_TIME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SOUND_SUSPEND_TIME_NL, "Geef de pauze interval wanneer audio onderloopt (kliks) gebeurt: 0 betekent dat er geen pauze zal gebeuren."},
	/* pl */ {IDCLS_SOUND_SUSPEND_TIME_PL, "Okre∂l czas przerwy w przypadku niedope≥nienia audio (trzaskÛw). 0 oznacza brak przerwy."},
	/* ru */ {IDCLS_SOUND_SUSPEND_TIME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SOUND_SUSPEND_TIME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SOUND_SUSPEND_TIME_TR, ""},  /* fuzzy */
#endif
	
	/* sound.c */
	/* en */ {IDCLS_SOUND_VOLUME,    N_("Specify the sound volume (0..100)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SOUND_VOLUME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SOUND_VOLUME_DE, "Sound Lautst‰rke spezifieren (0..100)"},
	/* es */ {IDCLS_SOUND_VOLUME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SOUND_VOLUME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SOUND_VOLUME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SOUND_VOLUME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SOUND_VOLUME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SOUND_VOLUME_NL, "Geef het geluidsvolume (0..100)"},
	/* pl */ {IDCLS_SOUND_VOLUME_PL, "Okre∂l g≥o∂no∂Ê dºwiÍku (0..100)"},
	/* ru */ {IDCLS_SOUND_VOLUME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SOUND_VOLUME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SOUND_VOLUME_TR, ""},  /* fuzzy */
#endif
	
	/* sysfile.c */
	/* en */ {IDCLS_P_PATH,    N_("<Path>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_PATH_DA, "<Sti>"},
	/* de */ {IDCLS_P_PATH_DE, "<Pfad>"},
	/* es */ {IDCLS_P_PATH_ES, "<Camino>"},
	/* fr */ {IDCLS_P_PATH_FR, "<Emplacement>"},
	/* hu */ {IDCLS_P_PATH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_PATH_IT, "<Percorso>"},
	/* ko */ {IDCLS_P_PATH_KO, "<±Ê>"},
	/* nl */ {IDCLS_P_PATH_NL, "<Pad>"},
	/* pl */ {IDCLS_P_PATH_PL, "<¶cieøka>"},
	/* ru */ {IDCLS_P_PATH_RU, "<ø„‚Ï>"},
	/* sv */ {IDCLS_P_PATH_SV, "<Sˆkv‰g>"},
	/* tr */ {IDCLS_P_PATH_TR, "<Yol>"},
#endif
	
	/* sysfile.c */
	/* en */ {IDCLS_DEFINE_SYSTEM_FILES_PATH,    N_("Define search path to locate system files")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_DA, "Angiv sti til systemfiler"},
	/* de */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_DE, "Suchpfad f¸r Systemdateien w‰hlen"},
	/* es */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_ES, "Definir camino de b˙squeda para los ficheros locales de sistema"},
	/* fr */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_FR, "DÈfinir le chemin de recherche pour trouver les fichiers systËmes"},
	/* hu */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_HU, "KeresÈsi ˙tvonal megad·sa a rendszerf·jlok megtal·l·s·ra"},
	/* it */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_IT, "Definisci il path di ricerca per cercare i file di sistema"},
	/* ko */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_NL, "Geef het zoek pad waar de systeem bestanden te vinden zijn"},
	/* pl */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_PL, "Definiuj ∂cieøkÍ wyszukiwania plikÛw systemowych"},
	/* ru */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_RU, "Define search path to locate system files"},
	/* sv */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_SV, "Ange sˆkv‰g fˆr att hitta systemfiler"},
	/* tr */ {IDCLS_DEFINE_SYSTEM_FILES_PATH_TR, "Sistem dosyalar˝n˝n konumunu belirlemek iÁin arama yolu tan˝mlay˝n"},
#endif
	
	/* traps.c */
	/* en */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION,    N_("Enable general mechanisms for fast disk/tape emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_DA, "AktivÈr generelle mekanismer for hurtig disk-/bÂndemulering"},
	/* de */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_DE, "Allgemeine Mechanismen f¸r schnelle Disk/Band Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_ES, "Permitir mecanismos generales para emulaciÛn r·pida de cinta/disco"},
	/* fr */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_FR, "Activer les mÈchanismes gÈnÈraux pour l'Èmulation disque/datassette rapide"},
	/* hu */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_HU, "A lemez/szalag emul·ciÛt gyorsÌtÛ mechanizmusok engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_IT, "Attiva meccanismo generale per l'emulazione veloce del disco/cassetta"},
	/* ko */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_NL, "Algemene methoden voor snelle disk/tape emulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_PL, "W≥±cz mechanizm szybkiej emulacji dysku i ta∂my"},
	/* ru */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_RU, "Enable general mechanisms for fast disk/tape emulation"},
	/* sv */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_SV, "Aktivera generella mekanismer fˆr snabb disk-/bandemulering"},
	/* tr */ {IDCLS_ENABLE_TRAPS_FAST_EMULATION_TR, "H˝zl˝ disk/teyp em¸lasyonu iÁin genel mekanizmay˝ aktif et"},
#endif
	
	/* traps.c */
	/* en */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION,    N_("Disable general mechanisms for fast disk/tape emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_DA, "DeaktivÈr generelle mekanismer for hurtig disk-/bÂndemulering"},
	/* de */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_DE, "Allgemeine Mechanismen f¸r schnelle Disk/Band Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_ES, "Deshabilitar mecanismos generales para emulaciÛn r·pida de cinta/disco"},
	/* fr */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_FR, "DÈsactiver les mÈchanismes gÈnÈraux pour l'Èmulation disque/datassette rapide"},
	/* hu */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_HU, "A lemez/szalag emul·ciÛt gyorsÌtÛ mechanizmusok tilt·sa"},
	/* it */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_IT, "Disattiva meccanismo generale per l'emulazione veloce del disco/cassetta"},
	/* ko */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_NL, "Algemene methoden voor snelle disk/tape emulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_PL, "Wy≥±cz mechanizm szybkiej emulacji dysku i ta∂my"},
	/* ru */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_RU, "Disable general mechanisms for fast disk/tape emulation"},
	/* sv */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_SV, "Inaktivera generella mekanismer fˆr snabb disk-/bandemulering"},
	/* tr */ {IDCLS_DISABLE_TRAPS_FAST_EMULATION_TR, "H˝zl˝ disk/teyp em¸lasyonu iÁin genel mekanizmay˝ pasifle˛tir"},
#endif
	
	/* vsync.c */
	/* en */ {IDCLS_LIMIT_SPEED_TO_VALUE,    N_("Limit emulation speed to specified value")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_LIMIT_SPEED_TO_VALUE_DA, "BegrÊns emuleringshastighed til angiven vÊrdi"},
	/* de */ {IDCLS_LIMIT_SPEED_TO_VALUE_DE, "Emulationsgeschwindigkeit auf Wert beschr‰nken."},
	/* es */ {IDCLS_LIMIT_SPEED_TO_VALUE_ES, "Limitar la velocidad de emulaciÛn al valor indicado"},
	/* fr */ {IDCLS_LIMIT_SPEED_TO_VALUE_FR, "Limiter la vitesse d'Èmulation ‡ une valeur specifiÈe"},
	/* hu */ {IDCLS_LIMIT_SPEED_TO_VALUE_HU, "Emul·ciÛs sebessÈg lehat·rol·sa adott ÈrtÈkre"},
	/* it */ {IDCLS_LIMIT_SPEED_TO_VALUE_IT, "Limita la velocit‡ di emulazione al valore specificato"},
	/* ko */ {IDCLS_LIMIT_SPEED_TO_VALUE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_LIMIT_SPEED_TO_VALUE_NL, "Beperk emulatiesnelheid tot de opgegeven waarde"},
	/* pl */ {IDCLS_LIMIT_SPEED_TO_VALUE_PL, "Ograniczaj prÍdko∂Ê emulacji do podanej warto∂ci"},
	/* ru */ {IDCLS_LIMIT_SPEED_TO_VALUE_RU, "æ”‡–›ÿÁÿ‚Ï ·⁄ﬁ‡ﬁ·‚Ï Ì‹„€ÔÊÿÿ ‘ﬁ ﬁﬂ‡’‘’€’››ﬁ”ﬁ ◊›–Á’›ÿÔ"},
	/* sv */ {IDCLS_LIMIT_SPEED_TO_VALUE_SV, "Begr‰nsa emuleringshastighet till angivet v‰rde"},
	/* tr */ {IDCLS_LIMIT_SPEED_TO_VALUE_TR, "Em¸lasyon h˝z˝n˝ belirtilmi˛ deer ile limitle"},
#endif
	
	/* vsync.c */
	/* en */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES,    N_("Update every <value> frames (`0' for automatic)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_DA, "OpdatÈr for hver <vÊrdi> billeder (\"0\" for automatisk)"},
	/* de */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_DE, "Jedes <Wert> Bild aktualisieren (`0' f¸r Automatik)"},
	/* es */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_ES, "Actualizar cada <valor> cuadros (`0' for automatic)"},
	/* fr */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_FR, "Mise ‡ jour toutes les <valeur> images (`0' pour auto.)"},
	/* hu */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_HU, "FrissÌtsen minden <ÈrtÈk> kÈpkocka elteltÈvel (0 automatikust jelent)"},
	/* it */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_IT, "Aggiorna ogni <valore> frame (`0' per automatico)"},
	/* ko */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_NL, "Elke <waarde> beeld vernieuwen (`0' voor automatisch)"},
	/* pl */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_PL, "Aktualizuj co <warto∂Ê> klatek (`0' - automatycznie)"},
	/* ru */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_RU, "Update every <value> frames (`0' for automatic)"},
	/* sv */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_SV, "Uppdatera varje <v‰rde> ramar (\"0\" fˆr automatiskt)"},
	/* tr */ {IDCLS_UPDATE_EVERY_VALUE_FRAMES_TR, "Her <deer> framede bir g¸ncelle (otomatik iÁin `0')"},
#endif
	
	/* vsync.c */
	/* en */ {IDCLS_ENABLE_WARP_MODE,    N_("Enable warp mode")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_WARP_MODE_DA, "AktivÈr warp-tilstand"},
	/* de */ {IDCLS_ENABLE_WARP_MODE_DE, "Warp Mode aktivieren"},
	/* es */ {IDCLS_ENABLE_WARP_MODE_ES, "Permitir m·xima velocidad"},
	/* fr */ {IDCLS_ENABLE_WARP_MODE_FR, "Activer le Mode Turbo"},
	/* hu */ {IDCLS_ENABLE_WARP_MODE_HU, "Hipergyors mÛd engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_WARP_MODE_IT, "Attiva modalit‡ turbo"},
	/* ko */ {IDCLS_ENABLE_WARP_MODE_KO, "ø˙«¡ ∏µÂ ªÁøÎ∞°¥…«œ∞‘ «œ±‚"},
	/* nl */ {IDCLS_ENABLE_WARP_MODE_NL, "Activeer warpmodus"},
	/* pl */ {IDCLS_ENABLE_WARP_MODE_PL, "W≥±cz tryb przy∂pieszony (Turbo)"},
	/* ru */ {IDCLS_ENABLE_WARP_MODE_RU, "Enable warp mode"},
	/* sv */ {IDCLS_ENABLE_WARP_MODE_SV, "Aktivera warpl‰ge"},
	/* tr */ {IDCLS_ENABLE_WARP_MODE_TR, "Warp modu aktif et"},
#endif
	
	/* vsync.c */
	/* en */ {IDCLS_DISABLE_WARP_MODE,    N_("Disable warp mode")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_WARP_MODE_DA, "DeaktivÈr warp-tilstand"},
	/* de */ {IDCLS_DISABLE_WARP_MODE_DE, "Warp Mode deaktivieren"},
	/* es */ {IDCLS_DISABLE_WARP_MODE_ES, "Deshabilitar modo Ultrar·pido"},
	/* fr */ {IDCLS_DISABLE_WARP_MODE_FR, "DÈsactiver le Mode Turbo"},
	/* hu */ {IDCLS_DISABLE_WARP_MODE_HU, "Hipergyors mÛd tilt·sa"},
	/* it */ {IDCLS_DISABLE_WARP_MODE_IT, "Disattiva modalit‡ turbo"},
	/* ko */ {IDCLS_DISABLE_WARP_MODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_WARP_MODE_NL, "Warpmodus uitschakelen"},
	/* pl */ {IDCLS_DISABLE_WARP_MODE_PL, "Wy≥±cz tryb przyspieszony"},
	/* ru */ {IDCLS_DISABLE_WARP_MODE_RU, "Disable warp mode"},
	/* sv */ {IDCLS_DISABLE_WARP_MODE_SV, "Inaktivera warpl‰ge"},
	/* tr */ {IDCLS_DISABLE_WARP_MODE_TR, "Warp modu pasifle˛tir"},
#endif
	
	/* translate.c */
	/* en */ {IDCLS_P_ISO_LANGUAGE_CODE,    N_("<iso language code>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_ISO_LANGUAGE_CODE_DA, "<iso-sprogkode>"},
	/* de */ {IDCLS_P_ISO_LANGUAGE_CODE_DE, "<iso Sprachcode>"},
	/* es */ {IDCLS_P_ISO_LANGUAGE_CODE_ES, "<cÛdigo ISO idioma>"},
	/* fr */ {IDCLS_P_ISO_LANGUAGE_CODE_FR, "<code language ISO>"},
	/* hu */ {IDCLS_P_ISO_LANGUAGE_CODE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_ISO_LANGUAGE_CODE_IT, "<codice iso lingua>"},
	/* ko */ {IDCLS_P_ISO_LANGUAGE_CODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_ISO_LANGUAGE_CODE_NL, "<iso taalcode>"},
	/* pl */ {IDCLS_P_ISO_LANGUAGE_CODE_PL, "<Kod jÍzykowy iso>"},
	/* ru */ {IDCLS_P_ISO_LANGUAGE_CODE_RU, "<iso language code>"},
	/* sv */ {IDCLS_P_ISO_LANGUAGE_CODE_SV, "<iso-sprÂkkod>"},
	/* tr */ {IDCLS_P_ISO_LANGUAGE_CODE_TR, "<iso dil kodu>"},
#endif
	
	/* translate.c */
	/* en */ {IDCLS_SPECIFY_ISO_LANG_CODE,    N_("Specify the iso code of the language")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_ISO_LANG_CODE_DA, "Angiv ISO-koden for sproget"},
	/* de */ {IDCLS_SPECIFY_ISO_LANG_CODE_DE, "Iso Sprachcode spezifizieren"},
	/* es */ {IDCLS_SPECIFY_ISO_LANG_CODE_ES, "Especificar el cÛdigo ISO para el idioma"},
	/* fr */ {IDCLS_SPECIFY_ISO_LANG_CODE_FR, "SpÈcifier le code iso du langage"},
	/* hu */ {IDCLS_SPECIFY_ISO_LANG_CODE_HU, "Adja meg a nyelv iso kÛdj·t"},
	/* it */ {IDCLS_SPECIFY_ISO_LANG_CODE_IT, "Specifica il codice ISO della lingua"},
	/* ko */ {IDCLS_SPECIFY_ISO_LANG_CODE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_ISO_LANG_CODE_NL, "Geef de isocode van de taal"},
	/* pl */ {IDCLS_SPECIFY_ISO_LANG_CODE_PL, "Okre∂l kod iso jÍzyka"},
	/* ru */ {IDCLS_SPECIFY_ISO_LANG_CODE_RU, "Specify the iso code of the language"},
	/* sv */ {IDCLS_SPECIFY_ISO_LANG_CODE_SV, "Ange ISO-koden fˆr sprÂket"},
	/* tr */ {IDCLS_SPECIFY_ISO_LANG_CODE_TR, "Dil iÁin iso kodu belirt"},
#endif
	
	/* c64/plus256k.c */
	/* en */ {IDCLS_SPECIFY_PLUS256K_NAME,    N_("Specify name of PLUS256K image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PLUS256K_NAME_DA, "Angiv navn pÂ PLUS256K-image"},
	/* de */ {IDCLS_SPECIFY_PLUS256K_NAME_DE, "Name der PLUS256K Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_PLUS256K_NAME_ES, "Especificar nombre imagen RAM PLUS256K"},
	/* fr */ {IDCLS_SPECIFY_PLUS256K_NAME_FR, "SpÈcifier le nom de l'image PLUS256K RAM"},
	/* hu */ {IDCLS_SPECIFY_PLUS256K_NAME_HU, "Adja meg a PLUS256K-s kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_PLUS256K_NAME_IT, "Specifica il nome immagine PLUS256K"},
	/* ko */ {IDCLS_SPECIFY_PLUS256K_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PLUS256K_NAME_NL, "Geef de naam van het PLUS256K bestand"},
	/* pl */ {IDCLS_SPECIFY_PLUS256K_NAME_PL, "Okre∂l nazwÍ obrazu PLUS256K"},
	/* ru */ {IDCLS_SPECIFY_PLUS256K_NAME_RU, "Specify name of PLUS256K image"},
	/* sv */ {IDCLS_SPECIFY_PLUS256K_NAME_SV, "Ange namn pÂ PLUS256K-avbildning"},
	/* tr */ {IDCLS_SPECIFY_PLUS256K_NAME_TR, "PLUS256K imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/plus60k.c */
	/* en */ {IDCLS_SPECIFY_PLUS60K_NAME,    N_("Specify name of PLUS60K image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_PLUS60K_NAME_DA, "Angiv navn pÂ PLUS60K-image"},
	/* de */ {IDCLS_SPECIFY_PLUS60K_NAME_DE, "Name der PLUS60K Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_PLUS60K_NAME_ES, "Especificar nombre imagen PLUS60K"},
	/* fr */ {IDCLS_SPECIFY_PLUS60K_NAME_FR, "SpÈcifier le nom de l'image PLUS60K RAM"},
	/* hu */ {IDCLS_SPECIFY_PLUS60K_NAME_HU, "Adja meg a nevÈt a PLUS60K kÈpm·snak"},
	/* it */ {IDCLS_SPECIFY_PLUS60K_NAME_IT, "Specifica il nome immagine PLUS60K"},
	/* ko */ {IDCLS_SPECIFY_PLUS60K_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_PLUS60K_NAME_NL, "Geef de naam van het PLUS60K bestand"},
	/* pl */ {IDCLS_SPECIFY_PLUS60K_NAME_PL, "Okre∂l nazwÍ obrazu PLUS60K"},
	/* ru */ {IDCLS_SPECIFY_PLUS60K_NAME_RU, "Specify name of PLUS60K image"},
	/* sv */ {IDCLS_SPECIFY_PLUS60K_NAME_SV, "Ange namn pÂ plUS60K-avbildning"},
	/* tr */ {IDCLS_SPECIFY_PLUS60K_NAME_TR, "PLUS60K imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/plus60k.c */
	/* en */ {IDCLS_PLUS60K_BASE,    N_("Base address of the PLUS60K expansion. (0xD040/0xD100)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_PLUS60K_BASE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_PLUS60K_BASE_DE, "Basis Adresse f¸r PLUS60K Erweiterung. (0xD040/0xD100)"},
	/* es */ {IDCLS_PLUS60K_BASE_ES, "DirecciÛn base de la expansiÛn PLUS60K. (0xD040/0xD100)"},
	/* fr */ {IDCLS_PLUS60K_BASE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_PLUS60K_BASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_PLUS60K_BASE_IT, "Indirizzo base espansione PLUS60K. (0xD040/0xD100)"},
	/* ko */ {IDCLS_PLUS60K_BASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_PLUS60K_BASE_NL, "Basisadres van de PLUS60K geheugenuitbreiding. (0xD040/0xD100)"},
	/* pl */ {IDCLS_PLUS60K_BASE_PL, "Adres bazowy rozszerzenia PLUS60K. (0xD040/0xD100)"},
	/* ru */ {IDCLS_PLUS60K_BASE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_PLUS60K_BASE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_PLUS60K_BASE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/c64_256k.c */
	/* en */ {IDCLS_SPECIFY_C64_256K_NAME,    N_("Specify name of 256K image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_C64_256K_NAME_DA, "Angiv navn pÂ 256k-image"},
	/* de */ {IDCLS_SPECIFY_C64_256K_NAME_DE, "Name der 256K Imagedatei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_C64_256K_NAME_ES, "Especificar nombre imagen 256K"},
	/* fr */ {IDCLS_SPECIFY_C64_256K_NAME_FR, "SpÈcifier le nom de l'image 256K"},
	/* hu */ {IDCLS_SPECIFY_C64_256K_NAME_HU, "Adja meg a 256K-s kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_C64_256K_NAME_IT, "Specifica il nome immagine 256K"},
	/* ko */ {IDCLS_SPECIFY_C64_256K_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_C64_256K_NAME_NL, "Geef de naam van het 256K bestand"},
	/* pl */ {IDCLS_SPECIFY_C64_256K_NAME_PL, "Okre∂l nazwÍ obrazu 256K"},
	/* ru */ {IDCLS_SPECIFY_C64_256K_NAME_RU, "Specify name of 256K image"},
	/* sv */ {IDCLS_SPECIFY_C64_256K_NAME_SV, "Ange namn pÂ 256K-avbildning"},
	/* tr */ {IDCLS_SPECIFY_C64_256K_NAME_TR, "256K imaj˝n˝n ismini belirt"},
#endif
	
	/* c64/c64_256k.c */
	/* en */ {IDCLS_C64_256K_BASE,    N_("Base address of the 256K expansion. (0xDE00/0xDE80/0xDF00/0xDF80)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_C64_256K_BASE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_C64_256K_BASE_DE, "Basis Adresse f¸r 256K RAM Erweiterung. (0xDE00/0xDE80/0xDF00/0xDF80)"},
	/* es */ {IDCLS_C64_256K_BASE_ES, "DirecciÛn base de la expansiÛn 256K. (0xDE00/0xDE80/0xDF00/0xDF80)"},
	/* fr */ {IDCLS_C64_256K_BASE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_C64_256K_BASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_C64_256K_BASE_IT, "Indirizzo base dell'espansione 256K. (0xDE00/0xDE80/0xDF00/0xDF80)"},
	/* ko */ {IDCLS_C64_256K_BASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_C64_256K_BASE_NL, "Basisadres van de 256K geheugenuitbreiding. (0xDE00/0xDE80/0xDF00/0xDF80)"},
	/* pl */ {IDCLS_C64_256K_BASE_PL, "Adres bazowy rozszerzenia 256K. (0xDE00/0xDE80/0xDF00/0xDF80)"},
	/* ru */ {IDCLS_C64_256K_BASE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_C64_256K_BASE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_C64_256K_BASE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/c64-memory-hacks.c */
	/* en */ {IDCLS_SET_C64_MEMORY_HACK,    N_("Set the 'memory expansion hack' device (0: None, 1: C64 256K, 2: +60K, 3: +256K)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_C64_MEMORY_HACK_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_C64_MEMORY_HACK_DE, "Speichererweiterungshack Ger‰t (0: kein, 1: C64 256K, 2: +60K, 3: +256K)"},
	/* es */ {IDCLS_SET_C64_MEMORY_HACK_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_C64_MEMORY_HACK_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_C64_MEMORY_HACK_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_C64_MEMORY_HACK_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_C64_MEMORY_HACK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_C64_MEMORY_HACK_NL, "Stel het 'geheugen uitbreidingen hack' apparaat in (0: Geen, 1: C64 256K, 2: +60K, 3: +256K)"},
	/* pl */ {IDCLS_SET_C64_MEMORY_HACK_PL, "Ustaw urz±dzenie 'hacka' rozszerzenia pamiÍci (0: Brak, 1: 256K C64, 2: +60K, 3: +256K)"},
	/* ru */ {IDCLS_SET_C64_MEMORY_HACK_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_C64_MEMORY_HACK_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_C64_MEMORY_HACK_TR, ""},  /* fuzzy */
#endif
	
	/* plus4/plus4memhacks.c */
	/* en */ {IDCLS_SET_PLUS4_MEMORY_HACK,    N_("Set the 'memory expansion hack' device (0: None, 1: CSORY 256K, 2: HANNES 256K, 3: HANNES 1024K, 4: HANNES 4096K)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_PLUS4_MEMORY_HACK_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_PLUS4_MEMORY_HACK_DE, "Speicher Erweiterungshack Ger‰t (0: Kein, 1: CSORY 256K, 2: HANNES 256K, 3: HANNES 1024K, 4: HANNES 4096K)"},
	/* es */ {IDCLS_SET_PLUS4_MEMORY_HACK_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_PLUS4_MEMORY_HACK_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_PLUS4_MEMORY_HACK_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_PLUS4_MEMORY_HACK_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_PLUS4_MEMORY_HACK_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_PLUS4_MEMORY_HACK_NL, "Stel het 'geheugen uitbreidingen hack' apparaat in (0: Geen, 1: CSORY 256K, 2: HANNES 256K, 3: HANNES 1024K, 4: HANNES 4096K)"},
	/* pl */ {IDCLS_SET_PLUS4_MEMORY_HACK_PL, "Ustaw urz±dzenie 'hacka' rozszerzenia pamiÍci (0: Brak, 1: 256K CSORY, 2: 256K HANNES, 3: 1024K HANNES, 4: 4096K HANNES)"},
	/* ru */ {IDCLS_SET_PLUS4_MEMORY_HACK_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_PLUS4_MEMORY_HACK_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_PLUS4_MEMORY_HACK_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_STB_CART,    N_("Attach raw Structured Basic cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_STB_CART_DA, "Tilslut rÂ Strucured BASIC-cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_STB_CART_DE, "Erweiterungsmodul Structured Basic (raw) Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_STB_CART_ES, "Insertar imagen cartucho Structured Basic"},
	/* fr */ {IDCLS_ATTACH_RAW_STB_CART_FR, "InsÈrer une cartouche Raw Structured Basic"},
	/* hu */ {IDCLS_ATTACH_RAW_STB_CART_HU, "Structured Basic cartridge kÈpm·s csatol·sa"},
	/* it */ {IDCLS_ATTACH_RAW_STB_CART_IT, "Seleziona immagine cartuccia Structured Basic"},
	/* ko */ {IDCLS_ATTACH_RAW_STB_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_STB_CART_NL, "Koppel binair Structured Basic cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_STB_CART_PL, "Zamontuj surowy obraz kartridøa Structured Basic"},
	/* ru */ {IDCLS_ATTACH_RAW_STB_CART_RU, "Attach raw Structured Basic cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_STB_CART_SV, "Anslut rÂ Strucured Basic-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_STB_CART_TR, "D¸z yap˝land˝r˝lm˝˛ Basic kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_SE5_CART,    N_("Attach raw 16kB Super Explode V5 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_SE5_CART_DA, "Tilslut rÂt 16kB Super Explode V5 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_SE5_CART_DE, "16kB Super Explode V5 Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_SE5_CART_ES, "Insertar imagen cartucho Super Explode V5 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_SE5_CART_FR, "Attacher une image de cartouche 16 ko Super Explode V5"},
	/* hu */ {IDCLS_ATTACH_RAW_SE5_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_SE5_CART_IT, "Seleziona immagine cartuccia Super Explode V5 di 16kB"},
	/* ko */ {IDCLS_ATTACH_RAW_SE5_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_SE5_CART_NL, "Koppel binair 16kB Super Snapshot V5 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_SE5_CART_PL, "Zamontuj surowy obraz kartridøa 16kB Super Explode V5"},
	/* ru */ {IDCLS_ATTACH_RAW_SE5_CART_RU, "Attach raw 16kB Super Explode V5 cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_SE5_CART_SV, "Anslut rÂ 16kB Super Explode V5-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_SE5_CART_TR, "D¸z 16kB Super Explode V5 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART,    N_("Attach raw 64kB Super Games cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_DA, "Tilslut rÂt 64kB Super Games cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_DE, "64kB Super Games Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_ES, "Insertar imagen cartucho Super Games 64KB"},
	/* fr */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_FR, "Attacher une image de cartouche 64 ko Super Games"},
	/* hu */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_IT, "Seleziona immagine cartuccia Super Games di 64kB"},
	/* ko */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_NL, "Koppel binair 64kB Super Games cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_PL, "Zamontuj surowy obraz kartridøa 64kB Super Games"},
	/* ru */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_RU, "Attach raw 64kB Super Games cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_SV, "Anslut rÂ 64kB-Super Games-insticksmodulfil"},
	/* tr */ {IDCLS_ATTACH_RAW_SUPER_GAMES_CART_TR, "D¸z 64kB Super Games kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART,    N_("Attach raw Silverrock 128 cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_DA, "Tilslut rÂt Silverrock 128 cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_DE, "Silverrock 128 Erweiterungsmodul (raw) Image einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_ES, "Insertar imagen cartucho Silverrock 128"},
	/* fr */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_FR, "Attacher une image de cartouche RAW Silverrock 128"},
	/* hu */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_IT, "Seleziona immagine cartuccia Silverrock 128"},
	/* ko */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_NL, "Koppel binair Silverrock 128 cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_PL, "Zamontuj surowy obraz kartridøa Silverrock 128"},
	/* ru */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_SV, "Anslut rÂ Silverrock 128-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_SILVERROCK_128_CART_TR, "D¸z Silverrock 128 kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64/cart/c64cart.c */
	/* en */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART,    N_("Attach raw 16kB Simons Basic cartridge image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_DA, "Tilslut rÂt 16kB Simons Basic cartridge image"},
	/* de */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_DE, "16kB Simons Basic Image (raw) einlegen"},
	/* es */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_ES, "Insertar imagen cartucho Simons Basic 16KB"},
	/* fr */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_FR, "Attacher une image de cartouche 16 ko Simons Basic"},
	/* hu */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_IT, "Seleziona immagine cartuccia Simons Basic di 16kB"},
	/* ko */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_NL, "Koppel binair 16kB Simons Basic cartridge bestand"},
	/* pl */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_PL, "Zamontuj surowy obraz kartridøa 16kB Simons Basic"},
	/* ru */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_RU, "Attach raw 16kB Simons Basic cartridge image"},
	/* sv */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_SV, "Anslut rÂ 16kB Simons Basic-insticksmodulavbildning"},
	/* tr */ {IDCLS_ATTACH_RAW_SIMONS_BASIC_CART_TR, "D¸z 16kB Simons Basic kartu˛ imaj˝ yerle˛tir"},
#endif
	
	/* c64dtv/c64dtvblitter.c */
	/* en */ {IDCLS_SPECIFY_DTV_REVISION,    N_("Specify DTV Revision (2: DTV2, 3: DTV3)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_DTV_REVISION_DA, "Angiv DTV-revision (2: DTV2, 3: DTV3)"},
	/* de */ {IDCLS_SPECIFY_DTV_REVISION_DE, "DTV Revision w‰hlen (2: DTV2, 3: DTV3)"},
	/* es */ {IDCLS_SPECIFY_DTV_REVISION_ES, "Especificar revisiÛn DTV (2: DTV2, 3: DTV3)"},
	/* fr */ {IDCLS_SPECIFY_DTV_REVISION_FR, "SpÈcifier la rÈvision DTV (2: DTV2, 3: DTV3)"},
	/* hu */ {IDCLS_SPECIFY_DTV_REVISION_HU, "Adja meg a DTV verziÛj·t (2: DTV2, 3: DTV3)"},
	/* it */ {IDCLS_SPECIFY_DTV_REVISION_IT, "Specifica la revisione DTV (2: DTV2, 3: DTV3)"},
	/* ko */ {IDCLS_SPECIFY_DTV_REVISION_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_DTV_REVISION_NL, "Geef de DTV revisie (2: DTV2, 3: DTV3)"},
	/* pl */ {IDCLS_SPECIFY_DTV_REVISION_PL, "Okre∂l wersjÍ DTV (2: DTV2, 3: DTV3)"},
	/* ru */ {IDCLS_SPECIFY_DTV_REVISION_RU, "Specify DTV Revision (2: DTV2, 3: DTV3)"},
	/* sv */ {IDCLS_SPECIFY_DTV_REVISION_SV, "Ange DTV-revision (2: DTV2, 3: DTV3)"},
	/* tr */ {IDCLS_SPECIFY_DTV_REVISION_TR, "DTV Revizyonunu belirt (2: DTV2, 3: DTV3)"},
#endif
	
	/* c64dtv/c64dtvblitter.c */
	/* en */ {IDCLS_ENABLE_DTV_BLITTER_LOG,    N_("Enable DTV blitter logs.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DTV_BLITTER_LOG_DA, "AktivÈr DTV-blitterlog."},
	/* de */ {IDCLS_ENABLE_DTV_BLITTER_LOG_DE, "DTV Blitter Log aktivieren."},
	/* es */ {IDCLS_ENABLE_DTV_BLITTER_LOG_ES, "Permitir informes blitter DTV"},
	/* fr */ {IDCLS_ENABLE_DTV_BLITTER_LOG_FR, "Activer la journalisation pour DTV blitter"},
	/* hu */ {IDCLS_ENABLE_DTV_BLITTER_LOG_HU, "DTV blit naplÛk engedÈlyezÈse."},
	/* it */ {IDCLS_ENABLE_DTV_BLITTER_LOG_IT, "Attiva log del blitter DTV."},
	/* ko */ {IDCLS_ENABLE_DTV_BLITTER_LOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DTV_BLITTER_LOG_NL, "DTV blitter logs inschakelen."},
	/* pl */ {IDCLS_ENABLE_DTV_BLITTER_LOG_PL, "W≥±cz logi blittera DTV."},
	/* ru */ {IDCLS_ENABLE_DTV_BLITTER_LOG_RU, "Enable DTV blitter logs."},
	/* sv */ {IDCLS_ENABLE_DTV_BLITTER_LOG_SV, "Aktivera DTV-blitterloggar."},
	/* tr */ {IDCLS_ENABLE_DTV_BLITTER_LOG_TR, "DTV blitter loglar˝n˝ aktif et"},
#endif
	
	/* c64dtv/c64dtvblitter.c */
	/* en */ {IDCLS_DISABLE_DTV_BLITTER_LOG,    N_("Disable DTV blitter logs.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DTV_BLITTER_LOG_DA, "DeaktivÈr DTV-blitterlog."},
	/* de */ {IDCLS_DISABLE_DTV_BLITTER_LOG_DE, "DTV Blitter Log deaktivieren."},
	/* es */ {IDCLS_DISABLE_DTV_BLITTER_LOG_ES, "Deshabilitar informes blitter DTV"},
	/* fr */ {IDCLS_DISABLE_DTV_BLITTER_LOG_FR, "DÈsactiver la journalisation pour DTV blitter"},
	/* hu */ {IDCLS_DISABLE_DTV_BLITTER_LOG_HU, "DTV blit naplÛk tilt·sa."},
	/* it */ {IDCLS_DISABLE_DTV_BLITTER_LOG_IT, "Disattiva log del blitter DTV."},
	/* ko */ {IDCLS_DISABLE_DTV_BLITTER_LOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DTV_BLITTER_LOG_NL, "DTV Blitter logs uitschakelen."},
	/* pl */ {IDCLS_DISABLE_DTV_BLITTER_LOG_PL, "Wy≥±cz logi blittera DTV"},
	/* ru */ {IDCLS_DISABLE_DTV_BLITTER_LOG_RU, "Disable DTV blitter logs."},
	/* sv */ {IDCLS_DISABLE_DTV_BLITTER_LOG_SV, "Inaktivera DTV-blitterloggar."},
	/* tr */ {IDCLS_DISABLE_DTV_BLITTER_LOG_TR, "DTV blitter loglar˝n˝ pasifle˛tir"},
#endif
	
	/* c64dtv/c64dtvdma.c */
	/* en */ {IDCLS_ENABLE_DTV_DMA_LOG,    N_("Enable DTV DMA logs.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DTV_DMA_LOG_DA, "AktivÈr DTV DMA-log."},
	/* de */ {IDCLS_ENABLE_DTV_DMA_LOG_DE, "DTV DMA Logs aktivieren."},
	/* es */ {IDCLS_ENABLE_DTV_DMA_LOG_ES, "Permitir informes DTV DMA."},
	/* fr */ {IDCLS_ENABLE_DTV_DMA_LOG_FR, "Activer la journalisation DMA DTV."},
	/* hu */ {IDCLS_ENABLE_DTV_DMA_LOG_HU, "DTV DMA naplÛk engedÈlyezÈse."},
	/* it */ {IDCLS_ENABLE_DTV_DMA_LOG_IT, "Attiva log DMA DTV."},
	/* ko */ {IDCLS_ENABLE_DTV_DMA_LOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DTV_DMA_LOG_NL, "DTV DMA logs inschakelen."},
	/* pl */ {IDCLS_ENABLE_DTV_DMA_LOG_PL, "W≥±cz logowanie DTV DMA"},
	/* ru */ {IDCLS_ENABLE_DTV_DMA_LOG_RU, "Enable DTV DMA logs."},
	/* sv */ {IDCLS_ENABLE_DTV_DMA_LOG_SV, "Aktivera DTV DMA-loggning."},
	/* tr */ {IDCLS_ENABLE_DTV_DMA_LOG_TR, "DTV DMA loglar˝n˝ aktif et"},
#endif
	
	/* c64dtv/c64dtvdma.c */
	/* en */ {IDCLS_DISABLE_DTV_DMA_LOG,    N_("Disable DTV DMA logs.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DTV_DMA_LOG_DA, "DeaktivÈr DTV DMA-log."},
	/* de */ {IDCLS_DISABLE_DTV_DMA_LOG_DE, "DTV DMA Logs deaktivieren."},
	/* es */ {IDCLS_DISABLE_DTV_DMA_LOG_ES, "Deshabilitar informes DTV DMA."},
	/* fr */ {IDCLS_DISABLE_DTV_DMA_LOG_FR, "DÈsactiver la journalisation DTV DMA."},
	/* hu */ {IDCLS_DISABLE_DTV_DMA_LOG_HU, "DTV DMA naplÛk tilt·sa."},
	/* it */ {IDCLS_DISABLE_DTV_DMA_LOG_IT, "Disattiva log DMA DTV."},
	/* ko */ {IDCLS_DISABLE_DTV_DMA_LOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DTV_DMA_LOG_NL, "DTV DMA logs uitschakelen."},
	/* pl */ {IDCLS_DISABLE_DTV_DMA_LOG_PL, "Wy≥±cz logowanie DTV DMA"},
	/* ru */ {IDCLS_DISABLE_DTV_DMA_LOG_RU, "Disable DTV DMA logs."},
	/* sv */ {IDCLS_DISABLE_DTV_DMA_LOG_SV, "Inaktivera DTV DMA-loggning."},
	/* tr */ {IDCLS_DISABLE_DTV_DMA_LOG_TR, "DTV DMA loglar˝n˝ pasifle˛tir"},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDCLS_SPECIFY_C64DTVROM_NAME,    N_("Specify name of C64DTV ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_C64DTVROM_NAME_DA, "Angiv navn pÂ C64DTV-ROM-image"},
	/* de */ {IDCLS_SPECIFY_C64DTVROM_NAME_DE, "Name von C64DTV ROM Datei w‰hlen"},
	/* es */ {IDCLS_SPECIFY_C64DTVROM_NAME_ES, "Especificar nombre imagen ROM C64DTV"},
	/* fr */ {IDCLS_SPECIFY_C64DTVROM_NAME_FR, "SpÈcifier le nom de l'image ROM C64DTV"},
	/* hu */ {IDCLS_SPECIFY_C64DTVROM_NAME_HU, "Adja meg a C64DTV ROM kÈpm·s nevÈt"},
	/* it */ {IDCLS_SPECIFY_C64DTVROM_NAME_IT, "Specifica il nome immagine ROM DTV"},
	/* ko */ {IDCLS_SPECIFY_C64DTVROM_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_C64DTVROM_NAME_NL, "Geef de naam van her C64DTV ROM bestand"},
	/* pl */ {IDCLS_SPECIFY_C64DTVROM_NAME_PL, "Okre∂l nazwÍ obrazu ROM C64DTV"},
	/* ru */ {IDCLS_SPECIFY_C64DTVROM_NAME_RU, "Specify name of C64DTV ROM image"},
	/* sv */ {IDCLS_SPECIFY_C64DTVROM_NAME_SV, "Ange namn pÂ C64DTV-ROM-avbildning"},
	/* tr */ {IDCLS_SPECIFY_C64DTVROM_NAME_TR, "C64DTV ROM imaj˝n˝n ismini belirt"},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDCLS_ENABLE_C64DTVROM_RW,    N_("Enable writes to C64DTV ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_C64DTVROM_RW_DA, "Tillad skrivning til DTV-ROM-image"},
	/* de */ {IDCLS_ENABLE_C64DTVROM_RW_DE, "Erlaube Schreibzugriff auf DTV ROM Datei"},
	/* es */ {IDCLS_ENABLE_C64DTVROM_RW_ES, "Permitir escritura en la imagen C64DTV ROM"},
	/* fr */ {IDCLS_ENABLE_C64DTVROM_RW_FR, "Permetre l'Ècriture ‡ l'image ROM C64DTV"},
	/* hu */ {IDCLS_ENABLE_C64DTVROM_RW_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_C64DTVROM_RW_IT, "Attiva scrittura su immagini ROM C64DTV"},
	/* ko */ {IDCLS_ENABLE_C64DTVROM_RW_KO, "C64DTV ROM ¿ÃπÃ¡ˆø° æ≤±‚ ∞°¥…«œ∞‘ «œ±‚"},
	/* nl */ {IDCLS_ENABLE_C64DTVROM_RW_NL, "Activeer schrijven naar C64DTV ROM bestand"},
	/* pl */ {IDCLS_ENABLE_C64DTVROM_RW_PL, "W≥±cz zapis do obrazu ROM C64DTV"},
	/* ru */ {IDCLS_ENABLE_C64DTVROM_RW_RU, "Enable writes to C64DTV ROM image"},
	/* sv */ {IDCLS_ENABLE_C64DTVROM_RW_SV, "Aktivera skrivning till C64DTV-ROM-avbildning"},
	/* tr */ {IDCLS_ENABLE_C64DTVROM_RW_TR, "C64DTV ROM imaj˝nda yazmalar˝ aktif et"},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDCLS_DISABLE_C64DTVROM_RW,    N_("Disable writes to C64DTV ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_C64DTVROM_RW_DA, "Forbyd skrivning til C64DTV-ROM-image."},
	/* de */ {IDCLS_DISABLE_C64DTVROM_RW_DE, "Schreibzugriff auf C64DTV ROM Image verhindern"},
	/* es */ {IDCLS_DISABLE_C64DTVROM_RW_ES, "Deshabilitar grabaciÛn en imagen  C64DTV ROM"},
	/* fr */ {IDCLS_DISABLE_C64DTVROM_RW_FR, "DÈsactiver l'Ècriture ‡ l'image ROM C64DTV"},
	/* hu */ {IDCLS_DISABLE_C64DTVROM_RW_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_C64DTVROM_RW_IT, "Disattiva scrittura sull'immagine ROM C64DTV"},
	/* ko */ {IDCLS_DISABLE_C64DTVROM_RW_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_C64DTVROM_RW_NL, "Schrijven naar C64DTV ROM bestand uitschakelen"},
	/* pl */ {IDCLS_DISABLE_C64DTVROM_RW_PL, "Wy≥±cz zapis do obrazu ROM C64DTV"},
	/* ru */ {IDCLS_DISABLE_C64DTVROM_RW_RU, "Disable writes to C64DTV ROM image"},
	/* sv */ {IDCLS_DISABLE_C64DTVROM_RW_SV, "Inaktivera skrivning till C64DTV-ROM-avbildning"},
	/* tr */ {IDCLS_DISABLE_C64DTVROM_RW_TR, "C64DTV ROM imaj˝na yazmay˝ pasifle˛tir"},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDCLS_ENABLE_DTV_FLASH_LOG,    N_("Enable DTV flash chip logs.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DTV_FLASH_LOG_DA, "AktivÈr logs for DTV-flashkreds."},
	/* de */ {IDCLS_ENABLE_DTV_FLASH_LOG_DE, "DTV Flashchip Log aktivieren."},
	/* es */ {IDCLS_ENABLE_DTV_FLASH_LOG_ES, "Permitir informes chip DTV flash"},
	/* fr */ {IDCLS_ENABLE_DTV_FLASH_LOG_FR, "Activer la journalisation pour DTV Flash Chip"},
	/* hu */ {IDCLS_ENABLE_DTV_FLASH_LOG_HU, "DTV flash chip naplÛk engedÈlyezÈse."},
	/* it */ {IDCLS_ENABLE_DTV_FLASH_LOG_IT, "Attiva log del chip flash DTV."},
	/* ko */ {IDCLS_ENABLE_DTV_FLASH_LOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DTV_FLASH_LOG_NL, "DTV flash chip logs inschakelen."},
	/* pl */ {IDCLS_ENABLE_DTV_FLASH_LOG_PL, "W≥±cz logi uk≥adu flash DTV."},
	/* ru */ {IDCLS_ENABLE_DTV_FLASH_LOG_RU, "Enable DTV flash chip logs."},
	/* sv */ {IDCLS_ENABLE_DTV_FLASH_LOG_SV, "Aktivera loggar fˆr DTV-flashkrets."},
	/* tr */ {IDCLS_ENABLE_DTV_FLASH_LOG_TR, "DTV flash Áip loglar˝n˝ aktif et."},
#endif
	
	/* c64dtv/c64dtvflash.c */
	/* en */ {IDCLS_DISABLE_DTV_FLASH_LOG,    N_("Disable DTV flash chip logs.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DTV_FLASH_LOG_DA, "DeaktivÈr logs for DTV-flashkreds."},
	/* de */ {IDCLS_DISABLE_DTV_FLASH_LOG_DE, "DTV Flashchip Log deaktivieren."},
	/* es */ {IDCLS_DISABLE_DTV_FLASH_LOG_ES, "Inhabilitar informes chip DTV flash"},
	/* fr */ {IDCLS_DISABLE_DTV_FLASH_LOG_FR, "DÈsactiver la journalisation pour DTV Flash Chip"},
	/* hu */ {IDCLS_DISABLE_DTV_FLASH_LOG_HU, "DTV flash chip naplÛk tilt·sa."},
	/* it */ {IDCLS_DISABLE_DTV_FLASH_LOG_IT, "Disattiva log del chip flash DTV."},
	/* ko */ {IDCLS_DISABLE_DTV_FLASH_LOG_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DTV_FLASH_LOG_NL, "DTV flash chip logs uitschakelen."},
	/* pl */ {IDCLS_DISABLE_DTV_FLASH_LOG_PL, "Wy≥±cz logi uk≥adu flash DTV"},
	/* ru */ {IDCLS_DISABLE_DTV_FLASH_LOG_RU, "Disable DTV flash chip logs."},
	/* sv */ {IDCLS_DISABLE_DTV_FLASH_LOG_SV, "Inaktivera loggar fˆr DTV-flashkrets."},
	/* tr */ {IDCLS_DISABLE_DTV_FLASH_LOG_TR, "DTV flash Áip loglar˝n˝ pasifle˛tir."},
#endif
	
	/* c64dtv/flash-trap.c */
	/* en */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS,    N_("Use <name> as directory for flash file system device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_DA, "Brug <navn> som katalog for filsystembaseret flashenhed"},
	/* de */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_DE, "Benutze <Name> f¸r Verzeichnis Ger‰t Flash Dateisystem"},
	/* es */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_ES, "Usar <nombre> como directorio para perifÈrico flash files system"},
	/* fr */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_FR, "Utiliser le rÈpertoire <nom> comme systËme de fichiers flash"},
	/* hu */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_HU, "<nÈv> kˆnyvt·r haszn·lata a flash f·jlrendszer eszkˆzhˆz"},
	/* it */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_IT, "Una <nome> come directory per il file system flash"},
	/* ko */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_NL, "Gebruik <naam> als directory voor het flash bestandssysteem apparaat"},
	/* pl */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_PL, "Uøyj <nazwa> jako katalogu urz±dzenia systemu plikÛw flash"},
	/* ru */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_RU, "Use <name> as directory for flash file system device"},
	/* sv */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_SV, "Ange <namn> som katalog fˆr filsystemsbaserad flashenhet"},
	/* tr */ {IDCLS_USE_AS_DIRECTORY_FLASH_FS_TR, "Flash dosya sistem ayg˝t˝ iÁin <isim>'i dizin olarak kullan"},
#endif
	
	/* c64dtv/flash-trap.c */
	/* en */ {IDCLS_ENABLE_TRUE_FLASH_FS,    N_("Enable true hardware flash file system")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TRUE_FLASH_FS_DA, "AktivÈr Êgte hardwareemulering af flashfilsystem"},
	/* de */ {IDCLS_ENABLE_TRUE_FLASH_FS_DE, "Pr‰zises Hardware Flashdateisystem aktivieren"},
	/* es */ {IDCLS_ENABLE_TRUE_FLASH_FS_ES, "Permitir flash file system verdadero (hardware)"},
	/* fr */ {IDCLS_ENABLE_TRUE_FLASH_FS_FR, "Activer le systËme de fichier flash matÈriel rÈel"},
	/* hu */ {IDCLS_ENABLE_TRUE_FLASH_FS_HU, "Igazi hardver flash f·jlrendszer engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_TRUE_FLASH_FS_IT, "Attiva emulazione hardware del file system flash"},
	/* ko */ {IDCLS_ENABLE_TRUE_FLASH_FS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TRUE_FLASH_FS_NL, "Activeer hardwarmatige flash bestandssysteem"},
	/* pl */ {IDCLS_ENABLE_TRUE_FLASH_FS_PL, "W≥±cz prawdziwy sprzÍtowy system plikÛw flash"},
	/* ru */ {IDCLS_ENABLE_TRUE_FLASH_FS_RU, "Enable true hardware flash file system"},
	/* sv */ {IDCLS_ENABLE_TRUE_FLASH_FS_SV, "Aktivera ‰kta maskinvaru-flashfilsystem"},
	/* tr */ {IDCLS_ENABLE_TRUE_FLASH_FS_TR, "GerÁek donan˝m flash dosya sistemini aktif et"},
#endif
	
	/* c64dtv/flash-trap.c */
	/* en */ {IDCLS_DISABLE_TRUE_FLASH_FS,    N_("Disable true hardware flash file system")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TRUE_FLASH_FS_DA, "DeaktivÈr Êgte hardwareemulering af flashfilsystem"},
	/* de */ {IDCLS_DISABLE_TRUE_FLASH_FS_DE, "Pr‰zises Hardware Flashdateisystem deaktivieren"},
	/* es */ {IDCLS_DISABLE_TRUE_FLASH_FS_ES, "Deshabilitar flash file system verdadero (hardware)"},
	/* fr */ {IDCLS_DISABLE_TRUE_FLASH_FS_FR, "DÈsactiver le systËme de fichiers matÈriel rÈel"},
	/* hu */ {IDCLS_DISABLE_TRUE_FLASH_FS_HU, "Igazi hardver flash f·jlrendszer tilt·sa"},
	/* it */ {IDCLS_DISABLE_TRUE_FLASH_FS_IT, "Disattiva emulazione hardware del file system flash"},
	/* ko */ {IDCLS_DISABLE_TRUE_FLASH_FS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TRUE_FLASH_FS_NL, "Hardwarematig flash bestandssysteem uitschakelen"},
	/* pl */ {IDCLS_DISABLE_TRUE_FLASH_FS_PL, "Wy≥±cz prawdziwy system plikÛw flash"},
	/* ru */ {IDCLS_DISABLE_TRUE_FLASH_FS_RU, "Disable true hardware flash file system"},
	/* sv */ {IDCLS_DISABLE_TRUE_FLASH_FS_SV, "Inaktivera maskinvaruemelring fˆr flashfilsystem"},
	/* tr */ {IDCLS_DISABLE_TRUE_FLASH_FS_TR, "GerÁek donan˝m flash dosya sistemini pasifle˛tir"},
#endif
	
#ifdef HAVE_MIDI
	/* midi.c */
	/* en */ {IDCLS_ENABLE_MIDI_EMU,    N_("Enable MIDI emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_MIDI_EMU_DA, "AktivÈr MIDI-emulering"},
	/* de */ {IDCLS_ENABLE_MIDI_EMU_DE, "MIDI Emulation aktivieren"},
	/* es */ {IDCLS_ENABLE_MIDI_EMU_ES, "Habilitar emulaciÛn MIDI"},
	/* fr */ {IDCLS_ENABLE_MIDI_EMU_FR, "Activer l'Èmulation MIDI"},
	/* hu */ {IDCLS_ENABLE_MIDI_EMU_HU, "MIDI emul·ciÛ engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_MIDI_EMU_IT, "Attiva emulazione MIDI"},
	/* ko */ {IDCLS_ENABLE_MIDI_EMU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_MIDI_EMU_NL, "MIDI emulatie inschakelen"},
	/* pl */ {IDCLS_ENABLE_MIDI_EMU_PL, "W≥±cz emulacjÍ MIDI"},
	/* ru */ {IDCLS_ENABLE_MIDI_EMU_RU, "≤⁄€ÓÁÿ‚Ï Ì‹„€ÔÊÿÓ MIDI"},
	/* sv */ {IDCLS_ENABLE_MIDI_EMU_SV, "Aktivera MIDI-emulering"},
	/* tr */ {IDCLS_ENABLE_MIDI_EMU_TR, "MIDI em¸lasyonunu aktif et"},
#endif
	
	/* midi.c */
	/* en */ {IDCLS_DISABLE_MIDI_EMU,    N_("Disable MIDI emulation")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_MIDI_EMU_DA, "DeaktivÈr MIDI-emulering"},
	/* de */ {IDCLS_DISABLE_MIDI_EMU_DE, "MIDI Emulation deaktivieren"},
	/* es */ {IDCLS_DISABLE_MIDI_EMU_ES, "Deshabilitar emulaciÛn MIDI"},
	/* fr */ {IDCLS_DISABLE_MIDI_EMU_FR, "DÈsactiver l'Èmulation MIDI"},
	/* hu */ {IDCLS_DISABLE_MIDI_EMU_HU, "MIDI emul·ciÛ tilt·sa"},
	/* it */ {IDCLS_DISABLE_MIDI_EMU_IT, "Disattiva emulazione MIDI"},
	/* ko */ {IDCLS_DISABLE_MIDI_EMU_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_MIDI_EMU_NL, "MIDI emulatie uitschakelen"},
	/* pl */ {IDCLS_DISABLE_MIDI_EMU_PL, "Wy≥±cz emulacjÍ MIDI"},
	/* ru */ {IDCLS_DISABLE_MIDI_EMU_RU, "Disable MIDI emulation"},
	/* sv */ {IDCLS_DISABLE_MIDI_EMU_SV, "Inaktivera MIDI-emulering"},
	/* tr */ {IDCLS_DISABLE_MIDI_EMU_TR, "MIDI em¸lasyonunu pasifle˛tir"},
#endif
	
	/* c64/cart/c64-midi.c */
	/* en */ {IDCLS_SPECIFY_C64_MIDI_TYPE,    N_("MIDI interface type (0: Sequential, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_C64_MIDI_TYPE_DA, "MIDI-interfacetype (0: Sekventiel, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* de */ {IDCLS_SPECIFY_C64_MIDI_TYPE_DE, "MIDI Interface Typ (0: Sequential, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* es */ {IDCLS_SPECIFY_C64_MIDI_TYPE_ES, "Tipo interface MIDI (0: Secuencial, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* fr */ {IDCLS_SPECIFY_C64_MIDI_TYPE_FR, "Type d'interface MIDI (0:SÈquentielle, 1:Passport, 2:DATEL, 3:Namesoft, 4:Maplin)"},
	/* hu */ {IDCLS_SPECIFY_C64_MIDI_TYPE_HU, "MIDI interfÈsz tÌpus (0: soros, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* it */ {IDCLS_SPECIFY_C64_MIDI_TYPE_IT, "Tipo interfaccia MIDI (0: Sequential, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* ko */ {IDCLS_SPECIFY_C64_MIDI_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_C64_MIDI_TYPE_NL, "MIDI interface soort (0: Sequential, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* pl */ {IDCLS_SPECIFY_C64_MIDI_TYPE_PL, "Typ interfejsu MIDI (0: Sequential, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* ru */ {IDCLS_SPECIFY_C64_MIDI_TYPE_RU, "MIDI interface type (0: Sequential, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* sv */ {IDCLS_SPECIFY_C64_MIDI_TYPE_SV, "Typ av MIDI-gr‰nssnitt (0: sekventiell, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
	/* tr */ {IDCLS_SPECIFY_C64_MIDI_TYPE_TR, "MIDI arabirim tipi (0: Sequential, 1: Passport, 2: DATEL, 3: Namesoft, 4: Maplin)"},
#endif
#endif
	
	/* plus4/digiblaster.c */
	/* en */ {IDCLS_ENABLE_DIGIBLASTER,    N_("Enable the digiblaster add-on")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DIGIBLASTER_DA, "AktivÈr digiblaster-tilf¯jelse"},
	/* de */ {IDCLS_ENABLE_DIGIBLASTER_DE, "Digiblaster add-on aktivieren"},
	/* es */ {IDCLS_ENABLE_DIGIBLASTER_ES, "Permitir complementos digiblaster"},
	/* fr */ {IDCLS_ENABLE_DIGIBLASTER_FR, "Activer le add-on digiblaster"},
	/* hu */ {IDCLS_ENABLE_DIGIBLASTER_HU, "Digiblaster engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_DIGIBLASTER_IT, "Attiva l'estensione digiblaster"},
	/* ko */ {IDCLS_ENABLE_DIGIBLASTER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DIGIBLASTER_NL, "Digiblaster add-on inschakelen"},
	/* pl */ {IDCLS_ENABLE_DIGIBLASTER_PL, "W≥±cz dodatek digiblaster"},
	/* ru */ {IDCLS_ENABLE_DIGIBLASTER_RU, "Enable the digiblaster add-on"},
	/* sv */ {IDCLS_ENABLE_DIGIBLASTER_SV, "Aktivera digiblaster-till‰gg"},
	/* tr */ {IDCLS_ENABLE_DIGIBLASTER_TR, "Digiblaster eklentisini aktif et"},
#endif
	
	/* plus4/digiblaster.c */
	/* en */ {IDCLS_DISABLE_DIGIBLASTER,    N_("Disable the digiblaster add-on")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DIGIBLASTER_DA, "DeaktivÈr digiblaster-tilf¯jelse"},
	/* de */ {IDCLS_DISABLE_DIGIBLASTER_DE, "Digiblaster add-on deaktivieren"},
	/* es */ {IDCLS_DISABLE_DIGIBLASTER_ES, "Deshabilitar complementos digiblaster"},
	/* fr */ {IDCLS_DISABLE_DIGIBLASTER_FR, "DÈsactiver le add-on digiblaster"},
	/* hu */ {IDCLS_DISABLE_DIGIBLASTER_HU, "Digiblaster tilt·sa"},
	/* it */ {IDCLS_DISABLE_DIGIBLASTER_IT, "Disattiva l'estensione digiblaster"},
	/* ko */ {IDCLS_DISABLE_DIGIBLASTER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DIGIBLASTER_NL, "Digiblaster add-on uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DIGIBLASTER_PL, "Wy≥±cz dodatek digiblaster"},
	/* ru */ {IDCLS_DISABLE_DIGIBLASTER_RU, "Disable the digiblaster add-on"},
	/* sv */ {IDCLS_DISABLE_DIGIBLASTER_SV, "Inaktivera digiblaster-till‰gg"},
	/* tr */ {IDCLS_DISABLE_DIGIBLASTER_TR, "Digiblaster eklentisini pasifle˛tir"},
#endif
	
	/* plus4/plus4speech.c */
	/* en */ {IDCLS_ENABLE_PLUS4SPEECH,    N_("Enable the v364 speech add-on")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_PLUS4SPEECH_DA, "AktivÈr v364 Speech-tilf¯jelse"},
	/* de */ {IDCLS_ENABLE_PLUS4SPEECH_DE, "v364 Spracherweiterung aktivieren"},
	/* es */ {IDCLS_ENABLE_PLUS4SPEECH_ES, "Permitir complementos v364 speech"},
	/* fr */ {IDCLS_ENABLE_PLUS4SPEECH_FR, "Activer le add-on V364 Speech"},
	/* hu */ {IDCLS_ENABLE_PLUS4SPEECH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_PLUS4SPEECH_IT, "Attiva l'estensione v364 speech"},
	/* ko */ {IDCLS_ENABLE_PLUS4SPEECH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_PLUS4SPEECH_NL, "v364 spraak add-on inschakelen"},
	/* pl */ {IDCLS_ENABLE_PLUS4SPEECH_PL, "W≥±cz rozszerzenie mowy v364"},
	/* ru */ {IDCLS_ENABLE_PLUS4SPEECH_RU, "Enable the v364 speech add-on"},
	/* sv */ {IDCLS_ENABLE_PLUS4SPEECH_SV, "Aktivera v364-rˆsttill‰gg"},
	/* tr */ {IDCLS_ENABLE_PLUS4SPEECH_TR, "V364 speech eklentisini aktif et"},
#endif
	
	/* plus4/plus4speech.c */
	/* en */ {IDCLS_DISABLE_PLUS4SPEECH,    N_("Disable the v364 speech add-on")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_PLUS4SPEECH_DA, "DeaktivÈr v364 Speech-tilf¯jelse"},
	/* de */ {IDCLS_DISABLE_PLUS4SPEECH_DE, "v364 Spracherweiterung deaktivieren"},
	/* es */ {IDCLS_DISABLE_PLUS4SPEECH_ES, "Deshabilitar complementos v364 speech"},
	/* fr */ {IDCLS_DISABLE_PLUS4SPEECH_FR, "DÈsactiver le add-on V364 Speech"},
	/* hu */ {IDCLS_DISABLE_PLUS4SPEECH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_PLUS4SPEECH_IT, "Disattiva l'estensione v364 speech"},
	/* ko */ {IDCLS_DISABLE_PLUS4SPEECH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_PLUS4SPEECH_NL, "v364 spraak add-on uitschakelen"},
	/* pl */ {IDCLS_DISABLE_PLUS4SPEECH_PL, "Wy≥±cz rozszerzenie mowy v364"},
	/* ru */ {IDCLS_DISABLE_PLUS4SPEECH_RU, "Disable the v364 speech add-on"},
	/* sv */ {IDCLS_DISABLE_PLUS4SPEECH_SV, "Inaktivera v364-rˆsttill‰gg"},
	/* tr */ {IDCLS_DISABLE_PLUS4SPEECH_TR, "V364 speech eklentisini pasifle˛tir"},
#endif
	
	/* plus4/plus4speech.c */
	/* en */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE,    N_("Attach Speech ROM image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_DA, "Tilslut Speech ROM image"},
	/* de */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_DE, "Spracherweiterung ROM Imagedatei einlegen"},
	/* es */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_ES, "Insertar imagen ROM Speech"},
	/* fr */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_FR, "Attacher une image Speech RO"},
	/* hu */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_IT, "Seleziona immagine Speech ROM"},
	/* ko */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_NL, "Koppel spraak ROM bestand"},
	/* pl */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_PL, "Zamontuj obraz ROM mowy"},
	/* ru */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_RU, "Attach Speech ROM image"},
	/* sv */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_SV, "Anslut rˆst-ROM-avbildning"},
	/* tr */ {IDCLS_ATTACH_SPEECH_ROM_IMAGE_TR, "Speech ROM imaj˝n˝ yerle˛tir"},
#endif
	
	/* plus4/sidcartjoy.c */
	/* en */ {IDCLS_ENABLE_SIDCARTJOY,    N_("Enable SID cartridge joystick")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SIDCARTJOY_DA, "AktivÈr SID-cartridge joystick"},
	/* de */ {IDCLS_ENABLE_SIDCARTJOY_DE, "SIDcart Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_SIDCARTJOY_ES, "Permitir joystick cartucho SID"},
	/* fr */ {IDCLS_ENABLE_SIDCARTJOY_FR, "Activer Joystick cartouche SID"},
	/* hu */ {IDCLS_ENABLE_SIDCARTJOY_HU, "SID cartridge botkorm·ny engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_SIDCARTJOY_IT, "Attiva joystick su cartuccia SID"},
	/* ko */ {IDCLS_ENABLE_SIDCARTJOY_KO, "SID ƒ´∆Æ∏Æ¡ˆ ¡∂¿ÃΩ∫∆Ω ªÁøÎ«œ±‚"},
	/* nl */ {IDCLS_ENABLE_SIDCARTJOY_NL, "SID cartridge joystick inschakelen"},
	/* pl */ {IDCLS_ENABLE_SIDCARTJOY_PL, "W≥±cz joystick kartridøa SID"},
	/* ru */ {IDCLS_ENABLE_SIDCARTJOY_RU, "Enable SID cartridge joystick"},
	/* sv */ {IDCLS_ENABLE_SIDCARTJOY_SV, "Aktivera SID-insticksmodul-styrspak"},
	/* tr */ {IDCLS_ENABLE_SIDCARTJOY_TR, "SID kartu˛u joystick'ini aktif et"},
#endif
	
	/* plus4/sidcartjoy.c */
	/* en */ {IDCLS_DISABLE_SIDCARTJOY,    N_("Disable SID cartridge joystick")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SIDCARTJOY_DA, "DeaktivÈr SID-cartridge joystick"},
	/* de */ {IDCLS_DISABLE_SIDCARTJOY_DE, "SIDcart Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_SIDCARTJOY_ES, "Deshabilitar joystick cartucho SID"},
	/* fr */ {IDCLS_DISABLE_SIDCARTJOY_FR, "DÈsactiver Joystick cartouche SID"},
	/* hu */ {IDCLS_DISABLE_SIDCARTJOY_HU, "SID cartridge botkorm·ny tilt·sa"},
	/* it */ {IDCLS_DISABLE_SIDCARTJOY_IT, "Disattiva joystick su cartuccia SID"},
	/* ko */ {IDCLS_DISABLE_SIDCARTJOY_KO, "SID ƒ´∆Æ∏Æ¡ˆ ¡∂¿ÃΩ∫∆Ω ªÁøÎ«œ¡ˆ æ ±‚"},
	/* nl */ {IDCLS_DISABLE_SIDCARTJOY_NL, "SID cartridge joystick uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SIDCARTJOY_PL, "Wy≥±cz joystick kartridøa SID"},
	/* ru */ {IDCLS_DISABLE_SIDCARTJOY_RU, "Disable SID cartridge joystick"},
	/* sv */ {IDCLS_DISABLE_SIDCARTJOY_SV, "Inaktivera SID-insticksmodul-styrspak"},
	/* tr */ {IDCLS_DISABLE_SIDCARTJOY_TR, "SID kartu˛u joystick'ini pasifle˛tir"},
#endif
	
	/* c64/cart/sfx_soundexpander.c */
	/* en */ {IDCLS_ENABLE_SFX_SE,    N_("Enable the SFX Sound Expander cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SFX_SE_DA, "AktivÈr SFX sound Expander cartridget"},
	/* de */ {IDCLS_ENABLE_SFX_SE_DE, "SFX Soundexpander Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_SFX_SE_ES, "Permitir cartucho SFX Sound Expander"},
	/* fr */ {IDCLS_ENABLE_SFX_SE_FR, "Activer la cartouche SFX Sound Expander"},
	/* hu */ {IDCLS_ENABLE_SFX_SE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SFX_SE_IT, "Attiva cartuccia SFX Sound Expander"},
	/* ko */ {IDCLS_ENABLE_SFX_SE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SFX_SE_NL, "SFX Sound Expander cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_SFX_SE_PL, "W≥±cz kartridø SFX Sound Expander"},
	/* ru */ {IDCLS_ENABLE_SFX_SE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SFX_SE_SV, "Aktivera SFX Sound Expander-insticksmodul"},
	/* tr */ {IDCLS_ENABLE_SFX_SE_TR, "SFX Sound Expander kartu˛unu aktif et"},
#endif
	
	/* c64/cart/sfx_soundexpander.c */
	/* en */ {IDCLS_DISABLE_SFX_SE,    N_("Disable the SFX Sound Expander cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SFX_SE_DA, "DeaktivÈr SFX Sound Expander cartridge"},
	/* de */ {IDCLS_DISABLE_SFX_SE_DE, "SFX Soundexpander Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_SFX_SE_ES, "Deshabilitar cartucho SFX Sound Expander"},
	/* fr */ {IDCLS_DISABLE_SFX_SE_FR, "DÈsactiver la cartouche SFX Sound Expander"},
	/* hu */ {IDCLS_DISABLE_SFX_SE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SFX_SE_IT, "Disattiva cartuccia SFX SFX Sound Expander"},
	/* ko */ {IDCLS_DISABLE_SFX_SE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SFX_SE_NL, "SFX Sound Expander cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SFX_SE_PL, "Wy≥±cz kartridø SFX Sound Expander"},
	/* ru */ {IDCLS_DISABLE_SFX_SE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SFX_SE_SV, "Inaktivera SFX Sound Expander-insticksmodul"},
	/* tr */ {IDCLS_DISABLE_SFX_SE_TR, "SFX Sound Expander kartu˛unu pasifle˛tir"},
#endif
	
	/* c64/cart/sfx_soundexpander.c */
	/* en */ {IDCLS_SET_YM_CHIP_TYPE,    N_("Set YM chip type (3526 / 3812)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_YM_CHIP_TYPE_DA, "VÊlg YM chiptype"},
	/* de */ {IDCLS_SET_YM_CHIP_TYPE_DE, "YM Chip Typ (3526 / 3812) bestimmen"},
	/* es */ {IDCLS_SET_YM_CHIP_TYPE_ES, "Seleccionar tipo chip YM (3526 / 3812)"},
	/* fr */ {IDCLS_SET_YM_CHIP_TYPE_FR, "RÈgler le type de puce YM (3526  3812)"},
	/* hu */ {IDCLS_SET_YM_CHIP_TYPE_HU, "YM csip tÌpus megad·sa (3526 / 3812)"},
	/* it */ {IDCLS_SET_YM_CHIP_TYPE_IT, "Imposta tipo chip YM (3526 / 3812)"},
	/* ko */ {IDCLS_SET_YM_CHIP_TYPE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_YM_CHIP_TYPE_NL, "Stel de YM chip soort in (3526 / 3812)"},
	/* pl */ {IDCLS_SET_YM_CHIP_TYPE_PL, "Okre∂l typ uk≥adu YM (3526 / 3812)"},
	/* ru */ {IDCLS_SET_YM_CHIP_TYPE_RU, "Set YM chip type (3526 / 3812)"},
	/* sv */ {IDCLS_SET_YM_CHIP_TYPE_SV, "V‰lj YM-kretstyp (3526 / 3812)"},
	/* tr */ {IDCLS_SET_YM_CHIP_TYPE_TR, "YM Áip tipini ayarla (3526 / 3812)"},
#endif
	
	/* c64/cart/sfx_soundsampler.c */
	/* en */ {IDCLS_ENABLE_SFX_SS,    N_("Enable the SFX Sound Sampler cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SFX_SS_DA, "AktivÈr SFX Sound Sampler cartridge"},
	/* de */ {IDCLS_ENABLE_SFX_SS_DE, "SFX Sound Sampler Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_SFX_SS_ES, "Permitir cartucho SFX Sound Sampler"},
	/* fr */ {IDCLS_ENABLE_SFX_SS_FR, "Activer la cartouche SFX Sound Sampler"},
	/* hu */ {IDCLS_ENABLE_SFX_SS_HU, "SFX Sound Sampler engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_SFX_SS_IT, "Attiva cartuccia SFX Sound Sampler"},
	/* ko */ {IDCLS_ENABLE_SFX_SS_KO, "SFX Sound Sampler ƒ´∆Æ∏Æ¡ˆ ∏¶ ¿€µøΩ√≈≤¥Ÿ"},
	/* nl */ {IDCLS_ENABLE_SFX_SS_NL, "SFX Sound Sampler cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_SFX_SS_PL, "W≥±cz kartridø SFX Sound Sampler"},
	/* ru */ {IDCLS_ENABLE_SFX_SS_RU, "Enable the SFX Sound Sampler cartridge"},
	/* sv */ {IDCLS_ENABLE_SFX_SS_SV, "Aktivera SFX Sound Sampler-insticksmodul"},
	/* tr */ {IDCLS_ENABLE_SFX_SS_TR, "SFX Sound Sampler kartu˛unu aktif et"},
#endif
	
	/* c64/cart/sfx_soundsampler.c */
	/* en */ {IDCLS_DISABLE_SFX_SS,    N_("Disable the SFX Sound Sampler cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SFX_SS_DA, "DeaktivÈr SFX Sound Sampler cartridge"},
	/* de */ {IDCLS_DISABLE_SFX_SS_DE, "SFX Sound Sampler Modul  deaktivieren"},
	/* es */ {IDCLS_DISABLE_SFX_SS_ES, "Deshabilitar cartucho SFX Sound Sampler"},
	/* fr */ {IDCLS_DISABLE_SFX_SS_FR, "DÈsactiver la cartouche SFX Sound Sampler"},
	/* hu */ {IDCLS_DISABLE_SFX_SS_HU, "SFX Sound Sampler tilt·sa"},
	/* it */ {IDCLS_DISABLE_SFX_SS_IT, "Disattiva cartuccia SFX Sound Sampler"},
	/* ko */ {IDCLS_DISABLE_SFX_SS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SFX_SS_NL, "SFX Sound Sampler cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SFX_SS_PL, "Wy≥±cz kartridø SFX Sound Sampler"},
	/* ru */ {IDCLS_DISABLE_SFX_SS_RU, "Disable the SFX Sound Sampler cartridge"},
	/* sv */ {IDCLS_DISABLE_SFX_SS_SV, "Inaktivera SFX Sound Sampler-insticksmodul"},
	/* tr */ {IDCLS_DISABLE_SFX_SS_TR, "SFX Sound Sampler kartu˛unu pasifle˛tir"},
#endif
	
	/* c64/cart/cpmcart.c */
	/* en */ {IDCLS_ENABLE_CPM_CART,    N_("Enable the CP/M cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_CPM_CART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_CPM_CART_DE, "CP/M Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_CPM_CART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_CPM_CART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_CPM_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_CPM_CART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_CPM_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_CPM_CART_NL, "CP/M cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_CPM_CART_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_CPM_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_CPM_CART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_CPM_CART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/cpmcart.c */
	/* en */ {IDCLS_DISABLE_CPM_CART,    N_("Disable the CP/M cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_CPM_CART_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_CPM_CART_DE, "CP/M Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_CPM_CART_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_CPM_CART_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_CPM_CART_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_CPM_CART_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_CPM_CART_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_CPM_CART_NL, "CP/M cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_CPM_CART_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_CPM_CART_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_CPM_CART_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_CPM_CART_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_ENABLE_TURBOMASTER,    N_("Enable the TurboMaster cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_TURBOMASTER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_TURBOMASTER_DE, "TurboMaster Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_TURBOMASTER_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_TURBOMASTER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_TURBOMASTER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_TURBOMASTER_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_TURBOMASTER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_TURBOMASTER_NL, "TurboMaster cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_TURBOMASTER_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_TURBOMASTER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_TURBOMASTER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_TURBOMASTER_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_DISABLE_TURBOMASTER,    N_("Disable the TurboMaster cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_TURBOMASTER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_TURBOMASTER_DE, "TurboMaster Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_TURBOMASTER_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_TURBOMASTER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_TURBOMASTER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_TURBOMASTER_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_TURBOMASTER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_TURBOMASTER_NL, "TurboMaster cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_TURBOMASTER_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_TURBOMASTER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_TURBOMASTER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_TURBOMASTER_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_TURBOMASTER_SWITCH_ROM_1,    N_("Set TurboMaster ROM switch to ROM 1")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_DE, "TurboMaster ROM Schalter auf ROM 1 setzen"},
	/* es */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_NL, "Schakel TurboMaster ROM schakelaar naar ROM 1"},
	/* pl */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_TURBOMASTER_SWITCH_ROM_1_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_TURBOMASTER_SWITCH_ROM_2,    N_("Set TurboMaster ROM switch to ROM 2")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_DE, "TurboMaster ROM Schalter auf ROM 2 setzen"},
	/* es */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_NL, "Schakel TurboMaster ROM schakelaar naar ROM 2"},
	/* pl */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_TURBOMASTER_SWITCH_ROM_2_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT,    N_("Set TurboMaster speed control switch to software")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_DE, "TurboMaster Geschwindigkeits Kontrollschalter auf Software setzen"},
	/* es */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_NL, "Schakel TurboMaster snelheidscontrole schakelaar naar software"},
	/* pl */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_TURBOMASTER_SWITCH_SPEED_SOFT_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL,    N_("Set TurboMaster speed control switch to manual")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_DE, "TurboMaster Geschwindigkeits Kontrollschalter auf Manuell setzen"},
	/* es */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_NL, "Schakel TurboMaster snelheidscontrole schakelaar naar handmatig"},
	/* pl */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_TURBOMASTER_SWITCH_SPEED_MANUAL_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ,    N_("Set TurboMaster clock speed switch to 4 Mhz")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_DE, "TurboMaster Taktgeschwindigkeitsschalter auf 4Mhz setzen"},
	/* es */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_NL, "Schakel TurboMaster klok snelheid schakelaar naar 4 Mhz"},
	/* pl */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_TURBOMASTER_SWITCH_4_MHZ_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ,    N_("Set TurboMaster clock speed switch to 1 Mhz")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_DE, "TurboMaster Taktgeschwindigkeitsschalter auf 1Mhz setzen"},
	/* es */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_NL, "Schakel TurboMaster klok snelheid schakelaar naar 1 Mhz"},
	/* pl */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_TURBOMASTER_SWITCH_1_MHZ_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME,    N_("Specify TurboMaster ROM 1 filename")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_DE, "TurboMaster ROM 1 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_NL, "Geef de naam van het TurboMaster ROM 1 bestand"},
	/* pl */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_TURBOMASTER_ROM1_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/turbomaster.c */
	/* en */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME,    N_("Specify TurboMaster ROM 2 filename")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_DE, "TurboMaster ROM 2 w‰hlen"},
	/* es */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_NL, "Geef de naam van het TurboMaster ROM 2 bestand"},
	/* pl */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SPECIFY_TURBOMASTER_ROM2_NAME_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/easyflash.c */
	/* en */ {IDCLS_ENABLE_EASYFLASH_JUMPER,    N_("Enable EasyFlash jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_EASYFLASH_JUMPER_DA, "AktivÈr EasyFlash-jumper"},
	/* de */ {IDCLS_ENABLE_EASYFLASH_JUMPER_DE, "EasyFlash Jumper aktivieren"},
	/* es */ {IDCLS_ENABLE_EASYFLASH_JUMPER_ES, "Permitir puente EasyFlash"},
	/* fr */ {IDCLS_ENABLE_EASYFLASH_JUMPER_FR, "Activer le jumper EasyFlash"},
	/* hu */ {IDCLS_ENABLE_EASYFLASH_JUMPER_HU, "EasyFlash Jumper engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_EASYFLASH_JUMPER_IT, "Attiva ponticello EasyFlash"},
	/* ko */ {IDCLS_ENABLE_EASYFLASH_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_EASYFLASH_JUMPER_NL, "EasyFlash jumper inschakelen"},
	/* pl */ {IDCLS_ENABLE_EASYFLASH_JUMPER_PL, "W≥±cz prze≥±cznik EasyFlash"},
	/* ru */ {IDCLS_ENABLE_EASYFLASH_JUMPER_RU, "Enable EasyFlash jumper"},
	/* sv */ {IDCLS_ENABLE_EASYFLASH_JUMPER_SV, "Aktivera EasyFlash-bygel"},
	/* tr */ {IDCLS_ENABLE_EASYFLASH_JUMPER_TR, "EasyFlash jumper'˝ aktif et"},
#endif
	
	/* c64/cart/easyflash.c */
	/* en */ {IDCLS_DISABLE_EASYFLASH_JUMPER,    N_("Disable EasyFlash jumper")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_EASYFLASH_JUMPER_DA, "DeaktivÈr EasyFlash-jumper"},
	/* de */ {IDCLS_DISABLE_EASYFLASH_JUMPER_DE, "EasyFlash Jumer deaktivieren"},
	/* es */ {IDCLS_DISABLE_EASYFLASH_JUMPER_ES, "Deshabilitar puente EasyFlash"},
	/* fr */ {IDCLS_DISABLE_EASYFLASH_JUMPER_FR, "DÈsactiver le jumper EasyFlash"},
	/* hu */ {IDCLS_DISABLE_EASYFLASH_JUMPER_HU, "EasyFlash Jumper tilt·sa"},
	/* it */ {IDCLS_DISABLE_EASYFLASH_JUMPER_IT, "Disattiva ponticello EasyFlash"},
	/* ko */ {IDCLS_DISABLE_EASYFLASH_JUMPER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_EASYFLASH_JUMPER_NL, "EasyFlash jumper uitschakelen"},
	/* pl */ {IDCLS_DISABLE_EASYFLASH_JUMPER_PL, "Wy≥±cz prze≥±cznik EasyFlash"},
	/* ru */ {IDCLS_DISABLE_EASYFLASH_JUMPER_RU, "Disable EasyFlash jumper"},
	/* sv */ {IDCLS_DISABLE_EASYFLASH_JUMPER_SV, "Inaktivera EasyFlash-bygel"},
	/* tr */ {IDCLS_DISABLE_EASYFLASH_JUMPER_TR, "EasyFlash jumper'˝ pasifle˛tir"},
#endif
	
	/* c64/cart/easyflash.c */
	/* en */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING,    N_("Enable writing to EasyFlash .crt image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_DA, "Tillad skrivning til EasyFlash CRT-image"},
	/* de */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_DE, "Schreibzugriff auf EasyFlash CRT Image erlauben"},
	/* es */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_ES, "Permitir grabar imagen .crt EasyFlash"},
	/* fr */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_FR, "Activer l'Ècriture pour les images EasyFlash .crt"},
	/* hu */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_HU, "EasyFlash .crt kÈpm·sba Ìr·s engedÈlyezÈse"},
	/* it */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_IT, "Attiva scrittura sull'immagine .crt EasyFlash"},
	/* ko */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_NL, "Schrijven naar het EasyFlash .crt bestand inschakelen"},
	/* pl */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_PL, "W≥±cz zapis do obrazu .crt EasyFlash"},
	/* ru */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_RU, "Enable writing to EasyFlash .crt image"},
	/* sv */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_SV, "Aktivera skrivning till EasyFlash-.crt-avbildning."},
	/* tr */ {IDCLS_ENABLE_EASYFLASH_CRT_WRITING_TR, "EasyFlash .crt imaj˝na yazmay˝ aktif et"},
#endif
	
	/* c64/cart/easyflash.c */
	/* en */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING,    N_("Disable writing to EasyFlash .crt image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_DA, "Forbyd skrivning til EasyFlash CRT-image"},
	/* de */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_DE, "Schreibzugriff auf EasyFlash ROM Image verhindern"},
	/* es */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_ES, "Deshabilitar grabar imagen .crt EasyFlash"},
	/* fr */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_FR, "DÈsactiver l'Ècriture pour les images EasyFlash .crt"},
	/* hu */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_HU, "EasyFlash .crt kÈpm·sba Ìr·s tilt·sa"},
	/* it */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_IT, "Disattiva scrittura sull'immagine .crt EasyFlash"},
	/* ko */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_NL, "Dchrijven naar het EasyFlash .crt bestand uitschakelen"},
	/* pl */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_PL, "Wy≥±cz zapis do obrazu .crt EasyFlash"},
	/* ru */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_RU, "Disable writing to EasyFlash .crt image"},
	/* sv */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_SV, "Inaktivera skrivning till EasyFlash-.crt-avbildning."},
	/* tr */ {IDCLS_DISABLE_EASYFLASH_CRT_WRITING_TR, "EasyFlash .crt imaj˝na yazmay˝ pasifle˛tir"},
#endif
	
	/* c64/cart/easyflash.c */
	/* en */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE,    N_("Enable EasyFlash .crt image optimize on write")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_DE, "EasyFlash .crt Image beim Speichern optimieren"},
	/* es */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_NL, "Optimalizeren van het schrijven naar het EasyFlash .crt bestand inschakelen"},
	/* pl */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_PL, "W≥±cz optymalizacjÍ zapisu do obrazu .crt EasyFlash"},
	/* ru */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_EASYFLASH_CRT_OPTIMIZE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/easyflash.c */
	/* en */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE,    N_("Disable writing to EasyFlash .crt image")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_DA, "Forbyd skrivning til EasyFlash CRT-image"},
	/* de */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_DE, "Schreibzugriff auf EasyFlash ROM Image verhindern"},
	/* es */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_ES, "Deshabilitar grabar imagen .crt EasyFlash"},
	/* fr */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_FR, "DÈsactiver l'Ècriture pour les images EasyFlash .crt"},
	/* hu */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_HU, "EasyFlash .crt kÈpm·sba Ìr·s tilt·sa"},
	/* it */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_IT, "Disattiva scrittura sull'immagine .crt EasyFlash"},
	/* ko */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_NL, "Dchrijven naar het EasyFlash .crt bestand uitschakelen"},
	/* pl */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_PL, "Wy≥±cz zapis do obrazu .crt EasyFlash"},
	/* ru */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_RU, "Disable writing to EasyFlash .crt image"},
	/* sv */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_SV, "Inaktivera skrivning till EasyFlash-.crt-avbildning."},
	/* tr */ {IDCLS_DISABLE_EASYFLASH_CRT_OPTIMIZE_TR, "EasyFlash .crt imaj˝na yazmay˝ pasifle˛tir"},
#endif
	
	/* userport/userport_digimax.c */
	/* en */ {IDCLS_ENABLE_USERPORT_DIGIMAX,    N_("Enable the userport DigiMAX device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_USERPORT_DIGIMAX_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_USERPORT_DIGIMAX_DE, "Userport DigiMAX Ger‰t aktivieren"},
	/* es */ {IDCLS_ENABLE_USERPORT_DIGIMAX_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_USERPORT_DIGIMAX_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_USERPORT_DIGIMAX_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_USERPORT_DIGIMAX_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_USERPORT_DIGIMAX_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_USERPORT_DIGIMAX_NL, "Userport DigiMAX apparaat inschakelen"},
	/* pl */ {IDCLS_ENABLE_USERPORT_DIGIMAX_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_USERPORT_DIGIMAX_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_USERPORT_DIGIMAX_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_USERPORT_DIGIMAX_TR, ""},  /* fuzzy */
#endif
	
	/* userport/userport_digimax.c */
	/* en */ {IDCLS_DISABLE_USERPORT_DIGIMAX,    N_("Disable the userport DigiMAX device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_USERPORT_DIGIMAX_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_USERPORT_DIGIMAX_DE, "Userport DigiMAX Ger‰t deaktivieren"},
	/* es */ {IDCLS_DISABLE_USERPORT_DIGIMAX_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_USERPORT_DIGIMAX_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_USERPORT_DIGIMAX_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_USERPORT_DIGIMAX_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_USERPORT_DIGIMAX_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_USERPORT_DIGIMAX_NL, "Userport DigiMAX apparaat uitschakelen"},
	/* pl */ {IDCLS_DISABLE_USERPORT_DIGIMAX_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_USERPORT_DIGIMAX_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_USERPORT_DIGIMAX_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_USERPORT_DIGIMAX_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/shortbus_digimax.c */
	/* en */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX,    N_("Enable the Short Bus DigiMAX expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_DE, "Shortbus DigiMAX Erweiterung aktivieren"},
	/* es */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_NL, "Short Bus DigiMAX uitbreiding inschakelen"},
	/* pl */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_SHORTBUS_DIGIMAX_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/shortbus_digimax.c */
	/* en */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX,    N_("Disable the Short Bus DigiMAX expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_DE, "Shortbus DigiMAX Erweiterung deaktivieren"},
	/* es */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_NL, "Short Bus DigiMAX uitbreiding uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_SHORTBUS_DIGIMAX_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/shortbus_etfe.c */
	/* en */ {IDCLS_ENABLE_SHORTBUS_ETFE,    N_("Enable the Short Bus ETFE expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SHORTBUS_ETFE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_SHORTBUS_ETFE_DE, "Short Bus ETFE Erweiterung aktivieren"},
	/* es */ {IDCLS_ENABLE_SHORTBUS_ETFE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_ENABLE_SHORTBUS_ETFE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_SHORTBUS_ETFE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SHORTBUS_ETFE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_ENABLE_SHORTBUS_ETFE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SHORTBUS_ETFE_NL, "Short Bus ETFE uitbreiding inschakelen"},
	/* pl */ {IDCLS_ENABLE_SHORTBUS_ETFE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_ENABLE_SHORTBUS_ETFE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SHORTBUS_ETFE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_SHORTBUS_ETFE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/shortbus_etfe.c */
	/* en */ {IDCLS_DISABLE_SHORTBUS_ETFE,    N_("Disable the Short Bus ETFE expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SHORTBUS_ETFE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_SHORTBUS_ETFE_DE, "Short Bus ETFE Erweiterung deaktivieren"},
	/* es */ {IDCLS_DISABLE_SHORTBUS_ETFE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_DISABLE_SHORTBUS_ETFE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_SHORTBUS_ETFE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SHORTBUS_ETFE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_DISABLE_SHORTBUS_ETFE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SHORTBUS_ETFE_NL, "Short Bus ETFE uitbreiding uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SHORTBUS_ETFE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_DISABLE_SHORTBUS_ETFE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SHORTBUS_ETFE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_SHORTBUS_ETFE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/digimax.c */
	/* en */ {IDCLS_ENABLE_DIGIMAX,    N_("Enable the DigiMAX cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DIGIMAX_DA, "AktivÈr DigiMAX cartridge"},
	/* de */ {IDCLS_ENABLE_DIGIMAX_DE, "DigiMAX Cartridge aktivieren"},
	/* es */ {IDCLS_ENABLE_DIGIMAX_ES, "Permitir cartucho DigMAX"},
	/* fr */ {IDCLS_ENABLE_DIGIMAX_FR, "Activer la cartouche DigiMAX"},
	/* hu */ {IDCLS_ENABLE_DIGIMAX_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DIGIMAX_IT, "Attiva la cartuccia DigiMAX"},
	/* ko */ {IDCLS_ENABLE_DIGIMAX_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DIGIMAX_NL, "DigiMAX cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_DIGIMAX_PL, "W≥±cz kartridø DigiMAX"},
	/* ru */ {IDCLS_ENABLE_DIGIMAX_RU, "Enable the DigiMAX cartridge"},
	/* sv */ {IDCLS_ENABLE_DIGIMAX_SV, "Aktivera DigiMAX-insticksmodul"},
	/* tr */ {IDCLS_ENABLE_DIGIMAX_TR, "DigiMAX kartu˛unu aktif et"},
#endif
	
	/* c64/cart/digimax.c */
	/* en */ {IDCLS_DISABLE_DIGIMAX,    N_("Disable the DigiMAX cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DIGIMAX_DA, "DeaktivÈr DigiMAX cartridge"},
	/* de */ {IDCLS_DISABLE_DIGIMAX_DE, "DigiMAX Cartridge deaktivieren"},
	/* es */ {IDCLS_DISABLE_DIGIMAX_ES, "Deshabilitar cartucho DigMAX"},
	/* fr */ {IDCLS_DISABLE_DIGIMAX_FR, "DÈsactiver la cartouche DigiMAX"},
	/* hu */ {IDCLS_DISABLE_DIGIMAX_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DIGIMAX_IT, "Disattiva la cartuccia DigiMAX"},
	/* ko */ {IDCLS_DISABLE_DIGIMAX_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DIGIMAX_NL, "DigiMAX cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DIGIMAX_PL, "Wy≥±cz kartridø DigiMAX"},
	/* ru */ {IDCLS_DISABLE_DIGIMAX_RU, "Disable the DigiMAX cartridge"},
	/* sv */ {IDCLS_DISABLE_DIGIMAX_SV, "Inaktivera DigiMAX-insticksmodul"},
	/* tr */ {IDCLS_DISABLE_DIGIMAX_TR, "DigiMAX kartu˛unu pasifle˛tir"},
#endif
	
	/* c64/cart/digimax.c */
	/* en */ {IDCLS_DIGIMAX_BASE,    N_("Base address of the DigiMAX cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DIGIMAX_BASE_DA, "Baseadresse for DigiMAX cartridge"},
	/* de */ {IDCLS_DIGIMAX_BASE_DE, "Basis Adresse f¸r DigiMAX Erweiterung"},
	/* es */ {IDCLS_DIGIMAX_BASE_ES, "DirecciÛn base para cartucho DigMAX"},
	/* fr */ {IDCLS_DIGIMAX_BASE_FR, "Adresse de base de la cartouche DigiMAX"},
	/* hu */ {IDCLS_DIGIMAX_BASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DIGIMAX_BASE_IT, "Indirizzo base della cartuccia DigiMAX"},
	/* ko */ {IDCLS_DIGIMAX_BASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DIGIMAX_BASE_NL, "Basisadres van de DigiMAX cartridge"},
	/* pl */ {IDCLS_DIGIMAX_BASE_PL, "Adres bazowy kartridøa DigiMAX"},
	/* ru */ {IDCLS_DIGIMAX_BASE_RU, "Base address of the DigiMAX cartridge"},
	/* sv */ {IDCLS_DIGIMAX_BASE_SV, "Basadress fˆr DigiMAC-insticksmodulen"},
	/* tr */ {IDCLS_DIGIMAX_BASE_TR, "DigiMAX kartu˛unun taban adresi"},
#endif
	
	/* c64/cart/shortbus_digimax.c */
	/* en */ {IDCLS_SHORTBUS_DIGIMAX_BASE,    N_("Base address of the Short Bus DigiMAX expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SHORTBUS_DIGIMAX_BASE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SHORTBUS_DIGIMAX_BASE_DE, "Basis Adresse f¸r Shortbus DigiMAX Erweiterung"},
	/* es */ {IDCLS_SHORTBUS_DIGIMAX_BASE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SHORTBUS_DIGIMAX_BASE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SHORTBUS_DIGIMAX_BASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SHORTBUS_DIGIMAX_BASE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SHORTBUS_DIGIMAX_BASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SHORTBUS_DIGIMAX_BASE_NL, "Basisadres van de Short Bus DigiMAX uitbreiding"},
	/* pl */ {IDCLS_SHORTBUS_DIGIMAX_BASE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SHORTBUS_DIGIMAX_BASE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SHORTBUS_DIGIMAX_BASE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SHORTBUS_DIGIMAX_BASE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/shortbus_etfe.c */
	/* en */ {IDCLS_SHORTBUS_ETFE_BASE,    N_("Base address of the Short Bus ETFE expansion")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SHORTBUS_ETFE_BASE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SHORTBUS_ETFE_BASE_DE, "Basis Adresse f¸r Short Bus ETFE Erweiterung"},
	/* es */ {IDCLS_SHORTBUS_ETFE_BASE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SHORTBUS_ETFE_BASE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SHORTBUS_ETFE_BASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SHORTBUS_ETFE_BASE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SHORTBUS_ETFE_BASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SHORTBUS_ETFE_BASE_NL, "Basisadres van de Short Bus ETFE uitbreiding"},
	/* pl */ {IDCLS_SHORTBUS_ETFE_BASE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_SHORTBUS_ETFE_BASE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SHORTBUS_ETFE_BASE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SHORTBUS_ETFE_BASE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ds12c887rtc.c */
	/* en */ {IDCLS_ENABLE_DS12C887RTC,    N_("Enable the DS12C887 RTC cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DS12C887RTC_DA, "AktivÈr DS12C887 RTC-cartridge"},
	/* de */ {IDCLS_ENABLE_DS12C887RTC_DE, "DS12C887 RTC Modul aktivieren"},
	/* es */ {IDCLS_ENABLE_DS12C887RTC_ES, "Permitir cartucho DS12C887 RTC"},
	/* fr */ {IDCLS_ENABLE_DS12C887RTC_FR, "Activer la cartouche RTC DS12C887"},
	/* hu */ {IDCLS_ENABLE_DS12C887RTC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DS12C887RTC_IT, "Attiva la cartuccia DS12C887 RTC"},
	/* ko */ {IDCLS_ENABLE_DS12C887RTC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DS12C887RTC_NL, "DS12C887 RTC cartridge inschakelen"},
	/* pl */ {IDCLS_ENABLE_DS12C887RTC_PL, "W≥±cz kartridø DS12C887 RTC"},
	/* ru */ {IDCLS_ENABLE_DS12C887RTC_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_DS12C887RTC_SV, "Aktivera DS12C887 RTC-insticksmodul"},
	/* tr */ {IDCLS_ENABLE_DS12C887RTC_TR, "DS12C887 RTC kartu˛unu aktif et"},
#endif
	
	/* c64/cart/ds12c887rtc.c */
	/* en */ {IDCLS_DISABLE_DS12C887RTC,    N_("Disable the DS12C887 RTC cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DS12C887RTC_DA, "DeaktivÈr DS12C887 RTC-cartridge"},
	/* de */ {IDCLS_DISABLE_DS12C887RTC_DE, "DS12C887 RTC Modul deaktivieren"},
	/* es */ {IDCLS_DISABLE_DS12C887RTC_ES, "Deshabilitar cartucho DS12C887 RTC"},
	/* fr */ {IDCLS_DISABLE_DS12C887RTC_FR, "DÈsactiver la cartouche DS12C887 RTC"},
	/* hu */ {IDCLS_DISABLE_DS12C887RTC_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DS12C887RTC_IT, "Disattiva la cartuccia DS12C887 RTC"},
	/* ko */ {IDCLS_DISABLE_DS12C887RTC_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DS12C887RTC_NL, "DS12C887 RTC cartridge uitschakelen"},
	/* pl */ {IDCLS_DISABLE_DS12C887RTC_PL, "Wy≥±cz kartridø DS12C887 RTC"},
	/* ru */ {IDCLS_DISABLE_DS12C887RTC_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_DS12C887RTC_SV, "Inaktivera DS12C887 RTC-insticksmodul"},
	/* tr */ {IDCLS_DISABLE_DS12C887RTC_TR, "DS12C887 RTC kartu˛unu pasifle˛tir"},
#endif
	
	/* c64/cart/ds12c887rtc.c */
	/* en */ {IDCLS_DS12C887RTC_RUNMODE_HALTED,    N_("Set the RTC oscillator to 'halted'")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_DE, "Setze RTC Oszillator auf 'gestoppt'"},
	/* es */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_ES, "Seleccionar oscilador RTC en 'halted' (parada)"},
	/* fr */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_IT, "Imposta l'oscillatore RTC a 'halted'"},
	/* ko */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_NL, "Stel de RTC oscillator in als 'gestopt'"},
	/* pl */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_PL, "Ustaw oscylator RTC jako 'zatrzymany'"},
	/* ru */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DS12C887RTC_RUNMODE_HALTED_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ds12c887rtc.c */
	/* en */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING,    N_("Set the RTC oscillator to 'running'")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_DE, "Setze RTC Oszillator auf 'laufend'"},
	/* es */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_ES, "Seleccionar RTC oscillator en 'ejecutando'"},
	/* fr */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_IT, "Imposta l'oscillatore RTC a 'running'"},
	/* ko */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_NL, "Stel de RTC oscillator in als 'werkend'"},
	/* pl */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_PL, "Ustaw oscylator RTC jako 'uruchomiony'"},
	/* ru */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DS12C887RTC_RUNMODE_RUNNING_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ds12c887rtc.c */
	/* en */ {IDCLS_DS12C887RTC_BASE,    N_("Base address of the DS12C887 RTC cartridge")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DS12C887RTC_BASE_DA, "Baseadresse for DS12C887 RTC-cartridge"},
	/* de */ {IDCLS_DS12C887RTC_BASE_DE, "Basis Adresse f¸r DS12C887 RTC Modul"},
	/* es */ {IDCLS_DS12C887RTC_BASE_ES, "DirecciÛn base para cartucho DS12C887 RTC"},
	/* fr */ {IDCLS_DS12C887RTC_BASE_FR, "Adresse de base de la cartouche RTC DS12C887"},
	/* hu */ {IDCLS_DS12C887RTC_BASE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DS12C887RTC_BASE_IT, "Indirizzo base della cartuccia DS12C887 RTC"},
	/* ko */ {IDCLS_DS12C887RTC_BASE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DS12C887RTC_BASE_NL, "Basisadres van de DS12C887 RTC cartridge"},
	/* pl */ {IDCLS_DS12C887RTC_BASE_PL, "Adres bazowy kartridøa DS12C887 RTC"},
	/* ru */ {IDCLS_DS12C887RTC_BASE_RU, "Base address of the DS12C887 RTC cartridge"},
	/* sv */ {IDCLS_DS12C887RTC_BASE_SV, "Basadress fˆr DS12C887 RTC-insticksmodulen"},
	/* tr */ {IDCLS_DS12C887RTC_BASE_TR, "DS12C887 RTC kartu˛unun taban adresi"},
#endif
	
	/* c64/cart/ds12c887rtc.c */
	/* en */ {IDCLS_ENABLE_DS12C887RTC_SAVE,    N_("Enable saving of the DS12C887 RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_DS12C887RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_DS12C887RTC_SAVE_DE, "DS12C887 RTC Daten bei ƒnderung speichern."},
	/* es */ {IDCLS_ENABLE_DS12C887RTC_SAVE_ES, "Permitir grabar datos de  DS12C887 RTC cuando cambien."},
	/* fr */ {IDCLS_ENABLE_DS12C887RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_DS12C887RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_DS12C887RTC_SAVE_IT, "Attiva il salvataggio dei dati RTC del DS12C887 alla modifica."},
	/* ko */ {IDCLS_ENABLE_DS12C887RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_DS12C887RTC_SAVE_NL, "Opslaan van de DS12C887 RTC gegevens bij wijziging inschakelen."},
	/* pl */ {IDCLS_ENABLE_DS12C887RTC_SAVE_PL, "Zapisuj dane zegara czasu rzeczywistego DS12C887 przy zmianie"},
	/* ru */ {IDCLS_ENABLE_DS12C887RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_DS12C887RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_ENABLE_DS12C887RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/ds12c887rtc.c */
	/* en */ {IDCLS_DISABLE_DS12C887RTC_SAVE,    N_("Disable saving of the DS12C887 RTC data when changed.")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_DS12C887RTC_SAVE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_DS12C887RTC_SAVE_DE, "DS12C887 RTC Daten bei ƒnderung nicht speichern."},
	/* es */ {IDCLS_DISABLE_DS12C887RTC_SAVE_ES, "Deshabilitar grabar datos de  DS12C887 RTC cuando cambien."},
	/* fr */ {IDCLS_DISABLE_DS12C887RTC_SAVE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_DS12C887RTC_SAVE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_DS12C887RTC_SAVE_IT, "Disattiva il salvataggio dei dati RTC del DS12C887 alla modifica."},
	/* ko */ {IDCLS_DISABLE_DS12C887RTC_SAVE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_DS12C887RTC_SAVE_NL, "Opslaan van de DS12C887 RTC gegevens bij wijziging uitschakelen."},
	/* pl */ {IDCLS_DISABLE_DS12C887RTC_SAVE_PL, "Nie zapisuj danych zegara czasu rzeczywistego DS12C887 przy zmianie"},
	/* ru */ {IDCLS_DISABLE_DS12C887RTC_SAVE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_DS12C887RTC_SAVE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_DISABLE_DS12C887RTC_SAVE_TR, ""},  /* fuzzy */
#endif
	
	/* c64/cart/rrnetmk3.c */
	/* en */ {IDCLS_CLOCKPORT_DEVICE,    N_("Clockport device")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_CLOCKPORT_DEVICE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_CLOCKPORT_DEVICE_DE, "Clockport Ger‰t"},
	/* es */ {IDCLS_CLOCKPORT_DEVICE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_CLOCKPORT_DEVICE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_CLOCKPORT_DEVICE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_CLOCKPORT_DEVICE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_CLOCKPORT_DEVICE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_CLOCKPORT_DEVICE_NL, "Clockport apparaat"},
	/* pl */ {IDCLS_CLOCKPORT_DEVICE_PL, ""},  /* fuzzy */
	/* ru */ {IDCLS_CLOCKPORT_DEVICE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_CLOCKPORT_DEVICE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_CLOCKPORT_DEVICE_TR, ""},  /* fuzzy */
#endif
	
#ifdef HAVE_NETWORK
	/* network.c */
	/* en */ {IDCLS_P_HOSTNAME,    N_("<hostname>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_HOSTNAME_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_HOSTNAME_DE, "<Hostname>"},
	/* es */ {IDCLS_P_HOSTNAME_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_P_HOSTNAME_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_HOSTNAME_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_HOSTNAME_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_P_HOSTNAME_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_HOSTNAME_NL, "<Hostnaam>"},
	/* pl */ {IDCLS_P_HOSTNAME_PL, "<nazwa hosta>"},
	/* ru */ {IDCLS_P_HOSTNAME_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_HOSTNAME_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_HOSTNAME_TR, ""},  /* fuzzy */
#endif
	
	/* network.c */
	/* en */ {IDCLS_P_PORT,    N_("<port>")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_P_PORT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_P_PORT_DE, "<Port>"},
	/* es */ {IDCLS_P_PORT_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_P_PORT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_P_PORT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_P_PORT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_P_PORT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_P_PORT_NL, "<poort>"},
	/* pl */ {IDCLS_P_PORT_PL, "<port>"},
	/* ru */ {IDCLS_P_PORT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_P_PORT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_P_PORT_TR, ""},  /* fuzzy */
#endif
	
	/* network.c */
	/* en */ {IDCLS_SET_NETPLAY_SERVER,    N_("Set the netplay server hostname")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_NETPLAY_SERVER_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_NETPLAY_SERVER_DE, "Netplay Server Name setzen"},
	/* es */ {IDCLS_SET_NETPLAY_SERVER_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_NETPLAY_SERVER_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_NETPLAY_SERVER_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_NETPLAY_SERVER_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_NETPLAY_SERVER_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_NETPLAY_SERVER_NL, "Stel de netplay server hostnaam in"},
	/* pl */ {IDCLS_SET_NETPLAY_SERVER_PL, "Wprowadº nazwÍ serwera gry sieciowej"},
	/* ru */ {IDCLS_SET_NETPLAY_SERVER_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_NETPLAY_SERVER_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_NETPLAY_SERVER_TR, ""},  /* fuzzy */
#endif
	
	/* network.c */
	/* en */ {IDCLS_SET_NETPLAY_BIND_ADDRESS,    N_("Set the netplay binding address")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_DE, "Netplay Bindadresse setzen"},
	/* es */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_NL, "Stel het netplay binding adres in"},
	/* pl */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_PL, "Wprowadº adres gry sieciowej"},
	/* ru */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_NETPLAY_BIND_ADDRESS_TR, ""},  /* fuzzy */
#endif
	
	/* network.c */
	/* en */ {IDCLS_SET_NETPLAY_PORT,    N_("Set the netplay port (0..65535)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_NETPLAY_PORT_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_NETPLAY_PORT_DE, "Netplay TCP Port setzen (0..65535)"},
	/* es */ {IDCLS_SET_NETPLAY_PORT_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_NETPLAY_PORT_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_NETPLAY_PORT_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_NETPLAY_PORT_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_NETPLAY_PORT_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_NETPLAY_PORT_NL, "Stel de netplay poort in (0..65535)"},
	/* pl */ {IDCLS_SET_NETPLAY_PORT_PL, "Wybierz port TCP gry sieciowej (0..65535)"},
	/* ru */ {IDCLS_SET_NETPLAY_PORT_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_NETPLAY_PORT_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_NETPLAY_PORT_TR, ""},  /* fuzzy */
#endif
	
	/* network.c */
	/* en */ {IDCLS_SET_NETPLAY_CONTROL,    N_("Set the netplay control elements (keyboard, joystick1, joystick2, devices and resources), each item takes a value (0: None, 1: Server, 2: Client, 3: Both)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_NETPLAY_CONTROL_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_NETPLAY_CONTROL_DE, "Netplay Steuerelemente (Tastatur, Joystick 1, Joystick 2, Ger‰te und\nResourcen), f¸r jedes Element (0: Kein, 1: Server, 2: Client, 3: Beide)"},
	/* es */ {IDCLS_SET_NETPLAY_CONTROL_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_NETPLAY_CONTROL_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_NETPLAY_CONTROL_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_NETPLAY_CONTROL_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_NETPLAY_CONTROL_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_NETPLAY_CONTROL_NL, "Stel de netplay besturingsonderdelen in (toetsenbord, joystick1, joystick2, apparaten en bronnen), elk onderdeel heeft een waarde (0: Geen, 1: Server, 2: Client, 3: Beide)"},
	/* pl */ {IDCLS_SET_NETPLAY_CONTROL_PL, "Ustaw elementy kontrolne gry sieciowej (klawiatura, jostick1, joystick2, urz±dzenia i zasoby), kaødy element przyjmuje warto∂Ê (0: Brak, 1: Serwer, 2: Klient, 3: Serwer+Klient)"},
	/* ru */ {IDCLS_SET_NETPLAY_CONTROL_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_NETPLAY_CONTROL_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_NETPLAY_CONTROL_TR, ""},  /* fuzzy */
#endif
#endif
	
	/* scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_SET_SIMM_SIZE,    N_("Size of the SIMM RAM (0/1/4/8/16 MB)")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_SIMM_SIZE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_SIMM_SIZE_DE, "Grˆﬂe des SIMM RAM (0/1/4/8/16 MB)"},
	/* es */ {IDCLS_SET_SIMM_SIZE_ES, "TamaÒo de la RAM SIMM (0/1/4/8/16 MB)"},
	/* fr */ {IDCLS_SET_SIMM_SIZE_FR, "Grandeur de la RAM SIMM (0/1/4/8/16 mo)"},
	/* hu */ {IDCLS_SET_SIMM_SIZE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_SIMM_SIZE_IT, "Dimensione della RAM SIMM (0/1/4/8/16 MB)"},
	/* ko */ {IDCLS_SET_SIMM_SIZE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_SIMM_SIZE_NL, "Grootte van het SIMM RAM (0/1/4/8/16 MB)"},
	/* pl */ {IDCLS_SET_SIMM_SIZE_PL, "Ilo∂Ê SIMM RAM-u (0/1/4/8/16 MB)"},
	/* ru */ {IDCLS_SET_SIMM_SIZE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_SIMM_SIZE_SV, "Storlek pÂ SIMM-RAM (0/1/4/8/16 MB)"},
	/* tr */ {IDCLS_SET_SIMM_SIZE_TR, ""},  /* fuzzy */
#endif
	
	/* scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_JIFFY_SWITCH,    N_("Turn on Jiffy switch")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_JIFFY_SWITCH_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_JIFFY_SWITCH_DE, "Jiffy Schalter einschalten"},
	/* es */ {IDCLS_ENABLE_JIFFY_SWITCH_ES, "Encender conmutador Jiffy"},
	/* fr */ {IDCLS_ENABLE_JIFFY_SWITCH_FR, "Activer la jiffy Switch"},
	/* hu */ {IDCLS_ENABLE_JIFFY_SWITCH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_JIFFY_SWITCH_IT, "Attiva pulsante Jiffy"},
	/* ko */ {IDCLS_ENABLE_JIFFY_SWITCH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_JIFFY_SWITCH_NL, "Jiffy schakelaar inschakelen"},
	/* pl */ {IDCLS_ENABLE_JIFFY_SWITCH_PL, "W≥±cz prze≥±cznik Jiffy"},
	/* ru */ {IDCLS_ENABLE_JIFFY_SWITCH_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_JIFFY_SWITCH_SV, "SlÂ pÂ Jiffyomkopplare"},
	/* tr */ {IDCLS_ENABLE_JIFFY_SWITCH_TR, ""},  /* fuzzy */
#endif
	
	/* scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_JIFFY_SWITCH,    N_("Turn off Jiffy switch")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_JIFFY_SWITCH_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_JIFFY_SWITCH_DE, "Jiffy Schalter ausschalten"},
	/* es */ {IDCLS_DISABLE_JIFFY_SWITCH_ES, "Apagar conmutador Jiffy"},
	/* fr */ {IDCLS_DISABLE_JIFFY_SWITCH_FR, "DÈsactiver Jiffy Switch"},
	/* hu */ {IDCLS_DISABLE_JIFFY_SWITCH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_JIFFY_SWITCH_IT, "Disattiva pulsante Jiffy"},
	/* ko */ {IDCLS_DISABLE_JIFFY_SWITCH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_JIFFY_SWITCH_NL, "Jiffy schakelaar uitschakelen"},
	/* pl */ {IDCLS_DISABLE_JIFFY_SWITCH_PL, "Wy≥±cz prze≥±cznik Jiffy"},
	/* ru */ {IDCLS_DISABLE_JIFFY_SWITCH_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_JIFFY_SWITCH_SV, "SlÂ av Jiffyomkopplare"},
	/* tr */ {IDCLS_DISABLE_JIFFY_SWITCH_TR, ""},  /* fuzzy */
#endif
	
	/* scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_ENABLE_SPEED_SWITCH,    N_("Turn on Speed switch")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_SPEED_SWITCH_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_SPEED_SWITCH_DE, "Speed Schalter einschalten"},
	/* es */ {IDCLS_ENABLE_SPEED_SWITCH_ES, "Encender conmutador de velocidad"},
	/* fr */ {IDCLS_ENABLE_SPEED_SWITCH_FR, "Activer Speed Switch"},
	/* hu */ {IDCLS_ENABLE_SPEED_SWITCH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_SPEED_SWITCH_IT, "Attiva pulsante di velocit‡"},
	/* ko */ {IDCLS_ENABLE_SPEED_SWITCH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_SPEED_SWITCH_NL, "Snelheid schakelaar inschakelen"},
	/* pl */ {IDCLS_ENABLE_SPEED_SWITCH_PL, "W≥±cz prze≥±cznik Speed"},
	/* ru */ {IDCLS_ENABLE_SPEED_SWITCH_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_SPEED_SWITCH_SV, "SlÂ pÂ hastighetsomkopplare"},
	/* tr */ {IDCLS_ENABLE_SPEED_SWITCH_TR, ""},  /* fuzzy */
#endif
	
	/* scpu64/scpu64-cmdline-options.c */
	/* en */ {IDCLS_DISABLE_SPEED_SWITCH,    N_("Turn off Speed switch")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_SPEED_SWITCH_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_SPEED_SWITCH_DE, "Speed Schalter ausschalten"},
	/* es */ {IDCLS_DISABLE_SPEED_SWITCH_ES, "Apagar conmutador de velocidad"},
	/* fr */ {IDCLS_DISABLE_SPEED_SWITCH_FR, "DÈsactiver la Speed Switch"},
	/* hu */ {IDCLS_DISABLE_SPEED_SWITCH_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_SPEED_SWITCH_IT, "Disattiva pulsante di velocit‡"},
	/* ko */ {IDCLS_DISABLE_SPEED_SWITCH_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_SPEED_SWITCH_NL, "Snelheid schakelaar uitschakelen"},
	/* pl */ {IDCLS_DISABLE_SPEED_SWITCH_PL, "Wy≥±cz prze≥±cznik Speed"},
	/* ru */ {IDCLS_DISABLE_SPEED_SWITCH_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_SPEED_SWITCH_SV, "SlÂ av hastighetsomkopplare"},
	/* tr */ {IDCLS_DISABLE_SPEED_SWITCH_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pethre.c */
	/* en */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD,    N_("Enable HiRes Emulation Board")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_DE, "HiRes Emulation Board aktivieren"},
	/* es */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_ES, "Permitir emulaciÛn tableta de alta resoluciÛn"},
	/* fr */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_IT, "Attiva scheda HiRes Emulation"},
	/* ko */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_NL, "PET High Res Emulation bord inschakelen"},
	/* pl */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_PL, "W≥±cz kartÍ emulacji HiRes"},
	/* ru */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_SV, "Aktivera hˆgupplˆsningsemuleringskort"},
	/* tr */ {IDCLS_ENABLE_HIRES_EMULATION_BOARD_TR, ""},  /* fuzzy */
#endif
	
	/* pet/pethre.c */
	/* en */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD,    N_("Disable HiRes Emulation Board")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_DE, "HiRes Emulation Board aktivieren"},
	/* es */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_ES, "Deshabilitar emulaciÛn tableta de alta resoluciÛn"},
	/* fr */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_IT, "Disattiva scheda HiRes Emulation"},
	/* ko */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_NL, "PET High Res Emulation bord uitschakelen"},
	/* pl */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_PL, "Wy≥±cz kartÍ emulacji HiRes"},
	/* ru */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_SV, "hˆgupplˆsningsemuleringskort Inaktivera"},
	/* tr */ {IDCLS_DISABLE_HIRES_EMULATION_BOARD_TR, ""},  /* fuzzy */
#endif
	
#ifdef HAVE_RAWDRIVE
	/* diskimage/rawimage.c */
	/* en */ {IDCLS_SET_RAW_DRIVE_DEVICE,    N_("Set raw drive device name")},
#ifdef HAS_TRANSLATION
	/* da */ {IDCLS_SET_RAW_DRIVE_DEVICE_DA, ""},  /* fuzzy */
	/* de */ {IDCLS_SET_RAW_DRIVE_DEVICE_DE, "Raw Laufwerk Ger‰tename setzen"},
	/* es */ {IDCLS_SET_RAW_DRIVE_DEVICE_ES, ""},  /* fuzzy */
	/* fr */ {IDCLS_SET_RAW_DRIVE_DEVICE_FR, ""},  /* fuzzy */
	/* hu */ {IDCLS_SET_RAW_DRIVE_DEVICE_HU, ""},  /* fuzzy */
	/* it */ {IDCLS_SET_RAW_DRIVE_DEVICE_IT, ""},  /* fuzzy */
	/* ko */ {IDCLS_SET_RAW_DRIVE_DEVICE_KO, ""},  /* fuzzy */
	/* nl */ {IDCLS_SET_RAW_DRIVE_DEVICE_NL, "Stel binair drive apparaatnaam in"},
	/* pl */ {IDCLS_SET_RAW_DRIVE_DEVICE_PL, "Wprowadº nazwÍ urz±dzenia surowego napÍdu"},
	/* ru */ {IDCLS_SET_RAW_DRIVE_DEVICE_RU, ""},  /* fuzzy */
	/* sv */ {IDCLS_SET_RAW_DRIVE_DEVICE_SV, ""},  /* fuzzy */
	/* tr */ {IDCLS_SET_RAW_DRIVE_DEVICE_TR, ""},  /* fuzzy */
#endif
#endif
	
};



////
////
////// end of traslate_text.c


/* --------------------------------------------------------------------- */

static char *get_string_by_id(int id)
{
    unsigned int k;

    for (k = 0; k < countof(string_table); k++) {
        if (string_table[k].resource_id == id) {
            return string_table[k].text;
        }
    }
    return NULL;
}

static char *sid_return = NULL;

/* special case translation, this command-line option normally
   produces alot of lines (which only differ slightly) for the
   translators to translate, this function builds up the total
   command-line options from smaller translation pieces.
 */
static char *translate_and_build_sid_cmdline_option(int en_resource)
{
    char *old, *new;

    /* check if sid_return is already built */
    if (sid_return != NULL) {
        lib_free(sid_return);
    }

    /* start building up the command-line */
    old = lib_stralloc("Specify SID engine and model (");

    /* add fast sid options */
    new = util_concat(old, translate_text(IDCLS_FASTSID_ENGINE_MODEL), NULL);
    lib_free(old);
    old = new;


#ifdef HAVE_RESID
    /* add resid options if available */
    if (en_resource != IDCLS_SPECIFY_SIDCART_ENGINE_MODEL) {
        new = util_concat(old, ", ", translate_text(IDCLS_RESID_ENGINE_MODEL), NULL);
        lib_free(old);
        old = new;
    }

    /* add residdtv options if available */
    if (en_resource == IDCLS_SPECIFY_SIDDTV_ENGINE_MODEL) {
        new = util_concat(old, ", ", translate_text(IDCLS_RESIDDTV_ENGINE_MODEL), NULL);
        lib_free(old);
        old = new;
    }
#endif

#ifdef HAVE_CATWEASELMKIII
    /* add catweasel options if available */
    new = util_concat(old, ", ", translate_text(IDCLS_CATWEASELMKIII_ENGINE_MODEL), NULL);
    lib_free(old);
    old = new;
#endif

#ifdef HAVE_HARDSID
    /* add hardsid options if available */
    new = util_concat(old, ", ", translate_text(IDCLS_HARDSID_ENGINE_MODEL), NULL);
    lib_free(old);
    old = new;
#endif

#ifdef HAVE_PARSID
    /* add parsid options if available */
    new = util_concat(old, ", ", translate_text(IDCLS_PARSID_ENGINE_MODEL), NULL);
    lib_free(old);
    old = new;
#endif

	/*
#ifdef HAVE_RESID_FP
    // add resid-fp options if available
    if (en_resource != IDCLS_SPECIFY_SIDCART_ENGINE_MODEL) {
        new = util_concat(old, ", ", translate_text(IDCLS_RESIDFP_ENGINE_MODEL), NULL);
        lib_free(old);
        old = new;
    }
#endif
	 */
	
	
#ifdef HAVE_SSI2001
    /* add ssi2001 options if available */
    new = util_concat(old, ", ", translate_text(IDCLS_SSI2001_ENGINE_MODEL), NULL);
    lib_free(old);
    old = new;
#endif

    /* add ending bracket */
    new = util_concat(old, ")", NULL);
    lib_free(old);

    sid_return = new;

    return sid_return;
}

#ifdef HAS_TRANSLATION
#include "translate_table.h"

static char *text_table[countof(translate_text_table)][countof(language_table)];

static void translate_text_init(void)
{
    unsigned int i, j;
    char *temp;

    for (i = 0; i < countof(language_table); i++) {
        for (j = 0; j < countof(translate_text_table); j++) {
            if (translate_text_table[j][i] == 0) {
                text_table[j][i] = NULL;
            } else {
                temp = get_string_by_id(translate_text_table[j][i]);
                text_table[j][i] = intl_convert_cp(temp, language_cp_table[i]);
            }
        }
    }
}

char translate_id_error_text[30];

char *translate_text(int en_resource)
{
    unsigned int i;
    char *retval = NULL;

    if (en_resource == IDCLS_UNUSED) {
        return NULL;
    }

    if (en_resource == 0) {
        log_error(LOG_DEFAULT, "TRANSLATE ERROR: ID 0 was requested.");
        return "ID 0 translate error";
    }

    /* handle sid cmdline special case translations */
    if (en_resource == IDCLS_SPECIFY_SIDCART_ENGINE_MODEL ||
        en_resource == IDCLS_SPECIFY_SID_ENGINE_MODEL ||
        en_resource == IDCLS_SPECIFY_SIDDTV_ENGINE_MODEL) {
        return translate_and_build_sid_cmdline_option(en_resource);
    }

    if (en_resource < 0x10000) {
        retval = intl_translate_text(en_resource);
    } else {
        for (i = 0; i < countof(translate_text_table); i++) {
            if (translate_text_table[i][0] == en_resource) {
                if (translate_text_table[i][current_language_index] != 0 &&
                    text_table[i][current_language_index] != NULL &&
                    strlen(text_table[i][current_language_index]) != 0) {
                    retval = text_table[i][current_language_index];
                } else {
                    retval = text_table[i][0];
                }
            }
        }
    }

    if (retval == NULL) {
        log_error(LOG_DEFAULT, "TRANSLATE ERROR: ID %d was requested, and would be returning NULL.", en_resource);
        sprintf(translate_id_error_text, "ID %d translate error", en_resource);
        retval = translate_id_error_text;
    }

    return retval;
}

int translate_res(int en_resource)
{
    return intl_translate_res(en_resource);
}

/* --------------------------------------------------------------------- */

static int set_current_language(const char *lang, void *param)
{
    unsigned int i;

    util_string_set(&current_language, "en");
    current_language_index = 0;

    if (strlen(lang) != 2) {
        return 0;
    }

    for (i = 0; i < countof(language_table); i++) {
        if (!strcasecmp(lang, language_table[i])) {
            current_language_index = i;
            util_string_set(&current_language, language_table[i]);
            intl_update_ui();
            return 0;
        }
    }

    return 0;
}

static const resource_string_t resources_string[] = {
    { "Language", "en", RES_EVENT_NO, NULL,
      &current_language, set_current_language, NULL },
    RESOURCE_STRING_LIST_END
};

int translate_resources_init(void)
{
    intl_init();
    translate_text_init();

    return resources_register_string(resources_string);
}

static char *lang_list = NULL;

void translate_resources_shutdown(void)
{
    unsigned int i, j;

    for (i = 0; i < countof(language_table); i++) {
        for (j = 0; j < countof(translate_text_table); j++) {
            lib_free(text_table[j][i]);
        }
    }
    intl_shutdown();
    lib_free(current_language);

    /* check if sid_return is already built */
    if (sid_return != NULL) {
        lib_free(sid_return);
    }

    lib_free(lang_list);
}

static cmdline_option_t cmdline_options[] =
{
    { "-lang", SET_RESOURCE, 1,
      NULL, NULL, "Language", NULL,
      USE_PARAM_ID, USE_DESCRIPTION_COMBO,
      IDCLS_P_ISO_LANGUAGE_CODE, IDCLS_SPECIFY_ISO_LANG_CODE,
      NULL, NULL },
    CMDLINE_LIST_END
};

int translate_cmdline_options_init(void)
{
    char *temp_list = NULL;
    int i;

    lang_list = util_concat(". (", language_table[0], NULL);
    for (i = 1; i < countof(language_table); i++) {
        if (countof(language_table) == i + 1) {
            temp_list = util_concat(lang_list, "/", language_table[i], ")", NULL);
        } else {
            temp_list = util_concat(lang_list, "/", language_table[i], NULL);
        }
        lib_free(lang_list);
        lang_list = temp_list;
    }
    
    cmdline_options[0].description = lang_list;

    return cmdline_register_options(cmdline_options);
}

void translate_arch_language_init(void)
{
    char *lang;

    lang = intl_arch_language_init();
    set_current_language(lang, "");
}
#else

char *translate_text(int en_resource)
{
    if (en_resource == IDCLS_UNUSED) {
        return NULL;
    }

    if (en_resource == 0) {
        log_error(LOG_DEFAULT, "TRANSLATE ERROR: ID 0 was requested.");
        return "ID 0 translate error";
    }

    /* handle sid cmdline special case translations */
    if (en_resource == IDCLS_SPECIFY_SIDCART_ENGINE_MODEL ||
        en_resource == IDCLS_SPECIFY_SID_ENGINE_MODEL ||
        en_resource == IDCLS_SPECIFY_SIDDTV_ENGINE_MODEL) {
        return translate_and_build_sid_cmdline_option(en_resource);
    }

    return _(get_string_by_id(en_resource));
}
#endif
